      subroutine HZC87112(IFLAG)
****************************************************************************
*
* written by: Tancredi Carli and Renate Mohr
*             (carli@mail.desy.de)
* Seagull plot from EMC data
* Ref.: Z. Phys. C 36 (1987) 527
****************************************************************************
      IMPLICIT NONE
*
#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc" 
      Integer iflag,ihep,i,j,k,l
      Character*6 xxxx
*
      Real pi,rd,ee,eb
      Logical take,evcut,lp
      Integer iel,idum,id
      Parameter (pi=3.1415927,rd=180./pi,ee=280.)
      Integer nentry
      Double precision pbeam(4),pgam(4),pcm(4),ph(5),phc(4)
      Real x,y,q2,w2,enel,ptel,thel,efwd,th,rap,pt,et,p
      Integer nwt,loc
      Character chtitl*80
      Integer nev
      Real xf,pt2,w
      Integer ierr,ibeam,igam
      Data lp/.false./
*     EMC Zeit f Phys
      Data xxxx/'c87112'/
*
******************************************************************************
* Data points from plots
*
*
       Integer nx
       Parameter (nx=16)
       Real pts(nx),epts(nx)
* Seagull
       Data pts/0.29,0.28,0.32,0.26,0.32,0.23,0.20,0.15,
     &          0.16,0.22,0.32,0.45,0.55,0.61,0.72,0.76/
* error
       Data epts/0.02,0.001,0.03,0.001,0.04,0.01,0.001,0.001,
     &           0.001,0.001,0.001,0.02,0.03,0.02,0.14,0.06/
*
******************************************************************************
      NENTRY=NENTRY+1
*
      IF (iflag.eq.1) then
*
*      Initialisation: The following MUST always be done
*      (i) make subdirectory in PAWC
*       - use the name as the xxxxxx in HZxxxxxx subroutine
*      (i) make subdirectory in o/p file
*
       Call hcdir('//PAWC',' ')
       call hmdir(xxxx,'S')
       Call hcdir('//HISTO',' ')
       call hmdir(xxxx,'S')
*
*      book your histograms  (data: ID<0)
*                                     (VZERO, VMIN, VMAX see CERNLIB F121)
*     MC Histo with non-equid. Data-Binning
*
      CALL HBOOK1(50,'Mean Pt**2 vs xF  ',16,-0.8,0.8,0.)
      CALL HBOOK1(51,'Mean Pt**2 vs xF  ',16,-0.8,0.8,0.)
      CALL HBOOK1(-50,'Mean Pt**2 vs xF ',16,-0.8,0.8,0.)
      nev=0
*
*********************************************************************
*
      Else if(iflag.eq.2) then
*      Filling: The following MUST always be done
*      (i) move to the correct sub-directory in PAWC
*
       call hcdir('//PAWC/'//xxxx,' ')
*
*      Event selection
*
       q2=real(HZDISKIN(1))
       x =real(HZDISKIN(2))
       y =real(HZDISKIN(3))
       w2=real(HZDISKIN(4))
       w=sqrt(w2)
       if (lp) write(6,*) 'Hz'//xxxx,
     & ' x= ',x,' q2 ',q2,' y= ',y,' w2= ',w2
*
*      get position of scattered electron, energy,Pt and polar angle
*
       iel=HZIDELEC(idum)
       if (iel.eq.-1) then
        write(6,*) 'Hz'//xxxx,' electron not found '
        return
       endif
       ptel= real(sqrt(PHEP(1,iel)**2+PHEP(2,iel)**2))
       enel=real(PHEP(4,iel))
       Thel=real(HZPHMANG(PHEP(3,iel),
     &               sqrt(PHEP(1,iel)**2+PHEP(2,iel)**2)))*rd
       if (lp) write(6,*) 'Hz'//xxxx,
     &                       ' enel= ',enel,' thel= ',thel
*
*      cut on kinematics
*
       evcut= enel.gt.20.
       evcut= thel.gt.0.57.and. evcut
       evcut= w2.ge.16..and. w2.le.400 .and. evcut
       evcut= q2.gt.4. and. evcut
       evcut=y.lt.0.9.and.evcut
       if (lp) write(6,*) 'Hz'//xxxx,
     &                         ' efwd= ',efwd,' evcut= ',evcut
*
*      set up CMS boost needed later
*
       ierr=HZIBEAM(ibeam,idum)
       if (.not.(ierr.eq.1)) then
        write(6,*) 'HZ'//xxxx,' beams not found ! '
        return
       else
        Do i=1,4
         pbeam(i)=PHEP(i,IBEAM)
        enddo
       endif
*
       eb=real(PHEP(4,idum))
       if (abs(abs(eb)-ee).gt.0.2) then
        if (nentry.lt.10) then
         write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,
     &  ' but ',eb,' abs= ',abs(abs(eb)-ee)
        elseif (nentry.eq.10) then
         write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,
     &   ' ...last message ! '
        endif
C        return
       endif
*
       ierr=HZIPGAM(ph)
       if (ierr.eq.-1) then
        write(6,*) 'HZ'//xxxx,' boson vector not found ! '
C        call VZERO(ph,5)
        Do i=1,4
         ph(i)=0.
        enddo
       else
        Do i=1,4
         pgam(i)=ph(i)
        enddo
       endif
*
       call HZHCMINI(pbeam,pgam,ierr)
       if (ierr.eq.1) then
        write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
        return
       endif
*
*      count entries for normalization
*
       if (evcut) nev=nev+1
*
*      particle loop
       do 20 ihep=1,nhep
*
*       skip unstable particles and electron
        if (isthep(ihep).ne.1.or.ihep.eq.iel) goto 20
*
*       transform in CMS
*
        Do i=1,4
         Phc(i)=PHEP(i,ihep)
        enddo
        call HZHCM(phc,pcm,ierr)
        if (ierr.eq.1) then
         write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
         return
        endif
*                                       acceptance cut
*
        if (IHCHRG(IHEP).ne.0) then
         p      = real(sqrt(pcm(1)**2+pcm(2)**2+pcm(3)**2 ))
         pt     = real(sqrt( pcm(1)**2+pcm(2)**2 ))
         et     = real(pcm(4)) * pt / p
         th     = (acos(real(pcm(3))/p))
         th     = sign(max(abs(th),1.0e-05),th)
*
         if (lp) write(6,*) 'Hz'//xxxx,
     &   'lab: et,pt,rap,th ',et,pt,rap,th*rd
*
         xf=2.*real(pcm(3))/w
         pt2=real(pcm(1)**2 + pcm(2)**2)
*
         CALL HF1(50,XF,PT2*wtx)
         CALL HF1(51,XF,wtx)
        endif
*
 20    continue
*
************************************************************************
*
      Else if(iflag.eq.3) then
*      Termination: The following MUST always be done
*      (i) Move to the correct PAW subdirectory
*
       call hcdir('//PAWC/'//xxxx,' ')
*
* Histo normalization
*
       CALL HOPERA(50,'/ ',51,50,1.,1.)
       CALL HDELET(51)
*
* seagull
*
       call hpak (-50,pts)
       call hpake (-50,epts)
*
* Evaluate chi**2
*
* seagull
*
      call hzchisq(-50,50)
C       call hzchisq(-117,117)
*
      Endif
*
      RETURN
      END
