       Subroutine Hz00166(iflag)
*****************************************************************************
* This photoproduction routine has to be run twice with the following
* code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the RESOLVED component run.
*       +0 for both together.
* for all three phases (iflag=1,2,3)
*
* iflag = 1,2 or 3 depending on wether it is the initialization,
*		processing or termination phase (respectively)
*
* The default jetfinder is KTCLUS
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
* This routine produces two Monte Carlo graphs.
* id 10 is the normalised xgamma cross section in nb
* id 20 is the normalised ptrel cross section in nb
*		These graphs will be
*		meaningless unless Xsec and Ntot are set before
*		calling the termination routine.
*		(Xsec - total cross section returned by MC)
*		(Ntot - number of events passed to this routine)
*
* it also produces the related data plots as -10 and -20
*
* PLEASE NOTE it will also produce the plots:
*   1010 xgamma of the "beauty" part
*   2010 xgamma of the "other" part (includes b(bar) -> c(bar) -> e- as well as c -> e-)
*   1020 ptrel of the "beauty" part
*   2020 ptrel of the "other" part (includes b(bar) -> c(bar) -> e- as well as c -> e-)
* this will allow you to perfom your own fit of "b"/other component as in the paper.
* BUT ONLY IF THE MC SPECIFIC SECTION HAS BEEN ADDED FOR YOUR MC!!! CHECK BELOW IN CODE!!!
*
* Cuts : 	Q2 < 1
*		0.2 < y_bj < 0.8
*     Jets:  ET1 > 7 GeV, ET2 > 6 GeV, |eta| < 2.4
*     electron: pT > 1.3 |eta|<1.1
*
* Recommended value for Ptmin is 3 GeV (to be set in MC set up)
*
* Author : Mark Hayes (mark.hayes@desy.de)
*
* Reference : DESY-00-166
*
*****************************************************************************
*
* HERA Tuning defined commons
 
      IMPLICIT NONE
 
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"

      CHARACTER*6 jetf
c      DOUBLE PRECISION Jets(50,8)
      DOUBLE PRECISION CONER
      SAVE CONER

c --- pt_rel binning
      REAL xbins(9)
      DATA xbins/0.00,0.25,0.50,0.8,1.2,1.6,2.1,2.7,4.0/
      REAL wbins(8)

c --- the points
      REAL xg(8),xge(8),pt(8),pte(8)
      DATA xg/0.014288,0.0497754,0.0982585,0.0964189,
     +        0.100557,0.150494,0.412008,0.454369/
      DATA xge/0.0244473,0.0233268,0.0237086,0.0232385,
     +         0.0283313,0.0313688,0.053832,0.0615889/
      DATA pt/0.0898535,0.274556,0.14203,0.0566657,
     +        0.0114294,0.0090096,0.00637402,0.00149107/
      DATA pte/0.0292629,0.0414354,0.0191583,0.00817701,
     +         0.00399393,0.00307557,0.00208075,0.000854962/
	 
C --- cuts arrays
      REAL ybjcut(2),q2cut(2)
      REAL rapcut(2),Etcut(5)
      SAVE YBJCUT, Q2CUT, RAPCUT, ETCUT 

      INTEGER Njet,Maxjets
      PARAMETER (MaxJets=6)
      REAL jetet(MaxJets),jetphi(MaxJets),jetrap(MaxJets)
      REAL q2,ybj,xgam,mom_e,rap_e,phi_e,ptrel
      INTEGER mother
      REAL jetx(MaxJets),jety(MaxJets),jetz(MaxJets),dist(MaxJets)
      REAL modj,modej,dphi,deta
      REAL elecE,elecP
      INTEGER myil,myip
      REAL px,py,pz,ptsq,ppsq
 
      INTEGER chjet, iproc, iflag
      SAVE chjet
      INTEGER hzflag
      REAL sqrt, lumpb
      INTEGER i,j,k

C --- choose same jet finder as in the paper --- KTCLUS
      DATA chjet/3/

      CHARACTER*6 xxxx
      DATA xxxx /'d00166'/

****** set the jet cone radius.
****** CALL hzjetrad(1,coner) sets the radius to coner
****** CALL hzjetrad(2,coner) sets coner to the radius, or -1. 
******  check if the radius has been set eleswhere
      CALL hzjetrad(2,coner)
******  if not set it to the value used in the paper.
      IF (coner.LT.0.0) THEN
	      coner=1.0
      ENDIF

****** Determine the process type (single/double resolved etc)
      iproc=(mod(iflag,10000)/1000)
      IF ((iproc.LT.0).OR.(iproc.GT.2)) THEN
         WRITE(HZUNIT,*) xxxx,': illegal IPROC:',iproc
         RETURN
      ENDIF

****** choose jet finder
      IF ((MOD(iflag,100)/10.GT.0).AND.MOD(iflag,10).eq.1) THEN
         chjet=INT(MOD(iflag,100)/10)
      ENDIF

********************************  Initialise ********************************* 
      IF (MOD(iflag,10).EQ.1) THEN

C *** Initialisation run ***
	    CALL HCDIR('//HISTO',' ')

	  IF (iproc.le.1) THEN
	    CALL HCDIR('//HISTO',' ')
	    CALL HMDIR(xxxx,'S')
	    CALL HCDIR('//PAWC',' ')
	    CALL HMDIR(xxxx,'S')
 
	    CALL HBOOK1(-10,'Data cross section (nb) xgamma'
     &		,8 ,0.,1. ,0.)
	    CALL HBOOKB(-20,'Data cross section (nb) ptrel'
     &		,8 ,xbins ,0.)

	    CALL HPAK(-10,xg)
	    CALL HPAKE(-10,xge)
	    CALL HPAK(-20,pt)
	    CALL HPAKE(-20,pte)
 
	  ENDIF
 
	  CALL HCDIR('//HISTO/'//xxxx,' ')
	  CALL HCDIR('//PAWC/'//xxxx,' ')
	  CALL HBOOK1(10+iproc*100,'Monte Carlo events xgamma'
     &          ,8 ,0.,1. ,0.)
	  CALL HBOOKB(20+iproc*100,'Monte Carlo events ptrel'
     &          ,8 ,xbins ,0.)
c --- these plots are only produced if your MC is supported (See header comment)
	  CALL HBOOK1(1010+iproc*100,'Monte Carlo events xgamma, B part'
     &          ,8 ,0.,1. ,0.)
	  CALL HBOOKB(1020+iproc*100,'Monte Carlo events ptrel, B part'
     &          ,8 ,xbins ,0.)
	  CALL HBOOK1(2010+iproc*100,'Monte Carlo events xgamma,
     &          other part',8 ,0.,1. ,0.)
	  CALL HBOOKB(2020+iproc*100,
     &         'Monte Carlo events ptrel, other part'
     &          ,8 ,xbins ,0.)
  
C --- basic cuts for rapidity plots
	  Etcut(1)=7.0 ! first jet
          Etcut(2)=6.0 ! second jet
          rapcut(1)=2.4
 
          CALL HZJTNAME(chjet,jetf)
 
	  WRITE(6,*)'**********************************************'
	  WRITE(6,*)'* Hz00166 called, histograms will be output. *'
	  WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
	  WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
	  WRITE(6,*)'**********************************************'
 
 1001	  FORMAT(A29,F5.2,A13)
 
	ELSE IF (MOD(iflag,10).eq.2) THEN
C *** Filling runs ***
	
C *** Change directory to our graphs ***
	  CALL HCDIR('//HISTO/'//xxxx,' ')
	  CALL HCDIR('//PAWC/'//xxxx,' ')
 
	  q2=REAL(hzphokin(1))
	  IF (q2.eq.-1.0) THEN
c	    PRINT*,'Hz00166: HZPHOKIN returns error for Q2'
	  ELSEIF (q2.lt.0.0) THEN
	    q2=abs(q2)
	  ENDIF
	  ybj=REAL(hzphokin(3))
	  IF (ybj.eq.-1.0) THEN
c	    PRINT*,'Hz00166: HZPHOKIN returns error for ybj'
	  ENDIF
 
C *** Fill our histogram ***

C --- photoproduction cuts
 	  ybjcut(1)=0.20
	  ybjcut(2)=0.80
	  q2cut(1)=0.
	  q2cut(2)=1.
 
	  IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(2))
     &		.and.(q2.lt.q2cut(2))) Then
 
	   hzflag=HZIBEAM(myip,myil)
	   IF (hzflag.ge.0) Then
 	    elecE=abs(PHEP(4,myil))
	   ELSE
	    PRINT*,'Hz00166: Electron not found. E=0.001'
	    PRINT*,'         event will be rejected'
	    elecE=0.001
	   ENDIF
           ElecP=820.0

           CALL hzjtfind(chjet,CONER,NJET,Jets)
 
	   IF (NJET.ge.2) THEN
	     j=1
             Do i=1,Njet
	       IF (abs(Jets(i,1)).lt.rapcut(1)
     &             .and.j.le.MaxJets ) THEN ! only keep jets in our range
                 JetEt(j)=Jets(i,3)
                 JetPhi(j)=Jets(i,2)
                 JetRap(j)=Jets(i,1)
                 Jetx(j)=Jets(i,5)
                 Jety(j)=Jets(i,6)
                 Jetz(j)=Jets(i,7)
                 j=j+1
	       ENDIF ! abs(Jets(i,1)).lt.rapcut(1)
             ENDDO
	     NJET=j-1 ! redefine njet
 
           IF (JetEt(1).gt.Etcut(1).and.JetEt(2).gt.Etcut(2)
     +            .and.abs(Jetrap(1)).lt.rapcut(1)
     +            .and.abs(Jetrap(2)).lt.rapcut(1)) THEN
c --- things to look at
       xgam = (jetet(1)*exp(-1*jetrap(1))+jetet(2)*exp(-1*jetrap(2)))
     +          /(2*ybj*elecE)

                DO i=1,NHEP
                  IF (ISTHEP(i).eq.1.and.IDHEP(i).eq.11) THEN
c --- only looking at final state electrons... calculate kinematics
                    px=PHEP(1,i)
                    py=PHEP(2,i)
                    pz=PHEP(3,i)
                    ptsq=px**2+py**2
                    ppsq=(SQRT(ptsq+pz**2)+ABS(pz))**2
                    IF (ptsq.le.4.25E-18*ppsq) THEN
                      rap_e=20
                    ELSE
                      rap_e=0.5*LOG(ppsq/ptsq)
                    ENDIF
                    rap_e=SIGN(rap_e,pz)
                    IF (ptsq.EQ.0.) THEN
                      phi_e=0.
                    ELSE
                      phi_e=ATAN2(py,px)
                    ENDIF
                    mom_e=sqrt(px**2+py**2)
c --- perform standard analysis cuts
                    IF (abs(rap_e).lt.1.1.and.mom_e.gt.1.6) THEN
                      IF (JMOHEP(1,i).gt.0.and.JMOHEP(1,i).lt.NHEP) THEN
                       mother=IDHEP(JMOHEP(1,i))
                      ELSE
                       mother=0
                      ENDIF
                      DO j=1,MaxJets
                        deta=abs(rap_e-Jetrap(j))
                        dphi=abs(phi_e-Jetphi(j))
                        IF (dphi.gt.3.141592654) dphi=dphi-2*3.141592654
                        IF (dphi.lt.-3.141592654) dphi=dphi+2*3.141592654
                        dist(j)=(deta)**2+(dphi)**2
		      ENDDO		! over MaxJets
                      j=1
                      DO k=2,MaxJets
                        IF (dist(k).lt.dist(j)) j=k
                      ENDDO ! loop over MaxJets
                      modej=(py*jetz(j)-pz*jety(j))**2
     +                  +(-px*jetz(j)+pz*jetx(j))**2
     +                   +(px*jety(j)-py*jetx(j))**2
                      modj=jetx(j)**2+jety(j)**2+jetz(j)**2
                      ptrel=sqrt(modej/modj)
c --- fill the plots ...
		      CALL HF1(10+iproc*100,REAL(xgam),1.)
		      CALL HF1(20+iproc*100,REAL(ptrel),1.)

c --- THIS IS THE MC SPECIFIC SECTION // please add to for your MC!!!!
          IF (GEN.eq.'HRW') THEN
            IF (mother.gt.-550.and.mother.lt.-500) THEN
               CALL HF1(1010+iproc*100,REAL(xgam),1.)
               CALL HF1(1020+iproc*100,REAL(ptrel),1.)
            ENDIF               ! B meson -> e-
            IF (.not.(mother.gt.-550.and.mother.lt.-500)
     +           .and.mother.ne.111) THEN
               CALL HF1(2010+iproc*100,REAL(xgam),1.)
               CALL HF1(2020+iproc*100,REAL(ptrel),1.)
            ENDIF               ! everything else other than pi0
          ENDIF ! GEN.eq.HRW
c --- end of MC specific section...

                    ENDIF ! abs(rap_e).lt.1.1.and.mom_e.gt.1.6
                  ENDIF ! ISTHEP.eq.1.and.IDHEP(i).eq.11
                ENDDO ! loop over HEPEVTP

	      ENDIF ! jet cuts
	    ENDIF ! Njet.gt.2.0
	  ENDIF ! Q2 & y cuts
 
	ELSE IF (MOD(iflag,10).eq.3) THEN
C *** Termination run ***
          IF (iproc.eq.0.or.iproc.eq.2) then
             write(hzunit,*) 'HZ00166: Finishing off.'
          endif
          
C     *** Change directory ***
          CALL HCDIR('//HISTO/'//xxxx,' ')
          CALL HCDIR('//PAWC/'//xxxx,' ')
          
          IF (Xsec.le.0) THEN
            PRINT*,'HZ00166:termination called with zero cross section'
            PRINT*,'         cross section graph meaningless'
            PRINT*,'	     in process :',iproc*1000
            Xsec=1
	  ENDIF
	  IF (Ntot.le.0) THEN
	    PRINT*,'HZ00166: termination called with no total events'
	    PRINT*,'          cross section graph meaningless'
	    PRINT*,'	      in process :',iproc*1000
	    Ntot=1
	  ENDIF
 
C --- store Xsec and Ntot for both direct and resolved events.
          lumpb = ntot/xsec

          CALL HZHINRM(iproc*100+10,0,lumpb,1)
          CALL HZHINRM(iproc*100+20,0,lumpb,1)

          CALL HZHINRM(iproc*100+1010,0,lumpb,1)
          CALL HZHINRM(iproc*100+1020,0,lumpb,1)
          CALL HZHINRM(iproc*100+2010,0,lumpb,1)
          CALL HZHINRM(iproc*100+2020,0,lumpb,1)

	  IF (iproc.eq.2) THEN
             CALL HOPERA(110,'+e',210,10,1.,1.)
             CALL HOPERA(120,'+e',220,20,1.,1.)
             CALL HOPERA(1110,'+e',1210,1010,1.,1.)
             CALL HOPERA(1120,'+e',1220,1020,1.,1.)
             CALL HOPERA(2110,'+e',2210,2010,1.,1.)
             CALL HOPERA(2120,'+e',2220,2020,1.,1.)

          ENDIF ! iproc.eq.2 
	ELSE
C *** End ***
	  PRINT*,'Hz00166: Please run routine with iflag set to 1,2 or 3'
	  PRINT*,'         with +1000 or +2000 for DIR or RES events.'
	ENDIF
 
	RETURN
	END

