      SUBROUTINE HZ01220(iflag)
c     -----------------------------------------------------------------------
c     
c     This photoproduction routine has to be run twice with the
c     following
c     code additions:
c     +1000 for the DIRECT component run.
c     +2000 for the RESOLVED component run.
c     +0 for both together
c     for all three phases (iflag=1,2,3)
c     
c     +10000 to run on parton showers instead of final state particles.
c     +100000 is used by NLO integration options.
c     
c     iflag = 1,2 or 3 depending on whether it is the initialization,
c     processing or termination phase (respectively)
c     
c     The default jetfinder is KTCLUS(
c     To change jetfinder use these values to call the initialisation
c     phase.
c     1 + 10 c (jetfinder number)
c     
c     The cos theta* plots:
c     id 11 costheta* for low xg
c     id 12 costheta* for high xg
c
c     The actual id of the histgram is 1000ci+id
c     where i is an index for the ET cut and id is given below.
c
c     ---------------------------------------------------------
c     id 13 is d(sigma)/d(eta) in pb for high x_gamma, 2nd jet
c     backward
c     id 14 is d(sigma)/d(eta) in pb for high x_gamma, 2nd jet
c     central
c     id 15 is d(sigma)/d(eta) in pb for high x_gamma, 2nd jet
c     forward
c     id 16 is d(sigma)/d(eta) in pb for low x_gamma, 2nd jet
c     backward
c     id 17 is d(sigma)/d(eta) in pb for low x_gamma, 2nd jet
c     central
c     id 18 is d(sigma)/d(eta) in pb for low x_gamma, 2nd jet
c     forward
c     
c     The range of integration is the highest ET jet above ETcut for
c     five cuts  given in the array etcut below. Both jets are above
c     etcut(0) for all graphs.
c     id 19 is d(sigma)/d(et) in pb for low x_gamma, 1<eta_2<2,
c     1<eta_1<2
c     id 20 is d(sigma)/d(et) in pb for low x_gamma, 0<eta_2<1,
c     1<eta_1<2
c     id 21 is d(sigma)/d(et) in pb for low x_gamma,-1<eta_2<0,
c     1<eta_1<2
c     id 22 is d(sigma)/d(et) in pb for low x_gamma, 0<eta_2<1,
c     0<eta_1<1
c     id 23 is d(sigma)/d(et) in pb for low x_gamma,-1<eta_2<0,
c     0<eta_1<1
c     id 24 is d(sigma)/d(et) in pb for low x_gamma,-1<eta_2<0,
c     -1<eta_1<0
c     id 25 is d(sigma)/d(et) in pb for high x_gamma, 1<eta_2<2,
c     1<eta_1<2
c     id 26 is d(sigma)/d(et) in pb for high x_gamma, 0<eta_2<1,
c     1<eta_1<2
c     id 27 is d(sigma)/d(et) in pb for high x_gamma,-1<eta_2<0,
c     1<eta_1<2
c     id 28 is d(sigma)/d(et) in pb for high x_gamma, 0<eta_2<1,
c     0<eta_1<1
c     id 29 is d(sigma)/d(et) in pb for high x_gamma,-1<eta_2<0,
c     0<eta_1<1
c     id 30 is d(sigma)/d(et) in pb for high x_gamma,-1<eta_2<0,
c     -1<eta_1<0
c     
c     The xgamma graph below is also made in four regions of ET of the
c     highest ET jet, given by the cuts in the etcut array, but
c     now bounded above by etcut i+1 and below by etcut i for i=1 to 5
c     id 31 to 34 is d(sigma)/d(xgamma) in pb
c     
c
c     All these graphs will be meaningless unless Xsec and Ntot are set
c     before calling the termination routine.
c     (Xsec - total cross section returned by MC)
c     (Ntot - number of events passed to this routine)
c     
c     Cuts :
c     Q2 < 1
c     0.2< y_bj < 0.85
c     Depending on graph Et > 11, 14
c     
c     Author : Jon Butterworth (J.Butterworth@ucl.ac.uk)
c     Reference : Eur.Phys.J.C23:615-631,2002 
c     
c     ---------------------------------------------------------------
*     
*     HERA Tuning defined commons

      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"

      CHARACTER*6 jetf

      CHARACTER*6 xxxx
      DATA xxxx /'d01220'/
      
      INTEGER NJET
c      DOUBLE PRECISION Jets(50,8)
      DOUBLE PRECISION CONER
      SAVE CONER
      
      DOUBLE PRECISION elecE
      INTEGER myil,myip

c     Iterators
      INTEGER i_etcut, i_jet, j_jet, i_plot, i_bin

      DOUBLE PRECISION xgam,xgamcut
      DOUBLE PRECISION phi(2),px(2),py(2),pz(2),ptsq(2)
      DOUBLE PRECISION ppsq(2),energy(2),mass(2)
      DOUBLE PRECISION rap(2),Et(2)
      DOUBLE PRECISION ybj,q2

C     --- cuts arrays
      DOUBLE PRECISION ybjcut(3),q2cut(2)
      DOUBLE PRECISION rapcut(4),mjjcut,etabarcut(2)
c     photoproduction cuts
      DATA ybjcut / 0.2, 0.5, 0.85 /
      DATA q2cut / 0., 1. /
      DATA xgamcut / 0.75 /
      DATA etabarcut / 0.1, 1.3 /
      DATA mjjcut / 42. /

c     rapidity cuts. the bins for the second jet.
      DATA rapcut /-1.0, 0.0, 1.0, 2.4  /
      
C     --- private copies of Xsec and Ntot from HERACMN
      DOUBLE PRECISION mhxsec(2),mhntot(2)

c     Unit to read data file from
      INTEGER ipunit
      PARAMETER(ipunit=8)

c     The data
c     --------------------------------------

c     ----------- costheta* stuff
      REAL costheta, deltaeta, mjj, etabar
c     Bins for the xg plots      
      INTEGER n_ctbins
      PARAMETER(n_ctbins=8)
c     xgbins1 is used for the lowest et bin. xgbins2 for the rest.
      REAL ctbins(n_ctbins+1)
      REAL ctsigma1(n_ctbins), ctsigma2(n_ctbins)
c     Statistical error
      REAL ctstat1(n_ctbins),ctstat2(n_ctbins)
c     Systematic error up (excluding energy scale)
c      REAL ctsys1(n_ctbins), ctsys2(n_ctbins)
      REAL ctsys1up(n_ctbins), ctsys2up(n_ctbins)
      REAL ctsys1down(n_ctbins), ctsys2down(n_ctbins)
      REAL cterr1(n_ctbins)
      REAL cterr2(n_ctbins)
      REAL ct_bw(n_ctbins), ct_zero(n_ctbins)

      DATA ct_zero /n_ctbins*0./
      DATA ctbins / .0, .1, .2, .3, .4, .5, .6 , .7, .8 /
      DATA ctsigma1 / 44.5, 40.1, 48.1, 54.5, 83.9,115.7,205.5,406.9/
      DATA ctsigma2 /141.8,142.1,155.3,182.7,233.8,283.9,439.7,686.6/
      DATA ctstat1 /3.5, 3.3, 3.6, 3.7, 4.7, 5.5, 7.6, 11.5/
      DATA ctstat2 /7.1, 7.1, 7.3, 7.9, 8.8, 9.7,12.2, 15.9/
c     Systematic error calculated at the average of up & down, added in
c     quadrature. Needs to be sqrted later.
c     See table 1 of the paper.
      DATA ctsys1up / 9.85, 15.53, 12.01
     &     , 21.36 , 27.54, 25 
     &     ,281.7,355.94/
      DATA ctsys1down /18.44, 7.45 , 15.3 
     &     , 7.06, 42.12
     &     , 37.06, 136.58, 1303.38
     &     /
      DATA ctsys2up /
     &  46.25,
     &  39.78,
     &  104.66,
     &  80.33,
     &  169.6,
     &  171.17,
     &  350.33,
     &  1291.04/
      DATA ctsys2down /
     &  92.65,
     &  131.36,
     &  87.2,
     &  101.54,
     &  193.7,
     &  214.65,
     &  612.68,
     &  845.65/
    
c     ----------- ET stuff      
c     Etcut = regions of ET used for the xgamma plots.
      DOUBLE PRECISION Etcut(0:5)
c     Bins for the ET plots
      INTEGER n_etbins
      PARAMETER(n_etbins=10)
      REAL etbins(n_etbins+1)
      REAL eterr1(n_etbins),eterr2(n_etbins),eterr3(n_etbins),
     &     eterr4(n_etbins),eterr5(n_etbins),eterr6(n_etbins),
     &     eterr7(n_etbins),eterr8(n_etbins),eterr9(n_etbins),
     &     eterr10(n_etbins),eterr11(n_etbins),eterr12(n_etbins)
      REAL etsigma1(n_etbins),etsigma2(n_etbins),etsigma3(n_etbins)
     &     ,etsigma4(n_etbins),etsigma5(n_etbins),etsigma6(n_etbins)
     &     ,etsigma7(n_etbins),etsigma8(n_etbins),etsigma9(n_etbins)
     &     ,etsigma10(n_etbins),etsigma11(n_etbins),etsigma12(n_etbins)
c     Statistical error
      REAL etstat1(n_etbins),etstat2(n_etbins),etstat3(n_etbins)
     &     ,etstat4(n_etbins),etstat5(n_etbins),etstat6(n_etbins)
     &     ,etstat7(n_etbins),etstat8(n_etbins),etstat9(n_etbins)
     &     ,etstat10(n_etbins),etstat11(n_etbins),etstat12(n_etbins)
c     Systematic error up (excluding energy scale)
      REAL etsys1up(n_etbins),etsys2up(n_etbins),etsys3up(n_etbins)
     &     ,etsys4up(n_etbins),etsys5up(n_etbins),etsys6up(n_etbins)
     &     ,etsys7up(n_etbins),etsys8up(n_etbins),etsys9up(n_etbins)
     &     ,etsys10up(n_etbins),etsys11up(n_etbins),etsys12up(n_etbins)
c     Systematic error up (excluding energy scale)
      REAL etsys1down(n_etbins),etsys2down(n_etbins),etsys3down(n_etbins
     &     ),etsys4down(n_etbins),etsys5down(n_etbins)
     &     ,etsys6down(n_etbins),etsys7down(n_etbins)
     &     ,etsys8down(n_etbins),etsys9down(n_etbins)
     &     ,etsys10down(n_etbins),etsys11down(n_etbins)
     &     ,etsys12down(n_etbins)
      REAL et_bw(n_etbins), et_zero(n_etbins)

      DATA et_zero /n_etbins*0./
      DATA etbins / 14., 17., 21., 25., 29., 35.
     ,     , 41., 48., 55., 65., 75./
c     See table 3
      DATA etsigma1 / 0.3,  0.0, 0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0
     &     .0,  0.0 /
      DATA etsigma2 / 6.2, 2.1, 0.23, 0.032,  0.0,  0.0,  0.0,  0.0,  0
     &     .0,  0.0 /
      DATA etsigma3 /29.4, 12.7, 4.6, 1.2, 0.21, 0.0,  0.0,  0.0,  0
     &     .0,  0.0 /
      DATA etsigma4 /15.4,  6.0, 1.4, 0.38, 0.057,  0.0,  0.0,  0.0,  0
     &     .0,  0.0 /
      DATA etsigma5 /56.7, 26.6, 11.2, 3.7, 1.3, 0.35, 0.064, 0.019,  0
     &     .0,  0.0 /
      DATA etsigma6 /78.4, 38.2, 16.9, 6.3, 2.5, 0.92, 0.31, 0.12, 0.045
     &     ,  0.0 /
c     See table 2
      DATA etsigma7 /12.2, 3.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0
     &     .0,  0.0 /
      DATA etsigma8 /34.3, 16.0, 4.0, 0.74, 0.0,  0.0,  0.0,  0.0,  0
     &     .0,  0.0 /
      DATA etsigma9 /51.4, 26.1, 12.3, 6.3, 1.7, 0.56, 0.122,  0.0,  0
     &     .0,  0.0 /
      DATA etsigma10 /28.1, 15.8, 5.4, 1.6, 0.41, 0.043, 0.0,  0.0,  0
     &     .0,  0.0 /
      DATA etsigma11 /38.9, 23.1, 11.2, 5.3, 2.1, 0.87, 0.34, 0.13,  0
     &     .035,  0.0 /
      DATA etsigma12 / 1.5, 4.0, 5.0, 3.3, 1.9, 0.93, 0.41, 0.23, 0.033
     &     ,  0.041 /

c     see table 3
      DATA etstat1 / 0.08,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0
     &     .0,  0.0 /
      DATA etstat2 / 0.3,  0.1,  0.04,  0.015,  0.0,  0.0,  0.0,  0.0,
     &     0.0,  0.0 /
      DATA etstat3 / 0.8,  0.4,  0.3,  0.1,  0.04,  0.0,  0.0,  0.0,  0.0
     &     ,  0.0 /
      DATA etstat4 / 0.4,  0.2,  0.1,  0.05,  0.015,  0.0,  0.0,  0.0,
     &     0.0,  0.0 /
      DATA etstat5 / 0.8,  0.4,  0.3,  0.2,  0.1,  0.04,  0.016,  0.009,
     &     0.0,  0.0 /
      DATA etstat6 / 1.4,  0.7,  0.5,  0.3,  0.1,  0.08,  0.05,  0.03,
     &     0.016,  0.0 /
c     see table 2
      DATA etstat7 / 0.6,  0.3,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0
     &     ,  0.0 /
      DATA etstat8 / 0.7,  0.4,  0.2,  0.08,  0.0,  0.0,  0.0,  0.0,  0
     &     .0,  0.0 /
      DATA etstat9 / 1.1,  0.7,  0.5,  0.3,  0.1,  0.08,  0.037,  0.0,
     &     0.0,  0.0 /
      DATA etstat10 / 0.6,  0.4,  0.2,  0.1,  0.05,  0.014,  0.0,  0.0,
     &     0.0,  0.0 /
      DATA etstat11 / 0.7,  0.4,  0.3,  0.2,  0.1,  0.07,  0.04,  0.03,
     &     0.011,  0.0 /
      DATA etstat12 / 0.2,  0.2,  0.3,  0.2,  0.1,  0.1,  0.06,  0.05,
     &     0.015,  0.017 /

      DATA etsys1up / 
     &     0.005,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys1down / 
     &     0.0017,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys2up / 
     &     0.85,
     &     0.02,
     &     0.0026,
     &     0.00037,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys2down / 
     &     0.1,
     &     0.17,
     &     0.0037,
     &     0.000025,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys3up / 
     &     1.37,
     &     0.41,
     &     0.13,
     &     0.05,
     &     0.0053,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys3down / 
     &     4.82,
     &     2.21,
     &     0.29,
     &     0.0116,
     &     0.0005,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys4up / 
     &     5.93,
     &     0.34,
     &     0.0425,
     &     0.0018,
     &     0.00017,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys4down / 
     &     0.5,
     &     0.1,
     &     0.0116,
     &     0.0333,
     &     0.000485,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys5up / 
     &   6.97,
     &   1.94,
     &   0.37,
     &   0.02,
     &   0.0125,
     &   0.0025,
     &   0.000058,
     &   0.000197,
     &     0.0, 
     &     0.0 /
      DATA etsys5down / 
     &   5.41,
     &   1.25,
     &   0.74,
     &   0.08,
     &   0.0125,
     &   0.002,
     &   0.000106,
     &   26e-06,
     &     0.0, 
     &     0.0 /
      DATA etsys6up / 
     &   48.85,
     &   9.14,
     &   1.45,
     &   0.25,
     &   0.02,
     &   0.0106,
     &   0.0013,
     &   0.0002,
     &   0.000212,
     &     0.0 /
      DATA etsys6down / 
     &     37,
     &     3.37,
     &     0.58,
     &     0.1,
     &     0.01,
     &     0.0037,
     &     0.002,
     &     0.0002,
     &     0.000265,
     &     0.0 /
c     See table 2.
      DATA etsys7up / 
     &     13.32,
     &     0.17,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys7down / 
     &     1.46,
     &     0.37,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys8up / 
     &     6.53,
     &     0.74,
     &     0.02,
     &     0.0068,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys8down / 
     &     3.56,
     &     4.25,
     &     0.29,
     &     0.0298,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys9up / 
     &     2.72,
     &     1.3,
     &     0.25,
     &     0.1,
     &     0.1,
     &     0.0013,
     &     0.00097,
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys9down / 
     &     33.3,
     &     5.41,
     &     2.41,
     &     0.68,
     &     0.05,
     &     0.0068,
     &     0.000296,
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys10up / 
     &     12.56,
     &     0.45,
     &     0.08,
     &     0.05,
     &     0.0005,
     &     0.000185,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys10down / 
     &     1.7,
     &     1.8,
     &     0.02,
     &     0.02,
     &     0.0029,
     &     0.000178,
     &     0.0, 
     &     0.0, 
     &     0.0, 
     &     0.0 /
      DATA etsys11up / 
     &     1.37,
     &     0.68,
     &     0.17,
     &     0.0409,
     &     0.0101,
     &     0.0037,
     &     0.004,
     &     0.0002,
     &     0.000004,
     &     0.0 /
      DATA etsys11down / 
     &     6.05,
     &     2.25,
     &     0.45,
     &     0.08,
     &     0.0136,
     &     0.0034,
     &     0.0008,
     &     0.001,
     &     0.0002,
     &     0.0 /
      DATA etsys12up / 
     &     0.16,
     &     0.68,
     &     0.13,
     &     0.1,
     &     0.01,
     &     0.008,
     &     0.004,
     &     0.001,
     &     0.000845,
     &     0.00002/
      DATA etsys12down / 
     &    0.17,
     &    0.29,
     &    0.05,
     &    0.05,
     &    0.02,
     &    13e-04,
     &    13e-04,
     &    5e-04,
     &    68e-06,
     &    1.04e-04/


c     ---------- eta stuff (see Table 4 & Table 5)
c     Differential rapidity cross section indexed in all/highy all/dir,
c     etcut, and then bins in eta * 3 for the three eta regions of the
c     second jet.
      INTEGER n_etabins
      PARAMETER( n_etabins=7 )
      REAL etabins(n_etabins+1), etabins4(n_etabins)
      REAL dsdeta1(n_etabins),dsdeta2(n_etabins),dsdeta3(n_etabins)
     &     ,dsdeta4(n_etabins-1),dsdeta5(n_etabins),dsdeta6(n_etabins)
c     Statistical error
      REAL etastat1(n_etabins),etastat2(n_etabins),etastat3(n_etabins)
     &     ,etastat4(n_etabins-1),etastat5(n_etabins),etastat6(n_etabins
     &     )
c     Systematic error down 
      REAL etasys1up(n_etabins),etasys2up(n_etabins),etasys3up(n_etabins
     &     ),etasys4up(n_etabins-1),etasys5up(n_etabins)
     &     ,etasys6up(n_etabins)
      REAL etasys1down(n_etabins),etasys2down(n_etabins)
     &     ,etasys3down(n_etabins),etasys4down(n_etabins-1)
     &     ,etasys5down(n_etabins),etasys6down(n_etabins)
      REAL eta_bw(n_etabins), eta_zero(n_etabins)
      REAL eta_bw4(n_etabins-1)
      REAL error1(n_etabins),error2(n_etabins),error3(n_etabins)
      REAL error4(n_etabins-1),error5(n_etabins),error6(n_etabins)

      DATA eta_zero /n_etabins*0./
      DATA etabins / -1., -0.5, 0., 0.5, 1., 1.5, 2., 2.4 /
      DATA etabins4 / -0.5, 0., 0.5, 1., 1.5, 2., 2.4 /
      DATA dsdeta1 /  8.9,  88.2, 175.0, 195.8, 160.3, 130.0,  89.3/
      DATA dsdeta2 / 99.3, 272.0, 347.0, 347.1, 287.9, 208.1, 125.1/
      DATA dsdeta3 /115.4, 241.7, 315.7, 277.5, 104.0,  37.2,  13.2/
      DATA dsdeta4 /  2.2,  15.8,  40.0,  52.4,  60.4,  54.1 /
      DATA dsdeta5 /  4.1,  51.7, 131.7, 195.4, 230.1, 262.4, 243.7/
      DATA dsdeta6 / 20.5, 135.0, 288.8, 403.6, 403.0, 355.2, 305.0/ 
      DATA etastat1 / 1.0, 2.8, 3.8, 4.0, 3.4, 3.3, 3.5 /
      DATA etastat2 / 3.1, 4.6, 5.1, 5.0, 4.2, 3.7, 3.7 /
      DATA etastat3 / 3.2, 4.2, 4.6, 4.2, 2.4, 1.5, 1.1 /
      DATA etastat4 / 0.4, 1.0, 1.7, 1.8, 2.0, 2.4 /
      DATA etastat5 / 0.5, 1.9, 3.1, 3.8, 3.7, 4.1, 5.0 /
      DATA etastat6 / 1.2, 3.0, 4.4, 5.2, 4.8, 4.5, 5.3 /
      DATA etasys1up / 
     &  90.5,
     &  261,
     &  156.85,
     &  117.14,
     &  138.29,
     &  65.25,
     &  38.97/
      DATA etasys1down / 
     &  2.6,
     &  94.88,
     &  278.98,
     &  193.45,
     &  59.81,
     &  28.85,
     &  40.97/
      DATA etasys2up / 
     &  145.3,
     &  210.53,
     &  159.37,
     &  141.73,
     &  93.78,
     &  58.81,
     &  18.82/
      DATA etasys2down / 
     &  11.45,
     &  790.37,
     &  1543.33,
     &  1140.89,
     &  474.34,
     &  81,
     &  80.9/
      DATA etasys3up / 
     &  447.29,
     &  115.81,
     &  144.4,
     &  73.69,
     &  31.25,
     &  2.89,
     &  1.06/
      DATA etasys3down / 
     &  69.28,
     &  81.14,
     &  302.33,
     &  428.77,
     &  28.25,
     &  3.65,
     &  5.45/
      DATA etasys4up / 
     &  1.97,
     &  8.77,
     &  8.98,
     &  45.2,
     &  65.25,
     &  54.9/
      DATA etasys4down / 
     &  0.65,
     &  0.61,
     &  5.84,
     &  3.4,
     &  28.45,
     &  14.21/
      DATA etasys5up / 
     &  2.34,
     &  24.65,
     &  54.45,
     &  74.12,
     &  117.13,
     &  164.74,
     &  146.9/
      DATA etasys5down / 
     &  0.37,
     &  7.4,
     &  163.93,
     &  266.65,
     &  216.05,
     &  120.85,
     &  133.22/
      DATA etasys6up / 
     &  26.82,
     &  229.86,
     &  177.77,
     &  398.41,
     &  851.33,
     &  946.72,
     &  871.76/

      DATA etasys6down / 
     &  0.73,
     &  51.01,
     &  176.81,
     &  334.34,
     &  499.36,
     &  640,
     &  239.85/

c     ----------- xgamma stuff
c     Bins for the xg plots
      INTEGER n_xgbins1,n_xgbins2
      PARAMETER(n_xgbins1=8)
      PARAMETER(n_xgbins2=4)
      REAL xg_bw1(n_xgbins1), xg_bw2(n_xgbins2),xg_zero(n_xgbins1)

c     xgbins1 is used for the lowest 2 et bins. xgbins2 for the top two.
      REAL xgbins1(n_xgbins1+1),xgbins2(n_xgbins2+1)

      REAL xgsigma1(n_xgbins1), xgsigma2(n_xgbins1),xgsigma3(n_xgbins2)
     &     ,xgsigma4(n_xgbins2) 
      
c     Statistical error
      REAL xgstat1(n_xgbins1),xgstat2(n_xgbins1),xgstat3(n_xgbins2)
     &     ,xgstat4(n_xgbins2)
c     Systematic error
      REAL xgsys1up(n_xgbins1),xgsys2up(n_xgbins1),xgsys3up(n_xgbins2)
     &     ,xgsys4up(n_xgbins2)
      REAL xgsys1down(n_xgbins1),xgsys2down(n_xgbins1)
     &     ,xgsys3down(n_xgbins2),xgsys4down(n_xgbins2)
c     Full error.
      REAL xgerr1(n_xgbins1),xgerr2(n_xgbins1),xgerr3(n_xgbins2)
     &     ,xgerr4(n_xgbins2)

      DATA xg_zero /n_xgbins1*0./
      DATA xgbins1 / .1, .2, .3, .4, .5, .6 , .7, .8, 1.001/
      DATA xgbins2 / .1, .4, .6, .8, 1.001/

      DATA xgsigma1 /421.4, 631.6, 607.2, 599.9
     &     ,621.2, 695.8, 842.1, 1784.0 /
      DATA xgsigma2 / 251.2, 419.2, 523.1, 539.7,
     &     597.3, 641.3, 812.2, 1791.5/
      DATA xgsigma3 / 40.0, 89.8, 130.0, 332.2 /
      DATA xgsigma4 / 3.9, 12.7, 24.5, 77.6 /

      DATA xgstat1 / 14.1,  16.2, 15.0,  14.4,  14.6
     &     ,  15.4,  16.5,  18.8 /
      DATA xgstat2 / 9.6, 11.3, 12.5, 12.5, 13.2, 13.6
     &     , 15.2, 17.9/
      DATA xgstat3 / 1.8, 3.3, 4.0, 7.3/
      DATA xgstat4 / 0.6, 1.2, 1.7, 3.6/

      DATA xgsys1up /
     &  2870.26,
     &  2695.73,
     &  1867.28,
     &  2054.45,
     &  2946.6,
     &  1707.4,
     &  10621.81,
     &  19844.21/
      DATA xgsys1down /
     &  1397.81,
     &  3898.1,
     &  733.49,
     &  413.65,
     &  362.66,
     &  744.25,
     &  714.37,
     &  8876.9/

      DATA xgsys2up /
     &  353.45,
     &  1224.34,
     &  2274.1,
     &  616.85,
     &  1018.09,
     &  1318.05,
     &  1605.44,
     &  5279.17/
      DATA xgsys2down /
     &  401.89,
     &  808.1,
     &  903.13,
     &  742.73,
     &  937.7,
     &  692.37,
     &  1484.29,
     &  27640.57/
      DATA xgsys3up /
     &  74.56,
     &  44.37,
     &  31.3,
     &  1077.05/
      DATA xgsys3down /
     &  5.33,
     &  16.66,
     &  43.38,
     &  1185.29/
      DATA xgsys4up /
     &  0.25,
     &  2.42,
     &  6.53,
     &  22.81/
      DATA xgsys4down /
     &  4.04,
     &  0.73,
     &  7.94,
     &  48.1/

      DATA etcut / 11.D0, 14.D0, 17.D0, 25.D0, 35.D0, 90.D0 /

      INTEGER chjet, iproc, iflag
      SAVE chjet
C     --- choose same jet finder as in the paper --- KTCLUS
      DATA chjet/3/
      INTEGER hzflag
      
      LOGICAL parton_showers, lp, first
      INTEGER LENOCC      
      DATA lp /.false./
      
      parton_showers=.FALSE.
      
      
*     Determine the process type.
      iproc=(mod(iflag,10000)/1000)
      if ((iproc.lt.0).or.(iproc.gt.2)) then
         write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
         return
      endif

*     --- choose jet finder
      IF ((MOD(iflag,100)/10.gt.0).and.MOD(iflag,10).eq.1) THEN
         chjet=INT(MOD(iflag,100)/10)
      ENDIF
      
      IF (MOD(100000,iflag)/10000.gt.0) THEN
         parton_showers=.TRUE.
         chjet=-1*ABS(chjet)
      ELSE
         chjet=ABS(chjet)
      ENDIF
      
      IF (MOD(iflag,10).eq.1) THEN
C     *** Initialisation run ***
C     *** Create directories ***
         IF (iproc.le.1) THEN
            CALL HCDIR('//HISTO',' ')
            IF (parton_showers) THEN
               CALL HMDIR('ps'//xxxx,'S')
               CALL HCDIR('//PAWC',' ')
               CALL HMDIR('ps'//xxxx,'S')
            ELSE
               CALL HMDIR(xxxx,'S')
               CALL HCDIR('//PAWC',' ')
               CALL HMDIR(xxxx,'S')
            ENDIF
         ENDIF

         IF (parton_showers) THEN
            CALL HCDIR('//HISTO/ps'//xxxx,' ')
            CALL HCDIR('//PAWC/ps'//xxxx,' ')
         ELSE
            CALL HCDIR('//HISTO/'//xxxx,' ')
            CALL HCDIR('//PAWC/'//xxxx,' ')
         ENDIF
         
c     costheta*
         CALL HBOOKB(iproc*100+11,'MC: costheta* low xgamma'
     &        ,n_ctbins,ctbins,0.)

         CALL HBOOKB(iproc*100+12,'MC: costheta* high xgamma'
     &        ,n_ctbins,ctbins,0.)

c     Eta histograms
         
         CALL HBOOKB(iproc*100+13
     &        ,'MC: High xgamma, 2nd jet backward',n_etabins
     &        ,etabins,0.)

         CALL HBOOKB(iproc*100+14
     &        ,'MC: High xgamma, 2nd jet central',n_etabins
     &        ,etabins,0.)
         
         CALL HBOOKB(iproc*100+15
     &        ,'MC: High xgamma, 2nd jet forward',n_etabins
     &        ,etabins,0.)
         
         CALL HBOOKB(iproc*100+16
     &        ,'MC: Low xgamma, 2nd jet backward',n_etabins-1
     &        ,etabins4,0.)

         CALL HBOOKB(iproc*100+17
     &        ,'MC: Low xgamma, 2nd jet central',n_etabins,
     &           etabins,0.)

         CALL HBOOKB(iproc*100+18
     &        ,'MC: Low xgamma, 2nd jet forward',n_etabins,
     &        etabins,0.)

c     ET plots 
         CALL HBOOKB(iproc*100+19,'MC:Both Jets Forward, Low xgamma'
     &        ,n_etbins-9,etbins,0.)
         
         CALL HBOOKB(iproc*100+20,'MC: 1 Forward 2 middle, Low xgamma'
     &        ,n_etbins-6,etbins,0.)
         
         CALL HBOOKB(iproc*100+21,'MC: 1 Forward 2 back, Low xgamma'
     &        ,n_etbins-5,etbins,0.)

         CALL HBOOKB(iproc*100+22,'MC: Both jets central, Low xgamma'
     &        ,n_etbins-5,etbins,0.)
         
         CALL HBOOKB(iproc*100+23,'MC: 1 middle, 2 back, Low xgamma'
     &        ,n_etbins-2,etbins,0.)
         
         CALL HBOOKB(iproc*100+24,'MC: Both jet backward, Low xgamma'
     &        ,n_etbins-1,etbins,0.)
         
         CALL HBOOKB(iproc*100+25,'MC:Both Jets Forward, high xgamma'
     &        ,n_etbins-8,etbins,0.)
         
         CALL HBOOKB(iproc*100+26,'MC: 1 Forward 2 middle, high xgamma'
     &        ,n_etbins-6,etbins,0.)
         
         CALL HBOOKB(iproc*100+27,'MC: 1 Forward 2 back, high xgamma'
     &        ,n_etbins-3,etbins,0.)
         
         CALL HBOOKB(iproc*100+28,'MC: Both jets central, high xgamma'
     &        ,n_etbins-4,etbins,0.)
         
         CALL HBOOKB(iproc*100+29,'MC: 1 middle, 2 back, high xgamma'
     &        ,n_etbins-1,etbins,0.)
         
         CALL HBOOKB(iproc*100+30,'MC: Both jet backward, high xgamma'
     &        ,n_etbins,etbins,0.)

c     xgamma
         CALL HBOOKB(iproc*100+31
     &        ,'MC: xgamma a'
     &        ,n_xgbins1,xgbins1 ,0.)
         CALL HBOOKB(iproc*100+32
     &        ,'MC: xgamma b'
     &        ,n_xgbins1,xgbins1 ,0.)
         CALL HBOOKB(iproc*100+33
     &        ,'MC: xgamma c'
     &        ,n_xgbins2,xgbins2 ,0.)
         CALL HBOOKB(iproc*100+34
     &        ,'MC: xgamma d'
     &        ,n_xgbins2,xgbins2 ,0.)
         
         CALL HIDOPT(0,'stat')
         CALL HBARX(0)
         
C     --- standard cone radius is 1.0
         CALL HZJETRAD(2,CONER)
         IF (CONER.lt.0.0) THEN
            CONER=1.0
         ENDIF
         CALL HZJTNAME(chjet,jetf)
         
         WRITE(6,*)'**********************************************'
         WRITE(6,*)'* HZ01220 called, histograms will be output. *'
         WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
         WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
         WRITE(6,*)'**********************************************'

 1001    FORMAT(A29,F5.2,A13)

      ELSE IF (MOD(iflag,10).eq.2) THEN
C     *** Filling runs ***
         
C     *** Change directory to our graphs ***
         IF (parton_showers) THEN              
            CALL HCDIR('//PAWC/ps'//xxxx,' ')
         ELSE
            CALL HCDIR('//PAWC/'//xxxx,' ')
         ENDIF

         IF (gen(1:3).EQ.'HO1') THEN

            DO i_plot=11,34
               CALL HZSNLO(i_plot,i_plot+200,i_plot+100)
            ENDDO
            iproc = 1

         ENDIF

         q2=hzphokin(1)
         IF (q2.eq.-1) THEN
            PRINT*,'HZ01220: HZPHOKIN returns error for Q2'
         ELSEIF (q2.lt.0) THEN
            q2=abs(q2)
         ENDIF
         ybj=hzphokin(3)
         IF (ybj.eq.-1) THEN
            PRINT*,'HZ01220: HZPHOKIN returns error for ybj'
         ENDIF
         
C     *** Fill histograms ***
         
C     All events must pass the y cut and the Q2 cut.
         IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(3))
     &        .and.(q2.lt.q2cut(2))) Then
            
c     Get the initla state electron energy.
            hzflag=HZIBEAM(myip,myil)
            IF (hzflag.ge.0) Then
               elecE=abs(PHEP(4,myil))
            ELSE
               PRINT*,'HZ01220: Electron not found. E=0.001'
               PRINT*,'         event will be rejected'
               RETURN
            ENDIF
            
c     Find jets
            CALL HZJTFIND(chjet,CONER,NJET,Jets)
            
            IF (lp) THEN
               WRITE(*,*) 'Njet=',NJET
            ENDIF

            DO i_jet=1,2
               et(i_jet)=0.0
            ENDDO

c     If there aren't at least 2 jets, don't bother doing anything else.
            IF (njet.GE.2) THEN

               i_jet=0
               DO j_jet = 1,njet
c     Find the two highest ET jets in the rapidity interval.
c     Assumes jets are ordered in ET.
                  IF ( jets(j_jet,1).gt.rapcut(1).and.jets(j_jet,1).lt
     &                 .rapcut(4)) THEN
                     i_jet=i_jet+1
                     IF (i_jet.LE.2) THEN
                        Et(i_jet)=Jets(j_jet,3)
                        rap(i_jet)=Jets(j_jet,1)
                        phi(i_jet)=Jets(j_jet,2)
                     ENDIF

                  ENDIF
               ENDDO

c     All events must satisfy at least the lowest ET cut.
               IF (et(2).gt.etcut(0).AND.et(1).gt.etcut(1)) THEN
                  
c     Calculate xgamma
                  xgam=(Et(1)*exp(-1*rap(1))+Et(2)*exp(-1*rap(2)))
     &                 /(2*ybj*elecE)
                  deltaeta=abs(rap(1)-rap(2))
                  etabar = (rap(1)+rap(2))/2.0
                  costheta = abs(tanh(deltaeta/2))
c     TODO: is phi wrapping dealt with ok here??
                  mjj = sqrt(2.0*et(1)*et(2)*(cosh(deltaeta)
     +              -cos(phi(1)-phi(2))))
c                  write(*,*) mjj, etabar

c     Fill costheta.
                  IF ((mjj.GT.mjjcut).AND.etabar.GT.etabarcut(1).AND
     &                 .etabar.LT.etabarcut(2)) THEN
                     IF (xgam.LT.xgamcut) THEN
                        CALL HFILL(iproc*100+11,costheta,0.,wtx)
                     ELSE
                        CALL HFILL(iproc*100+12,costheta,0.,wtx)
                     ENDIF
                  ENDIF

                  IF (lp) THEN
                     WRITE(*,*) 'xgamma,wtx=',xgam,wtx
                  ENDIF


c     Fill the rapidity plots.
                  DO i_jet=1,2
                     j_jet=3-i_jet

                     IF (xgam.GE.xgamcut) THEN

                        IF (rap(i_jet).gt.rapcut(1).and.rap(i_jet).lt
     &                       .rapcut(2))THEN
                           
                           CALL HFILL(INT(iproc*100+13),
     &                          REAL(rap(j_jet)),0.,wtx)
                        ENDIF
                        
                        IF (rap(i_jet).gt.rapcut(2).and.rap(i_jet).lt
     &                       .rapcut(3))THEN
                           CALL HFILL(INT(iproc*100+14),
     &                          REAL(rap(j_jet)),0.,wtx)
                        ENDIF
                        
                        IF (rap(i_jet).gt.rapcut(3).and.rap(i_jet).lt
     &                       .rapcut(4))THEN
                           CALL HFILL(INT(iproc*100+15),
     &                          REAL(rap(j_jet)),0.,wtx)
                        ENDIF
                     
                     ELSE

                        IF (rap(i_jet).gt.rapcut(1).and.rap(i_jet).lt
     &                       .rapcut(2))THEN
                           CALL HFILL(INT(iproc*100+16),
     &                          REAL(rap(j_jet)),0.,wtx)
                        ENDIF
                        IF (rap(i_jet).gt.rapcut(2).and.rap(i_jet).lt
     &                       .rapcut(3))THEN
                           CALL HFILL(INT(iproc*100+17),
     &                          REAL(rap(j_jet)),0.,wtx)
                        ENDIF
                        IF (rap(i_jet).gt.rapcut(3).and.rap(i_jet).lt
     &                       .rapcut(4))THEN
                           CALL HFILL(INT(iproc*100+18),
     &                          REAL(rap(j_jet)),0.,wtx)
                        ENDIF
                        
                     ENDIF

                  ENDDO

c     Fill the ET  plots.
                  DO i_jet=1,2
                     j_jet=3-i_jet
                     
                     IF ((rap(i_jet).gt.rapcut(1).and.rap(i_jet).lt
     &                    .rapcut(2)).AND.(rap(j_jet).gt.rapcut(1).and
     &                    .rap(j_jet).lt.rapcut(2))) THEN
                        IF (xgam.LT.xgamcut) THEN
                           CALL HFILL(INT(iproc*100+19),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ELSE
                           CALL HFILL(INT(iproc*100+25),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ENDIF
                     ENDIF
                     
                     IF ((rap(i_jet).gt.rapcut(2).and.rap(i_jet).lt
     &                    .rapcut(3)).AND.(rap(j_jet).gt.rapcut(1).and
     &                    .rap(j_jet).lt.rapcut(2))) THEN
                        IF (xgam.LT.xgamcut) THEN
                           CALL HFILL(INT(iproc*100+20),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ELSE
                           CALL HFILL(INT(iproc*100+26),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ENDIF
                     ENDIF
                     
                     IF ((rap(i_jet).gt.rapcut(2).and.rap(i_jet).lt
     &                    .rapcut(3)).AND.(rap(j_jet).gt.rapcut(2).and
     &                    .rap(j_jet).lt.rapcut(3))) THEN
                        IF (xgam.LT.xgamcut) THEN
                           CALL HFILL(INT(iproc*100+21),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ELSE
                           CALL HFILL(INT(iproc*100+27),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ENDIF
                     ENDIF
                     
                     IF ((rap(i_jet).gt.rapcut(3).and.rap(i_jet).lt
     &                    .rapcut(4)).AND.(rap(j_jet).gt.rapcut(1).and
     &                    .rap(j_jet).lt.rapcut(2))) THEN
                        IF (xgam.LT.xgamcut) THEN
                           CALL HFILL(INT(iproc*100+22),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ELSE
                           CALL HFILL(INT(iproc*100+28),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ENDIF
                     ENDIF
                     
                     IF ((rap(i_jet).gt.rapcut(3).and.rap(i_jet).lt
     &                    .rapcut(4)).AND.(rap(j_jet).gt.rapcut(2).and
     &                    .rap(j_jet).lt.rapcut(3))) THEN
                        IF (xgam.LT.xgamcut) THEN
                           CALL HFILL(INT(iproc*100+23),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ELSE
                           CALL HFILL(INT(iproc*100+29),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ENDIF
                     ENDIF
                     
                     IF ((rap(i_jet).gt.rapcut(3).and.rap(i_jet).lt
     &                    .rapcut(4)).AND.(rap(j_jet).gt.rapcut(3).and
     &                    .rap(j_jet).lt.rapcut(4))) THEN
                        IF (xgam.LT.xgamcut) THEN
                           CALL HFILL(INT(iproc*100+24),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ELSE
                           CALL HFILL(INT(iproc*100+30),
     &                          REAL(MAX(et(j_jet),et(i_jet))) ,0.,wtx)
                        ENDIF
                     ENDIF

                  ENDDO

c     Fill xgamma plots
c     Loop over ET cuts
                  DO i_etcut= 1, 4
c     ET cut for rapidity plots
                     IF ((Et(1).gt.Etcut(i_etcut)).and.(et(1).lt
     &                    .ETcut(i_etcut+1)))THEN
                        CALL HFILL(INT(iproc*100+30+i_etcut),
     &                       REAL(xgam),0. ,wtx)
                     ENDIF
                  ENDDO
                  
               ENDIF
            ENDIF
         ENDIF                  ! Q2 & y cuts
         
      ELSE IF (MOD(iflag,10).eq.3) THEN
C     *** Termination run ***
C     *** Change directory ***
         IF (parton_showers) THEN
            CALL HCDIR('//PAWC/ps'//xxxx,' ')
         ELSE
            CALL HCDIR('//PAWC/'//xxxx,' ')
         ENDIF
         
C     *** Finish off histograms ***
         
         IF (Xsec.eq.0) THEN
            PRINT*,'HZ01220: termination called with zero cross section'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',iproc*1000
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,'HZ01220: termination called with no total events'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',iproc*1000
            Ntot=1
         ENDIF
         
c     Now rescale the MC by lumi & bin width.
c     (not for NLO calculations)
         IF (gen(1:3).NE.'HO1') THEN
            
            DO i_plot=11,34
               CALL HZHINRM(100*iproc+i_plot,0,REAL(ntot/(xsec*1000)),1)
            ENDDO
                        
         ENDIF      
                  
         IF (iproc.eq.2.or.iproc.eq.0) THEN
C     --- only finally
C     --- create proper cross-section data

            WRITE (HZUNIT,*) 'HZ01220: finishing off'

c     Book the costheta* plots.
            CALL HBOOKB(-11,'Data: costheta*, low xgamma'
     &           ,n_ctbins,ctbins,0.)
            
            CALL HBOOKB(-12,'Data: costheta*, high xgamma'
     &           ,n_ctbins,ctbins,0.)

            DO i_bin = 1,n_ctbins
c     See comment by "table 1".
               ctsys1up(i_bin)=sqrt(ctsys1up(i_bin))
               ctsys2up(i_bin)=sqrt(ctsys2up(i_bin))
               ctsys1down(i_bin)=sqrt(ctsys1down(i_bin))
               ctsys2down(i_bin)=sqrt(ctsys2down(i_bin))

               cterr1(i_bin)=sqrt(ctstat1(i_bin)**2+0.25*(ctsys1up(i_bin
     &              )+ctsys1down(i_bin))**2  )
               cterr2(i_bin)=sqrt(ctstat2(i_bin)**2+0.25*(ctsys2up(i_bin
     &              )+ctsys2down(i_bin))**2  )
               ctsigma1(i_bin)=ctsigma1(i_bin)+ (ctsys1up(i_bin)
     &              -ctsys1down(i_bin))/2.0
               ctsigma2(i_bin)=ctsigma2(i_bin)+ (ctsys2up(i_bin)
     &              -ctsys2down(i_bin))/2.0
            ENDDO
            CALL HPAK(-11,ctsigma1)
            CALL HPAKE(-11,cterr1)
            CALL HPAK(-12,ctsigma2)
            CALL HPAKE(-12,cterr2)

c     rapidity plots.
            CALL HBOOKB(-13
     &           ,'Data: Low xgamma, 2nd jet backward',n_etabins,
     &           etabins,0.)
            
            CALL HBOOKB(-14
     &           ,'Data: Low xgamma, 2nd jet central',n_etabins ,
     &           etabins,0.)
            CALL HBOOKB(-15
     &           ,'Data: Low xgamma, 2nd jet forward',n_etabins ,
     &           etabins,0.)
            
            CALL HBOOKB(-16
     &           ,'Data: High xgamma, 2nd jet backward'
     &           ,n_etabins-1,etabins4,0.)
            CALL HBOOKB(-17
     &           ,'Data: High xgamma, 2nd jet central'
     &           ,n_etabins ,etabins,0.)
            CALL HBOOKB(-18
     &           ,'Data: High xgamma, 2nd jet forward'
     &           ,n_etabins,etabins,0.)               
            DO i_bin=1,n_etabins                  
               etasys1up(i_bin)=sqrt(etasys1up(i_bin))
               etasys2up(i_bin)=sqrt(etasys2up(i_bin))
               etasys3up(i_bin)=sqrt(etasys3up(i_bin))
               etasys5up(i_bin)=sqrt(etasys5up(i_bin))
               etasys6up(i_bin)=sqrt(etasys6up(i_bin))

               etasys1down(i_bin)=sqrt(etasys1down(i_bin))
               etasys2down(i_bin)=sqrt(etasys2down(i_bin))
               etasys3down(i_bin)=sqrt(etasys3down(i_bin))
               etasys5down(i_bin)=sqrt(etasys5down(i_bin))
               etasys6down(i_bin)=sqrt(etasys6down(i_bin))

               error1(i_bin)=sqrt(etastat1(i_bin)**2+0.25
     &              *(etasys1up(i_bin)**2+etasys1down(i_bin)**2))
               error2(i_bin)=sqrt(etastat2(i_bin)**2+0.25
     &              *(etasys2up(i_bin)**2+etasys2down(i_bin)**2))
               error3(i_bin)=sqrt(etastat3(i_bin)**2+0.25
     &              *(etasys3up(i_bin)**2+etasys3down(i_bin)**2))

               IF (i_bin.NE.n_etabins) THEN
                  etasys4up(i_bin)=sqrt(etasys4up(i_bin))
                  etasys4down(i_bin)=sqrt(etasys4down(i_bin))
                  error4(i_bin)=sqrt(etastat4(i_bin)**2+0.25
     &                 *(etasys4up(i_bin)**2+etasys4down(i_bin)**2))
                  dsdeta4(i_bin)=dsdeta4(i_bin)+(etasys4up(i_bin)
     &                 -etasys4down(i_bin))/2.0
               ENDIF
               error5(i_bin)=sqrt(etastat5(i_bin)**2+0.25
     &              *(etasys5up(i_bin)**2+etasys5down(i_bin)**2))
               error6(i_bin)=sqrt(etastat6(i_bin)**2+0.25
     &              *(etasys6up(i_bin)**2+etasys6down(i_bin)**2))


               dsdeta1(i_bin)=dsdeta1(i_bin)+(etasys1up(i_bin)
     &              -etasys1down(i_bin))/2.0
               dsdeta2(i_bin)=dsdeta2(i_bin)+(etasys2up(i_bin)
     &              -etasys2down(i_bin))/2.0
               dsdeta3(i_bin)=dsdeta3(i_bin)+(etasys3up(i_bin)
     &              -etasys3down(i_bin))/2.0
               dsdeta5(i_bin)=dsdeta5(i_bin)+(etasys5up(i_bin)
     &              -etasys5down(i_bin))/2.0
               dsdeta6(i_bin)=dsdeta6(i_bin)+(etasys6up(i_bin)
     &              -etasys6down(i_bin))/2.0

            ENDDO
            CALL HPAK(-13,dsdeta1)
            CALL HPAKE(-13,error1)
            CALL HPAK(-14,dsdeta2)
            CALL HPAKE(-14,error2)
            CALL HPAK(-15,dsdeta3)
            CALL HPAKE(-15,error3)
            CALL HPAK(-16,dsdeta4)
            CALL HPAKE(-16,error4)
            CALL HPAK(-17,dsdeta5)
            CALL HPAKE(-17,error5)
            CALL HPAK(-18,dsdeta6)
            CALL HPAKE(-18,error6)
               
c     ET plots             
            CALL HBOOKB(-19,'Data:Both Jets Forward, all xgamma'
     &           ,n_etbins-9,etbins,0.)
            
            CALL HBOOKB(-20
     &           ,'Data: 1 Forward 2 middle, all xgamma',n_etbins-6
     &           ,etbins,0.)
            
            CALL HBOOKB(-21,'Data: 1 Forward 2 back, all xgamma'
     &           ,n_etbins-5,etbins,0.)
            
            CALL HBOOKB(-22,'Data: Both jets central, all xgamma'
     &           ,n_etbins-5,etbins,0.)
            
            CALL HBOOKB(-23,'Data: 1 middle, 2 back, all xgamma'
     &           ,n_etbins-2,etbins,0.)
            
            CALL HBOOKB(-24,'Data: Both jet backward, all xgamma'
     &           ,n_etbins-1,etbins,0.)
            
            CALL HBOOKB(-25,'Data:Both Jets Forward, high xgamma'
     &           ,n_etbins-8,etbins,0.)
            
            CALL HBOOKB(-26
     &           ,'Data: 1 Forward 2 middle, high xgamma',n_etbins-6
     &           ,etbins,0.)
            
            CALL HBOOKB(-27,'Data: 1 Forward 2 back, high xgamma'
     &           ,n_etbins-3,etbins,0.)
            
            CALL HBOOKB(-28
     &           ,'Data: Both jets central, high xgamma',n_etbins-4
     &           ,etbins,0.)
            
            CALL HBOOKB(-29,'Data: 1 middle, 2 back, high xgamma'
     &           ,n_etbins-1,etbins,0.)
            
            CALL HBOOKB(-30
     &           ,'Data: Both jet backward, high xgamma',n_etbins,etbins
     &           ,0.)
            DO i_bin=1,n_etbins
               etsys1up(i_bin)=sqrt(etsys1up(i_bin))
               etsys2up(i_bin)=sqrt(etsys2up(i_bin))
               etsys3up(i_bin)=sqrt(etsys3up(i_bin))
               etsys4up(i_bin)=sqrt(etsys4up(i_bin))
               etsys5up(i_bin)=sqrt(etsys5up(i_bin))
               etsys6up(i_bin)=sqrt(etsys6up(i_bin))
               etsys7up(i_bin)=sqrt(etsys7up(i_bin))
               etsys8up(i_bin)=sqrt(etsys8up(i_bin))
               etsys9up(i_bin)=sqrt(etsys9up(i_bin))
               etsys10up(i_bin)=sqrt(etsys10up(i_bin))
               etsys11up(i_bin)=sqrt(etsys11up(i_bin))
               etsys12up(i_bin)=sqrt(etsys12up(i_bin))

               etsys1down(i_bin)=sqrt(etsys1down(i_bin))
               etsys2down(i_bin)=sqrt(etsys2down(i_bin))
               etsys3down(i_bin)=sqrt(etsys3down(i_bin))
               etsys4down(i_bin)=sqrt(etsys4down(i_bin))
               etsys5down(i_bin)=sqrt(etsys5down(i_bin))
               etsys6down(i_bin)=sqrt(etsys6down(i_bin))
               etsys7down(i_bin)=sqrt(etsys7down(i_bin))
               etsys8down(i_bin)=sqrt(etsys8down(i_bin))
               etsys9down(i_bin)=sqrt(etsys9down(i_bin))
               etsys10down(i_bin)=sqrt(etsys10down(i_bin))
               etsys11down(i_bin)=sqrt(etsys11down(i_bin))
               etsys12down(i_bin)=sqrt(etsys12down(i_bin))



               eterr1(i_bin)= sqrt(etstat1(i_bin)**2+0.25
     &              *(etsys1up(i_bin)+etsys1down(i_bin))**2)
               eterr2(i_bin)=sqrt( etstat2(i_bin)**2+0.25
     &              *(etsys2up(i_bin)+etsys2down(i_bin))**2)
               eterr3(i_bin)=sqrt( etstat3(i_bin)**2+0.25
     &              *(etsys3up(i_bin)+etsys3down(i_bin))**2)
               eterr4(i_bin)=sqrt( etstat4(i_bin)**2+0.25
     &              *(etsys4up(i_bin)+etsys4down(i_bin))**2)
               eterr5(i_bin)=sqrt( etstat5(i_bin)**2+0.25
     &              *(etsys5up(i_bin)+etsys5down(i_bin))**2)
               eterr6(i_bin)=sqrt( etstat6(i_bin)**2+0.25
     &              *(etsys6up(i_bin)+etsys6down(i_bin))**2)
               eterr7(i_bin)=sqrt( etstat7(i_bin)**2+0.25
     &              *(etsys7up(i_bin)+etsys7down(i_bin))**2)
               eterr8(i_bin)=sqrt( etstat8(i_bin)**2+0.25
     &              *(etsys8up(i_bin)+etsys8down(i_bin))**2)
               eterr9(i_bin)=sqrt( etstat9(i_bin)**2+0.25
     &              *(etsys9up(i_bin)+etsys9down(i_bin))**2)
               eterr10(i_bin)=sqrt( etstat10(i_bin)**2+0.25
     &              *(etsys10up(i_bin)+etsys10down(i_bin))
     &              **2)
               eterr11(i_bin)=sqrt( etstat11(i_bin)**2+0.25
     &              *(etsys11up(i_bin)+etsys11down(i_bin))
     &              **2)
               eterr12(i_bin)=sqrt( etstat12(i_bin)**2+0.25
     &              *(etsys12up(i_bin)+etsys12down(i_bin))
     &              **2)
               
               etsigma1(i_bin)=etsigma1(i_bin)+ (etsys1up(i_bin)
     &              -etsys1down(i_bin))/2.0
               etsigma2(i_bin)=etsigma2(i_bin)+ (etsys2up(i_bin)
     &              -etsys2down(i_bin))/2.0
               etsigma3(i_bin)=etsigma3(i_bin)+ (etsys3up(i_bin)
     &              -etsys3down(i_bin))/2.0
               etsigma4(i_bin)=etsigma4(i_bin)+ (etsys4up(i_bin)
     &              -etsys4down(i_bin))/2.0
               etsigma5(i_bin)=etsigma5(i_bin)+ (etsys5up(i_bin)
     &              -etsys5down(i_bin))/2.0
               etsigma6(i_bin)=etsigma6(i_bin)+ (etsys6up(i_bin)
     &              -etsys6down(i_bin))/2.0
               etsigma7(i_bin)=etsigma7(i_bin)+ (etsys7up(i_bin)
     &              -etsys7down(i_bin))/2.0
               etsigma8(i_bin)=etsigma8(i_bin)+ (etsys8up(i_bin)
     &              -etsys8down(i_bin))/2.0
               etsigma9(i_bin)=etsigma9(i_bin)+ (etsys9up(i_bin)
     &              -etsys9down(i_bin))/2.0
               etsigma10(i_bin)=etsigma10(i_bin)+ (etsys10up(i_bin)
     &              -etsys10down(i_bin))/2.0
               etsigma11(i_bin)=etsigma11(i_bin)+ (etsys11up(i_bin)
     &              -etsys11down(i_bin))/2.0
               etsigma12(i_bin)=etsigma12(i_bin)+ (etsys12up(i_bin)
     &              -etsys12down(i_bin))/2.0


            ENDDO
            CALL HPAK(-19,etsigma1)
            CALL HPAKE(-19,eterr1)
            CALL HPAK(-20,etsigma2)
            CALL HPAKE(-20,eterr2)
            CALL HPAK(-21,etsigma3)
            CALL HPAKE(-21,eterr3)
            CALL HPAK(-22,etsigma4)
            CALL HPAKE(-22,eterr4)
            CALL HPAK(-23,etsigma5)
            CALL HPAKE(-23,eterr5)
            CALL HPAK(-24,etsigma6)
            CALL HPAKE(-24,eterr6)
            CALL HPAK(-25,etsigma7)
            CALL HPAKE(-25,eterr7)
            CALL HPAK(-26,etsigma8)
            CALL HPAKE(-26,eterr8)
            CALL HPAK(-27,etsigma9)
            CALL HPAKE(-27,eterr9)
            CALL HPAK(-28,etsigma10)
            CALL HPAKE(-28,eterr10)
            CALL HPAK(-29,etsigma11)
            CALL HPAKE(-29,eterr11)
            CALL HPAK(-30,etsigma12)
            CALL HPAKE(-30,eterr12)

c     Now the xgamma plots

c     xgamma
            CALL HBOOKB(-31
     &           ,'Data: xgamma a'
     &           ,n_xgbins1,xgbins1 ,0.)
            CALL HBOOKB(-32
     &           ,'Data: xgamma b'
     &           ,n_xgbins1,xgbins1 ,0.)
            CALL HBOOKB(-33
     &           ,'Data: xgamma c'
     &           ,n_xgbins2,xgbins2 ,0.)
            CALL HBOOKB(-34
     &           ,'Data: xgamma d'
     &           ,n_xgbins2,xgbins2 ,0.)


c     Loop over bins
            DO i_bin=1,n_xgbins1

               xgsys1up(i_bin)=sqrt(xgsys1up(i_bin))
               xgsys2up(i_bin)=sqrt(xgsys2up(i_bin))
               xgsys1down(i_bin)=sqrt(xgsys1down(i_bin))
               xgsys2down(i_bin)=sqrt(xgsys2down(i_bin))

               xgerr1(i_bin)=sqrt(xgstat1(i_bin)**2+0.25*(xgsys1up(i_bin
     &              )+xgsys1down(i_bin))**2)
               xgerr2(i_bin)=sqrt(xgstat2(i_bin)**2+0.25*(xgsys2up(i_bin
     &              )+xgsys2down(i_bin))**2)

               xgsigma1(i_bin)=xgsigma1(i_bin)+(xgsys1up(i_bin)
     &              -xgsys1down(i_bin))/2.0
               xgsigma2(i_bin)=xgsigma2(i_bin)+(xgsys2up(i_bin)
     &              -xgsys2down(i_bin))/2.0


            ENDDO
c     Loop over bins
            DO i_bin=1,n_xgbins2

               xgsys3up(i_bin)=sqrt(xgsys3up(i_bin))
               xgsys4up(i_bin)=sqrt(xgsys4up(i_bin))
               xgsys3down(i_bin)=sqrt(xgsys3down(i_bin))
               xgsys4down(i_bin)=sqrt(xgsys4down(i_bin))

               xgerr3(i_bin)=sqrt(xgstat3(i_bin)**2+0.25*(xgsys3up(i_bin
     &              )+xgsys3down(i_bin))**2)
               xgerr4(i_bin)=sqrt(xgstat4(i_bin)**2+0.25*(xgsys4up(i_bin
     &              )+xgsys4down(i_bin))**2)


               xgsigma3(i_bin)=xgsigma3(i_bin)+(xgsys3up(i_bin)
     &              -xgsys3down(i_bin))/2.0
               xgsigma4(i_bin)=xgsigma4(i_bin)+(xgsys4up(i_bin)
     &              -xgsys4down(i_bin))/2.0
               
            ENDDO
            CALL HPAK(-31,xgsigma1)
            CALL HPAKE(-31,xgerr1)
            CALL HPAK(-32,xgsigma2)
            CALL HPAKE(-32,xgerr2)
            CALL HPAK(-33,xgsigma3)
            CALL HPAKE(-33,xgerr3)
            CALL HPAK(-34,xgsigma4)
            CALL HPAKE(-34,xgerr4)


         ENDIF

         IF (iproc.eq.2) THEN
            IF (gen(1:3).NE.'HO1') THEN
               
               DO i_plot=11,34
                  
C     --- now add the two processes
                  CALL HOPERA(100+i_plot,'+e',200+i_plot,i_plot ,1.,1.)
               ENDDO
            ELSE

c     For NLO stuff, only need to take the bin width and a factor of
c     1000 and 0.5 into account. However, we must also deal with the
c     final round of integration first.
               DO i_plot=11,34
                  CALL HZSNLO(i_plot,i_plot+200,i_plot+100)
                  CALL HZHINRM(i_plot,0,1000.,1)
               ENDDO
               
            ENDIF      
            
         ENDIF
         
      ELSE
C     *** End ***
         PRINT*,'HZ01220:Please run routine with iflag set to 1,2 or 3'
         PRINT*,'        with +1000 or +2000 for DIR or RES events.'
      ENDIF

      RETURN
      END











