      subroutine HZ02079(IFLAG)
****************************************************************************
* Purpose: produced histograms for the following distributions:
*              - Et in three bins of eta_lab (-1./0.5/1.5/2.8): 5/10/20/40/70 GeV
*                (for eta between -1. and 0.5 only up to 40 GeV).
*                IDs 101-103
*              - Et for eta between 1.5 and 2.8 in bins of Q2 (5/10/20/35/70/100 GeV2)
*                IDs 201-205
*              - ET2/Q2 in the above  thre eta bins: 0.5/2./8./15./50./100.
*                IDs 301-303
*          Event selection:  
*              - 5 < Q2 < 100 GeV2 
*              - 0.2 < y < 0.6
*              - theta_e > 156
*              - Etbreit > 5GeV
*          Technical cuts:
*              - E-pz, zvtx, bdcdiff, ecra, Spacal fiducial cuts, 
*          Jet selection:
*              - incl. kT algorithm
*              - Etbreit > 5 GeV
*              - rap in lab between -1. and 2.8
*          References:
*              - T. Schoerner-Sadenius. Messung inklusiver Jet-Wirkungsquerschnitte 
*                                       mit dem H1-Detektor bei HERA. Dissertation 2001
*                                       (in german)
*              - H1 internal note H1-IN-584(05/2000)
*              - DESY ... HEP/EX ... ??????? 
* Arguments: iflag=1 initialisation
*            iflag=1 filling
*            iflag=3 termination
*
* written by   :  H. Jung on 27/03/2002
* Edit history :  T. Schoerner 21//04/2002 : 
*                    Completion and adaption to incl. jets analysis. 
*
* Produced histograms
*         IDs                   Purpose
*     101  -   103    Et distributions in eta bins to be filled by this code
*    -101  -  -103    Et distributions in eta bins DATA stat. err. only
*   -1101  - -1103    Et distributions in eta bins DATA stat. err. only
*     201  -   205    Et distributions forward in q2 bins to be filled by this code
*    -201  -  -205    Et distributions forward in q2 bins DATA stat. err. only
*   -1201  - -1205    Et distributions forward in q2 bins DATA stat. err. only
*     301  -   303    Et2/Q2 distributions in eta bins to be filled by this code
*    -301  -  -303    Et2/Q2 distributions in eta bins DATA stat. err. only
*   -1301  - -1303    Et2/Q2 distributions in eta bins DATA stat. err. only
****************************************************************************
*
      IMPLICIT NONE
*
****************************************
*     Sequences and HzTool functions
****************************************
*
*
#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzjetcmn.inc"
#include "hznlo.inc"
	
	Integer NLOJET
****************************************
*     User defined vars
****************************************
*
      Integer i,iflag
	Real nevt
*     DESY Preprint number
      Character*5 xxxx
      Data xxxx/'02079'/
      real sigm
      Integer ij,nj,ihep
      Double precision pbeam(4),pgam(4),ph(5),pbr(5)
      Double precision PLAB(5,NMXHEP)
	
      Integer numjet
      Parameter (numjet=200)
      Double precision pj(numjet,8),pjlab(numjet,8)
      Integer ierr
      Integer maxhi,modjet,maxjet
      Parameter (maxhi=100,modjet=3,maxjet=30)
      Real lumnb,lumpb,lum,sigma_tot
      Real ptq,raplab
      Double precision rcone
      Parameter (rcone=1.d0)

      Real pi,rd
      Integer iel,idum,ibeam
      Parameter (pi=3.1415927,rd=180./pi)
      Real x,y,q2
      real Ee, The, Phe, Ej, Thj, Phj, Ptj
*
*     Stuff for Et
*
      integer netback,net
      parameter (netback=3)
      parameter (net=4)
      real etbinback(netback+1)
      data etbinback/5.,10.,20.,40./
      real etbin(net+1)
      data etbin/5.,10.,20.,40.,70./
*
*     Stuff for Q2
*
      integer nq2
      parameter (nq2=5)
      real q2bin(nq2+1)
      data q2bin/5.,10.,20.,35.,70.,100./
*
*     Stuff for Et2/Q22
*
      Integer net2q2
      Parameter (net2q2=5)
      Real et2q2bin(net2q2+1)
      Data et2q2bin/0.5,2.,8.,15.,50.,100./
*
*     stuff for eta
*
      Integer neta
      Parameter (neta=3)
      Real eta(neta+1)
      Data eta/-1.,0.5,1.5,2.8/
*
*     selection limits
*
      Real ylow,yhigh
      Data ylow/0.2/
      data yhigh/0.6/
      Real q2low,q2high
      Data q2low/5./
      data q2high/100./
      Real etcut
      Data etcut/5./
      real thetcut
      data thetcut /156./
      real raplow,raphigh
      data raplow/-1./
      data raphigh/2.8/
*
*     selection flag
*
      Logical incljet
*     
***************************************************
*     Data points from plots
****************************************************
*     et backward data, stat. err. and tot. err.
      real etbw(3),xsetbw(3),xtetbw(3)
      data etbw / 561. , 31.3 , 0.43 /
      data xsetbw / 4. , 0.5 , 0.03 /
      data xtetbw / 34. , 2.4 , 0.06 /
*     et central data, stat. err. and tot. err.
      real etcn(4),xsetcn(4),xtetcn(4)
      data etcn / 272. , 28.2 , 1.4 , 0.017 /
      data xsetcn / 3. , 0.5 , 0.07 , 0.006 /
      data xtetcn / 20.2 , 2.2 , 0.13 , 0.004 /
*     et central data, stat. err. and tot. err.
      real etfw(4),xsetfw(4),xtetfw(4)
      data etfw / 238. , 21.4 , 0.94 , 0.026 /
      data xsetfw / 3. , 0.5 , 0.06 , 0.008 /
      data xtetfw / 18.0 , 1.7 , 0.10 , 0.005 /
*     et forward q2 5-10 data, stat. err. and tot. err.
      real eq1(4),xseq1(4),xteq1(4)
      data eq1 / 104. , 7.6 , 0.28 , 0.01 /
      data xseq1 / 3. , 0.4 , 0.03 , 0.005 /
      data xteq1 / 10. , 0.6 , 0.03 , 0.0025 /
*     et forward q2 10-20 data, stat. err. and tot. err.
      real eq2(4),xseq2(4),xteq2(4)
      data eq2 / 68.1 , 5.5 , 0.25 , 0.005 /
      data xseq2 / 1.6 , 0.2 , 0.03 , 0.003 /
      data xteq2 / 5.5 , 0.4 , 0.02 , 0.0015/
*     et forward q2 20-35 data, stat. err. and tot. err.
      real eq3(4),xseq3(4),xteq3(4)
      data eq3 / 34.3 , 3.7 , 0.17  , 0.01 /
      data xseq3 / 1.4 , 0.2 , 0.03 , 0.005 /
      data xteq3 / 3.7 , 0.35 , 0.025 , 0.0015 /
*     et forward q2 35-70 data, stat. err. and tot. err.
      real eq4(4),xseq4(4),xteq4(4)
      data eq4 / 28. , 3.5 , 0.16 , 0.005 /
      data xseq4 / 0.8 , 0.2 , 0.02 , 0.005 /
      data xteq4 / 2.0 , 0.35 , 0.025 , 0.0015 /
*     et forward q2 70-100 data, stat. err. and tot. err.
      real eq5(3),xseq5(3),xteq5(3)
      data eq5 / 6.5 , 1.03 , 0.055 /
      data xseq5 / 0.4 , 0. 08 , 0.015 /
      data xteq5 / 0.65 , 0.09 , 0.007 /
*     et2/q2 backward data, stat. err. and tot. err.
      real e2bw(5),xse2bw(5),xte2bw(5)
      data e2bw / 593. , 286. , 46.3 , 3.3 , 0.09 /
      data xse2bw / 6. , 3. , 0.9 , 0.1 , 0.01 /
      data xte2bw / 33.5 , 17.5 , 3.5 , 0.3 , 0.015 /
*     et2/q2 central data, stat. err. and tot. err.
      real e2cn(5),xse2cn(5),xte2cn(5)
      data e2cn / 280. , 149. , 28.9 , 3.3 , 0.22 /
      data xse2cn / 4. , 2. , 0.7 , 0.1 , 0.02 /
      data xte2cn / 20. , 10.5 , 2.3 , 0.25 , 0.025 /
*     et2/q2 central data, stat. err. and tot. err.
      real e2fw(5),xse2fw(5),xte2fw(5)
      data e2fw / 213. , 135. , 24.5 , 2.6 , 0.16 /
      data xse2fw / 5. , 2. , 0.8 , 0.1 , 0.02 /
      data xte2fw / 15. , 11.0 , 2.2 , 0.25 , 0.025 /
      
      save Nevt, NLOJET
*
************************************************
*     Init step
************************************************
*
      IF (iflag.eq.1) then
         Nevt=0
         Nevt=0
         NLO=0
	   NLOJET = 0
*
*     Initialisation: The following MUST always be done
*     (i) make subdirectory in PAWC
*     - use the name as the xxxxxx in HZxxxxxx subroutine
*     (i) make subdirectory in o/p file
*     
         Call hcdir('//PAWC',' ')
         call hmdir(xxxx,'S')
         Call hcdir('//HISTO',' ')
         call hmdir(xxxx,'S')
*
**********************************************
*     booking of histograms
**********************************************
*
*     distributions Et in eta bins
*
         call hbookb(101,'dsig/dEt -1<eta<0.5'
     &        ,netback,etbinback,0.0)
         call hbarx(101)
         call hbookb(-101,'H1 dsig/dEt -1<eta<0.5 stat err'
     &        ,netback,etbinback,0.0)
         call hbookb(-1101,'H1 dsig/dEt -1<eta<0.5 total err'
     &        ,netback,etbinback,0.0)
         call hbookb(102,'dsig/dEt 0.5<eta<1.5'
     &        ,net,etbin,0.0)
         call hbarx(102)
         call hbookb(-102,'H1 dsig/dEt 0.5<eta<1.5 stat err'
     &        ,net,etbin,0.0)
         call hbookb(-1102,'H1 d sig/dEt 0.5<eta<1.5 total err'
     &        ,net,etbin,0.0)
         call hbookb(103,'dsig/dEt 1.5<eta<2.8'
     &        ,net,etbin,0.0)
         call hbarx(103)
         call hbookb(-103,'H1 dsig/dEt 1.5<eta<2.8 stat err'
     &        ,net,etbin,0.0)
         call hbookb(-1103,'H1 dsig/dEt 1.5<eta<2.8 total err'
     &        ,net,etbin,0.0)
*     
*     distributions Et forward in Q2 bins
*     
         call hbookb(201,'dsig/dEt 1.5<eta<2.8 5<Q2<10'
     &        ,net,etbin,0.0)
         call hbarx(201)
         call hbookb(-201,'H1 dsig/dEt 1.5<eta<2.8 5<Q2<10 stat err'
     &        ,net,etbin,0.0)
         call hbookb(-1201,'H1 dsig/dEt 1.5<eta<2.8 5<Q2<10 tot err'
     &        ,net,etbin,0.0)
         call hbookb(202,'dsig/dEt 1.5<eta<2.8 10<Q2<20'
     &        ,net,etbin,0.0)
         call hbarx(202)
         call hbookb(-202,'H1 dsig/dEt 1.5<eta<2.8 10<Q2<20 stat err'
     &        ,net,etbin,0.0)
         call hbookb(-1202,'H1 dsig/dEt 1.5<eta<2.8 10<Q2<20 tot err'
     &        ,net,etbin,0.0)
         call hbookb(203,'dsig/dEt 1.5<eta<2.8 20<Q2<35'
     &        ,net,etbin,0.0)
         call hbarx(203)
         call hbookb(-203,'H1 dsig/dEt 1.5<eta<2.8 20<Q2<35 stat err '
     &        ,net,etbin,0.0)
         call hbookb(-1203,'H1 dsig/dEt 1.5<eta<2.8 20<Q2<35 tot err'
     &        ,net,etbin,0.0)
         call hbookb(204,'dsig/dEt 1.5<eta<2.8 35<Q2<70'
     &        ,net,etbin,0.0)
         call hbarx(204)
         call hbookb(-204,'H1 dsig/dEt 1.5<eta<2.8 35<Q2<70 stat err '
     &        ,net,etbin,0.0)
         call hbookb(-1204,'H1 d sig/dEt 1.5<eta<2.8 35<Q2<70 tot err'
     &        ,net,etbin,0.0)
         call hbookb(205,'d sig/dEt 1.5<eta<2.8 70<Q2<100'
     &        ,netback,etbinback,0.0)
         call hbarx(205)
         call hbookb(-205,'H1 dsig/dEt 1.5<eta<2.8 70<Q2<100 stat err'
     &        ,netback,etbinback,0.0)
         call hbookb(-1205,'H1 d sig/dEt 1.5<eta<2.8 70<Q2<100 tot err'
     &        ,netback,etbinback,0.0)
*     
*     distributions Et2/Q2 in eta bins
*
         call hbookb(301,'dsig/d(Et2/Q2) -1<eta<0.5'
     &        ,net2q2,et2q2bin,0.0)
         call hbarx(301)
         call hbookb(-301,'H1 dsig/d(Et2/Q2) -1<eta<0.5 stat err'
     &        ,net2q2,et2q2bin,0.0)
         call hbookb(-1301,'H1 dsig/d(Et2/Q2) -1<eta<0.5 tot err'
     &        ,net2q2,et2q2bin,0.0)
         call hbookb(302,'dsig/d(Et2/Q2) 0.5<eta<1.5'
     &        ,net2q2,et2q2bin,0.0)
         call hbarx(302)
         call hbookb(-302,'H1 dsig/d(Et2/Q2) 0.5<eta<1.5 stat err'
     &        ,net2q2,et2q2bin,0.0)
         call hbookb(-1302,'H1 dsig/d(Et2/Q2) 0.5<eta<1.5 tot err'
     &        ,net2q2,et2q2bin,0.0)
         call hbookb(303,'dsig/d(Et2/Q2) 1.5<eta<2.8'
     &        ,net2q2,et2q2bin,0.0)
         call hbarx(303)
         call hbookb(-303,'H1 dsig/d(Et2/Q2) 1.5<eta<2.8 stat err'
     &        ,net2q2,et2q2bin,0.0)
         call hbookb(-1303,'H1 dsig/d(Et2/Q2) 1.5<eta<2.8 tot err'
     &        ,net2q2,et2q2bin,0.0)
*
*     Make errors 
*
         do i=1,3
            xtetbw(i) = sqrt(xtetbw(i)**2 +   xsetbw(i)**2)
         enddo
         do i=1,4
            xtetcn(i) = sqrt(xtetcn(i)**2 + xsetcn(i)**2)
            xtetfw(i) = sqrt(xtetfw(i) **2 + xsetfw(i)**2)
         enddo
         do i=1,4
            xteq1(i) = sqrt(xteq1(i)**2 + xseq1(i)**2)
            xteq2(i) = sqrt(xteq2(i)**2 + xseq2(i)**2)
            xteq3(i) = sqrt(xteq3(i)**2 + xseq3(i)**2) 
            xteq4(i) = sqrt(xteq4(i)**2 + xseq4(i)**2) 
         enddo
         do i=1,3
            xteq5(i) = sqrt(xteq5(i)**2 + xseq5(i)**2) 
         enddo
         do i=1,5
            xte2bw(i) = sqrt(xte2bw(i)**2 + xse2bw(i)**2)
            xte2cn(i) = sqrt(xte2cn(i)**2 + xse2cn(i)**2) 
            xte2fw(i) = sqrt(xte2fw(i)**2 + xse2fw(i)**2) 
         enddo
*
*     Pack H1 data into histos
*     
         call hpak(-101,etbw)
         call hpake(-101,xsetbw)
         call hpak(-1101,etbw)
         call hpake(-1101,xtetbw)
         call hpak(-102,etcn)
         call hpake(-102,xsetcn)
         call hpak(-1102,etcn)
         call hpake(-1102,xtetcn)
         call hpak(-103,etfw)
         call hpake(-103,xsetfw)
         call hpak(-1103,etfw)
         call hpake(-1103,xtetfw)
*
         call hpak(-201,eq1)
         call hpake(-201,xseq1)
         call hpak(-1201,eq1)
         call hpake(-1201,xteq1)
         call hpak(-202,eq2)
         call hpake(-202,xseq2)
         call hpak(-1202,eq2)
         call hpake(-1202,xteq2)
         call hpak(-203,eq3)
         call hpake(-203,xseq3)
         call hpak(-1203,eq3)
         call hpake(-1203,xteq3)
         call hpak(-204,eq4)
         call hpake(-204,xseq4)
         call hpak(-1204,eq4)
         call hpake(-1204,xteq4)
         call hpak(-205,eq5)
         call hpake(-205,xseq5)
         call hpak(-1205,eq5)
         call hpake(-1205,xteq5)
*
         call hpak(-301,e2bw)
         call hpake(-301,xse2bw)
         call hpak(-1301,e2bw)
         call hpake(-1301,xte2bw)
         call hpak(-302,e2cn)
         call hpake(-302,xse2cn)
         call hpak(-1302,e2cn)
         call hpake(-1302,xte2cn)
         call hpak(-303,e2fw)
         call hpake(-303,xse2fw)
         call hpak(-1303,e2fw)
         call hpake(-1303,xte2fw)
*
*********************************************
*     Event processing
*********************************************
*
      Else if(iflag.eq.2) then
*     
*     Filling: The following MUST always be done
*     (i) move to the correct sub-directory in PAWC
*
         call hcdir('//PAWC/'//xxxx,' ')
*        
* allow for total xsection calculation
* 
         IF(NLO.eq.1.and.TOT.ne.1) GoTo 11112
            Nevt = Nevt + wtx
11112	   Continue
         if(Nevt.le.0) then
	      write(6,*) ' hz02079: NLO ',NLO,' TOT ',TOT,wtx,nevt
	   endif
	   NLOJET = DIJET + TRIPJET + QUADJET
*
         ierr=HZIBEAM(ibeam,idum)
*
         if (.not.(ierr.eq.1)) then
            write(6,*) 'HZ'//xxxx,' beams not found ! ',ibeam,idum
            goto 100
         else
            Do i=1,4
               pbeam(i)=PHEP(i,IBEAM)
            enddo
         endif
*     
*     get kinematics
*     
         q2=real(HZDISKIN(1))
         x =real(HZDISKIN(2))
         y =real(HZDISKIN(3))
*     
*     get electron
*     
         iel=HZIDELEC(idum)
         if (iel.eq.-1) then
            write(*,*) 'Hz'//xxxx,' electron not found '
            goto 11114
         endif
         Ee=real(PHEP(4,iel))
         The=real(HZPHMANG(PHEP(3,iel), sqrt(PHEP(1,iel)**2+PHEP(2,iel)
     +        **2)))*rd
         Phe = HZPHMANG(PHEP(1,iel),PHEP(2,iel))*rd
*     
********************************************
*     select DIS events
********************************************
*
         incljet=.true.
         if (y.lt.ylow) incljet=.false.
         if (y.gt.yhigh) incljet=.false.
         if (q2.lt.q2low) incljet=.false.
         if (q2.gt.q2high) incljet=.false.
         if (the.lt.thetcut) incljet=.false.
*
*     return if no DIS kinematics
*
         if(.not.incljet) goto 11114
	   IF((NLO.eq.1).AND.(NLOJET.eq.0)) goto 11114
*     
*     find photon for boost to Breit
*
         ierr=HZIPGAM(ph)
         if (ierr.eq.-1) then
            write(6,*) 'HZ'//xxxx,' boson vector not found ! '
            call VZERO(ph,5)
         else
            Do i=1,4
               pgam(i)=ph(i)
            enddo
         endif
*     
*     Initialize Breit frame boost
*
         call HZBRTINI(pbeam,pgam,ierr)
         if (ierr.eq.1) write(*,*) 'Hz'//xxxx,' Boost failed'
*     
*************************************************
*     prepare jet finding
*************************************************
*
*     Boost jet in the breit frame
*
         do 5 ihep=1,nhep
            Do i=1,4
               ph(i)=PHEP(i,ihep)
            enddo
*     
            Do i=1,5
               Plab(i,ihep)=PHEP(i,ihep)
            enddo
*
*     only boost stable particles and not the electron
*     (otherwise hzbrtola gives wrong result)
*     
            if (isthep(ihep).eq.1 .and. ihep.ne.iel) then
*     
               call HZBRT(ph,pbr,ierr)
               if (ierr.eq.1) then
                  write(6,*) 'HZ'//xxxx,' problem with boost to Breit !'
                  goto 100
               endif
*     
               Do i=1,4
                  PHEP(i,ihep)=Pbr(i)
               enddo
               PHEP(5,ihep)=0.d0
            endif
 5       continue
*     
*     restore the beam particles for the boost later back to lab
*     
         do ihep=1,2
            Do i=1,5
               PHEP(i,ihep)=Plab(i,ihep)
            enddo
         enddo
************************************************
*     do actual jet finding
*     modjet should be 3, rcone 1.
************************************************
         call hzjtfind(modjet,rcone,nj,pj)
*     
         if (nj.le.0) goto 100
         if (nj.gt.numjet) then
            write(6,*) 'hz',xxxx,' too many jets found ! '
            goto 100
         endif
*     
*     now boost jets back to lab frame
*     
         call hzbrtola(nj,pj,pjlab)	
*     
*     select jets (it is assumed that in pj they are ordered in pt)
*     
         do ij=1,nj
*     
            ej=real(pj(ij,4))
            ptj=real(pj(ij,3))
	    ptq=ptj**2/q2
	    raplab = real(pjlab(ij,1))
*
	    if(ptj.gt.etcut .and. 
     .           raplab.ge.raplow .and. raplab.lt.2.8) then
	       if(raplab.ge.eta(1).and.raplab.lt.eta(2)) then
                  call hfill(101,real(ptj),0.,wtx)
	          call hfill(301,real(ptq),0.,wtx)
	       elseif(raplab.ge.eta(2).and.raplab.lt.eta(3)) then
                  call hfill(102,real(ptj),0.,wtx)
	          call hfill(302,real(ptq),0.,wtx)
	       elseif(raplab.ge.eta(3).and.raplab.lt.eta(4)) then
                  call hfill(103,real(ptj),0.,wtx)
	          call hfill(303,real(ptq),0.,wtx)
                  do i=1,5
                     if(q2.ge.q2bin(i) .and. q2.lt.q2bin(i+1)) then
                        call hfill(200+i,real(ptj),1.,wtx)
                     endif                                  ! ask for specific Q2 bin
                  enddo                                     ! loop over 5 Q2 bins
               endif                                        ! rapidity cut
	    endif                                           ! ptj cut on jets
	 enddo                                              ! loop over jets
*
*     ready with jet selection and histo filling
*     
 100     Continue
       do ihep=1,nhep
           Do i=1,5
             PHEP(i,ihep)=Plab(i,ihep)
           enddo
       enddo
11114	 continue		 
*     
********************************************
*     Termination and normalisation
********************************************
*
      Else if(iflag.eq.3) then
*
*     Termination: The following MUST always be done
*     (i) Move to the correct PAW subdirectory
*     
         call hcdir('//PAWC/'//xxxx,' ')
*
         Write(6,*) 'xz'//xxxx,' X-section is',xsec
         lum=999999999.
*
*     x section in nb
*
         sigma_tot=xsec/1000.
         if (sigma_tot.ne.0.) then
            lum=real(nevt)/real(sigma_tot)
            lumpb=real(nevt)/real(xsec)
         else
            write(6,*) 'hz',xxxx,' sigma_tot=0 ! '
         endif
*     
*     Print total cross-section
*     
         write(6,*) 'hz',xxxx,' sigma_tot ',sigma_tot
*     
*     norm et and et2/q2 in eta bins
*
         do i=1,3
c            call hzhinrm(100+i, 0, 1./lum, 1 )
c            call hzhinrm(300+i, 0, 1./lum, 1 )
            call hzhinrm(100+i, 0, lumpb, 1 )
            call hzhinrm(300+i, 0, lumpb, 1 )
         enddo
*
*     norm et forward in Q2 bins 
*
         do i=1,5
c            call hzhinrm(200+i, 0, 1./lum, 1 )
            call hzhinrm(200+i, 0, lumpb, 1 )
         enddo
*
      endif                    !      iflag.eq.3
*     
      RETURN
*     
      END
