      SUBROUTINE hz07062(IFLAG)
****************************************************************************
*     [For subroutine naming conventions, see the hztool manual.]
*
*     Purpose: This routine reproduces the data from Fig.Xa, Yb, Yc and
*     Table Za from [reference], and produces histograms of simulated data
*     which can be compared to them.
*
*     Initial state: XX GeV things vs YY GeV other things.
*
*     Reference: [reference]
*     Arguments: iflag=10000*[A]+1000*[B]+100*[C]+10*[D]+1 initialisation
*                iflag=10000*[A]+1000*[B]+100*[C]+10*[D]+2 filling
*                iflag=10000*[A]+1000*[B]+100*[C]+10*[D]+3 termination
*
*     [A]= IPS   >0 to run on the final state of the parton shower, 0 to run on
*                final state particles. 
*     [B]= IPROC decides which process is being run when more than one is 
*                generated (e.g. single/double resolved and direct or 
*                diffractive and non-diffractive). Leave this to 0 if there is 
*                only one process type.
*     [C]= IRUN  =0 rerun jetfinder, =1 use results already stored.
*     [D]= JETF  decides jet finder type. Ignored if IRUN=1. If set to 0 the 
*                finder set by chjet below is used. This should be the finder
*                used in the paper (only necessary if using jets!).
*     See manual section "The analysis routines" for more details.
*     NB If you do not implement all these options, change the comments to say 
*     so!
*
*     It is assumed you run either (a) with iproc = 0
*                               or (b) with iproc = 1, 2... in order.
*     This example has only 2 processes (iproc=1,2) but others may be added.
*
*     Histograms: -N data for Fig.Xa
*                  N simulation for Fig.Xa
*                  ... etc.
*
*     written by: [me] on [today]
****************************************************************************
      IMPLICIT NONE

****** These included files contain function declarations and various 
****** parameter declarations which are required by the hztool routines.
****** In particular Xsec and Ntot (global variables) are defined in 
****** 'heracmn.inc'

#include "hzfunc.inc"
****** for histograms
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
****** (only necessary if using jets!)
#include "hzjetcmn.inc"
C hztool nlo common
      INTEGER NLO, TOT, DIJET, TRIPJET, QUADJET
      COMMON /HZNLO/NLO, TOT, DIJET, TRIPJET, QUADJET
	
	Integer NLOJET
	Real nevt
      DOUBLE PRECISION Jetslab(maxhzjets,8)

****** declare variables
      INTEGER ihep,iflag,chjet,iproc,njet,iloop
      Integer jgam,jbeam,ilep,ilepp,ipro
      Integer i,ierr,idum
      Double precision pbeam(4),pgam(4),pcms(4),ppro(4),ppho(4),ph(4)
      Double precision plab(5,NMXHEP),plep(4),plepp(4)
      
      Double Precision rap,rap1,rap2,rap3,Et,Et1,et2,et3,deltaphi,dph
      Double Precision xnorm,xnorm1
      Real px1,px2,py1,py2,sumpt,difpt
      Integer njet_acc
      Double Precision Pjet(4),Pjlab(4),eta
      Integer ijet,ipoint(100),ixbin,ihist
      REAL x,y,q2
      
      Real phi1,phi2,phi3,delta3rd
      REAL PI,RD
      PARAMETER (PI=3.1415927,RD=180./PI)
      
      DOUBLE PRECISION coner
      SAVE coner

      Double Precision STMAX1,STMAX2
      common/myscale/STMAX1,STMAX2
      Real nev801
      
      CHARACTER*8 xxxx
      logical ljets,ldis,hcm
****** initialize variables
      real q2min,q2max,ymin,ymax,xmin,xmax
      DATA q2min/10./
      DATA q2max/100./
      Real xbins(6),qbins(6)
      Data xbins/1.7e-4,3.0e-4,5.0e-4,10.e-4,25.e-4,100.e-4/
      Data qbins/10.,15.,20.,30.,50.,100./
      Integer nx,nq
      Data nx/5/,nq/5/
      Integer net
      Data net/8/
      Real etbins(9)
      Data etbins/5.,8.,12.,15.,20.,25.,30.,40.,60./
      integer nsumpt
      Data nsumpt/4/
      Real sumptbins(5)
      Data sumptbins/0.,4.,10.,16.,100./
      integer ndifpt
      data ndifpt/4/
      Real difptbins(5)
      Data difptbins/0.,0.5,0.7,0.85,1./
      Data ymin/0.1/
      Data ymax/0.6/
      Data xmin/0.0001/
      Data xmax/0.01/
      Real etamin,etamax,Et1min,Et2min
      Data etamin/-1.0/
      Data etamax/2.5/
      Data Et1min/7./
      Data Et2min/5./
      
      Real xbin(6)
      data xbin/0.00017,0.0003,0.0005,0.001,0.0025,0.01/
      Real xwidth(5)
      Data hcm/.true./
      
      Real phibin2(5)
      data phibin2/0.,0.786,1.571,2.356,3.1415/
      Real phibin3(4)
      data phibin3/0.,1.571,2.356,3.1415/
      
      
      Real sp21(4),sp21stat(4),sp21syu(4),sp21syd(4),sp21tot(4)
      data sp21/6.6e3,2.21e4,1.02e5,9.46e5/
      data sp21stat/1.8e3,3.5e3,7.2e3,2.0e4/
      data sp21syu/5.3e3,5.0e3,1.9e4,5.9e3/
      data sp21syd/3.3e3,3.8e3,4.2e3,7.9e3/
      Real sp22(4),sp22stat(4),sp22syu(4),sp22syd(4),sp22tot(4)
      data sp22/1.18e4,2.9e4,1.39e5,1.265e6/
      data sp22stat/2.4e3,3.4e3,6.9e3,1.8e4/
      data sp22syu/2.3e3,5.9e3,1.4e4,5.7e4/
      data sp22syd/4.7e3,1.1e3,1.3e4,5.1e4/
      Real sp23(4),sp23stat(4),sp23syu(4),sp23syd(4),sp23tot(4)
      data sp23/6.9e3,1.84e4,9.0e4,9.35e5/
      data sp23stat/1.0e3,1.7e3,3.6e3,1.0e4/
      data sp23syu/1.2e3,2.1e3,3.3e3,2.9e4/
      data sp23syd/2.9e3,4.3e3,5.9e3,3.0e4/
      Real sp24(4),sp24stat(4),sp24syu(4),sp24syd(4),sp24tot(4)
      data sp24/2.13e3,4.42e3,2.72e4,3.37e5/
      data sp24stat/3.0e2,3.6e2,1.1e3,3.2e3/
      data sp24syu/3.8e2,7.4e2,1.7e3,7.0e3/
      data sp24syd/7.0e2,6.2e2,5.2e2,7.7e3/
      Real sp25(4),sp25stat(4),sp25syu(4),sp25syd(4),sp25tot(4)
      data sp25/98,354,1.74e3,2.63e4/
      data sp25stat/22,53,1.2e2,4.1e2/
      data sp25syu/27,55,4.0e2,9.8e2/
      data sp25syd/34,97,1.9e2,5.9e2/
      
      Real sp31(3),sp31stat(3),sp31syu(3),sp31syd(3),sp31tot(3)
      data sp31/4.9e3,3.76e4,1.37e5/
      data sp31stat/1.0e3,4.0e3,7.2e3/
      data sp31syu/6.5e2,9.7e3,1.5e4/
      data sp31syd/3.5e3,8.6e3,3.3e4/
      Real sp32(3),sp32stat(3),sp32syu(3),sp32syd(3),sp32tot(3)
      data sp32/5.9e3,4.76e4,1.42e5/
      data sp32stat/8.6e2,3.7e3,5.5e3/ 
      data sp32syu/1.5e3,5.1e3,2.1e4/
      data sp32syd/3.9e3,5.2e3,2.2e4/
      Real sp33(3),sp33stat(3),sp33syu(3),sp33syd(3),sp33tot(3)
      data sp33/3.5e3,3.08e4,8.5e4/
      data sp33stat/4.4e2,2.0e3,2.9e3/
      data sp33syu/5.2e2,1.4e3,1.4e4/
      data sp33syd/1.4e3,5.3e3,6.9e3/
      Real sp34(3),sp34stat(3),sp34syu(3),sp34syd(3),sp34tot(3)
      data sp34/772,8.6e3,3.25e4/
      data sp34stat/103,6.0e2,1.1e3/
      data sp34syu/138,1.0e3,1.3e3/
      data sp34syd/65, 4.3e3,2.5e3/
      Real sp35(3),sp35stat(3),sp35syu(3),sp35syd(3),sp35tot(3)
      data sp35/ 42,548,1.83e3/
      data sp35stat/11, 81,1.5e2/
      data sp35syu/7,221,2.8e2/
      data sp35syd/9,78, 2e2/
      
      Real x2sec1(4),x21stat(4),x21sysu(4),x21sysd(4),x21tot(4)
      
      Data x2sec1/1.145e5,5.47e4,1.1e4,164/
      Data x21stat/3.7e3,  1.8e3, 7.5e2, 25/
      Data x21sysu/8.3e3,  3.5e3, 1.2e3, 36/ 
      Data x21sysd/8.1e3,  6.0e3, 2.1e3, 19/

      Real x2sec2(4),x22stat(4),x22sysu(4),x22sysd(4),x22tot(4)
      Data x2sec2/1.597e5,7.02e4,1.2e4, 162/
      Data x22stat/3.4e3,  1.6e3, 5.9e2, 16 /
      Data x22sysu/6.9e3,  4.8e3, 1.2e3, 52 / 
      Data x22sysd/4.9e3,  4.1e3, 3.3e3, 31 /
      
      Real x2sec3(4),x23stat(4),x23sysu(4),x23sysd(4),x23tot(4)
      Data x2sec3/1.194e5, 4.88e4, 7.5e3, 130/	  
      Data x23stat/1.9e3,   8.4e2,  3.0e2,  10/
      Data x23sysu/3.9e3,   2.1e3,  9.2e2,  10/
      Data x23sysd/3.5e3,   3.9e3,  1.3e3,  41/ 

      Real x2sec4(4),x24stat(4),x24sysu(4),x24sysd(4),x24tot(4)
      Data x2sec4/4.44e4, 1.54e4, 2.52e3, 37/
      Data x24stat/6.0e2,  2.5e2,  9.7e1,  3/
      Data x24sysu/1.8e3,  5.0e2,  1.6e2,  5/
      Data x24sysd/1.7e3,  5.1e2,  4.9e2,  12/

      Real x2sec5(4),x25stat(4),x25sysu(4),x25sysd(4),x25tot(4)
      Data x2sec5/3.73e3,  921,  153,  2.68/  
      Data x25stat/8.2e1,    26,   11,  0.35/
      Data x25sysu/1.1e2 ,   86,   16,  0.32 /
      Data x25sysd/1.3e2,    39,   27,  0.71 /
      
      Real x3sec1(4),x31stat(4),x31sysu(4),x31sysd(4),x31tot(4)
      Data x3sec1/ 6.7e3, 1.43e4, 5.3e3, 89  /
      Data x31stat/ 8.2e2, 9.6e2,  4.9e2, 17/
      Data x31sysu/ 1.3e3, 1.9e3,  3.6e2,  7 / 
      Data x31sysd/ 2.3e3, 3.3e3,  1.4e3,  23/

      Real x3sec2(4),x32stat(4),x32sysu(4),x32sysd(4),x32tot(4)
      Data x3sec2/ 6.8e3, 1.55e4, 5.7e3, 74  /
      Data x32stat/ 6.2e2, 7.4e2,  4.1e2, 10/
      Data x32sysu/ 1.4e3, 2.9e3,  4.3e2, 18/
      Data x32sysd/ 9.3e2, 2.7e3,  1.9e3, 37/

      Real x3sec3(4),x33stat(4),x33sysu(4),x33sysd(4),x33tot(4)
      Data x3sec3/4.17e3, 9.7e3, 3.04e3, 54/  	  
      Data x33stat/3.1e2,  4.0e2, 2.0e2,  6 / 
      Data x33sysu/4.9e2,  1.7e3 ,1.9e2,  3 / 
      Data x33sysd/4.4e2,  9.0e2, 5.6e2,  23 / 

      Real x3sec4(4),x34stat(4),x34sysu(4),x34sysd(4),x34tot(4)
      Data x3sec4/1.46e3, 1.2e3, 945, 15 / 
      Data x34stat/1.2e2,  1.5e2, 66,  1.8/
      Data x34sysu/1.6e2,  1.8e2, 51 , 2.1/
      Data x34sysd/2.2e2 , 1.7e2, 159, 3.6/

      Real x3sec5(4),x35stat(4),x35sysu(4),x35sysd(4),x35tot(4)
      Data x3sec5/123, 188, 48 ,0.43 /  
      Data x35stat/21,   17,  7, 0.15/
      Data x35sysu/17,   44, 11, 0.11 /  
      Data x35sysd/66,   17, 16, 0.21 / 
      
      
      
      
      Real xx2sec1(4),xx21stat(4),xx21sysu(4),xx21sysd(4),xx21tot(4)
      Data xx2sec1/ 2.2e4 ,1.95e5, 2.14e6, 3.28e6 / 
      Data xx21stat/4.4e3 ,2.1e4 , 1.0e5 , 9.0e4/
      Data xx21sysu/6.1e3 ,4.6e4 , 1.6e5 , 2.7e5/
      Data xx21sysd/3.3e3 ,2.3e4 , 4.6e5 , 1.8e5/

      Real xx2sec2(4),xx22stat(4),xx22sysu(4),xx22sysd(4),xx22tot(4)
      Data xx2sec2/ 3.5e4 ,2.77e5, 2.62e6, 4.54e6/
      Data xx22stat/ 5.0e3, 2.1e4,  8.2e4,  8.4e4/
      Data xx22sysu/ 9.3e3, 7.6e4,  2.1e5,  3.6e5/
      Data xx22sysd/ 1.4e4, 7.7e4,  4.6e5,  1.2e5/

      Real xx2sec3(4),xx23stat(4),xx23sysu(4),xx23sysd(4),xx23tot(4)
      Data xx2sec3/2.18e4, 1.61e5, 1.87e6, 3.37e6 /
      Data xx23stat/2.3e3 , 9.2e3,  4.4e4,  4.7e4/
      Data xx23sysu/ 4.6e3,  3.1e4,  8.2e4,  9.1e4/
      Data xx23sysd/ 6.3e3,  2.5e4,  2.8e4,  6.2e4/

      Real xx2sec4(4),xx24stat(4),xx24sysu(4),xx24sysd(4),xx24tot(4)
      Data xx2sec4/ 5.5e3, 4.43e4, 6.05e5, 1.272e6/
      Data xx24stat/ 5.3e2, 2.4e3,  1.3e4,  1.6e4/
      Data xx24sysu/ 1.1e3, 5.2e3,  1.3e4,  3.4e4/
      Data xx24sysd/ 1.4e3, 3.8e3,  5.3e4,  2.9e4/

      Real xx2sec5(4),xx25stat(4),xx25sysu(4),xx25sysd(4),xx25tot(4) 
      Data xx2sec5/ 341, 3.34e3, 3.94e4, 1.065e5 /  
      Data xx25stat/ 55 , 3.2e2,  1.4e3,  2.2e3 /
      Data xx25sysu/ 104, 2.8e2,  1.5e3, 3.2e3/ 
      Data xx25sysd/ 83,  3.5e2,  2.2e3, 3.1e3/


      Real xx3sec1(4),xx31stat(4),xx31sysu(4),xx31sysd(4),xx31tot(4)
      Data xx3sec1/ 0, 6.5e4, 4.5e5, 4.1e5  / 
      Data xx31stat/0, 1.2e4, 3.8e4, 3.1e4  /
      Data xx31sysu/ 0, 2.7e3, 1.3e5, 2.7e4/
      Data xx31sysd/0, 2.4e4, 1.1e5, 1.2e5/

      Real xx3sec2(4),xx32stat(4),xx32sysu(4),xx32sysd(4),xx32tot(4)
      Data xx3sec2/ 8.5e3, 7.8e4, 5.31e5, 4.1e5/   
      Data xx32stat/ 2.0e3, 1.0e4, 3.3e4, 2.2e4/
      Data xx32sysu/ 4.2e3, 1.4e4, 5.9e4, 8.0e4/
      Data xx32sysd/ 5.7e3, 5.1e4, 9.0e4, 4.8e4/

      Real xx3sec3(4),xx33stat(4),xx33sysu(4),xx33sysd(4),xx33tot(4)
      Data xx3sec3/4.7e3, 5.2e4, 3.02e5, 2.57e5/
      Data xx33stat/ 9.1e2, 5.9e3, 1.6e4,  1.2e4/
      Data xx33sysu/ 1.2e3, 3.2e3, 4.9e4,  1.9e4/
      Data xx33sysd/ 2.8e3, 1.5e4, 2.7e4, 2.2e4/

      Real xx3sec4(4),xx34stat(4),xx34sysu(4),xx34sysd(4),xx34tot(4)
      Data xx3sec4/ 1.18e3, 1.08e4, 1.15e5, 9.3e4/
      Data xx34stat/ 2.4e2,  1.3e3,  6.1e3,  4.4e3/
      Data xx34sysu/ 3.3e2 , 1.5e3,  4.9e3,  7.9e3 /
      Data xx34sysd/ 3.2e3 , 3.5e3,  9.9e3,  4.8e3 /

      Real xx3sec5(4),xx35stat(4),xx35sysu(4),xx35sysd(4),xx35tot(4)
      Data xx3sec5/0, 640, 6.4e3, 5.7e3   /
      Data xx35stat/ 0, 158, 7.1e2, 6.4e2/
      Data xx35sysu/ 0, 264, 1.7e3, 7.3e2/
      Data xx35sysd/ 0, 260, 6.5e2, 1.3e3/


      
c      integer err_flag
c	real p,pt,etr,th,rapr
     
****** save variables that retain their values between different subroutines
      SAVE chjet

****** (only necessary if using jets!)
****** Set the default jet finder as that in the paper.
****** (see the hztool manual section "The Jet Finders" for the different jet
******  finders available).
      DATA chjet/3/

****** Set xxxx equal to the arXiv:hep-ex number.
******  [For alternative subroutine naming conventions, see the manual.]
      DATA xxxx /'hz07062'/

****** set the jet cone radius. (only necessary if using jets!)
****** CALL hzjetrad(1,coner) sets the radius to coner
****** CALL hzjetrad(2,coner) sets coner to the radius, or -1. 
******  check if the radius has been set eleswhere
      CALL hzjetrad(2,coner)
******  if not set it to the value used in the paper.
      IF (coner.LT.0.0) THEN
	      coner=1.0
      ENDIF

****** Determine the process type (single/double resolved etc)
****** (only necessary if > 1 process types!)
      iproc=(mod(iflag,10000)/1000)
      IF ((iproc.LT.0).OR.(iproc.GT.2)) THEN
         WRITE(6,*) xxxx,': illegal IPROC:',iproc
         RETURN
      ENDIF

****** choose jet finder (only necessary if using jets!)
c      IF ((MOD(iflag,100)/10.GT.0).AND.MOD(iflag,10).eq.1) THEN
c         chjet=INT(MOD(iflag,100)/10)
c      ENDIF

****** Decide if we are running on parton showers or final state particles.
      IF (MOD(iflag,100000)/10000.GT.0) THEN
****** use a different paw directory for parton showers.
         xxxx = 'PS'//xxxx
****** tell jet finder that we running on parton showers 
****** (only necessary if using jets!)
         chjet=-1*ABS(chjet)
      ENDIF
******************************************************************************
********************************  Initialise ********************************* 
      IF (MOD(iflag,10).EQ.1) THEN
****** Initialisation: The following MUST always be done
******     (i) make subdirectory in PAWC
******     - use the name as the xxxxxx in HZHxxxxxx subroutine
******     (i) make subdirectory in o/p file
         
         CALL hcdir('//PAWC',' ')
         IF (iproc.LE.1) THEN
             CALL hmdir(xxxx,'S')
         ELSE 
             CALL hcdir(xxxx,' ')
         ENDIF
 
         CALL hcdir('//HISTO',' ')
         IF (iproc.LE.1) THEN
             CALL hmdir(xxxx,'S')
         ELSE 
             CALL hcdir(xxxx,' ')
         ENDIF
         Nevt=0
         Nev801=0
         NLO=0
	   NLOJET = 0
     
******  book your histograms using:
******   1D histo
******     CALL hbook1(long key, char* title, long bins,  float xmin,  
******                 float xmax, float VMX) 
******   2D histo
******     CALL hbook2(long key, char* title, long bins1, float xmin1, 
******                 float xmax1, long bins2, float xmin2, float xmax2, 
******                 float VMX) 
******  1D histo with variable bin width
******     CALL hbookb(long key, char* title, long bins,  float* xvalues, 
******                 float VMX)
******
******  VMX upper limit of single channel content, set to 0.0 which  means 1 
******  word per channel (no packing).  

******  Give key a different value depending on which process you are running 
******  (only necessary if > 1 process types!)
c        call hbookb(10,' dijet x ',nx,xbins,0.)
c        call hbookb(11,' 3jet x ',nx,xbins,0.)
c        call hbookb(20,' dijet q2',nq,qbins,0.)
c        call hbookb(21,' dijet q2',nq,qbins,0.)
c        CALL hbook1(101,'dijet rap1',7,-1.0,2.5,0.)
c        CALL hbook1(102,'dijet rap2',7,-1.0,2.5,0.)
c        CALL hbook1(111,'3-jet rap1',7,-1.0,2.5,0.)
c        CALL hbook1(112,'3-jet rap2',7,-1.0,2.5,0.)
c        CALL hbook1(113,'3-jet rap3',7,-1.0,2.5,0.)
c        CALL hbookb(201,'dijet Et1',net,etbins,0.)
c        CALL hbookb(202,'dijet Et2',net,etbins,0.)
c        CALL hbookb(211,'3-jet Et1',net,etbins,0.)
c        CALL hbookb(212,'3-jet Et2',net,etbins,0.)
c        CALL hbookb(213,'3-jet Et3',net,etbins,0.)
        CALL hbookb(301,' x1 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(302,' x2 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(303,' x3 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(304,' x4 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(305,' x5 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(401,' x1 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(402,' x2 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(403,' x3 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(404,' x4 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(405,' x5 delta phi 3-jet ',3,phibin3,0.)
        
        CALL hbookb(751,' x1 sumpt dijet ',nsumpt,sumptbins,0.)
        CALL hbookb(752,' x2 sumpt dijet ',nsumpt,sumptbins,0.)
        CALL hbookb(753,' x3 sumpt dijet ',nsumpt,sumptbins,0.)
        CALL hbookb(754,' x4 sumpt dijet ',nsumpt,sumptbins,0.)
        CALL hbookb(755,' x5 sumpt dijet ',nsumpt,sumptbins,0.)
        CALL hbookb(761,' x1 sumpt trijet ',nsumpt,sumptbins,0.)
        CALL hbookb(762,' x2 sumpt trijet ',nsumpt,sumptbins,0.)
        CALL hbookb(763,' x3 sumpt trijet ',nsumpt,sumptbins,0.)
        CALL hbookb(764,' x4 sumpt trijet ',nsumpt,sumptbins,0.)
        CALL hbookb(765,' x5 sumpt trijet ',nsumpt,sumptbins,0.)
        
        CALL hbookb(851,' x1 difpt dijet ',ndifpt,difptbins,0.)
        CALL hbookb(852,' x2 difpt dijet ',ndifpt,difptbins,0.)
        CALL hbookb(853,' x3 difpt dijet ',ndifpt,difptbins,0.)
        CALL hbookb(854,' x4 difpt dijet ',ndifpt,difptbins,0.)
        CALL hbookb(855,' x5 difpt dijet ',ndifpt,difptbins,0.)
        CALL hbookb(861,' x1 difpt trijet ',ndifpt,difptbins,0.)
        CALL hbookb(862,' x2 difpt trijet ',ndifpt,difptbins,0.)
        CALL hbookb(863,' x3 difpt trijet ',ndifpt,difptbins,0.)
        CALL hbookb(864,' x4 difpt trijet ',ndifpt,difptbins,0.)
        CALL hbookb(865,' x5 difpt trijet ',ndifpt,difptbins,0.)
        
        
c        CALL hbook1(501,' x1 delta phi of 3rd-jet low',4,0.0,3.15,0.)
c        CALL hbook1(502,' x2 delta phi of 3rd-jet low',4,0.0,3.15,0.)
c        CALL hbook1(503,' x3 delta phi of 3rd-jet low',4,0.0,3.15,0.)
c        CALL hbook1(504,' x4 delta phi of 3rd-jet low',4,0.0,3.15,0.)
c        CALL hbook1(505,' x5 delta phi of 3rd-jet low',4,0.0,3.15,0.)
c        CALL hbook1(511,' x1 delta phi of 3rd-jet hig',4,0.0,3.15,0.)
c        CALL hbook1(512,' x2 delta phi of 3rd-jet hig',4,0.0,3.15,0.)
c        CALL hbook1(513,' x3 delta phi of 3rd-jet hig',4,0.0,3.15,0.)
c        CALL hbook1(514,' x4 delta phi of 3rd-jet hig',4,0.0,3.15,0.)
c        CALL hbook1(515,' x5 delta phi of 3rd-jet hig',4,0.0,3.15,0.)
c        CALL hbook1(601,' x1 jet mult delta phi low',21,-0.5,20.5,0.)
c        CALL hbook1(602,' x2 jet mult delta phi low',21,-0.5,20.5,0.)
c        CALL hbook1(603,' x3 jet mult delta phi low',21,-0.5,20.5,0.)
c        CALL hbook1(604,' x4 jet mult delta phi low',21,-0.5,20.5,0.)
c        CALL hbook1(605,' x5 jet mult delta phi low',21,-0.5,20.5,0.)
c        CALL hbook1(611,' x1 jet mult delta phi hig',21,-0.5,20.5,0.)
c        CALL hbook1(612,' x2 jet mult delta phi hig',21,-0.5,20.5,0.)
c        CALL hbook1(613,' x3 jet mult delta phi hig',21,-0.5,20.5,0.)
c        CALL hbook1(614,' x4 jet mult delta phi hig',21,-0.5,20.5,0.)
c        CALL hbook1(615,' x5 jet mult delta phi hig',21,-0.5,20.5,0.)
        
c        call hbook1(801,' stmax ',100,0.,5.,0.)       
c        call hbook2(802,' stmax vrs Q2 ',100,0.,500.,100,0.,500.,0.)       
c        call hbook2(803,' stmax**2 vrs Q2 ',100,0.,2000.,100,0.,500.,0.)       
        
        CALL hbookb(-301,' x1 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(-302,' x2 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(-303,' x3 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(-304,' x4 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(-305,' x5 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(-401,' x1 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(-402,' x2 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(-403,' x3 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(-404,' x4 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(-405,' x5 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(-1301,' x1 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(-1302,' x2 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(-1303,' x3 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(-1304,' x4 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(-1305,' x5 delta phi dijet ',4,phibin2,0.)
        CALL hbookb(-1401,' x1 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(-1402,' x2 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(-1403,' x3 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(-1404,' x4 delta phi 3-jet ',3,phibin3,0.)
        CALL hbookb(-1405,' x5 delta phi 3-jet ',3,phibin3,0.)

        CALL hbookb(-751,' x1 sumpt dijet ',nsumpt,sumptbins,0.)
        CALL hbookb(-752,' x2 sumpt dijet ',nsumpt,sumptbins,0.)
        CALL hbookb(-753,' x3 sumpt dijet ',nsumpt,sumptbins,0.)
        CALL hbookb(-754,' x4 sumpt dijet ',nsumpt,sumptbins,0.)
        CALL hbookb(-755,' x5 sumpt dijet ',nsumpt,sumptbins,0.)
        CALL hbookb(-761,' x1 sumpt trijet ',nsumpt,sumptbins,0.)
        CALL hbookb(-762,' x2 sumpt trijet ',nsumpt,sumptbins,0.)
        CALL hbookb(-763,' x3 sumpt trijet ',nsumpt,sumptbins,0.)
        CALL hbookb(-764,' x4 sumpt trijet ',nsumpt,sumptbins,0.)
        CALL hbookb(-765,' x5 sumpt trijet ',nsumpt,sumptbins,0.)
        
        CALL hbookb(-851,' x1 difpt dijet ',ndifpt,difptbins,0.)
        CALL hbookb(-852,' x2 difpt dijet ',ndifpt,difptbins,0.)
        CALL hbookb(-853,' x3 difpt dijet ',ndifpt,difptbins,0.)
        CALL hbookb(-854,' x4 difpt dijet ',ndifpt,difptbins,0.)
        CALL hbookb(-855,' x5 difpt dijet ',ndifpt,difptbins,0.)
        CALL hbookb(-861,' x1 difpt trijet ',ndifpt,difptbins,0.)
        CALL hbookb(-862,' x2 difpt trijet ',ndifpt,difptbins,0.)
        CALL hbookb(-863,' x3 difpt trijet ',ndifpt,difptbins,0.)
        CALL hbookb(-864,' x4 difpt trijet ',ndifpt,difptbins,0.)
        CALL hbookb(-865,' x5 difpt trijet ',ndifpt,difptbins,0.)
        
        
        
        do i=1,4 
        sp21tot(i)=sqrt(sp21stat(i)**2+((sp21syu(i)+sp21syd(i))/2)**2)
        sp22tot(i)=sqrt(sp22stat(i)**2+((sp22syu(i)+sp22syd(i))/2)**2)
        sp23tot(i)=sqrt(sp23stat(i)**2+((sp23syu(i)+sp23syd(i))/2)**2)
        sp24tot(i)=sqrt(sp24stat(i)**2+((sp24syu(i)+sp24syd(i))/2)**2)
        sp25tot(i)=sqrt(sp25stat(i)**2+((sp25syu(i)+sp25syd(i))/2)**2)
        end do
        do i=1,3 
        sp31tot(i)=sqrt(sp31stat(i)**2+((sp31syu(i)+sp31syd(i))/2)**2)
        sp32tot(i)=sqrt(sp32stat(i)**2+((sp32syu(i)+sp32syd(i))/2)**2)
        sp33tot(i)=sqrt(sp33stat(i)**2+((sp33syu(i)+sp33syd(i))/2)**2)
        sp34tot(i)=sqrt(sp34stat(i)**2+((sp34syu(i)+sp34syd(i))/2)**2)
        sp35tot(i)=sqrt(sp35stat(i)**2+((sp35syu(i)+sp35syd(i))/2)**2)
        end do

        call hpak(-1301,sp21)
        call hpake(-1301,sp21stat)
        call hpak(-301,sp21)        
        call hpake(-301,sp21tot)
        call hpak(-1302,sp22)
        call hpake(-1302,sp22stat)
        call hpak(-302,sp22)
        call hpake(-302,sp22tot)
        call hpak(-1303,sp23)
        call hpake(-1303,sp23stat)
        call hpak(-303,sp23)
        call hpake(-303,sp23tot)
        call hpak(-1304,sp24)
        call hpake(-1304,sp24stat)
        call hpak(-304,sp24)
        call hpake(-304,sp24tot)
        call hpak(-1305,sp25)
        call hpake(-1305,sp25stat)
        call hpak(-305,sp25)
        call hpake(-305,sp25tot)

        call hpak(-1401,sp31)
        call hpake(-1401,sp31stat)
        call hpak(-401,sp31)
        call hpake(-401,sp31tot)
        call hpak(-1402,sp32)
        call hpake(-1402,sp32stat)
        call hpak(-402,sp32)
        call hpake(-402,sp32tot)
        call hpak(-1403,sp33)
        call hpake(-1403,sp33stat)
        call hpak(-403,sp33)
        call hpake(-403,sp33tot)
        call hpak(-1404,sp34)
        call hpake(-1404,sp34stat)
        call hpak(-404,sp34)
        call hpake(-404,sp34tot)
        call hpak(-1405,sp35)
        call hpake(-1405,sp35stat)
        call hpak(-405,sp35)
        call hpake(-405,sp35tot)
        
        do i=1,4
           x21tot(i) = sqrt(x21stat(i)**2 + 0.5*(x21sysu(i)**2 +x21sysd(i)**2))
           x22tot(i) = sqrt(x22stat(i)**2 + 0.5*(x22sysu(i)**2 +x22sysd(i)**2))
           x23tot(i) = sqrt(x23stat(i)**2 + 0.5*(x23sysu(i)**2 +x23sysd(i)**2))
           x24tot(i) = sqrt(x24stat(i)**2 + 0.5*(x24sysu(i)**2 +x24sysd(i)**2))
           x25tot(i) = sqrt(x25stat(i)**2 + 0.5*(x25sysu(i)**2 +x25sysd(i)**2))

           x31tot(i) = sqrt(x31stat(i)**2 + 0.5*(x31sysu(i)**2 +x31sysd(i)**2))
           x32tot(i) = sqrt(x32stat(i)**2 + 0.5*(x32sysu(i)**2 +x32sysd(i)**2))
           x33tot(i) = sqrt(x33stat(i)**2 + 0.5*(x33sysu(i)**2 +x33sysd(i)**2))
           x34tot(i) = sqrt(x34stat(i)**2 + 0.5*(x34sysu(i)**2 +x34sysd(i)**2))
           x35tot(i) = sqrt(x35stat(i)**2 + 0.5*(x35sysu(i)**2 +x35sysd(i)**2))

           xx21tot(i) = sqrt(xx21stat(i)**2 + 0.5*(xx21sysu(i)**2 +xx21sysd(i)**2))
           xx22tot(i) = sqrt(xx22stat(i)**2 + 0.5*(xx22sysu(i)**2 +xx22sysd(i)**2))
           xx23tot(i) = sqrt(xx23stat(i)**2 + 0.5*(xx23sysu(i)**2 +xx23sysd(i)**2))
           xx24tot(i) = sqrt(xx24stat(i)**2 + 0.5*(xx24sysu(i)**2 +xx24sysd(i)**2))
           xx25tot(i) = sqrt(xx25stat(i)**2 + 0.5*(xx25sysu(i)**2 +xx25sysd(i)**2))

           xx31tot(i) = sqrt(xx31stat(i)**2 + 0.5*(xx31sysu(i)**2 +xx31sysd(i)**2))
           xx32tot(i) = sqrt(xx32stat(i)**2 + 0.5*(xx32sysu(i)**2 +xx32sysd(i)**2))
           xx33tot(i) = sqrt(xx33stat(i)**2 + 0.5*(xx33sysu(i)**2 +xx33sysd(i)**2))
           xx34tot(i) = sqrt(xx34stat(i)**2 + 0.5*(xx34sysu(i)**2 +xx34sysd(i)**2))
           xx35tot(i) = sqrt(xx35stat(i)**2 + 0.5*(xx35sysu(i)**2 +xx35sysd(i)**2))

        enddo
        call hpak(-751,x2sec1)
        call hpake(-751,x21tot)
        call hpak(-752,x2sec2)
        call hpake(-752,x22tot)
        call hpak(-753,x2sec3)
        call hpake(-753,x23tot)
        call hpak(-754,x2sec4)
        call hpake(-754,x24tot)
        call hpak(-755,x2sec5)
        call hpake(-755,x25tot)

        call hpak(-761,x3sec1)
        call hpake(-761,x31tot)
        call hpak(-762,x3sec2)
        call hpake(-762,x32tot)
        call hpak(-763,x3sec3)
        call hpake(-763,x33tot)
        call hpak(-764,x3sec4)
        call hpake(-764,x34tot)
        call hpak(-765,x3sec5)
        call hpake(-765,x35tot)

        call hpak(-851,xx2sec1)
        call hpake(-851,xx21tot)
        call hpak(-852,xx2sec2)
        call hpake(-852,xx22tot)
        call hpak(-853,xx2sec3)
        call hpake(-853,xx23tot)
        call hpak(-854,xx2sec4)
        call hpake(-854,xx24tot)
        call hpak(-855,xx2sec5)
        call hpake(-855,xx25tot)

        call hpak(-861,xx3sec1)
        call hpake(-861,xx31tot)
        call hpak(-862,xx3sec2)
        call hpake(-862,xx32tot)
        call hpak(-863,xx3sec3)
        call hpake(-863,xx33tot)
        call hpak(-864,xx3sec4)
        call hpake(-864,xx34tot)
        call hpak(-865,xx3sec5)
        call hpake(-865,xx35tot)

       do i =1,5
           xwidth(i)=xbin(I+1)-xbin(i)
        enddo
******************************************************************************
**********************  Loop over events (fill) ******************************
      ELSE IF(MOD(iflag,10).EQ.2) THEN
******  Filling: The following MUST always be done
******    (i) move to the correct sub-directory in PAWC
         CALL hcdir('//PAWC/'//xxxx,' ')
* allow for total xsection calculation
* 
         IF(NLO.eq.1.and.TOT.ne.1) GoTo 11112
            Nevt = Nevt + wtx
11112	   Continue
	   NLOJET = DIJET + TRIPJET + QUADJET
******  example cut...
******  hzdiskin(x) returns the DIS kinematic variables 
******  (see hztool manual for details)
         q2  = real(hzdiskin(1))
         x=real(hzdiskin(2))
         y=real(hzdiskin(3))
         ljets = .true.
         ldis = .true.
         IF(q2.lt.q2min.or.q2.gt.q2max) ldis = .false.
         IF(y.lt.ymin.or.y.gt.ymax) ldis = .false.
         IF(x.lt.xmin.or.x.gt.xmax) ldis = .false.
         ixbin=-9999
         do i=1,5
         if(x.gt.xbin(i).and.x.le.xbin(i+1)) then
            ixbin=i
         endif 
         end do
         
         if(ldis) then
         IF (gen(1:3).eq.'HRW') then
            jgam = hzipgamn(pgam)
         else
            jgam = hzipgam(pgam)
         endif

         if (jgam.ne.1) then
            write(6,*) 'ERROR: No photon found'
            goto 50
         endif
         jbeam = hzibeam(ipro,ilep)
         ilepp=HZIDELEC(idum)
         
	   IF((NLO.eq.1).AND.(NLOJET.eq.0)) goto 11114
                  
         do iloop=1,4
            ppro(iloop) = phep(iloop,ipro)
            ppho(iloop) = pgam(iloop)
            plep(iloop) = phep(iloop,ilep)
            plepp(iloop) = phep(iloop,ilepp)
         enddo
         
c         call hzhcmini(ppro,ppho,ierr)
c          write(6,*) ' NLOJET ',NLOJET,DIJET,TRIPJET,QUADJET
c          call lihep(0)

          DO ihep=1,nhep
            Do i=1,4
              Ph(i)=PHEP(i,ihep)
              Plab(i,ihep)=PHEP(i,ihep)
              phep(i,ihep)=0.0
            enddo
* Boost to HCM frame

            if(hcm) then               
c              call HZHCM(ph,pcms,ierr)
              call hzhcmtol(0,plep,ppro,plepp,Ph,Pcms,ierr)
c             write(6,*) ' boost to hcm ', (Ph(i),i=1,5),(Pcms(i),i=1,4)
              Do i=1,4
                PHEP(i,ihep)=Pcms(i)
              enddo
            else
c              write(6,*) ' no boost to hcm '
              Do i=1,4
                PHEP(i,ihep)=Plab(i,ihep)
              enddo		     
	      endif
          ENDDO

****** run the hzjtfind jet finder (only necessary if using jets!)
******  chjet     : gives the type of jet finder to be used
******  coner     : gives the cone radius
******  njet      : gets set to the number of jets (maximum is 50)
******  jets(50,x): gets set to info about jets with
******             x=1,2 -> eta,phi of jets axis; x=3 -> eT of jet
******             x=4-8 -> e,px,py,pz,m of jet axis  
****** NB/ jets is declared in heracmn.inc
            CALL hzjtfind(chjet,coner,njet,jets)
c            call hzcmtola2(njet,jets,jetslab)
****** loop through all jets in the event
c test boost
c            Do i=1,4
c              Ph(i)=PHEP(i,1)
c            enddo
c            call lihep(0)
c            call hzhcmtol(1,plep,ppro,plepp,Ph,Pjlab,ierr)
c            write(6,*) ' boost check ',(ph(I),i=1,4),(pjlab(i),i=1,4)
c end test            
            IF (NJET.ge.2) THEN
            ijet = 0
            DO 20 iloop = 1, njet
             Do i=1,3
             pjet(i) = Jets(iloop,4+i)
             end do
             pjet(4) = Jets(iloop,4)
             call hzhcmtol(1,plep,ppro,plepp,Pjet,Pjlab,ierr)
c             call get_var(pjlab,p,pt,etr,th,rapr,err_flag)
c             jetslab(iloop,1)= rapr
c             write(6,*) ' eta ',rapr,eta(pjlab)
             jetslab(iloop,1)= eta(pjlab)
******  get rapidity of jet
		 rap=Jetslab(iloop,1)
		 Et=Jets(iloop,3)
c             write(6,*) ' jets ',njet,iloop,Et,rap
******  if the jet passes rapidity cuts
		 IF (rap.ge.etamin.and.rap.le.etamax) THEN
             if(Et.ge.Et2min) then
                ijet = ijet + 1
c store the jets 
                ipoint(ijet) = iloop 
c                write(6,*) ' jets ',njet,ijet,Et,rap
******  Fill your histograms
             ENDIF
             ENDIF
 20          CONTINUE
             Et1=0
             Et2=0
             Et3=0
             if(ijet.ge.2) then
               Et1=Jets(ipoint(1),3)
               px1=Jets(ipoint(1),5)
               py1=Jets(ipoint(1),6)
               Et2=Jets(ipoint(2),3)
               px2=Jets(ipoint(2),5)
               py2=Jets(ipoint(2),6)
	 	   rap1=Jetslab(ipoint(1),1)
	 	   rap2=Jetslab(ipoint(2),1)
               if(ijet.ge.3) then 
                    Et3=Jets(ipoint(3),3)
	 	        rap3=Jetslab(ipoint(3),1)
               endif
             endif
             if(Et1.ge.Et1min.and.Et2.ge.Et2min) then
                njet_acc = njet_acc+1
                sumpt = sqrt((px1+px2)**2 + (py1+py2)**2)
                difpt = sqrt((px1-px2)**2 + (py1-py2)**2)/2./Et1
                if(Et2.gt.(Et1+0.01)) write(6,*) ' error jets ', Et1,Et2
c                call hfill(10,real(x),0.,real(wtx))
c                call hfill(20,real(q2),0.,real(wtx))
                nev801=nev801 + 1
c                call hfill(801,real(log10(stmax2)),0.,real(wtx))
c                call hfill(802,real(stmax2),real(Q2),real(wtx))
c                call hfill(803,real(stmax2**2),real(Q2),real(wtx))
cc                write(6,*) ' hz07062: stmax2 = ',stmax1,stmax2
c need to sort jets in eta for the plots...                   
c		    CALL hfill(101,real(max(rap1,rap2)),0.,real(wtx))
c		    CALL hfill(102,real(min(rap1,rap2)),0.,real(wtx))
c		    CALL hfill(201,real(Et1),0.,real(wtx))
c		    CALL hfill(202,real(Et2),0.,real(wtx))
                phi3=-9999.
		    if(Et3.ge.Et2min) then
                if(Et3.ge.Et1) write(6,*) ' error 3-jets ',Et1,Et2,Et3
                if(Et3.ge.Et2) write(6,*) ' error 3-jets ',Et1,Et2,Et3
c                   call hfill(11,real(x),0.,real(wtx))
c                   call hfill(21,real(q2),0.,real(wtx))
c need to sort jets in eta for the plots...                   
c		       CALL hfill(111,real(rap1),0.,real(wtx))
c		       CALL hfill(112,real(rap2),0.,real(wtx))
c 		       CALL hfill(113,real(rap3),0.,real(wtx))
c                   CALL hfill(211,real(Et1),0.,real(wtx))
c                   CALL hfill(212,real(Et2),0.,real(wtx))
c                   CALL hfill(213,real(Et3),0.,real(wtx))
                    phi3=Jets(ipoint(3),2)
                   if(phi3.lt.0) phi3=phi3+2*pi
               endif
                phi1=Jets(ipoint(1),2)
                if(phi1.lt.0) phi1=phi1+2*pi
                phi2=Jets(ipoint(2),2)
                if(phi2.lt.0) phi2=phi2+2*pi
                deltaphi= abs(phi1 - phi2)
                if(deltaphi.gt.pi) deltaphi=abs(deltaphi-2*pi) 
                if(deltaphi.lt.2) then
c                   call hfill(600+ixbin,real(ijet),0.,real(wtx))
                   else
c                   call hfill(610+ixbin,real(ijet),0.,real(wtx))
                endif
                delta3rd= -9999.
                if(phi3.gt.-10.) then
                   delta3rd= abs(phi1 - phi3)
                   if(delta3rd.gt.pi) delta3rd=abs(delta3rd-2*pi) 
                endif   
c                write(6,*) deltaphi,Jets(ipoint(1),2),Jets(ipoint(2),2)
		  IF(PHi1.GT.PHi2) THEN
  	          DPH = (PHi1-PHi2)
 	         ELSE
  	          DPH = (PHi2-PHi1)
  	        ENDIF
   	        IF(DPH.GT.PI) THEN
   	         DPH = ABS(DPH-2*PI)   
  	        ENDIF
c             write(6,*) ' delta phi ', 
c     &        deltaphi,dph,Jets(ipoint(1),2),Jets(ipoint(2),2)
                call hfill(300+ixbin,real(deltaphi),0.,real(wtx))
                call hfill(750+ixbin,real(sumpt),0.,real(wtx))
                call hfill(850+ixbin,real(difpt),0.,real(wtx))
                
                if(Et3.ge.Et2min)then  
                    call hfill(400+ixbin,real(deltaphi),0.,real(wtx))
                    call hfill(760+ixbin,real(sumpt),0.,real(wtx))
                    call hfill(860+ixbin,real(difpt),0.,real(wtx))
                    if(deltaphi.lt.2) then
c                    call hfill(500+ixbin,real(delta3rd),0.,real(wtx))
                    else
c                    call hfill(510+ixbin,real(delta3rd),0.,real(wtx))
                    endif
                endif
             endif
             endif ! njet > 2         
             DO ihep=1,nhep
               Do i=1,4
                 phep(i,ihep)=Plab(i,ihep)
               enddo
             enddo
 50    continue        
       endif ! dis  
11114	 continue		 

*******************************************************************************
*******************************  Terminate ************************************
         
      ELSE IF(MOD(iflag,10).EQ.3) THEN

******  Termination: The following MUST always be done
******     (i) Move to the correct PAW subdirectory     
         CALL hcdir('//PAWC/'//xxxx,' ')

******  write the number of events and xsec for the process just finished.
         WRITE(6,*) xxxx,' X-section is',xsec, ' and number of events ' 
     +    ,nevt, ' for process ',iproc
  
c         call hzhinrm( 801, 0, nev801, 1 )
****** normalise the histograms using hzhinrm(iid,iidnew,nevt,iifl)
****** with arguments: iid = histogram id, 
******                 iidnew = new histo id (if 0, old histo is modified)
******                 nevt = normalization factor
******                 iifl =1: normalize errors too, otherwise they are zeroed
         do i =1,5
         ihist = 300+i
         xnorm=real(nevt/xsec)*xwidth(i)
         CALL hzhinrm(ihist,0,real(xnorm),1)
         ihist = 400+i
         CALL hzhinrm(ihist,0,real(xnorm),1)
c         ihist = 500+i
c         CALL hzhinrm(ihist,0,real(xnorm),1)
c         ihist = 510+i
c         CALL hzhinrm(ihist,0,real(xnorm),1)
         ihist = 750+i
         CALL hzhinrm(ihist,0,real(xnorm),1)
         ihist = 760+i
         CALL hzhinrm(ihist,0,real(xnorm),1)
         ihist = 850+i
         CALL hzhinrm(ihist,0,real(xnorm),1)
         ihist = 860+i
         CALL hzhinrm(ihist,0,real(xnorm),1)
         xnorm1=max(1.,real(njet_acc))
         write(6,*) ' xnorm1 ',xnorm1,njet_acc
         ihist = 600+i
c         call hopera(ihist,'+E',ihist,ihist,real(1./xnorm1),0.)
         ihist = 610+i
c         call hopera(ihist,'+E',ihist,ihist,real(1./xnorm1),0.)
         enddo
         do i=0,1
         ihist = 10+i
         xnorm=real(nevt/xsec)
c         CALL hzhinrm(ihist,0,real(xnorm),1)
         ihist = 20+i
         xnorm=real(nevt/xsec)
c         CALL hzhinrm(ihist,0,real(xnorm),1)
         enddo
         do i=1,2
         ihist = 100+i
         xnorm=real(nevt/xsec)
c         CALL hzhinrm(ihist,0,real(xnorm),1)
         ihist = 200+i
         xnorm=real(nevt/xsec)
c         CALL hzhinrm(ihist,0,real(xnorm),1)
         enddo

         do i=1,3
         ihist = 110+i
         xnorm=real(nevt/xsec)
c         CALL hzhinrm(ihist,0,real(xnorm),1)
         ihist = 210+i
         xnorm=real(nevt/xsec)
c         CALL hzhinrm(ihist,0,real(xnorm),1)
         enddo
      ENDIF

      RETURN

      END

      double precision function eta(p)
      implicit none
      Double precision p(4)
      Double Precision dpt2,dtemp
c calculate eta
      dpt2 = p(1)**2+p(2)**2
      dtemp = (SQRT(dpt2+p(3)**2)+ABS(p(3)))**2

c --- check if calculation is possible
      if (dpt2.le.4.25E-18*dtemp) then
         eta = 20.D0
      else

c --- get amount of pseudo-rapidity
         eta = 0.5D0*LOG(dtemp/dpt2)
      endif
      eta=SIGN(eta,p(3))
      Return
      end
