	Subroutine HZ94176(iflag)
*****************************************************************************
* This photoproduction routine has to be run twice with the following
* code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the RESOLVED component run.
*       +0 for both together
* for all three phases (iflag=1,2,3)
*
* iflag = 1,2 or 3 depending on wether it is the initialization,
*		processing or termination phase (respectively)
*
* The default jetfinder is PUCELL
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
* This routine produces ten Monte Carlo graphs.
* id 10-14 are only the number of unweighted events which pass the cuts
* id 20-24 are the normalised cross sections in nb. These graph will be
*		meaningless unless Xsec and Ntot are set before
*		calling the termination routine.
*		(Xsec - total cross section returned by MC)
*		(Ntot - number of events passed to this routine)
* This routine produces ten Data graphs
* id -20 to -24 correspond to 20-24 with statistical errors only.
* id -120 to -124 correspond to 20-24 with statistical and systematic errors.
*
* Cuts : 	Q2 < 4
*		0.2 < y_bj <0.85
*		Depending on graph Et > 8, 11, 17
*			and -1 < eta < 2, 1
*
* Recommended value for Ptmin is 5 GeV (to be set in MC set up)
*
* Author : Mark Hayes (mhayes@zow.desy.de)
*
* Reference : DESY-94-176
*             Phys. Lett. B342 (1995) 417-432.
*
*****************************************************************************
*
* HERA Tuning defined commons
 
	IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
 
 
        CHARACTER*6 jetf
	CHARACTER*6 xxxx
        DATA xxxx /'d94176'/
 
	INTEGER NJET
c	DOUBLE PRECISION Jets(50,8)
	DOUBLE PRECISION CONER
	SAVE CONER
	
	INTEGER loop,loop2,count
	DOUBLE PRECISION rap,Et
	DOUBLE PRECISION ybj,q2
	DOUBLE PRECISION ybjcut(2),q2cut(2)
	DOUBLE PRECISION rapcut(2,5),Etcut(5)
	SAVE YBJCUT, Q2CUT, ETCUT, RAPCUT

C *** Data for Et.gt.8 GeV
	REAL xbins(13)
	REAL pteta(12),ptcr(12),ptcre(12)
	REAL ptcresys(12),hiband(12),loband(12)
C *** Data points for Et.gt.8 GeV
	DATA xbins/-1.0, -0.75, -0.5, -0.25, 0, 0.25,
     &             0.5, 0.75, 1.0, 1.25, 1.5, 1.75, 2.0/
	DATA pteta /-0.88, -0.62, -0.38, -0.12, 0.12, 0.38,
     &               0.62, 0.88, 1.12, 1.38, 1.62, 1.88/
	DATA ptcr / 4.1, 6.6, 8.5, 10.9, 12.4, 12.8,
     &                13.7, 13.8, 15.2, 14.9, 16.4, 18.2/
	DATA ptcre/ 0.2, 0.2, 0.3, 0.3, 0.3, 0.3,
     &               0.3, 0.3, 0.4, 0.3, 0.4, 0.4/
	DATA ptcresys/ 0.6, 0.7, 0.7, 0.5, 0.8, 1.1,
     &                  0.9, 1.7, 1.4, 1.6, 2.8, 1.3/
	DATA hiband/1.1, 1.3, 1.7, 1.6, 1.9, 1.5,
     &               2.0, 2.0, 2.1, 2.2, 2.4, 2.6/
	DATA loband/-0.6, -1.0, -1.0, -1.2, -1.5, -1.4,
     &               -1.5, -1.3, -1.5, -1.4, -1.6, -2.0/
C *** Data for Et.gt.11 GeV
	REAL xbins2(13)
	REAL pt2eta(12),pt2cr(12),pt2cre(12)
	REAL pt2cresys(12),hi2band(12),lo2band(12)
C *** Data points for Et.gt.11 GeV
	DATA xbins2/-1.0, -0.75, -0.5, -0.25, 0, 0.25,
     &             0.5, 0.75, 1.0, 1.25, 1.5, 1.75, 2.0/
	DATA pt2eta /-0.88, -0.62, -0.38, -0.12, 0.12, 0.38,
     &               0.62, 0.88, 1.12, 1.38, 1.62, 1.88/
	DATA pt2cr / 0.46, 1.36, 2.05, 2.84, 3.08, 3.37,
     &               3.92, 3.85, 3.71, 3.94, 4.31, 4.83/
	DATA pt2cre/ 0.08, 0.13, 0.14, 0.16, 0.16, 0.16,
     &               0.18, 0.18, 0.19, 0.18, 0.19, 0.19/
	DATA pt2cresys/ 0.18, 0.24, 0.46, 0.47, 0.56, 0.56,
     &                  0.65, 0.72, 0.67, 0.65, 1.04, 0.63/
	DATA hi2band/0.46, 0.66, 0.49, 0.58, 0.68, 0.80,
     &               0.93, 1.17, 0.88, 0.81, 0.99, 1.14/
	DATA lo2band/-0.11, -0.45, -0.51, -0.67, -0.52, -0.55,
     &               -0.78, -0.70, -0.54, -0.58, -0.64, -0.71/
C *** Data for Et.gt.17 GeV
	REAL xbins3(9)
	REAL pt3eta(8),pt3cr(8),pt3cre(8)
	REAL pt3cresys(8),hi3band(8),lo3band(8)
C *** Data points for Et.gt.17 GeV
	DATA xbins3/-0.75, -0.25, 0.25,
     &             0.75, 1.0, 1.25, 1.5, 1.75, 2.0/
	DATA pt3eta /-0.50, 0.00, 0.50, 0.88, 1.12, 1.38, 1.62, 1.88/
	DATA pt3cr / 0.08, 0.29, 0.52, 0.65, 0.58, 0.63, 0.63, 0.60/
	DATA pt3cre/ 0.02, 0.04, 0.05, 0.08, 0.08, 0.08, 0.08, 0.07/
	DATA pt3cresys/0.05, 0.04, 0.06, 0.11, 0.14, 0.15, 0.18, 0.15/
	DATA hi3band/0.05, 0.14, 0.15, 0.12, 0.14, 0.10, 0.16, 0.17/
	DATA lo3band/-0.03,-0.06,-0.11,-0.14,-0.08,-0.14,-0.15,-0.15/
C *** Data for Et.gt.8 & -1<eta<2
	REAL xbins4(9)
	REAL pt4eta(8),pt4cr(8),pt4cre(8)
	REAL pt4cresys(8),hi4band(8),lo4band(8)
C *** Data points for Et.gt.8 & -1<eta<2
	DATA xbins4/8, 11, 14, 17, 21, 25, 29, 35, 41/
	DATA pt4eta/9.2, 12.2, 15.2, 18.6, 22.6, 26.7, 31.6, 37.6/
	DATA pt4cr/9.0, 2.17, 0.59, 0.199,
     &    0.058, 0.0195, 0.0084, 0.0041/
	DATA pt4cre/0.1, 0.04, 0.02, 0.011,
     &    0.006, 0.0032, 0.0019, 0.0012/
	DATA pt4cresys/0.4, 0.31, 0.05, 0.024,
     &    0.007, 0.0026, 0.0013, 0.0009/
	DATA hi4band/1.1, 0.51, 0.15, 0.052,
     &    0.014, 0.0063, 0.0024, 0.0021/
	DATA lo4band/-0.7, -0.38, -0.11, -0.040,
     &    -0.012, -0.0045, -0.0017, -0.0010/
C *** Data for Et.gt.8 & -1<eta<1
	REAL xbins5(7)
	REAL pt5eta(6),pt5cr(6),pt5cre(6)
	REAL pt5cresys(6),hi5band(6),lo5band(6)
C *** Data points for Et.gt.8 & -1<eta<1
	DATA xbins5/8, 11, 14, 17, 21, 25, 29/
	DATA pt5eta/9.2, 12.2, 15.2, 18.6, 22.6, 26.7/
	DATA pt5cr/5.10, 1.18, 0.322, 0.103, 0.0297, 0.0075/
	DATA pt5cre/0.05, 0.03, 0.015, 0.008, 0.0044, 0.0021/
	DATA pt5cresys/0.23, 0.14, 0.030, 0.009, 0.0059, 0.0012/
	DATA hi5band/0.66, 0.30, 0.082, 0.036, 0.0058, 0.0026/
	DATA lo5band/-0.44, -0.22, -0.067, -0.020, -0.0053, -0.0022/
 
	INTEGER chjet, iproc, iflag
	SAVE chjet
C --- same jet finder as in paper -- PUCELL
	DATA chjet/2/
 
	REAL pttemp(12)
	INTEGER ivec,jvec

*     Determine the process type.
	iproc=(mod(iflag,10000)/1000)
	if ((iproc.lt.0).or.(iproc.gt.2)) then
	   write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
	   return
	endif
	
C       --- choose jet finder
        IF ((MOD(iflag,100)/10.gt.0).and.MOD(iflag,10).eq.1) THEN
	   chjet=INT(MOD(iflag,100)/10)
        ENDIF
	
	IF (MOD(iflag,10).eq.1) THEN
C       *** Initialisation run ***
C       *** Create directories ***
	   IF (iproc.le.1) THEN
	      CALL HCDIR('//HISTO',' ')
	      CALL HMDIR(xxxx,'S')
	      CALL HCDIR('//PAWC',' ')
	      CALL HMDIR(xxxx,'S')
	   ENDIF

	   CALL HCDIR('//HISTO/'//xxxx,' ')
	   CALL HCDIR('//PAWC/'//xxxx,' ')
	   	      
	   CALL HBOOKB(100*iproc+20,'Monte Carlo cross section (nb)
     &  	Et.gt.8 GeV',12 ,xbins ,0.)
	   CALL HBOOKB(100*iproc+21,'Monte Carlo cross section (nb)
     &  	Et.gt.11 GeV',12 ,xbins2 ,0.)
	   CALL HBOOKB(100*iproc+22,'Monte Carlo cross section (nb)
     &  	Et.gt.17 GeV',8 ,xbins3 ,0.)
	   CALL HBOOKB(100*iproc+23,'Monte Carlo cross section (nb)
     &  	-1.lt.eta.lt.2',8 ,xbins4 ,0.)
	   CALL HBOOKB(100*iproc+24,'Monte Carlo cross section (nb)
     &  	-1.lt.eta.lt.1',6 ,xbins5 ,0.)
 
	   CALL HIDOPT(0,'stat')
	   CALL HBARX(0)
	   
	   ybjcut(1)=0.2
	   ybjcut(2)=0.85
	   q2cut(1)=0.
	   q2cut(2)=4.
	   
C       --- cuts for three rapidity plots
	   rapcut(1,1)=-1.
	   rapcut(2,1)=2.
	   Etcut(1)=8.0
	   
	   rapcut(1,2)=-1.
	   rapcut(2,2)=2.
	   Etcut(2)=11.0
	   
	   rapcut(1,3)=-1.
	   rapcut(2,3)=2.
	   Etcut(3)=17.0
C       --- cuts for two Et plots
	   rapcut(1,4)=-1.
	   rapcut(2,4)=2.
	   Etcut(4)=8.0
	   
	   rapcut(1,5)=-1.
	   rapcut(2,5)=1.
	   Etcut(5)=8.0
	   
C       --- standard cone radius is 1.0
	   CALL HZJETRAD(2,CONER)
	   IF (CONER.lt.0.0) THEN
	      CONER=1.0
	   ENDIF
	   CALL HZJTNAME(chjet,jetf)
	   WRITE(6,*)'**********************************************'
	   WRITE(6,*)'* HZ94176 called, histograms will be output. *'
	   WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
	   WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
	   WRITE(6,*)'**********************************************'
	   
 1001	   FORMAT(A29,F5.2,A13)
 
	ELSE IF (MOD(iflag,10).eq.2) THEN
C       *** Filling runs ***
	   
	   
C       *** Change directory to our graphs ***
	   CALL HCDIR('//PAWC/'//xxxx,' ')
	   
	   q2=hzphokin(1)
	   IF (q2.eq.-1) THEN
	      PRINT*,'HZ94176: HZPHOKIN returns error for Q2'
	   ELSEIF (q2.lt.0) THEN
	      q2=abs(q2)
	   ENDIF
	   ybj=hzphokin(3)
	   IF (ybj.eq.-1) THEN
	      PRINT*,'HZ94176: HZPHOKIN returns error for ybj'
	   ENDIF
	   
C       *** Fill our histogram ***
	   
	   IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(2))
     &		.and.(q2.lt.q2cut(2))) Then
	      
	      CALL hzjtfind(chjet,CONER,NJET,Jets)
	      
	      DO loop=1,NJET
		 Et=Jets(loop,3)
		 rap=Jets(loop,1)
		 DO loop2=1,3
		    
		    IF ((rap.ge.rapcut(1,loop2)).and.(rap.le.rapcut(2
     &  		 ,loop2)).and.(Et.gt.Etcut(loop2))) THEN
		       CALL HFILL(INT(iproc*100+19+loop2), REAL(rap) , 0
     &  		    .,wtx)
		    ENDIF
		 ENDDO
		 
		 DO loop2=4,5
		    IF ((rap.ge.rapcut(1,loop2)).and.(rap.le.rapcut(2
     &  		 ,loop2)).and.(Et.gt.Etcut(loop2))) THEN
		       CALL HFILL(INT(iproc*100+19+loop2), REAL(Et) , 0.
     &  		    ,wtx)
		    ENDIF
		 ENDDO
		 
	      EndDo		! loop
	   ENDIF
	   
	ELSE IF (MOD(iflag,10).eq.3) THEN
C       *** Termination run ***
C       *** Change directory ***
	   CALL HCDIR('//PAWC/'//xxxx,' ')
C       *** Finish off histograms ***
	   IF (iproc.eq.0.OR.iproc.eq.2) THEN
	      WRITE (HZUNIT,*) 'HZ94176: finishing off'
C       --- only bother outputting file if we are on the second run.
	      CALL HBOOKB(-20,'Data cross section (nb) Et.gt.8 GeV'
     &  	   ,12 ,xbins ,0.)
	      CALL HBOOKB(-21,'Data cross section (nb) Et.gt.11 GeV'
     &  	   ,12 ,xbins2 ,0.)
	      CALL HBOOKB(-22,'Data cross section (nb) Et.gt.17 GeV'
     &  	   ,8 ,xbins3 ,0.)
	      CALL HBOOKB(-23,'Data cross section (nb) -1.lt.eta.lt.2'
     &  	   ,8 ,xbins4 ,0.)
	      CALL HBOOKB(-24,'Data cross section (nb) -1.lt.eta.lt.1'
     &  	   ,6 ,xbins5 ,0.)
	      CALL HBOOKB(-120,'Data cross section (nb) Et.gt.8 GeV'
     &  	   ,12 ,xbins ,0.)
	      CALL HBOOKB(-121,'Data cross section (nb) Et.gt.11 GeV'
     &  	   ,12 ,xbins2 ,0.)
	      CALL HBOOKB(-122,'Data cross section (nb) Et.gt.17 GeV'
     &  	   ,8 ,xbins3 ,0.)
	      CALL HBOOKB(-123,'Data cross section (nb) -1.lt.eta.lt.2'
     &  	   ,8 ,xbins4 ,0.)
	      CALL HBOOKB(-124,'Data cross section (nb) -1.lt.eta.lt.1'
     &  	   ,6 ,xbins5 ,0.)
	      
	      CALL HPAK(-20,ptcr)
	      CALL HPAKE(-20,ptcre)
	      CALL HPAK(-21,pt2cr)
	      CALL HPAKE(-21,pt2cre)
	      CALL HPAK(-22,pt3cr)
	      CALL HPAKE(-22,pt3cre)
	      CALL HPAK(-23,pt4cr)
	      CALL HPAKE(-23,pt4cre)
	      CALL HPAK(-24,pt5cr)
	      CALL HPAKE(-24,pt5cre)
	      
	      DO ivec=1,12
		 pttemp(ivec)=sqrt(ptcre(ivec)**2+ptcresys(ivec)**2)
	      ENDDO
	      CALL HPAK(-120,ptcr)
	      CALL HPAKE(-120,pttemp)
	      
	      DO ivec=1,12
		 pttemp(ivec)=sqrt(pt2cre(ivec)**2+pt2cresys(ivec)**2)
	      ENDDO
	      CALL HPAK(-121,pt2cr)
	      CALL HPAKE(-121,pttemp)
	      
	      DO ivec=1,8
		 pttemp(ivec)=sqrt(pt3cre(ivec)**2+pt3cresys(ivec)**2)
	      ENDDO
	      CALL HPAK(-122,pt3cr)
	      CALL HPAKE(-122,pttemp)
	      
	      DO ivec=1,8
		 pttemp(ivec)=sqrt(pt4cre(ivec)**2+pt4cresys(ivec)**2)
	      ENDDO
	      CALL HPAK(-123,pt4cr)
	      CALL HPAKE(-123,pttemp)
	      
	      DO ivec=1,6
		 pttemp(ivec)=sqrt(pt5cre(ivec)**2+pt5cresys(ivec)**2)
	      ENDDO
	      CALL HPAK(-124,pt5cr)
	      CALL HPAKE(-124,pttemp)
	      
	   ENDIF
	   
	   IF (Xsec.eq.0) THEN
	      PRINT*
     &  	   ,'HZ94176: termination called with zero cross section'
	      PRINT*,'          cross section graph meaningless'
	      PRINT*,'	      in process :',iproc*1000
	      Xsec=1.
	   ENDIF
	   IF (Ntot.eq.0) THEN
	      PRINT*,'HZ94176: termination called with no total events'
	      PRINT*,'          cross section graph meaningless'
	      PRINT*,'	      in process :',iproc*1000
	      Ntot=1.
	   ENDIF
	   
C       --- now calculate cross-section graphs 20-24
	   DO ivec=1,5
	      CALL HZHINRM(100*iproc+19+ivec,0,REAL(ntot/xsec),1)
	   ENDDO
	   IF (iproc.EQ.2) THEN
	      CALL HOPERA(120,'+e',220,20,1.,1.)
	      CALL HOPERA(121,'+e',221,21,1.,1.)
	      CALL HOPERA(122,'+e',222,22,1.,1.)
	      CALL HOPERA(123,'+e',223,23,1.,1.)
	      CALL HOPERA(124,'+e',224,24,1.,1.)
	   ENDIF
	   
	ENDIF
	
	RETURN
	END
