*CMZ :  1.01/11 09/12/97  16.44.12  by  Tancredi Carli
*CMZ :  1.01/07 10/10/97  19.46.50  by  Tancredi Carli
*CMZ :  1.01/05 04/09/96  17.44.20  by   Tancredi Carli
*CMZ :  0.00/09 16/02/96  17.15.47  by  Tancredi Carli
*CMZ :  0.00/08 14/02/96  13.44.46  by  Tancredi Carli
*CMZ :  0.00/03 09/01/96  15.51.20  by  HZtool
*CMZ :  0.00/01 05/01/96  14.09.27  by  HZtool
*-- Author :
      Subroutine hz95084(iflag)
************************************************************************
* Subroutine hz95084                                                   *
* ==================                                                   *
*                                                                      *
* Author: Nick Brook (brook@v2.ph.gla.ac.uk)                           *
*                                                                      *
* This routine plots the differential multiplcities of K0 and Lambda   *
* versus both transverse momentum and pseudorapidity in a restricted   *
* pseudorapidity and pt range.                                         *
*                                                                      *
* Event selection:                                                     *
* 10 < Q2 < 640 GeV^2                                                  *
* .0003 < x < 0.01                                                     *
* y > 0.04                                                             *
*                                                                      *
* K0 selection                                                         *
* -1.3 < eta < 1.3                                                     *
* 0.5 < pt < 4.0 GeV                                                   *
*                                                                      *
* Lambda selection                                                     *
* -1.3 < eta < 1.3                                                     *
* 0.5 < pt < 3.5 GeV                                                   *
*                                                                      *
* Reference: Z Phys C68(1995) 29.                                      *
*            DESY 95-084                                               *
*                                                                      *
* Modified: add data histograms with statistical errors only           *
*           add check on electron beam energy
************************************************************************
*
* HERA tuning defined commons
*
      Implicit Double Precision(A-H,O-Z)
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzhbook.inc"
*
* Set up local variables
*
      Real nacc
      Integer ierr,ibeam,idum,nentry
      Character xxxx*5
      Data xxxx/'95084'/,nentry/0/
      Real eb,ee
      Parameter (ee=26.7)
      Character*80 chtitl
      Dimension q2cut(2), xcut(2), ycut(2), etacut(2)
      Real ptk(7), ptke(7), ptks(7), etak(8), etake(8), etaks(8)
      Real ptl(4), ptle(4), ptls(4), etal(4), etale(4), etals(4)
      Real ptkerr(7), etakerr(8), ptlerr(4), etalerr(4)
      Real xmi, xma, ymi, yma
*
* Data points from plots
*
      Data ptk /.1976,.0547,.0201,.0088,.0040,.0018,.0011/
      Data ptks/.0211,.0074,.0031,.0016,.0009,.0006,.0009/
      Data ptke/.0159,.005,.0024,.0014,.0008,.0005,.0004/
      Data etak/.077,.095,.118,.109,.118,.138,.178,.118/
      Data etaks/.018,.0147,.0174,.0162,.0174,.0201,.0364,.0262/
      Data etake/.012,.012,.015,.013,.013,.017,.022,.018/
      Data ptl/.0210,.0033,.001,.0003/
      Data ptls/.0051,.0008,.0004,.0002/
      DATA ptle/.0045,.0008,.0004,.0002/
      Data etal/.0088,.0159,.0143,.0157/
      Data etals/.0032,.0054,.0038,.0046/
      DATA etale/.0031,.0052,.0036,.0044/

      save Nacc, q2cut, xcut, ycut, ptupk, ptupl, ptlow, etacut
      save nentry

*
      If(iflag.eq.1) then
*
* Initialisation: The following MUST always be done
* (i) make subdirectory in PAWC
*      - use the name as the xxxxxx in HZxxxxxx subroutine
* (i) make subdirectory in o/p file
*
        Nacc= 0.
        Call hcdir('//PAWC',' ')
        call hmdir('95084','S')
        Call hcdir('//HISTO',' ')
        call hmdir('95084','S')
*
* book your histograms
*
        call hbook1(11,'1/N dn/d-pt(MC)',7,.5,4.0,0.)
        call hbook1(101,'1/N dn/d-pt(MC)',4,.5,3.5,0.)
        call hbook1(10,'1/N dn/d-eta(MC)',8,-1.3,1.3,0.)
        call hbook1(100,'1/N dn/d-eta(MC)',4,-1.3,1.3,0.)
*
* set your plot/paper specific cuts
*
        q2cut(1) = 10.
        q2cut(2) = 640.
        xcut(1) = 0.0003
        xcut(2) = 0.01
        ycut(1) = 0.04
        ycut(2) = 1.
        ptupk = 4.
        ptupl = 3.5
        ptlow = 0.5
        etacut(1) = -1.3
        etacut(2) =  1.3
*
      Else if(iflag.eq.2) then
* Filling: The following MUST always be done
* (i) move to the correct sub-directory in PAWC
*
       call hcdir('//PAWC/95084',' ')
*
* Check electron beam energy
*
        nentry=nentry+1
        ierr=HZIBEAM(ibeam,idum)
        if (.not.(ierr.eq.1)) then
         write(6,*) 'HZ'//xxxx,' beams not found ! '
         return
        endif
*
        eb=real(PHEP(4,idum))
        if (abs(abs(eb)-ee).gt.0.2) then
         if (nentry.lt.10) then
          write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,'  ! '
         elseif (nentry.eq.10) then
          write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,
     &   ' ...last message ! '
         endif
C         return
        endif
*
        q2  = hzdiskin(1)
        xbj = hzdiskin(2)
        ybj = hzdiskin(3)
        If(q2.gt.q2cut(1).and.q2.lt.q2cut(2).and.
     +     xbj.gt.xcut(1).and.xbj.lt.xcut(2).and.
     1     ybj.gt.ycut(1).and.ybj.lt.ycut(2)) then
          Nacc = Nacc + wtx
*
* Fill your histograms
*
          do 30 ihep = 1, nhep
            ioff =0
            if(abs(idhep(ihep)).eq.310.or.abs(idhep(ihep)).eq.130) then
              ioff = 1
              ptcut = ptupk
            else if(abs(idhep(ihep)).eq.3122) then
              ioff = 10
              ptcut = ptupl
            else
              goto 30
            endif
            pt = sqrt(phep(1,ihep)**2+phep(2,ihep)**2)
            the = hzphmang(phep(3,ihep),pt)
            rap = -1.*log(tan(the/2.))
            if(rap.gt.etacut(1).and.rap.lt.etacut(2).and.
     +         pt.gt.ptlow.and.pt.lt.ptcut) Then
              call hfill(ioff*10,real(rap),0.,wtx)
              call hfill((ioff*10)+1,real(pt),0.,wtx*0.5/real(pt))
            endif
 30       continue
        endif
*
      Else if(iflag.eq.3) then
* Termination: The following MUST always be done
* (i) Move to the correct PAW subdirectory
*
        call hcdir('//PAWC/95084',' ')
*
* Histo manipulation
*
        Call hgive(10,chtitl,nx,xmi,xma,ny,ymi,yma,nwt,loc)
        Call hopera(10,'+e',10,10,
     +            1./real(nacc)*real(nx)/(xma-xmi),0.)
        Call hgive(100,chtitl,nx,xmi,xma,ny,ymi,yma,nwt,loc)
        Call hopera(100,'+e',100,100,
     +            1./real(nacc)*real(nx)/(xma-xmi),0.)
        Call hgive(11,chtitl,nx,xmi,xma,ny,ymi,yma,nwt,loc)
        Call hopera(11,'+e',11,11,
     +            1./real(nacc)*real(nx)/(xma-xmi),0.)
        Call hgive(101,chtitl,nx,xmi,xma,ny,ymi,yma,nwt,loc)
        Call hopera(101,'+e',101,101,
     +            1./real(nacc)*real(nx)/(xma-xmi),0.)
*
C*TC>
        call hzhinfo(10,nacc)
        call hzhinfo(100,nacc)
        call hzhinfo(11,nacc)
        call hzhinfo(101,nacc)
C*TC<
*
* Deal with data plots
* 
        call hbook1(-11,'1/N dn/d-pt(data)',7,.5,4.0,0.)
        call hbook1(-101,'1/N dn/d-pt(data)',4,.5,3.5,0.)
        call hbook1(-10,'1/N dn/d-eta(data)',8,-1.3,1.3,0.)
        call hbook1(-100,'1/N dn/d-eta(data)',4,-1.3,1.3,0.)
C*TC>
        call hbook1(-1011,'1/N dn/d-pt(data)',7,.5,4.0,0.)
        call hbook1(-1101,'1/N dn/d-pt(data)',4,.5,3.5,0.)
        call hbook1(-1010,'1/N dn/d-eta(data)',8,-1.3,1.3,0.)
        call hbook1(-1100,'1/N dn/d-eta(data)',4,-1.3,1.3,0.)
C*TC<
        Do ivec = 1, 7
          ptkerr(ivec) = sqrt(ptke(ivec)**2 + ptks(ivec)**2)
        Enddo
        Do ivec = 1, 8
          etakerr(ivec) = sqrt(etake(ivec)**2 + etaks(ivec)**2)
        Enddo
        Do ivec = 1, 4
          ptlerr(ivec) = sqrt(ptle(ivec)**2 + ptls(ivec)**2)
          etalerr(ivec) = sqrt(etale(ivec)**2 + etals(ivec)**2)
        Enddo
        write(6,*) ' before hpak '
        Call hpak(-11,ptk)
        Call hpake(-11,ptkerr)
        Call hpak(-10,etak)
        Call hpake(-10,etakerr)
        Call hpak(-101,ptl)
        Call hpake(-101,ptlerr)
        Call hpak(-100,etal)
        Call hpake(-100,etalerr)
C*TC>
        Call hpak (-1011,ptk)
        Call hpake(-1011,ptke)
        Call hpak (-1010,etak)
        Call hpake(-1010,etake)
        Call hpak (-1101,ptl)
        Call hpake(-1101,ptle)
        Call hpak (-1100,etal)
        Call hpake(-1100,etale)
*
*       calculate chi2
*
        Call hzchisq(-10  ,10)
        Call hzchisq(-11  ,11)
        Call hzchisq(-100 ,100)
        Call hzchisq(-101 ,101)
        Call hzchisq(-1010,10)
        Call hzchisq(-1011,11)
        Call hzchisq(-1100,100)
        Call hzchisq(-1101,101)

C*TC<
        Write(6,*) ' X-section is',xsec
      Endif
*
      Return
      End
