	Subroutine HZ95115(iflag)
*****************************************************************************
* This photoproduction routine has to be run twice with the following
* code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the RESOLVED component run.
* for all three phases (iflag=1,2,3)
*
* iflag = 1,2 or 3 depending on wether it is the initialization,
*		processing or termination phase (respectively)
*
* The default jetfinder is EUCELL
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
* This routine produces three Monte Carlo graphs.
* id 10 is the normalised cross section in nb for most forward jet event with a
*		forward rapgap.
* id 20 is the integrated normalised cross section in nb for different forward
*		rapgaps.
*		These graphs will be
*		meaningless unless Xsec and Ntot are set before
*		calling the termination routine.
*		(Xsec - total cross section returned by MC)
*		(Ntot - number of events passed to this routine)
* And two equivalent DATA graphs -10,-20
*
* Cuts : 	Q2 < 4
*		0.2< y_bj < 0.85
*		-1 < /eta_jet < 1
*		maximum energy in the forward gap 400 MeV
*
* Recommended value for Ptmin is 3.0 GeV (to be set in MC set up)
*
* Author : Mark Hayes (Mark.Hayes@bris.ac.uk)
*
* Reference : DESY-95-115
*             	Phys. Lett. B356 (1995) 129-146
*
*
*****************************************************************************
*
* HERA Tuning defined commons
	
	IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
        Integer chjet
	SAVE chjet
C --- choose same finder as in paper --- EUCELL
	DATA chjet/1/
	CHARACTER*6 jetf
	
        INTEGER NJET
c        DOUBLE PRECISION Jets(50,8)
        DOUBLE PRECISION CONER
	SAVE CONER

C --- HBOOK stuff ---
	REAL HI
C --- HBOOK stuff ---

	LOGICAL gap
	INTEGER loop,loop2,count,etamxlp
	DOUBLE PRECISION partrap,dummy,maxgapE
	DOUBLE PRECISION phi(2),px(2),py(2),pz(2),ptsq(2)
	DOUBLE PRECISION ppsq(2),energy(2),mass(2)
	DOUBLE PRECISION rap(2),Et(2)
	DOUBLE PRECISION ybj,q2

C --- cuts arrays
	DOUBLE PRECISION ybjcut(2),q2cut(2)
	DOUBLE PRECISION rapcut(2),Etcut(5)
	DOUBLE PRECISION etamxct(6)
	SAVE YBJCUT,Q2CUT,RAPCUT,ETCUT,ETAMXCT

C --- private copies of Xsec and Ntot from HERACMN
	DOUBLE PRECISION mhxsec(2),mhntot(2)

C *** Data for jet vs. cross section graph
	REAL deta(4),creta(4),crsta(4),crsys1(4),crsys2u(4),crsys2d(4)
C *** Actual data
	DATA deta/-0.75, -0.25, 0.25, 0.75/
	DATA creta/44, 39, 37, 16/
	DATA crsta/12, 11, 10, 6/
	DATA crsys1/8, 5, 10, 7/
	DATA crsys2u/10, 8, 8, 3/
	DATA crsys2d/-8, -5, -6, -3/
C *** Data for jet vs. integrated cross section graph varying eta0_max
	REAL bins(7)
	REAL d2eta(6),cr2eta(6),cr2sta(6),cr2sys1(6)
        Real cr2sys2u(6),cr2sys2d(6)
C *** Actual data
	DATA bins/0.75, 1.25, 1.75, 1.85, 2.15, 2.25, 2.55/
	DATA d2eta/1.0, 1.5, 1.8, 2.0, 2.2, 2.4/
	DATA cr2eta/27.3, 51.7, 67, 98, 145, 208/
	DATA cr2sta/5.5, 8.5, 10, 12, 15, 18/
	DATA cr2sys1/9.9, 9.2, 12, 14, 29, 43/
	DATA cr2sys2u/5.9, 10.7, 13, 20, 30, 40/
	DATA cr2sys2d/-4.5, -9.1, -11, -17, -23, -34/

	INTEGER intiflag, iproc, iflag

	REAL pttemp(4),pttemp2(6)
	INTEGER ivec,jvec

C --- choose direct/resolved
C --- recommend 1 - DIRECT, 2 - RESOLVED
	IF (iflag.gt.1000) THEN
	  iproc=INT(iflag/1000)
	  intiflag=MOD(iflag,1000)
	ENDIF
	IF ((iproc.gt.2).or.(iproc.lt.1)) THEN
	  PRINT*,'HZ95115 : Badly called routine'
	  PRINT*,'          please set intiflag +1000 or +2000'
	ENDIF

C --- choose jet finder
	IF ((intiflag.gt.10).and.(MOD(intiflag,10).eq.1)) THEN
	  chjet=INT(intiflag/10)
	  intiflag=MOD(intiflag,10)
	ENDIF

	IF (intiflag.eq.1) THEN
C *** Initialisation run ***
C *** Create directories ***
	  IF (iproc.eq.1) THEN
	    CALL HCDIR('//HISTO',' ')
	    CALL HMDIR('95115','S')
	    CALL HCDIR('//PAWC',' ')
	    CALL HMDIR('95115','S')

	    CALL HCDIR('//HISTO/95115',' ')
	    CALL HCDIR('//PAWC/95115',' ')
	    CALL HBOOK1(10,'MC cross section (nb) gap at 1.8'
     &		,4 ,-1.0 ,1.0 ,0.)
	    CALL HBOOKB(20,'MC cross section (nb) vary gap'
     &		,6 ,bins ,0.)
	  ENDIF
	    CALL HCDIR('//HISTO/95115',' ')
	    CALL HCDIR('//PAWC/95115',' ')
	    CALL HBOOK1(iproc*100+10,
     &	  	'MC cross section No. events gap at 1.8'
     &		,4 ,-1.0 ,1.0 ,0.)
	    CALL HBOOKB(iproc*100+20,
     &		'MC cross section No. events vary gap'
     &		,6 ,bins ,0.)

C --- photoproduction cuts

	  ybjcut(1)=0.2
	  ybjcut(2)=0.85
	  q2cut(1)=0.
	  q2cut(2)=4.

C --- basic cuts for rapidity plots
	  Etcut(1)=8.0
	  maxgapE =0.4
	  rapcut(1)=-1.0
	  rapcut(2)=1.0
	  etamxct(1)=1.0
	  etamxct(2)=1.5
	  etamxct(3)=1.8
	  etamxct(4)=2.0
	  etamxct(5)=2.2
	  etamxct(6)=2.4

C --- standard cone radius is 1.0
	  CALL HZJETRAD(2,CONER)
	  IF (CONER.lt.0.0) THEN
	    CONER=1.0
	  ENDIF
          CALL HZJTNAME(chjet,jetf)
	  WRITE(6,*)'**********************************************'
	  WRITE(6,*)'* HZ95115 called, histograms will be output. *'
	  WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
	  WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
	  WRITE(6,*)'**********************************************'

 1001	  FORMAT(A29,F5.2,A13)

	ELSE IF (intiflag.eq.2) THEN
C *** Filling runs ***
	
C *** Change directory to our graphs ***
	  CALL HCDIR('//PAWC/95115',' ')

	  q2=hzphokin(1)
	  IF (q2.eq.-1) THEN
	    PRINT*,'HZ95115: HZPHOKIN returns error for Q2'
	  ELSEIF (q2.lt.0) THEN
	    q2=abs(q2)
	  ENDIF
	  ybj=hzphokin(3)
	  IF (ybj.eq.-1) THEN
	    PRINT*,'HZ95115: HZPHOKIN returns error for ybj'
	  ENDIF

C *** Fill our histogram ***

	  IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(2))
     &		.and.(q2.lt.q2cut(2))) Then

	    DO etamxlp=1,6
	    gap=.TRUE.
	    DO loop=1,NHEP
	      IF (ISTHEP(loop).eq.1) THEN
		px(1)=PHEP(1,loop)
	        py(1)=PHEP(2,loop)
	        pz(1)=PHEP(3,loop)
                ptsq(1)=px(1)**2+py(1)**2
                ppsq(1)=(SQRT(ptsq(1)+pz(1)**2)+ABS(pz(1)))**2
                IF (ptsq(1).le.4.25E-18*ppsq(1)) THEN
                  partrap=20
                ELSE
                  partrap=0.5*LOG(ppsq(1)/ptsq(1))
                ENDIF
                partrap=SIGN(partrap,pz(1))
		IF ((partrap.gt.etamxct(etamxlp))
     &			.and.(partrap.lt.4.3)
     &			.and.(PHEP(4,loop).gt.maxgapE))
     &			Then
			  gap=.FALSE.
			ENDIF
	      ENDIF ! final state particle
	    ENDDO	! scan of HEPEVT for gap
	    IF (gap) THEN
              CALL hzjtfind(chjet,CONER,NJET,Jets)

	      IF (NJET.gt.0) THEN
	        count=1
	        DO loop2=1,NJET
	          IF (Jets(loop2,1).gt.Jets(count,1).and.
     &		    Jets(loop2,1).lt.1.and.Jets(loop2,1).gt.-1) THEN
	            count=loop2
	          ENDIF
	        ENDDO ! loop2 find forward jet
	        Et(1)=Jets(count,3)
	        rap(1)=Jets(count,1)

	        If ( (Et(1).gt.Etcut(1)).and.(rap(1).lt.rapcut(2)).and.
     &		(rap(1).gt.rapcut(1))) Then
		  IF (etamxlp.eq.3)
     & 		     CALL HFILL(iproc*100+10, REAL(rap(1)), 0., wtx)

		  CALL HFILL(iproc*100+20, REAL(etamxct(etamxlp)), 0., wtx)
	        ENDIF	! cuts on Et and -1<rap<1
	      ENDIF	! Making sure NJET.gt.0
	    ENDIF	! had we a gap?
	    ENDDO 	! etamxlp
	  ENDIF ! y & Q2 cuts

	ELSE IF (intiflag.eq.3) THEN
C *** Termination run ***
C *** Change directory ***
	  CALL HCDIR('//PAWC/95115',' ')
C *** Finish off histograms ***

	  IF (Xsec.eq.0) THEN
	    PRINT*,'HZ95115: termination called with zero cross section'
	    PRINT*,'          cross section graph meaningless'
	    PRINT*,'	      in process :',iproc*1000
	    Xsec=1
	  ENDIF
	  IF (Ntot.eq.0) THEN
	    PRINT*,'HZ95115: termination called with no total events'
	    PRINT*,'          cross section graph meaningless'
	    PRINT*,'	      in process :',iproc*1000
	    Ntot=1
	  ENDIF

C --- store Xsec and Ntot for both direct and resolved events.
	  mhXsec(iproc)=Xsec*1000
	  mhntot(iproc)=Ntot*1000
C --- remembering in HZ95115 to convert to pb! IMPORTANT!

	  IF (iproc.eq.2) THEN
C --- only finally
C --- create proper cross-section data
	    CALL HBOOK1(-10,'Data cross section (nb) gap at 1.8'
     &		,4 ,-1.0 ,1.0 ,0.)
	    CALL HBOOKB(-20,'Data cross section (nb) vary gap'
     &		,6 ,bins ,0.)
	    CALL HBOOK1(-11,'Data cross section (nb) gap at 1.8 full err'
     &		,4 ,-1.0 ,1.0 ,0.)
	    CALL HBOOKB(-21,
     &          'Data cross section (nb) vary gap full errors'
     &		,6 ,bins ,0.)

	    CALL HPAK(-10,creta)
	    CALL HPAKE(-10,crsta)
	    DO loop=1,4
	      pttemp(loop)=sqrt(crsta(loop)**2+crsys1(loop)**2)
	    ENDDO
	    CALL HPAK(-11,creta)
	    CALL HPAKE(-11,pttemp)
	
	    CALL HPAK(-20,cr2eta)
	    CALL HPAKE(-20,cr2sta)
	    DO loop=1,6
	      pttemp2(loop)=sqrt(cr2sta(loop)**2+cr2sys1(loop)**2)
	    ENDDO
	    CALL HPAK(-21,cr2eta)
	    CALL HPAKE(-21,pttemp2)

C --- now calculate cross-section graphs 10,20
	    CALL HOPERA(110,'+e',210,10,
     &		REAL(2*mhXsec(1)/mhNtot(1)), REAL(2*mhXsec(2)/mhNtot(2)))

	    CALL HBOOKB(50,'renorm histo - temp'
     &		,6 ,bins ,0.)
	    DO ivec=1,6
	     bins(ivec)=1/(bins(ivec+1)-bins(ivec))
	    ENDDO
	    CALL  HPAK (50,bins)	
C	    CALL HOPERA(120,'*e',50,120,1.,1.)
C	    CALL HOPERA(220,'*e',50,220,1.,1.)
	    CALL HOPERA(120,'+e',220,20,
     &		REAL(mhXsec(1)/mhNtot(1)), REAL(mhXsec(2)/mhNtot(2)))

	    CALL HZCHISQ(-10,10)
	    CALL HZCHISQ(-20,20)
	    PRINT*,'Xsec(1)=',mhXsec(1)
	    PRINT*,'Xsec(2)=',mhXsec(2)
	    PRINT*,'Ntot(1)=',mhNtot(1)
	    PRINT*,'Ntot(2)=',mhNtot(2)
	  ENDIF

	ELSE
C *** End ***
	  PRINT*,'HZ95115: Please run routine with iflag set to 1,2 or 3'
	  PRINT*,'         with +1000 or +2000 for DIR or RES events.'
	ENDIF

	RETURN
	END































