      Subroutine HZ95194(iflag)
*****************************************************************************
*     This photoproduction routine has to be run twice with the following
*     code additions:
*     +0    for DIRECT and RESOLVED together.
*     +1000 for the DIRECT component run.
*     +2000 for the RESOLVED component run.
*     +3000 for a special colour singlet contribution from somewhere.
*     for all three phases (iflag=1,2,3)
*
*     iflag = 1,2 or 3 depending on wether it is the initialization,
*     processing or termination phase (respectively)
*
*     The default jetfinder is EUCELL To change jetfinder use these
*     values to call the initialisation phase.  1 + 10 * (jetfinder
*     number)
*
* This routine produces three Monte Carlo graphs.
* id 10 is the normalised cross section in nb for two jet events
* id 20 is the normalised cross section in nb for gap events
* id 30 is the fraction of 20 over 10
*            These graph will be
*            meaningless unless Xsec and Ntot are set before
*            calling the termination routine.
*            (Xsec - total cross section returned by MC)
*            (Ntot - number of events passed to this routine)
* And three equivalent DATA graphs -10,-20,-30
*
* Cuts :       Q2 < 4
*            0.2< y_bj < 0.85
*            /eta_jet < 2.5
*            mod(/etabar) < 0.75
*            deltaeta > 2
*            maximum energy in the gap 300 MeV
*
* Recommended value for Ptmin is 2.5 GeV (to be set in MC set up)
*
* Author : Mark Hayes (Mark.Hayes@bris.ac.uk)
*
* Reference : DESY-95-194
*               Submitted to PLB
*
*
*****************************************************************************
*
*     HERA Tuning defined commons
      
      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"

      Integer chjet, i
      SAVE chjet
C     --- choose same finder as in paper --- EUCELL
      DATA chjet/1/
      CHARACTER*6 jetf
      
      INTEGER NJET
c      DOUBLE PRECISION Jets(50,8)
      DOUBLE PRECISION CONER
      SAVE CONER

C     --- HBOOK stuff ---
      REAL HI
C     --- HBOOK stuff ---
      
      LOGICAL ingap, hexist
      INTEGER loop,loop2,count
      DOUBLE PRECISION deltaeta,etabar
      DOUBLE PRECISION partrap,dummy,maxgapE
      DOUBLE PRECISION phi(2),px(2),py(2),pz(2),ptsq(2)
      DOUBLE PRECISION ppsq(2),energy(2),mass(2)
      DOUBLE PRECISION rap(2),Et(2)
      DOUBLE PRECISION ybj,q2
 
C --- cuts arrays
      DOUBLE PRECISION ybjcut(2),q2cut(2)
      DOUBLE PRECISION rapcut(2),Etcut(5)
      SAVE YBJCUT, Q2CUT, RAPCUT, ETCUT
      

C --- private copies of Xsec and Ntot from HERACMN
      DOUBLE PRECISION mhxsec(2),mhntot(2)
 
C *** Data for non-gap cross section
      REAL deta(4),creta(4),crsta(4),crsys1(4),crsys2(4)
C *** Actual data
      DATA deta/2.25, 2.75, 3.25, 3.75/
      DATA creta/4.93, 3.06, 1.67, 0.54/
      DATA crsta/0.24, 0.15, 0.07, 0.03/
      DATA crsys1/0.83, 0.54, 0.31, 0.08/
      DATA crsys2/-0.68, -0.52, -0.19, -0.03/
C *** Data for gap cross section
      REAL deta2(4),cr2eta(4),cr2sta(4),cr2sys1(4),cr2sys2(4)
C *** Actual data
      DATA deta2/2.25, 2.75, 3.25, 3.72/
      DATA cr2eta/2.85, 0.66, 0.16, 0.06/
      DATA cr2sta/0.17, 0.06, 0.02, 0.01/
      DATA cr2sys1/0.45, 0.11, 0.03, 0.01/
      DATA cr2sys2/-0.45, -0.15, -0.04, -0.01/
C *** Data for gap fraction
      REAL deta3(4),freta(4),frsta(4),frsys1(4),frsys2(4)
C *** Actual data
      DATA deta3/2.23, 2.73, 3.22, 3.70/
      DATA freta/0.58, 0.22, 0.10, 0.11/
      DATA frsta/0.04, 0.02, 0.01, 0.02/
      DATA frsys1/0.04, 0.02, 0.01, 0.01/
      DATA frsys2/-0.02, -0.02, -0.02, -0.02/
 
      INTEGER iproc, iflag
 
      REAL lumnb
      INTEGER ivec,jvec

      Character*6 xxxx
      DATA xxxx /'d95194'/
      
*     Determine the process type.
      iproc=(mod(iflag,10000)/1000)
      if ((iproc.lt.0).or.(iproc.gt.3)) then
         write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
         return
      endif
      
*        --- choose jet finder
      IF ((MOD(iflag,100)/10.gt.0).and.MOD(iflag,10).eq.1) THEN
         chjet=INT(MOD(iflag,100)/10)
      ENDIF
      
      IF (MOD(iflag,10).eq.1) then
         
C     *** Initialisation run ***
C     *** Create directories ***
         IF (iproc.le.1) THEN
            CALL HCDIR('//HISTO',' ')
            CALL HMDIR(xxxx,'S')
            CALL HCDIR('//PAWC',' ')
            CALL HMDIR(xxxx,'S')
            
            CALL HCDIR('//HISTO/'//xxxx,' ')
            CALL HCDIR('//PAWC/'//xxxx,' ')

            CALL HBOOK1(30,'Monte Carlo gap fraction',4,2.0,4.0,0.)

C     --- only finally
C     --- create proper cross-section data
            CALL HBOOK1(-10,'Data cross section (nb) non-gap',4,2.0,4.0
     $           ,0.)
            CALL HBOOK1(-20,'Data cross section (nb) gap',4 ,2.0 ,4.0 ,0
     $           .)
            CALL HBOOK1(-30,'Data gap fraction',4 ,2.0 ,4.0 ,0.)
            
            CALL HPAK(-10,creta)
            CALL HPAKE(-10,crsta)
            CALL HPAK(-20,cr2eta)
            CALL HPAKE(-20,cr2sta)
            CALL HPAK(-30,freta)
            CALL HPAKE(-30,frsta)


         ENDIF
         CALL HCDIR('//HISTO/'//xxxx,' ')
         CALL HCDIR('//PAWC/'//xxxx,' ')
         CALL HBOOK1(iproc*100+10, 'Monte Carlo cross section No. events
     $        non-gap' ,4 ,2.0 ,4.0 ,0.)
         CALL HBOOK1(iproc*100+20, 'Monte Carlo cross section No. events
     $        gap' ,4 ,2.0 ,4.0 ,0.)
          
         CALL HIDOPT(0,'stat')
         CALL HBARX(0)
         
         
C     --- photoproduction cuts
         
         ybjcut(1)=0.2
         ybjcut(2)=0.85
         q2cut(1)=0.
         q2cut(2)=4.        
         
C     --- basic cuts for rapidity plots
         Etcut(1)=6.0
         maxgapE =0.3
         rapcut(1)=-2.5
         rapcut(2)=2.5
         
C     --- standard cone radius is 1.0
         CALL HZJETRAD(2,CONER)
         IF (CONER.lt.0.0) THEN
            CONER=1.0
         ENDIF
         CALL HZJTNAME(chjet,jetf)
         WRITE(HZUNIT,*)'**********************************************'
         WRITE(HZUNIT,*)'* HZ95194 called, histograms will be output. *'
         WRITE(HZUNIT,*)'*      Jet finder selected : ',jetf
     $        ,'             *'
         WRITE(HZUNIT,1001)'*         with cone radius : ',CONER,'   *'
         WRITE(HZUNIT,*)'**********************************************'
 
 1001    FORMAT(A29,F5.2,A13)
 
      ELSE IF (MOD(iflag,10).eq.2) THEN
C     *** Filling runs ***
         
C     *** Change directory to our graphs ***
         CALL HCDIR('//PAWC/'//xxxx,' ')
         
         q2=hzphokin(1)
         IF (q2.eq.-1) THEN
            PRINT*,'HZ95194: HZPHOKIN returns error for Q2'
         ELSEIF (q2.lt.0) THEN
            q2=abs(q2)
         ENDIF
         ybj=hzphokin(3)
         IF (ybj.eq.-1) THEN
            PRINT*,'HZ95194: HZPHOKIN returns error for ybj'
         ENDIF
         
C     *** Fill our histogram ***
         
         IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(2)).and.(q2.lt
     $        .q2cut(2))) Then
         
            CALL hzjtfind(chjet,coner,njet,Jets)
            
            DO i=1,2
               et(i)=0
            ENDDO
         
c     Make sure we find the highest ET jets which are within the
c     rapidity interval.
            IF (njet.ge.2) THEN
               i=0
               DO loop=1,njet
c     Search jets in the rapidity interval.
                  IF ( jets(loop,1).gt.rapcut(1).and.jets(loop,1).lt
     $                 .rapcut(2) ) THEN
                     i=i+1
                     IF (i.LE.2) THEN
                        Et(i)=Jets(loop,3)
                        rap(i)=Jets(loop,1)
                     ENDIF
                  ENDIF
               ENDDO

               deltaeta=abs(rap(1)-rap(2))
               etabar=abs((rap(1)+rap(2))/2)
               
               If ( (Et(2).gt.Etcut(1)).and.(rap(1).lt.rapcut(2)).and
     $              .(rap(2).lt.2.5).and.(etabar.lt.0.75).and.(deltaeta
     $              .gt.2) ) Then
 
                  CALL HFILL(iproc*100+10, REAL(deltaeta), 0., wtx)
                  IF (rap(1).lt.rap(2)) THEN
                     dummy=rap(1)
                     rap(1)=rap(2)
                     rap(2)=dummy
                  ENDIF
                  ingap=.TRUE.
                  DO loop=1,NHEP
                     IF (ISTHEP(loop).eq.1) THEN
                        px(1)=PHEP(1,loop)
                        py(1)=PHEP(2,loop)
                        pz(1)=PHEP(3,loop)
                        ptsq(1)=px(1)**2+py(1)**2
                        ppsq(1)=(SQRT(ptsq(1)+pz(1)**2)+ABS(pz(1)))**2
                        IF (ptsq(1).le.4.25E-18*ppsq(1)) THEN
                           partrap=20
                        ELSE
                           partrap=0.5*LOG(ppsq(1)/ptsq(1))
                        ENDIF
                        partrap=SIGN(partrap,pz(1))
                        IF ((partrap.gt.rap(2)+CONER+0.001).and.(partrap
     $                       .lt.rap(1)-CONER-0.001).and.(sqrt(ptsq(1))
     $                       .gt.maxgapE))    Then
                           ingap=.FALSE.
                        ENDIF
                     ENDIF      ! final state particle
                  ENDDO         ! scan of HEPEVT
                  IF (ingap) Then
                     CALL HFILL(iproc*100+20, REAL(deltaeta), 0., wtx)
                  ENDIF
               ENDIF            ! cuts
            ENDIF               ! End of Njet.ge.2
            
         ENDIF                  ! y & Q2 cuts
         
      ELSE IF (MOD(iflag,10).eq.3) THEN
C     *** Termination run ***
C     *** Change directory ***
         CALL HCDIR('//PAWC/'//xxxx,' ')
C     *** Finish off histograms ***
         
         IF (Xsec.eq.0) THEN
            PRINT*,'HZ95194: termination called with zero cross section'
            PRINT*,'            cross section graph meaningless'
            PRINT*,'             in process :',iproc*1000
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,'HZ95194: termination called with no total events'
            PRINT*,'            cross section graph meaningless'
            PRINT*,'             in process :',iproc*1000
            Ntot=1
         ENDIF

c     Normalise cross section:
         lumnb = ntot/xsec
         CALL HZHINRM(100*iproc+10,0,lumnb,1)
         CALL HZHINRM(100*iproc+20,0,lumnb,1)
         
         IF (iproc.eq.2) THEN
            
C     --- now calculate cross-section graphs 10,20
            CALL HOPERA(110,'+e',210,10,1.,1.)
            CALL HOPERA(120,'+e',220,20,1.,1.)

         ENDIF

         IF (iproc.ne.1) then
            write(hzunit,*) 'HZ95194: finishing off with process ',iproc
         ENDIF

         IF (iproc.eq.3) THEN

C     --- now calculate cross-section graphs 10,20
            CALL HOPERA(310,'+e',10,10,1.,1.)
            CALL HOPERA(320,'+e',20,20,1.,1.)

         ENDIF

c     Do this for every iproc its the last one that counts!
         
         IF (HEXIST(10)) THEN
C     --- check that we can divide one by the other
            ingap=.TRUE.
            DO loop=1,4
               IF (HI(10,loop).lt.0.00001) ingap=.FALSE.
            ENDDO
            IF (ingap) Then
               CALL HOPERA(20,'/e',10, 30, 1., 1.)
            ELSE
               PRINT*,'HZ95194 : Some channels in histogram 10 are zero'
               PRINT*,'            will not calculate fraction graph.'
            ENDIF
         ENDIF
 
      ELSE
C     *** End ***
         PRINT*,'HZ95194: Please run routine with iflag set to 1,2 or 3'
         PRINT*,'          with +1000 or +2000 for DIR or RES events.'
      ENDIF
      
      RETURN
      END
      
