*CMZ :  2.00/04 31/07/2000  19.44.18  by  Tancredi Carli
*CMZ :  1.02/00 07/10/98  19.54.14  by  Tancredi Carli
*-- Author : Tancredi Carli
      subroutine hz97098 (iflag)
**********************************************************************
* Running: Ee=27.5, Ep=820, no polarization
*
* purpose: produce histograms for:
*
*          event shape variables: thrust, jet broadening, jet mass
*          ----------------------
*
*          Q-bins: low Q sample:
*                  1) 7 < Q < 8 GeV
*                  2) 8 < Q < 10 GeV
*                  hi Q sample:
*                  3) 14 < Q < 16 GeV
*                  4) 16 < Q < 20 GeV
*                  5) 20 < Q < 30 GeV
*                  6) 30 < Q < 50 GeV
*                  7) 50 < Q < 100 GeV
*
*          event selection:
*          i)   energy of scattered lepton > 10 GeV
*          ii)  polar angle of scattered lepton
*               within (157,173) for low Q sample or
*               within (30,150) for hi Q sample
*          iii) hadronic energy in forward region (polar angle
*               within (4,15)) > 0.5 GeV
*          iv)  total hadronic energy in Breit current
*               hemisphere > 0.1 GeV
*          v)   0.05 < y < 0.80
*          vi)  number of particles in Breit current hemisphere >= 2
*          vii) abs(value of evt.sh.var.-upper limit of evt.sh.var.)>nl
*               abs(value of evt.sh.var.-lower limit of evt.sh.var.)>nl
*               with nl > 5*10**-5
*               (separate for each variable)
*
*
* produced histos:
*
*            distributions: (QbinNo = 1..7 s.a.)
*            ID = 10 + QbinNo:     1/N dn/d(1-Tc)
*            ID = 20 + QbinNo:     1/N dn/d(1-Tz)
*            ID = 30 + QbinNo:     1/N dn/dBc
*            ID = 40 + QbinNo:     1/N dn/d(rho c)
*            mean values:
*            ID = 10:     <1-Tc>
*            ID = 20:     <1-Tz>/2
*            ID = 30:     <Bc>
*            ID = 40:     <rho c>
*
*            H1 data histograms have corresponding negative numbers.
*            Data histos with only statistic or only systematic errors
*            are stored with offset -100 and -200.
*
*
* Arguments: iflag=1 initialisation
*            iflag=2 filling
*            iflag=3 termination
*
* called subroutines and functions:
*            from hbook lib:  hcdir,hmdir,hbookb,hbook1,
*                             hpak,hpake,hbarx,hfill
*            from HzTool lib: devshp
*                             HzDiskin,HzIdelec,HzIpgam,HzIbeam,
*                             HzBrtini,HzBrt,HzHinrm,HzHinfo,HzChisq
*            and some subroutines contained in this file
*
* written by: Andreas von Manteuffel
*
* Reference:  DESY-97-098
*
**********************************************************************

      Implicit None

**********************************************************************
*
*                      declarations
*
**********************************************************************

*
#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
*
* subroutine-specific settings
*-----------------------------

* constants:

      Real pi,rd
      Parameter (PI=3.1415927,RD=180./PI)
      Logical highQ, lowQ
      Parameter (highQ = .true., lowQ = .false.)

* general variables:

*     DESY Preprint number
      Character*6 xxxx
      Data xxxx/'97098'/

      Logical lp
      Data lp /.true./
      save lp

* histo variables:

      Character*80 chtitl

* kinematic variables:

      Logical Qtype
      Real Q, x, y
      Real costheta, thetae, thetah, temp
      Double Precision Q2
      Double Precision Ehadfwd, EhadBrt
      Double Precision Plab (4)
      Double Precision Pbrt (4)
      Double Precision Phad (4, NMXHEP)
      Double Precision pbeam(4), pgam(4), p5(5)

* event shape variables:

      Real Qbinning(9)
      Data Qbinning / 7.0, 8.0, 10.0, 14.0, 16.0,
     .                20.0, 30.0, 50.0, 100.0 /

*     for H1 data

*      Real mQ(7)
      Real TcM(8), TcMae(8), TcMye(8)
      Real TzMh(8), TzMhae(8), TzMhye(8)
      Real BcM(8), BcMae(8), BcMye(8)
      Real rcM(8), rcMae(8), rcMye(8)

      Real Tc1n(10), Tc1ce(10), Tc1ae(10), Tc1ye(10)
      Real Tc2n(10), Tc2ce(10), Tc2ae(10), Tc2ye(10)
      Real Tc3n(10), Tc3ce(10), Tc3ae(10), Tc3ye(10)
      Real Tc4n(10), Tc4ce(10), Tc4ae(10), Tc4ye(10)
      Real Tc5n(10), Tc5ce(10), Tc5ae(10), Tc5ye(10)
      Real Tc6n(10), Tc6ce(10), Tc6ae(10), Tc6ye(10)
      Real Tc7n(10), Tc7ce(10), Tc7ae(10), Tc7ye(10)

      Real Bc1n(10), Bc1ce(10), Bc1ae(10), Bc1ye(10)
      Real Bc2n(10), Bc2ce(10), Bc2ae(10), Bc2ye(10)
      Real Bc3n(10), Bc3ce(10), Bc3ae(10), Bc3ye(10)
      Real Bc4n(10), Bc4ce(10), Bc4ae(10), Bc4ye(10)
      Real Bc5n(10), Bc5ce(10), Bc5ae(10), Bc5ye(10)
      Real Bc6n(10), Bc6ce(10), Bc6ae(10), Bc6ye(10)
      Real Bc7n(10), Bc7ce(10), Bc7ae(10), Bc7ye(10)

      Real rQ1n(10), rQ1ce(10), rQ1ae(10), rQ1ye(10)
      Real rQ2n(10), rQ2ce(10), rQ2ae(10), rQ2ye(10)
      Real rQ3n(10), rQ3ce(10), rQ3ae(10), rQ3ye(10)
      Real rQ4n(10), rQ4ce(10), rQ4ae(10), rQ4ye(10)
      Real rQ5n(10), rQ5ce(10), rQ5ae(10), rQ5ye(10)
      Real rQ6n(10), rQ6ce(10), rQ6ae(10), rQ6ye(10)
      Real rQ7n(10), rQ7ce(10), rQ7ae(10), rQ7ye(10)

      Real Tz1n(10), Tz1ce(10), Tz1ae(10), Tz1ye(10)
      Real Tz2n(10), Tz2ce(10), Tz2ae(10), Tz2ye(10)
      Real Tz3n(10), Tz3ce(10), Tz3ae(10), Tz3ye(10)
      Real Tz4n(10), Tz4ce(10), Tz4ae(10), Tz4ye(10)
      Real Tz5n(10), Tz5ce(10), Tz5ae(10), Tz5ye(10)
      Real Tz6n(10), Tz6ce(10), Tz6ae(10), Tz6ye(10)
      Real Tz7n(10), Tz7ce(10), Tz7ae(10), Tz7ye(10)

*     for MC calculation

      Real esMean(4,7)
      Real esMeanE(4,7)
      Real esMtmp(8)
      Real esMEtmp(8)

      Double Precision nl
      Parameter (nl=5.D-5)

      Double Precision lolimit (4)
      Double Precision hilimit (4)
      Double Precision evshpMC (4)
      Double Precision evshpSR (13)
      Double Precision esSum (4, 7)
      Double Precision esSqSum (4, 7)
      save esSum
      save esSqSum
      save lolimit, hilimit


* counting, index and errorcode variables:

      Integer ihep, iel, ibeam, igam
      Integer idum, ierr, iflag, id
      Integer ichn, iQbin, curQbin, iesType
      Integer ihad, icomp
      Integer inBrtCrr
      Real Nevt(4,7)
      Real locut(4,7)
      Real hicut(4,7)
      Double Precision ddum
      save Nevt

*********************************************************************
*
*                     data points from H1
*
*********************************************************************


* H1 event shape means:

*      Data mQ / 7.46, 8.74, 14.97, 17.75, 23.62, 36.72, 63.44 /

      Data TcM    / 0.1678, 0.1631, 0.0000, 0.1251, 0.1193,
     .              0.1072, 0.0880, 0.0746 /
      Data TcMae  / 0.0019, 0.0021, 0.0000, 0.0016, 0.0012,
     .              0.0012, 0.0018, 0.0037 /
      Data TcMye  / 0.0023, 0.0019, 0.0000, 0.0021, 0.0022,
     .              0.0025, 0.0021, 0.0023 /

      Data TzMh   / 0.2182, 0.2009, 0.0000, 0.1555, 0.1351,
     .              0.1125, 0.0912, 0.0635 /
      Data TzMhae / 0.0019, 0.0022, 0.0000, 0.0018, 0.0014,
     .              0.0013, 0.0019, 0.0035 /
      Data TzMhye / 0.0024, 0.0023, 0.0000, 0.0023, 0.0025,
     .              0.0024, 0.0023, 0.0049 /

      Data BcM    / 0.3566, 0.3414, 0.0000, 0.2978, 0.2704,
     .              0.2394, 0.2039, 0.1654 /
      Data BcMae  / 0.0019, 0.0023, 0.0000, 0.0018, 0.0013,
     .              0.0014, 0.0023, 0.0046 /
      Data BcMye  / 0.0071, 0.0076, 0.0000, 0.0069, 0.0067,
     .              0.0070, 0.0093, 0.0107 /

      Data rcM    / 0.1055, 0.1009, 0.0000, 0.0871, 0.0816,
     .              0.0780, 0.0637, 0.0512 /
      Data rcMae  / 0.0015, 0.0015, 0.0000, 0.0010, 0.0008,
     .              0.0008, 0.0013, 0.0024 /
      Data rcMye  / 0.0023, 0.0024, 0.0000, 0.0031, 0.0020,
     .              0.0030, 0.0030, 0.0021 /

* H1 event shape distributions:

      Data   Tc1n  / 2.0877, 3.0518, 3.9578, 3.8372, 3.1103,
     .               2.3335, 1.3402, 0.2815, 0.0000, 0.0000 /
      Data   Tc1ce / 0.5323, 0.4970, 0.4708, 0.5005, 0.4913,
     .               0.3423, 0.2596, 0.0671, 0.0000, 0.0000 /
      Data   Tc1ae / 0.0966, 0.1168, 0.1330, 0.1310, 0.1179,
     .               0.1021, 0.0774, 0.0355, 0.0000, 0.0000 /
      Data   Tc1ye / 0.5235, 0.4830, 0.4516, 0.4830, 0.4769,
     .               0.3267, 0.2478, 0.0569, 0.0000, 0.0000 /

      Data   Tc2n  / 2.0072, 3.2660, 4.1687, 3.7592, 3.0034,
     .               2.3871, 1.1947, 0.2138, 0.0000, 0.0000 /
      Data   Tc2ce / 0.4431, 0.4062, 0.3139, 0.3568, 0.3525,
     .               0.3007, 0.3037, 0.0822, 0.0000, 0.0000 /
      Data   Tc2ae / 0.1052, 0.1342, 0.1517, 0.1440, 0.1287,
     .               0.1148, 0.0812, 0.0343, 0.0000, 0.0000 /
      Data   Tc2ye / 0.4305, 0.3833, 0.2749, 0.3264, 0.3282,
     .               0.2779, 0.2927, 0.0747, 0.0000, 0.0000 /

      Data   Tc3n  / 3.1568, 5.2517, 4.5898, 3.2778, 1.8134,
     .               1.1952, 0.5285, 0.1692, 0.0176, 0.0000 /
      Data   Tc3ce / 0.4201, 0.3345, 0.3114, 0.2590, 0.1978,
     .               0.1698, 0.1153, 0.0626, 0.0230, 0.0000 /
      Data   Tc3ae / 0.1247, 0.1609, 0.1504, 0.1271, 0.0945,
     .               0.0767, 0.0510, 0.0289, 0.0093, 0.0000 /
      Data   Tc3ye / 0.4011, 0.2933, 0.2727, 0.2257, 0.1738,
     .               0.1515, 0.1034, 0.0556, 0.0210, 0.0000 /

      Data   Tc4n  / 2.8891, 5.8336, 4.9827, 2.9736, 1.6689,
     .               0.9901, 0.5237, 0.1305, 0.0078, 0.0000 /
      Data   Tc4ce / 0.3910, 0.4581, 0.4625, 0.3573, 0.1551,
     .               0.1097, 0.0757, 0.0401, 0.0064, 0.0000 /
      Data   Tc4ae / 0.0967, 0.1374, 0.1270, 0.0981, 0.0735,
     .               0.0566, 0.0412, 0.0206, 0.0050, 0.0000 /
      Data   Tc4ye / 0.3788, 0.4370, 0.4447, 0.3436, 0.1366,
     .               0.0940, 0.0635, 0.0344, 0.0039, 0.0000 /

      Data   Tc5n  / 3.9188, 6.7630, 4.2459, 2.3695, 1.3226,
     .               0.8789, 0.3943, 0.1006, 0.0063, 0.0000 /
      Data   Tc5ce / 0.6255, 0.5140, 0.3791, 0.2762, 0.1830,
     .               0.1138, 0.0661, 0.0233, 0.0083, 0.0000 /
      Data   Tc5ae / 0.1173, 0.1542, 0.1221, 0.0912, 0.0682,
     .               0.0556, 0.0372, 0.0188, 0.0047, 0.0000 /
      Data   Tc5ye / 0.6144, 0.4903, 0.3589, 0.2607, 0.1698,
     .               0.0992, 0.0547, 0.0137, 0.0068, 0.0000 /

      Data   Tc6n  / 6.4097, 6.6005, 2.9957, 1.7533, 1.2299,
     .               0.7981, 0.1782, 0.0347, 0.0000, 0.0000 /
      Data   Tc6ce / 0.4972, 0.4151, 0.2244, 0.1925, 0.1420,
     .               0.1039, 0.0553, 0.0306, 0.0000, 0.0000 /
      Data   Tc6ae / 0.2481, 0.2518, 0.1696, 0.1298, 0.1087,
     .               0.0876, 0.0414, 0.0182, 0.0000, 0.0000 /
      Data   Tc6ye / 0.4309, 0.3300, 0.1469, 0.1422, 0.0914,
     .               0.0560, 0.0367, 0.0246, 0.0000, 0.0000 /

      Data   Tc7n  / 9.8287, 4.7100, 2.5708, 1.3032, 0.9634,
     .               0.4957, 0.0808, 0.0000, 0.0000, 0.0000 /
      Data   Tc7ce / 0.7650, 0.5787, 0.4209, 0.2774, 0.2544,
     .               0.1661, 0.0738, 0.0000, 0.0000, 0.0000 /
      Data   Tc7ae / 0.6833, 0.4730, 0.3495, 0.2488, 0.2139,
     .               0.1535, 0.0619, 0.0000, 0.0000, 0.0000 /
      Data   Tc7ye / 0.3440, 0.3333, 0.2346, 0.1227, 0.1377,
     .               0.0635, 0.0403, 0.0000, 0.0000, 0.0000 /


      Data   Bc1n  / 0.0000, 0.0485, 0.2351, 0.8510, 1.6524,
     .               2.7155, 3.4655, 3.5857, 3.7861, 3.6558 /
      Data   Bc1ce / 0.0000, 0.0538, 0.1069, 0.2621, 0.2574,
     .               0.4552, 0.3371, 0.8237, 0.3572, 0.3846 /
      Data   Bc1ae / 0.0000, 0.0147, 0.0324, 0.0617, 0.0859,
     .               0.1102, 0.1245, 0.1266, 0.1301, 0.1278 /
      Data   Bc1ye / 0.0000, 0.0517, 0.1019, 0.2548, 0.2426,
     .               0.4417, 0.3133, 0.8139, 0.3327, 0.3627 /

      Data   Bc2n  / 0.0116, 0.1025, 0.4408, 1.2082, 2.2011,
     .               2.9718, 3.2057, 3.3449, 3.5951, 2.9183 /
      Data   Bc2ce / 0.0129, 0.0613, 0.1912, 0.3510, 0.6052,
     .               0.2860, 0.6425, 0.3384, 0.3923, 0.2642 /
      Data   Bc2ae / 0.0080, 0.0238, 0.0493, 0.0816, 0.1102,
     .               0.1280, 0.1330, 0.1358, 0.1408, 0.1269 /
      Data   Bc2ye / 0.0101, 0.0565, 0.1848, 0.3414, 0.5951,
     .               0.2558, 0.6286, 0.3099, 0.3662, 0.2317 /

      Data   Bc3n  / 0.0092, 0.2407, 1.1766, 2.1920, 3.2427,
     .               2.7891, 2.7533, 2.5151, 2.2751, 2.1063 /
      Data   Bc3ce / 0.0115, 0.0756, 0.2135, 0.1674, 0.2969,
     .               0.2869, 0.2177, 0.1909, 0.1473, 0.1921 /
      Data   Bc3ae / 0.0067, 0.0344, 0.0761, 0.1039, 0.1264,
     .               0.1172, 0.1165, 0.1178, 0.1104, 0.1066 /
      Data   Bc3ye / 0.0094, 0.0673, 0.1994, 0.1312, 0.2687,
     .               0.2618, 0.1839, 0.1502, 0.0976, 0.1598 /

      Data   Bc4n  / 0.0189, 0.4277, 1.9846, 3.7219, 3.4220,
     .               2.8145, 2.2936, 2.1189, 1.8803, 1.2980 /
      Data   Bc4ce / 0.0639, 0.1326, 0.1521, 0.2687, 0.1755,
     .               0.2112, 0.2077, 0.1637, 0.1667, 0.1754 /
      Data   Bc4ae / 0.0078, 0.0372, 0.0802, 0.1098, 0.1053,
     .               0.0955, 0.0862, 0.0828, 0.0780, 0.0648 /
      Data   Bc4ye / 0.0634, 0.1273, 0.1293, 0.2452, 0.1403,
     .               0.1884, 0.1889, 0.1412, 0.1473, 0.1630 /

      Data   Bc5n  / 0.0381, 1.0561, 3.5305, 4.2269, 3.2213,
     .               2.4026, 1.8228, 1.4735, 1.2234, 1.0049 /
      Data   Bc5ce / 0.1019, 0.2156, 0.2112, 0.2372, 0.1847,
     .               0.1623, 0.2044, 0.1681, 0.1342, 0.1574 /
      Data   Bc5ae / 0.0116, 0.0609, 0.1114, 0.1219, 0.1064,
     .               0.0919, 0.0800, 0.0720, 0.0656, 0.0594 /
      Data   Bc5ye / 0.1013, 0.2068, 0.1794, 0.2035, 0.1510,
     .               0.1338, 0.1881, 0.1519, 0.1170, 0.1457 /

      Data   Bc6n  / 0.2039, 3.2918, 4.7431, 3.7597, 2.5149,
     .               1.7113, 1.4350, 0.9139, 0.7761, 0.6503 /
      Data   Bc6ce / 0.0549, 0.2307, 0.2652, 0.3063, 0.2733,
     .               0.2366, 0.1921, 0.1794, 0.1594, 0.1580 /
      Data   Bc6ae / 0.0443, 0.1778, 0.2135, 0.1900, 0.1554,
     .               0.1282, 0.1174, 0.0937, 0.0863, 0.0790 /
      Data   Bc6ye / 0.0323, 0.1470, 0.1574, 0.2402, 0.2248,
     .               0.1988, 0.1521, 0.1530, 0.1340, 0.1368 /

      Data   Bc7n  / 0.8798, 6.3943, 4.1233, 3.0085, 2.1221,
     .               0.9383, 1.1414, 0.6296, 0.4602, 0.3023 /
      Data   Bc7ce / 0.2969, 0.6440, 0.5603, 0.4946, 0.3825,
     .               0.3122, 0.2898, 0.2179, 0.2005, 0.1960 /
      Data   Bc7ae / 0.2044, 0.5512, 0.4426, 0.3781, 0.3175,
     .               0.2111, 0.2329, 0.1729, 0.1479, 0.1198 /
      Data   Bc7ye / 0.2153, 0.3331, 0.3436, 0.3189, 0.2132,
     .               0.2300, 0.1724, 0.1326, 0.1354, 0.1551 /


      Data   rQ1n  / 4.8012, 6.3386, 4.5072, 2.3381, 1.1538,
     .               0.5376, 0.2707, 0.0349, 0.0134, 0.0000 /
      Data   rQ1ce / 2.1514, 1.0142, 1.1607, 0.7741, 0.5244,
     .               0.3359, 0.4088, 0.0372, 0.0412, 0.0000 /
      Data   rQ1ae / 0.1466, 0.1684, 0.1420, 0.1023, 0.0719,
     .               0.0490, 0.0348, 0.0125, 0.0077, 0.0000 /
      Data   rQ1ye / 2.1464, 1.0002, 1.1520, 0.7673, 0.5195,
     .               0.3323, 0.4073, 0.0350, 0.0404, 0.0000 /

      Data   rQ2n  / 4.7909, 6.5121, 4.7740, 2.2681, 1.1284,
     .               0.3522, 0.1111, 0.0288, 0.0046, 0.0077 /
      Data   rQ2ce / 1.9725, 1.2987, 1.6879, 0.5421, 0.7114,
     .               0.2422, 0.1813, 0.0330, 0.0091, 0.0180 /
      Data   rQ2ae / 0.1626, 0.1895, 0.1623, 0.1119, 0.0789,
     .               0.0441, 0.0248, 0.0126, 0.0051, 0.0065 /
      Data   rQ2ye / 1.9658, 1.2848, 1.6801, 0.5305, 0.7070,
     .               0.2381, 0.1796, 0.0305, 0.0075, 0.0168 /

      Data   rQ3n  / 5.6025, 7.5452, 3.8745, 1.7123, 0.6989,
     .               0.3300, 0.1171, 0.0286, 0.0169, 0.0100 /
      Data   rQ3ce / 0.6977, 0.5385, 0.3632, 0.2932, 0.2224,
     .               0.1669, 0.1038, 0.0538, 0.0259, 0.0124 /
      Data   rQ3ae / 0.1662, 0.1929, 0.1382, 0.0919, 0.0587,
     .               0.0403, 0.0240, 0.0119, 0.0091, 0.0070 /
      Data   rQ3ye / 0.6776, 0.5028, 0.3359, 0.2784, 0.2145,
     .               0.1619, 0.1010, 0.0524, 0.0242, 0.0103 /

      Data   rQ4n  / 5.8796, 8.0333, 3.4587, 1.3468, 0.6857,
     .               0.3460, 0.1535, 0.0609, 0.0219, 0.0136 /
      Data   rQ4ce / 0.8759, 0.6765, 0.5210, 0.2107, 0.1642,
     .               0.0917, 0.0532, 0.0241, 0.0148, 0.0089 /
      Data   rQ4ae / 0.1380, 0.1613, 0.1059, 0.0661, 0.0471,
     .               0.0335, 0.0223, 0.0141, 0.0084, 0.0066 /
      Data   rQ4ye / 0.8650, 0.6570, 0.5101, 0.2001, 0.1573,
     .               0.0853, 0.0483, 0.0196, 0.0121, 0.0060 /

      Data   rQ5n  / 7.1262, 7.5957, 2.8019, 1.2664, 0.5955,
     .               0.3118, 0.1798, 0.0648, 0.0322, 0.0116 /
      Data   rQ5ce / 0.3919, 0.3084, 0.1587, 0.0953, 0.0833,
     .               0.0610, 0.0368, 0.0218, 0.0126, 0.0077 /
      Data   rQ5ae / 0.1583, 0.1635, 0.0993, 0.0667, 0.0458,
     .               0.0331, 0.0251, 0.0151, 0.0106, 0.0064 /
      Data   rQ5ye / 0.3585, 0.2615, 0.1238, 0.0680, 0.0696,
     .               0.0513, 0.0269, 0.0157, 0.0068, 0.0043 /

      Data   rQ6n  / 10.1500, 5.7132, 2.1353, 0.9746, 0.5177,
     .               0.2396, 0.1181, 0.0511, 0.0619, 0.0000 /
      Data   rQ6ce / 0.5295, 0.3389, 0.2012, 0.1366, 0.0867,
     .               0.0549, 0.0433, 0.0234, 0.0267, 0.0000 /
      Data   rQ6ae / 0.3124, 0.2344, 0.1433, 0.0968, 0.0706,
     .               0.0480, 0.0337, 0.0222, 0.0244, 0.0000 /
      Data   rQ6ye / 0.4275, 0.2448, 0.1413, 0.0963, 0.0503,
     .               0.0267, 0.0272, 0.0075, 0.0108, 0.0000 /

      Data   rQ7n  / 12.1888, 4.1333, 1.3569, 1.2022, 0.1850,
     .               0.1583, 0.0776, 0.0776, 0.0000, 0.0000 /
      Data   rQ7ce / 0.8506, 0.5336, 0.3832, 0.3521, 0.2244,
     .               0.1892, 0.1118, 0.0764, 0.0000, 0.0000 /
      Data   rQ7ae / 0.7628, 0.4442, 0.2545, 0.2395, 0.0940,
     .               0.0869, 0.0608, 0.0608, 0.0000, 0.0000 /
      Data   rQ7ye / 0.3763, 0.2957, 0.2865, 0.2581, 0.2037,
     .               0.1680, 0.0938, 0.0463, 0.0000, 0.0000 /


      Data   Tz1n  / 0.3194, 1.2483, 1.6760, 1.8007, 1.5085,
     .               1.2338, 0.9597, 0.7942, 0.3717, 0.0876 /
      Data   Tz1ce / 0.0919, 0.1087, 0.2119, 0.1224, 0.0806,
     .               0.0986, 0.2136, 0.1005, 0.0731, 0.0611 /
      Data   Tz1ae / 0.0267, 0.0528, 0.0612, 0.0634, 0.0581,
     .               0.0525, 0.0463, 0.0421, 0.0288, 0.0140 /
      Data   Tz1ye / 0.0880, 0.0950, 0.2029, 0.1047, 0.0559,
     .               0.0835, 0.2085, 0.0913, 0.0672, 0.0595 /

      Data   Tz2n  / 0.5315, 1.5205, 1.8928, 1.7275, 1.3058,
     .               1.1317, 0.9133, 0.6219, 0.2721, 0.0829 /
      Data   Tz2ce / 0.1725, 0.2173, 0.2157, 0.2438, 0.3373,
     .               0.1442, 0.0816, 0.1366, 0.1145, 0.0475 /
      Data   Tz2ae / 0.0383, 0.0648, 0.0723, 0.0690, 0.0600,
     .               0.0559, 0.0502, 0.0414, 0.0274, 0.0151 /
      Data   Tz2ye / 0.1682, 0.2074, 0.2032, 0.2339, 0.3319,
     .               0.1329, 0.0643, 0.1302, 0.1111, 0.0450 /

      Data   Tz3n  / 1.1746, 2.4551, 1.8801, 1.3135, 1.0126,
     .               0.7693, 0.6577, 0.4146, 0.2738, 0.0486 /
      Data   Tz3ce / 0.0893, 0.1452, 0.1324, 0.1081, 0.0900,
     .               0.0843, 0.0723, 0.0535, 0.0457, 0.0215 /
      Data   Tz3ae / 0.0538, 0.0778, 0.0681, 0.0569, 0.0499,
     .               0.0435, 0.0403, 0.0320, 0.0260, 0.0109 /
      Data   Tz3ye / 0.0713, 0.1225, 0.1135, 0.0919, 0.0749,
     .               0.0722, 0.0600, 0.0428, 0.0376, 0.0185 /

      Data   Tz4n  / 1.8672, 3.1374, 1.7572, 1.0802, 0.7486,
     .               0.5941, 0.3737, 0.2730, 0.1441, 0.0245 /
      Data   Tz4ce / 0.1554, 0.1534, 0.1053, 0.0985, 0.0554,
     .               0.0501, 0.0401, 0.0370, 0.0301, 0.0215 /
      Data   Tz4ae / 0.0550, 0.0713, 0.0533, 0.0418, 0.0348,
     .               0.0310, 0.0246, 0.0210, 0.0153, 0.0063 /
      Data   Tz4ye / 0.1454, 0.1358, 0.0908, 0.0891, 0.0431,
     .               0.0393, 0.0316, 0.0305, 0.0259, 0.0206 /

      Data   Tz5n  / 3.1173, 3.1771, 1.3783, 0.8068, 0.5588,
     .               0.3631, 0.2630, 0.2119, 0.0996, 0.0242 /
      Data   Tz5ce / 0.1321, 0.1430, 0.0826, 0.0688, 0.0674,
     .               0.0470, 0.0513, 0.0480, 0.0356, 0.0325 /
      Data   Tz5ae / 0.0740, 0.0747, 0.0492, 0.0376, 0.0313,
     .               0.0253, 0.0215, 0.0193, 0.0132, 0.0065 /
      Data   Tz5ye / 0.1094, 0.1219, 0.0664, 0.0576, 0.0597,
     .               0.0396, 0.0466, 0.0439, 0.0331, 0.0319 /

      Data   Tz6n  / 4.9564, 2.4416, 1.0788, 0.5708, 0.3575,
     .               0.1930, 0.1551, 0.1425, 0.0858, 0.0185 /
      Data   Tz6ce / 0.1655, 0.1172, 0.0745, 0.0580, 0.0481,
     .               0.0377, 0.0330, 0.0304, 0.0243, 0.0140 /
      Data   Tz6ae / 0.1543, 0.1083, 0.0720, 0.0524, 0.0414,
     .               0.0304, 0.0273, 0.0262, 0.0203, 0.0094 /
      Data   Tz6ye / 0.0599, 0.0447, 0.0192, 0.0248, 0.0244,
     .               0.0223, 0.0185, 0.0154, 0.0133, 0.0104 /

      Data   Tz7n  / 6.3357, 1.9493, 0.7622, 0.4186, 0.1981,
     .               0.2249, 0.0389, 0.0308, 0.0178, 0.0000 /
      Data   Tz7ce / 0.3903, 0.2224, 0.1481, 0.1133, 0.0812,
     .               0.0824, 0.0439, 0.0390, 0.0337, 0.0000 /
      Data   Tz7ae / 0.3879, 0.2152, 0.1346, 0.0997, 0.0686,
     .               0.0731, 0.0304, 0.0270, 0.0206, 0.0000 /
      Data   Tz7ye / 0.0436, 0.0560, 0.0617, 0.0539, 0.0435,
     .               0.0381, 0.0316, 0.0282, 0.0267, 0.0000 /


*********************************************************************
*
*                      Initialization
*
*********************************************************************


*

      if (iflag.eq.1) then
*
*
*      Initialisation: The following MUST always be done
*      (i) make subdirectory in PAWC
*       - use the name as the xxxxxx in HZxxxxxx subroutine
*      (ii) make subdirectory in o/p file
*

         call hcdir('//PAWC',' ')
         call hmdir(xxxx,'S')
         call hcdir('//HISTO',' ')
         call hmdir(xxxx,'S')

* hello world !

	 WRITE(6,*)'***************************************************'
	 WRITE(6,*)'HZ'//xxxx//' called'
     	 WRITE(6,*)'***************************************************'

*
* remind:
*     HistoIDs:  ID < 0  for H1 data
*                ID > 0  for corresponding generator histos
*               |ID| = i        for combined errors
*               |ID| = 100 + i  for statist. errors only
*               |ID| = 200 + i  for system. errors only
*
*


* DISTRIBUTION - HISTOS:
* build up H1-histos and prepare MC-histos

         write(chtitl,*) '1/N dn/d(1-Tc) in Breit frame'
         call H1distr(11,chtitl,10,0.,.5,Tc1n,Tc1ce,Tc1ae,Tc1ye)
         call H1distr(12,chtitl,10,0.,.5,Tc2n,Tc2ce,Tc2ae,Tc2ye)
         call H1distr(13,chtitl,10,0.,.5,Tc3n,Tc3ce,Tc3ae,Tc3ye)
         call H1distr(14,chtitl,10,0.,.5,Tc4n,Tc4ce,Tc4ae,Tc4ye)
         call H1distr(15,chtitl,10,0.,.5,Tc5n,Tc5ce,Tc5ae,Tc5ye)
         call H1distr(16,chtitl,10,0.,.5,Tc6n,Tc6ce,Tc6ae,Tc6ye)
         call H1distr(17,chtitl,10,0.,.5,Tc7n,Tc7ce,Tc7ae,Tc7ye)
         call MCdistr(10,chtitl,10,0.,.5, 7)

         write(chtitl,*) '1/N dn/d(1-Tz) in Breit frame'
         call H1distr(21,chtitl,10,0.,1.,Tz1n,Tz1ce,Tz1ae,Tz1ye)
         call H1distr(22,chtitl,10,0.,1.,Tz2n,Tz2ce,Tz2ae,Tz2ye)
         call H1distr(23,chtitl,10,0.,1.,Tz3n,Tz3ce,Tz3ae,Tz3ye)
         call H1distr(24,chtitl,10,0.,1.,Tz4n,Tz4ce,Tz4ae,Tz4ye)
         call H1distr(25,chtitl,10,0.,1.,Tz5n,Tz5ce,Tz5ae,Tz5ye)
         call H1distr(26,chtitl,10,0.,1.,Tz6n,Tz6ce,Tz6ae,Tz6ye)
         call H1distr(27,chtitl,10,0.,1.,Tz7n,Tz7ce,Tz7ae,Tz7ye)
         call MCdistr(20,chtitl,10,0.,1., 7)

         write(chtitl,*) '1/N dn/d(Bc) in Breit frame'
         call H1distr(31,chtitl,10,0.,.5,Bc1n,Bc1ce,Bc1ae,Bc1ye)
         call H1distr(32,chtitl,10,0.,.5,Bc2n,Bc2ce,Bc2ae,Bc2ye)
         call H1distr(33,chtitl,10,0.,.5,Bc3n,Bc3ce,Bc3ae,Bc3ye)
         call H1distr(34,chtitl,10,0.,.5,Bc4n,Bc4ce,Bc4ae,Bc4ye)
         call H1distr(35,chtitl,10,0.,.5,Bc5n,Bc5ce,Bc5ae,Bc5ye)
         call H1distr(36,chtitl,10,0.,.5,Bc6n,Bc6ce,Bc6ae,Bc6ye)
         call H1distr(37,chtitl,10,0.,.5,Bc7n,Bc7ce,Bc7ae,Bc7ye)
         call MCdistr(30,chtitl,10,0.,.5, 7)

         write(chtitl,*) '1/N dn/d(rho Q) in Breit frame'
         call H1distr(41,chtitl,10,0.,.5,rQ1n,rQ1ce,rQ1ae,rQ1ye)
         call H1distr(42,chtitl,10,0.,.5,rQ2n,rQ2ce,rQ2ae,rQ2ye)
         call H1distr(43,chtitl,10,0.,.5,rQ3n,rQ3ce,rQ3ae,rQ3ye)
         call H1distr(44,chtitl,10,0.,.5,rQ4n,rQ4ce,rQ4ae,rQ4ye)
         call H1distr(45,chtitl,10,0.,.5,rQ5n,rQ5ce,rQ5ae,rQ5ye)
         call H1distr(46,chtitl,10,0.,.5,rQ6n,rQ6ce,rQ6ae,rQ6ye)
         call H1distr(47,chtitl,10,0.,.5,rQ7n,rQ7ce,rQ7ae,rQ7ye)
         call MCdistr(40,chtitl,10,0.,.5, 7)


* MEAN VALUE - HISTOS:
* build up H1-histos and prepare MC-histos

         write(chtitl,*) 'mean(1-Tc) in Breit frame'
         call H1means(10,chtitl,8,Qbinning,TcM,TcMae,TcMye)
         call MCmeans(10,chtitl,8,Qbinning)
         write(chtitl,*) 'mean((1-Tz)/2) in Breit frame'
         call H1means(20,chtitl,8,Qbinning,TzMh,TzMhae,TzMhye)
         call MCmeans(20,chtitl,8,Qbinning)
         write(chtitl,*) 'mean(1-Bc) in Breit frame'
         call H1means(30,chtitl,8,Qbinning,BcM,BcMae,BcMye)
         call MCmeans(30,chtitl,8,Qbinning)
         write(chtitl,*) 'mean(rho Q) in Breit frame'
         call H1means(40,chtitl,8,Qbinning,rcM,rcMae,rcMye)
         call MCmeans(40,chtitl,8,Qbinning)


* prepare control histo

         write(chtitl,*) 'dn/dQ in lab. frame - control plot'
         call hbook1(1, chtitl, 100, 0., 100., 0.)
         call hbarx(1)


* initialize counter variables

         do iesType = 1, 4
            do iQbin = 1, 7
               esSum(iesType, iQbin) = 0.D0
               esSqSum(iesType, iQbin) = 0.D0
               Nevt(iesType, iQbin) = 0.
               locut(iesType, iQbin) = 0.
               hicut(iesType, iQbin) = 0.
            enddo
         enddo

* initialize exclusive limits for event shapes in MC calculation

      lolimit(1) = nl
      lolimit(2) = nl
      lolimit(3) = nl
      lolimit(4) = nl
      hilimit(1) = 5.D-1 - nl
      hilimit(2) = 1.D0 - nl
      hilimit(3) = 5.D-1 - nl
      hilimit(4) = 5.D-1 - nl


*********************************************************************
*
*                      Event Processing
*
*********************************************************************

      else if (iflag.eq.2) then


*      Filling: The following MUST always be done
*      (i) move to the correct sub-directory in PAWC
*
         call hcdir('//PAWC/'//xxxx,' ')

*----------------
* Event selection
*----------------

* get Q2
         Q2 = HzDiskin(1)

* get Q and fill control histo
         if (Q2.le.0.D0) then
            write(6,*) 'Hz'//xxxx, ' ERROR: Q2 less or equal zero !!!'
            goto 9990
         else
            Q = real(sqrt(Q2))
            call hfill(1,Q,0.,wtx)
         endif


* Q-binning:  Q has to be within [7 GeV, 10 GeV] (low Q) or
*-----------              within [14 GeV, 100 GeV] (high Q)

         if ((Q.ge.7.).and.(Q.le.10.)) then
            Qtype = lowQ
            if (Q.lt.8.)  then
               curQbin = 1
            else
               curQbin = 2
            endif
         elseif ((Q.ge.14.).and.(Q.le.100.)) then
            Qtype = highQ
            if (Q.lt.16.) then
               curQbin = 3
            elseif (Q.lt.20.) then
               curQbin = 4
            elseif (Q.lt.30.) then
               curQbin = 5
            elseif (Q.lt.50.) then
               curQbin = 6
            else
               curQbin = 7
            endif
         else
            goto 9990
         endif


* detect lepton
         iel = HzIdelec(idum)
         if ( (iel.eq.-1).or.
     .        (iel.lt.1).or.(iel.gt.NMXHEP) ) then
            write(6,*) 'Hz'//xxxx,' ERROR: lepton not found !!!'
            goto 9990
         endif

* CUT (i): energy of scattered lepton > 10 GeV
*---------
         if (real(PHEP(4,iel)).le.10.) goto 9990


* get polar angle of scattered lepton
         costheta =  real(PHEP(3,iel)) /
     .        sqrt( real(PHEP(1,iel))**2 + real(PHEP(2,iel))**2
     .        + real(PHEP(3,iel))**2 )
         if (abs(costheta).gt.1.) then
            write(6,*) 'Hz'//xxxx, ' ERROR: abs(cos(thetae)) > 1 !!!'
            goto 9990
         endif
         thetae = RD * acos(costheta)


* CUT (ii): polar angle of scattered lepton within [157, 173] (low Q)
*----------                              or within [30, 150] (high Q)

         if (Qtype.eqv.lowQ) then
            if ((thetae.le.157.).or.(thetae.ge.173.)) then
               goto 9990
            endif
         elseif (Qtype.eqv.highQ) then
            if ((thetae.le.30.).or.(thetae.ge.150.)) then
               goto 9990
            endif
         endif

* get y
         y = Real(HzDiskin(3))

* CUT (v): y within [.05,0.8]
*---------
         if ((y.lt.0.05).or.(y.gt.0.80)) goto 9990


* init boost to Breit frame
         ierr = HzIbeam(ibeam,idum)
         if ( (ierr.ne.1).or.
     .        (ibeam.lt.1).or.(ibeam.gt.NMXHEP).or.
     .        (idum.lt.1).or.(idum.gt.NMXHEP)       ) then
            write(6,*) 'HZ'//xxxx,' ERROR: beams not found !!!'
            goto 9990
         else
            Do icomp = 1, 4
               pbeam(icomp)=PHEP(icomp,IBEAM)
            enddo
         endif

         ierr = HzIpgam(p5)
         if (ierr.eq.-1) then
            write(6,*) 'HZ'//xxxx,' ERROR: boson vector not found !!!'
            goto 9990
         else
            Do icomp = 1, 4
               pgam(icomp) = p5(icomp)
            enddo
         endif

         call HzBrtini (pbeam, pgam, ierr)
         if (ierr.eq.-1) then
            write(6,*)
     .          'HZ'//xxxx,' problem with boost to Breit frame !'
            goto 9990
         endif

* loop over HEP common
         ihad = 0
         Ehadfwd = 0.D0
         inBrtCrr = 0
         EhadBrt = 0.D0
         do 5000 ihep = 1, nhep
*     skip lepton and unstable particles
            if ((isthep(ihep).ne.1).or.(ihep.eq.iel))  goto 5000

*     get polar angle of hadron
            temp =  sqrt( real(PHEP(1,ihep))**2 + real(PHEP(2,ihep))**2
     .           + real(PHEP(3,ihep))**2 )
            if (temp.ne.0.) then
               costheta =  real(PHEP(3,ihep)) / temp
               if (abs(costheta).gt.1.) then
                  write(6,*) 'Hz'//xxxx, 'ERROR: abs(cos(thetah))>1 !!!'
                  goto 9990
               endif
               thetah = RD * acos(costheta)
            else
               thetah = 0.
            endif

*     sum up hadronic energy in forward direction
            if ((thetah.gt.4.).and.(thetah.lt.15.))  then
               Ehadfwd = Ehadfwd + PHEP(4, ihep)
            endif

*     perform boost to Breit frame
            do icomp = 1, 4
               Plab(icomp) = PHEP(icomp, IHEP)
            enddo
            call HzBrt(plab, pbrt, ierr)
            if (ierr.eq.-1) then
               write(6,*) 'HZ'//xxxx,
     .              ' problem with boost to Breit frame !'
               goto 9990
            endif

*     copy to array suitable for subroutine devshp
            ihad = ihad + 1
            Phad(1, ihad) = pbrt(4)
            Phad(2, ihad) = pbrt(1)
            Phad(3, ihad) = pbrt(2)
            Phad(4, ihad) = pbrt(3)

*     particle in Breit current hemisphere ?
            if (pbrt(3).LT.0.D0) then
*     sum up number of particles in Breit current hemisphere
               inBrtCrr = inBrtCrr + 1
*     sum up hadronic energy in Breit current hemisphere
               EhadBrt = EhadBrt + Pbrt(4)
            endif

 5000    continue


* CUT (iii): hadronic energy in forward direction >= 0.5 GeV
*-----------
         if (real(Ehadfwd).lt.0.5) goto 9990

* CUT (vi): number of particles in Breit current hemisphere >= 2
*----------
         if (inBrtCrr.lt.2) goto 9990

* CUT (iv): total hadronic energy in Breit current hemisphere  >= 0.1 Q
*----------
         if (real(EhadBrt).lt.(0.1*Q)) goto 9990



*-------------------------------------
* Calculation of event shape variables
*-------------------------------------

* get event shape variables
         call devshp (Phad, ihad, Q2, evshpSR, idum, ddum)

* copy into internal array
         evshpMC(1) = evshpSR(1)
         evshpMC(2) = evshpSR(3)
         evshpMC(3) = evshpSR(5)
         evshpMC(4) = evshpSR(6)

* calculate sums for mean values and their standard deviations
         do iesType = 1, 4
            esSum(iesType, curQbin) = esSum(iesType, curQbin)
     .                              + dble(wtx)*evshpMC(iesType)
            esSqSum(iesType, curQbin) = esSqSum(iesType, curQbin)
     .                              + dble(wtx)*(evshpMC(iesType)**2.)
         enddo


*--------------
* Histo filling
*--------------


* fill histos and increase counters for number of accepted events
         do iesType = 1, 4

* CUT (vii): values of evt.shp.var. in allowed regions (s.a.)
*-----------
           if ((evshpMC(iesType).gt.lolimit(iesType)).and.
     .         (evshpMC(iesType).lt.hilimit(iesType))) then
              call hfill(iesType*10+curQbin,
     .                   Real(evshpMC(iesType)),0.,wtx)
              Nevt(iesType,curQbin) = Nevt(iesType,curQbin) + wtx
           elseif (evshpMC(iesType).le.lolimit(iesType)) then
              locut(iesType,curQbin) = locut(iesType,curQbin) + wtx
           elseif (evshpMC(iesType).ge.hilimit(iesType)) then
              hicut(iesType,curQbin) = hicut(iesType,curQbin) + wtx
           endif
         enddo


 9990    continue



*********************************************************************
*
*                      Termination
*
*********************************************************************

      else if(iflag.eq.3) then

*      Termination: The following MUST always be done
*      (i) Move to the correct PAW subdirectory
*
         call hcdir('//PAWC/'//xxxx,' ')

* write out number of accepted events to histos
         do iesType = 1, 4
            do iQbin = 1, 7
               if (Nevt(iesType,iQbin).gt.0.) then
                  call HzHinrm(iesType*10+iQbin,0,Nevt(iesType,iQbin),1)
               endif
               call HzHinfo(iesType*10+iQbin, Nevt(iesType,iQbin))
            enddo
         enddo

* get mean values and errors of event shape variables
         do iesType = 1, 4
            do iQbin = 1, 7
               if (Nevt(iesType,iQbin).le.0.) then
                  esMean(iesType,iQbin) = 0.
                  esMeanE(iesType,iQbin) = 0.
               else
                  esMean(iesType,iQbin) = Real(esSum(iesType,iQbin))
     .                 / Nevt(iesType,iQbin)
                  if (Nevt(iesType,iQbin).ge.2.) then
                     esMeanE(iesType,iQbin) =
     .                sqrt(  ( Real(esSqSum(iesType,iQbin)) -
     .                         Real(esSum(iesType,iQbin)**2)
     .                         / Nevt(iesType,iQbin) )
     .                  /((Nevt(iesType,iQbin)-1)*Nevt(iesType,iQbin)))
                  else
                     esMeanE(iesType,iQbin) = 0.
                     write(6,*)
     .                 'Hz'//xxxx,' only one event in Q-bin No.:',iQbin
                  endif
               endif
            enddo
         enddo

* histo for mean thrust Tz should contain <Tz/2>
         do iQbin = 1, 7
            esMean(2,iQbin) = esMean(2,iQbin)/2.
            esMeanE(2,iQbin) = esMeanE(2,iQbin)/2.
         enddo

* fill mean histos
* (3rd channel contains zeros)
         do iesType = 1, 4
            do ichn = 1, 2
               esMtmp(ichn) = esMean(iesType, ichn)
               esMEtmp(ichn) = esMeanE(iesType, ichn)
            enddo
            esMtmp(3) = 0.
            esMEtmp(3) = 0.
            do ichn = 4, 8
               esMtmp(ichn) = esMean(iesType, ichn-1)
               esMEtmp(ichn) = esMeanE(iesType, ichn-1)
            enddo
            call hpak(iesType*10, esMtmp)
            call hpake(iesType*10, esMEtmp)
         enddo

* get chi-squares and write them to n-tuple
         do iesType = 1, 4
            call HzChisq(-(iesType*10),iesType*10)
         enddo

         do iesType = 1, 4
            do iQbin = 1, 7
               call HzChisq(-(iesType*10+iQbin),iesType*10+iQbin)
            enddo
         enddo

* write out statistics
         if (lp) then
           write(6,*) 'Hz'//xxxx, 'statistics:'
           do iesType = 1, 4
             write(6,*) '----- for event shape variable', iesType
             do iQbin = 1, 7
                if (Nevt(iesType,iQbin).ne.0.) then
                   temp = (hicut(iesType,iQbin)+locut(iesType,iQbin))/
     .                     Nevt(iesType,iQbin)
                else
                   temp = 0.
                endif
                write(6,*) 'Qbin:',iQbin,'N accepted:',
     .                     Nevt(iesType,iQbin),
     .                     'N hicut:', hicut(iesType,iQbin),
     .                     'N locut:', locut(iesType,iQbin),
     .                     '(N hicut + N locut) / N total :', temp
             enddo
           enddo
         endif
      endif


      return

      end
