      Subroutine HZ98018(iflag)
*****************************************************************************
*     This routine has to be run twice with the following code
c     additions:
*     +1000 for the DIRECT component run.
*     +2000 for the RESOLVED component run.
*     +0 for both in the same run.
*     for all three phases (iflag=1,2,3)
*     
*     +10000 to run on parton showers instead of final state particles.
*     
*     iflag = 1,2 or 3 depending on whether it is the initialization,
*     processing or termination phase (respectively)
*     
*     The default jetfinder is PUCELL
*     To change jetfinder use these values to call the initialisation
c     phase.
*     1 + 10 * (jetfinder number)
*     
*     This routine produces 7 Monte Carlo graphs.
*     id 10 is the cross section differential in nb for ET > 14 GeV
c     134<W<277GeV
*     id 20 is the cross section differential in nb for ET > 17 GeV
c     134<W<277GeV
*     id 30 is the cross section differential in nb for ET > 21 GeV
c     134<W<277GeV
*     id 40 is the cross section differential in nb for ET > 25 GeV
c     134<W<277GeV
*     id 50 is the cross section differential in nb for ET > 14 GeV
c     134<W<190GeV
*     id 60 is the cross section differential in nb for ET > 14 GeV
c     190<W<233GeV
*     id 70 is the cross section differential in nb for ET > 14 GeV
c     233<W<277GeV
*     These graph will be
*     meaningless unless Xsec and Ntot are set before
*     calling the termination routine.
*     (Xsec - total cross section returned by MC)
*     (Ntot - number of events passed to this routine)
*     
*     Cuts :        Q2 < 4 GeV2
*     0.2< y_bj < 0.85
*     
*     
*     Author : Jon Butterworth jmb@hep.ucl.ac.uk
*     JMB Fixed for weighted events, 23/7/99
*     Reference : DESY-98-018
*     The European Physical Journal C4 (1998) 591-606
*     
*     
*****************************************************************************
*     
*     HERA Tuning defined commons

      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
      
      CHARACTER*6 jetf
      CHARACTER*8 xxxx
      DATA xxxx /'d98018'/
      
      INTEGER NJET
c      DOUBLE PRECISION Jets(50,8)
      DOUBLE PRECISION CONER
      SAVE CONER

      DOUBLE PRECISION elecE
      INTEGER myil,myip
      INTEGER loop,count
      DOUBLE PRECISION phi(2),px(2),py(2),pz(2),ptsq(2)
      DOUBLE PRECISION ppsq(2),energy(2),mass(2)
      DOUBLE PRECISION rap,Et
      DOUBLE PRECISION ybj,q2
      REAL lumnb
      
C     --- cuts arrays
      DOUBLE PRECISION ybjcut(4),q2cut(2)
      DOUBLE PRECISION Etcut(4)
      SAVE ybjcut, etcut, q2cut

C     *** Data for ET > 14 GeV, full W range
      INTEGER nbin, j
      PARAMETER(nbin=12)
      REAL eta(nbin),etacr(nbin),cresta(nbin)
      REAL cresys(nbin),band(nbin)
      DATA eta/-0.88,-0.62,-0.38,-0.12, 0.12,0.38,0.62,0.88,1.12,1.38,
     &     1.62,1.88/
      DATA etacr/135,345,690,1040,1330,1535,1790,1785,1715,1690,1655
     &     ,1785/ 
      DATA cresta/15, 25, 35, 40, 45, 45, 50, 50, 50, 50, 50, 50/
      DATA cresys/10,90,50, 120,90, 170,60,80, 110,80, 110, 100/
      
C     *** Data for ET > 17 GeV, full W range
      INTEGER nbin2
      PARAMETER(nbin2=11)
      REAL eta2(nbin2),etacr2(nbin2),cresta2(nbin2),cresys2(nbin2)
      DATA eta2/-0.62,-0.38,-0.12, 0.12,0.38,0.62,0.88,1.12,1.38,1.62,1
     &     .88/
      DATA etacr2/80,185,355,500,625,755,750,725,690,710,665/
      DATA cresta2/10,15,25,25,30,35,35,35,30,35,30/
      DATA cresys2/10,30,30,30,50,40,20,50,40,70,30/
      
      
C     *** Data for ET > 21 GeV, full W range
      INTEGER nbin3
      PARAMETER(nbin3=5)
      REAL eta3(nbin3),etacr3(nbin3),cresta3(nbin3),cresys3(nbin3)
      DATA eta3/-0.25,0.25,0.75,1.25,1.75/
      DATA etacr3/50,205,305,280,235/
      DATA cresta3/5,10,15,15,15/
      DATA cresys3/15,15,5,20,30/
      
C     *** Data for ET > 25 GeV, full W range
      INTEGER nbin4
      PARAMETER(nbin4=4)
      REAL eta4(nbin4),etacr4(nbin4),cresta4(nbin4),cresys4(nbin4)
      DATA eta4/0.25,0.75,1.25,1.75/
      DATA etacr4/70,125,125,120/
      DATA cresta4/5,10,10,10/
      DATA cresys4/5,15,15,10/
      
C     *** Data for ET > 14 GeV, low W range
      INTEGER nbin5
      PARAMETER(nbin5=4)
      REAL eta5(nbin5),etacr5(nbin5),cresta5(nbin5),cresys5(nbin5)
      DATA eta5/0.25,0.75,1.25,1.75/
      DATA etacr5/340,720,700,725/
      DATA cresta5/15,25,25,25/
      DATA cresys5/70,60,50,50/
      
C     *** Data for ET > 14 GeV, middle W range
      INTEGER nbin6
      PARAMETER(nbin6=5)
      REAL eta6(nbin6),etacr6(nbin6),cresta6(nbin6),cresys6(nbin6)
      DATA eta6/-0.25,0.25,0.75,1.25,1.75/
      DATA etacr6/305,585,555,495,500/
      DATA cresta6/15,20,20,20,20/
      DATA cresys6/60,30,50,30,40/
      
C     *** Data for ET > 14 GeV, high W range
      INTEGER nbin7
      PARAMETER(nbin7=6)
      REAL eta7(nbin7),etacr7(nbin7),cresta7(nbin7),cresys7(nbin7)
      DATA eta7/-0.75,-0.25,0.25,0.75,1.25,1.75/
      DATA etacr7/220,535,500,495,475,475/
      DATA cresta7/15,20,20,20,20,20/
      DATA cresys7/30,30,40,30,30,20/
      
      INTEGER chjet, iproc, iflag
      SAVE chjet
C     --- choose same jet finder as in the paper --- PXCONE
      DATA chjet/7/
      INTEGER hzflag
      
      REAL pttemp(nbin),pttemp2(nbin2),pttemp3(nbin3),pttemp4(nbin4)
     &     ,pttemp5(nbin5),pttemp6(nbin6),pttemp7(nbin7)
      INTEGER ivec,jvec
      REAL sqrt
      
      LOGICAL parton_showers

      CALL hzjetrad(2,coner)
******  if not set it to the value used in the paper.
      IF (coner.LT.0.0) THEN
	      coner=1.0
      ENDIF

****** Determine the process type (single/double resolved etc)
****** (only necessary if > 1 process types!)
      iproc=(mod(iflag,10000)/1000)
      IF ((iproc.LT.0).OR.(iproc.GT.2)) THEN
         WRITE(HZUNIT,*) xxxx,': illegal IPROC:',iproc
         RETURN
      ENDIF

****** choose jet finder (only necessary if using jets!)
      IF ((MOD(iflag,100)/10.GT.0).AND.MOD(iflag,10).eq.1) THEN
         chjet=INT(MOD(iflag,100)/10)
      ENDIF

******Decide if we are running on parton showers or final state
******particles.
      IF (MOD(iflag,100000)/10000.GT.0) THEN
****** use a different paw directory for parton showers.
         xxxx = 'PS'//xxxx
****** tell jet finder that we running on parton showers 
****** (only necessary if using jets!)
         chjet=-1*ABS(chjet)
      ENDIF
******************************************************************************
********************************Initialise ********************************* 
      IF (MOD(iflag,10).EQ.1) THEN
C     *** Initialisation run ***
C     *** Create directories ***
         IF (iproc.le.1) THEN
            CALL HCDIR('//HISTO',' ')
            CALL HMDIR(xxxx,'S')
            CALL HCDIR('//PAWC',' ')
            CALL HMDIR(xxxx,'S')
            
C     --- create proper cross-section data
            CALL HBOOK1(-10,'Data cross section (nb) E?T!"g# 14 GeV'
     &           ,nbin ,-1.0 ,2.0 ,0.)

            CALL HBOOK1(-20,'Data cross section (nb) E?T!"g# 17 GeV'
     &           ,nbin2 ,-0.75 ,2.0 ,0.)

            CALL HBOOK1(-30,'Data cross section (nb) E?T!"g# 21 GeV'
     &           ,nbin3 ,-0.5 ,2.0 ,0.)
            
            CALL HBOOK1(-40,'Data cross section (nb) E?T!"g# 25 GeV'
     &           ,nbin4 ,0.0 ,2.0 ,0.)
            
            CALL HBOOK1(-50,'Data cross section (nb) 134"l#W"l190 GeV'
     &           ,nbin5 ,0.0 ,2.0 ,0.)
            
            CALL HBOOK1(-60,'Data cross section (nb) 190"l#W"233 GeV'
     &           ,nbin6 ,-0.5 ,2.0 ,0.)
            
            CALL HBOOK1(-70,'Data cross section (nb) 233"l#W"l277 GeV'
     &           ,nbin7 ,-1.0 ,2.0 ,0.)

            CALL HPAK(-10,etacr)
            CALL HPAK(-20,etacr2)
            CALL HPAK(-30,etacr3)
            CALL HPAK(-40,etacr4)
            CALL HPAK(-50,etacr5)
            CALL HPAK(-60,etacr6)
            CALL HPAK(-70,etacr7)

            DO ivec=1,nbin
               pttemp(ivec)=sqrt(cresta(ivec)**2+cresys(ivec)**2)
            ENDDO
            CALL HPAKE(-10,pttemp)

            DO ivec=1,nbin2
               pttemp2(ivec)=sqrt(cresta2(ivec)**2+cresys2(ivec)**2)
            ENDDO
            CALL HPAKE(-20,pttemp2)

            DO ivec=1,nbin3
               pttemp3(ivec)=sqrt(cresta3(ivec)**2+cresys3(ivec)**2)
            ENDDO
            CALL HPAKE(-30,pttemp3)

            DO ivec=1,nbin4
               pttemp4(ivec)=sqrt(cresta4(ivec)**2+cresys4(ivec)**2)
            ENDDO
            CALL HPAKE(-40,pttemp4)

            DO ivec=1,nbin5
               pttemp5(ivec)=sqrt(cresta5(ivec)**2+cresys5(ivec)**2)
            ENDDO
            CALL HPAKE(-50,pttemp5)

            DO ivec=1,nbin6
               pttemp6(ivec)=sqrt(cresta6(ivec)**2+cresys6(ivec)**2)
            ENDDO
            CALL HPAKE(-60,pttemp6)

            DO ivec=1,nbin7
               pttemp7(ivec)=sqrt(cresta7(ivec)**2+cresys7(ivec)**2)
            ENDDO
            CALL HPAKE(-70,pttemp7)

         ENDIF
         CALL HCDIR('//HISTO/'//xxxx,' ')
         CALL HCDIR('//PAWC/'//xxxx,' ')
         
         CALL HBOOK1(100*iproc+10, 
     $'MC cross section (nb) E?T !"g# 14 GeV' ,nbin ,-1.0 ,2.0 
     $,0.)
            
         CALL HBOOK1(100*iproc+20,
     $'MC cross section (nb) E?T !"g# 17 GeV' ,nbin2 ,-0.75 ,2.0
     $           ,0.)
            
         CALL HBOOK1(100*iproc+30,
     $          'MC cross section (nb) E?T !"g# 21 GeV' 
     $           ,nbin3 ,-0.5 ,2.0 ,0.)
            
         CALL HBOOK1(100*iproc+40,
     $   'MC cross section (nb) E?T !"g# 25 GeV'
     $        ,nbin4 ,-0.0 ,2.0 ,0.)
            
         CALL HBOOK1(100*iproc+50,
     $          'MC cross section (nb) 134"l#W"l190 GeV'
     &           ,nbin5 ,0.0 ,2.0 ,0.)
            
         CALL HBOOK1(100*iproc+60,
     $        'MC cross section (nb) 190"l#W"233 GeV'
     &          ,nbin6 ,-0.5 ,2.0 ,0.)
            
         CALL HBOOK1(100*iproc+70,
     $            'MC cross section (nb) 233"l#W"l277 GeV'
     &           ,nbin7 ,-1.0 ,2.0 ,0.)

         CALL HIDOPT(0,'stat')
         CALL HBARX(0)

C     --- photoproduction cuts
         
         ybjcut(1)=0.2
         ybjcut(2)=0.4
         ybjcut(3)=0.6
         ybjcut(4)=0.85
         q2cut(1)=0.
         q2cut(2)=4.
         
         Etcut(1)=14.
         Etcut(2)=17.
         Etcut(3)=21.
         Etcut(4)=25.
         
C     --- standard cone radius 1.0
         CALL HZJETRAD(2,CONER)
         IF (CONER.lt.0.0) THEN
            CONER=1.0
         ENDIF
         CALL HZJTNAME(chjet,jetf)
         
         WRITE(6,*)'**********************************************'
         WRITE(6,*)'* HZ98018 called, histograms will be output. *'
         WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
         WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
         WRITE(6,*)'**********************************************'
         
 1001    FORMAT(A29,F5.2,A13)
         
      ELSE IF (MOD(iflag,10).eq.2) THEN
C     *** Filling runs ***
         
C     *** Change directory to our graphs ***
         CALL HCDIR('//PAWC/'//xxxx,' ')

         IF (gen(1:3).EQ.'HO1') THEN

            DO j=10,70,10
               CALL HZSNLO(j,j+200,j+100)
            ENDDO
            iproc=1

         ENDIF
         
         q2=hzphokin(1)
         IF (q2.eq.-1) THEN
            PRINT*,'HZ98018: HZPHOKIN returns error for Q2'
         ELSEIF (q2.lt.0) THEN
            q2=abs(q2)
         ENDIF
         ybj=hzphokin(3)
         IF (ybj.eq.-1) THEN
            PRINT*,'HZ98018: HZPHOKIN returns error for ybj'
         ENDIF
         
C     *** Fill histograms for full range ***
         
         IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(4))
     &        .and.(q2.lt.q2cut(2))) Then
            
            elecE=27.52
            hzflag=HZIBEAM(myip,myil)
            IF (hzflag.ge.0) Then
               elecE=abs(PHEP(4,myil))
            ELSE
               PRINT*,'HZ98018: Electron not found. E=0.001'
               PRINT*,'         event will be rejected'
               elecE=0.001
            ENDIF
            
            CALL hzjtfind(chjet,CONER,NJET,Jets)
            
            DO loop=1,NJET
               Et=Jets(loop,3)
               rap=Jets(loop,1)
               
               IF (Et.gt.Etcut(1)) THEN
                  CALL HFILL(INT(iproc*100+10), REAL(rap),0.,wtx)
                  IF (ybj.lt.ybjcut(2)) THEN
                     CALL HFILL(INT(iproc*100+50), REAL(rap),0.,wtx)
                  ELSEIF (ybj.lt.ybjcut(3)) THEN
                     CALL HFILL(INT(iproc*100+60), REAL(rap),0.,wtx)
                  ELSE
                     CALL HFILL(INT(iproc*100+70), REAL(rap),0.,wtx)
                  ENDIF
               ENDIF
               IF (Et.gt.Etcut(2)) THEN
                  CALL HFILL(INT(iproc*100+20), REAL(rap), 0. ,wtx)
               ENDIF
               IF (Et.gt.Etcut(3)) THEN
                  CALL HFILL(INT(iproc*100+30), REAL(rap), 0. ,wtx)
               ENDIF
               IF (Et.gt.Etcut(4)) THEN
                  CALL HFILL(INT(iproc*100+40), REAL(rap), 0. ,wtx)
               ENDIF
               
            ENDDO
            
         ENDIF                  ! Q2 & y cuts
         
      ELSE IF (MOD(iflag,10).eq.3) THEN

         IF (iproc.eq.0.or.iproc.eq.2) THEN
            write(hzunit,*) 'HZ98018: Finishing off'
         ENDIF

C     *** Termination run ***
C     *** Change directory ***
         CALL HCDIR('//PAWC/'//xxxx,' ')
C     *** Finish off histograms ***

         IF (Xsec.eq.0) THEN
            PRINT*,'HZ98018: termination called with zero cross section'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',iproc*1000
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,'HZ98018: termination called with no total events'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',iproc*1000
            Ntot=1
         ENDIF

         lumnb = REAL(ntot/(1000.*xsec))
   
         IF (gen(1:3).NE.'HO1') THEN
C     --- normalise resolved/direct component histograms
            DO loop=10,70,10
               CALL HZHINRM(100*iproc+loop,0,lumnb,1)
            ENDDO

            IF (iproc.eq.2) THEN
               DO loop=10,70,10
                  CALL HOPERA(100+loop,'+e',200+loop,loop,1.,1.)
               ENDDO
            ENDIF

         ELSE

            IF (iproc.eq.2) THEN
               DO j=10,70,10
                  CALL HZSNLO(j,j+200,j+100)
               ENDDO
               CALL HOPERA(10,'+e',10,10,2000.,0.)
               CALL HOPERA(20,'+e',20,20,2000.,0.)
               CALL HOPERA(30,'+e',30,30,2000.,0.)
               CALL HOPERA(40,'+e',40,40,2000.,0.)
               CALL HOPERA(50,'+e',50,50,2000.,0.)
               CALL HOPERA(60,'+e',60,60,2000.,0.)
               CALL HOPERA(70,'+e',70,70,2000.,0.)

            ENDIF
         ENDIF

      ELSE
C     *** End ***
         PRINT*
     &        ,'HZ98018:Please run routine with iflag set to 1,2 or 3'
         PRINT*,'    with +1000 or +2000 for DIR or RES events.'
      ENDIF

      RETURN
      END

