*CMZ :  1.01/15 17/06/98  15.03.55  by  Pierre Van Mechelen
*-- Author :    Pierre Van Mechelen   06/05/98
c-----------------------------------------------------------------------

      SUBROUTINE HZ98044 (iflag)

c-----------------------------------------------------------------------
c---- Description

c Purpose: Study multiplicity structure of the hadronic final state in
c          diffractive deep inelastic scattering
c
c Event selection: 7.5 < Q2 < 100 GeV2
c                  0.05 < y < 0.6
c                  xpom < 0.05
c                  3 < Mx < 36 GeV
c                  |t| < 1 GeV2
c                  My < 1.6 GeV
c
c Reference: DESY 98-044
c            Eur. Phys. J. C (1998)
c
c Arguments: iflag=1 initialisation
c            iflag=1 filling
c            iflag=3 termination
c
c Written by: Pierre Van Mechelen on 06/05/98

c-----------------------------------------------------------------------
c---- Declarations

      IMPLICIT NONE

c.... HzTool stuff
#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"

c.... input variable
      INTEGER iflag

c.... local variables
      LOGICAL          iacc(NMXHEP)
      INTEGER          nacc, ihep, i, j, k, ierr, ibp, ibl, isl,
     &                 id, nt, nf, nb, np, nn,
     &                 index(NMXHEP)
      REAL             p, pz, gap, tmp, w2, mp, nev(7),
     &                 xvec(4), yvec(4), rap(NMXHEP),
     &                 mt(7), mf(7), mb(7), dt(7), df(7), db(7),
     &                 rt(7), rf(7), rb(7), cd(7), fb(7)
      PARAMETER        (mp = 0.938)
      DOUBLE PRECISION pbeam(4), gamma(4), p_lab(4), p_hcms(4)
      CHARACTER*4      low, high

c.... Mx bin limits
      REAL mxbins1(8), mxbins2(4)
      DATA mxbins1 / 4., 6., 8., 11., 15., 19., 24., 36./
      DATA mxbins2 / 3., 8., 15., 30. /

c.... kinematical selection criteria
      REAL q2, q2_low, q2_high
      DATA q2_low  / 7.5  /, q2_high / 100. /
      REAL y,  y_low,  y_high
      DATA y_low   / 0.05 /, y_high  / 0.6  /
      REAL xp, xp_high
      DATA xp_high / 0.05 /
      REAL mx, mx_low, mx_high
      DATA mx_low  / 3.   /, mx_high / 36.  /
      REAL t,  t_high
      DATA t_high  / 1.   /
      REAL my, my_high
      DATA my_high / 1.6  /

c-----------------------------------------------------------------------
c---- Corrected data from H1

      REAL pn(20,3,4), pne(20,3,4), pns(20,3,4)
      REAL pn_tmp(20), pne_tmp(20), pns_tmp(20)

      DATA pn    /
*     forward 1
     &              5.87, 20.96, 28.85, 23.59, 12.86,
     &              5.32,  1.89,  0.48,  0.13,  0.009,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     forward 2
     &              1.60,  8.17, 14.55, 23.24, 19.93,
     &             14.45,  9.28,  4.69,  2.55,  0.90,
     &              0.46,  0.08,  0.024,  0.022,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     forward 3
     &              0.42,  3.30,  6.85, 12.82, 16.29,
     &             17.61, 14.97, 10.85,  7.41,  4.50,
     &              2.48,  1.21,  0.73,  0.34,  0.07,
     &              0.03,  0.00,  0.00,  0.00,  0.00,
*     backward 1
     &              6.38, 21.63, 29.13, 23.63, 12.87,
     &              4.52,  1.29,  0.40,  0.10,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     backward 2
     &              1.17,  8.12, 16.17, 22.27, 20.85,
     &             14.79,  8.93,  4.29,  2.28,  0.64,
     &              0.33,  0.09,  0.021,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     backward 3
     &              0.59,  3.30,  7.55, 13.51, 15.64,
     &             15.80, 13.91, 11.25,  8.26,  4.71,
     &              2.96,  1.13,  0.67,  0.42,  0.16,
     &              0.04,  0.00,  0.00,  0.00,  0.00,
*     positives 1
     &              1.00, 17.14, 38.88, 29.53, 10.74,
     &              2.46,  0.21,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     positives 2
     &              0.03,  2.49, 14.04, 27.54, 27.91,
     &             16.90,  7.40,  2.66,  0.85,  0.09,
     &              0.031,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     positives 3
     &              0.005,  0.71,  4.29, 12.16, 19.09,
     &             21.24, 17.25, 12.61,  7.19,  3.26,
     &              1.47,  0.42,  0.20,  0.019,  0.013,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     negatives 1
     &              0.93, 15.92, 39.40, 29.76, 11.29,
     &              2.51,  0.16,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     negatives 2
     &              0.03,  2.43, 14.34, 27.82, 27.63,
     &             16.65,  7.33,  2.64,  0.95,  0.09,
     &              0.026,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     negatives 3
     &              0.005,  0.53,  3.69, 12.40, 19.77,
     &             22.44, 17.64, 11.78,  6.56,  3.16,
     &              1.42,  0.35,  0.15,  0.028,  0.023,
     &              0.00,  0.00,  0.00,  0.00,  0.00 /

      DATA pne   /
*     forward 1
     &              0.48,  1.08,  1.02,  0.97,  0.72,
     &              0.36,  0.18,  0.07,  0.03,  0.007,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     forward 2
     &              0.13,  0.58,  0.76,  1.03,  0.82,
     &              0.66,  0.48,  0.26,  0.21,  0.10,
     &              0.09,  0.01,  0.011,  0.010,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     forward 3
     &              0.04,  0.39,  0.54,  0.76,  0.81,
     &              0.74,  0.67,  0.52,  0.38,  0.25,
     &              0.16,  0.13,  0.11,  0.08,  0.02,
     &              0.01,  0.00,  0.00,  0.00,  0.00,
*     backward 1
     &              0.53,  1.07,  1.07,  0.88,  0.62,
     &              0.30,  0.15,  0.11,  0.04,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     backward 2
     &              0.08,  0.48,  0.65,  0.73,  0.76,
     &              0.63,  0.47,  0.28,  0.24,  0.08,
     &              0.08,  0.06,  0.005,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     backward 3
     &              0.04,  0.23,  0.35,  0.50,  0.54,
     &              0.53,  0.51,  0.45,  0.38,  0.25,
     &              0.23,  0.08,  0.10,  0.14,  0.07,
     &              0.04,  0.00,  0.00,  0.00,  0.00,
*     positives 1
     &              0.10,  0.94,  1.45,  1.25,  0.67,
     &              0.26,  0.04,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     positives 2
     &              0.01,  0.22,  0.64,  0.99,  0.89,
     &              0.71,  0.38,  0.19,  0.10,  0.02,
     &              0.016,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     positives 3
     &              0.001,  0.08,  0.37,  0.63,  0.75,
     &              0.83,  0.69,  0.59,  0.38,  0.23,
     &              0.16,  0.06,  0.07,  0.010,  0.011,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     negatives 1
     &              0.09,  1.09,  1.72,  1.11,  0.66,
     &              0.29,  0.03,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     negatives 2
     &              0.01,  0.25,  0.79,  1.06,  0.89,
     &              0.65,  0.39,  0.21,  0.13,  0.02,
     &              0.013,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     negatives 3
     &              0.001,  0.08,  0.35,  0.72,  0.78,
     &              0.82,  0.69,  0.46,  0.36,  0.23,
     &              0.16,  0.05,  0.03,  0.009,  0.026,
     &              0.00,  0.00,  0.00,  0.00,  0.00 /

      DATA pns   /
*     forward 1
     &              1.42,  4.24,  5.44,  4.11,  2.33,
     &              1.47,  0.83,  0.36,  0.17,  0.102,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     forward 2
     &              0.70,  2.41,  3.19,  4.20,  3.42,
     &              2.48,  2.00,  1.38,  0.80,  0.72,
     &              0.31,  0.31,  0.003,  0.002,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     forward 3
     &              0.24,  1.16,  2.09,  3.22,  3.94,
     &              4.20,  2.84,  2.10,  1.30,  1.05,
     &              0.64,  0.53,  0.33,  0.19,  0.14,
     &              0.08,  0.00,  0.00,  0.00,  0.00,
*     backward 1
     &              2.42,  4.63,  5.74,  4.01,  1.96,
     &              1.32,  0.62,  0.22,  0.05,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     backward 2
     &              0.36,  2.05,  4.02,  5.04,  4.20,
     &              2.87,  1.68,  1.31,  0.91,  0.66,
     &              0.32,  0.13,  0.006,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     backward 3
     &              0.19,  1.01,  2.37,  4.36,  3.67,
     &              3.56,  2.70,  2.28,  1.52,  1.78,
     &              1.07,  1.05,  0.58,  0.26,  0.12,
     &              0.10,  0.00,  0.00,  0.00,  0.00,
*     positives 1
     &              0.35,  4.34,  7.45,  4.85,  2.56,
     &              1.13,  0.43,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     positives 2
     &              0.05,  0.91,  3.52,  5.79,  5.35,
     &              3.09,  2.12,  1.35,  0.57,  0.28,
     &              0.008,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     positives 3
     &              0.001,  0.33,  1.63,  3.88,  5.08,
     &              4.76,  3.73,  2.21,  2.18,  1.33,
     &              0.79,  0.54,  0.15,  0.001,  0.001,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     negatives 1
     &              0.40,  4.01,  7.37,  4.97,  2.38,
     &              1.21,  0.46,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     negatives 2
     &              0.05,  1.01,  3.69,  5.72,  5.26,
     &              2.99,  1.95,  1.31,  0.52,  0.34,
     &              0.007,  0.00,  0.00,  0.00,  0.00,
     &              0.00,  0.00,  0.00,  0.00,  0.00,
*     negatives 3
     &              0.001,  0.19,  1.20,  3.83,  4.92,
     &              4.86,  3.45,  2.30,  1.89,  1.41,
     &              0.64,  0.61,  0.10,  0.001,  0.001,
     &              0.00,  0.00,  0.00,  0.00,  0.00 /

      REAL m (7,3),   d (7,3),   r (7,3)
      REAL me(7,3),   de(7,3),   re(7,3)
      REAL ms(7,3),   ds(7,3),   rs(7,3)
      REAL m_tmp(7),  d_tmp(7),  r_tmp(7)
      REAL me_tmp(7), de_tmp(7), re_tmp(7)
      REAL ms_tmp(7), ds_tmp(7), rs_tmp(7)

      DATA m    / 4.66, 5.72, 6.87, 8.18, 9.49,10.85,12.32,
     &            2.32, 3.00, 3.52, 4.08, 4.71, 5.36, 6.28,
     &            2.29, 2.74, 3.33, 4.13, 4.70, 5.53, 6.06 /
      DATA me   / 0.08, 0.09, 0.08, 0.11, 0.16, 0.16, 0.18,
     &            0.05, 0.07, 0.06, 0.06, 0.10, 0.11, 0.11,
     &            0.05, 0.05, 0.05, 0.07, 0.09, 0.11, 0.12 /
      DATA ms   / 0.23, 0.21, 0.28, 0.39, 0.44, 0.54, 0.72,
     &            0.12, 0.13, 0.15, 0.19, 0.20, 0.23, 0.32,
     &            0.13, 0.11, 0.18, 0.25, 0.40, 0.42, 0.47 /

      DATA d    / 1.86, 2.03, 2.43, 2.88, 3.21, 3.47, 4.28,
     &            1.29, 1.51, 1.76, 1.95, 2.12, 2.22, 2.76,
     &            1.31, 1.40, 1.60, 1.91, 2.16, 2.39, 2.81 /
      DATA de   / 0.05, 0.05, 0.05, 0.05, 0.08, 0.11, 0.12,
     &            0.03, 0.03, 0.04, 0.04, 0.05, 0.06, 0.06,
     &            0.03, 0.03, 0.02, 0.03, 0.04, 0.04, 0.08 /
      DATA ds   / 0.14, 0.15, 0.16, 0.19, 0.25, 0.25, 0.27,
     &            0.14, 0.20, 0.20, 0.17, 0.12, 0.13, 0.17,
     &            0.18, 0.08, 0.16, 0.23, 0.25, 0.16, 0.32 /

      DATA r    / 0.944, 0.951, 0.979, 1.001, 1.009, 1.010, 1.039,
     &            0.877, 0.919, 0.966, 0.984, 0.991, 0.985, 1.034,
     &            0.894, 0.897, 0.930, 0.972, 0.998, 1.006, 1.049 /
      DATA re   / 0.009, 0.006, 0.005, 0.004, 0.006, 0.006, 0.006,
     &            0.014, 0.011, 0.011, 0.009, 0.010, 0.010, 0.009,
     &            0.016, 0.013, 0.007, 0.007, 0.008, 0.008, 0.010 /
      DATA rs   / 0.022, 0.017, 0.013, 0.011, 0.014, 0.012, 0.016,
     &            0.058, 0.031, 0.028, 0.036, 0.019, 0.018, 0.022,
     &            0.038, 0.030, 0.041, 0.027, 0.031, 0.015, 0.020 /

      REAL dcd(7), dcde(7), dcds(7)

      DATA dcd  / 1.94, 1.90, 2.15, 2.23, 2.30, 2.56, 2.50 /
      DATA dcde / 0.07, 0.05, 0.07, 0.06, 0.08, 0.11, 0.10 /
      DATA dcds / 0.12, 0.10, 0.10, 0.17, 0.16, 0.19, 0.17 /

      REAL drho(3), drhoe(3), drhos(3)

      DATA drho  / 0.081, 0.108, 0.194 /
      DATA drhoe / 0.032, 0.030, 0.033 /
      DATA drhos / 0.035, 0.041, 0.023 /

      REAL drap(12,3),   drape(12,3),   draps(12,3)
      REAL drap_tmp(12), drape_tmp(12), draps_tmp(12)

      DATA drap   / 0.03, 0.14, 0.41, 0.91, 1.32, 1.84,
     &              1.92, 1.58, 0.84, 0.37, 0.10, 0.04,
*
     &              0.17, 0.48, 1.08, 1.53, 1.91, 2.17,
     &              2.21, 2.01, 1.64, 1.02, 0.51, 0.14,
*
     &              0.58, 1.09, 1.38, 2.09, 2.28, 2.43,
     &              2.46, 2.33, 2.06, 1.82, 1.06, 0.50 /

      DATA drape  / 0.01, 0.02, 0.02, 0.04, 0.05, 0.06,
     &              0.06, 0.05, 0.03, 0.02, 0.01, 0.01,
*
     &              0.07, 0.05, 0.07, 0.06, 0.06, 0.06,
     &              0.07, 0.05, 0.05, 0.04, 0.03, 0.01,
*
     &              0.69, 0.16, 0.10, 0.07, 0.07, 0.08,
     &              0.07, 0.07, 0.06, 0.06, 0.04, 0.03 /

      DATA draps  / 0.02, 0.02, 0.04, 0.06, 0.07, 0.11,
     &              0.11, 0.11, 0.04, 0.03, 0.03, 0.01,
*
     &              0.16, 0.12, 0.18, 0.12, 0.12, 0.11,
     &              0.15, 0.11, 0.11, 0.07, 0.05, 0.04,
*
     &              0.90, 0.63, 0.29, 0.24, 0.14, 0.15,
     &              0.18, 0.14, 0.12, 0.10, 0.09, 0.06 /

      REAL mx1(7), mx2(7), mx3(3), mx4(3)

      DATA mx1 /  3190.0,  3710.0,  7001.5, 10114.0,
     &            9176.7,  9968.4,  16354.2 /
      DATA mx2 /   638.0,   530.0,    737.0,  778.0,
     &             543.0,   468.0,    562.0 /
      DATA mx3 /  8056.8, 17271.0, 28674.9 /
      DATA mx4 /  1492.0,  1515.0,  1359.0 /

      save nacc


c-----------------------------------------------------------------------
c---- Initialisation

      IF (iflag .EQ. 1) THEN

c....   set number of accepted events to zero
        nacc= 0

c....   create subdirectories for use with HBOOK/PAW
        CALL HCDIR ('//PAWC', ' ')
        CALL HMDIR ('98044', 'S')
        CALL HCDIR ('//HISTO', ' ')
        CALL HMDIR ('98044', 'S')

c....   book moca histograms

        DO i = 1, 3
          WRITE (low, '(F4.0)') mxbins2(i)
          WRITE (high,'(F4.0)') mxbins2(i+1)
          id = i*10
          CALL HBOOK1 (id,
     &      'Moca MD - '//low//' .lt. Mx .lt. '//high//' GeV, total',
     &      40, -0.5, 39.5, 0.)
          CALL HBOOK1 (id+1,
     &      'Moca MD - '//low//' .lt. Mx .lt. '//high//' GeV, forward',
     &      40, -0.5, 39.5, 0.)
          CALL HBOOK1 (id+2,
     &      'Moca MD - '//low//' .lt. Mx .lt. '//high//' GeV, backward',
     &      40, -0.5, 39.5, 0.)
          CALL HBOOK1 (id+3,
     &     'Moca MD - '//low//' .lt. Mx .lt. '//high//' GeV, positives',
     &      40, -0.5, 39.5, 0.)
          CALL HBOOK1 (id+4,
     &     'Moca MD - '//low//' .lt. Mx .lt. '//high//' GeV, negatives',
     &      40, -0.5, 39.5, 0.)
          CALL HBOOK1 (id+5,
     &  'Moca rapidity spectrum, '//low//' .lt. Mx .lt. '//high//' GeV',
     &      12, -3.0, +3.0, 0.)
        ENDDO

        DO i = 1, 7
          WRITE (low, '(F4.0)') mxbins1(i)
          WRITE (high,'(F4.0)') mxbins1(i+1)
          id = 100+i*10
          CALL HBOOK1 (id,
     &      'Moca MD - '//low//' .lt. Mx .lt. '//high//', total',
     &      40, -0.5, 39.5, 0.)
          CALL HBOOK1 (id+1,
     &      'Moca MD - '//low//' .lt. Mx .lt. '//high//', forward',
     &      40, -0.5, 39.5, 0.)
          CALL HBOOK1 (id+2,
     &      'Moca MD - '//low//' .lt. Mx .lt. '//high//', backward',
     &      40, -0.5, 39.5, 0.)
          CALL HBOOK1 (id+5,
     &  'Moca rapidity spectrum, '//low//' .lt. Mx .lt. '//high//' GeV',
     &      12, -3.0, +3.0, 0.)
        ENDDO

        CALL HBOOKB (40, 'Moca <n> vs. W, full',
     &               7, mxbins1, 0.)
        CALL HBOOKB (41, 'Moca <n> vs. W, forward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (42, 'Moca <n> vs. W, backward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (43, 'Moca dn/dy (y=0) vs. W',
     &               7, mxbins1, 0.)
        CALL HBOOKB (50, 'Moca D vs. W, full',
     &               7, mxbins1, 0.)
        CALL HBOOKB (51, 'Moca D vs. W, forward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (52, 'Moca D vs. W, backward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (60, 'Moca R2 vs. W, full',
     &               7, mxbins1, 0.)
        CALL HBOOKB (61, 'Moca R2 vs. W, forward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (62, 'Moca R2 vs. W, backward',
     &               7, mxbins1, 0.)

        CALL HBOOKB (70, 'Moca rho vs. W', 7, mxbins1, 0.)

        CALL HBOOKB (90, 'Auxiliary histogram for Mx binning',
     &               7, mxbins1, 0.)
        CALL HBOOKB (91, 'Auxiliary histogram for Mx binning',
     &               7, mxbins1, 0.)
        CALL HBOOKB (92, 'Auxiliary histogram for Mx binning',
     &               3, mxbins2, 0.)
        CALL HBOOKB (93, 'Auxiliary histogram for Mx binning',
     &               3, mxbins2, 0.)

c....   book data histograms (stat errors only)

        DO i = 1, 3
          WRITE (low, '(F4.0)') mxbins2(i)
          WRITE (high,'(F4.0)') mxbins2(i+1)
          id = -i*10
          CALL HBOOK1 (id-1,
     &      'Data MD - '//low//' .lt. Mx .lt. '//high//', forward',
     &               20, -0.5, 19.5, 0.)
          CALL HBOOK1 (id-2,
     &      'Data MD - '//low//' .lt. Mx .lt. '//high//', backward',
     &               20, -0.5, 19.5, 0.)
          CALL HBOOK1 (id-3,
     &      'Data MD - '//low//' .lt. Mx .lt. '//high//', positives',
     &               20, -0.5, 19.5, 0.)
          CALL HBOOK1 (id-4,
     &      'Data MD - '//low//' .lt. Mx .lt. '//high//', negatives',
     &               20, -0.5, 19.5, 0.)
          CALL HBOOK1 (id-5,
     &  'Data rapidity spectrum, '//low//' .lt. Mx .lt. '//high//' GeV',
     &               12, -3.0, +3.0, 0.)
        ENDDO

        CALL HBOOKB (-40, 'Data <n> vs. W, full',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-41, 'Data <n> vs. W, forward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-42, 'Data <n> vs. W, backward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-43, 'Data dn/dy (y=0) vs. W',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-50, 'Data D vs. W, full',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-51, 'Data D vs. W, forward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-52, 'Data D vs. W, backward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-60, 'Data R2 vs. W, full',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-61, 'Data R2 vs. W, forward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-62, 'Data R2 vs. W, backward',
     &               7, mxbins1, 0.)

        CALL HBOOKB (-70, 'Data rho vs. W', 3, mxbins2, 0.)

        CALL HBOOKB (-90, 'Auxiliary histogram for Mx binning',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-91, 'Auxiliary histogram for Mx binning',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-92, 'Auxiliary histogram for Mx binning',
     &               3, mxbins2, 0.)
        CALL HBOOKB (-93, 'Auxiliary histogram for Mx binning',
     &               3, mxbins2, 0.)

c....   book data histograms (stat+sys errors)

        DO i = 1, 3
          WRITE (low, '(F4.0)') mxbins2(i)
          WRITE (high,'(F4.0)') mxbins2(i+1)
          id = -100-i*10
          CALL HBOOK1 (id-1,
     &      'Data MD - '//low//' .lt. Mx .lt. '//high//', forward',
     &               20, -0.5, 19.5, 0.)
          CALL HBOOK1 (id-2,
     &      'Data MD - '//low//' .lt. Mx .lt. '//high//', backward',
     &               20, -0.5, 19.5, 0.)
          CALL HBOOK1 (id-3,
     &      'Data MD - '//low//' .lt. Mx .lt. '//high//', positives',
     &               20, -0.5, 19.5, 0.)
          CALL HBOOK1 (id-4,
     &      'Data MD - '//low//' .lt. Mx .lt. '//high//', negatives',
     &               20, -0.5, 19.5, 0.)
          CALL HBOOK1 (id-5,
     &  'Data rapidity spectrum, '//low//' .lt. Mx .lt. '//high//' GeV',
     &               12, -3.0, +3.0, 0.)
        ENDDO

        CALL HBOOKB (-140, 'Data <n> vs. W, full',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-141, 'Data <n> vs. W, forward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-142, 'Data <n> vs. W, backward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-143, 'Data dn/dy (y=0) vs. W',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-150, 'Data D vs. W, full',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-151, 'Data D vs. W, forward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-152, 'Data D vs. W, backward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-160, 'Data R2 vs. W, full',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-161, 'Data R2 vs. W, forward',
     &               7, mxbins1, 0.)
        CALL HBOOKB (-162, 'Data R2 vs. W, backward',
     &               7, mxbins1, 0.)

        CALL HBOOKB (-170, 'Data rho vs. W', 3, mxbins2, 0.)

c-----------------------------------------------------------------------
c---- Event analysis

      ELSEIF (iflag .EQ. 2) THEN

c....   move to the correct subdirectory in PAWC
        CALL HCDIR ('//PAWC/98044', ' ')

c....   get vectors for boost to hCMS
        ierr = HZIBEAM (ibp, ibl)
        isl = HZIDELEC(id)
        IF (ierr .EQ. 1 .AND. isl .GT. 0) THEN
          DO i = 1, 4
            pbeam(i) = PHEP (i, ibp)
            gamma(i) = PHEP (i, ibl) - PHEP (i, isl)
          ENDDO
        ELSEIF (ierr .NE. 1 .AND. isl .GT. 0) THEN
          PRINT *, 'HZ98044 -- Warning: No beam(s) found,'
          PRINT *, '        -- proton beam energy set to 820 GeV,'
          PRINT *, '        -- positron beam energy set to 27.55 GeV.'
          IF (ierr .EQ.  0) ibp = 0
          IF (ierr .EQ. -1) ibl = 0
          pbeam(1) = 0.
          pbeam(2) = 0.
          pbeam(3) = +820.
          pbeam(4) = +820.
          gamma(1) = - PHEP (1, isl)
          gamma(2) = - PHEP (2, isl)
          gamma(3) = -27.55 - PHEP (3, isl)
          gamma(4) = +27.55 - PHEP (4, isl)
        ELSEIF (isl .LE. 0) THEN
          PRINT *, 'HZ98044 -- Warning: No scattered positron found,'
          PRINT *, '        -- event skipped !'
          GOTO 999
        ENDIF

c....   initialise boost to hCMS
        CALL HZHCMINI (pbeam, gamma, ierr)
        IF (ierr .EQ. 1) THEN
          PRINT *, 'HZ98044 -- Warning: boost to hadronic CMS failed,'
          PRINT *, '        -- event skipped !'
          GOTO 999
        ENDIF

c....   select final state particles and store pseudorapidity in hCMS
        DO ihep = 1, NHEP
          iacc(ihep) = .FALSE.
          rap(ihep) = 9999.
          IF (
     &        ISTHEP(ihep) .EQ. 1 .AND.
     &        ihep .NE. ibp       .AND.
     &        ihep .NE. ibl       .AND.
     &        ihep .NE. isl       .AND.
     &        .TRUE.) THEN
            iacc(ihep) = .TRUE.
            p_lab(1) = PHEP (1, ihep)
            p_lab(2) = PHEP (2, ihep)
            p_lab(3) = PHEP (3, ihep)
            p_lab(4) = PHEP (4, ihep)
            CALL HZHCM(p_lab, p_hcms, ierr)
            IF (ierr .EQ. 1) THEN
              PRINT *,
     &          'HZ98044 -- Warning: boost to hadronic CMS failed,'
              PRINT *, '        -- event skipped !'
              GOTO 999
            ENDIF
c           rap is pseudorapidity !
            p = SQRT(p_hcms(1)**2+p_hcms(2)**2+p_hcms(3)**2)
            pz = p_hcms(3)
            IF (p .GT. ABS(pz)) rap(ihep) = 0.5*(LOG(p+pz)-LOG(p-pz))
          ENDIF
        ENDDO

c....   sort particles with increasing pseudorapidity
        CALL SORTZV (rap, index, NHEP, 1, 0, 0)

c....   look for largest gap
        gap =  0.
        DO ihep = 2, NHEP
          tmp = rap(index(ihep)) - rap(index(ihep-1))
          IF (tmp .GT. gap .AND. iacc(index(ihep))) THEN
            gap = tmp
            id = ihep
          ENDIF
        ENDDO
c       store boundary between X and Y in variable gap
        gap = rap(index(id-1)) + gap/2

c....   get DIS kinematics
        q2 = HZDISKIN (1)
        y  = HZDISKIN (3)
        w2 = HZDISKIN (4)

c....   get diffractive DIS kinematics and select particles of system X
        DO i = 1, 4
          xvec(i) = 0.
          yvec(i) = 0.
        ENDDO
        DO ihep = 1, NHEP
          IF (iacc(ihep)) THEN
            IF (rap(ihep) .GT. gap) THEN
              xvec(1) = xvec(1) + PHEP (1, ihep)
              xvec(2) = xvec(2) + PHEP (2, ihep)
              xvec(3) = xvec(3) + PHEP (3, ihep)
              xvec(4) = xvec(4) + PHEP (4, ihep)
            ELSE
              iacc(ihep) = .FALSE.
              yvec(1) = yvec(1) + PHEP (1, ihep)
              yvec(2) = yvec(2) + PHEP (2, ihep)
              yvec(3) = yvec(3) + PHEP (3, ihep)
              yvec(4) = yvec(4) + PHEP (4, ihep)
            ENDIF
          ENDIF
        ENDDO
        mx = xvec(4)**2 - xvec(1)**2 - xvec(2)**2 - xvec(3)**2
        IF (mx .GT. 0) THEN
          mx = SQRT(mx)
        ELSE
          mx = 0.
        ENDIF
        my = yvec(4)**2 - yvec(1)**2 - yvec(2)**2 - yvec(3)**2
        IF (my .GT. 0) THEN
          my = SQRT(my)
        ELSE
          my = 0.
        ENDIF
        t = (pbeam(4)-yvec(4))**2 - (pbeam(1)-yvec(1))**2
     &    - (pbeam(2)-yvec(2))**2 - (pbeam(3)-yvec(3))**2
        xp = (q2 + mx**2 - t)/(q2 + w2 - mp**2)

c....   select event
        IF (
     &      q2 .GT. q2_low      .AND.  q2 .LT. q2_high  .AND.
     &      y  .GT. y_low       .AND.  y  .LT. y_high   .AND.
     &      mx .GT. mx_low      .AND.  mx .LT. mx_high  .AND.
     &      my .LT. my_high     .AND.
     &      ABS(t) .LT. t_high  .AND.
     &      xp .LT. xp_high     .AND.
     &      .TRUE.) THEN

          nacc = nacc + 1

c....     initialise boost to gamma-pom CMS
          pbeam(1) = xp*pbeam(1)
          pbeam(2) = xp*pbeam(2)
          pbeam(3) = xp*pbeam(3)
          pbeam(4) = xp*pbeam(4)
          CALL HZHCMINI (pbeam, gamma, ierr)

c....     get number of particles in gamma-pom CMS
          nt = 0
          nf = 0
          nb = 0
          np = 0
          nn = 0
          DO ihep = 1, NHEP
            IF (iacc(ihep) .AND. IHCHRG(ihep) .NE. 0) THEN
              p_lab(1) = PHEP(1, ihep)
              p_lab(2) = PHEP(2, ihep)
              p_lab(3) = PHEP(3, ihep)
              p_lab(4) = PHEP(4, ihep)
              CALL HZHCM (p_lab, p_hcms, ierr)
c             rap is real rapidity !
              p = p_hcms(4)
              pz = p_hcms(3)
              rap(ihep) = 9999.
              IF (p .GT. ABS(pz)) rap(ihep) = 0.5*(LOG(p+pz)-LOG(p-pz))
              nt = nt + 1
              IF (rap(ihep) .GT. 0) nf = nf + 1
              IF (rap(ihep) .LT. 0) nb = nb + 1
              IF (IHCHRG(ihep) .GT. 0) np = np + 1
              IF (IHCHRG(ihep) .LT. 0) nn = nn + 1
            ENDIF
          ENDDO

c....     fill histograms
          DO i = 1, 3
            id = i*10
            IF (mx .GT. mxbins2(i) .AND. mx .LT. mxbins2(i+1)) THEN
              CALL HFILL (id,   FLOAT(nt), 0., wtx)
              CALL HFILL (id+1, FLOAT(nf), 0., wtx)
              CALL HFILL (id+2, FLOAT(nb), 0., wtx)
              CALL HFILL (id+3, FLOAT(np), 0., wtx)
              CALL HFILL (id+4, FLOAT(nn), 0., wtx)
              DO ihep = 1, NHEP
                IF (iacc(ihep) .AND. IHCHRG(ihep) .NE. 0)
     &            CALL HFILL (id+5, rap(ihep), 0., wtx)
              ENDDO
            ENDIF
          ENDDO
          DO i = 1, 7
            id = 100+i*10
            IF (mx .GT. mxbins1(i) .AND. mx .LT. mxbins1(i+1)) THEN
              CALL HFILL (id,   FLOAT(nt), 0., wtx)
              CALL HFILL (id+1, FLOAT(nf), 0., wtx)
              CALL HFILL (id+2, FLOAT(nb), 0., wtx)
              DO ihep = 1, NHEP
                IF (iacc(ihep) .AND. IHCHRG(ihep) .NE. 0)
     &            CALL HFILL (id+5, rap(ihep), 0., wtx)
              ENDDO
            ENDIF
          ENDDO

          CALL HFILL (90, mx, 0., mx*wtx)
          CALL HFILL (91, mx, 0., wtx)
          CALL HFILL (92, mx, 0., mx*wtx)
          CALL HFILL (93, mx, 0., wtx)

        ENDIF

c-----------------------------------------------------------------------
c---- Termination

      ELSEIF (iflag .EQ. 3) THEN

c....   move to the correct PAW subdirectory
        call hcdir('//PAWC/98044',' ')

c....   normalise multiplicity distributions and rapidity spectra
        CALL HUNPAK (93, nev, ' ', 1)
        DO i = 1, 3
          id = i*10
          CALL HZHINRM (id,   0, nev(i)/100, 1)
          CALL HZHINRM (id+1, 0, nev(i)/100, 1)
          CALL HZHINRM (id+2, 0, nev(i)/100, 1)
          CALL HZHINRM (id+3, 0, nev(i)/100, 1)
          CALL HZHINRM (id+4, 0, nev(i)/100, 1)
          CALL HZHINRM (id+5, 0, nev(i), 1)
        ENDDO
        CALL HUNPAK (91, nev, ' ', 1)
        DO i = 1, 7
          id = 100+i*10
          CALL HZHINRM (id,   0, nev(i)/100, 1)
          CALL HZHINRM (id+1, 0, nev(i)/100, 1)
          CALL HZHINRM (id+2, 0, nev(i)/100, 1)
          CALL HZHINRM (id+5, 0, nev(i), 1)
        ENDDO

c....   extract moments, central density and forward-backward corr.
        DO i = 1, 7
          id = 100+i*10
          mt(i) = HSTATI (id,   1, ' ', 1)
          mf(i) = HSTATI (id+1, 1, ' ', 1)
          mb(i) = HSTATI (id+2, 1, ' ', 1)
          dt(i) = HSTATI (id,   2, ' ', 1)
          df(i) = HSTATI (id+1, 2, ' ', 1)
          db(i) = HSTATI (id+2, 2, ' ', 1)
          rt(i) = 1 + (dt(i)**2 - mt(i))/mt(i)**2
          rf(i) = 1 + (df(i)**2 - mf(i))/mf(i)**2
          rb(i) = 1 + (db(i)**2 - mb(i))/mb(i)**2
          cd(i) = (HX(id+5, -0.25) + HX(id+5, 0.25))/2
          fb(i) = (dt(i)**2 - df(i)**2 - db(i)**2)/(2*df(i)*db(i))
        ENDDO
        CALL HPAK (40, mt)
        CALL HPAK (41, mf)
        CALL HPAK (42, mb)
        CALl HPAK (43, cd)
        CALL HPAK (50, dt)
        CALL HPAK (51, df)
        CALL HPAK (52, db)
        CALL HPAK (60, rt)
        CALL HPAK (61, rf)
        CALL HPAK (62, rb)
        CALL HPAK (70, fb)

c....   store H1 data in histograms
        DO k = 1, 4
          DO j = 1, 3
            DO i = 1, 20
              pn_tmp (i) = pn (i,j,k)
              pne_tmp(i) = pne(i,j,k)
              pns_tmp(i) = SQRT(pne(i,j,k)**2 + pns(i,j,k)**2)
            ENDDO
            id = -10*j-k
            CALL HPAK  (id, pn_tmp)
            CALL HPAKE (id, pne_tmp)
            CALL HPAK  (id-100, pn_tmp)
            CALL HPAKE (id-100, pns_tmp)
          ENDDO
        ENDDO

        DO j = 1, 3
          DO i = 1, 7
            m_tmp(i)  = m(i,j)
            me_tmp(i) = me(i,j)
            ms_tmp(i) = SQRT(me(i,j)**2 + ms(i,j)**2)
            d_tmp(i)  = d(i,j)
            de_tmp(i) = de(i,j)
            ds_tmp(i) = SQRT(de(i,j)**2 + ds(i,j)**2)
            r_tmp(i)  = r(i,j)
            re_tmp(i) = re(i,j)
            rs_tmp(i) = SQRT(re(i,j)**2 + rs(i,j)**2)
          ENDDO
          CALL HPAK  (-40-j+1,  m_tmp)
          CALL HPAKE (-40-j+1,  me_tmp)
          CALL HPAK  (-140-j+1, m_tmp)
          CALL HPAKE (-140-j+1, ms_tmp)
          CALL HPAK  (-50-j+1,  d_tmp)
          CALL HPAKE (-50-j+1,  de_tmp)
          CALL HPAK  (-150-j+1, d_tmp)
          CALL HPAKE (-150-j+1, ds_tmp)
          CALL HPAK  (-60-j+1,  r_tmp)
          CALL HPAKE (-60-j+1,  re_tmp)
          CALL HPAK  (-160-j+1, r_tmp)
          CALL HPAKE (-160-j+1, rs_tmp)
        ENDDO

        CALL HPAK  (-43, dcd)
        CALL HPAKE (-43, dcde)
        DO i = 1, 7
          dcds(i) = SQRT(dcde(i)**2 + dcds(i)**2)
        ENDDO
        CALL HPAK  (-143, dcd)
        CALL HPAKE (-143, dcds)

        CALL HPAK  (-70, drho)
        CALL HPAKE (-70, drhoe)
        DO i = 1, 3
          drhos(i) = SQRT(drhoe(i)**2 + drhos(i)**2)
        ENDDO
        CALL HPAK  (-170, drho)
        CALL HPAKE (-170, drhos)

        DO j = 1, 3
          DO i = 1, 12
            drap_tmp(i)  = drap(i,j)
            drape_tmp(i) = drape(i,j)
            draps_tmp(i) = SQRT(drape(i,j)**2 + draps(i,j)**2)
          ENDDO
          id = -10*j-5
          CALL HPAK  (id, drap_tmp)
          CALL HPAKE (id, drape_tmp)
          CALL HPAK  (id-100, drap_tmp)
          CALL HPAKE (id-100, draps_tmp)
        ENDDO

        CALL HPAK  (-90, mx1)
        CALL HPAK  (-91, mx2)
        CALL HPAK  (-92, mx3)
        CALL HPAK  (-93, mx4)

c....   print number of accepted events
        PRINT *, 'HZ98044 -- Number of accepted moca events: ', nacc

      ENDIF

c----------------------------------------------------------------------
c---- Return

999   RETURN
      END
