      subroutine HZ98050(IFLAG)
****************************************************************************
* Purpose: Produce fraction of events with a forward jet using a cone
* algorithm (R=1).
*
* The event is accepted if it fulfills the requirements:
*   y>0.1, E_positron >10 GeV
*   0.00045 < x < 0.045
*   1 jet with cone algorithm (radius 1, PXCONE) and:
*     Et > 5 GeV
*     0 < eta < 2.6
*     x_jet > 0.036
*     0.5 < E_t**2/Q2 < 2
*     cos(theta_breit) > 0 (jet in fragmentation region of the breit frame)
*
*    Only 1 entry per event (even if there are 2 jets fulfilling the cuts).
*
*    histograms:
*    1    forward jet cross section in x_bjorken bins in the MC.
*   -1    forward jet cross section in x_bjorken bins in the data.
*     The data are corrected to hadron level. The parton level corrections
*     are not included.
*
*     700  eta distribution of the highest xjet jet
*     701  eta distribution of the second highest xjet jet
*     710  Et distribution of the highest xjet jet
*     711  Et distribution of the second highest xjet jet
*     720  Et2/Q2 distribution of the highest xjet jet (no Et2/Q2 cut applied)
*     721             "            second      "
*
*     These are parton level results. A direct comparison with the data is
*     bound to hadronization uncertainties.
*
*     300  MEPJET cross sections (scale 0.25*kt**2) for forward jets
*     301  MEPJET cross sections (scale 2*kt**2) for forward jets
*     400  BFKL LO for forward jets
*     401  BFKL born for forward jets
*
*     The program gives the number of events with 1 or 2 forward jets
*     The systematics (in the data) are printed at the end
*
* Event selection:
*
* Running:
*
* Arguments: iflag=1 initialisation
*            iflag=1 filling
*            iflag=3 termination
*
* written by:
****************************************************************************
      IMPLICIT NONE
#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hznlo.inc"
*
      Integer iprog
      Real qp_3,qp_2
      Real pthg1,shg1,xpg1,zqg1
      Common/partons/pthg1,shg1,xpg1,zqg1,iprog
c  ntuple jets
      Integer njet,ipro
      Real pthg,shg,xpg,zqg,pth,sh,xp,zq1,zq2,deta
      common/cwnco1/x,q2,y,ipro
      Integer kind(32)
      Integer numjet
      Parameter (numjet=200)
      Double precision pj(numjet,8)
      Double precision pbeam(4),pgam(4),pcm(4),ph(5)
      Double precision pold(4),pnew(4)
*
      Integer i,iflag,ihep,k1
      Character*5 xxxx
*
      Real pi,rd,eb,ee
      Parameter (ee=27.5)
      Integer iel,idum,ibeam,igam,ifj,afj
      Parameter (pi=3.1415927,rd=180./pi)
      Integer nentry
      Real x,y,q2,enel,thel,efwd,th,lx,lxj
      Real ptj21,pt2q21,etaj1
      Real thj,xj,ptb,costhb
      Integer ibin,nj,ij,naj,naj2,np,np4,np5,nsel
*
      Integer ierr
      Integer maxhi,modjet,maxjet
      Parameter (maxhi=100,modjet=2,maxjet=30)
      Real  selj(maxjet,4),help,empz,empzh,xg,empzl,empzhl,xgl
      Integer ipjet(maxjet),l,k,j
      Real nev(maxhi),lum,xw,nev2(maxhi)
      save nev2
      Double precision rcone
      Parameter (rcone=1.)
      Logical lp,fjet,twojet
      Data lp/.false./
      Real nxg(3)
*

      real q2as,ulalps,a1,a2
      double precision alphas2,alphas
      real xjet1
      real etaj(5),xjet(5),pt2q2(5),ptj2(5)
      real eta0,pt2q20,xjet0,pt0

*********************************************************************
*
      Integer nbin,nbin1
      Parameter (nbin=8)
      Parameter (nbin1=16)
      real xout(nbin),xerr(nbin)
      real fx(nbin+1),ptqbin(nbin1+1)
      Data   fx/0.00045,0.0008,0.0014,0.0025,0.0045,0.008,0.014,
     &            0.025,0.045/
      data ptqbin /0.01,0.017,0.03,
     +  0.06,0.1,0.17,0.3,0.6,1.,1.7,3.,6.,10.,17.,30.,60.,100./
*
*********************************************************************
      Integer nx
      Real n2,nall,nall2
      Real xmin,xmax
      Data xxxx/'98050'/,NENTRY/0/
      save xxxx,nev,nall
*********************************************************************
*
*                      data points from zeus
*
*********************************************************************
      real sup(8),sdown(8)
      real fjxs(8),efjxs(8),sysup(8),sysdown(8)
      real fjetq(8),efjetq(8)
      data fjxs /114.,96.2,77.8,34.4,14.1,6.53,2.65,0.65/
      data efjxs /9.7,6.5,4.7,2.2,1.0,0.54,0.25,0.09/
      data sysup /29.,8.2,5.2,3.8,2.5,0.1,0.3,0.1/
      data sysdown /15.,8.2,6.9,1.9,1.2,0.7,0.3,0.4/
      data fjetq /59.5,164,255,288,190,41.2,2.95,0.120/
      data efjetq /32.3,32.5,21.8,11.6,5.93,1.41,0.19,0.02/
      save sup,sdown
*********************************************************************
*
*                      MEPJET cross sections
*
*********************************************************************
*
* for scale 0.25*kt2
*
      real mep1(8)
      data mep1 /22.90,26.96,20.99,13.46,6.20,2.73,1.24,0.34/
*
* forscale 2*kt2
*
      real mep2(8)
      data mep2 /13.46,21.29,16.80,10.41,5.18,2.43,1.05,0.32/
*********************************************************************
*
*                      BARTELS and Wusthoff cross sections
*
*********************************************************************
*
*     LO BFKL
*
      real bfkl(8)
      data bfkl /147.7,150.9,104.6,46.51,16.26,5.25,1.44,0.23/
*
*     1st term
*
      real born(8)
      data born /28.93,35.50,34.26,18.32,8.38,3.20,1.07,0.19/
	
	Integer NLOJET
*********************************************************************
*
*                      Initialization
*
*********************************************************************
      NENTRY=NENTRY+1
*
      IF (iflag.eq.1) then
      NLO=0
	NLOJET = 0
*
       nxg(1)=0.
       nxg(2)=0.
       nxg(3)=0.
       nall=0.
       nall2=0.
       n2=0.
       do i = 1,maxhi
          nev(i) = 0.
          nev2(i) = 0.
       enddo
*
*
* Initialisation: The following MUST always be done
* (i) make subdirectory in PAWC
* - use the name as the xxxxxx in HZxxxxxx subroutine
* (i) make subdirectory in o/p file
*
        Call hcdir('//PAWC',' ')
        call hmdir(xxxx,'S')
        Call hcdir('//HISTO',' ')
        call hmdir(xxxx,'S')
*
* book histos
*
*
       call hbook1(31,' nall/cross ',3,0.,3.,0.)
       call hbookb(1 ,' 1 fjets XBj ',nbin,fx,0.)
       call hbookb(-1,' 1 fjets XBj ',nbin,fx,0.)
       call hbook1(700,'ethad 1st fjet',30,0.,30.,0.)
       call hbook1(701,'ethad 2nd fjet',30,0.,30.,0.)
       call hbook1(710,'etahad 1st fjet',15,0.,3.,0.)
       call hbook1(711,'etahad 2nd fjet',15,0.,3.,0.)
       call hbookb(720,'pt2q2 1st fjet',nbin1,ptqbin,0.)
       call hbookb(721,'pt2q2 2nd fjet',nbin1,ptqbin,0.)
       call hbookb(300 ,'MEPJET scale 0.25kt2 ',nbin,fx,0.)
       call hbookb(301 ,'MEPJET scale 2kt2',nbin,fx,0.)
       call hbookb(400 ,'BFKL LO ',nbin,fx,0.)
       call hbookb(401 ,'BFKL 1st term',nbin,fx,0.)
       call hbookb(-720 ,' 1 fjets Et2/Q2 ',nbin1,ptqbin,0.)
*
*      pack data into histograms ; 8 bins
*
*
       Call hpak (-1,fjxs)
       Call hpake(-1,efjxs)
*
*     pack theory points
*
       call hpak(300,mep1)
       call hpak(301,mep2)
       call hpak(400,bfkl)
       call hpak(401,born)
*
*      add sytematical and statistical error
*
       Do i=1,8
          sup(i) = sqrt(efjxs(i)**2+sysup(i)**2)
          sdown(i) = sqrt(efjxs(i)**2+sysdown(i)**2)
       enddo
*
         Do I=1,8
            fjetq(i)=fjetq(i)/1000.
            efjetq(i)=efjetq(i)/1000.
         Enddo
         Call hpak (-720,fjetq)
         Call hpake(-720,efjetq)
*********************************************************************
*
*                      Event Processing
*
*********************************************************************
      Else if(iflag.eq.2) then
         do i = 1,5
            etaj(i) = 0.
            pt2q2(i) = 0.
            xjet(i) = 0.
            ptj2(i) = 0.
         enddo

*
*     Filling: The following MUST always be done
*     (i) move to the correct sub-directory in PAWC
*
         call hcdir('//PAWC/'//xxxx,' ')
*
         xw=wtx
*
         IF(NLO.eq.1.and.TOT.ne.1) GoTo 11112
c	      write(6,*) ' 2nd NLO ',NLO,' TOT ',TOT,wtx,Nall
c	      write(6,*) ' 2nd NLO  hzp02133',NLO,' TOT ',TOT,wtx,Nall
            Nall = Nall + wtx
11112	   Continue
         if(Nall.le.0) then
	      write(6,*) ' hzp02133 NLO ',NLO,' TOT ',TOT,wtx,Nall
	   endif
	   NLOJET = DIJET + TRIPJET + QUADJET
         ierr=HZIBEAM(ibeam,idum)
         if (.not.(ierr.eq.1)) then
            write(6,*) 'HZ'//xxxx,' beams not found ! '
            return
         else
            Do i=1,4
               pbeam(i)=PHEP(i,IBEAM)
            enddo
         endif
*
         eb=real(PHEP(4,idum))
         if (abs(abs(eb)-ee).gt.0.2) then
            if (nentry.lt.10) then
               write(6,*) 'HZ'//xxxx,' Electron not at ',ee,' ! ',eb
            elseif (nentry.eq.10) then
               write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,
     &              ' ...last message ! '
            endif
            return
         endif
*
* Event selection
*
         q2=real(HZDISKIN(1))
         x =real(HZDISKIN(2))
         y =real(HZDISKIN(3))

         deta = -999.
         njet = 0
         ipro = iprog
*
*
*     get electron variables for kinematic selection
*
       iel=HZIDELEC(idum)
       if (iel.eq.-1) then
        write(*,*) 'Hz'//xxxx,' electron not found '
        return
       endif
       enel=real(PHEP(4,iel))
       thel=real(HZPHMANG(PHEP(3,iel),
     $      sqrt(PHEP(1,iel)**2+PHEP(2,iel)**2)))
*
*     Momentum of the gamma
*
       pgam(1)=-real(PHEP(1,iel))
       pgam(2)=-real(PHEP(2,iel))
       pgam(3)=-eb - real(PHEP(3,iel))
       pgam(4)= eb - real(PHEP(4,iel))
*
       if (lp) then
        write(6,*) 'Hz'//xxxx,' enel= ',enel,' thel= ',thel
        write(6,*) 'Hz'//xxxx,' y= ',y,' q2= ',q2
       endif
*
       fjet=.true.
       if (y.lt.0.1)    fjet=.false.
       if (enel.lt.10.0) then
        fjet=.false.
       endif
*
* Initialize Breit frame boost
*
       call HZBRTINI(pbeam,pgam,ierr)
       if (ierr.eq.1) write(*,*) 'Hz'//xxxx,' Boost failed'
*
* find jets
*
       if (fjet) then
*
          call hzjtfind(modjet,rcone,nj,pj)
*
          if (lp) then
             write(6,*) 'forward jet analysis: nj= ',nj
          endif
          if (nj.gt.numjet) then
             write(6,*) 'hz',xxxx,' too many jets found ! '
             return
          endif
*
          naj=0
          ifj=0
          do 100 ij=1,nj
*
             pold(1) = pj(ij,5)
             pold(2) = pj(ij,6)
             pold(3) = pj(ij,7)
             pold(4) = pj(ij,4)
*
*     Boost jet in the breit frame
*
             call HZBRT(pold,pnew,ierr)
             if (ierr.eq.1) write(*,*) 'Hz'//xxxx,' Boost failed'
             ptb = sqrt(real(pnew(1))**2+real(pnew(2))**2)
             costhb = real(pnew(3)) / sqrt(ptb**2 + real(pnew(3))**2)
*
             if (lp) write(6,'(a,4f9.3)')
     &            ' Jet= ',(real(pj(ij,i)),i=4,7)
*
             ptj21=real(pj(ij,3))
             pt2q21=ptj21**2/q2
             thj=real(HZPHMANG(pj(ij,7),pj(ij,3)) )
             etaj1 = pj(ij,1)
*
*     Get the forward jets
*
             if (
     &          ptj21.gt.5.and.
     &            etaj1.lt.2.6.and.
     &            etaj1.gt.0.) then
                xjet1=real(pj(ij,4))/820.*cos(thj)
                if (costhb.gt.0) then
                   ifj = ifj + 1
                   xjet(ifj) = xjet1
                   ptj2(ifj) = ptj21
                   pt2q2(ifj) = pt2q21
                 etaj(ifj) = etaj1
              endif
           endif
 100    continue
        if (ifj.eq.0)  return
*
*     Save an event if there is a f.j. with 0.5<Et2/Q2<2
*     and xjet>0.036
*
        afj = 0
        do i = 1,ifj
           if (pt2q2(i).lt.2.and.pt2q2(i).gt.0.5) then
              if (xjet(i).gt.0.036) then
                 afj = afj + 1
              endif
           endif
        enddo
        if (afj.ge.1) call hf1(1,x,xw)
*
        if (x.gt.fx(1).and.x.lt.fx(9)) then
           if (afj.eq.1)  nev(1)=nev(1)+xw
           if (afj.eq.2)  nev2(1)=nev2(1)+xw
        endif
*
*     Sort jets by xjet
*
        if (ifj.eq.1) goto 12
        xjet0 = 0.
        do j = 1,ifj
           xjet0 = xjet(j)
           l = j+1
           k1 = 0
           do k  = l,ifj
              if (xjet(k).gt.xjet0) then
                 xjet0 = xjet(k)
                 eta0 = etaj(k)
                 pt0 = ptj2(k)
                 pt2q20 = pt2q2(k)
                 k1 = k
              endif
           enddo
           if (k1.eq.0) goto 12
           xjet(k1) = xjet(j)
           etaj(k1) = etaj(j)
           ptj2(k1) = ptj2(j)
           pt2q2(k1) = pt2q2(j)
           xjet(j) = xjet0
           etaj(j) = eta0
           ptj2(j) = pt0
           pt2q2(j) = pt2q20
        enddo
 12     continue
*
*     Store the histograms
*     highest xjet
*
        if (xjet(1).gt.0.036) then
           call hf1(720,pt2q2(1),xw)
           if (pt2q2(1).gt.0.5.and.pt2q2(1).lt.2.) then
              call hf1(700,ptj2(1),xw)
              call hf1(710,etaj(1),xw)
           endif
        endif
*
*     second highest xjet
*
        if (ifj.eq.1) goto 2000
        if (xjet(2).gt.0.036) then
           if (x.gt.fx(1).and.x.lt.fx(9)) then
              call hf1(721,pt2q2(2),xw)
              if (pt2q2(2).gt.0.5.and.pt2q2(2).lt.2.) then
                 call hf1(701,ptj2(2),xw)
                 call hf1(711,etaj(2),xw)
              endif
           endif
        endif
*
 2000   continue
11114    continue
      endif
*
*********************************************************************
*
*     Termination
*
*********************************************************************

      Else if(iflag.eq.3) then

* Termination: The following MUST always be done
* (i) Move to the correct PAW subdirectory
*
       call hcdir('//PAWC/'//xxxx,' ')
*
       lum=999999999.
       if (xsec.ne.0.) then
        lum=real(nall)/real(xsec)
       else
        write(6,*) 'hz',xxxx,' xsec=0 ! '
       endif
*
        write(6,*) 'hz0050: Total Cross= ',
     &  real(xsec),' nall= ',nall
        write(6,*) '  ZEUS: # of 1 fjets= ',nev(1),' lumi= ',lum
        write(6,*) '  ZEUS: # of 2 fjets= ',nev2(1),' lumi= ',lum
        write(6,*) 'Systematics up (+ statistics)', sup
        write(6,*) 'Systematics down (+ statistics)', sdown
*
* convert in nb
*
        lum = lum * 1000.
*
*
*
        call HBOOKB(1111,'Tx',8,fx,0.)
        call HBARX(0)

        do I = 1,8
           xout(i)=fx(i+1)-fx(i)
           xerr(i)=0.
        enddo
        call HPAK(1111,xout)
        call HPAKE(1111,xerr)
*
*     Normalise to luminosity
*
        call hopera(1,'+E',1,1,1./lum,0.)
        call hopera(700,'+E',700,700,1./lum,0.)
        call hopera(710,'+E',710,710,1./lum,0.)
        call hopera(720,'+E',720,720,1./lum,0.)
        call hopera(701,'+E',701,701,1./lum,0.)
        call hopera(711,'+E',711,711,1./lum,0.)
        call hopera(721,'+E',721,721,1./lum,0.)
        call hopera(1,'/E',1111,1,1.,1.)
        call hopera(720,'/E',1112,720,1.,1.)
        call hzhinfo(1,nev(1))
        call hzchisq(-1,1)
*
      endif
*
      RETURN
      END
