*CMZ :  2.00/06 02/05/2002  14.48.48  by  Tancredi Carli
*CMZ :  1.02/05 13/01/99  17.22.17  by  Tancredi Carli
*CMZ :  1.02/01 23/10/98  17.57.20  by  Tancredi Carli
*CMZ :  1.02/00 02/10/98  13.00.48  by  Tancredi Carli
*CMZ :  1.01/15 04/09/98  19.48.48  by  Tancredi Carli
*-- Author : Tancredi Carli
      subroutine HZ98076(IFLAG)
****************************************************************************
* Purpose:
*  Dijet fraction for 5 < Q2 < 100 GeV2
*  as a function of Q2 and x
*  using the cone algorithm
*
* Event selection:
*  156 < theta_e' < 173 deg.
*  E' > 11 GeV
*  y  > 0.05
*
* Jet reconstruction and selection in photon-proton cms:
*  R = 1, p_t_min = 5 GeV, f = 0.75
*  exactly two jets per event fulfilling the above criteria are demanded,
*  in addition |eta_jet1 - eta_jet2| < 2.
*
* In data 112806 DIS events are selected of which 4957 are di-jet events.
* The dijet fractions have been corrected to the hadron level in the phase
* space region given by the cuts under event selection above. They are
* given in bins of Q2 (5 < Q2 < 100 GeV2), integrated over x, and in bins
* of x (0.0001 < x < 0.01), integrated over Q2.
*
* Statistical and systematic errors are included. Not included is an overall
* systematic error of +10% and -8% for the symmetric and +11% and -8% for
* the asymmetric and the sum scenario, arising from the uncertainty of the
* hadronic energy scale of the calorimeter and the uncertainty of the rad.
* QED corrections.
*
* Running:
*
*  Beams: e+ at 27.5 GeV, and p at 820 GeV
*
*  In generating events, besides applying the cuts under event selection
*  above, the ranges in Q2 and x should not have stricter limits than:
*  5 < Q2 < 120 GeV2  and  0.00009 < x < 0.023.
*
* Arguments: iflag=1 initialisation
*            iflag=1 filling
*            iflag=3 termination
*
* written by:
****************************************************************************
      IMPLICIT NONE
#include "hzfunc.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
*
      Integer kind(32)
      Integer numjet
      Parameter (numjet=200)
      Double precision pj(numjet,8)
      Double precision pbeam(4),pgam(4),pcm(4),ph(5),plab(5,NMXHEP)
*
      Integer i,iflag,ihep
      Character*5 xxxx
*
      Real pi,rd,eb,ee
      Parameter (ee=27.5)
      Integer iel,idum,ibeam
      Parameter (pi=3.1415927,rd=180./pi)
      Integer nentry
      Real x,y,q2,enel,thel,th
      Real ptjcut,ptj,ptj1,ptj2,pt2r,thj,deta,thj1,thj2
      Parameter (ptjcut=5.)
      Integer nj,ij,j,nsel
*
      Integer ierr
      Integer maxhi,modjet,maxjet
      Parameter (maxhi=100,modjet=2,maxjet=30)
      Integer ipjet(maxjet),l,k
      Real xw
      Double precision rcone
      Parameter (rcone=1.)
      Logical lp,twojet
      Data lp/.false./
*
* symmetric scenario (5/5 GeV)
*
      Integer nbin
      Parameter (nbin=6)
      Real r2x(nbin),str2x(nbin),sypr2x(nbin),symr2x(nbin),
     &     totpr2x(nbin),totmr2x(nbin),rx(nbin+1)
      Real r2q(nbin),str2q(nbin),sypr2q(nbin),symr2q(nbin),
     &     totpr2q(nbin),totmr2q(nbin),rq2(nbin+1)
      Real nq2(nbin)
*
      Data r2x   / 0.0463, 0.0457, 0.0471, 0.0489, 0.0533, 0.0686/
*     stat. error
      Data str2x / 0.0030, 0.0022, 0.0018, 0.0016, 0.0025, 0.0045/
*     syst. plus error
      Data sypr2x/ 0.0057, 0.0038, 0.0025, 0.0053, 0.0033, 0.0156/
*     syst. minus error
      Data symr2x/-0.0040,-0.0047,-0.0052,-0.0039,-0.0042,-0.0113/
*     bins in x
      Data rx/0.0001,0.00025,0.0005,0.001,0.0025,0.005,0.01/
**
      Data r2q   / 0.0403, 0.0382, 0.0463, 0.0501, 0.0666, 0.0926/
*     stat. error
      Data str2q / 0.0020, 0.0018, 0.0021, 0.0021, 0.0026, 0.0040/
*     syst. plus error
      Data sypr2q/ 0.0049, 0.0032, 0.0029, 0.0058, 0.0061, 0.0063/
*     syst. minus error
      Data symr2q/-0.0038,-0.0024,-0.0020,-0.0058,-0.0035,-0.0097/
*     bins in Q2
      Data  rq2/5.0,11.0,15.0,20.0,30.0,50.0,100.0/
*
* asymmetric or difference scenario (5/7 GeV)
*
      Real r2xd(nbin),str2xd(nbin),sypr2xd(nbin),symr2xd(nbin),
     &     totpr2xd(nbin),totmr2xd(nbin)
      Real r2qd(nbin),str2qd(nbin),sypr2qd(nbin),symr2qd(nbin),
     &     totpr2qd(nbin),totmr2qd(nbin)
*
      Data r2xd   / 0.0309, 0.0272, 0.0278, 0.0304, 0.0329, 0.0436/
*     stat. error
      Data str2xd / 0.0023, 0.0015, 0.0013, 0.0012, 0.0018, 0.0034/
*     syst. plus error
      Data sypr2xd/ 0.0022, 0.0017, 0.0023, 0.0019, 0.0023, 0.0150/
*     syst. minus error
      Data symr2xd/-0.0024,-0.0014,-0.0028,-0.0024,-0.0018,-0.0069/
**
      Data r2qd   / 0.0237, 0.0214, 0.0280, 0.0285, 0.0427, 0.0632/
*     stat. error
      Data str2qd / 0.0014, 0.0012, 0.0015, 0.0014, 0.0020, 0.0031/
*     syst. plus error
      Data sypr2qd/ 0.0044, 0.0012, 0.0028, 0.0020, 0.0031, 0.0031/
*     syst. minus error
      Data symr2qd/-0.0030,-0.0007,-0.0012,-0.0015,-0.0022,-0.0042/
*
* sum scenario (13 GeV)
*
      Real r2xs(nbin),str2xs(nbin),sypr2xs(nbin),symr2xs(nbin),
     &     totpr2xs(nbin),totmr2xs(nbin)
      Real r2qs(nbin),str2qs(nbin),sypr2qs(nbin),symr2qs(nbin),
     &     totpr2qs(nbin),totmr2qs(nbin)
*
      Data r2xs   / 0.0293, 0.0266, 0.0274, 0.0304, 0.0334, 0.0447/
*     stat. error
      Data str2xs / 0.0023, 0.0015, 0.0013, 0.0012, 0.0019, 0.0035/
*     syst. plus error
      Data sypr2xs/ 0.0014, 0.0020, 0.0017, 0.0023, 0.0019, 0.0104/
*     syst. minus error
      Data symr2xs/-0.0027,-0.0014,-0.0017,-0.0028,-0.0021,-0.0059/
**
      Data r2qs   / 0.0225, 0.0208, 0.0280, 0.0296, 0.0417, 0.0635/
*     stat. error
      Data str2qs / 0.0014, 0.0012, 0.0015, 0.0015, 0.0020, 0.0031/
*     syst. plus error
      Data sypr2qs/ 0.0029, 0.0022, 0.0023, 0.0013, 0.0025, 0.0029/
*     syst. minus error
      Data symr2qs/-0.0018,-0.0008,-0.0015,-0.0023,-0.0051,-0.0033/
*
*********************************************************************
*
*********************************************************************
      Integer nx
      Real n2,nall,nall2,n2d,n2s
      Real xmin,xmax
      save xxxx,nall,n2,nentry, n2d, n2s
      Data xxxx/'98076'/,NENTRY/0/
*********************************************************************
*
*                      Initialization
*
*********************************************************************
      NENTRY=NENTRY+1
*
      IF (iflag.eq.1) then
*
       nall=0.
       nall2=0.
       n2=0.
       n2d=0.
       n2s=0.
*
*
* Initialisation: The following MUST always be done
* (i) make subdirectory in PAWC
* - use the name as the xxxxxx in HZxxxxxx subroutine
* (i) make subdirectory in o/p file
*
        Call hcdir('//PAWC',' ')
        call hmdir(xxxx,'S')
        Call hcdir('//HISTO',' ')
        call hmdir(xxxx,'S')
*
* book histos
*
       call hbookb(120,' dR2/dx (symm. 5/5 GeV)       ',nbin, rx,0.)
       call hbookb(121,' all x        ',nbin, rx,0.)
       call hbookb(130,' dR2/dQ2 (symm. 5/5 GeV)      ',nbin, rq2,0.)
       call hbookb(131,' all Q2       ',nbin, rq2,0.)
       call hbook1(150,'  TH jet1      ',30,0.,180.,0.)
       call hbook1(151,'  TH jet2      ',30,0.,180.,0.)
       call hbook1(152,'  Pt jet1      ',30,0.,30.,0.)
       call hbook1(153,'  Pt jet2      ',30,0.,30.,0.)
*
       call hbookb(220,' dR2/dx (asymm. 5/7 GeV)      ',nbin, rx,0.)
       call hbookb(230,' dR2/dQ2 (asymm. 5/5 GeV)     ',nbin, rq2,0.)
*
       call hbookb(320,' dR2/dx (sum. 13 GeV)         ',nbin, rx,0.)
       call hbookb(330,' dR2/dQ2 (sum 13 GeV)         ',nbin, rq2,0.)
*
       call hbookb(-120,'  dR2/dx       ',nbin, rx,0.)
       call hbookb(-121,'  dR2/dx       ',nbin, rx,0.)
       call hbookb(-122,'  dR2/dx       ',nbin, rx,0.)
*
       call hbookb(-130,'  dR2/dQ2      ',nbin,rq2,0.)
       call hbookb(-131,'  dR2/dQ2      ',nbin,rq2,0.)
       call hbookb(-132,'  dR2/dQ2      ',nbin,rq2,0.)
*
       call hbookb(-220,'  dR2/dx       ',nbin, rx,0.)
       call hbookb(-221,'  dR2/dx       ',nbin, rx,0.)
       call hbookb(-222,'  dR2/dx       ',nbin, rx,0.)
       call hbookb(-230,'  dR2/dQ2      ',nbin,rq2,0.)
       call hbookb(-231,'  dR2/dQ2      ',nbin,rq2,0.)
       call hbookb(-232,'  dR2/dQ2      ',nbin,rq2,0.)
*
       call hbookb(-320,'  dR2/dx       ',nbin, rx,0.)
       call hbookb(-321,'  dR2/dx       ',nbin, rx,0.)
       call hbookb(-322,'  dR2/dx       ',nbin, rx,0.)
       call hbookb(-330,'  dR2/dQ2      ',nbin,rq2,0.)
       call hbookb(-331,'  dR2/dQ2      ',nbin,rq2,0.)
       call hbookb(-332,'  dR2/dQ2      ',nbin,rq2,0.)
*
*
* pack data into histograms
* pack data et/deta in the 9 bins into histograms
*
         Do i=1,nbin
          totpr2x(i) =sqrt(str2x(i)**2 +sypr2x(i)**2)
          totmr2x(i) =sqrt(str2x(i)**2 +symr2x(i)**2)
          totpr2xd(i)=sqrt(str2xd(i)**2+sypr2xd(i)**2)
          totmr2xd(i)=sqrt(str2xd(i)**2+symr2xd(i)**2)
          totpr2xs(i)=sqrt(str2xs(i)**2+sypr2xs(i)**2)
          totmr2xs(i)=sqrt(str2xs(i)**2+symr2xs(i)**2)
          totpr2q(i) =sqrt(str2q(i)**2 +sypr2q(i)**2)
          totmr2q(i) =sqrt(str2q(i)**2 +symr2q(i)**2)
          totpr2qd(i)=sqrt(str2qd(i)**2+sypr2qd(i)**2)
          totmr2qd(i)=sqrt(str2qd(i)**2+symr2qd(i)**2)
          totpr2qs(i)=sqrt(str2qs(i)**2+sypr2qs(i)**2)
          totmr2qs(i)=sqrt(str2qs(i)**2+symr2qs(i)**2)
         enddo
**
         Call hpak (-120,r2x)
         Call hpake(-120,totpr2x)
         Call hpak (-121,r2x)
         Call hpake(-121,totmr2x)
         Call hpak (-122,r2x)
         Call hpake(-122,str2x)
*
         Call hpak (-130,r2q)
         Call hpake(-130,totpr2q)
         Call hpak (-131,r2q)
         Call hpake(-131,totmr2q)
         Call hpak (-132,r2q)
         Call hpake(-132,str2q)
**
         Call hpak (-220,r2xd)
         Call hpake(-220,totpr2xd)
         Call hpak (-221,r2xd)
         Call hpake(-221,totmr2xd)
         Call hpak (-222,r2xd)
         Call hpake(-222,str2xd)
*
         Call hpak (-230,r2qd)
         Call hpake(-230,totpr2qd)
         Call hpak (-231,r2qd)
         Call hpake(-231,totmr2qd)
         Call hpak (-232,r2qd)
         Call hpake(-232,str2qd)
**
         Call hpak (-320,r2xs)
         Call hpake(-320,totpr2xs)
         Call hpak (-321,r2xs)
         Call hpake(-321,totmr2xs)
         Call hpak (-322,r2xs)
         Call hpake(-322,str2xs)
*
         Call hpak (-330,r2qs)
         Call hpake(-330,totpr2qs)
         Call hpak (-331,r2qs)
         Call hpake(-331,totmr2qs)
         Call hpak (-332,r2qs)
         Call hpake(-332,str2qs)
*
*********************************************************************
*
*                      Event Processing
*
*********************************************************************
      Else if(iflag.eq.2) then
*
* Filling: The following MUST always be done
* (i) move to the correct sub-directory in PAWC
*
       call hcdir('//PAWC/'//xxxx,' ')
*
       xw=wtx
       if (xsec.gt.0) then
        nall=nall+xw
C        write(6,*) xsec,' wtx= ',wtx,' nall= ',nall
       endif
*
       ierr=HZIBEAM(ibeam,idum)
       if (.not.(ierr.eq.1)) then
        write(6,*) 'HZ'//xxxx,' beams not found ! '
        return
       else
        Do i=1,4
         pbeam(i)=PHEP(i,IBEAM)
        enddo
       endif
*
       eb=real(PHEP(4,idum))
       if (abs(abs(eb)-ee).gt.0.2) then
        if (nentry.lt.10) then
         write(6,*) 'HZ'//xxxx,' Electron not at ',ee,' ! ',eb
        elseif (nentry.eq.10) then
         write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,
     &   ' ...last message ! '
        endif
        return
       endif
*
* Event selection
*
       q2=real(HZDISKIN(1))
       x =real(HZDISKIN(2))
       y =real(HZDISKIN(3))
*
       if (lp) then
        write(*,*) 'Hz'//xxxx,' x y q2=',x,y,q2
       endif
*
* get electron variables for kinematic selection
*
       iel=HZIDELEC(idum)
       if (iel.eq.-1) then
         write(*,*) 'Hz'//xxxx,' electron not found '
         return
       endif
       enel=real(PHEP(4,iel))
       thel=real(HZPHMANG(PHEP(3,iel),
     $      sqrt(PHEP(1,iel)**2+PHEP(2,iel)**2)))*rd
*
       if (lp) then
        write(6,*) 'Hz'//xxxx,' enel= ',enel,' thel= ',thel
        write(6,*) 'Hz'//xxxx,' y= ',y,' q2= ',q2
       endif
*
       twojet=.true.
       if (y.lt.0.05) then
         twojet=.false.
       elseif (enel.lt.11.0) then
         twojet=.false.
       elseif (thel.lt.156.0) then
         twojet=.false.
       elseif (thel.gt.173.0) then
        twojet=.false.
       endif
*
* find jets
*
       if (twojet) then
*
        if (xsec.gt.0.) then
         nall2=nall2+xw
*
         call hfill(121,x,0.,xw)
         call hfill(131,q2,0.,xw)
*
        endif
*
        ierr=HZIPGAM(ph)
        if (ierr.eq.-1) then
         write(6,*) 'HZ'//xxxx,' boson vector not found ! '
C         call VZERO(ph,5)
         do i=1,5
          ph(i)=0.
         enddo
        else
         Do i=1,4
          pgam(i)=ph(i)
         enddo
        endif
*
        call HZHCMINI(pbeam,pgam,ierr)
        if (ierr.eq.1) then
         write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
         return
        endif
*
*       loop and boost
*
        do 5 ihep=1,nhep
         Do i=1,4
          ph(i)=PHEP(i,ihep)
          plab(i,ihep)=PHEP(i,ihep)
         enddo
*
         call HZHCM(ph,pcm,ierr)
         if (ierr.eq.1) then
          write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
          return
         endif
*
         if (isthep(ihep).eq.1) then
          th=real(HZPHMANG(PHEP(3,ihep),
     $       sqrt(PHEP(1,ihep)**2+PHEP(2,ihep)**2)))*rd
         endif
*
         Do i=1,4
          PHEP(i,ihep)=Pcm(i)
         enddo
*
  5     continue
*
*       find the jets
*
        call hzjtfind(modjet,rcone,nj,pj)
        if (nj.gt.numjet) then
         write(6,*) 'hz',xxxx,' too many jets found ! '
         return
        endif
*
        nsel=0
*
        do ij=1,nj
*
         if (lp) write(6,'(a,4f9.3)')
     &   ' Jet= ',(real(pj(ij,i)),i=4,7)
*
         ptj=real(pj(ij,3))
*
* pt > 5 GeV deta<2
*
         if (ptj.gt.ptjcut) then
          nsel = nsel + 1
          ipjet(nsel)=ij
         endif
        enddo
*
        if (nsel.eq.2) then
         if (lp) write(6,*) nsel,' jet analysis: jets found ! '
*
         deta=abs(pj(ipjet(1),1)-pj(ipjet(2),1))
*
         if (deta.lt.2.) then
          n2=n2+xw
          if (xsec.eq.0) then
           xsec=2
          endif
          call hfill(120,x,0.,xw)
          call hfill(130,q2,0.,xw)
          thj=real(HZPHMANG(pj(ipjet(1),7),pj(ipjet(1),3)) )*rd
          thj1=thj
          call hfill(150,thj1,0.,xw)
          thj2=real(HZPHMANG(pj(ipjet(2),7),pj(ipjet(2),3)) )*rd
          call hfill(151,thj2,0.,xw)
          call hfill(152,real(pj(ipjet(1),3)),0.,xw)
          call hfill(153,real(pj(ipjet(2),3)),0.,xw)
*
          ptj1=real(pj(ipjet(1),3))
          ptj2=real(pj(ipjet(2),3))
          if (ptj1.gt.ptjcut+2.) then
           n2d=n2d+xw
           call hfill(220,x,0.,xw)
           call hfill(230,q2,0.,xw)
          endif
          if (ptj1+ptj2.gt.13.) then
           n2s=n2s+xw
           call hfill(320,x,0.,xw)
           call hfill(330,q2,0.,xw)
          endif
         endif
        endif
*
        do ihep=1,nhep
         Do i=1,4
          PHEP(i,ihep)= plab(i,ihep)
         enddo
        enddo
*
       endif
*********************************************************************
*
*                      Termination
*
*********************************************************************

      Else if(iflag.eq.3) then

* Termination: The following MUST always be done
* (i) Move to the correct PAW subdirectory
*
       call hcdir('//PAWC/'//xxxx,' ')
*
        write(6,*) 'hz',xxxx,' Total Cross= ',
     &  real(xsec),' nall= ',nall
*
*      normalize the histos
*
        write(6,*) 'hz',xxxx,nall2,' out of which: 2+1 jets= ',n2
*
        call hopera(120,'/E',121,120,1.,1.)
        call hzhinfo(120,n2)
        call hzchisq(-120,120)
        call hopera(130,'/E',131,130,1.,1.)
        call hzhinfo(130,n2)
        call hzchisq(-130,130)
*
        call hopera(220,'/E',121,220,1.,1.)
        call hzhinfo(220,n2d)
        call hzchisq(-220,220)
        call hopera(230,'/E',131,230,1.,1.)
        call hzhinfo(230,n2d)
        call hzchisq(-230,230)
*
        call hopera(320,'/E',121,320,1.,1.)
        call hzhinfo(320,n2s)
        call hzchisq(-320,320)
        call hopera(330,'/E',131,330,1.,1.)
        call hzhinfo(330,n2s)
        call hzchisq(-330,330)
*
        if (nall2.ne.0.) then
         do i=0,3
          call hopera(150+i,'+E',150+i,150+i,1./nall2,0.)
          call hzhinfo(150+i,nall2)
         enddo
*
        endif
*
      endif
*
      RETURN
      END
