	Subroutine HZ98087(iflag)
*****************************************************************************
* Running (1994/95): Ee=27.6GeV, Ep=820GeV, no polarization
*
* Produce (2+1) jet rates vs Q2, Jet profiles etc.
*
*      produced histogramms for data:
*               -4: 2+1 jet rates in 4 bins, stat. errors included
*                   40GeV^2<Q^2<4000GeV^2
*             -104: as above, sys. errors also included
*             -204: data on parton level assuming lepto6.5
*      generated histogramms:
*             cuts: W^2>5000GeV^2
*                   zp-cut: zp>0.1
*                   zp=E(jet(i))*(1-cos(th(jet(i))))/den
*                   den=SUM(j=1,2)[E(jet(j))*(1-cos(th(jet(j))))]
*             evcut1: 40<Q^2<100,en. of scattered e+ >14GeV,
*                                th. of scattered e+: 160<th<173
*             evcut2: Q^2>100,y<0.7,en. of scattered e+ >11GeV,
*                                   th. of scattered e+: 10<th<150
*                                   delta-cut:38<delta<70 [GeV]
*             calculated is the ratio of 2+1 jet events to total
*             ((1+1)+(2+1))jet events.
*             where +1 stands for the proton remnant jet.
*
*      histo-numbers:  1: 1+1 jet events are counted
*                      2: 2+1 events
*                      3: total
*                      4: ratio
*      other histos:  10: Dphi eflow forw.
*                     11: Dphi eflow back.
*                     12: Deta eflow forw.
*                     13: Deta eflow back.
*
*                     20: thjet,min
*                     21: thjet,max
*                     22: zp
*                     23: ptjet1 (transversal jet1 momentum)
*                     24: ptjet2 (transversal jet2 momentum)
*
* Arguments: iflag=1 initialisation
*            iflag=2 filling
*            iflag=3 termination
*
*
*  modified by: Reimer Selle (rselle@rec06.desy.de)
*  Corrected: T. Carli 18.1. routine contained preliminary data
*****************************************************************************
*
	IMPLICIT NONE
*
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"

*
*                                          subroutine-related var.
*                                          -----------------------

      Integer chjet,iflag,ihep
      DATA chjet/5/
      CHARACTER*6 jetf
      CHARACTER*5 xxxx
      Data xxxx/'98087'/
*
      Integer iel,idum,numjet
      Integer ijetb,ijetf
      Real nev
      Data nev/0./
      Real ee,eb
      Real thmax,thmin,thx,thn,th,et,nom,pom
      Real x,y,q2,w2,thel,enel,rd,pi,dpi
      Parameter (pi=3.1415927,rd=180./pi,dpi=2*pi,ee=27.6)
      Logical lp,evcut,evcut1,evcut2,dcut
      Data lp/.false./
      Real thj,thjet,cosjet,ptjet1,ptjet2
*
      Integer njet
      DOUBLE PRECISION coner
      Real qb(5),deta,dphi,eta,phi
      Data qb/40.,100.,300.,700.,4000./
*
      INTEGER i,j,k,ierr,ibeam
*                                            for zp-cut,delta-cut
      Real etmax,etmin,den,zpmax,zpmin,zp
      Real delta,wx
*
      Integer nentry
      Data nentry/0/
*
*********************************************************************
*
*                      data points from H1
*
*********************************************************************
*
*       Variable names:  - ra21: 2+1-jet rate
*                        - rae21: stat. error for ra21
*                        - ras21: stat. and sys errors
*                        - qb (see above): bins in Q^2
      Integer qbin
      Parameter (qbin=4)
*
      Real ra21(qbin), rae21(qbin),ras21(qbin)
      Real pra21(qbin), prae21(qbin),pras21(qbin)
      Data  ra21/33.0e-3,60.0e-3,72.0e-3,82.0e-3/
      Data rae21/4.0e-3,4.0e-3,5.0e-3,8.0e-3/
      Data ras21/8.0e-3,5.0e-3,5.0e-3,8.0e-3/
*
      Data pra21/47.0e-3,73.0e-3,91.0e-3,99.0e-3/
      Data prae21/6.0e-3,5.0e-3,6.0e-3,10.0e-3/
      Data pras21/9.0e-3,8.0e-3,10.0e-3,11.0e-3/
*
*********************************************************************
*
*                      Initialization
*
*********************************************************************

      NENTRY=NENTRY+1
*
      IF (iflag.eq.1) then
*
*
*      Initialisation: The following MUST always be done
*      (i) make subdirectory in PAWC
*       - use the name as the xxxxxx in HZxxxxxx subroutine
*      (i) make subdirectory in o/p file
*
       Call hcdir('//PAWC',' ')
       call hmdir(xxxx,'S')
       Call hcdir('//HISTO',' ')
       call hmdir(xxxx,'S')
*
*                                         book and fill data histos
*
       call hbookb(  -4,'2+1 jet rate',4,qb,0.)
       call hbookb(-104,'2+1 jet rate',4,qb,0.)
       call hbookb(-204,'2+1 jet rate',4,qb,0.)
*
       call hpak (-4,ra21)
       call hpake(-4,rae21)
*                                         include sys. errors
       call hpak (-104,ra21)
       call hpake(-104,ras21)
*
       call hpak (-204,pra21)
       call hpake(-204,pras21)
*                                         book generator histos

       call hbookb(1,'sample 1+1',4,qb,0.)
       call  hcopy(1,2,'sample 2+1')
       call  hcopy(1,3,'total')
       call  hcopy(1,4,'Ratio')

       call hbook1(10,' Dphi eflow forw.',50,-pi,pi,0.)
       call hbook1(11,' Dphi eflow back.',50,-pi,pi,0.)
       call hbook1(12,' Deta eflow forw.',50,-3.,4.,0.)
       call hbook1(13,' Deta eflow back.',50,-3.,4.,0.)

*
       call hbook1(20,'thjet-min ',29,0.,145.,0.)
       call hbook1(21,'thjet-max ',29,0.,145.,0.)
       call hbook1(22,'  zp      ',10,0.,0.5,0.)
       call hbook1(23,' ptjet1    ',20,0.,50.,0.)
       call hbook1(24,' ptjet2    ',20,0.,50.,0.)
       call hbook1(25,' E-Pz      ',20,20.,70.,0.)

*
* print-out name of jet-algo used
*
      CALL HZJTNAME(chjet,jetf)
      WRITE(6,*) '**********************************************'
      WRITE(6,*) '* HZ',xxxx,' called                         *'
C      WRITE(6,*) '* Jet finder selected : ',jetf,'             *'
      WRITE(6,*) '**********************************************'
*



*********************************************************************
*
*                      Event Processing
*
*********************************************************************



      Else if(iflag.eq.2) then

*      Filling: The following MUST always be done
*      (i) move to the correct sub-directory in PAWC
*
       call hcdir('//PAWC/'//xxxx,' ')
*
*                                             Event selection
*
       wx=wtx
       q2=real(HZDISKIN(1))
       x =real(HZDISKIN(2))
       y =real(HZDISKIN(3))
       w2=real(HZDISKIN(4))
*
       if (lp) write(6,*) 'Hz'//xxxx,
     & ' x= ',x,' q2 ',q2,' y= ',y,' w2= ',w2
*
*                                             get electron variables
*
       iel=HZIDELEC(idum)
       if (iel.eq.-1) then
        write(6,*) 'Hz'//xxxx,' electron not found '
        goto 9990
       endif
       enel=real(PHEP(4,iel))
       thel=real(HZPHMANG(PHEP(3,iel),
     &               sqrt(PHEP(1,iel)**2+PHEP(2,iel)**2)))*rd
*
       if (lp) write(6,*) 'Hz'//xxxx,
     & ' enel= ',enel,' thel= ',thel
*
       if (lp) write(6,'(A,4f9.3)') 'elec: e,pz,py,pz= ',
     & phep(4,iel),phep(1,iel),phep(2,iel),phep(3,iel)
*
*                                             kinematic selection
*
       evcut=w2.gt. 5000.
       if(.not.evcut) goto 9990
*                                             cuts for sample 1
       evcut1=q2.gt.40. .and. q2.lt.100.
       evcut1=enel.gt.14..and.evcut1
       evcut1=thel.gt.160. .and. thel.lt.173. .and. evcut1

*                                             cuts for sample 2
       evcut2=q2.gt.100. .and. y.lt.0.7
       evcut2=enel.gt.11..and.evcut2
       evcut2=thel.gt.10. .and. thel.lt.150. .and. evcut2
*
       if (lp) then
        write(6,*) ' evcut1= ',evcut1,' evcut2= ',evcut2
       endif
*
*                                             additional zp-cut later on
*                                             delta-cut for evcut2 later
*                                      'pre-check' for generated events
*
       ierr=HZIBEAM(ibeam,idum)
       if (.not.(ierr.eq.1)) then
        write(6,*) 'HZ'//xxxx,' beams not found ! '
        goto 9990
       endif
*
       eb=real(PHEP(4,idum))
       if (abs(abs(eb)-ee).gt.0.2) then
          if (nentry.lt.10) then
           write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,'  ! ',eb
          elseif (nentry.eq.10) then
           write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,
     &     ' ...last message ! '
          endif
        goto 9990
       endif
*                                                   delta-cut
       delta=0.
       Do 5 ihep=1,nhep
         if (isthep(ihep).ne.1) goto 5
         delta = (PHEP(4,ihep)-PHEP(3,ihep))+delta
 5     continue
*
       call hf1(25,delta,wx)
*
*
C       if (lp) write(6,*) ' delta= ',delta
*
       dcut= delta.gt.38..and.delta.lt.70.
       evcut2=evcut2.and.dcut

       if (.not.(evcut1.or.evcut2)) goto 9990

*	
       coner=0.02
       if (lp) then
        write(6,*) ' ..calling hzjtfind chjet= ',chjet,' ycut= ',coner
       endif
*
       if (lp) write(6,*) ' chjet= ',chjet
       CALL hzjtfind(chjet,CONER,NJET,Jets)
*
       thmin=999.
       thmax=-999.
       thn=999.
       thx=-999.
       etmax=0.
       etmin=0.
       zpmax=0.
       zpmin=0.
       den=0.
       zp=0.
       ptjet1=0.
       ptjet2=0.
*
       numjet=0
*                                         not more than 3 jets!
       if (njet.gt.3) goto 9990

*
       do i=1,njet
        if (lp) then
         write(6,'(a,4(x,f9.3),a,f9.3)') ' e,px,py,pz= ',
     &   (real(jets(i,j)),j=4,7),' M= ',
     &   real(jets(i,4)**2-(jets(i,5)**2+jets(i,6)**2+jets(i,7)**2))
        endif
*
        if (jets(i,4).ne.0.) then
C         thj=real(acos(jets(i,7)/jets(i,4)))
         den=jets(i,5)**2+jets(i,6)**2+jets(i,7)**2
         if (den.gt.0.) then
          thj=real(acos(jets(i,7)/sqrt(den)))
         else
          thj=0.
         endif
C        thj=real(HZPHMANG(JETS(i,7),
C     &               sqrt(JETS(i,5)**2+JETS(i,6)**2)))
        else
         thj=0.
        endif
*

        if (thj*rd.ge.10. .and. thj*rd.le.145.) then
         numjet=numjet+1
         if (thj*rd.gt.thmax) then
          thmax=thj*rd
          thx=thj
          etmax=real(jets(i,4))
C          zpmax=real(jets(i,4))-real(jets(i,7))
          zpmax=etmax*(1.-cos(thj))
          ijetf=i
         endif
         if (thj*rd.lt.thmin) then
          thmin=thj*rd
          thn=thj
          etmin=real(jets(i,4))
C          zpmin=real(jets(i,4))-real(jets(i,7))
          zpmin=etmin*(1.-cos(thj))
          ijetb=i
         endif
*
        endif
       enddo
*
C       zpmax= etmax*(1.-cos(thx))
C       zpmin= etmin*(1.-cos(thn))
*
       den=zpmax+zpmin
       if (den.ne.0.) then
        zpmax=zpmax/den
        zpmin=zpmin/den
       else
C        write(6,*) '***Hz',xxxx,' den= 0 !?!? ',thx*rd,thn*rd
        return
       endif
*
       if (zpmax.lt.zpmin) then
          zp=zpmax
       else
          zp=zpmin
       endif
*
       if (numjet.gt.2) goto 9990
*
       if (lp) then
        write(6,*) ' thmin= ',thmin,' thmax= ',thmax
        write(6,*) ' zp= ',zp,' zpx= ',zpmax,' zpi= ',zpmin,den
       endif
*
       if (zp.lt.0.1) goto 9990
*
       if (lp) then
        write(6,*) ' njet= ',njet,' numjet= ',numjet
       endif
*
       if (numjet.eq.1 ) then
        if (xsec.gt.0) then
        call hf1(1,q2,wx)
        endif
       elseif (numjet.eq.2 ) then
        nev=nev+wtx
C        write(6,*) ' wtx= ',wtx,' nev= ',nev
        call hf1(2,q2,wx)
*
        call hf1(20,thmin,wx)
        call hf1(21,thmax,wx)
        call hf1(22,zp,wx)
*
        ptjet1=real(sqrt(jets(1,5)**2+jets(1,6)**2))
        ptjet2=real(sqrt(jets(2,5)**2+jets(2,6)**2))
*
        call hf1(23,ptjet1,wx)
        call hf1(24,ptjet2,wx)
*
        if (lp) then
         write(6,*) ' thmin= ',thmin,' thmax= ',thmax
         write(6,*) ' zp= ',zp,' ptjet= ',ptjet1,ptjet2
         write(6,*) ' ijetf= ',ijetf,' ijetb= ',ijetb
        endif
*

        Do 10 ihep=1,nhep
         if (isthep(ihep).ne.1.or.ihep.eq.iel) goto 10
*
         th=real(HZPHMANG(PHEP(3,ihep),
     &              sqrt(PHEP(1,ihep)**2+PHEP(2,ihep)**2)))
         et=real(PHEP(4,ihep))*sin(th)
             if (abs(     th*rd).lt.1.e-4 .or.
     &          abs(180.-th*rd).lt.1.e-4 ) then
C               write(6,*) xxxx,' th = ',th
                goto 10

             else
                eta = -log(tan( th/2. ))
             endif

         phi=real(HZPHMANG(PHEP(1,ihep),PHEP(2,ihep)))
*
         deta=real(JETS(ijetb,1))-eta
         dphi=real(JETS(ijetb,2))-phi
*
         call hf1(13,deta,et*wx)
         call hf1(11,dphi,et*wx)
*
         deta=real(JETS(ijetf,1))-eta
         dphi=real(JETS(ijetf,2))-phi
         call hf1(12,deta,et*wx)
         call hf1(10,dphi,et*wx)
*
 10     continue
         endif

9990   continue
*
*********************************************************************
*
*                      Termination
*
*********************************************************************

      Else if(iflag.eq.3) then

*      Termination: The following MUST always be done
*      (i) Move to the correct PAW subdirectory
*
       call hcdir('//PAWC/'//xxxx,' ')

        write(*,*) '**hzth002: twojet=',nev
*                                              Normalization
       call HZHINRM(10,0,nev,1)
       call HZHINRM(11,0,nev,1)
       call HZHINRM(12,0,nev,1)
       call HZHINRM(13,0,nev,1)
*                                  N. for thmax,thmin,zp,ptjet1,2
       pom=nev
       if (pom.ne.0) then
        nom= 1/pom
       else
        nom=1.
       endif
*
       call hopera (20,'+E',20,20,nom,0.)
       call hopera (21,'+E',21,21,nom,0.)
       call hopera (22,'+E',22,22,nom,0.)
       call hopera (23,'+E',23,23,nom,0.)
       call hopera (24,'+E',24,24,nom,0.)
*
*                                  alternative norm for 22 in 220!
*                                  N. for 2+1 jet-ratio
        call hopera(1,'+',2,3,1.,1.)
        call hopera(2,'/E',3,4,1.,1.)
        call hzchisq(-4,4)
*
      ENDIF
*

      RETURN
      END
