*CMZ :  1.02/06 22/03/99  09.37.35  by  Hannes Jung
*CMZ :  1.02/00 06/10/98  23.41.03  by  Tancredi Carli
*-- Author :
      subroutine HZ98143(IFLAG)
****************************************************************************
* Purpose: histograms for the fwd jet analysis
*
* Event selection: y>0.1, 0.0001<x<0.004, E'>11, 160<Th_e<173 -> DIS
*                  Ej>28.7, PTj>3.5, 7<Th_j<20 -> Fwd Jets
*                  0.5<PTj**2/Q2<2      -> Mueller Jets
* jet algo used: CDF with ptmin=2
* Arguments: iflag=1 initialisation
*            iflag=2 filling
*            iflag=3 termination
*
* written by: G. Contreras/Th. Wengler/H. Jung
*
* Purpose: histograms for the fwd pi0 and
*          Charged Particle Analysis by H1
* Plots for fwd-part. paper
* For the fwd jets:
* histos for fwd jets
*  301 dsigma/dx  fwd jet (pt>3.5)
*  302 dsigma/dx  fwd jet (pt>5.0)
*  303 dsigma/dphi fwd jet, highx
*  304 dsigma/dphi fwd jet, lowx
* DATA H1
* -301 dsigma/dx  fwd jet (pt>3.5) (sta err)
* -1301 dsigma/dx  fwd jet (pt>3.5) (tot err)
* -302 dsigma/dx  fwd jet (pt>5.0) (sta err)
* -1302 dsigma/dx  fwd jet (pt>5.0) (tot err)
* -303 dsigma/dphi fwd jet, highx  (sta err)
* -1303 dsigma/dphi fwd jet, highx  (tot err)
* -304 dsigma/dphi fwd jet, lowx  (sta err)
* -1304 dsigma/dphi fwd jet, lowx  (tot err)
*
*
* For the pi0 analysis: 3 plots: 0.01 < x_pi < 0.015, Pt_pi0 > 1GeV
*                                0.015< x_pi < 0.15,  Pt_pi0 > 1GeV
*                                0.01 < x_pi < 0.15,  Pt_pi0 > 2GeV
*
* For the Carged Particles: 1 plot: 0.01 < x_pi < 0.015, Pt_pi0 > 1GeV
*
* Output Histograms:
*
*--------------------------------------------------------------------
* Histogram   Description
*--------------------------------------------------------------------
*                                              Data Histograms
*--------------------------------------------------------------------
* -201 1/N dn_pi/dx_bj, (pi0),0.01<x_pi<0.015, Pt_pi0>1GeV (sta err)
*-1201 1/N dn_pi/dx_bj, (pi0),0.01<x_pi<0.015, Pt_pi0>1GeV  (tot err)
* -202 1/N dn_pi/dx_bj, (pi0),0.015<x_p <0.15, Pt_pi >1GeV  (sta err)
*-1202 1/N dn_pi/dx_bj, (pi0),0.015<x_pi<0.15, Pt_pi0>1GeV  (tot err)
* -203 1/N dn_pi/dx_bj, (pi0),0.01<x_pi<0.15, Pt_pi0>2GeV  (sta err)
*-1203 1/N dn_pi/dx_bj, (pi0),0.01<x_pi<0.15, Pt_pi0>2GeV  (tot err)
* -204 1/N dn_pi/dx_bj, (pi-+pi+)/2,0.01<x_pi<0.015, Pt_pi0>1GeV (sta)
*-1204 1/N dn_pi/dx_bj, (pi-+pi+)/2,0.01<x_pi<0.015, Pt_pi0>1GeV (tot)
* -205 1/N dn_p/dx_bj, Ch. Part.,0.01<x_pi<0.015, Pt_pi0>1GeV  (sta)
*-1205 1/N dn_p/dx_bj, Ch. Part.,0.01<x_pi<0.015, Pt_pi0>1GeV (tot)
*--------------------------------------------------------------------
*                                              Monte Carlo Histograms
*--------------------------------------------------------------------
*  201   1/N dn_pi/dx_bj, (pi0), 0.01 < x_pi < 0.015, Pt_pi0>1GeV
*  202   1/N dn_pi/dx_bj, (pi0), 0.015< x_pi < 0.15,  Pt_pi0>1GeV
*  203   1/N dn_pi/dx_bj, (pi0), 0.01 < x_pi < 0.15,  Pt_pi0>2GeV
*  204   1/N dn_pi/dx_bj, (pi-+pi+)/2, 0.01<x_pi<0.015, Pt_pi0>1GeV
*  205   1/N dn_p/dx_bj, Char. Part., 0.01<x_pi<0.015, Pt_pi0>1GeV
****************************************************************************
      IMPLICIT NONE
#include "hzfunc.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzjetcmn.inc"

*
      Integer numjet
      Parameter (numjet=50)
      Double precision pj(maxhzjets,8)
*
      Integer i,iflag
      Character*5 xxxx
      save xxxx
      Integer nentry
*      Data xxxx/'98143'/,NENTRY/0/
      Data xxxx/'98143'/,NENTRY/0/

      Real Npass
      Real ptpi,thpi,xpi
*
      Real pi,rd
      Integer iel,idum,ibeam
      Parameter (pi=3.1415927,rd=180./pi)
      Real x,y,q2
      Integer nj,ij
*
      Integer ierr
      Integer maxhi,modjet,maxjet
      Parameter (maxhi=100,modjet=2,maxjet=30)
      Real lumnb,lumpb
      Double precision rcone
      Parameter (rcone=1.d0)
***********************************
*
* Define bins for histograms
*
* bin in x
      real bx(7)
      data bx/0.0001,0.0005,0.001,0.0015,0.002,0.003,0.004/
* bin in Delta_phi
      real bdphi(7)
      data bdphi/0.0,30.,60.,90.,120.,150.,180./
* pi0 bins in x_bj
      real bxpi(6)
      data bxpi/0.0002,0.00052,0.00084,0.0012,0.00164,0.00236/
* pi+-/Charged part.  bins in x_bj
      real bxpm(6)
      data bxpm/0.0003,0.0006,0.0009,0.0013,0.0017,0.0025/
*
* Define Variables
*
* Nevt -> number of events
* Njet -> number of jets
* Nfwd -> number of FWD jets
* Nmue -> number of Mueller jets
      real Nevt, Njet, Nfwd, Nmue
      save Nevt, Njet, Nfwd, Nmue
* Ee -> electron energy
* The -> electron theta
* Phe -> electron phi
* Ej ->  jet energy
* Thj -> jet theta
* Phj -> jet phi
* Ptj -> jet pt
      real Ee, The, Phe, Ej, Thj, Phj, Ptj
      real deltaphi,pt2q2
*
* define cut parameters for fwd. jets
*
      real ylow, xlow, xmed, xhigh, eelow, thelow, thehigh
      real ejlow, ptjlow, thjlow, thjhigh, pt2q2low, pt2q2high
      parameter(ylow=0.1)
      parameter(xlow=0.0001)
      parameter(xmed=0.001)
      parameter(xhigh=0.004)
      parameter(eelow=11.)
      parameter(thelow=160.)
      parameter(thehigh=173.)
      parameter(ejlow=28.7)
      parameter(ptjlow=3.5)
      parameter(thjlow=7.)
      parameter(thjhigh=20.)
      parameter(pt2q2low=0.5)
      parameter(pt2q2high=2.)

* define cut parameters  for fwd pi
      real ypilow,xpilow,xpihigh,eepilow,thepilow,thepihigh
      parameter(ypilow=0.1)
      parameter(xpilow=0.0002)
      parameter(xpihigh=0.00236)
      parameter(eepilow=12.)
      parameter(thepilow=156.)
      parameter(thepihigh=173.)
      real xpi1lo,xpi1hi,xpi2lo,xpi2hi,xpi3lo,xpi3hi
      real ptpi3lo,ptpilow,thpilow,thpihigh
      parameter(xpi1lo=0.01)
      parameter(xpi2lo=0.015)
      parameter(xpi3lo=0.01)
      parameter(xpi1hi=0.015)
      parameter(xpi2hi=0.15)
      parameter(xpi3hi=0.15)
      parameter(ptpi3lo=2.0)
      parameter(ptpilow=1.0)
      parameter(thpilow=5.0)
      parameter(thpihigh=25.0)
      logical fjet,fpi
      data fjet/.true./,fpi/.true./
*
* define H1 data
*
* cross section vs x (data, stat error, tot error) pt>3.5
      real csx(6),csxse(6),csxte(6)
      data csx/81.,171.,112.,69.,67.,32./
      data csxse/12.,8.,6.,5.,5.,3./
      data csxte/17.,27.,17.,12.,12.,6./
* cross section vs x Pt>5
      real csx6(6),csx6se(6),csx6te(6)
      data csx6/11.,63.,66.,48.,55.,28./
      data csx6se/8.,5.,5.,4.,5.,3./
      data csx6te/9.,13.,10.,10.,10.,6.0/
* cross section vs Delta_phi high x
      real csph(6), csphse(6), csphte(6)
      data csph/28.,36.,43.,49.,60.,69./
      data csphse/3.,3.,4.,4.,4.,5./
      data csphte/5.,5.,7.,9.,9.,13./
* cross section vs Delta_phi low x
      real cspl(6), csplse(6), csplte(6)
      data cspl/36.,40.,45.,45.,42.,51./
      data csplse/5.,4.,7.,7.,7.,4./
      data csplte/7.,7.,9.,10.,10.,10./
**********************************************************************
* define H1 data for new cuts for fwd-part. paper**************
**********************************************************************
*
* pi0 data 1/N dn_pi/dx_bj for the four dif. lower cutoffs in xpi ----
      real pi0x11(5),pi0xse11(5),pi0xte11(5)
      real pi0x12(5),pi0xse12(5),pi0xte12(5)
      real pi0x13(5),pi0xse13(5),pi0xte13(5)
* data
      data pi0x11/18.62, 10.97, 9.41, 6.06, 2.98/
      data pi0x12/14.86,  7.44, 5.44, 2.84, 1.28/
      data pi0x13/10.81,  6.65, 5.23, 2.24, 1.04/
* stat. error
      data pi0xse11/2.13, 1.52, 1.48, 1.06, 0.56/
      data pi0xse12/2.07, 1.25, 0.92, 0.56, 0.26/
      data pi0xse13/1.71, 1.33, 1.06, 0.50, 0.23/
* total error
      data pi0xte11/4.67, 2.94, 3.73, 1.83, 0.80/
      data pi0xte12/5.33, 2.19, 2.40, 1.37, 0.45/
      data pi0xte13/4.10, 3.12, 2.07, 0.80, 0.38/

* pi+-/2 extrapol. data 1/N dn/dpi for lowest cutoff in xpi ----------
      real pipx1(5),pipxse1(5),pipxte1(5)

* data
      data pipx1/15.59, 15.10, 10.28, 5.12, 2.29/
* stat. error
      data pipxse1/1.99, 2.37, 1.7, 1.1, 0.47/
* total error
      data pipxte1/5.01, 4.97, 3.39, 1.67, 0.92/
* Charged part. data 1/N dn/dpi for lowest cutoff in xpi -------------
      real cpx1(5),cpxse1(5),cpxte1(5)
* data
      data cpx1/48.73, 47.19, 32.14, 16.0, 7.17/
* stat. error
      data cpxse1/6.2, 7.4, 5.3, 3.4, 1.47/
* total error
      data cpxte1/15.6, 15.5, 10.6, 5.3, 2.9/

      save npass

*********************************************************************
*
*                      Initialization
*
*********************************************************************
      NENTRY=NENTRY+1
*
      IF (iflag.eq.1) then
*
         Nevt=0
         Npass=0
*
*
* Initialisation: The following MUST always be done
* (i) make subdirectory in PAWC
* - use the name as the xxxxxx in HZxxxxxx subroutine
* (i) make subdirectory in o/p file
*
         Call hcdir('//PAWC',' ')
         call hmdir(xxxx,'S')
         Call hcdir('//HISTO',' ')
         call hmdir(xxxx,'S')
*
* book histos
*
* histos for events after DIS cuts
         call hbookb(101,'x_bj Dis',6,bx,0.0)
         call hbook1(103,'y Dis',50,0.1,0.8,0.)
         call hbook1(104,'Q2 Dis',50,0.,100.,0.)

* histos for fwd jets
         call hbookb(301,'x_bj fwd jet (pt>3.5)',6,bx,0.0)
         call hbookb(302,'x_bj fwd jet (pt>5.0)',6,bx,0.0)
         call hbookb(303,'Ph1j-phe fwd jet highx',6,bdphi,0.)
         call hbookb(304,'Ph1j-phe fwd jet lowx',6,bdphi,0.)
* histos for fwd pi

         call hbookb(201,'x_bj fwd pi0',5,bxpi,0.0)
         call hbookb(202,'x_bj fwd pi0',5,bxpi,0.0)
         call hbookb(203,'x_bj fwd pi0',5,bxpi,0.0)
         call hbookb(204,'x_bj fwd pi+-',5,bxpm,0.0)
         call hbookb(205,'x_bj ch.part.',5,bxpm,0.0)

* data histos
* divide by 1000 to go from pb to nb
         do i=1,6
            csx(i) = csx(i)/1000.
            csxse(i) = csxse(i)/1000.
            csxte(i) = csxte(i)/1000.
            csx6(i) = csx6(i)/1000.
            csx6se(i) = csx6se(i)/1000.
            csx6te(i) = csx6te(i)/1000.
         enddo
* cross section vs x (data stat error, tot error) pt>3.5
         call hbookb(-301,'H1 cs vs x (stat err)',6,bx,0.)
         call hpak(-301,csx)
         call hpake(-301,csxse)
         call hbookb(-1301,'H1 cs vs x (tot err)',6,bx,0.)
         call hpak(-1301,csx)
         call hpake(-1301,csxte)
* cross section vs x Pt>5
         call hbookb(-302,'H1 cs vs x (stat err,pt>5.)',6,bx,0.)
         call hpak(-302,csx6)
         call hpake(-302,csx6se)
         call hbookb(-1302,'H1 cs vs x (tot err,pt>5.)',6,bx,0.)
         call hpak(-1302,csx6)
         call hpake(-1302,csx6te)
* cross section vs Delta_phi high x
         call hbookb(-303,'H1 cs vs dphi (high x, stat)',6,bdphi,0.)
         call hpak(-303,csph)
         call hpake(-303,csphse)
         call hbookb(-1303,'H1 cs vs dphi (high x, tot)',6,bdphi,0.)
         call hpak(-1303,csph)
         call hpake(-1303,csphte)
* cross section vs Delta_phi low x
         call hbookb(-304,'H1 cs vs dphi (low x, stat)',6,bdphi,0.)
         call hpak(-304,cspl)
         call hpake(-304,csplse)
         call hbookb(-1304,'H1 cs vs dphi (low x, tot)',6,bdphi,0.)
         call hpak(-1304,cspl)
         call hpake(-1304,csplte)
* data for fwd pi
         call hbookb(-201,'H1 cs x_bj fwd pi0 stat',5,bxpi,0.0)
         call hbookb(-1201,'H1 cs x_bj fwd pi0 tot',5,bxpi,0.0)
         call hbookb(-202,'H1 cs x_bj fwd pi0 stat',5,bxpi,0.0)
         call hbookb(-1202,'H1 cs x_bj fwd pi0 tot',5,bxpi,0.0)
         call hbookb(-203,'H1 cs x_bj fwd pi0 stat',5,bxpi,0.0)
         call hbookb(-1203,'H1 cs x_bj fwd pi0 tot',5,bxpi,0.0)
         call hbookb(-204,'H1 cs x_bj fwd pi+- stat',5,bxpm,0.0)
         call hbookb(-1204,'H1 cs x_bj fwd pi+- tot',5,bxpm,0.0)
         call hbookb(-205,'H1 cs x_bj ch.part. stat',5,bxpm,0.0)
         call hbookb(-1205,'H1 cs x_bj ch.part. tot',5,bxpm,0.0)
* -- pi0
         call hpak(-201,pi0x11)
         call hpake(-201,pi0xse11)
         call hpak(-1201,pi0x11)
         call hpake(-1201,pi0xte11)

         call hpak(-202,pi0x12)
         call hpake(-202,pi0xse12)
         call hpak(-1202,pi0x12)
         call hpake(-1202,pi0xte12)

         call hpak(-203,pi0x13)
         call hpake(-203,pi0xse13)
         call hpak(-1203,pi0x13)
         call hpake(-1203,pi0xte13)
* -- pi+-
         call hpak(-204,pipx1)
         call hpake(-204,pipxse1)
         call hpak(-1204,pipx1)
         call hpake(-1204,pipxte1)
* -- charg. part.
         call hpak(-205,cpx1)
         call hpake(-205,cpxse1)
         call hpak(-1205,cpx1)
         call hpake(-1205,cpxte1)


*
*********************************************************************
*
*                      Event Processing
*
*********************************************************************
      Else if(iflag.eq.2) then
*
* Filling: The following MUST always be done
* (i) move to the correct sub-directory in PAWC
*
         call hcdir('//PAWC/'//xxxx,' ')
*
         Nevt = Nevt + wtx
*
         ierr=HZIBEAM(ibeam,idum)

*
* get kinematics
*
         q2=real(HZDISKIN(1))
         x =real(HZDISKIN(2))
         y =real(HZDISKIN(3))
*
* get electron
*
         iel=HZIDELEC(idum)
         if (iel.eq.-1) then
            write(*,*) 'Hz'//xxxx,' electron not found '
            return
         endif
         Ee=real(PHEP(4,iel))
         The=real(HZPHMANG(PHEP(3,iel), sqrt(PHEP(1,iel)**2+PHEP(2,iel)
     +   **2)))*rd
         Phe = HZPHMANG(PHEP(1,iel),PHEP(2,iel))*rd
*
* select DIS events for fwd jets
*
         fjet=.true.
         if (ylow.gt.y) fjet=.false.
         if (xlow.gt.x.or.xhigh.lt.x) fjet=.false.
         if (eelow.gt.Ee.or.thelow.gt.The.
     +      or.thehigh.lt.The) fjet=.false.

* select DIS events for fwd pi
         fpi=.true.
         if (x.lt.xpilow) fpi=.false.
         if (x.gt.xpihigh) fpi=.false.
         if (y.lt.ypilow) fpi=.false.
         if (Ee.lt.Eepilow.or.The.lt.Thepilow. or.The.gt.Thepihigh)
     +   fpi=.false.
*
* fill Dis histos
*
         if(fjet.or.fpi) then
            call hfill(101,x,0.,wtx)
c            call hfill(102,x,0.,wtx)
            call hfill(103,y,0.,wtx)
            call hfill(104,q2,0.,wtx)
         endif
         
c         call hzlihep(0)
         
         if(fjet) then
*
* find jets
*
            call hzjtfind(modjet,rcone,nj,pj)
*
            if (nj.le.0) return
            if (nj.gt.numjet) then
               write(6,*) 'hz',xxxx,' too many jets found ! '
               return
            endif
            Njet=0
            Nfwd = 0
            Nmue = 0
*
* look for fwd jets, it is assumed that in pj they are ordered in pt
            do 10   ij=1,nj
*
               ej=real(pj(ij,4))
               ptj=real(pj(ij,3))
               thj=real(HZPHMANG(pj(ij,7),pj(ij,3)) )*rd
               Phj =real(HZPHMANG(pj(ij,5),pj(ij,6)))*rd
*
               if (ptj.lt.ptjlow) goto 10
               Njet=Njet+1
               if ( (thj.gt.thjlow.and.thj.lt.thjhigh) .and.
     +         (ej.gt.ejlow) ) then
*     you got a fwd jet!
                  Nfwd = Nfwd + 1

* mueller jets
                  pt2q2=Ptj**2/q2
                  if (pt2q2.gt.pt2q2low.and.pt2q2.lt.pt2q2high) then
                     Nmue = Nmue+1
                     if (Nmue.eq.1) then
                        call hfill(301,x,0.,wtx)
                        deltaphi=abs(Phj-Phe)
                        if(deltaphi.gt.180.) deltaphi=-deltaphi+360.
                        if (x.lt.xmed) call hfill(303,deltaphi,0.,wtx)
                        if (x.gt.xmed) call hfill(304,deltaphi,0.,wtx)
                        if (Ptj.gt.5.) then
                           call hfill(302,x,0.,wtx)
                        endif
                     endif
                  endif
               endif
   10       continue
         endif
         if(fpi) then
* count number of events passing the DIS cuts
            Npass=Npass+WTX

* find pi0s, pi+- and Charg. Part.  and fill histos
            do i=1,nhep
               ptpi=sqrt(PHEP(1,i)**2+PHEP(2,i)**2)
               thpi=real(HZPHMANG(PHEP(3,i), sqrt(PHEP(1,i)**2+PHEP(2,
     +         i)**2)))*rd
               xpi=real(PHEP(4,i))/820.0

               if(ptpi.gt.ptpilow.and. thpi.gt.thpilow.and. thpi.lt.thp
     +         ihigh) then
* pi0's ---------------------------------------------------
                  if(iabs(idhep(i)).eq.111) then
                     if(xpi.gt.xpi1lo.and.xpi.lt.xpi1hi) call hf1(201,
     +               x,wtx)
                     if(xpi.gt.xpi2lo.and.xpi.lt.xpi2hi) call hf1(202,
     +               x,wtx)
                     if(xpi.gt.xpi3lo.and.xpi.lt.xpi3hi.and. ptpi.gt.pt
     +               pi3lo) call hf1(203,x,wtx)
                  endif

* pi+- ----------------------------------------------------
                  if(iabs(idhep(i)).eq.211) then
                     if(xpi.gt.xpi1lo.and.xpi.lt.xpi1hi) call hf1(204,
     +               x,wtx)
                  endif

* charged particles ---------------------------------------
                  if((isthep(i).eq.1).and.(ihchrg(i).ne.0)) then
                     if(xpi.gt.xpi1lo.and.xpi.lt.xpi1hi) call hf1(205,
     +               x,wtx)
                  endif
               endif

            enddo
         endif

*
*********************************************************************
*
*                      Termination
*
*********************************************************************

      Else if(iflag.eq.3) then

* Termination: The following MUST always be done
* (i) Move to the correct PAW subdirectory
*
         call hcdir('//PAWC/'//xxxx,' ')
*
         lumnb=999999.
         lumpb=999999.
         if (xsec.ne.0.) then
            lumnb=1000.*real(Nevt)/real(xsec)
            lumpb=real(Nevt)/real(xsec)
            print *,'Number of events procesed:',Nevt
            print *,'Total cross section:',xsec
            print *,'Equivalent luminosity [nb] :',lumnb
         else
            write(6,*) 'hz',xxxx,' xsec=0 ! '
         endif
*
*      normalize the histos
*
* jets
         call hopera(101,'+E',101,101,1./lumnb,0.)
         call hopera(103,'+E',103,103,1./lumnb,0.)
         call hopera(104,'+E',104,104,1./lumnb,0.)

         call hopera(301,'+E',301,301,1./lumnb,0.)
         call hopera(302,'+E',302,302,1./lumnb,0.)
         call hopera(303,'+E',303,303,1./lumpb,0.)
         call hopera(304,'+E',304,304,1./lumpb,0.)
         call hzhinrm(101, 0, 1. , 1 )
         call hzhinrm(103, 0, 1. , 1 )
         call hzhinrm(104, 0, 1. , 1 )
         call hzhinrm(301, 0, 1. , 1 )
         call hzhinrm(302, 0, 1. , 1 )
         call hzhinrm(303, 0, 1. , 1 )
         call hzhinrm(304, 0, 1. , 1 )
         call hzhinrm(-301, 0, 1. , 1 )
         call hzhinrm(-302, 0, 1. , 1 )
         call hzhinrm(-303, 0, 1. , 1 )
         call hzhinrm(-304, 0, 1. , 1 )
         call hzhinrm(-1301, 0, 1. , 1 )
         call hzhinrm(-1302, 0, 1. , 1 )
         call hzhinrm(-1303, 0, 1. , 1 )
         call hzhinrm(-1304, 0, 1. , 1 )
* pi0,pi+-,charg.part. histograms
         do i=201,203
            call hzhinrm(i, 0,real(Npass) , 1 )
         enddo
         call hzhinrm(204, 0,real(2*Npass) , 1 )
         call hzhinrm(205, 0,real(Npass) , 1 )

      endif
*
      RETURN
      END
