	Subroutine HZ98162(iflag)
*****************************************************************************
*       This photoproduction routine has to be run twice with the
c       following
*       code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the RESOLVED component run.
*       +0 for both together
*       for all three phases (iflag=1,2,3)
*       
*       +10000 to run on parton showers instead of final state particles
c       .
*       
*       iflag = 1,2 or 3 depending on whether it is the initialization,
*       processing or termination phase (respectively)
*       
*       The default jetfinder is KTCLUS
*       To change jetfinder use these values to call the initialisation
c       phase.
*       1 + 10 * (jetfinder number)
*       
*       Monte Carlo histograms produced:
*       id 160-164 are the number of unweighted direct events which pass
*       the cuts
*       id 260-264 are the number of unweighted resolved events which pass
*       the cuts
*       id 60-64 are the cross section in nb.
*       In fact only the mass cross section taken directly from the
*       paper. The other cross sections are given initially area
*       normalised to
*       one. In this SR they are multiplied by 0.162nb, the total 3 jet
*       cross section. There is an additional scale uncertainty on this
*       number of +0.032 - 0.025nb which is not included in this SR.
*       
*       These graphs are 
*       meaningless unless Xsec and Ntot are set before
*       calling the termination routine.
*       (Xsec - total cross section returned by MC)
*       (Ntot - number of events passed to this routine)
*       
*       Data graphs produced:
*       id -60 to -64 are measured data, corresponding to MC graphs 60
c       -64.
*       The errors are the quadrative addition of systematic and
c       statistical
*       errors, with the  largest value (up or down) taken as error.
c       This is
*       only an approximation to the asymetric errors of published
c       graphs.
*       
*       Cuts : 	Q2 < 1 GeV^2           ^  
*       0.2 < y < 0.8          ^  
*       And at least two jets with
*       E_T^jet > 6 GeV and    ^  
*       & third with  E_T^jet > 5 GeV
*       eta^JET < |2.4|        ^  
*       In addition   M345 > 50 GeV^2        ^  
*       X_3 < 0.95             ^  
*       |cos(theta_3)| < 0.8   ^  
*       
*       Recommended value for Ptmin is 6.5 GeV (to be set in driver
c       routine)
*       
*       Author : Eileen Heaphy
*       
*****************************************************************************
*       
*       HERA Tuning defined commons
	
	implicit none
        
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
	character*6 jetf

c       Total cross section in nb.
	real total_3j_xsec
	parameter(total_3j_xsec=0.162)

	integer NJET, counter(0:2,2)
	SAVE COUNTER
	double precision  CONER
	SAVE CONER
	double precision ybj,q2
	double precision P1_dbl(4),P2_dbl(4)
	double precision P1_lab(4),P2_lab(4)
	double precision mhxsec(2),mhntot(2) 
	double precision xboost,yboost,zboost
	double precision Pnew_dbl(4),P3new_dbl(4),P4new_dbl(4)
     &     ,P5new_dbl(4) 
	double precision ptot_act(4),p3_act(4),p4_act(4),p5_act(4)
	real dir_xsec, res_xsec, tot_xsec

	integer cut_test(2)
	integer loop,loop2,count, i, j, k
	real    rap,Et(3), Mass
	real    ybjcut(2),q2cut(2),Masscut
	real    rapcut(2),Etcut(2),X3_cut,cos_t3_cut(2)
	save    ybjcut,q2cut,Masscut,rapcut,Etcut,X3_cut,cos_t3_cut
	
	real    Ptest(5),Etest(5),Mtest(5), test_value(5)
	real    Ptot(4), P3(4),P4(4),P5(4),Imass, Mom2_tot
	real    E2, E, Pnew(4),P3new(4),P4new(4),P5new(4)
	real    P3lab(4),P4lab(4),P5lab(4),test(4)
	real    X3,X4,X5 ,Pav(4),P1(4),P2(4)
	real    cross1(4),cross2(4),dot1,dot2, dot3,dot4,dot5,dot6
	real    cos_psi3,cos_theta3, sum(3), psi3
	real    testy(4)


C       *** Data for Invariant Mass
	real xbin0(10)
	real bin0_err(9)
	real mass_cr(9)
	real stat_err(9)	! statistical error
	real x0_sys_up(9)	! systematic err +
	real x0_sys_dn(9)	! systematic err -
	real x0_es_up(9)	! escale systematic err +
	real x0_es_dn(9)	! escale systematic err -
	real err0(9)

c       *** cos_theta_3
	real xbin1(17)
	real bin1_err(16)
	real cos_t3_cr(16)
	real stat1(16)
	real sys1_up(16), es1_up(16)
	real sys1_dn(16), es1_dn(16)
	real err1(16)

c       *** psi_3
	real xbin2(17)
	real bin2_err(16)
	real psi3_cr(16)
	real stat2(16)
	real sys2_up(16),es2_up(16)
	real sys2_dn(16),es2_dn(16)
	real err2(16)

c       *** X3
	real xbin3(8)
	real bin3_err(7)
	real X3_cr(7)
	real stat3(7)
	real sys3_up(7),es3_up(7)
	real sys3_dn(7),es3_dn(7)
	real err3(7)

c       *** X4
	real xbin4(11)
	real bin4_err(10)
	real X4_cr(10)
	real stat4(10)
	real sys4_up(10),es4_up(10)
	real sys4_dn(10),es4_dn(10)
	real err4(10)

	data xbin0/50, 55, 60, 65, 72, 80, 90, 100, 120, 160/
	data mass_cr/12.57, 6.855, 4.733, 2.455, 1.575, 0.535, 0.305,
     &    0.158, 0.023/
	data stat_err/0.535, 0.364, 0.325, 0.176, 0.142, 0.074,
     &    0.055, 0.031, 0.011/
	data x0_sys_up/1.566, 0.443, 0.884, 0.18, 0.151, 0.114, 0.031,
     &    0.009, 0.005/
	data x0_sys_dn/0.511, 0.244, 0.219, 0.072, 0.163, 0.091, 0.027, 
     &    0.056, 0.008/   
	data x0_es_up/2.330, 1.222, 0.649, 0.805, 0.406, 0.115, 0.113,
     &    0.075, 0.000/
	data x0_es_dn/1.686, 0.987, 0.913, 0.361, 0.308, 0.084, 0.061, 
     &    0.038, 0.006/   

	data xbin1/-0.8, -0.7, -0.6, -0.5, -0.4, -0.3, -0.2, -0.1, 0.0,
     &    0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8/	
	data cos_t3_cr/197.2, 141.3, 95.31, 86.78, 69.50, 45.44, 59.90, 
     &    49.90, 59.74, 58.56, 86.29, 73.62, 85.68, 119.5,
     &    159.9, 242.2/
	data stat1/13.0, 11.7, 9.17, 9.14, 8.36,
     &    5.90    , 7.89, 6.38, 7.63, 
     &    7.79    , 10.3, 8.45, 9.41, 
     &    12.0    , 12.9, 17.1 /
	data sys1_up/ 34.9, 8.57, 21.2, 5.28, 9.83, 23.2, 9.99, 21.5
     &    , 14.9, 4.36, 4.58, 23.8, 8.91, 13.2, 23.9, 35.4 /
	data sys1_dn/ 7.49, 17.8, 8.64, 1.49, 7.65, 2.65, 5.15, 
     &    2.20    , 7.70, 15.2, 8.04, 6.20, 4.23, 27.4, 8.07, 41.5
     &    /       
	data es1_up/ 46.6, 19.6, 15.9, 10.2, 13.2, 12.7, 15.3, 11.6
     &    , 13.5, 13.4, 21.6, 13.1, 19.3, 22.6, 28.9, 44.9 /
	data es1_dn/ 26.8, 25.9, 14.4, 8.56, 11.9, 6.86, 12.5, 
     &    5.38        , 9.19, 9.40, 15.5, 10.0, 14.4, 23.4, 17.4, 39
     &    .8/       
c	data cos_t3_cr/1.209, 0.866, 0.584, 0.532, 0.426, 0.279, 0.367, 
c	1    0.306, 0.366, 0.359, 0.529, 0.451, 0.525, 0.732,
c	2    0.981, 1.485/
c	data stat1/0.08, 0.072, 0.056, 0.056, 0.051,
c	1    0.036, 0.048, 0.039, 0.047, 
c	2    0.048, 0.063, 0.052, 0.058, 
c	3    0.074, 0.079, 0.105 /
c	data sys1_up/0.111, 0.02, 0.075, 0.05, 0.022, 0.119, 0.456, 0
c	4    .103,0.057, 0.022, 0.034, 0.117, 0.035, 0.081, 0.076, 0.105
c	1    /
c	data sys1_dn/0.044, 0.171, 0.041, 0.045, 0.039, 0.019, 0.05, 0
c	2    .015,0.039, 0.112, 0.091, 0.038, 0.031, 0.214, 0.024, 0.214
c	1    /       

	data xbin2/0.0, 0.2, 0.4, 0.6, 0.8, 1.0, 1.2, 1.4, 1.6, 1.8,
     &    2.0, 2.2, 2.4, 2.6, 2.8, 3.0, 3.2/
	data psi3_cr/48.33, 54.53, 54.63, 59.31, 50.11, 35.03, 35.75,
     &    41.60    , 34.66, 48.66, 52.36, 55.42, 60.45, 67.48,
     &    72.05    , 58.51/ 
	data stat2/5.33, 5.78, 5.73, 5.07,
     &    4.06 ,
     &    4.17    , 4.17, 4.72, 4.04, 
     &    5.04    , 5.21, 5.15, 5.57,
     &    6.01    , 6.49, 5.40/
	data sys2_up/10.7, 23.5, 11.1, 9.21, 8.35, 4.02, 4.14, 
     &    6.13    ,8.04, 6.45, 2.68, 5.57, 6.44, 8.30, 4.18,
     &    10.6    /
	data sys2_dn/3.82, 2.71, 5.58, 2.70, 2.95, 0.83, 6.33, 
     &    6.12    , 4.59, 5.61, 7.52, 3.55, 4.01, 1.38, 5.70,
     &    9.37    /
	data es2_up/8.13, 10.2, 15.0, 10.3, 10.0, 8.02, 7.24, 
     &    9.34    , 6.84, 8.13, 8.87, 10.6, 11.1, 16.6, 14.0,
     &    11.6    /
	data es2_dn/8.86, 6.41, 10.8, 9.02, 6.25, 6.02, 6.29, 
     &    5.40    , 3.66, 7.92, 7.05, 8.75, 9.30, 10.4, 11.0,
     &    9.31    /


	data xbin3/0.67, 0.71, 0.75, 0.79, 0.83, 0.87, 0.91, 0.95/
	data X3_cr/74.26, 207.0, 328.0, 456.1, 736.5, 1017.0, 1315.0/
	data stat3/13.0, 22.1, 26.5, 32.3, 42.9, 52.5, 62.3/
	data sys3_up/14.4, 27.3, 25.4, 58.3, 83.2, 86.0, 58.6/
	data sys3_dn/14.4, 11.4, 23.4, 38.0, 55.7, 64.6, 50.7/
	data es3_up/19.2, 42.6, 84.3, 53.2, 136., 229., 266./
	data es3_dn/13.3, 33.0, 58.0, 54.2, 106., 165., 203./


	data xbin4/0.5, 0.55, 0.6, 0.65, 0.7, 0.75, 0.8, 0.85,
     &    0.9, 0.95, 1.0/
	data X4_cr/43.56, 268.5, 498.5, 637.9, 596.2, 480.5, 484.0,
     &    270.5    , 17.69, 0.0/
	data stat4/11.9, 24.3, 32.1, 34.9, 35.3, 30.8, 35.6,
     &    30.3    , 9.55, 0.00/
	data sys4_up/6.72, 51.2, 68.2, 78.6, 47.8, 120., 22.0,
     &    42.7    , 15.9, 0.00/
	data sys4_dn/5.25, 9.98, 27.6, 30.3, 57.5, 52.9, 81.8,
     &    32.2    , 4.65, 0.00/
	data es4_up/11.0, 45.8, 96.4, 134., 120., 93.6, 98.9,
     &    52.1    , 2.32, 0.00/
	data es4_dn/3.75, 41.6, 70.8, 92.1, 104., 61.3, 87.5,
     &    46.7    , 2.38, 0.00/

	integer chjet, iproc, iflag
	save chjet

C       --- same jet finder as in paper -- KTCLUS
	data chjet/3/

	real pttemp(12)
	integer ivec,jvec

*       Set this to the hep-ex number.
*       [For alternative subroutine naming conventions, see the manual.]
	Character*6 xxxx
	DATA xxxx /'d98162'/

	real lumnb

	LOGICAL parton_showers

	parton_showers=.FALSE.

      
*       Determine the process type.
	iproc=(mod(iflag,10000)/1000)
	if ((iproc.lt.0).or.(iproc.gt.2)) then
	   write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
	   return
	endif
	
*       --- choose jet finder
	IF ((MOD(iflag,100)/10.gt.0).and.MOD(iflag,10).eq.1) THEN
	   chjet=INT(MOD(iflag,100)/10)
	ENDIF
	
*       Decide whether we are running on parton showers or final state particles.
	IF (MOD(iflag,100000)/10000.gt.0) THEN
	   parton_showers=.TRUE.
*       use a different paw directory for parton showers.
	   xxxx = 'PS'//xxxx
	   chjet=-1*ABS(chjet)
	ENDIF
	
	IF (MOD(iflag,10).eq.1) then
	   
C       *** Initialisation run ***
C       *** Create directories ***

	   if (iproc.le.1) then
	      CALL HCDIR('//HISTO',' ')
	      IF (parton_showers) THEN
		 CALL HMDIR('ps'//xxxx,'S')
		 CALL HCDIR('//PAWC',' ')
		 CALL HMDIR('ps'//xxxx,'S')
	      ELSE
		 CALL HMDIR(xxxx,'S')
		 CALL HCDIR('//PAWC',' ')
		 CALL HMDIR(xxxx,'S')
	      ENDIF

c       ---- Book Histograms for data

	      call hbookb(-60,'data m345',9 ,xbin0 ,0.)
	      call hbookb(-61,'data cos_theta3',16,xbin1,0.)
	      call hbookb(-62,'data psi3',16,xbin2,0.)
	      call hbookb(-63,'data X3',7,xbin3,0.)
	      call hbookb(-64,'data X4',10,xbin4,0.)


c       Create histograms for systematic + statistical errors
c       Also multiply to put it in pb. 
	      do i=1,9
		 x0_sys_up(i) = sqrt(x0_sys_up(i)**2+x0_es_up(i)**2)
		 x0_sys_dn(i) = sqrt(x0_sys_dn(i)**2+x0_es_dn(i)**2)
		 err0(i) = 0.001*sqrt(stat_err(i)**2
     &  	      +0.25* (x0_sys_up(i)+x0_sys_dn(i))**2 )
		 mass_cr(i) = 0.001*( mass_cr(i)+ ( x0_sys_up(i)
     &  	      -x0_sys_dn(i) )/2.0)
	      enddo
	      
	      do i=1,16
		 sys1_up(i) = sqrt(sys1_up(i)**2+es1_up(i)**2)
		 sys1_dn(i) = sqrt(sys1_dn(i)**2+es1_dn(i)**2)
		 err1(i) = 0.001*sqrt(stat1(i)**2
     &  	      +0.25* (sys1_up(i)+sys1_dn(i))**2 )
		 cos_t3_cr(i) = 0.001*( cos_t3_cr(i)+ ( sys1_up(i)
     &  	      -sys1_dn(i) )/2.0)
	      enddo
	      
	      do i=1,16
		 sys2_up(i) = sqrt(sys2_up(i)**2+es2_up(i)**2)
		 sys2_dn(i) = sqrt(sys2_dn(i)**2+es2_dn(i)**2)
		 err2(i) = 0.001*sqrt(stat2(i)**2
     &  	      +0.25* (sys2_up(i)+sys2_dn(i))**2 )
		 psi3_cr(i) = 0.001*( psi3_cr(i)+ ( sys2_up(i)
     &  	      -sys2_dn(i) )/2.0)

	      enddo
	      
	      do i=1,7
		 sys3_up(i) = sqrt(sys3_up(i)**2+es3_up(i)**2)
		 sys3_dn(i) = sqrt(sys3_dn(i)**2+es3_dn(i)**2)
		 err3(i) = 0.001*sqrt(stat3(i)**2
     &  	      +0.25* (sys3_up(i)+sys3_dn(i))**2 )
		 x3_cr(i) = 0.001*( x3_cr(i)+ ( sys3_up(i)
     &  	      -sys3_dn(i) )/2.0)
	      enddo
	      
	      do i=1,10
		 sys4_up(i) = sqrt(sys4_up(i)**2+es4_up(i)**2)
		 sys4_dn(i) = sqrt(sys4_dn(i)**2+es4_dn(i)**2)
		 err4(i) = 0.001*sqrt(stat4(i)**2
     &  	      +0.25* (sys4_up(i)+sys4_dn(i))**2 )
		 x4_cr(i) = 0.001*( x4_cr(i)+ ( sys4_up(i)
     &  	      -sys4_dn(i) )/2.0)
	      enddo
	      
c       ---- Fill Histograms with data -------
	      
	      call hpak(-60,mass_cr)
	      call hpake(-60,err0)
	      
	      call hpak(-61,cos_t3_cr)
	      call hpake(-61,err1)
	      
	      call hpak(-62,psi3_cr)
	      call hpake(-62,err2)
	      
	      call hpak(-63,X3_cr)
	      call hpake(-63,err3)
	      
	      call hpak(-64,X4_cr)
	      call hpake(-64,err4)
	      
	   endif

	   IF (parton_showers) THEN
	      CALL HCDIR('//HISTO/ps'//xxxx,' ')
	      CALL HCDIR('//PAWC/ps'//xxxx,' ')
	   ELSE
	      CALL HCDIR('//HISTO/'//xxxx,' ')
	      CALL HCDIR('//PAWC/'//xxxx,' ')
	   ENDIF
	   
	   call hbookb(100*iproc+60,'Invariant Mass',9,xbin0,0.)
	   call hbookb(100*iproc+61,'Cos_theta3',16,xbin1,0.)
	   call hbookb(100*iproc+62,'psi3',16,xbin2,0.)
	   call hbookb(100*iproc+63,'X3',7,xbin3,0.)
	   call hbookb(100*iproc+64,'X4',10,xbin4,0.)

	   CALL HIDOPT(0,'stat')
	   CALL HBARX(0)

	   
c       -CUTS--------
	   ybjcut(1)=0.2
	   ybjcut(2)=0.8
	   
	   q2cut(1)=0.
	   q2cut(2)=1
	   
	   rapcut(1)= -2.4
	   rapcut(2)=2.4
	   
	   Etcut(1)=5.0
	   Etcut(2)=6.0
	   
	   Masscut=50
	   
	   X3_cut=0.95
	   
	   cos_t3_cut(1)=-0.8
	   cos_t3_cut(2)= 0.8
	   
	   if (iproc.eq.1) then
	      counter(1,1) = 0
	      counter(1,2) = 0
	   else
	      counter(2,1) = 0
	      counter(2,2) = 0
	   endif

C       --- standard cone radius is 1.0
	   CALL HZJETRAD(2,CONER)
	   if (CONER.lt.0.0) then
	      CONER=1.0
	   endif
	   CALL HZJTNAME(chjet,jetf)
	   WRITE(HZUNIT,*)
     &       '**********************************************'
	   WRITE(HZUNIT,*)
     &       '* HZ98162 called, histograms will be output. *'
	   WRITE(HZUNIT,*)
     &       '*     Jet finder selected : ',jetf,'           *'
	   WRITE(HZUNIT,1001)
     &       '*        with cone radius : ',CONER,'   *'
	   WRITE(HZUNIT,*)
     &       '**********************************************'
	   
 1001	   FORMAT(A29,F5.2,A13)
	   
	else if (MOD(iflag,10).eq.2) then ! 
C       *** Filling runs ***
	   
C       *** Change directory to our graphs ***
	   IF (parton_showers) THEN              
              CALL HCDIR('//PAWC/ps'//xxxx,' ')
           ELSE
              CALL HCDIR('//PAWC/'//xxxx,' ')
           ENDIF
	   
	   IF (gen(1:3).EQ.'HO1') THEN
	      
	      DO j=0,4
		 CALL HZSNLO(60+j,260+j,160+j)
	      ENDDO
	      iproc = 1
	   	      
	   ENDIF


c       *** get jets  
	   q2=hzphokin(1)
	   if (q2.eq.-1) then
	      PRINT*,'HZ98162: HZPHOKIN returns error for Q2'
	   elseif (q2.lt.0) then
	      q2=abs(q2)
	   endif
	   ybj=hzphokin(3)
	   if (ybj.eq.-1) then
	      PRINT*,'HZ98162: HZPHOKIN returns error for ybj'
	   endif
	   
	   CALL hzjtfind(chjet,CONER,NJET,Jets)
	   IF (NJET.LT.3) THEN
	      counter(iproc,2) = counter(iproc,2) + 1
	      RETURN
	   ENDIF
	   
	   DO i=1,3
	      et(i)=0.0
	   ENDDO

c       Take the three highest ET jets in the visible rapidity range.
	   i=0
	   do loop=1,njet
	      IF ( jets(loop,1).gt.rapcut(1).and.jets(loop,1).lt
     &  	   .rapcut(2)) THEN
		 i=i+1
		 IF (i.EQ.1) THEN
		    P3lab(4) = real(Jets(loop,4))
		    ET(i)=jets(loop,3)		    
		    DO j=1,3
		       P3lab(j) = real(Jets(loop,j+4))
		    ENDDO
		 ELSE IF (i.EQ.2) THEN
		    P4lab(4) = real(Jets(loop,4))
		    ET(i)=jets(loop,3)
		    DO j=1,3
		       P4lab(j) = real(Jets(loop,j+4))
		    ENDDO
		 ELSE IF (i.EQ.3) THEN
		    P5lab(4) = real(Jets(loop,4))
		    ET(i)=jets(loop,3)
		    DO j=1,3
		       P5lab(j) = real(Jets(loop,j+4))
		    ENDDO
		 ENDIF
	      ENDIF
	   ENDDO

c       Check the ET
	   IF (et(2).lt.etcut(2).or.et(3).lt.etcut(1)) THEN
	      counter(iproc,2) = counter(iproc,2) + 1
	      RETURN
	   ENDIF

	   do i=1,4
	      Ptot(i) = P3lab(i) + P4lab(i) + P5lab(i)
	   enddo
	   
	   Mom2_tot = 0
	   
	   do i = 1,3
	      Mom2_tot = Mom2_tot + Ptot(i)*Ptot(i)
	   enddo
	   
	   E2 = Ptot(4)*Ptot(4)
	   E  = sqrt(E2)
	   Mass = sqrt(E2 - Mom2_tot)
	   
C**********DO BOOST**into cms frame for 3-jets****
	   
	   
	   xboost = -1.0*dble(Ptot(1)) / dble(E)
	   yboost = -1.0*dble(Ptot(2)) / dble(E)
	   zboost = -1.0*dble(Ptot(3)) / dble(E)
	   
	   do i=1,4
	      p3_act(i) = dble(P3lab(i))
	      p4_act(i) = dble(P4lab(i))
	      p5_act(i) = dble(P5lab(i))
	      ptot_act(i) = dble(Ptot(i))
	   enddo
	   
	   CALL HZBOOST(xboost,yboost, zboost,p3_act,P3new_dbl)
	   CALL HZBOOST(xboost,yboost, zboost,p4_act,P4new_dbl)
	   CALL HZBOOST(xboost,yboost, zboost,p5_act,P5new_dbl)
	   CALL HZBOOST(xboost,yboost, zboost,ptot_act,Pnew_dbl)
	   
	   do i=1,4
	      P3new(i) = real(P3new_dbl(i))
	      P4new(i) = real(P4new_dbl(i))
	      P5new(i) = real(P5new_dbl(i))
	      Pnew(i)  = real(Pnew_dbl(i))
	   enddo
	   
c****   swap so E3 > E4 > E5 ***
*******************************
	   
	   If (P3new(4).lt.P4new(4)) then
	      do i=1,4
		 testy(i) = P3new(i)
		 P3new(i) = P4new(i) 
		 P4new(i) = testy(i)
	      enddo
	   endif
	   
	   If (P4new(4).lt.P5new(4)) then
	      do i=1,4
		 testy(i) = P4new(i)
		 P4new(i) = P5new(i) 
		 P5new(i) = testy(i)
	      enddo
	   endif
	   
	   If (P3new(4).lt.P4new(4)) then
	      do i=1,4
		 testy(i) = P3new(i)
		 P3new(i) = P4new(i) 
		 P4new(i) = testy(i)
	      enddo
	   endif
	   
	   do i=1,4
	      P3(i) = P3new(i)
	      P4(i) = P4new(i)
	      P5(i) = P5new(i)
	   enddo
	   
********************************
******  DALITZ variables X3,X4, X5
	   X3 = (2 * P3(4)) /Mass
	   X4 = (2 * P4(4)) /Mass
	   X5 = (2 * P5(4)) /Mass
	   
******* Pav ********= P1 - P2*************
	   P1_lab(1) = 0 
	   P1_lab(2) = 0
	   P1_lab(3) = 820.0
	   P1_lab(4) = 820.0
	   
	   
	   P2_lab(1) = 0
	   P2_lab(2) = 0
	   P2_lab(3) = -27.52
	   P2_lab(4) = 27.52
	   
	   
	   CALL HZBOOST(xboost,yboost,zboost, P1_lab,P1_dbl)
	   CALL HZBOOST(xboost,yboost,zboost, P2_lab,P2_dbl)
	   
	   do i=1,4
	      P1(i) = real(P1_dbl(i))
	      P2(i) = real(P2_dbl(i))
	   enddo
	   
	   do i=1,3
	      Pav(i) = P1(i) - P2(i)
	   enddo
	   
**********calculate cos theta 3  ***
	   dot4 = 0
	   dot5 = 0
	   dot6 = 0
	   do i=1,3
	      dot4 = dot4 + Pav(i)*P3(i)
	      dot5 = dot5 + Pav(i)*Pav(i)
	      dot6 = dot6 + P3(i)*P3(i)
	   enddo
	   
	   cos_theta3 = dot4 / ( sqrt(dot5)*sqrt(dot6) )
	   
**********cos psi 3 = cos_psi3
	   
	   
	   call cross(P3,Pav,cross1)
	   call cross(P4,P5,cross2)
	   
	   dot1 = 0
	   dot2 = 0
	   dot3 = 0
	   
	   do i=1,3
	      dot1 = dot1 + cross1(i)*cross1(i)
	      dot2 = dot2 + cross2(i)*cross2(i)
	      dot3 = dot3 + cross1(i)*cross2(i)
	   enddo
	   
	   
	   cos_psi3 = dot3 / (sqrt(dot1) * sqrt(dot2) )
	   psi3     = acos( cos_psi3)
	   
*****   Implement cuts ******************
	   
	   cut_test(1) = 0
	   cut_test(2) = 0
	   
cccc    if successful event cut_test(1) = 2 or 3
	   
	   IF ( ybj.ge.ybjcut(1).and.ybj.le.ybjcut(2) ) then
	      cut_test(2) = cut_test(2) + 1
	   ENDIF
	   
	   IF ( q2.lt.q2cut(2).and.Mass.gt.Masscut ) then
	      cut_test(2) = cut_test(2) + 1
	   ENDIF
	   
	   IF ( X3.lt.X3_cut) then
	      cut_test(2) = cut_test(2) + 1
	   endif
	   
	   if ( cos_theta3.gt.cos_t3_cut(1).and.cos_theta3.lt
     &  	.cos_t3_cut(2) ) then
	      cut_test(2) = cut_test(2) + 1
	   endif
	   
CCC     IF SUCCESFUL  cut_test(2) = 4
	   
	   If ( cut_test(2).eq.4) then !$
	      counter(iproc,1) = counter(iproc,1) + 1
	      CALL HFILL(iproc*100+60, REAL(Mass) , 0. ,wtx)
	      CALL HFILL(iproc*100+61, REAL(cos_theta3) , 0. ,wtx)
	      CALL HFILL(iproc*100+62, REAL(psi3) , 0. ,wtx)
	      CALL HFILL(iproc*100+63, REAL(X3), 0. ,wtx)
	      CALL HFILL(iproc*100+64, REAL(X4), 0. ,wtx)
	      
	   ELSE			!$
	      counter(iproc,2) = counter(iproc,2) + 1
	   ENDIF		!$
	   
	ELSE IF (MOD(iflag,10).eq.3) THEN !
C       *** Termination run ***

	   if (iproc.eq.0.or.iproc.eq.2) then
	      write(hzunit,*) 'HZ98162: finishing off'
	   endif
	   
C       *** Change directory ***
	   IF (parton_showers) THEN
              CALL HCDIR('//PAWC/ps'//xxxx,' ')
           ELSE
              CALL HCDIR('//PAWC/'//xxxx,' ')
           ENDIF

C       *** Finish off histograms ***
	   IF (Xsec.eq.0) THEN
	      PRINT*
     &     ,'HZ98162: termination called with zero cross section'
	      PRINT*,'          cross section graph meaningless'
	      PRINT*,'	      in process :',iproc*1000
	      Xsec=1
	   ENDIF
	   IF (Ntot.eq.0) THEN
	      PRINT*,'HZ98162: termination called with no total events'
	      PRINT*,'          cross section graph meaningless'
	      PRINT*,'	      in process :',iproc*1000
	      Ntot=1
	   ENDIF
	   
	   IF (gen(1:3).NE.'HO1') THEN
	      
	      lumnb=ntot/xsec
	      DO j=0,4
		 CALL HZHINRM(100*iproc+60+j,0,lumnb,1)
	      ENDDO

	      IF (iproc.eq.2) THEN !%
	      
c       Add resolved & direct
		 DO j=0,4
		    CALL HOPERA(160+j,'+e',260+j,60+j,1.,1.)
		 ENDDO
		 
	      ENDIF
	      
	   ELSE
c       For NLO stuff, only need to take the bin width into account.
c       However, we must also deal with the final
c       round of integration first.
	      DO j=0,4
		 CALL HZSNLO(60+j,260+j,160+j)
		 call hzhinrm(60+j,0,1.,1)
	      ENDDO

	   ENDIF
	   
	ELSE			!
C       *** End ***
	   PRINT*
     &  	,'HZ98162: Please run routine with iflag set to 1,2 or 3'
	ENDIF			!
	
        
	RETURN
	END
