      Subroutine HZ99057(iflag)
*****************************************************************************
*     This photoproduction routine has to be run twice with the
c     following
*     code additions:
*     +1000 for the DIRECT component run.
*     +2000 for the RESOLVED component run.
*     +0 for both together
*     for all three phases (iflag=1,2,3)
*     
*     +10000 to run on parton showers instead of final state particles.
*     
*     iflag = 1,2 or 3 depending on wether it is the initialization,
*     processing or termination phase (respectively)
*     
*     The default jetfinder is KTCLUS
*     To change jetfinder use these values to call the initialisation
c     phase.
*     1 + 10 * (jetfinder number)
*     
*     This routine produces the follwoing Monte Carlo graphs:
*     id 10 is d(sigma)/d(eta) in pb for all x_gamma, all y, 2nd jet
c     backward
*     id 11 is d(sigma)/d(eta) in pb for all x_gamma, all y, 2nd jet
c     central
*     id 12 is d(sigma)/d(eta) in pb for all x_gamma, all y, 2nd jet
c     forward
*     id 13 is d(sigma)/d(eta) in pb for high x_gamma, all y, 2nd jet
c     backward
*     id 14 is d(sigma)/d(eta) in pb for high x_gamma, all y, 2nd jet
c     central
*     id 15 is d(sigma)/d(eta) in pb for high x_gamma, all y, 2nd jet
c     forward
*     id 16 is d(sigma)/d(eta) in pb for all x_gamma, high y, 2nd jet
c     backward
*     id 17 is d(sigma)/d(eta) in pb for all x_gamma, high y, 2nd jet
c     central
*     id 18 is d(sigma)/d(eta) in pb for all x_gamma, high y, 2nd jet
c     forward
*     id 19 is d(sigma)/d(eta) in pb for high x_gamma, high y, 2nd jet
c     backward
*     id 20 is d(sigma)/d(eta) in pb for high x_gamma, high y, 2nd jet
c     central
*     id 21 is d(sigma)/d(eta) in pb for high x_gamma, high y, 2nd jet
c     forward
*     
*     id 22 is d(sigma)/d(et) in pb for all x_gamma, 1<eta_2<2,
c     1<eta_1<2
*     id 23 is d(sigma)/d(et) in pb for all x_gamma, 0<eta_2<1,
c     1<eta_1<2
*     id 24 is d(sigma)/d(et) in pb for all x_gamma,-1<eta_2<0,
c     1<eta_1<2
*     id 25 is d(sigma)/d(et) in pb for all x_gamma, 0<eta_2<1,
c     0<eta_1<1
*     id 26 is d(sigma)/d(et) in pb for all x_gamma,-1<eta_2<0,
c     0<eta_1<1
*     id 27 is d(sigma)/d(et) in pb for all x_gamma,-1<eta_2<0,
c     -1<eta_1<0
*     id 28 is d(sigma)/d(et) in pb for high x_gamma, 1<eta_2<2,
c     1<eta_1<2
*     id 29 is d(sigma)/d(et) in pb for high x_gamma, 0<eta_2<1,
c     1<eta_1<2
*     id 30 is d(sigma)/d(et) in pb for high x_gamma,-1<eta_2<0,
c     1<eta_1<2
*     id 31 is d(sigma)/d(et) in pb for high x_gamma, 0<eta_2<1,
c     0<eta_1<1
*     id 32 is d(sigma)/d(et) in pb for high x_gamma,-1<eta_2<0,
c     0<eta_1<1
*     id 33 is d(sigma)/d(et) in pb for high x_gamma,-1<eta_2<0,
c     -1<eta_1<0
*     
*     These graphs will be
*     meaningless unless Xsec and Ntot are set before
*     calling the termination routine.
*     (Xsec - total cross section returned by MC)
*     (Ntot - number of events passed to this routine)
*     
*     Note: Since the systematic errors on the data are very asymmetric,
*     they are not included properly on these plots. The lowest (up or
*     down) error is included. The rest should be dealt with seperately
*     in a fitting procedure.
*     
*     
*     Cuts :        Q2 < 1
*     0.2< y_bj < 0.85
*     Depending on graph Et > 11, 14
*     
*     
*     Author : Jon Butterworth (jmb@hep.ucl.ac.uk)
*     JMB - fixed for weighted events 23/7/99
*     Reference : DESY-99-057
*     
*     
*     
*****************************************************************************
*     
*     HERA Tuning defined commons

      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"

      CHARACTER*6 jetf
      
      INTEGER NJET
c      DOUBLE PRECISION Jets(50,8)
      DOUBLE PRECISION CONER
      SAVE CONER
      
      DOUBLE PRECISION elecE
      INTEGER i,j,myil,myip
      INTEGER loop,loop2,count
      DOUBLE PRECISION xgam
      DOUBLE PRECISION phi(2),px(2),py(2),pz(2),ptsq(2)
      DOUBLE PRECISION ppsq(2),energy(2),mass(2)
      DOUBLE PRECISION rap(2),Et(2)
      DOUBLE PRECISION ybj,q2

C     --- cuts arrays
      DOUBLE PRECISION ybjcut(3),q2cut(2)
      DOUBLE PRECISION rapcut(4),Etcut(5)
      SAVE ybjcut,q2cut,rapcut,Etcut

c     Luminosity in nb
      REAL lumnb

C     *** Data for all xgamma, all y 2nd jet backwards
      REAL eta10(5),etacr10(5),cresta10(5)
      REAL cresys10(5)
      DATA eta10/-0.25, 0.25, 0.75, 1.25, 1.75/
      DATA etacr10/88., 209., 258., 240., 201./
      DATA cresta10/5., 9., 9., 9., 8./
      DATA cresys10/6.,10.,4.,9.,7./

C     *** Data for all xgamma, all y, 2nd jet central
      REAL eta11(6),etacr11(6),cresta11(6)
      REAL cresys11(6) 
      DATA eta11/-0.75,-0.25, 0.25, 0.75, 1.25, 1.75/
      DATA etacr11/115.,353., 513., 558., 541., 486./
      DATA cresta11/7., 11., 13., 14., 13.,13./
      DATA cresys11/4.,11.,16.,19.,1.,6./

C     *** Data for all xgamma, all y, 2nd jet forward
      REAL eta12(6),etacr12(6),cresta12(6)
      REAL cresys12(6) 
      DATA eta12/-0.75,-0.25, 0.25, 0.75, 1.25, 1.75/
      DATA etacr12/113., 328., 479., 549., 416., 358./
      DATA cresta12/6., 11., 12., 14., 12.,11./
      DATA cresys12/10.,7.,4.,1.,9.,12./

C     *** Data for high xgamma, all y 2nd jet backwards
      REAL eta13(5),etacr13(5),cresta13(5)
      REAL cresys13(5)
      DATA eta13/-0.25, 0.25, 0.75, 1.25, 1.75/
      DATA etacr13/80., 185., 204., 173., 129./
      DATA cresta13/5., 8., 8., 8., 7./
      DATA cresys13/8.,13.,9.,15.,12./

C     *** Data for high xgamma, all y, 2nd jet central
      REAL eta14(6),etacr14(6),cresta14(6)
      REAL cresys14(6) 
      DATA eta14/-0.75,-0.25, 0.25, 0.75, 1.25, 1.75/
      DATA etacr14/109.,283., 359., 339., 273., 195./
      DATA cresta14/7., 10., 11., 11., 9.,8./
      DATA cresys14/3.,14.,19.,26.,15.,14./

C     *** Data for high xgamma, all y, 2nd jet forward
      REAL eta15(6),etacr15(6),cresta15(6)
      REAL cresys15(6) 
      DATA eta15/-0.75,-0.25, 0.25, 0.75, 1.25, 1.75/
      DATA etacr15/94.,210., 241., 227., 95., 30./
      DATA cresta15/6., 8., 9., 9., 6.,3./
      DATA cresys15/11.,14.,20.,7.,6.,2./

C     *** Data for all xgamma, high y 2nd jet backwards
      REAL eta16(5),etacr16(5),cresta16(5)
      REAL cresys16(5)
      DATA eta16/-0.25, 0.25, 0.75, 1.25, 1.75/
      DATA etacr16/88., 208., 232., 185., 152./
      DATA cresta16/5., 9., 9., 8., 7./
      DATA cresys16/6.,12.,2.,11.,2./

C     *** Data for all xgamma, high y, 2nd jet central
      REAL eta17(6),etacr17(6),cresta17(6)
      REAL cresys17(6) 
      DATA eta17/-0.75,-0.25, 0.25, 0.75, 1.25, 1.75/
      DATA etacr17/115.,326., 284., 218., 162., 153./
      DATA cresta17/7., 11., 10., 9., 7.,7./
      DATA cresys17/4.,11.,6.,3.,12.,3./

C     *** Data for all xgamma, high y, 2nd jet forward
      REAL eta18(6),etacr18(6),cresta18(6)
      REAL cresys18(6) 
      DATA eta18/-0.75,-0.25, 0.25, 0.75, 1.25, 1.75/
      DATA etacr18/110.,227., 186., 128., 122., 117./
      DATA cresta18/6., 9., 8., 6., 6.,6./
      DATA cresys18/10.,8.,4.,11.,4.,11./

C     *** Data for high xgamma, high y 2nd jet backwards
      REAL eta19(5),etacr19(5),cresta19(5)
      REAL cresys19(5)
      DATA eta19/-0.25, 0.25, 0.75, 1.25, 1.75/
      DATA etacr19/80., 183., 178., 122., 84./
      DATA cresta19/5., 8., 8., 6., 5./
      DATA cresys19/8.,15.,4.,13.,3./

C     *** Data for high xgamma, high y, 2nd jet central
      REAL eta20(6),etacr20(6),cresta20(6)
      REAL cresys20(6) 
      DATA eta20/-0.75,-0.25, 0.25, 0.75, 1.25, 1.75/
      DATA etacr20/108.,255., 152., 66., 27., 15./
      DATA cresta20/7., 9., 7., 5., 3.,2./
      DATA cresys20/4.,9.,5.,0.,2.,2./

C     *** Data for high xgamma, high y, 2nd jet forward
      REAL eta21(4),etacr21(4),cresta21(4)
      REAL cresys21(4) 
      DATA eta21/-0.75,-0.25, 0.25, 0.75/
      DATA etacr21/91.,118., 37., 5./
      DATA cresta21/6., 6., 3., 1.2/
      DATA cresys21/10.,5.,0.,2.5/

C     *** Data for all xgamma, 1<eta_2<2, 1<eta_1<2
      REAL etcr22(8),cresta22(8)
      REAL cresysu22(8),cresysd22(8),binedge22(9) 
      DATA etcr22/49.6,30.4,15.0,6.2,2.8,1.53,0.39,0.099/
      DATA cresta22/2.3,1.6,1.1,0.7,0.4,0.29,0.14,0.070/
      DATA cresysu22/4.0,1.0,0.5,0.7,0.3,0.05,0.05,0.183/
      DATA cresysd22/2.0,2.9,1.9,0.7,0.3,0.6,0.06,0.009/
      DATA binedge22/14.,17.,21.,25.,29.,35.,41.,48.,55./
      
C     *** Data for all xgamma, 0<eta_2<1, 1<eta_1<2
      REAL etcr23(8),cresta23(8)
      REAL cresysu23(8),cresysd23(8),binedge23(9) 
      DATA etcr23/73.7,40.4,17.9,8.2,2.8,1.18,0.20,0.28/
      DATA cresta23/2.0,1.3,0.9,0.6,0.3,0.18,0.07,0.10/
      DATA cresysu23/1.7,0.7,0.2,0.1,0.7,0.25,0.15,0.04/
      DATA cresysd23/3.0,2.6,1.3,1.0,0.6,0.41,0.03,0.19/
      DATA binedge23/14.,17.,21.,25.,29.,35.,41.,48.,55./
      
C     *** Data for all xgamma, -1<eta_2<0, 1<eta_1<2
      REAL etcr24(5),cresta24(5)
      REAL cresysu24(5),cresysd24(5),binedge24(6) 
      DATA etcr24/41.2,16.9,5.1,1.56,0.42/
      DATA cresta24/1.6,0.8,0.5,0.26,0.11/
      DATA cresysu24/2.4,0.5,0.7,0.17,0.24/
      DATA cresysd24/7.1,1.4,0.4,0.23,0.05/
      DATA binedge24/14.,17.,21.,25.,29.,35./
      
C     *** Data for all xgamma, 0<eta_2<1, 0<eta_1<1
      REAL etcr25(6),cresta25(6)
      REAL cresysu25(6),cresysd25(6),binedge25(7) 
      DATA etcr25/81.8,42.5,18.2,7.5,2.4,0.49/
      DATA cresta25/3.0,1.9,1.2,0.8,0.4,0.16/
      DATA cresysu25/3.1,0.5,2.4,0.3,0.1,0.14/
      DATA cresysd25/1.8,4.8,2.5,0.7,0.3,0.04/
      DATA binedge25/14.,17.,21.,25.,29.,35.,41./
      
C     *** Data for all xgamma, -1<eta_2<0, 0<eta_1<1
      REAL etcr26(5),cresta26(5)
      REAL cresysu26(5),cresysd26(5),binedge26(6) 
      DATA etcr26/46.2,18.3,4.4,0.97,0.093/
      DATA cresta26/1.7,0.9,0.4,0.20,0.046/
      DATA cresysu26/4.4,0.7,0.6,0.29,0.030/
      DATA cresysd26/8.9,3.1,0.2,0.29,0.063/
      DATA binedge26/14.,17.,21.,25.,29.,35./
      
C     *** Data for all xgamma, -1<eta_2<0, -1<eta_1<0
      REAL etcr27(2),cresta27(2)
      REAL cresysu27(2),cresysd27(2),binedge27(3) 
      DATA etcr27/13.6,1.81/
      DATA cresta27/1.2,0.37/
      DATA cresysu27/0.9,1.16/
      DATA cresysd27/2.2,0.54/
      DATA binedge27/14.,17.,21./
      
C     *** Data for high xgamma, 1<eta_2<2, 1<eta_1<2
      REAL etcr28(8),cresta28(8)
      REAL cresysu28(8),cresysd28(8),binedge28(9) 
      DATA etcr28/1.28,4.2,4.1,2.8,1.29,0.91,0.24,0.099/
      DATA cresta28/0.37,0.6,0.6,0.5,0.28,0.23,0.11,0.070/
      DATA cresysu28/1.32,0.6,0.3,0.3,0.38,0.05,0.07,0.096/
      DATA cresysd28/0.73,1.2,0.5,0.5,0.21,0.32,0.06,0.009/
      DATA binedge28/14.,17.,21.,25.,29.,35.,41.,48.,55./
      
C     *** Data for high xgamma, 0<eta_2<1, 1<eta_1<2
      REAL etcr29(8),cresta29(8)
      REAL cresysu29(8),cresysd29(8),binedge29(9) 
      DATA etcr29/29.0,18.4,8.7,5.1,1.85,0.83,0.125,0.21/
      DATA cresta29/1.3,0.9,0.6,0.4,0.23,0.15,0.056,0.09/
      DATA cresysu29/1.1,1.6,0.6,0.1,0.39,0.30,0.092,0.06/
      DATA cresysd29/3.4,1.9,0.7,0.8,0.35,0.32,0.012,0.16/
      DATA binedge29/14.,17.,21.,25.,29.,35.,41.,48.,55./
      
C     *** Data for high xgamma, -1<eta_2<0, 1<eta_1<2
      REAL etcr30(5),cresta30(5)
      REAL cresysu30(5),cresysd30(5),binedge30(6) 
      DATA etcr30/26.2,11.8,4.1,1.48,0.39/
      DATA cresta30/1.2,0.7,0.4,0.19,0.046/
      DATA cresysu30/3.2,0.7,0.6,0.13,0.20/
      DATA cresysd30/5.9,1.1,0.5,0.22,0.02/
      DATA binedge30/14.,17.,21.,25.,29.,35./
      
C     *** Data for high xgamma, 0<eta_2<1, 0<eta_1<1
      REAL etcr31(6),cresta31(6)
      REAL cresysu31(6),cresysd31(6),binedge31(7) 
      DATA etcr31/48.6,27.8,13.2,6.1,1.9,0.49/
      DATA cresta31/2.3,1.5,1.0,0.7,0.3,0.16/
      DATA cresysu31/4.4,1.6,2.1,0.1,0.2,0.11/
      DATA cresysd31/2.1,3.5,2.0,1.0,0.3,0.10/
      DATA binedge31/14.,17.,21.,25.,29.,35.,41./
      
C     *** Data for high xgamma, -1<eta_2<0, 0<eta_1<1
      REAL etcr32(5),cresta32(5)
      REAL cresysu32(5),cresysd32(5),binedge32(6) 
      DATA etcr32/37.1,15.4,4.2,0.93,0.093/
      DATA cresta32/1.5,0.8,0.4,0.19,0.046/
      DATA cresysu32/3.5,1.1,0.5,0.22,0.030/
      DATA cresysd32/7.7,2.5,0.2,0.32,0.063/
      DATA binedge32/14.,17.,21.,25.,29.,35./
      
C     *** Data for high xgamma, -1<eta_2<0, -1<eta_1<0
      REAL etcr33(2),cresta33(2)
      REAL cresysu33(2),cresysd33(2),binedge33(3) 
      DATA etcr33/12.4,1.74/
      DATA cresta33/1.2,0.36/
      DATA cresysu33/1.0,1.12/
      DATA cresysd33/2.0,0.52/
      DATA binedge33/14.,17.,21./
      
      INTEGER chjet, iproc, iflag
      SAVE chjet
C     --- choose same jet finder as in the paper --- KTCLUS
      DATA chjet/3/
      INTEGER hzflag
      
      REAL pttemp(9)
      INTEGER ivec,jvec
      REAL sqrt
      REAL bw(8), zero(8)
      DATA zero /8*0./

      LOGICAL parton_showers

      CHARACTER*8 xxxx
      DATA xxxx /'d99057'/

****** set the jet cone radius.
******  check if the radius has been set eleswhere
      CALL hzjetrad(2,coner)
******  if not set it to the value used in the paper.
      IF (coner.LT.0.0) THEN
	      coner=1.0
      ENDIF

****** Determine the process type (single/double resolved etc)
      iproc=(mod(iflag,10000)/1000)
      IF ((iproc.LT.0).OR.(iproc.GT.2)) THEN
         WRITE(HZUNIT,*) xxxx,': illegal IPROC:',iproc
         RETURN
      ENDIF

****** choose jet finder
      IF ((MOD(iflag,100)/10.GT.0).AND.MOD(iflag,10).eq.1) THEN
         chjet=INT(MOD(iflag,100)/10)
      ENDIF

****** Decide if we are running on parton showers or final state particles.
      IF (MOD(iflag,100000)/10000.GT.0) THEN
****** use a different paw directory for parton showers.
         xxxx = 'PS'//xxxx
****** tell jet finder that we running on parton showers 
****** (only necessary if using jets!)
         chjet=-1*ABS(chjet)
      ENDIF
******************************************************************************

********************************  Initialise ********************************* 
      IF (MOD(iflag,10).EQ.1) THEN
C     *** Initialisation run ***
C     *** Create directories ***
         IF (iproc.le.1) THEN
            CALL HCDIR('//HISTO',' ')
            CALL HMDIR(xxxx,'S')
            CALL HCDIR('//PAWC',' ')
            CALL HMDIR(xxxx,'S')
            
C     --- create proper cross-section data

            CALL HBOOK1(-10,'Data: All y,xgamma, 2nd jet backward'
     &           ,5 ,-0.5 ,2.0 ,0.)
            CALL HBOOK1(-11,'Data: All y,xgamma, 2nd jet central'
     &           ,6 ,-1.0 ,2.0 ,0.)
            CALL HBOOK1(-12,'Data: All y,xgamma, 2nd jet forward'
     &           ,6 ,-1.0 ,2.0 ,0.)

            CALL HBOOK1(-13,'Data: All y,high xgamma, 2nd jet backward'
     &           ,5 ,-0.5 ,2.0 ,0.)
            CALL HBOOK1(-14,'Data: All y,high xgamma, 2nd jet central'
     &           ,6 ,-1.0 ,2.0 ,0.)
            CALL HBOOK1(-15,'Data: All y,high xgamma, 2nd jet forward'
     &           ,6 ,-1.0 ,2.0 ,0.)

            CALL HBOOK1(-16,'Data: High y,all xgamma, 2nd jet backward'
     &           ,5 ,-0.5 ,2.0 ,0.)
            CALL HBOOK1(-17,'Data: High y,all xgamma, 2nd jet central'
     &           ,6 ,-1.0 ,2.0 ,0.)
            CALL HBOOK1(-18,'Data: High y,all xgamma, 2nd jet forward'
     &           ,6 ,-1.0 ,2.0 ,0.)

            CALL HBOOK1(-19,'Data: High y,high xgamma, 2nd jet backward'
     &           ,5 ,-0.5 ,2.0 ,0.)
            CALL HBOOK1(-20,'Data: High y,high xgamma, 2nd jet central'
     &           ,6 ,-1.0 ,2.0 ,0.)
            CALL HBOOK1(-21,'Data: High y,high xgamma, 2nd jet forward'
     &           ,4 ,-1.0 ,1.0 ,0.)
            
c     ET plots
            
            CALL HBOOKB(-22,'Data:Both Jets Forward, all xgamma'
     &           ,8,binedge22,0.)
            
            CALL HBOOKB(-23
     &           ,'Data: 1 Forward 2 middle, all xgamma',8,binedge23,0.)
            
            CALL HBOOKB(-24,'Data: 1 Forward 2 back, all xgamma'
     &           ,5,binedge24,0.)
            
            CALL HBOOKB(-25,'Data: Both jets central, all xgamma'
     &           ,6,binedge25,0.)
            
            CALL HBOOKB(-26,'Data: 1 middle, 2 back, all xgamma'
     &           ,5,binedge26,0.)
            
            CALL HBOOKB(-27,'Data: Both jet backward, all xgamma'
     &           ,2,binedge27,0.)
            
            CALL HBOOKB(-28,'Data:Both Jets Forward, high xgamma'
     &           ,8,binedge28,0.)
            
            CALL HBOOKB(-29
     &           ,'Data: 1 Forward 2 middle, high xgamma',8,binedge29,0.
     &           )
            
            CALL HBOOKB(-30,'Data: 1 Forward 2 back, high xgamma'
     &           ,5,binedge30,0.)
            
            CALL HBOOKB(-31
     &           ,'Data: Both jets central, high xgamma',6,binedge31,0.)
            
            CALL HBOOKB(-32,'Data: 1 middle, 2 back, high xgamma'
     &           ,5,binedge32,0.)
            
            CALL HBOOKB(-33
     &           ,'Data: Both jet backward, high xgamma',2,binedge33,0.)
            
            
            CALL HPAK(-10,etacr10)
            DO ivec=1,5
               pttemp(ivec)=sqrt(cresta10(ivec)**2+cresys10(ivec)**2)
            ENDDO
            CALL HPAKE(-10,pttemp)

            CALL HPAK(-11,etacr11)
            DO ivec=1,6
               pttemp(ivec)=sqrt(cresta11(ivec)**2+cresys11(ivec)**2)
            ENDDO
            CALL HPAKE(-11,pttemp)

            CALL HPAK(-12,etacr12)
            DO ivec=1,6
               pttemp(ivec)=sqrt(cresta12(ivec)**2+cresys12(ivec)**2)
            ENDDO
            CALL HPAKE(-12,pttemp)

            CALL HPAK(-13,etacr13)
            DO ivec=1,5
               pttemp(ivec)=sqrt(cresta13(ivec)**2+cresys13(ivec)**2)
            ENDDO
            CALL HPAKE(-13,pttemp)

            CALL HPAK(-14,etacr14)
            DO ivec=1,6
               pttemp(ivec)=sqrt(cresta14(ivec)**2+cresys14(ivec)**2)
            ENDDO
            CALL HPAKE(-14,pttemp)

            CALL HPAK(-15,etacr15)
            DO ivec=1,6
               pttemp(ivec)=sqrt(cresta15(ivec)**2+cresys15(ivec)**2)
            ENDDO
            CALL HPAKE(-15,pttemp)

            CALL HPAK(-16,etacr16)
            DO ivec=1,5
               pttemp(ivec)=sqrt(cresta16(ivec)**2+cresys16(ivec)**2)
            ENDDO
            CALL HPAKE(-16,pttemp)

            CALL HPAK(-17,etacr17)
            DO ivec=1,6
               pttemp(ivec)=sqrt(cresta17(ivec)**2+cresys17(ivec)**2)
            ENDDO
            CALL HPAKE(-17,pttemp)

            CALL HPAK(-18,etacr18)
            DO ivec=1,6
               pttemp(ivec)=sqrt(cresta18(ivec)**2+cresys18(ivec)**2)
            ENDDO
            CALL HPAKE(-18,pttemp)

            CALL HPAK(-19,etacr19)
            DO ivec=1,5
               pttemp(ivec)=sqrt(cresta19(ivec)**2+cresys19(ivec)**2)
            ENDDO
            CALL HPAKE(-19,pttemp)

            CALL HPAK(-20,etacr20)
            DO ivec=1,6
               pttemp(ivec)=sqrt(cresta20(ivec)**2+cresys20(ivec)**2)
            ENDDO
            CALL HPAKE(-20,pttemp)

            CALL HPAK(-21,etacr21)
            DO ivec=1,4
               pttemp(ivec)=sqrt(cresta21(ivec)**2+cresys21(ivec)**2)
            ENDDO
            CALL HPAKE(-21,pttemp)

            CALL HPAK(-22,etcr22)
            DO ivec=1,8
               pttemp(ivec)=sqrt(cresta22(ivec)**2+( (cresysu22(ivec)
     &              +cresysd22(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-22,pttemp)

            CALL HPAK(-23,etcr23)
            DO ivec=1,8
               pttemp(ivec)=sqrt(cresta23(ivec)**2+( (cresysu23(ivec)
     &              +cresysd23(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-23,pttemp)

            CALL HPAK(-24,etcr24)
            DO ivec=1,5
               pttemp(ivec)=sqrt(cresta24(ivec)**2+( (cresysu24(ivec)
     &              +cresysd24(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-24,pttemp)

            CALL HPAK(-25,etcr25)
            DO ivec=1,6
               pttemp(ivec)=sqrt(cresta25(ivec)**2+( (cresysu25(ivec)
     &              +cresysd25(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-25,pttemp)

            CALL HPAK(-26,etcr26)
            DO ivec=1,5
               pttemp(ivec)=sqrt(cresta26(ivec)**2+( (cresysu26(ivec)
     &              +cresysd26(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-26,pttemp)

            CALL HPAK(-27,etcr27)
            DO ivec=1,2
               pttemp(ivec)=sqrt(cresta27(ivec)**2+( (cresysu27(ivec)
     &              +cresysd27(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-27,pttemp)

            CALL HPAK(-28,etcr28)
            DO ivec=1,8
               pttemp(ivec)=sqrt(cresta28(ivec)**2+( (cresysu28(ivec)
     &              +cresysd28(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-28,pttemp)

            CALL HPAK(-29,etcr29)
            DO ivec=1,8
               pttemp(ivec)=sqrt(cresta29(ivec)**2+( (cresysu29(ivec)
     &              +cresysd29(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-29,pttemp)

            CALL HPAK(-30,etcr30)
            DO ivec=1,5
               pttemp(ivec)=sqrt(cresta30(ivec)**2+( (cresysu30(ivec)
     &              +cresysd30(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-30,pttemp)

            CALL HPAK(-31,etcr31)
            DO ivec=1,6
               pttemp(ivec)=sqrt(cresta31(ivec)**2+( (cresysu31(ivec)
     &              +cresysd31(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-31,pttemp)

            CALL HPAK(-32,etcr32)
            DO ivec=1,5
               pttemp(ivec)=sqrt(cresta32(ivec)**2+( (cresysu32(ivec)
     &              +cresysd32(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-32,pttemp)

            CALL HPAK(-33,etcr33)
            DO ivec=1,2
               pttemp(ivec)=sqrt(cresta33(ivec)**2+( (cresysu33(ivec)
     &              +cresysd33(ivec))/2.0 )**2 )
            ENDDO
            CALL HPAKE(-33,pttemp)

         ENDIF
         CALL HCDIR('//HISTO/'//xxxx,' ')
         CALL HCDIR('//PAWC/'//xxxx,' ')
         
         CALL HBOOK1(iproc*100+10,'MC: All y,xgamma, 2nd jet backward'
     &        ,5 ,-0.5 ,2.0 ,0.)
         CALL HBOOK1(iproc*100+11,'MC: All y,xgamma, 2nd jet central'
     &        ,6 ,-1.0 ,2.0 ,0.)
         CALL HBOOK1(iproc*100+12,'MC: All y,xgamma, 2nd jet forward'
     &        ,6 ,-1.0 ,2.0 ,0.)
         
         CALL HBOOK1(iproc*100+13
     &        ,'MC: All y,high xgamma, 2nd jet backward',5 ,-0.5 ,2.0 ,0
     &        .)
         CALL HBOOK1(iproc*100+14
     &        ,'MC: All y,high xgamma, 2nd jet central',6 ,-1.0 ,2.0 ,0.
     &        )
         CALL HBOOK1(iproc*100+15
     &        ,'MC: All y,high xgamma, 2nd jet forward',6 ,-1.0 ,2.0 ,0.
     &        )
         
         CALL HBOOK1(iproc*100+16
     &        ,'MC: High y,all xgamma, 2nd jet backward',5 ,-0.5 ,2.0 ,0
     &        .)
         CALL HBOOK1(iproc*100+17
     &        ,'MC: High y,all xgamma, 2nd jet central',6 ,-1.0 ,2.0 ,0.
     &        )
         CALL HBOOK1(iproc*100+18
     &        ,'MC: High y,all xgamma, 2nd jet forward',6 ,-1.0 ,2.0 ,0.
     &        )
         
         CALL HBOOK1(iproc*100+19
     &        ,'MC: High y,high xgamma, 2nd jet backward',5 ,-0.5 ,2.0
     &        ,0.)
         CALL HBOOK1(iproc*100+20
     &        ,'MC: High y,high xgamma, 2nd jet central',6 ,-1.0 ,2.0 ,0
     &        .)
         CALL HBOOK1(iproc*100+21
     &        ,'MC: High y,high xgamma, 2nd jet forward',4 ,-1.0 ,1.0 ,0
     &        .)

c     ET plots

         CALL HBOOKB(iproc*100+22,'MC:Both Jets Forward, all xgamma',8
     &        ,binedge22,0.)
         
         CALL HBOOKB(iproc*100+23,'MC: 1 Forward 2 middle, all xgamma',8
     &        ,binedge23,0.)
         
         CALL HBOOKB(iproc*100+24,'MC: 1 Forward 2 back, all xgamma',5
     &        ,binedge24,0.)

         CALL HBOOKB(iproc*100+25,'MC: Both jets central, all xgamma',6
     &        ,binedge25,0.)
         
         CALL HBOOKB(iproc*100+26,'MC: 1 middle, 2 back, all xgamma',5
     &        ,binedge26,0.)
         
         CALL HBOOKB(iproc*100+27,'MC: Both jet backward, all xgamma',2
     &        ,binedge27,0.)
         
         CALL HBOOKB(iproc*100+28,'MC:Both Jets Forward, high xgamma',8
     &        ,binedge28,0.)
         
         CALL HBOOKB(iproc*100+29,'MC: 1 Forward 2 middle, high xgamma'
     &        ,8,binedge29,0.)
         
         CALL HBOOKB(iproc*100+30,'MC: 1 Forward 2 back, high xgamma',5
     &        ,binedge30,0.)
         
         CALL HBOOKB(iproc*100+31,'MC: Both jets central, high xgamma',6
     &        ,binedge31,0.)
         
         CALL HBOOKB(iproc*100+32,'MC: 1 middle, 2 back, high xgamma',5
     &        ,binedge32,0.)
         
         CALL HBOOKB(iproc*100+33,'MC: Both jet backward, high xgamma',2
     &        ,binedge33,0.)

         CALL HIDOPT(0,'stat')
         CALL HBARX(0)
         
         
C     --- photoproduction cuts
         
         ybjcut(1)=0.2
         ybjcut(2)=0.5
         ybjcut(3)=0.85
         q2cut(1)=0.
         q2cut(2)=1.
         
C     --- basic cuts for et and  rapidity plots
         Etcut(1)=14.0
         Etcut(2)=11.0
         rapcut(1)=-1.0
         rapcut(2)=0.0
         rapcut(3)=1.0
         rapcut(4)=2.0
         
C     --- standard cone radius is 1.0
         CALL HZJETRAD(2,CONER)
         IF (CONER.lt.0.0) THEN
            CONER=1.0
         ENDIF
         CALL HZJTNAME(chjet,jetf)
         
         WRITE(6,*)'**********************************************'
         WRITE(6,*)'* HZ99057 called, histograms will be output. *'
         WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
         WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
         WRITE(6,*)'**********************************************'

 1001    FORMAT(A29,F5.2,A13)

      ELSE IF (MOD(iflag,10).eq.2) THEN
C     *** Filling runs ***
         
C     *** Change directory to our graphs ***
         CALL HCDIR('//PAWC/'//xxxx,' ')

         IF (gen(1:3).EQ.'HO1') THEN

            DO j=10,33
               CALL HZSNLO(j,j+200,j+100)
            ENDDO
            iproc=1

         ENDIF

         
         q2=hzphokin(1)
         IF (q2.eq.-1) THEN
            PRINT*,'HZ99057: HZPHOKIN returns error for Q2'
         ELSEIF (q2.lt.0) THEN
            q2=abs(q2)
         ENDIF
         ybj=hzphokin(3)
         IF (ybj.eq.-1) THEN
            PRINT*,'HZ99057: HZPHOKIN returns error for ybj'
         ENDIF
         
C     *** Fill our histogram ***
         
         IF ((ybj.ge.ybjcut(1)).and.(ybj.le.ybjcut(3))
     &        .and.(q2.lt.q2cut(2))) Then
            
            elecE=27.5
            hzflag=HZIBEAM(myip,myil)
            IF (hzflag.ge.0) Then
               elecE=abs(PHEP(4,myil))
            ELSE
               PRINT*,'HZ99057: Electron not found. E=0.001'
               PRINT*,'         event will be rejected'
               elecE=0.001
            ENDIF
            
            CALL hzjtfind(chjet,CONER,NJET,Jets)
            
            DO i=1,2
               et(i)=0.0
            ENDDO

            IF (NJET.ge.2) THEN
               i=0
               DO loop=1,njet
c     Search jets in the rapidity interval.
                  IF ( jets(loop,1).gt.rapcut(1).and.jets(loop,1).lt
     &                 .rapcut(4) ) THEN
                     i=i+1
                     IF (i.LE.2) THEN
                        Et(i)=Jets(loop,3)
                        rap(i)=Jets(loop,1)
                     ENDIF
                  ENDIF
               ENDDO
               
c     ET cut
               IF ((Et(2).gt.Etcut(2)).and.(et(1).gt.ETcut(1))) THEN

c     Calculate xgamma
                  xgam=(Et(1)*exp(-1*rap(1))+Et(2)*exp(-1*rap(2)))
     &                 /(2*ybj*elecE)

c     Fill the rapidity plots.
                  DO i=1,2
                     j=3-i
                     IF (rap(i).gt.rapcut(1).and.rap(i).lt.rapcut(2))
     &                    THEN
                        CALL HFILL(INT(iproc*100+10), REAL(rap(j)) ,
     &                       0.,wtx)
                     ENDIF
                     IF (rap(i).gt.rapcut(2).and.rap(i).lt.rapcut(3))
     &                    THEN
                        CALL HFILL(INT(iproc*100+11), REAL(rap(j)) ,
     &                       0.,wtx)
                     ENDIF
                     IF (rap(i).gt.rapcut(3).and.rap(i).lt.rapcut(4))
     &                    THEN
                        CALL HFILL(INT(iproc*100+12), REAL(rap(j)) ,
     &                       0.,wtx)
                     ENDIF
                     IF (xgam.ge.0.75) THEN
                        IF (rap(i).gt.rapcut(1).and.rap(i).lt
     &                       .rapcut(2))THEN
                           CALL HFILL(INT(iproc*100+13), REAL(rap(j))
     &                          ,0.,wtx)
                        ENDIF
                        IF (rap(i).gt.rapcut(2).and.rap(i).lt
     &                       .rapcut(3))THEN
                           CALL HFILL(INT(iproc*100+14), REAL(rap(j))
     &                          ,0.,wtx)
                        ENDIF
                        IF (rap(i).gt.rapcut(3).and.rap(i).lt
     &                       .rapcut(4))THEN
                           CALL HFILL(INT(iproc*100+15), REAL(rap(j))
     &                          ,0.,wtx)
                        ENDIF
                        
                     ENDIF
                     IF (ybj.gt.ybjcut(2)) THEN
                        
                        IF (rap(i).gt.rapcut(1).and.rap(i).lt
     &                       .rapcut(2))THEN
                           CALL HFILL(INT(iproc*100+16), REAL(rap(j))
     &                          ,0.,wtx)
                        ENDIF
                        IF (rap(i).gt.rapcut(2).and.rap(i).lt
     &                       .rapcut(3))THEN
                           CALL HFILL(INT(iproc*100+17), REAL(rap(j))
     &                          ,0.,wtx)
                        ENDIF
                        IF (rap(i).gt.rapcut(3).and.rap(i).lt
     &                       .rapcut(4))THEN
                           CALL HFILL(INT(iproc*100+18), REAL(rap(j))
     &                          ,0.,wtx)
                        ENDIF
                        IF (xgam.ge.0.75) THEN
                           IF (rap(i).gt.rapcut(1).and.rap(i).lt
     &                          .rapcut(2))THEN
                              CALL HFILL(INT(iproc*100+19),
     &                             REAL(rap(j)),0.,wtx)
                           ENDIF
                           IF (rap(i).gt.rapcut(2).and.rap(i).lt
     &                          .rapcut(3))THEN
                              CALL HFILL(INT(iproc*100+20),
     &                             REAL(rap(j)),0.,wtx)
                           ENDIF
                           IF (rap(i).gt.rapcut(3).and.rap(i).lt
     &                          .rapcut(4))THEN
                              CALL HFILL(INT(iproc*100+21),
     &                             REAL(rap(j)),0.,wtx)
                           ENDIF
                        ENDIF
                     ENDIF
                  ENDDO
                  
                  
c     Fill the ET  plots.
                  DO i=1,2
                     j=3-i
                     
                     IF ((rap(i).gt.rapcut(3).and.rap(i).lt.rapcut(4)
     &                    ).AND.(rap(j).gt.rapcut(3).and.rap(j).lt
     &                    .rapcut(4))) THEN
                        CALL HFILL(INT(iproc*100+22), REAL(MAX(et(j)
     &                       ,et(i))) ,0.,wtx)
                        IF (xgam.GT.0.75) THEN
                           CALL HFILL(INT(iproc*100+28),
     &                          REAL(MAX(et(j),et(i))) ,0.,wtx)
                        ENDIF
                     ENDIF
                     
                     IF ((rap(i).gt.rapcut(3).and.rap(i).lt.rapcut(4)
     &                    ).AND.(rap(j).gt.rapcut(2).and.rap(j).lt
     &                    .rapcut(3))) THEN
                        CALL HFILL(INT(iproc*100+23), REAL(MAX(et(j)
     &                       ,et(i))) ,0.,wtx)
                        IF (xgam.GT.0.75) THEN
                           CALL HFILL(INT(iproc*100+29),
     &                          REAL(MAX(et(j),et(i))) ,0.,wtx)
                        ENDIF
                     ENDIF
                     
                     IF ((rap(i).gt.rapcut(3).and.rap(i).lt.rapcut(4)
     &                    ).AND.(rap(j).gt.rapcut(1).and.rap(j).lt
     &                    .rapcut(2))) THEN
                        CALL HFILL(INT(iproc*100+24), REAL(MAX(et(j)
     &                       ,et(i))) ,0.,wtx)
                        IF (xgam.GT.0.75) THEN
                           CALL HFILL(INT(iproc*100+30),
     &                          REAL(MAX(et(j),et(i))) ,0.,wtx)
                        ENDIF
                     ENDIF
                     
                     IF ((rap(i).gt.rapcut(2).and.rap(i).lt.rapcut(3)
     &                    ).AND.(rap(j).gt.rapcut(2).and.rap(j).lt
     &                    .rapcut(3))) THEN
                        CALL HFILL(INT(iproc*100+25), REAL(MAX(et(j)
     &                       ,et(i))) ,0.,wtx)
                        IF (xgam.GT.0.75) THEN
                           CALL HFILL(INT(iproc*100+31),
     &                          REAL(MAX(et(j),et(i))) ,0.,wtx)
                        ENDIF
                     ENDIF
                     
                     IF ((rap(i).gt.rapcut(2).and.rap(i).lt.rapcut(3)
     &                    ).AND.(rap(j).gt.rapcut(1).and.rap(j).lt
     &                    .rapcut(2))) THEN
                        CALL HFILL(INT(iproc*100+26), REAL(MAX(et(j)
     &                       ,et(i))) ,0.,wtx)
                        IF (xgam.GT.0.75) THEN
                           CALL HFILL(INT(iproc*100+32),
     &                          REAL(MAX(et(j),et(i))) ,0.,wtx)
                        ENDIF
                     ENDIF
                     
                     IF ((rap(i).gt.rapcut(1).and.rap(i).lt.rapcut(2)
     &                    ).AND.(rap(j).gt.rapcut(1).and.rap(j).lt
     &                    .rapcut(2))) THEN
                        CALL HFILL(INT(iproc*100+27), REAL(MAX(et(j)
     &                       ,et(i))) ,0.,wtx)
                        IF (xgam.GT.0.75) THEN
                           CALL HFILL(INT(iproc*100+33),
     &                          REAL(MAX(et(j),et(i))) ,0.,wtx)
                        ENDIF
                     ENDIF
                     
                  ENDDO
               ENDIF
            ENDIF
            
         ENDIF                  ! Q2 & y cuts
         
      ELSE IF (MOD(iflag,10).eq.3) THEN
C     *** Termination run ***
C     *** Change directory ***
         IF (iproc.eq.0.or.iproc.eq.2) THEN
            WRITE(HZUNIT,*) 'HZ99057: Finishing off'
         ENDIF

         CALL HCDIR('//PAWC/'//xxxx,' ')
         
C     *** Finish off histograms ***
         
         IF (Xsec.eq.0) THEN
            PRINT*,'HZ99057: termination called with zero cross section'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',iproc*1000
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,'HZ99057: termination called with no total events'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',iproc*1000
            Ntot=1
         ENDIF
         
C     --- store Xsec and Ntot for both direct and resolved events.
c         mhXsec(iproc)=Xsec*1000.
c         mhntot(iproc)=Ntot

         lumnb = ntot/(1000.*xsec)

         
C     --- only finally

         IF (gen(1:3).NE.'HO1') THEN

C     --- normalise resolved/direct component histograms for eta plots
            DO i=10,21
               CALL HZHINRM(100*iproc+i,0,lumnb,1)
            ENDDO
            DO i=22,33
               CALL HZHINRM(100*iproc+i,0,lumnb,1)
            ENDDO
            
            IF (iproc.eq.2) THEN

C     --- now calculate cross-section graphs (eta and et)
               DO i=10,33
                  CALL HOPERA(100+i,'+e',200+i,i,1.,1.)
               ENDDO

            ENDIF

         endif

         IF (iproc.eq.2.and.gen(1:3).EQ.'HO1') THEN
               
            DO j=10,33
               CALL HZSNLO(j,j+200,j+100)
            ENDDO
            
            DO i=10,21
               CALL HOPERA(i,'+e',i,i,2000.0,0.)
            ENDDO
            
C     --- normalise resolved/direct component histograms for et plots
            DO i=22,33
               CALL HOPERA(i,'*e',30+i,i,1000.,1.)
            ENDDO

         ENDIF
         
      ELSE
C     *** End ***
         PRINT*,'HZ99057:Please run routine with iflag set to 1,2 or 3'
         PRINT*,'        with +1000 or +2000 for DIR or RES events.'
      ENDIF
      
      RETURN
      END
