      SUBROUTINE HZH0307071(iflag)
************************************************************************
*     This routine reproduces the ZEUS plots on charm dijets for the
*     virtual photon. Unfortunately the data have only appeared in
*     conference papers and theses. The best hep-ex reference is
*     hep-ex/0307071, presented by P. bussey on behalf of ZEUS at Photon
*     2003: International Conference on the Structure and Interactions
*     of the Photon and 15th International Workshop on Photon-Photon
*     Collisions, Frascati, Italy, 7-11 Apr 2003, Published in
*     Nucl.Phys.Proc.Suppl.126:17-21,2004.  More details can be found in
*     Matthew Lightwood, "Dijet Production and MultiScale QCD at HERA ",
*     UCL PhD Thesis, June 2004.
*
*     The plots are essentially those from hep-ex/0404033 but with the
*     addition of a D* requirement. The only on with data is that which
*     is an official ZEUS public plot; the resolved vs direct ratio as a
*     function of Q2.
*
*     Initial state: 27 GeV positrons vs 820 GeV protons.
*
*     Reference: [reference]
*     Arguments: iflag=10000*[A]+1000*[B]+10*[D]+1 initialisation
*                iflag=10000*[A]+1000*[B]+10*[D]+2 filling
*                iflag=10000*[A]+1000*[B]+10*[D]+3 termination
*
*     [A]= IPS   >0 for partons shower running, 0 otherwise
*     [B]= IPROC decides which process is being run
*     [D]= JETF  decides jet finder type. Ignored if IRUN=1.
*     See manual section "The analysis routines" for more details.
*
*     It is assumed you run either (a) with iproc = 0
*                               or (b) with iproc = 1, 2. in order.
*
*     Histograms: -N data for Fig.Xa
*                  N simulation for Fig.Xa
*                  ... etc.
*
*     Routine by Matt Lightwood and Jon Butterworth
*
****************************************************************************
      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
c     Jetfinder stuff

      CHARACTER*6 jetf
      
      INTEGER NJET
      DOUBLE PRECISION et(10),rap(10),phi(10)
      DOUBLE PRECISION Jets_lab(maxhzjets,8),et_lab(10),rap_lab(10)
      DOUBLE PRECISION CONER
      SAVE CONER

      DOUBLE PRECISION elecE
      INTEGER myil,myip

c     Iterators. i_jet not an iterator.

      INTEGER i_bw,i_plot,i_jet,j_jet,i_loop,j_loop,i

c     Physics stuff

      DOUBLE PRECISION q2,ybj,xgamma,E_e,E_p,x

      PARAMETER (E_e = 27.5, E_p = 820.0)
c stuff for boost msl
      DOUBLE PRECISION pgam(4),pgam4(4),pbeam(4),phcm(4)
      DOUBLE PRECISION phc(4),jets_hcm(50,4),plab(4)
      DOUBLE PRECISION ppp(4,512)
      INTEGER lepid,IErr,pbid,ebid,J,p_count
      REAL p,pt,e_t(100),th,rap_hcm,eta(100)

c stuff for ktclus call
      integer ktmode,npjets,npart, parttojet(512), nsjets
      integer mjet(4,512),nppp
      REAL ecut, yy(512), ktpjet(4,512), drad
      REAL ktireco, scale, p_t
      double precision px,py,pz,ptsq,ppsq, eta_hcm,et_hcm

c     Cuts

      DOUBLE PRECISION ybjcut(2)
      DATA ybjcut / 0.2, 0.55 /      
      REAL q2cut(2)
      DATA q2cut / 0., 1. /  
      REAL xgcuts(3)
      DATA xgcuts / 0., 0.75, 1.001 /
      REAL etacut(2)
      DATA etacut / -2.4, 2.4 /
      REAL etcut(2)
      DATA etcut / 6.5, 7.5 /

c     declare the shift in eta and mean_et variable
      REAL eta_shift, mean_et, eta_f

      REAL lumnb

c     q2 bin information

      REAL q2_bins_php(2)
      REAL q2_bins_dis(5)
      REAL q2_bins_dis1(2)
      REAL q2_bins_dis2(2)
      REAL q2_bins_dis3(2)
      REAL q2_bins_dis4(2)
      DATA q2_bins_php / 0., 1.0/
      DATA q2_bins_dis / 1.5, 4.5, 10.5, 49.0, 5000.0 /
      DATA q2_bins_dis1 / 1.5, 4.5 /
      DATA q2_bins_dis2 / 4.5, 10.5 /
      DATA q2_bins_dis3 / 10.5, 49. /
      DATA q2_bins_dis4 / 49., 5000. /
c      
c     eta bin information

      REAL eta_bins(5)
      DATA eta_bins / -3., -1.8, 0., 0.8, 2.4 /

c     dsigma/det bin information

      REAL dsigma(7)
      DATA dsigma / 7.5, 9., 12.5, 15. , 22., 30. , 40. /
    
c     xgamma bin information

      REAL xg_bins(3)
      DATA xg_bins / 0., 0.75, 1.001 /


c     Some useful declarations
      INTEGER chjet, iproc, iflag
      SAVE chjet

c     Choose KT clus. Frixiones code uses ES Jet. Doesn't matter.

       DATA chjet/3/
c
       INTEGER hzflag
c some stuff added for charm version
       integer iq_bin
       integer idmo1,idmo2,ihep,idhe
       integer id_dsdaut, ndstars,nds_pass
       integer i_dscand,i_piscand,i_picand,i_kcand 
       logical lctag,lstag
       Double precision phcdstr(4,3)
       Double precision ds_px,ds_py,ds_pz,ds_E,ds_ptsq,ds_ppsq
       Double precision et_dstar(4), eta_dstar(4), pt_dstar(4)
c end of new stuff
c
c     Don't know if all these are neccessary. Check.
      
      LOGICAL parton_showers, lp, first 
      DATA lp, first /.FALSE., .TRUE./      

      Character*8 xxxx
      DATA xxxx /'H0307071'/
      
*     Determine the process type.
      iproc=(mod(iflag,10000)/1000)
      if ((iproc.lt.0).or.(iproc.gt.2)) then
         write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
         return
      endif

*     --- choose jet finder
      IF ((MOD(iflag,100)/10.gt.0).and.MOD(iflag,10).eq.1) THEN
         chjet=INT(MOD(iflag,100)/10)
      ENDIF

      IF (MOD(iflag,10).eq.1) then
*     Initialisation: The following MUST always be done
*     (i) make subdirectory in PAWC
*     - use the name as the xxxx in HZxxxx subroutine
*     (i) make subdirectory in o/p file

*     
*     book your MC histograms
*
         if (iproc.le.1) then

            IF (parton_showers) THEN
               Call hcdir('//HISTO',' ')
               CALL HMDIR('PS'//xxxx,'S')
               Call hcdir('//PAWC',' ')
               CALL HMDIR('PS'//xxxx,'S')
            ELSE
               Call hcdir('//HISTO',' ')
               CALL HMDIR(xxxx,'S')
               Call hcdir('//PAWC',' ')
               CALL HMDIR(xxxx,'S')
            ENDIF
        ENDIF

c     Change in to the correct directories.
*******************************************************
*		event processing
*******************************************************

        IF (parton_showers) THEN
           CALL HCDIR('//HISTO/PS'//XXXX,' ')
           CALL HCDIR('//PAWC/PS'//XXXX,' ')
        ELSE
           CALL HCDIR('//HISTO/'//XXXX,' ')
           CALL HCDIR('//PAWC/'//XXXX,' ')
        ENDIF     

            
c     eta plot
        CALL HBOOKB(100*iproc+13,'dsig/deta_f php',4,eta_bins,0.)
        CALL HBOOKB(100*iproc+25,'dsig/deta_f bpc',4,eta_bins,0.)
        CALL HBOOKB(100*iproc+26,'dsig/deta_f dis1',4,eta_bins,0.)
        CALL HBOOKB(100*iproc+27,'dsig/deta_f dis2',4,eta_bins,0.)
        CALL HBOOKB(100*iproc+28,'dsig/deta_f dis3',4,eta_bins,0.)
        CALL HBOOKB(100*iproc+29,'dsig/deta_f dis4',4,eta_bins,0.)
        
c     dsigma/detj1 plot
c     
        CALL HBOOKB(100*iproc+14,'dsigma/detj1 php',6,dsigma,0.)
        CALL HBOOKB(100*iproc+31,'dsigma/detj1 bpc',6,dsigma,0.)
        CALL HBOOKB(100*iproc+32,'dsigma/detj1 dis1',6,dsigma,0.)
        CALL HBOOKB(100*iproc+33,'dsigma/detj1 dis2',6,dsigma,0.)
        CALL HBOOKB(100*iproc+34,'dsigma/detj1 dis3',6,dsigma,0.)
        CALL HBOOKB(100*iproc+35,'dsigma/detj1 dis4',6,dsigma,0.)           
c     
c     dsig/dQ^2 in bins of xgamma and etj1^2
c     php
c     
        CALL HBOOKB(100*iproc+40,'dsigma/dq2 php tot',1,q2_bins_php
     $       ,0.)
        CALL HBOOKB(100*iproc+41,'dsigma/dq2 php low',1,q2_bins_php
     $       ,0.)
        CALL HBOOKB(100*iproc+42,'dsigma/dq2 php hi',1,q2_bins_php,0
     $       .)
c     
c     dis
c     
        CALL HBOOKB(100*iproc+43,'dsigma/dq2 dis1 tot',4,q2_bins_dis
     $       ,0.)
        CALL HBOOKB(100*iproc+44,'dsigma/dq2 dis1 low',4,q2_bins_dis
     $       ,0.)
        CALL HBOOKB(100*iproc+45,'dsigma/dq2 dis1 hi',4,q2_bins_dis
     $       ,0.)

c     HIDOPT sets the mean value and RMS to be computed at filling time
c     (double precision).  See HBOOK manual

         CALL HIDOPT(0,'stat')

c     Store the errors for 1D histograms in memory and superimpose them
c     on the plot during output. This routine must be called after
c     booking but before filling.

         CALL HBARX(0)

c     Standard cone radius is 1.0

         CALL HZJETRAD(2,CONER)

         IF (CONER.lt.0.0) THEN
            CONER=1.0
         ENDIF

         CALL HZJTNAME(chjet,jetf)
         
         WRITE(HZUNIT,*)'**********************************************'
         WRITE(HZUNIT,*
     $        )'* HZH0307071  called, histograms will be output. *'
         WRITE(HZUNIT,*)'*     Jet finder selected : ',jetf
     $        ,'           *'
         WRITE(HZUNIT,1001)'*        with cone radius : ',CONER,'   *'
         WRITE(HZUNIT,*)'**********************************************'

 1001    FORMAT(A29,F5.2,A13)         

      ELSE IF (MOD(iflag,10).eq.2) THEN

c     Filling runs 
         
c     Change directory to our graphs

         IF (parton_showers) THEN              
            CALL HCDIR('//PAWC/PS'//XXXX,' ')
         ELSE
            CALL HCDIR('//PAWC/'//XXXX,' ')
         ENDIF


c- get useful variables
         q2=real(HZDISKIN(1))
	 ybj=HZDISKIN(3)
	 x = real(hzdiskin(2))
c
c
c- some checks on the variables
         IF (q2.EQ.-1) THEN
            WRITE(HZUNIT,*) 'HZH0307071: HZPHOKIN returns error for Q2'
         ELSEIF (q2.LT.0) THEN
            q2=ABS(q2)
         ENDIF
         IF (ybj.EQ.-1) THEN
            WRITE(HZUNIT,*) 'HZH0307071: HZPHOKIN returns error for y'
         ENDIF
c     check yjb cut and the Q2 cut.
 
         IF ((ybj.GE.ybjcut(1)).and.(ybj.LE.ybjcut(2)) .AND.q2.GT.0.
     $        .AND.(q2.LT.q2_bins_dis(5))) THEN
c
c if we get this far initialize boost
c some checks of the photon and electron properties                        
c
           
c go get the beam properties
c
            hzflag=HZIBEAM(pbid,ebid)

            if (hzflag.ne.1) then
               write(hzunit,*) 'ERROR: Problem finding beams'
            endif

c
c     Get the initial state electron energy.(incomin e+) lepton ID -
            lepid=hzidelec(0)
            if (lepid.lt.0) then
               write (6,*) 'Id of electron not found'
               return
            endif
            
            IF (hzflag.GE.0) THEN
               elecE=ABS(PHEP(4,ebid))
            ELSE
               WRITE(HZUNIT,*) 'HZH0307071: Electron not found. E=0.001'
               WRITE(HZUNIT,*) '         event will be rejected'
               RETURN
            ENDIF

c CCbar stuff here

            ndstars=0
            nds_pass=0
c     
            Do ihep=1,nhep
               idhe=idhep(ihep)
               if ( idhe .eq. 413 ) then
                  ndstars = ndstars + 1
                  Do i=1,4
                     phcdstr(i,ndstars)=PHEP(i,ihep)
                  enddo
               endif
            enddo
c
c  zero the dstar variable arrays 
c
            do j=1,4
               et_dstar(j) = 0.0
               eta_dstar(j) = 0.0
            enddo
c     work out the D* kinematics
            Do i=1, ndstars
               ds_px = phcdstr(1,i)
               ds_py = phcdstr(2,i)
               ds_pz = phcdstr(3,i)
               ds_E  = phcdstr(4,i)
               ds_ptsq = ds_px**2 + ds_py**2
               ds_ppsq = (sqrt(ds_ptsq+ds_pz**2)+abs(ds_pz))**2
               pt_dstar(i) = sqrt(ds_ptsq)
               eta_dstar(i) = SIGN(0.5*Log(ds_ppsq/ds_ptsq), ds_pz)
c     does it pass cuts??
               if(pt_dstar(i).gt.3.0.and.eta_dstar(i).lt.1.5
     &              .and.eta_dstar(i).gt.-1.5) then
                  nds_pass = nds_pass + 1
               endif
            EndDo
            if(nds_pass.gt.0.and.lp) then
               write(*,*)'!!!!!!!event passing!!!!!!!!!!!'
               write(*,*)'pt dstar:   ',pt_dstar(1)
               write(*,*)'eta dstar:   ',eta_dstar(1)
            endif

            IF (nds_pass.eq.0) goto 99
c
c     Find jets
            DO i_loop=1,50
               DO j_loop=1,8
                  Jets(i_loop,j_loop)=0.
                  Jets_lab(i_loop,j_loop)=0.			       
               ENDDO
            ENDDO
c     
            NJET=0
            
            CALL HZJTFIND(3,CONER,NJET,Jets)
            
            IF (lp) THEN
               WRITE(*,*) 'Njet=',NJET
            ENDIF
            
c     calculate the shift in eta if we are in photoproduction
             eta_shift=0.
c            eta_shift=0.5*LOG((E_p)/(E_e*ybj)) 
            i_jet=0
            DO j_jet = 1,NJET

c     Find the two highest ET jets in the rapidity interval.
c     Assumes jets are ordered in ET.

               IF (((Jets(j_jet,1)).GT.etacut(1))
     &              .AND.((Jets(j_jet,1)).LT.etacut(2))
     &              .AND.(Jets(j_jet,3).GT.etcut(1))) THEN
                  i_jet=i_jet+1
                  IF (i_jet.LE.2) THEN
                     et(i_jet)=Jets(j_jet,3)
                     rap(i_jet)=Jets(j_jet,1)
                  ENDIF
               ENDIF
            ENDDO
c     
c     If there are not at least two jets satisfying the cuts exit
c                                  
            IF (i_jet.LT.2) GOTO 99
            
            
c     Remember, I am assuming the jets are ordered in ET. 
              
            IF ((et(1).GT.etcut(2))) THEN
               
c     - calculate xgamma using jets selected in the CMS frame
c     - but boosted back to the lab frame
c                  
               xgamma=((et(1)*EXP(-1*rap(1)))+(et(2)*
     &              EXP(-1*rap(2))))/(2*elecE*ybj)          
               
               eta_f = -9999
               DO i=1,i_jet
                  IF((rap(i)).GT.eta_f) THEN
                     eta_f = rap(i)
                  ENDIF
                  
               ENDDO          
               
               
c     fill the eta_f distributions
	       IF(q2.GT.q2_bins_php(1).AND.q2.LT.q2_bins_php(2)) THEN
                  CALL HFILL((100*iproc)+13,
     &                 REAL(eta_f),0.,wtx)
	       ENDIF
	       IF(q2.GT.q2_bins_dis1(1).AND.q2.LT.q2_bins_dis1(2)) THEN
                  CALL HFILL((100*iproc)+26,
     &                 REAL(eta_f),0.,wtx)
	       ENDIF
	       IF(q2.GT.q2_bins_dis2(1).AND.q2.LT.q2_bins_dis2(2)) THEN
                  CALL HFILL((100*iproc)+27,
     &                 REAL(eta_f),0.,wtx)
	       ENDIF
	       IF(q2.GT.q2_bins_dis3(1).AND.q2.LT.q2_bins_dis3(2)) THEN
                  CALL HFILL((100*iproc)+28,
     &                 REAL(eta_f),0.,wtx)
	       ENDIF
	       IF(q2.GT.q2_bins_dis4(1).AND.q2.LT.q2_bins_dis4(2)) THEN
                  CALL HFILL((100*iproc)+29,
     &                 REAL(eta_f),0.,wtx)
	       ENDIF
c     
	       IF ((xgamma.GT.xgcuts(1)).AND.
     &              (xgamma.LT.xgcuts(3))) THEN
                  if(q2.lt.1.0) then
                     CALL HFILL((100*iproc)+40,
     &                    REAL(q2),0.,wtx)
                  endif
                  if(q2.ge.1.5) then
                     CALL HFILL((100*iproc)+43,
     &                    REAL(q2),0.,wtx)
                  endif
                  
	       ENDIF
c     
               IF ((xgamma.GT.xgcuts(1)).AND. (xgamma.LT.xgcuts(2)))
     $              THEN
                  if(q2.lt.1.0) then
                     CALL HFILL((100*iproc)+41, REAL(q2),0.,wtx)
                  endif
                  if(q2.ge.1.5) then
                     CALL HFILL((100*iproc)+44, REAL(q2),0.,wtx)
                  endif
                  
	       ENDIF
c     
               IF ((xgamma.GT.xgcuts(2)).AND.
     &              (xgamma.LT.xgcuts(3))) THEN
                  if(q2.lt.1.0) then
                     CALL HFILL((100*iproc)+42, REAL(q2),0.,wtx)
                  endif	
                  if(q2.ge.1.5) then
                     CALL HFILL((100*iproc)+45, REAL(q2),0.,wtx)
                  endif
                  
	       ENDIF
               

               IF (et(1).LT.dsigma(7)) THEN
                  IF(q2.GT.q2_bins_php(1).AND.q2.LT.q2_bins_php(2)) THEN
                     CALL HFILL((100*iproc)+14,REAL(et(1)), 0.,wtx)
                  ENDIF
                  IF(q2.GT.q2_bins_dis1(1).AND.q2.LT.q2_bins_dis1(2))
     $                 THEN
                     CALL HFILL((100*iproc)+32,REAL(et(1)), 0.,wtx)
                  ENDIF               
                  IF(q2.GT.q2_bins_dis2(1).AND.q2.LT.q2_bins_dis2(2))
     $                 THEN
                     CALL HFILL((100*iproc)+33,REAL(et(1)), 0.,wtx)
     $                    
                  ENDIF
                  IF(q2.GT.q2_bins_dis3(1).AND.q2.LT.q2_bins_dis3(2))
     $                 THEN
                     CALL HFILL((100*iproc)+34,REAL(et(1)),
     &                    0.,wtx)               
                  ENDIF
                  IF(q2.GT.q2_bins_dis4(1).AND.q2.LT.q2_bins_dis4(2))
     $                 THEN
                     CALL HFILL((100*iproc)+35,REAL(et(1)), 0.,wtx)
     $                    
                  ENDIF
                  
               ENDIF  
               
c     end et cut
               
            ENDIF
            
c     For q2 and ybj cut
         ENDIF
         
      ELSE IF (MOD(iflag,10).eq.3) THEN

c     Termination run 
         if (iproc.eq.0.or.iproc.eq.2) then
            write(hzunit,*) 'HZ'//xxxx//': finishing off'
         endif
         
c     Change directory
         
         IF (parton_showers) THEN
            CALL HCDIR('//PAWC/PS'//XXXX,' ')
         ELSE
            CALL HCDIR('//PAWC/'//XXXX,' ')
         ENDIF
         
c     Finish off histograms
         
         IF (Xsec.eq.0) THEN
            WRITE(HZUNIT,*) 
     & 'HZH0307071: termination called with zero cross section'
            WRITE(HZUNIT,*) 
     &   '       cross section graph meaningless'
            WRITE(HZUNIT,*) '        in process :',iproc*1000
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            WRITE(HZUNIT,*) 
     &    'HZH0307071: termination called with no total events'
            WRITE(HZUNIT,*) 
     &    '      cross section graph meaningless'
            WRITE(HZUNIT,*) '         in process :',iproc*1000
            Ntot=1
         ENDIF


c - now weight the histograms 

         lumnb = ntot/xsec
         
c     - eta plots
         CALL HZHINRM(100*iproc+13,0,lumnb,1)
         DO i=25,29
            CALL HZHINRM(100*iproc+i,0,lumnb,1)
         ENDDO
c     - et plots
         CALL HZHINRM(100*iproc+14,0,lumnb,1)
         DO i=31,35
            CALL HZHINRM(100*iproc+i,0,lumnb,1)
         ENDDO
         
c     - q2 plots
         DO i=40,45
            CALL HZHINRM(100*iproc+i,0,lumnb,1)
         ENDDO

         
         if (iproc.eq.2) then
            CALL HOPERA(113,'+e',213,13,1.,1.)
            CALL HOPERA(114,'+e',214,14,1.,1.)
            DO i=25,29
               CALL HOPERA(100+i,'+e',200+i,i,1.,1.)
            ENDDO
            DO i=31,35
               CALL HOPERA(100+i,'+e',200+i,i,1.,1.)
            ENDDO
            DO i=40,45
               CALL HOPERA(100+i,'+e',200+i,i,1.,1.)
            ENDDO
         endif
         
c     - dsigma dq2 - will do bin division in root macros
c     - much easier that way!!
c     
         
c     dsig/det^2 - need to still devide by q2 binning

      ELSE

c     End 

         WRITE(HZUNIT,*)
     $    'HZH0307071:Please run routine with iflag set to 1,2 or 3'
         WRITE(HZUNIT,*) 
     &    '     with +1000 or +2000 for DIR or RES events.'
      
      ENDIF
      
99    CONTINUE

      RETURN

      END
