      SUBROUTINE HZH0312057(IFLAG)
****************************************************************************
*     
* produces histograms for the ZEUS paper hep-ex/0312057
*
* Beauty production measured using decays into muons in
* dijet events in ep collisions sqrt(s)=318 GeV
*
* A. beauty cross section for events with two jets and a muon
*
*    sigma(ep->e bbbar X->e jj mu X) [pb]
*
*    in 3 bins of the region:
*                            1: Rear
*                            2: Barrel
*                            3: Forward
*
*    cuts:
*
*    general: Q^2 < 1
*             0.2 < Y < 0.8
*
*    jets: 
*    KT-algorithm running in massive mode (E recombination scheme) 
*    on b hadrons (jetf.eq.10)
*    NJETS > 1
*    |ETA_JET| < 2.5 
*    p_T_Jet_1 > 7 GeV
*    p_T_Jet_2 > 6 GeV
*
*    muon:
*          coming from b decays
*          p_mu   > 2.5 GeV                for -1.6  < eta_mu < -0.9
*          p_T_mu > 2.5 GeV                for -0.9  < eta_mu <  1.3
*          p_T_mu > 1 GeV and p_mu > 4 GeV for  1.48 < eta_mu <  2.3
*
* B. diff. beauty cross section as function of x_gamma
*
*    dsigma/dx_gamma (ep->e bbbar X->e jj mu X) [pb]
*
*    in 4 bins of x_gamma = ((E-p_z)_JET1 + (E-p_z)_JET2) / (2*Y*Ee)
*
*    cuts:
*
*    general: Q^2 < 1
*             0.2 < Y < 0.8
*
*    jets: 
*    KT-algorithm running in massive mode (E recombination scheme) 
*    on b hadrons (jetf.eq.10)
*    NJETS > 1
*    |ETA_JET| < 2.5 
*    p_T_Jet_1 > 7 GeV
*    p_T_Jet_2 > 6 GeV
*
*    muon:
*          coming from b decays
*          p_mu   > 2.5 GeV                for -1.6  < eta_mu < -0.9
*          p_T_mu > 2.5 GeV                for -0.9  < eta_mu <  1.3
*          p_T_mu > 1 GeV and p_mu > 4 GeV for  1.48 < eta_mu <  2.3
*
* C. diff. beauty cross section as function of eta_mu
*
*    dsigma/deta_mu (ep->e bbbar X->e jj mu X) [pb]
*
*    in 4 bins of eta_mu
*
*    cuts:
*
*    general: Q^2 < 1
*             0.2 < Y < 0.8
*
*    jets: 
*    KT-algorithm running in massive mode (E recombination scheme) 
*    on b hadrons (jetf.eq.10)
*    NJETS > 1
*    |ETA_JET| < 2.5 
*    p_T_Jet_1 > 7 GeV
*    p_T_Jet_2 > 6 GeV
*
*    muon:
*          coming from b decays
*          p_T_mu > 2.5 GeV
*
* D. diff. beauty cross section as function of p_T_mu
*
*    dsigma/dp_T_mu (ep->e bbbar X->e jj mu X) [pb/GeV]
*
*    in 3 bins of p_T_mu
*
*    cuts:
*
*    general: Q^2 < 1
*             0.2 < Y < 0.8
*
*    jets: 
*    KT-algorithm running in massive mode (E recombination scheme) 
*    on b hadrons (jetf.eq.10)
*    NJETS > 1
*    |ETA_JET| < 2.5 
*    p_T_Jet_1 > 7 GeV
*    p_T_Jet_2 > 6 GeV
*
*    muon:
*          coming from b decays
*          p_T_mu > 2.5 GeV
*          -1.6  < eta_mu <  2.3
*
* E. diff. beauty cross section as function of eta of the jet associated to the muon
*
*    dsigma/deta_jmu (ep->e bbbar X->e jj mu X) [pb]
*
*    in 4 bins of eta_jmu
*
*    cuts:
*
*    general: Q^2 < 1
*             0.2 < Y < 0.8
*
*    jets: 
*    KT-algorithm running in massive mode (E recombination scheme) 
*    on b hadrons (jetf.eq.10)
*    NJETS > 1
*    |ETA_JET| < 2.5 
*    p_T_Jet_1 > 7 GeV
*    p_T_Jet_2 > 6 GeV
*
*    muon:
*          coming from b decays
*          p_mu   > 2.5 GeV                for -1.6  < eta_mu < -0.9
*          p_T_mu > 2.5 GeV                for -0.9  < eta_mu <  1.3
*          p_T_mu > 1 GeV and p_mu > 4 GeV for  1.48 < eta_mu <  2.3
*
*    for muons associated to jet with:
*          p_T_mu-jet > 6 GeV
*          |eta_mu-jet| < 2.5
* 
* F. diff. beauty cross section as function of p_T of the jet associated to the muon
*
*    dsigma/dp_T_jmu (ep->e bbbar X->e jj mu X) [pb/GeV]
*
*    in 3 bins of p_T_jb
*
*    cuts:
*
*    general: Q^2 < 1
*             0.2 < Y < 0.8
*
*    jets: 
*    KT-algorithm running in massive mode (E recombination scheme) 
*    on b hadrons (jetf.eq.10)
*    NJETS > 1
*    |ETA_JET| < 2.5 
*    p_T_Jet_1 > 7 GeV
*    p_T_Jet_2 > 6 GeV
*
*    muon:
*          coming from b decays
*          p_mu   > 2.5 GeV                for -1.6  < eta_mu < -0.9
*          p_T_mu > 2.5 GeV                for -0.9  < eta_mu <  1.3
*          p_T_mu > 1 GeV and p_mu > 4 GeV for  1.48 < eta_mu <  2.3
*
*    for muons associated to jet with:
*          p_T_mu-jet > 6 GeV
*          |eta_mu-jet| < 2.5
* 
* G. diff. beauty cross section as function of eta of the jet containing a B hadron
*
*    dsigma/deta_jb (ep->e bbbar X->e jj X) [pb]
*
*    in 4 bins of eta_jb
*
*    cuts:
*
*    general: Q^2 < 1
*             0.2 < Y < 0.8
*
*    jets: 
*    KT-algorithm running in massive mode (E recombination scheme) 
*    on b hadrons (jetf.eq.10)
*    NJETS > 1
*    |ETA_JET| < 2.5 
*    PT_Jet_1 > 7 GeV
*    PT_Jet_2 > 6 GeV
*
* H. diff. beauty cross section as function of p_T of the jet containing a B hadron
*
*    dsigma/dp_T_jb (ep->e bbbar X->e jj X) [pb/GeV]
*
*    in 3 bins of p_T_jb
*
*    cuts:
*
*    general: Q^2 < 1
*             0.2 < Y < 0.8
*
*    jets: 
*    KT-algorithm running in massive mode (E recombination scheme) 
*    on b hadrons (jetf.eq.10)
*    NJETS > 1
*    |ETA_JET| < 2.5 
*    PT_Jet_1 > 7 GeV
*    PT_Jet_2 > 6 GeV
*
* I. diff. beauty cross section as function of the b-quark transverse momentum
*
*    dsigma/dp_T_b(ep->ebX) [pb/GeV]
*
*    in 3 bins of p_T_b 
*
*    cuts:
*
*    general: Q^2 < 1
*             0.2 < Y < 0.8
*             |eta_b| < 2
*
* Arguments:  iflag=1 initialisation/booking of histograms
*             iflag=2 event processing/filling of MC histograms
*             iflag=3 termination/final normalisation
*                     (the variables Xsec (total cross section) and
*                      Ntot (total number of events) have to be set before
*                      calling the termination routine in order to get a
*                      correct normalisation for the MC histograms)
*
*             This photoproduction routine has to be run twice with the
*             following code additions:
* 	              +1000 for the DIRECT component run.
*	              +2000 for the RESOLVED component run.
*                     +0 for both together
*             for all three phases (iflag=1,2,3)
*
* written by: Oliver Gutsche (oliver.gutsche@desy.de) on 20.01.2004
*****************************************************************************
*
* Output graphs (all given in pb):
*---------------
* DATA
*
* histograms -1, ... -7 contain the measured cross section data,
* -1        sigma(ep->e bbbar X->e jj mu X)            for cross section A
* -2        dsigma/dx_gamma (ep->e bbbar X->e jj mu X) for cross section B
* -3        dsigma/deta_mu (ep->e bbbar X->e jj mu X)  for cross section C
* -4        dsigma/dp_T_mu (ep->e bbbar X->e jj mu X)  for cross section D
* -5        dsigma/deta_jmu (ep->e bbbar X->e jj mu X) for cross section E
* -6        dsigma/dp_T_jmu (ep->e bbbar X->e jj mu X) for cross section F
* -7        dsigma/deta_jb (ep->e bbbar X->e jj X)     for cross section G
* -8        dsigma/dp_T_jb (ep->e bbbar X->e jj X)     for cross section H
* -9        dsigma/dp_T_b(ep->ebX)                     for cross section I
*
*
* Monte-Carlo
*   1       sigma(ep->e bbbar X->e jj mu X)            for cross section A
*   2       dsigma/dx_gamma (ep->e bbbar X->e jj mu X) for cross section B
*   3       dsigma/deta_mu (ep->e bbbar X->e jj mu X)  for cross section C
*   4       dsigma/dp_T_mu (ep->e bbbar X->e jj mu X)  for cross section D
*   5       dsigma/deta_jmu (ep->e bbbar X->e jj mu X) for cross section E
*   6       dsigma/dp_T_jmu (ep->e bbbar X->e jj mu X) for cross section F
*   7       dsigma/deta_jb (ep->e bbbar X->e jj X)     for cross section G
*   8       dsigma/dp_T_jb (ep->e bbbar X->e jj X)     for cross section H
*   9       dsigma/dp_T_b(ep->ebX)                     for cross section I
*  99       temporary histogram                        for cross section I
* 101       sigma(ep->e bbbar X->e jj mu X)            for cross section A DIRECT component
* 102       dsigma/dx_gamma (ep->e bbbar X->e jj mu X) for cross section B DIRECT component
* 103       dsigma/deta_mu (ep->e bbbar X->e jj mu X)  for cross section C DIRECT component
* 104       dsigma/dp_T_mu (ep->e bbbar X->e jj mu X)  for cross section D DIRECT component
* 105       dsigma/deta_jmu (ep->e bbbar X->e jj mu X) for cross section E DIRECT component
* 106       dsigma/dp_T_jmu (ep->e bbbar X->e jj mu X) for cross section F DIRECT component
* 107       dsigma/deta_jb (ep->e bbbar X->e jj X)     for cross section G DIRECT component
* 108       dsigma/dp_T_jb (ep->e bbbar X->e jj X)     for cross section H DIRECT component
* 109       dsigma/dp_T_b(ep->ebX)                     for cross section I DIRECT component
* 199       temporary histogram                        for cross section I DIRECT component
* 201       sigma(ep->e bbbar X->e jj mu X)            for cross section A RESOLVED component
* 202       dsigma/dx_gamma (ep->e bbbar X->e jj mu X) for cross section B RESOLVED component
* 203       dsigma/deta_mu (ep->e bbbar X->e jj mu X)  for cross section C RESOLVED component
* 204       dsigma/dp_T_mu (ep->e bbbar X->e jj mu X)  for cross section D RESOLVED component
* 205       dsigma/deta_jmu (ep->e bbbar X->e jj mu X) for cross section E RESOLVED component
* 206       dsigma/dp_T_jmu (ep->e bbbar X->e jj mu X) for cross section F RESOLVED component
* 207       dsigma/deta_jb (ep->e bbbar X->e jj X)     for cross section G RESOLVED component
* 208       dsigma/dp_T_jb (ep->e bbbar X->e jj X)     for cross section H RESOLVED component
* 209       dsigma/dp_T_b(ep->ebX)                     for cross section I RESOLVED component
* 299       temporary histogram for                    for cross section I RESOLVED component
*
*****************************************************************************
      IMPLICIT NONE
     
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
     
      Character xxxx*8
      Data xxxx/'H0312057'/
     
C     constants
      DOUBLE PRECISION pi
c      double precision hzp
      PARAMETER(pi=3.141592653589793238)
     
C     variables
      INTEGER IFLAG,IPROC,ID,I,J,K
      LOGICAL first
      Data first/.true./
      Save first

*     bins, data and errors for cross section A.
      INTEGER NABIN
      PARAMETER (NABIN=3)
      REAL ABIN(NABIN+1)
      REAL ADATA(NABIN),AERROR(NABIN)
      
      DATA ABIN /0.5,1.5,2.5,3.5/
      SAVE ABIN
      DATA ADATA /6.5,38.2,16.6/
      SAVE ADATA
      DATA AERROR /1.86,6.72,5.66/
      SAVE AERROR
      
*     bins, data and errors for cross section B.
      INTEGER NBBIN
      PARAMETER (NBBIN=4)
      REAL BBIN(NBBIN+1)
      REAL BDATA(NBBIN),BERROR(NBBIN)
      
      DATA BBIN /0.,0.25,0.5,0.75,1./
      SAVE BBIN
      DATA BDATA /12.9,20.8,30.0,165.0/
      SAVE BDATA
      DATA BERROR /9.84,10.91,8.25,36.77/
      SAVE BERROR
      
*     bins, data and errors for cross section C.
      INTEGER NCBIN
      PARAMETER (NCBIN=4)
      REAL CBIN(NCBIN+1)
      REAL CDATA(NCBIN),CERROR(NCBIN)
      
      DATA CBIN /-1.6,-0.75,0.25,1.3,2.3/
      SAVE CBIN
      DATA CDATA /4.6,18.8,16.7,10.0/
      SAVE CDATA
      DATA CERROR /1.58,3.48,3.76,3.47/
      SAVE CERROR
      
*     bins, data and errors for cross section D.
      INTEGER NDBIN
      PARAMETER (NDBIN=3)
      REAL DBIN(NDBIN+1)
      REAL DDATA(NDBIN),DERROR(NDBIN)
      
      DATA DBIN /2.5,4.,6.,10./
      SAVE DBIN
      DATA DDATA /16.1,7.1,1.69/
      SAVE DDATA
      DATA DERROR /4.36,1.64,0.4/
      SAVE DERROR
      
*     bins, data and errors for cross section E.
      INTEGER NEBIN
      PARAMETER (NEBIN=4)
      REAL EBIN(NEBIN+1)
      REAL EDATA(NEBIN),EERROR(NEBIN)
      
      DATA EBIN /-1.6,-0.6,0.4,1.4,2.5/
      SAVE EBIN
      DATA EDATA /8.6,18.9,14.3,14.4/
      SAVE EDATA
      DATA EERROR /2.48,3.56,4.44,4.99/
      SAVE EERROR
      
*     bins, data and errors for cross section F.
      INTEGER NFBIN
      PARAMETER (NFBIN=3)
      REAL FBIN(NFBIN+1)
      REAL FDATA(NFBIN),FERROR(NFBIN)
      
      DATA FBIN /6.,11.,16.,30./
      SAVE FBIN
      DATA FDATA /6.41,2.98,0.51/
      SAVE FDATA
      DATA FERROR /1.41,0.9,0.14/
      SAVE FERROR
      
*     bins, data and errors for cross section G.
      INTEGER NGBIN
      PARAMETER (NGBIN=4)
      REAL GBIN(NGBIN+1)
      REAL GDATA(NGBIN),GERROR(NGBIN)
      
      DATA GBIN /-1.6,-0.6,0.4,1.4,2.5/
      SAVE GBIN
      DATA GDATA /152.0,356.0,275.0,229.0/
      SAVE GDATA
      DATA GERROR /42.45,71.85,85.76,81.84/
      SAVE GERROR
      
*     bins, data and errors for cross section H.
      INTEGER NHBIN
      PARAMETER (NHBIN=3)
      REAL HBIN(NHBIN+1)
      REAL HDATA(NHBIN),HERROR(NHBIN)
      
      DATA HBIN /6.,11.,16.,30./
      SAVE HBIN
      DATA HDATA /137.0,43.8,5.7/
      SAVE HDATA
      DATA HERROR /30.41,13.20,1.56/
      SAVE HERROR
      
*     bins, data and errors for cross section I.
      INTEGER NIBIN
      PARAMETER (NIBIN=3)
      REAL IBIN(NIBIN+1)
      REAL IDATA(NIBIN),IERROR(NIBIN)
      
      DATA IBIN /6.,11.,16.,30./
      SAVE IBIN
      DATA IDATA /70.0,14.4,1.37/
      SAVE IDATA
      DATA IERROR /15.74,4.7,0.5/
      SAVE IERROR

      INTEGER NTEMPBIN
      PARAMETER (NTEMPBIN=5)
      REAL TEMPBIN(NTEMPBIN+1)
      DATA TEMPBIN/8.,9.,13.1,14.1,21.75,22.75/
      REAL temparr(NTEMPBIN+2),temperr(NTEMPBIN+2)

      REAL Q2,Y,LUMPB
      INTEGER BEAMFLAG,PROTID,LEPID
      REAL EE
     
C     final state muon list including IHEP id (1), 
C     id of originating b-quark from BBBAR list (2),
C     id of originating B-hadron from BHAD list (3),
C     jet id in which muon is contained (4)
      INTEGER NMAXMU,NMUIND,MUMOID
      PARAMETER (NMAXMU=6,NMUIND=4)
      INTEGER NMU,MU(NMAXMU,NMUIND)
      REAL MUPT,MUETA

*     b quark variables
      INTEGER NB,NMAXB
      INTEGER NBBAR,NMAXBBAR
      INTEGER NBBBAR,NMAXBBBAR
      PARAMETER (NMAXB=3,NMAXBBAR=3,NMAXBBBAR=6)
      INTEGER B(NMAXB),BBAR(NMAXBBAR),BBBAR(NMAXBBBAR)
      LOGICAL replace
     
C     list of B hadrons
      INTEGER NBHADMAX,NBHADIND
      PARAMETER (NBHADMAX=10,NBHADIND=2)
      INTEGER NBHAD,BHAD(NBHADMAX,NBHADIND)
     
      INTEGER nbrange
      PARAMETER (nbrange = 84)
      INTEGER brange(nbrange)
      data brange / 521,-521,511,-511,553,551,20553,555,100553,10551,
     +     120553,100555,200553,300553,400553,9010553,5122,-5122,-531,
     +     531,-541,541,-523,523,-513,513,-533,533,-543,543,5222,-5222,
     +     5212,-5212,5112,-5112,5322,-5322,5232,-5232,5312,-5312,5132,
     +     -5132,5332,-5332,5224,-5224,5214,-5214,5114,-5114,5334,-5334,
     +     10511,-10511,10513,-10513,10521,-10521,10523,-10523,10531,
     +     -10531,10533,-10533,10541,-10541,10543,-10543,515,-515,525,
     +     -525,535,-535,20513,-20513,20523,-20523,20533,-20533,20543,
     +     -20543 /
      LOGICAL inlist

C     jet finder
C     array has 10 digits, first id from jetfinder, following 8 correspond to
C     jetfinder output array, 10 is pt
      INTEGER NJETS,TEMPNJETS
      DOUBLE PRECISION JET(MAXHZJETS,10),TEMPJET(MAXHZJETS,8)
      LOGICAL twojetcut
     
C     div. variables
      REAL XGAMMA
      INTEGER exchange,loop_1,loop_2,loop_3
      LOGICAL filled,matched
      INTEGER MAXARRAY
      PARAMETER(MAXARRAY=1024)
      INTEGER STEPINDEX,TOTINDEX,ARRAY(MAXARRAY)

*     Determine the process type.
      iproc=(mod(iflag,10000)/1000)
      if ((iproc.lt.0).or.(iproc.gt.2)) then
         write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
         return
      endif


      IF (MOD(iflag,10).eq.1) then
C     *** Initialisation run ***
     
         IF (first) THEN
            CALL HCDIR('//PAWC',' ')
            CALL HMDIR(xxxx,'S')
            CALL HCDIR('//HISTO',' ')
            CALL HMDIR(xxxx,'S')
         ELSE
            CALL HCDIR('//PAWC/'//xxxx,' ')
            CALL HCDIR('//HISTO/'//xxxx,' ')
         ENDIF

         write(hzunit,*) 'HZ'//xxxx//': Booking histograms'
     
C     book mc histos
         ID=1+100*iproc
         CALL HBOOKB(ID,' [s](ep->e bbbar X->e jj [m] X) ',NABIN,
     +        ABIN,0.)
         CALL HBARX(ID)

         ID=2+100*iproc
         CALL HBOOKB(ID,' d[s]/dx?[g](ep->e bbbar X->e jj [m] X) ',
     +        NBBIN,BBIN,0.)
         CALL HBARX(ID)

         ID=3+100*iproc
         CALL HBOOKB(ID,' d[s]/d[e]^[m](ep->e bbbar X->e jj [m] X) ',
     +        NCBIN,CBIN,0.)
         CALL HBARX(ID)

         ID=4+100*iproc
         CALL HBOOKB(ID,' d[s]/dp?T^[m](ep->e bbbar X->e jj [m] X) ',
     +        NDBIN,DBIN,0.)
         CALL HBARX(ID)

         ID=5+100*iproc
         CALL HBOOKB(ID,' d[s]/d[e]^{[m]-jet}(ep->e bbbar X->
     +e jj [m] X) ',NEBIN,EBIN,0.)
         CALL HBARX(ID)

         ID=6+100*iproc
         CALL HBOOKB(ID,' d[s]/dp?T^{[m]-jet}(ep->e bbbar X->
     +e jj [m] X) ',NFBIN,FBIN,0.)
         CALL HBARX(ID)

         ID=7+100*iproc
         CALL HBOOKB(ID,' d[s]/d[e]^{jb}(ep->e bbbar X->e jj X) ',NGBIN,
     +        GBIN,0.)
         CALL HBARX(ID)

         ID=8+100*iproc
         CALL HBOOKB(ID,' d[s]/dp?T^{jb}(ep->e bbbar X->e jj X) ',NHBIN,
     +        HBIN,0.)
         CALL HBARX(ID)

         IF (iproc.ne.0) then
            ID=9+100*iproc
            CALL HBOOKB(ID,' d[s]/dp?T^{b}(ep->e b X) ',NIBIN,IBIN,0.)
            CALL HBARX(ID)

            ID=99+100*iproc
            CALL HBOOKB(ID,'temp',NTEMPBIN,TEMPBIN,0.)
            CALL HBARX(ID)
         ENDIF

         IF (first) THEN
c     Book the data.
            ID=1
            CALL HBOOKB(-ID,' [s](ep->e bbbar X->e jj [m] X) ',
     +           NABIN,ABIN,0.)
            CALL HPAK(-ID,ADATA)
            CALL HPAKE(-ID,AERROR)

            ID=2
            CALL HBOOKB(-ID,' d[s]/dx?[g](ep->e bbbar X->e jj [m] X) ',
     +           NBBIN,BBIN,0.)
            CALL HPAK(-ID,BDATA)
            CALL HPAKE(-ID,BERROR)
         
            ID=3
            CALL HBOOKB(-ID,' d[s]/d[e]^[m](ep->e bbbar X->
     +e jj [m] X) ',NCBIN,CBIN,0.)
            CALL HPAK(-ID,CDATA)
            CALL HPAKE(-ID,CERROR)
            
            ID=4
            CALL HBOOKB(-ID,' d[s]/dp?T^[m](ep->e bbbar X->
     +           e jj [m] X) ',NDBIN,DBIN,0.)
            CALL HPAK(-ID,DDATA)
            CALL HPAKE(-ID,DERROR)
            
            ID=5
            CALL HBOOKB(-ID,' d[s]/d[e]^{j [m]-jet}
     +(ep->e bbbar X->e jj [m] X) ',
     +           NEBIN,EBIN,0.)
            CALL HPAK(-ID,EDATA)
            CALL HPAKE(-ID,EERROR)
            
            ID=6
            CALL HBOOKB(-ID,' d[s]/dp?T^{j [m]-jet}'//
     +           '(ep->e bbbar X->e jj [m] X) ',
     +           NFBIN,FBIN,0.)
            CALL HPAK(-ID,FDATA)
            CALL HPAKE(-ID,FERROR)
            
            ID=7
            CALL HBOOKB(-ID,' d[s]/d[e]^{jb}(ep->e bbbar X->e jj X) ',
     +           NGBIN,GBIN,0.)
            CALL HPAK(-ID,GDATA)
            CALL HPAKE(-ID,GERROR)
            
            ID=8
            CALL HBOOKB(-ID,' d[s]/dp?T^{jb}(ep->e bbbar X->e jj X) ',
     +           NHBIN,HBIN,0.)
            CALL HPAK(-ID,HDATA)
            CALL HPAKE(-ID,HERROR)
            
            ID=9
            CALL HBOOKB(ID,' d[s]/dp?T^{b}(ep->e b X) ',NIBIN,IBIN,0.)
            CALL HBARX(ID)
            CALL HBOOKB(-ID,' d[s]/dp?T^{b}(ep->e b X) ',NIBIN,IBIN,0.)
            CALL HPAK(-ID,IDATA)
            CALL HPAKE(-ID,IERROR)
            
            ID=99
            CALL HBOOKB(ID,'temp',NTEMPBIN,TEMPBIN,0.)
            CALL HBARX(ID)
            
            first=.FALSE.
*     
         ENDIF
*     
C==================================================================
*     
      ELSE IF (MOD(iflag,10).EQ.2) THEN
C     ***  Event loop, fill MC histograms ***

C     initialise variables
         Q2=0
         Y=0
         EE=0
         BEAMFLAG=99
         PROTID=0
         LEPID=0
         NB=0
         DO i=1,NMAXB
            B(i)=0
         ENDDO
         NBBAR=0
         DO i=1,NMAXBBAR
            BBAR(i)=0
         ENDDO
         NBBBAR=0
         DO i=1,NMAXBBBAR
            BBBAR(i)=0
         ENDDO
         NMU=0
         DO I=1,NMAXMU
            DO J=1,NMUIND
               MU(i,j)=0
            ENDDO
         ENDDO
         MUMOID=0
         NBHAD=0
         DO I=1,NBHADMAX
            DO J=1,NBHADIND
               BHAD(I,J)=0
            ENDDO
         ENDDO
         TEMPNJETS=0
         NJETS=0
         DO I=1,MAXHZJETS
            DO J=1,8
               TEMPJET(i,j)=0
               JET(i,j)=0
            ENDDO
            JET(i,9)=0
            JET(i,10)=0
         ENDDO
         XGAMMA=0.0
         inlist=.false.
         twojetcut=.false.
         replace=.false.
         stepindex=0
         totindex=0
         DO I=1,MAXARRAY
            ARRAY(I)=0
         ENDDO
         loop_1=0
         loop_2=0
         loop_3=0
         filled=.false.
         matched=.false.         

         CALL HCDIR('//PAWC/'//xxxx,' ')
         CALL HCDIR('//HISTO/'//xxxx,' ')

C     get beam particle and lepton energy
         BEAMFLAG=HZIBEAM(PROTID,LEPID)
         IF ( BEAMFLAG.eq.0 ) THEN
            WRITE(HZUNIT,*) 'Beam: Only Lepton was found, Skip Event'
            GOTO 65
         ELSE IF ( BEAMFLAG.eq.-1 ) THEN
            WRITE(HZUNIT,*) 'Beam: Only Proton was found, Skip Event'
            GOTO 65
         ELSE IF ( BEAMFLAG.gt.1.OR.BEAMFLAG.lt.-1 ) THEN
            WRITE(HZUNIT,*) 'Beam: Something went wrong, Skip Event'
            GOTO 65
         ENDIF
         EE=ABS(PHEP(4,LEPID))
     
C     test kinematic range
         Q2=REAL(HZPHOKIN(1))
         Y=REAL(HZPHOKIN(3))     
         IF (Q2.GT.1.) GOTO 65
         IF (Y.LE.0.2.OR.Y.GE.0.8) GOTO 65
         
         DO I=1,NHEP

C     search for mu+- in final state, make mu list
            IF ( ISTHEP(i).eq.1 ) THEN
               IF ( IDHEP(i).eq.13.OR.IDHEP(i).eq.-13 ) THEN
C     particle id of mother of muon
                  MUMOID=IDHEP(JMOHEP(1,i))
                  IF ( MUMOID.eq.0) THEN
                     WRITE(HZUNIT,*) 'Muon Mother = 0, reject event'
                     GOTO 65
                  ENDIF
C     check if muon mother is not a pion or kaon
C     ID's: 211,-211,311,321,-321
                  IF ( (.NOT.(MUMOID.eq.211)).AND.
     +                 (.NOT.(MUMOID.eq.-211)).AND.
     +                 (.NOT.(MUMOID.eq.311)).AND.
     +                 (.NOT.(MUMOID.eq.321)).AND.
     +                 (.NOT.(MUMOID.eq.-321))) THEN
                     NMU=NMU+1
                     IF ( NMU.gt.NMAXMU ) THEN
                        WRITE(HZUNIT,*) 'Number of muons ',
     +                       NMU,
     1                       ' larger than array with maximal ',
     2                       NMAXMU,' entries, skip event'
                        GOTO 65
                     ENDIF
                     MU(NMU,1)=i
                  ENDIF
               ENDIF
            ENDIF
               
*     search for b, pythia style, ISTHEP<21
            IF ( GEN.eq.'PYT' ) THEN
*     first found b quark is kept, chain b quarks are discarded
               IF ( IDHEP(i).eq.5.AND.ISTHEP(i).ge.10 ) THEN
                  IF (NB.ge.NMAXB) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB
     $                    ,' b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NB
                        IF ( B(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     $                          JMOHEP(1,JMOHEP(1,i)).eq.B(j)) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))). gt.0
     $                          .AND. JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq
     $                          . B(j)) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NB=NB+1
                        B(NB)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ELSE IF ( IDHEP(i).eq.-5.AND.ISTHEP(i).ge.10 ) 
     +                 THEN
                  IF (NBBAR.ge.NMAXBBAR) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB,
     $                    ' anti b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NBBAR
                        IF ( BBAR(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,i)).eq.
     +                          BBAR(j) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                          BBAR(j)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NBBAR=NBBAR+1
                        BBAR(NBBAR)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ENDIF
            ELSE IF ( GEN.eq.'HRW' ) THEN
*     take all b quarks which have ISTHEP=2 (parton before hadronisation
*     quarks have as second mother color information, so only consider
*     first mother
               IF ( IDHEP(i).eq.5.AND.ISTHEP(i).eq.2 ) THEN
                  IF (NB.ge.NMAXB) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB
     $                    ,' b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NB
                        IF ( B(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,i)).eq.B(j)
     +                          ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                          B(j)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NB=NB+1
                        B(NB)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ELSE IF ( IDHEP(i).eq.-5.AND.ISTHEP(i).eq.2 ) 
     +                 THEN
                  IF (NBBAR.ge.NMAXBBAR) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB,
     $                    ' anti b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NBBAR
                        IF ( BBAR(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,i)).eq.
     +                          BBAR(j) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                          BBAR(j)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NBBAR=NBBAR+1
                        BBAR(NBBAR)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ENDIF
            ELSE
               write(hzunit,*) xxxx
     $              ,' generator not implemented for this routine'
               RETURN
            ENDIF
            
*     look for B hadrons, select first in decay chain
            DO j=1,nbrange
               IF ( IDHEP(i).eq.brange(j) ) THEN
*     check if 3 consecutive mothers of i are included in BHAD
                  inlist=.false.
                  DO k=1,NBHAD
                     IF ( BHAD(k,1).eq.JMOHEP(1,i) ) THEN
                        inlist=.true.
                     ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                       JMOHEP(1,JMOHEP(1,i)).eq.BHAD(k,1)
     +                       ) THEN
                        inlist=.true.
                     ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).gt.
     +                       0.AND.
     +                       JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                       BHAD(k,1)
     +                       ) THEN
                        inlist=.true.
                     ENDIF
                  ENDDO
                  IF ( .NOT.inlist ) THEN   
                     IF ( NBHAD.ge.NBHADMAX ) THEN
                        write(hzunit,*) xxxx,' ERROR: More than ',
     +                       NBHADMAX,' B hadrons'
                        RETURN
                     ELSE
                        NBHAD=NBHAD+1
                        BHAD(NBHAD,1)=i
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
            
         ENDDO
     
C     check if b or bbar exist
         IF ( NB.eq.0.AND.NBBAR.eq.0 ) GOTO 65
       
C     sort in pt 
         DO i=1,NB
            DO j=(i+1),NB
               IF ( HZPT(B(j)).gt.HZPT(B(i)) ) THEN
                  exchange = B(j)
                  B(j) = B(i)
                  B(i) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NBBAR
            DO j=(i+1),NBBAR
               IF ( HZPT(BBAR(j)).gt.HZPT(BBAR(i)) ) THEN
                  exchange = BBAR(j)
                  BBAR(j) = BBAR(i)
                  BBAR(i) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NBBBAR
            DO j=(i+1),NBBBAR
               IF ( HZPT(BBBAR(j)).gt.HZPT(BBBAR(i)) ) THEN
                  exchange = BBBAR(j)
                  BBBAR(j) = BBBAR(i)
                  BBBAR(i) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NBHAD
            DO j=(i+1),NBHAD
               IF ( HZPT(BHAD(j,1)).gt.HZPT(BHAD(i,1)) ) THEN
                  exchange = BHAD(j,1)
                  BHAD(j,1) = BHAD(i,1)
                  BHAD(i,1) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NMU
            DO j=(i+1),NMU
               IF ( HZPT(MU(j,1)).gt.HZPT(MU(i,1)) ) THEN
                  exchange = MU(j,1)
                  MU(j,1) = MU(i,1)
                  MU(i,1) = exchange
               ENDIF
            ENDDO
         ENDDO




C     call jet finder in massive kt mode, returns pt ordered jets
         CALL HZJTFIND(10,1.0D0,TEMPNJETS,TEMPJET)

C     check for jet eta cut
C     fill in new list, first array digit is old jetindex
         DO I=1,TEMPNJETS
            IF ( ABS(REAL(TEMPJET(I,1))).le.2.5 ) THEN
               NJETS=NJETS+1
               JET(NJETS,1)=I
               DO J=2,9
                  JET(NJETS,J)=TEMPJET(I,J-1)
               ENDDO
               JET(NJETS,10)=SQRT(JET(NJETS,6)**2+JET(NJETS,7)**2)
            ENDIF
         ENDDO

*     check for jet cuts
         IF ( NJETS.ge.2 ) THEN
            IF ( REAL(JET(1,10)).gt.7.0.AND.REAL(JET(2,10)).gt.6.0 )
     $           THEN
               twojetcut=.true.
            ENDIF
         ENDIF

*     check in which jet the b hadron with highest pt is
         IF ( twojetcut ) THEN
            DO I=1,NBHAD
               DO J=1,NJETS
                  IF ( REAL(JET(J,10)).gt.6.0 ) THEN
                     IF ( IJETNO(BHAD(I,1)).eq.JET(J,1) ) THEN
                        BHAD(I,2)=J
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF

*     if one or more muons, check for muons from which b-quark in list bbbar they come from
         matched=.false.
         stepindex=0
         totindex=0
         IF ( NMU.ge.1 ) THEN
            DO loop_1=1,NMU
               IF ( NBBBAR.ge.1 ) THEN
                  DO loop_2=1,NBBBAR
                     matched=.false.
                     stepindex=0
                     totindex=0
                     IF ( JDAHEP(1,BBBAR(loop_2)).gt.0 ) THEN
                        IF ( JDAHEP(1,BBBAR(loop_2)).le.
     +                       JDAHEP(2,BBBAR(loop_2)) ) THEN
                           DO loop_3=JDAHEP(1,BBBAR(loop_2)),
     +                          JDAHEP(2,BBBAR(loop_2))
                              IF ( loop_3.eq.MU(loop_1,1) ) THEN
                                 MU(loop_1,2)=BBBAR(loop_2)
                                 matched=.true.
                                 stepindex=totindex
                              ELSE
                                 IF ( totindex.ge.maxarray ) THEN
                                    write(hzunit,*) xxxx
     $                               ,' ERROR: More than ',MAXARRAY,'
     $                                   entries in array'
                                    RETURN
                                 ELSE
                                    totindex=totindex+1
                                    array(totindex)=loop_3
                                 ENDIF
                              ENDIF
                           ENDDO
                        ELSE
                           IF ( JDAHEP(1,BBBAR(loop_2)).eq.MU(loop_1,1)
     +                          ) THEN
                              MU(loop_1,2)=BBBAR(loop_2)
                              matched=.true.
                              stepindex=totindex
                           ELSE
                              IF ( totindex.ge.maxarray ) THEN
                                 write(hzunit,*) xxxx
     $                                ,' ERROR: More than ',MAXARRAY
     $                                ,' entries in array'
                                 RETURN
                              ELSE
                                 totindex=totindex+1
                                 array(totindex)=JDAHEP(1,BBBAR(loop_2))
                              ENDIF
                           ENDIF
                        ENDIF
                     ENDIF
                     IF ( .NOT.matched ) THEN
                        DO WHILE ( stepindex.lt.totindex )
                           IF ( .NOT.matched ) THEN
                              stepindex=stepindex+1
                              IF ( JDAHEP(1,array(stepindex)).gt.0) THEN
                                 IF ( JDAHEP(1,array(stepindex)).le.
     +                                JDAHEP(2,array(stepindex)) ) THEN
                                    DO loop_3=JDAHEP(1,
     +                                   array(stepindex)),
     +                                JDAHEP(2,array(stepindex))
                                       IF ( .NOT.Matched ) THEN
                                          IF ( loop_3.eq.MU(loop_1,1) 
     +                                         ) THEN
                                             MU(loop_1,2)=
     +                                            BBBAR(loop_2)
                                             matched=.true.
                                             stepindex=totindex
                                          ELSE
                                             IF ( totindex.ge.
     +                                            maxarray ) 
     +                                            THEN
                                                write(hzunit,*) xxxx,
     $                                               ' ERROR:',
     $                                               ' More than ',
     $                                               MAXARRAY,
     $                                               ' entries in ',
     $                                               'array'
                                                RETURN
                                             ELSE
                                                totindex=totindex+1
                                                array(totindex)=
     +                                               loop_3
                                             ENDIF
                                          ENDIF
                                       ENDIF
                                    ENDDO
                                 ELSE
                                    IF ( JDAHEP(1,array(stepindex))
     +                                   .eq.MU(loop_1,1) ) THEN
                                       MU(loop_1,2)=BBBAR(loop_2)
                                       matched=.true.
                                       stepindex=totindex
                                    ELSE
                                       IF ( totindex.ge.maxarray ) 
     +                                      THEN
                                          write(hzunit,*) xxxx,
     $                                         ' ERROR: More than ',
     $                                         MAXARRAY,
     $                                         ' entries in array'
                                          RETURN
                                       ELSE
                                          totindex=totindex+1
                                          array(totindex)=
     +                                         JDAHEP(1,
     +                                         array(stepindex))
                                       ENDIF
                                    ENDIF
                                 ENDIF
                              ENDIF
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDDO
               ENDIF
            ENDDO
         ENDIF
         
*     if one or more muons, check for muons from which B-hadron in list
*     bhad they come from
         matched=.false.
         stepindex=0
         totindex=0
         IF ( NMU.ge.1 ) THEN
            DO loop_1=1,NMU
               IF ( NBHAD.ge.1 ) THEN
                  DO loop_2=1,NBHAD
                     matched=.false.
                     stepindex=0
                     totindex=0
                     IF ( JDAHEP(1,BHAD(loop_2,1)).gt.0 ) THEN
                        IF ( JDAHEP(1,BHAD(loop_2,1)).le.
     +                       JDAHEP(2,BHAD(loop_2,1)) ) THEN
                           DO loop_3=JDAHEP(1,BHAD(loop_2,1)),
     +                          JDAHEP(2,BHAD(loop_2,1))
                              IF ( loop_3.eq.MU(loop_1,1) ) THEN
                                 MU(loop_1,3)=BHAD(loop_2,1)
                                 matched=.true.
                                 stepindex=totindex
                              ELSE
                                 IF ( totindex.ge.maxarray ) THEN
                                    write(hzunit,*) xxxx
     $                                 ,' ERROR: More than ',MAXARRAY,'
     $                                   entries in array'
                                    RETURN
                                 ELSE
                                    totindex=totindex+1
                                    array(totindex)=loop_3
                                 ENDIF
                              ENDIF
                           ENDDO
                        ELSE
                           IF ( JDAHEP(1,BHAD(loop_2,1)).eq.MU(loop_1,1)
     +                          ) THEN
                              MU(loop_1,3)=BHAD(loop_2,1)
                              matched=.true.
                              stepindex=totindex
                           ELSE
                              IF ( totindex.ge.maxarray ) THEN
                                 write(hzunit,*) xxxx
     $                                ,' ERROR: More than ',MAXARRAY
     $                                ,' entries in array'
                                 RETURN
                              ELSE
                                 totindex=totindex+1
                                 array(totindex)=
     +                                JDAHEP(1,BHAD(loop_2,1))
                              ENDIF
                           ENDIF
                        ENDIF
                     ENDIF
                     IF ( .NOT.matched ) THEN
                        DO WHILE ( stepindex.lt.totindex )
                           IF ( .NOT.matched ) THEN
                              stepindex=stepindex+1
                              IF ( JDAHEP(1,array(stepindex)).gt.0) THEN
                                 IF ( JDAHEP(1,array(stepindex)).le.
     +                                JDAHEP(2,array(stepindex)) ) THEN
                                    DO loop_3=JDAHEP(1,
     +                                   array(stepindex)),
     +                                JDAHEP(2,array(stepindex))
                                       IF ( .NOT.Matched ) THEN
                                          IF ( loop_3.eq.MU(loop_1,1) 
     +                                         ) THEN
                                             MU(loop_1,3)=
     +                                            BHAD(loop_2,1)
                                             matched=.true.
                                             stepindex=totindex
                                          ELSE
                                             IF ( totindex.ge.
     +                                            maxarray ) 
     +                                            THEN
                                                write(hzunit,*) xxxx,
     $                                               ' ERROR:',
     $                                               ' More than ',
     $                                               MAXARRAY,
     $                                               ' entries in ',
     $                                               'array'
                                                RETURN
                                             ELSE
                                                totindex=totindex+1
                                                array(totindex)=
     +                                               loop_3
                                             ENDIF
                                          ENDIF
                                       ENDIF
                                    ENDDO
                                 ELSE
                                    IF ( JDAHEP(1,array(stepindex))
     +                                   .eq.MU(loop_1,1) ) THEN
                                       MU(loop_1,3)=BHAD(loop_2,1)
                                       matched=.true.
                                       stepindex=totindex
                                    ELSE
                                       IF ( totindex.ge.maxarray ) 
     +                                      THEN
                                          write(hzunit,*) xxxx,
     $                                         ' ERROR: More than ',
     $                                         MAXARRAY,
     $                                         ' entries in array'
                                          RETURN
                                       ELSE
                                          totindex=totindex+1
                                          array(totindex)=
     +                                         JDAHEP(1,
     +                                         array(stepindex))
                                       ENDIF
                                    ENDIF
                                 ENDIF
                              ENDIF
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDDO
               ENDIF
            ENDDO
         ENDIF

*     get jet id's for muons by connection via the B-hadron the muon
*     comes from
         DO I=1,NMU
            IF ( MU(I,3).gt.0 ) THEN
               DO J=1,NBHAD
                  IF ( BHAD(J,1).eq.MU(I,3) ) THEN
                     MU(I,4)=BHAD(J,2)
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
         
C         IF ( nmu.gt.0 ) THEN
C            write(hzunit,*) xxxx,' twojetcut ',twojetcut
C            write(hzunit,*) xxxx,' bbbar information'
C            DO I=1,NBBBAR
C               write(hzunit,*) 'bbbar ',I,' id ',BBBAR(I)
C            ENDDO
C            write(hzunit,*) xxxx,' bhad information'
C            DO I=1,NBHAD
C               write(hzunit,*) 'bhad ',I,' id ',BHAD(I,1),' jet ',BHAD(I,2)
C            ENDDO
C            write(hzunit,*) xxxx,' Muon information'
C            DO I=1,NMU
C               write(hzunit,*) 'muon ',I,' id ',MU(I,1),' quark ',MU(I,2),
C     +              ' bhad ',MU(I,3),' jet ',MU(I,4)
C            ENDDO
C         ENDIF


*     fill histo A
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     fill region 1
                  IF ( REAL(HZP(MU(I,1))).gt.2.5.AND.
     +                 REAL(HZETA(MU(I,1))).gt.-1.6.AND.
     +                 REAL(HZETA(MU(I,1))).LT.-0.9 ) THEN
*     fill if muon comes from b decay
                     IF ( MU(I,2).gt.0 ) THEN
                        CALL HFILL(1+100*IPROC,1.,0.,1.)
*                        filled=.true.
                     ENDIF
*     fill region 2
                  ELSE IF ( REAL(HZPT(MU(I,1))).gt.2.5.AND.
     +                    REAL(HZETA(MU(I,1))).gt.-0.9.AND.
     +                    REAL(HZETA(MU(I,1))).LT.1.3 ) THEN
*     fill if muon comes from b decay
                     IF ( MU(I,2).gt.0 ) THEN
                        CALL HFILL(1+100*IPROC,2.,0.,1.)
*                        filled=.true.
                     ENDIF
*     fill region 3
                  ELSE IF ( REAL(HZP(MU(I,1))).gt.4.AND.
     +                    REAL(HZPT(MU(I,1))).gt.1.0.AND.
     +                    REAL(HZETA(MU(I,1))).gt.1.48.AND.
     +                    REAL(HZETA(MU(I,1))).LT.2.3 ) THEN
*     fill if muon comes from b decay
                     IF ( MU(I,2).gt.0 ) THEN
                        CALL HFILL(1+100*IPROC,3.,0.,1.)
*                        filled=.true.
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

*     fill histo B
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            XGAMMA=((JET(1,5)-JET(1,8))+
     +           (JET(2,5)-JET(2,8)))/(2*Y*EE)
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     fill region 1
                  IF ( REAL(HZP(MU(I,1))).gt.2.5.AND.
     +                 REAL(HZETA(MU(I,1))).gt.-1.6.AND.
     +                 REAL(HZETA(MU(I,1))).LT.-0.9 ) THEN
*     fill if muon comes from b decay
                     IF ( MU(I,2).gt.0 ) THEN
                        CALL HFILL(2+100*IPROC,XGAMMA,0.,1.)
*                        filled=.true.
                     ENDIF
                  ELSE IF ( REAL(HZPT(MU(I,1))).gt.2.5.AND.
     +                    REAL(HZETA(MU(I,1))).gt.-0.9.AND.
     +                    REAL(HZETA(MU(I,1))).LT.1.3 ) THEN
*     fill if muon comes from b decay
                     IF ( MU(I,2).gt.0 ) THEN
                        CALL HFILL(2+100*IPROC,XGAMMA,0.,1.)
*                        filled=.true.
                     ENDIF
                  ELSE IF ( REAL(HZP(MU(I,1))).gt.4.AND.
     +                    REAL(HZPT(MU(I,1))).gt.1.0.AND.
     +                    REAL(HZETA(MU(I,1))).gt.1.48.AND.
     +                    REAL(HZETA(MU(I,1))).LT.2.3 ) THEN
*     fill if muon comes from b decay
                     IF ( MU(I,2).gt.0 ) THEN
                        CALL HFILL(2+100*IPROC,XGAMMA,0.,1.)
*                        filled=.true.
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
         
*     fill histo C
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     muon cuts
                  IF ( REAL(HZPT(MU(I,1))).gt.2.5 ) THEN
*     fill if muon comes from b decay
                     IF ( MU(I,2).gt.0 ) THEN
                        CALL HFILL(3+100*IPROC,REAL(HZETA(MU(I,1))),
     +                       0.,1.)
*                        filled=.true.
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

*     fill histo D
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     muon cuts
                  IF ( REAL(HZPT(MU(I,1))).gt.2.5 ) THEN
                     IF ( REAL(HZETA(MU(I,1))).gt.-1.6.AND.
     +                    REAL(HZETA(MU(I,1))).lt.2.3 ) THEN
*     fill if muon comes from b decay
                        IF ( MU(I,2).gt.0 ) THEN
                           CALL HFILL(4+100*IPROC,REAL(HZPT(MU(I,1))),
     +                          0.,1.)
*                           filled=.true.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

*     fill histo E
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     fill if muon comes from b decay
                  IF ( MU(I,2).gt.0 ) THEN
*     get jet which contains muon and fill in muon regions
                     IF ( MU(I,4).gt.0) THEN 
*     fill region 1
                        IF ( REAL(HZP(MU(I,1))).gt.2.5.AND.
     +                       REAL(HZETA(MU(I,1))).gt.-1.6.AND.
     +                       REAL(HZETA(MU(I,1))).LT.-0.9 ) THEN
                           CALL HFILL(5+100*IPROC,REAL(JET(MU(I,4),2)),
     +                          0.,1.)
*                           filled=.true.
*     fill region 2
                        ELSE IF ( REAL(HZPT(MU(I,1))).gt.2.5.AND.
     +                          REAL(HZETA(MU(I,1))).gt.-0.9.AND.
     +                          REAL(HZETA(MU(I,1))).LT.1.3 ) THEN
                           CALL HFILL(5+100*IPROC,REAL(JET(MU(I,4),2)),
     +                          0.,1.)
*                           filled=.true.
*     fill region 3
                        ELSE IF ( REAL(HZP(MU(I,1))).gt.4.AND.
     +                          REAL(HZPT(MU(I,1))).gt.1.0.AND.
     +                          REAL(HZETA(MU(I,1))).gt.1.48.AND.
     +                          REAL(HZETA(MU(I,1))).LT.2.3 ) THEN
                           CALL HFILL(5+100*IPROC,REAL(JET(MU(I,4),2)),
     +                          0.,1.)
*                           filled=.true.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

*     fill histo F
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     fill if muon comes from b decay
                  IF ( MU(I,2).gt.0 ) THEN
*     get jet which contains muon and fill in muon regions
                     IF ( MU(I,4).gt.0) THEN 
*     fill region 1
                        IF ( REAL(HZP(MU(I,1))).gt.2.5.AND.
     +                       REAL(HZETA(MU(I,1))).gt.-1.6.AND.
     +                       REAL(HZETA(MU(I,1))).LT.-0.9 ) THEN
                           CALL HFILL(6+100*IPROC,REAL(JET(MU(I,4),10)),
     +                          0.,1.)
*                           filled=.true.
*     fill region 2
                        ELSE IF ( REAL(HZPT(MU(I,1))).gt.2.5.AND.
     +                          REAL(HZETA(MU(I,1))).gt.-0.9.AND.
     +                          REAL(HZETA(MU(I,1))).LT.1.3 ) THEN
                           CALL HFILL(6+100*IPROC,REAL(JET(MU(I,4),10)),
     +                          0.,1.)
*                           filled=.true.
*     fill region 3
                        ELSE IF ( REAL(HZP(MU(I,1))).gt.4.AND.
     +                          REAL(HZPT(MU(I,1))).gt.1.0.AND.
     +                          REAL(HZETA(MU(I,1))).gt.1.48.AND.
     +                          REAL(HZETA(MU(I,1))).LT.2.3 ) THEN
                           CALL HFILL(6+100*IPROC,REAL(JET(MU(I,4),10)),
     +                          0.,1.)
*                           filled=.true.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF



*     fill histo G
         filled=.false.
         if ( twojetcut ) THEN
            DO I=1,NBHAD
               IF ( BHAD(I,2).gt.0 ) THEN
                  CALL HFILL(7+100*IPROC,REAL(JET(BHAD(I,2),2)),0.,1.)
               ENDIF
            ENDDO
         ENDIF
         

*     fill histo H
         filled=.false.
         if ( twojetcut ) THEN
            DO I=1,NBHAD
               IF ( BHAD(I,2).gt.0 ) THEN
                  CALL HFILL(8+100*IPROC,REAL(JET(BHAD(I,2),10)),0.,1.)
               ENDIF
            ENDDO
         ENDIF

*     fill histo I
         filled=.false.
         IF ( NBBBAR.gt.0 ) THEN
            DO I=1,NBBBAR
               IF ( ABS(REAL(HZETA(BBBAR(I)))).lt.2 ) THEN
                  IF ( .NOT.filled ) THEN
                     CALL HFILL(99+100*IPROC,REAL(HZPT(BBBAR(I))),0.,1.)
*     filled=.true.
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

 65      continue

C==================================================================

      ELSE IF (MOD(iflag,10).eq.3) THEN
C     *** Termination, fix normalisation of MC histograms ***

         IF (IPROC.EQ.0.OR.IPROC.EQ.2) THEN
            WRITE(HZUNIT,*) 'HZ'//xxxx//': finishing off'
         ENDIF

         CALL HCDIR('//PAWC/'//xxxx,' ')
         CALL HCDIR('//HISTO/'//xxxx,' ')

         IF (Xsec.eq.0) THEN
            WRITE(HZUNIT,*) xxxx
     $           ,': termination called with zero cross section'
            WRITE(HZUNIT,*) '          cross section graph meaningless'
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            WRITE(HZUNIT,*) xxxx,' no total events'
            WRITE(HZUNIT,*) '          cross section graph meaningless'
            Ntot=1
         ENDIF
*     norm factor for pb
         lumpb=REAL(Ntot/(1000.*xsec))

c         write(hzunit,*) ' iproc= ',iproc
c         write(hzunit,*) 'HZ',xxxx,': ntot= ', ntot,' xsec (nb)= ',xsec
c         write(hzunit,*) 'norm(',iproc,'): ',norm(iproc)

*     normalise histograms
         CALL HOPERA(100*iproc+1,'+e',100*iproc+1,100*iproc+1,
     +        1/lumpb,0.)
         DO I=2,8
            ID=100*iproc+i
            CALL HZHINRM(ID,0,lumpb,1)
         ENDDO
*     normalise temp histogram, divide by two for gain of statistics by
*     filling b and anti b
         ID=100*iproc+99
         CALL HZHINRM(ID,0,2.*lumpb,1)
* copy bins 1,3,5 to final histogram 100*iproc+9
         CALL HUNPAK(ID,temparr,'',0)
         CALL HUNPKE(ID,temperr,'',0)
         temparr(2)=temparr(3)
         temparr(3)=temparr(5)
         temperr(2)=temperr(3)
         temperr(3)=temperr(5)
         ID=100*iproc+9
         CALL HPAK(ID,temparr)
         CALL HPAKE(ID,temperr)

C     adding up direct and resolved
         IF (iproc.eq.2) THEN
            CALL HOPERA(101,'+e',201,1,1.,1.)
            CALL HOPERA(102,'+e',202,2,1.,1.)
            CALL HOPERA(103,'+e',203,3,1.,1.)
            CALL HOPERA(104,'+e',204,4,1.,1.)
            CALL HOPERA(105,'+e',205,5,1.,1.)
            CALL HOPERA(106,'+e',206,6,1.,1.)
            CALL HOPERA(107,'+e',207,7,1.,1.)
            CALL HOPERA(108,'+e',208,8,1.,1.)
            CALL HOPERA(199,'+e',299,99,1.,1.)
*     copy bins 1,3,5 of histogram 99 to final histogram 9
            ID=99
            CALL HUNPAK(ID,temparr,'',0)
            CALL HUNPKE(ID,temperr,'',0)
            temparr(2)=temparr(3)
            temparr(3)=temparr(5)
            temperr(2)=temperr(3)
            temperr(3)=temperr(5)
            ID=9
            CALL HPAK(ID,temparr)
            CALL HPAKE(ID,temperr)
         ENDIF
*     
      ELSE
         WRITE(HZUNIT,*) 'HZ',xxxx,' Unknown flag',iflag
      ENDIF
      RETURN
      END
                          
