      SUBROUTINE hzh0502010(IFLAG)
****************************************************************************
*  Measurement of beauty production at HERA using events with muons and jets.
*  By H1 Collaboration (A. Aktas et al.). DESY-05-004, Feb 2005. 33pp.
*  Published in Eur.Phys.J.C41:453-467,2005.
*  e-Print: hep-ex/0502010   
* produces histograms for the H1 paper hep-ex/0502010
* based on the routine hzh0312057
*
* Author: Tobias Toll
*
*****************************************************************************
      IMPLICIT NONE
     
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
     
      Character xxxx*8
      Data xxxx/'hz05004'/
     
C     constants
      DOUBLE PRECISION pi
c      double precision hzp
      PARAMETER(pi=3.141592653589793238)
     
C     variables
      INTEGER IFLAG,IPROC,ID,I,J,K
      LOGICAL first
      Data first/.true./
      Save first

*     bins, data and errors for cross section A. x_gamma
      INTEGER NABIN
      PARAMETER (NABIN=3)
      REAL ABIN(NABIN+1)
      REAL ADATA(NABIN),AERROR(NABIN)
      
      DATA ABIN /0.2, 0.5, 0.75, 1.0/
      SAVE ABIN
      DATA ADATA /17.2, 21.4, 86.6/
      SAVE ADATA
      DATA AERROR /5.1, 6.0, 15.2/!pythia
      SAVE AERROR
      
*     bins, data and errors for cross section B. eta(mu)
      INTEGER NBBIN
      PARAMETER (NBBIN=4)
      REAL BBIN(NBBIN+1)
      REAL BDATA(NBBIN),BERROR(NBBIN)
      
      DATA BBIN /-.55, -.15, .25, .65, 1.1/
      SAVE BBIN
      DATA BDATA /19.1, 23.4, 23.9, 21.8/
      SAVE BDATA
      DATA BERROR /4.3, 5.2, 5.1, 4.9/
      SAVE BERROR
      
*     bins, data and errors for cross section D. pt(mu)
      INTEGER NDBIN
      PARAMETER (NDBIN=3)
      REAL DBIN(NDBIN+1)
      REAL DDATA(NDBIN),DERROR(NDBIN)
      
      DATA DBIN /2.5, 3.3, 5.0, 12./
      SAVE DBIN
      DATA DDATA /24.4, 8.1, 1.15/
      SAVE DDATA
      DATA DERROR /4.8, 1.6, .24/
      SAVE DERROR
      
*     bins, data and errors for cross section F. pt(jet1)
      INTEGER NFBIN
      PARAMETER (NFBIN=3)
      REAL FBIN(NFBIN+1)
      REAL FDATA(NFBIN),FERROR(NFBIN)
      
      DATA FBIN /7., 10., 14., 25./
      SAVE FBIN
      DATA FDATA /6.3, 2.9, .83/
      SAVE FDATA
      DATA FERROR /1.2, 0.6, 0.18/
      SAVE FERROR
      

      INTEGER NTEMPBIN
      PARAMETER (NTEMPBIN=5)
      REAL TEMPBIN(NTEMPBIN+1)
      DATA TEMPBIN/8.,9.,13.1,14.1,21.75,22.75/
      REAL temparr(NTEMPBIN+2),temperr(NTEMPBIN+2)

      REAL Q2,Y,LUMPB
      INTEGER BEAMFLAG,PROTID,LEPID
      REAL EE
     
C     final state muon list including IHEP id (1), 
C     id of originating b-quark from BBBAR list (2),
C     id of originating B-hadron from BHAD list (3),
C     jet id in which muon is contained (4)
      INTEGER NMAXMU,NMUIND,MUMOID
      PARAMETER (NMAXMU=6,NMUIND=4)
      INTEGER NMU,MU(NMAXMU,NMUIND)
      REAL MUPT,MUETA

*     b quark variables
      INTEGER NB,NMAXB
      INTEGER NBBAR,NMAXBBAR
      INTEGER NBBBAR,NMAXBBBAR
      PARAMETER (NMAXB=3,NMAXBBAR=3,NMAXBBBAR=6)
      INTEGER B(NMAXB),BBAR(NMAXBBAR),BBBAR(NMAXBBBAR)
      LOGICAL replace
     
C     list of B hadrons
      INTEGER NBHADMAX,NBHADIND
      PARAMETER (NBHADMAX=10,NBHADIND=2)
      INTEGER NBHAD,BHAD(NBHADMAX,NBHADIND)
     
      INTEGER nbrange
      PARAMETER (nbrange = 84)
      INTEGER brange(nbrange)
      data brange / 521,-521,511,-511,553,551,20553,555,100553,10551,
     +     120553,100555,200553,300553,400553,9010553,5122,-5122,-531,
     +     531,-541,541,-523,523,-513,513,-533,533,-543,543,5222,-5222,
     +     5212,-5212,5112,-5112,5322,-5322,5232,-5232,5312,-5312,5132,
     +     -5132,5332,-5332,5224,-5224,5214,-5214,5114,-5114,5334,-5334,
     +     10511,-10511,10513,-10513,10521,-10521,10523,-10523,10531,
     +     -10531,10533,-10533,10541,-10541,10543,-10543,515,-515,525,
     +     -525,535,-535,20513,-20513,20523,-20523,20533,-20533,20543,
     +     -20543 /
      LOGICAL inlist

C     jet finder
C     array has 10 digits, first id from jetfinder, following 8 correspond to
C     jetfinder output array, 10 is pt
      INTEGER NJETS,TEMPNJETS
      DOUBLE PRECISION JET(MAXHZJETS,10),TEMPJET(MAXHZJETS,8)
      LOGICAL twojetcut
     
C     div. variables
      REAL XGAMMA
      INTEGER exchange,loop_1,loop_2,loop_3
      LOGICAL filled,matched
      INTEGER MAXARRAY
      PARAMETER(MAXARRAY=1024)
      INTEGER STEPINDEX,TOTINDEX,ARRAY(MAXARRAY)

c     deltaphi
      real dphi, nevt, E_pzh

*     Determine the process type.
      iproc=(mod(iflag,10000)/1000)
      if ((iproc.lt.0).or.(iproc.gt.2)) then
         write(HZUNIT,*) 'HZ',xxxx,': illegal IPROC:',iproc
         return
      endif


      IF (MOD(iflag,10).eq.1) then
C     *** Initialisation run ***
         IF (first) THEN
            CALL HCDIR('//PAWC',' ')
            CALL HMDIR(xxxx,'S')
            CALL HCDIR('//HISTO',' ')
            CALL HMDIR(xxxx,'S')
         ELSE
            CALL HCDIR('//PAWC/'//xxxx,' ')
            CALL HCDIR('//HISTO/'//xxxx,' ')
         ENDIF

         write(hzunit,*) 'HZ'//xxxx//': Booking histograms'
     
C     book mc histos
         ID=1+100*iproc
         CALL HBOOKB(ID,' d[s]/dx?[g](ep->e bbbar X->e jj [m] X) ',
     +        NABIN,ABIN,0.)
         CALL HBARX(ID)

         ID=2+100*iproc
         CALL HBOOKB(ID,' d[s]/d[e]^[m](ep->e bbbar X->e jj [m] X) ',
     +        NBBIN,BBIN,0.)
         CALL HBARX(ID)

         ID=4+100*iproc
         CALL HBOOKB(ID,' d[s]/dp?T^[m](ep->e bbbar X->e jj [m] X) ',
     +        NDBIN,DBIN,0.)
         CALL HBARX(ID)

         ID=6+100*iproc
         CALL HBOOKB(ID,' d[s]/dp?T^{jet1}(ep->e bbbar X->
     +        e jj [m] X) ',NFBIN,FBIN,0.)
         CALL HBARX(ID)

         IF (first) THEN
c     Book the data.
            ID=1
            CALL HBOOKB(-ID,' d[s]/dx?[g](ep->e bbbar X->e jj [m] X) ',
     +           NABIN,ABIN,0.)
            CALL HPAK(-ID,ADATA)
            CALL HPAKE(-ID,AERROR)
         
            ID=2
            CALL HBOOKB(-ID,' d[s]/d[e]^[m](ep->e bbbar X->
     +e jj [m] X) ',NBBIN,BBIN,0.)
            CALL HPAK(-ID,BDATA)
            CALL HPAKE(-ID,BERROR)
            
            ID=4
            CALL HBOOKB(-ID,' d[s]/dp?T^[m](ep->e bbbar X->
     +           e jj [m] X) ',NDBIN,DBIN,0.)
            CALL HPAK(-ID,DDATA)
            CALL HPAKE(-ID,DERROR)
            
            ID=6
            CALL HBOOKB(-ID,' d[s]/dp?T^{j [m]-jet}'//
     +           '(ep->e bbbar X->e jj [m] X) ',
     +           NFBIN,FBIN,0.)
            CALL HPAK(-ID,FDATA)
            CALL HPAKE(-ID,FERROR)
            
            first=.FALSE.
*     
         ENDIF
*     
C==================================================================
*     
      ELSE IF (MOD(iflag,10).EQ.2) THEN
C     ***  Event loop, fill MC histograms ***


C     initialise variables
         nevt=nevt+1.
         Q2=0
         Y=0
         EE=0
         BEAMFLAG=99
         PROTID=0
         LEPID=0
         NB=0
         DO i=1,NMAXB
            B(i)=0
         ENDDO
         NBBAR=0
         DO i=1,NMAXBBAR
            BBAR(i)=0
         ENDDO
         NBBBAR=0
         DO i=1,NMAXBBBAR
            BBBAR(i)=0
         ENDDO
         NMU=0
         DO I=1,NMAXMU
            DO J=1,NMUIND
               MU(i,j)=0
            ENDDO
         ENDDO
         MUMOID=0
         NBHAD=0
         DO I=1,NBHADMAX
            DO J=1,NBHADIND
               BHAD(I,J)=0
            ENDDO
         ENDDO
         TEMPNJETS=0
         NJETS=0
         DO I=1,MAXHZJETS
            DO J=1,8
               TEMPJET(i,j)=0
               JET(i,j)=0
            ENDDO
            JET(i,9)=0
            JET(i,10)=0
         ENDDO
         XGAMMA=0.0
         inlist=.false.
         twojetcut=.false.
         replace=.false.
         stepindex=0
         totindex=0
         DO I=1,MAXARRAY
            ARRAY(I)=0
         ENDDO
         loop_1=0
         loop_2=0
         loop_3=0
         filled=.false.
         matched=.false.         

         CALL HCDIR('//PAWC/'//xxxx,' ')
         CALL HCDIR('//HISTO/'//xxxx,' ')

C     get beam particle and lepton energy
         BEAMFLAG=HZIBEAM(PROTID,LEPID)
         IF ( BEAMFLAG.eq.0 ) THEN
            WRITE(HZUNIT,*) 'Beam: Only Lepton was found, Skip Event'
            GOTO 65
         ELSE IF ( BEAMFLAG.eq.-1 ) THEN
            WRITE(HZUNIT,*) 'Beam: Only Proton was found, Skip Event'
            GOTO 65
         ELSE IF ( BEAMFLAG.gt.1.OR.BEAMFLAG.lt.-1 ) THEN
            WRITE(HZUNIT,*) 'Beam: Something went wrong, Skip Event'
            GOTO 65
         ENDIF
         EE=ABS(PHEP(4,LEPID))
     
C     test kinematic range
         Q2=REAL(HZPHOKIN(1))
         Y=REAL(HZPHOKIN(3))     
         IF (Q2.GT.1.0) GOTO 65
         IF (Y.LE.0.2.OR.Y.GE.0.8) GOTO 65
         
         DO I=1,NHEP

C     search for mu+- in final state, make mu list
            IF ( ISTHEP(i).eq.1 ) THEN
               IF ( IDHEP(i).eq.13.OR.IDHEP(i).eq.-13 ) THEN
C     particle id of mother of muon
                  MUMOID=IDHEP(JMOHEP(1,i))
                  IF ( MUMOID.eq.0) THEN
                     WRITE(HZUNIT,*) 'Muon Mother = 0, reject event'
                     GOTO 65
                  ENDIF
C     check if muon mother is not a pion or kaon
C     ID's: 211,-211,311,321,-321
                  IF ( (.NOT.(MUMOID.eq.211)).AND.
     +                 (.NOT.(MUMOID.eq.-211)).AND.
     +                 (.NOT.(MUMOID.eq.311)).AND.
     +                 (.NOT.(MUMOID.eq.321)).AND.
     +                 (.NOT.(MUMOID.eq.-321))) THEN
                     NMU=NMU+1
                     IF ( NMU.gt.NMAXMU ) THEN
                        WRITE(HZUNIT,*) 'Number of muons ',
     +                       NMU,
     1                       ' larger than array with maximal ',
     2                       NMAXMU,' entries, skip event'
                        GOTO 65
                     ENDIF
                     MU(NMU,1)=i
                  ENDIF
               ENDIF
            ENDIF
               
*     search for b, pythia style, ISTHEP<21
            IF ( GEN.eq.'PYT' ) THEN
*     first found b quark is kept, chain b quarks are discarded
               IF ( IDHEP(i).eq.5.AND.ISTHEP(i).ge.10 ) THEN
                  IF (NB.ge.NMAXB) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB
     $                    ,' b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NB
                        IF ( B(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     $                          JMOHEP(1,JMOHEP(1,i)).eq.B(j)) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))). gt.0
     $                          .AND. JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq
     $                          . B(j)) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NB=NB+1
                        B(NB)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ELSE IF ( IDHEP(i).eq.-5.AND.ISTHEP(i).ge.10 ) 
     +                 THEN
                  IF (NBBAR.ge.NMAXBBAR) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB,
     $                    ' anti b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NBBAR
                        IF ( BBAR(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,i)).eq.
     +                          BBAR(j) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                          BBAR(j)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NBBAR=NBBAR+1
                        BBAR(NBBAR)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ENDIF
            ELSE IF ( GEN.eq.'HRW' ) THEN
*     take all b quarks which have ISTHEP=2 (parton before hadronisation
*     quarks have as second mother color information, so only consider
*     first mother
               IF ( IDHEP(i).eq.5.AND.ISTHEP(i).eq.2 ) THEN
                  IF (NB.ge.NMAXB) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB
     $                    ,' b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NB
                        IF ( B(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,i)).eq.B(j)
     +                          ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                          B(j)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NB=NB+1
                        B(NB)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ELSE IF ( IDHEP(i).eq.-5.AND.ISTHEP(i).eq.2 ) 
     +                 THEN
                  IF (NBBAR.ge.NMAXBBAR) THEN
                     write(hzunit,*) xxxx,' ERROR: More than ',NMAXB,
     $                    ' anti b quarks'
                     RETURN
                  ELSE
*     check 3 consecutive mother if they are already in list 
                     inlist=.false.
                     DO j=1,NBBAR
                        IF ( BBAR(j).eq.JMOHEP(1,i) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,i)).eq.
     +                          BBAR(j) ) THEN
                           inlist=.true.
                        ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).
     +                          gt.0.AND.
     +                          JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                          BBAR(j)
     +                          ) THEN
                           inlist=.true.
                        ENDIF
                     ENDDO
                     IF ( .NOT.inlist ) THEN
                        NBBAR=NBBAR+1
                        BBAR(NBBAR)=i
                        NBBBAR=NBBBAR+1
                        BBBAR(NBBBAR)=i
                     ENDIF
                  ENDIF
               ENDIF
            ELSE
               write(hzunit,*) xxxx
     $              ,' generator not implemented for this routine'
               RETURN
            ENDIF
            
*     look for B hadrons, select first in decay chain
            DO j=1,nbrange
               IF ( IDHEP(i).eq.brange(j) ) THEN
*     check if 3 consecutive mothers of i are included in BHAD
                  inlist=.false.
                  DO k=1,NBHAD
                     IF ( BHAD(k,1).eq.JMOHEP(1,i) ) THEN
                        inlist=.true.
                     ELSE IF ( JMOHEP(1,JMOHEP(1,i)).gt.0.AND.
     +                       JMOHEP(1,JMOHEP(1,i)).eq.BHAD(k,1)
     +                       ) THEN
                        inlist=.true.
                     ELSE IF ( JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).gt.
     +                       0.AND.
     +                       JMOHEP(1,JMOHEP(1,JMOHEP(1,i))).eq.
     +                       BHAD(k,1)
     +                       ) THEN
                        inlist=.true.
                     ENDIF
                  ENDDO
                  IF ( .NOT.inlist ) THEN   
                     IF ( NBHAD.ge.NBHADMAX ) THEN
                        write(hzunit,*) xxxx,' ERROR: More than ',
     +                       NBHADMAX,' B hadrons'
                        RETURN
                     ELSE
                        NBHAD=NBHAD+1
                        BHAD(NBHAD,1)=i
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
            
         ENDDO
     
C     check if b or bbar exist
         IF ( NB.eq.0.AND.NBBAR.eq.0 ) GOTO 65
       
C     sort in pt 
         DO i=1,NB
            DO j=(i+1),NB
               IF ( HZPT(B(j)).gt.HZPT(B(i)) ) THEN
                  exchange = B(j)
                  B(j) = B(i)
                  B(i) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NBBAR
            DO j=(i+1),NBBAR
               IF ( HZPT(BBAR(j)).gt.HZPT(BBAR(i)) ) THEN
                  exchange = BBAR(j)
                  BBAR(j) = BBAR(i)
                  BBAR(i) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NBBBAR
            DO j=(i+1),NBBBAR
               IF ( HZPT(BBBAR(j)).gt.HZPT(BBBAR(i)) ) THEN
                  exchange = BBBAR(j)
                  BBBAR(j) = BBBAR(i)
                  BBBAR(i) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NBHAD
            DO j=(i+1),NBHAD
               IF ( HZPT(BHAD(j,1)).gt.HZPT(BHAD(i,1)) ) THEN
                  exchange = BHAD(j,1)
                  BHAD(j,1) = BHAD(i,1)
                  BHAD(i,1) = exchange
               ENDIF
            ENDDO
         ENDDO
         DO i=1,NMU
            DO j=(i+1),NMU
               IF ( HZPT(MU(j,1)).gt.HZPT(MU(i,1)) ) THEN
                  exchange = MU(j,1)
                  MU(j,1) = MU(i,1)
                  MU(i,1) = exchange
               ENDIF
            ENDDO
         ENDDO




C     call jet finder in massive kt mode, returns pt ordered jets
         CALL HZJTFIND(10,1.0D0,TEMPNJETS,TEMPJET)

C     check for jet eta cut
C     fill in new list, first array digit is old jetindex
         DO I=1,TEMPNJETS
            IF ( ABS(REAL(TEMPJET(I,1))).le.2.5 ) THEN
               NJETS=NJETS+1
               JET(NJETS,1)=I
               DO J=2,9
                  JET(NJETS,J)=TEMPJET(I,J-1)
               ENDDO
               JET(NJETS,10)=SQRT(JET(NJETS,6)**2+JET(NJETS,7)**2)
            ENDIF
         ENDDO

*     check for jet cuts
         IF ( NJETS.ge.2 ) THEN
            IF ( REAL(JET(1,10)).gt.7.0.AND.REAL(JET(2,10)).gt.6.0 )
     $           THEN
               twojetcut=.true.
            ENDIF
         ENDIF

*     check in which jet the b hadron with highest pt is
         IF ( twojetcut ) THEN
            DO I=1,NBHAD
               DO J=1,NJETS
                  IF ( REAL(JET(J,10)).gt.6.0 ) THEN
                     IF ( IJETNO(BHAD(I,1)).eq.JET(J,1) ) THEN
                        BHAD(I,2)=J
                     ENDIF
                  ENDIF
               ENDDO
            ENDDO
         ENDIF

*     if one or more muons, check for muons from which b-quark in list bbbar they come from
         matched=.false.
         stepindex=0
         totindex=0
         IF ( NMU.ge.1 ) THEN
            DO loop_1=1,NMU
               IF ( NBBBAR.ge.1 ) THEN
                  DO loop_2=1,NBBBAR
                     matched=.false.
                     stepindex=0
                     totindex=0
                     IF ( JDAHEP(1,BBBAR(loop_2)).gt.0 ) THEN
                        IF ( JDAHEP(1,BBBAR(loop_2)).le.
     +                       JDAHEP(2,BBBAR(loop_2)) ) THEN
                           DO loop_3=JDAHEP(1,BBBAR(loop_2)),
     +                          JDAHEP(2,BBBAR(loop_2))
                              IF ( loop_3.eq.MU(loop_1,1) ) THEN
                                 MU(loop_1,2)=BBBAR(loop_2)
                                 matched=.true.
                                 stepindex=totindex
                              ELSE
                                 IF ( totindex.ge.maxarray ) THEN
                                    write(hzunit,*) xxxx
     $                               ,' ERROR: More than ',MAXARRAY,'
     $                                   entries in array'
                                    RETURN
                                 ELSE
                                    totindex=totindex+1
                                    array(totindex)=loop_3
                                 ENDIF
                              ENDIF
                           ENDDO
                        ELSE
                           IF ( JDAHEP(1,BBBAR(loop_2)).eq.MU(loop_1,1)
     +                          ) THEN
                              MU(loop_1,2)=BBBAR(loop_2)
                              matched=.true.
                              stepindex=totindex
                           ELSE
                              IF ( totindex.ge.maxarray ) THEN
                                 write(hzunit,*) xxxx
     $                                ,' ERROR: More than ',MAXARRAY
     $                                ,' entries in array'
                                 RETURN
                              ELSE
                                 totindex=totindex+1
                                 array(totindex)=JDAHEP(1,BBBAR(loop_2))
                              ENDIF
                           ENDIF
                        ENDIF
                     ENDIF
                     IF ( .NOT.matched ) THEN
                        DO WHILE ( stepindex.lt.totindex )
                           IF ( .NOT.matched ) THEN
                              stepindex=stepindex+1
                              IF ( JDAHEP(1,array(stepindex)).gt.0) THEN
                                 IF ( JDAHEP(1,array(stepindex)).le.
     +                                JDAHEP(2,array(stepindex)) ) THEN
                                    DO loop_3=JDAHEP(1,
     +                                   array(stepindex)),
     +                                JDAHEP(2,array(stepindex))
                                       IF ( .NOT.Matched ) THEN
                                          IF ( loop_3.eq.MU(loop_1,1) 
     +                                         ) THEN
                                             MU(loop_1,2)=
     +                                            BBBAR(loop_2)
                                             matched=.true.
                                             stepindex=totindex
                                          ELSE
                                             IF ( totindex.ge.
     +                                            maxarray ) 
     +                                            THEN
                                                write(hzunit,*) xxxx,
     $                                               ' ERROR:',
     $                                               ' More than ',
     $                                               MAXARRAY,
     $                                               ' entries in ',
     $                                               'array'
                                                RETURN
                                             ELSE
                                                totindex=totindex+1
                                                array(totindex)=
     +                                               loop_3
                                             ENDIF
                                          ENDIF
                                       ENDIF
                                    ENDDO
                                 ELSE
                                    IF ( JDAHEP(1,array(stepindex))
     +                                   .eq.MU(loop_1,1) ) THEN
                                       MU(loop_1,2)=BBBAR(loop_2)
                                       matched=.true.
                                       stepindex=totindex
                                    ELSE
                                       IF ( totindex.ge.maxarray ) 
     +                                      THEN
                                          write(hzunit,*) xxxx,
     $                                         ' ERROR: More than ',
     $                                         MAXARRAY,
     $                                         ' entries in array'
                                          RETURN
                                       ELSE
                                          totindex=totindex+1
                                          array(totindex)=
     +                                         JDAHEP(1,
     +                                         array(stepindex))
                                       ENDIF
                                    ENDIF
                                 ENDIF
                              ENDIF
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDDO
               ENDIF
            ENDDO
         ENDIF
         
*     if one or more muons, check for muons from which B-hadron in list
*     bhad they come from
         matched=.false.
         stepindex=0
         totindex=0
         IF ( NMU.ge.1 ) THEN
            DO loop_1=1,NMU
               IF ( NBHAD.ge.1 ) THEN
                  DO loop_2=1,NBHAD
                     matched=.false.
                     stepindex=0
                     totindex=0
                     IF ( JDAHEP(1,BHAD(loop_2,1)).gt.0 ) THEN
                        IF ( JDAHEP(1,BHAD(loop_2,1)).le.
     +                       JDAHEP(2,BHAD(loop_2,1)) ) THEN
                           DO loop_3=JDAHEP(1,BHAD(loop_2,1)),
     +                          JDAHEP(2,BHAD(loop_2,1))
                              IF ( loop_3.eq.MU(loop_1,1) ) THEN
                                 MU(loop_1,3)=BHAD(loop_2,1)
                                 matched=.true.
                                 stepindex=totindex
                              ELSE
                                 IF ( totindex.ge.maxarray ) THEN
                                    write(hzunit,*) xxxx
     $                                 ,' ERROR: More than ',MAXARRAY,'
     $                                   entries in array'
                                    RETURN
                                 ELSE
                                    totindex=totindex+1
                                    array(totindex)=loop_3
                                 ENDIF
                              ENDIF
                           ENDDO
                        ELSE
                           IF ( JDAHEP(1,BHAD(loop_2,1)).eq.MU(loop_1,1)
     +                          ) THEN
                              MU(loop_1,3)=BHAD(loop_2,1)
                              matched=.true.
                              stepindex=totindex
                           ELSE
                              IF ( totindex.ge.maxarray ) THEN
                                 write(hzunit,*) xxxx
     $                                ,' ERROR: More than ',MAXARRAY
     $                                ,' entries in array'
                                 RETURN
                              ELSE
                                 totindex=totindex+1
                                 array(totindex)=
     +                                JDAHEP(1,BHAD(loop_2,1))
                              ENDIF
                           ENDIF
                        ENDIF
                     ENDIF
                     IF ( .NOT.matched ) THEN
                        DO WHILE ( stepindex.lt.totindex )
                           IF ( .NOT.matched ) THEN
                              stepindex=stepindex+1
                              IF ( JDAHEP(1,array(stepindex)).gt.0) THEN
                                 IF ( JDAHEP(1,array(stepindex)).le.
     +                                JDAHEP(2,array(stepindex)) ) THEN
                                    DO loop_3=JDAHEP(1,
     +                                   array(stepindex)),
     +                                JDAHEP(2,array(stepindex))
                                       IF ( .NOT.Matched ) THEN
                                          IF ( loop_3.eq.MU(loop_1,1) 
     +                                         ) THEN
                                             MU(loop_1,3)=
     +                                            BHAD(loop_2,1)
                                             matched=.true.
                                             stepindex=totindex
                                          ELSE
                                             IF ( totindex.ge.
     +                                            maxarray ) 
     +                                            THEN
                                                write(hzunit,*) xxxx,
     $                                               ' ERROR:',
     $                                               ' More than ',
     $                                               MAXARRAY,
     $                                               ' entries in ',
     $                                               'array'
                                                RETURN
                                             ELSE
                                                totindex=totindex+1
                                                array(totindex)=
     +                                               loop_3
                                             ENDIF
                                          ENDIF
                                       ENDIF
                                    ENDDO
                                 ELSE
                                    IF ( JDAHEP(1,array(stepindex))
     +                                   .eq.MU(loop_1,1) ) THEN
                                       MU(loop_1,3)=BHAD(loop_2,1)
                                       matched=.true.
                                       stepindex=totindex
                                    ELSE
                                       IF ( totindex.ge.maxarray ) 
     +                                      THEN
                                          write(hzunit,*) xxxx,
     $                                         ' ERROR: More than ',
     $                                         MAXARRAY,
     $                                         ' entries in array'
                                          RETURN
                                       ELSE
                                          totindex=totindex+1
                                          array(totindex)=
     +                                         JDAHEP(1,
     +                                         array(stepindex))
                                       ENDIF
                                    ENDIF
                                 ENDIF
                              ENDIF
                           ENDIF
                        ENDDO
                     ENDIF
                  ENDDO
               ENDIF
            ENDDO
         ENDIF

*     get jet id's for muons by connection via the B-hadron the muon
*     comes from
         DO I=1,NMU
            IF ( MU(I,3).gt.0 ) THEN
               DO J=1,NBHAD
                  IF ( BHAD(J,1).eq.MU(I,3) ) THEN
                     MU(I,4)=BHAD(J,2)
                  ENDIF
               ENDDO
            ENDIF
         ENDDO
         
C         IF ( nmu.gt.0 ) THEN
C            write(hzunit,*) xxxx,' twojetcut ',twojetcut
C            write(hzunit,*) xxxx,' bbbar information'
C            DO I=1,NBBBAR
C               write(hzunit,*) 'bbbar ',I,' id ',BBBAR(I)
C            ENDDO
C            write(hzunit,*) xxxx,' bhad information'
C            DO I=1,NBHAD
C               write(hzunit,*) 'bhad ',I,' id ',BHAD(I,1),' jet ',BHAD(I,2)
C            ENDDO
C            write(hzunit,*) xxxx,' Muon information'
C            DO I=1,NMU
C               write(hzunit,*) 'muon ',I,' id ',MU(I,1),' quark ',MU(I,2),
C     +              ' bhad ',MU(I,3),' jet ',MU(I,4)
C            ENDDO
C         ENDIF


*     fill histo B
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            XGAMMA=((JET(1,5)-JET(1,8))+
     +           (JET(2,5)-JET(2,8)))/(2*Y*EE)
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
                  IF ( REAL(HZPT(MU(I,1))).gt.2.5.AND.
     +                 REAL(HZETA(MU(I,1))).gt.-0.55.AND.
     +                 REAL(HZETA(MU(I,1))).LT.1.1 ) THEN
*     fill if muon comes from b decay
                     IF ( MU(I,2).gt.0 ) THEN
                        CALL HFILL(1+100*IPROC,XGAMMA,0.,wtx)
*                        filled=.true.
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
         
*     fill histo C
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     muon cuts
                  IF ( REAL(HZPT(MU(I,1))).gt.2.5 ) THEN
*     fill if muon comes from b decay
                     IF ( MU(I,2).gt.0 ) THEN
                        CALL HFILL(2+100*IPROC,REAL(HZETA(MU(I,1))),
     +                       0.,wtx)
*                        filled=.true.
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

*     fill histo D
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     muon cuts
                  IF ( REAL(HZPT(MU(I,1))).gt.2.5 ) THEN
                     IF ( REAL(HZETA(MU(I,1))).gt.-0.55.AND.
     +                    REAL(HZETA(MU(I,1))).lt.1.1 ) THEN
*     fill if muon comes from b decay
                        IF ( MU(I,2).gt.0 ) THEN
                           CALL HFILL(4+100*IPROC,REAL(HZPT(MU(I,1))),
     +                          0.,wtx)
*                           filled=.true.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

*     fill histo F
         filled=.false.
         IF ( twojetcut.AND.NMU.gt.0 ) THEN
            DO I=1,NMU
               IF ( .NOT.filled ) THEN
*     fill if muon comes from b decay
                  IF ( MU(I,2).gt.0 ) THEN
                     IF ( REAL(HZPT(MU(I,1))).gt.2.5.AND.
     +                    REAL(HZETA(MU(I,1))).gt.-.55.AND.
     +                    REAL(HZETA(MU(I,1))).LT.1.1 ) THEN
                        CALL HFILL(6+100*IPROC,REAL(JET(1,10)),
     +                       0.,wtx)
*                           filled=.true.
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF

 65      continue

C==================================================================

      ELSE IF (MOD(iflag,10).eq.3) THEN
C     *** Termination, fix normalisation of MC histograms ***

         IF (IPROC.EQ.0.OR.IPROC.EQ.2) THEN
            WRITE(HZUNIT,*) 'HZ'//xxxx//': finishing off'
         ENDIF

         CALL HCDIR('//PAWC/'//xxxx,' ')
         CALL HCDIR('//HISTO/'//xxxx,' ')

         IF (Xsec.eq.0) THEN
            WRITE(HZUNIT,*) xxxx
     $           ,': termination called with zero cross section'
            WRITE(HZUNIT,*) '          cross section graph meaningless'
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            WRITE(HZUNIT,*) xxxx,' no total events'
            WRITE(HZUNIT,*) '          cross section graph meaningless'
            Ntot=1
         ENDIF
*     norm factor for pb
         lumpb=REAL(Ntot/(xsec))
         write(hzunit,*) '#events=', ntot, 'xsec=', xsec, 'lumpb=',lumpb
c         write(hzunit,*) ' iproc= ',iproc
c         write(hzunit,*) 'HZ',xxxx,': ntot= ', ntot,' xsec (nb)= ',xsec
c         write(hzunit,*) 'norm(',iproc,'): ',norm(iproc)

*     normalise histograms
         DO I=1,10
            ID=100*iproc+i
            CALL HZHINRM(ID,0,lumpb,1)
         ENDDO
         
C     adding up direct and resolved
         IF (iproc.eq.2) THEN
            CALL HOPERA(101,'+e',201,1,1.,1.)
            CALL HOPERA(102,'+e',202,2,1.,1.)
            CALL HOPERA(103,'+e',203,3,1.,1.)
            CALL HOPERA(104,'+e',204,4,1.,1.)
            CALL HOPERA(105,'+e',205,5,1.,1.)
            CALL HOPERA(106,'+e',206,6,1.,1.)
            CALL HOPERA(107,'+e',207,7,1.,1.)
            CALL HOPERA(108,'+e',208,8,1.,1.)
            CALL HOPERA(109,'+e',209,12,1.,1.)
            CALL HOPERA(110,'+e',210,10,1.,1.)
         ENDIF
*     
      ELSE
         WRITE(HZUNIT,*) 'HZ',xxxx,' Unknown flag',iflag
      ENDIF
      RETURN
      END
                          
