      subroutine HZH9810020(IFLAG)
*
*************************************************************************
* Purpose: book and fill histograms for the H1 paper, DESY-98-148,
*          Charged Particle Cross Sections in Photoproduction
*
* Authors: Stefan Lausberg   (Heidelberg "Miniresearch" student)
*          Victor Lendermann (supervisor)        Created at: 31.01.2005
*
* Arguments: iflag=1000*[IPROC]+1 initialisation
*            iflag=1000*[IPROC]+2 filling
*            iflag=1000*[IPROC]+3 termination
*
* IPROC = 1 direct photoproduction
* IPROC = 2 resolved photoproduction
* IPROC = 0 both together
*
* Beams: 27.5 GeV positrons, 820 GeV protons   [1994 HERA running]
*
* Event selection:   0.3 < y < 0.7, which corresponds to 165. < W < 251.
*                    (the cuts are actually applied on W)
*                    0.00 < Q**2 < 0.01
*                    -1.0 < eta < 1.0
*                    pt > 2 or pt > 3
*
* Histograms filled by the MC generator:
*
*     101 Transverse momentum squared       for Figure 2a
*         d sigma / d pt^2 d eta            -1.0 < eta < 1.0
*
*     103 Pseudorapidity (pt > 2 GeV/c)     for Figure 3a
*         d sigma / d eta                   -1.0 < eta < 1.0
*
*     104 Pseudorapidity (pt > 3 GeV/c)     for Figure 3b
*         d sigma / d eta                   -1.0 < eta < 1.0
*
* DATA histograms have corresponding negative numbers: -101, -103, -104.
* They are filled by values published in paper DESY-98-148.
*
*************************************************************************
*
      IMPLICIT NONE
*
* Function declarations for Hztool functions
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzhbook.inc"
* #include "hzjetcmn.inc"
*
      Character*8 xxxx
      Integer IFLAG, IPROC
      Real    Nevt, lumnb, lumpb
      save nevt
*
      Data xxxx /'H9810020'/
      save xxxx
*
      integer icycle, istat           ! histo storage options
      integer istef                   ! counter for loops
      double precision pt             ! transverse momentum
      real hzeta2, W
      real cont1(10), cont2(10)       ! arrays for HUNPAK
      integer charge                  ! charge of created particle
*
      real Flux                       ! photon flux F (see eq. 2)
      parameter (Flux = 0.0136)
*
* --- Values to fill data histograms - Table 1, page 11
* Measured differential x-sections as a function of pt
* Abscissa: ptx          Ordinate: dxpt
*                        Errors:   dxpte
*
      real ptx(27)
      data ptx /
     +     2.00, 2.08, 2.16, 2.24, 2.34, 2.44, 2.54, 2.66, 2.78,
     +     2.90, 3.06, 3.22, 3.38, 3.58, 3.78, 4.02, 4.30, 4.58,
     +     4.92, 5.32, 5.72, 6.20, 6.90, 7.80, 8.80, 10.0, 12.0 /
*
      real dxpt(26)
      data dxpt /
     +     685.0, 539.1, 434.2, 359.9, 274.3, 216.6, 184.8, 135.9,
     +     102.8, 78.32, 56.41, 45.62, 31.01, 23.47, 14.33, 11.79,
     +     6.905, 5.210, 3.224, 1.673, 1.148, 0.631, 0.408, 0.232,
     +     0.066, 0.019 /
*
      real dxpte(26)
      data dxpte /
     +      35.2,  29.3,  24.7,  20.7,  16.7,  13.9,  12.1,  9.53,
     +      7.64,  5.97,  4.59,  3.97,  2.82,  2.32,  1.49,  1.33,
     +     0.885, 0.692, 0.465, 0.283, 0.225, 0.126, 0.100, 0.068,
     +     0.020, 0.007 /
*
* --- Values to fill data histograms - Table 2, page 11
* Measured differential x-sections as a function of pseudorapidity eta
* Abscissa: eta          Ordinate: dxsc2   and  dxsc3
*                        Errors:   dxsc2f  and  dxsc3f
* The indices 2 and 3 stand for 2 and 3 GeV/c
*
      real eta(11)
      data eta /
     +-1.0 , -0.8 , -0.6 , -0.4 , -0.2 ,
     + 0.0 ,  0.2 ,  0.4 ,  0.6 ,  0.8 ,
     + 1.0 /
*
      real dxsc2(10)
      data dxsc2 /
     + 1431.1 , 1473.1 , 1556.3 , 1593.3 , 1611.9 ,
     + 1643.8 , 1730.2 , 1721.3 , 1790.4 , 1721.2 /
*
      real dxsc3(10)
      data dxsc3 /
     + 293.9 , 312.3 , 327.6 , 350.3 , 337.4 ,
     + 343.7 , 396.8 , 346.1 , 353.3 , 351.4 /
*
      real dxsc2f(10)
      data dxsc2f /
     + 77.8 , 79.4 , 83.2 , 85.1 , 85.3 ,
     + 87.5 , 91.2 , 90.9 , 94.0 , 90.6 /
*
      real dxsc3f(10)
      data dxsc3f /
     + 27.2 , 28.4 , 29.3 , 30.7 , 29.6 ,
     + 29.8 , 34.8 , 31.4 , 31.3 , 31.5 /
*
*************************************************************************
*

*     Determine the process type.
      iproc=iflag/1000
      if ((iproc.lt.0).or.(iproc.gt.2)) then
         write(hzunit,*) 'HZ',xxxx,': illegal IPROC:',iproc
         return
      endif

*
*************************************************************************
*
*                      Initialization
*
*************************************************************************
* Initialisation: The following MUST always be done
* (i) make subdirectory in PAWC
*    - use the name as the xxxxxx in HZxxxxxx subroutine
* (i) make subdirectory in o/p file
*
      if (MOD(iflag,10).eq.1) then
         write(hzunit,*) 'HZ',xxxx,': booking histograms'
         Nevt = 0.
*
* Change to  //PAWC (' ' means CHANGE, 'R' means READ)
         call hcdir('//PAWC',' ')
* Subdirectory xxxx is created. 'S' changes it to the current directory
* (xxxx is a character variable, which was set to xxxx = '98148')
         if (iproc.le.1) then
            call hmdir(xxxx,'S')
         else
            call hcdir(xxxx,' ')  
         endif
 
         call hcdir('//HISTO',' ')
         if (iproc.le.1) then 
            call hmdir(xxxx,'S')
         else
            call hcdir(xxxx,' ')  
         endif

*
         call hbookb( 1000*iproc+101,'MC p?t! (Fig. 2a)'  ,26,ptx,0.)
         call hbook1( 1000*iproc+103,'MC [c] (Fig. 3a)'   ,10,-1.,1.,0.)
         call hbook1( 1000*iproc+104,'MC [c] (Fig. 3b)'   ,10,-1.,1.,0.)

         call hbarx(1000*iproc+101)
         call hbarx(1000*iproc+103)
         call hbarx(1000*iproc+104)
*
         if (iproc.le.1) THEN
            call hbookb(-101,'Data p?t! (Fig. 2a)',26,ptx,0.)
            call hbook1(-103,'Data [c] (Fig. 3a)' ,10,-1.0,1.0,0.)
            call hbook1(-104,'Data [c] (Fig. 3b)' ,10,-1.0,1.0,0.)

            call hpak(-101,dxpt)
            call hpake(-101,dxpte)
            call hpak(-103,dxsc2)
            call hpak(-104,dxsc3)
            call hpake(-103,dxsc2f)
            call hpake(-104,dxsc3f)

         endif

      endif
*
*************************************************************************
*
*                      Event Processing
*
*************************************************************************
* Filling: The following MUST always be done
* (i) move to the correct sub-directory in PAWC
*
      if (MOD(iflag,10).eq.2) then
*
         call hcdir('//PAWC/'//xxxx,' ')      ! change to //PAWC/98148
         Nevt = Nevt + wtx
*
         W = sqrt(real(HZDISKIN(4)))
         if (W.gt.165. .and. W.lt.251.) then
            do 100 istef = 1, nhep
               hzeta2 = real(HZETA(istef))
               if (hzeta2.ge.-1. .and. hzeta2.le.1.) then
                  charge = int(HZLCHGE(idhep(istef))/3)
*                     charged      and  stable
                  if (charge.ne.0 .and. isthep(istef).eq.1) then
                     pt = dsqrt(phep(1,istef)**2 + phep(2,istef)**2)
                     if (pt.gt.0D0) then
* filled weight = event weight / (2 p_t) / 2 (=delta eta from -1 to 1)
                        call hfill(1000*iproc+101, real(pt), 0., wtx/4.
     $                       /real(pt))
                     endif
                     if (pt.gt.2D0) then
c      print*,' pt = ',pt,' charge = ',charge,' id = ',idhep(istef)
                        call hfill(1000*iproc+103, hzeta2, 0., wtx)
                        if (pt.gt.3D0) then
                           call hfill(1000*iproc+104, hzeta2, 0., wtx)
                        endif
                     endif
                  endif
               endif
 100        continue
         endif
*
      endif
*
*************************************************************************
*
*                      Termination
*
*************************************************************************
*
      if (MOD(iflag,10).eq.3) then
*
         call hcdir('//PAWC/'//xxxx,' ')
         lumnb = 999999.
         lumpb = 999999.
         if (xsec.ne.0.) then
            lumnb = Nevt / real(xsec)
            lumpb = Nevt / (1000.*real(xsec))
c            if (iproc.eq.0)  write(hzunit,*) 'Resolved+direct : '
c            if (iproc.eq.1)  write(hzunit,*) 'Direct : '
c            if (iproc.eq.2)  write(hzunit,*) 'Resolved : '
c            write(hzunit,*) 'Number of events processed   : '
c     $           ,nevt
c            write(hzunit,*) 'Total cross section [nb]     : '
c     $           ,xsec
c            write(hzunit,*) 'Equivalent luminosity [nb-1] : ',lumnb
         else
            write(hzunit,*) 'HZ',xxxx,': xsec = 0 !!!'
         endif
*
*     normalise histograms
         call hzhinrm(1000*iproc+101,0,lumnb*Flux,1)
         call hzhinrm(1000*iproc+103,0,lumnb*Flux,1)
         call hzhinrm(1000*iproc+104,0,lumnb*Flux,1)
*
c         call hunpak(1000*iproc+103,cont1,'prox',0)
c         call hunpak(1000*iproc+104,cont2,'prox',0)
*
*         write(6,*) ' '
*         write(6,*) ' '
*         write(6,'(a11,a11,a29,a10)') ' ','Monte Carlo',' ',
*     &        'Experiment'
*         write(6,*) ' '
*         write(6,'(a8,2a15,a10,2a15)') 'eta','dsigma/deta',
*     &        'dsigma/deta',' ','dsigma/deta','dsigma/deta'
*         write(6,'(a8,2a15,a10,2a15)') ' ','pt > 2 GeV/c',
*     &        'pt > 3 GeV/c',' ','pt > 2 GeV/c','pt > 3 GeV/c'
*         write(6,*) ' '
*         do istef=1,10
*            write(6,'(f8.1,2f15.2,a10,2f15.2)') eta(istef),
*     &           cont1(istef),cont2(istef),' ',
*     &           dxsc2(istef),dxsc3(istef)
*         enddo

*     finally, if this is the last pass and we generated direct and resolved
*     seperately, add them.
         if (iproc.eq.2) then
            call hopera(1101,'+e',2101,101,1.,1.)
            call hopera(1103,'+e',2103,103,1.,1.)
            call hopera(1104,'+e',2104,104,1.,1.)
         endif

*
      endif
*
*************************************************************************
*
*                      End of mainprogram
*
*************************************************************************
*
      RETURN
      END
*
