	Subroutine hzi02849(iflag)
*****************************************************************************
*       This subroutine calculates the variables that span the multijet 
*       parameter space for 3 and 4-jet events.
*       
*       This photoproduction routine has to be run twice with the
c       following
*       code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the RESOLVED component run.
*       for all three phases (iflag=1,2,3)
*       
*       +10000 to run on parton showers instead of final state particles
c       .
*       
*       iflag = 1,2 or 3 depending on whether it is the initialization,
*       processing or termination phase (respectively)
*       
*       The default jetfinder is KTCLUS
*       To change jetfinder use these values to call the initialisation
c       phase.
*       1 + 10 * (jetfinder number)
*       
*       It is possible to produce sets of histograms with up to four
c       different
*       invariant mass cuts. To set the number to 1, 2, 3 or 4, change
c       the
*       value of the parameter massnum accordingly and comment out any 
*       masscut(i) that will now be out of array bounds
*       
*       Monte Carlo histograms produced:
*       
*       All histograms beginning with '3' are for 3-jets
*       1   id 3010-3016 !
*       2   id 3020-3026 ! are the number of direct events
*       3   id 3030-3036 ! which pass the cuts
*       4   id 3040-3046 !
*       
*       1   id 3050-3056 !
*       2   id 3060-3066 ! are the number of resolved events
*       3   id 3070-3076 ! which pass the cuts
*       4   id 3080-3086 !
*       
*       1   id 310-316   !
*       2   id 320-326   ! are cross-sections in nb
*       3   id 330-336   ! 
*       4   id 340-346   !
*       
*       All histograms beginning with '4' are for 4-jets
*       1   id 4010-4018 !
*       2   id 4020-4028 ! are the number of direct events
*       3   id 4030-4038 ! which pass the cuts
*       4   id 4040-4048 !
*       
*       1   id 4050-4058 !
*       2   id 4060-4068 ! are the number of resolved events
*       3   id 4070-4078 ! which pass the cuts
*       4   id 4080-4088 !
*       
*       1   id 410-418   !
*       2   id 420-428   ! are cross-sections in nb
*       3   id 430-438   ! 
*       4   id 440-448   !

*       These graphs are 
*       meaningless unless Xsec and Ntot are set before
*       calling the termination routine.
*       (Xsec - total cross section returned by MC)
*       (Ntot - number of events passed to this routine)
*       
*       Cuts : 	Q2 < 1 GeV^2           ^  
*       0.2 < y < 0.8          ^  
*       And at least two jets with
*       E_T^jet > 6 GeV and    ^  
*       & third & fourth with  
*       E_T^jet > 5 GeV        ^
*       eta^JET < |2.4|        ^  
*       In addition   X_3 < 0.95             ^  
*       |cos(theta_3)| < 0.8   ^  
*       
*       Acta Phys.Polon.B33:3123-3128,2002
*       Author : Claire Gwenlan
*       
*****************************************************************************
*       
*       HERA Tuning defined commons
	
	implicit none
        
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
	character*6 jetf

	integer NJET, njeti, counter3j(0:2,2,4),counter4j(0:2,2,4)
	save counter4j, counter3j
        integer loop, loop2, count, i, j, k
        integer massnum
        parameter (massnum=4)
	double precision  CONER
	save coner
        double precision elecE, protE
	double precision ybj,q2,xgam3,xgam4,xprot3,xprot4
	double precision P1_dbl(4),P2_dbl(4)
	double precision P1_lab(4),P2_lab(4)
	double precision mhxsec(2),mhntot(2) 
        real dir_xsec3j(massnum),res_xsec3j(massnum),tot_xsec3j(massnum)
        real dir_xsec4j(massnum),res_xsec4j(massnum),tot_xsec4j(massnum)
        real rap(4), Et(4), Ettmp

        integer cut_test(2)
        real    ybjcut(2),q2cut(2),Masscut(massnum)
	real    rapcut(2),Etcut(2),X3_cut,cos_t3_cut(2)
        save    ybjcut,q2cut,Masscut
	save    rapcut,Etcut,X3_cut,cos_t3_cut
	
        double precision xboost,yboost,zboost
	double precision P3new_dbl(4),P4new_dbl(4),P5new_dbl(4)
     1                   ,P6new_dbl(4) 
	double precision P3_act(4),P4_act(4),P5_act(4),P6_act(4)
        double precision Pnew_dbl(4), Ptot_act(4)
	
        real    P3lab(4),P4lab(4),P5lab(4),P6lab(4)
        real    Ptot_test(6,4),Mom2_tot_test(6),E2_test(6),Mass_test(6)
        real    m1, m2, m3, m4, m5, m6
	real    Ptot(4), PA(4), PB(4), P3(4),P4(4),P5(4)
        real    Mom2_tot, E2, E, m345, m3456
	real    Pnew(4), PAnew(4), PBnew(4)
        real    P3new(4),P4new(4),P5new(4),P6new(4)
        real    P3new_prime(4),P4new_prime(4),P5new_prime(4)
        real    P3_prime(4),P4_prime(4),P5_prime(4)
	real    PAB(4),Pav(4),P1(4),P2(4)

	real    cross1(4),cross2(4),cross3(4),cross4(4)
        real    dot1,dot2,dot3,dot4,dot5,dot6,dot7,dot8,dot9
	real    testy(4)
        real    X3, X4, X5, cos_theta3, cos_psi3, psi3
        real    X3_prime, X4_prime, X5_prime
        real    cos_theta3_prime, cos_psi3_prime, psi3_prime
        real    cos_psiAB, psiAB, XA

C *** Data for Invariant Mass
	real xbin0(16),xbin0_new(16)
	real bin0_err(15)

	real xbin00(10),xbin00_new(10)
	real bin00_err(9)

	integer nbins1, nbins2, nbins5
	integer nbins11, nbins12, nbins15
	integer nbins111, nbins112, nbins115
	parameter( nbins1=16, nbins2=16, nbins5=16 )
	parameter( nbins11=12, nbins12=12, nbins15=12 )
	parameter( nbins111=10, nbins112=10, nbins115=10 )

c       *** cos_theta_3
	real xbin1(nbins1+1),xbin1_new(nbins1+1)
	real bin1_err(nbins1)

	real xbin11(nbins11+1),xbin11_new(nbins11+1)
	real bin11_err(nbins11)

	real xbin111(nbins111+1),xbin111_new(nbins111+1)
	real bin111_err(nbins111)

c       *** psi_3
	real xbin2(nbins2+1),xbin2_new(nbins2+1)
	real bin2_err(nbins2)

	real xbin12(nbins12+1),xbin12_new(nbins12+1)
	real bin12_err(nbins12)

	real xbin112(nbins112+1),xbin112_new(nbins112+1)
	real bin112_err(nbins112)

c       *** X3
	real xbin3(8),xbin3_new(8)
	real bin3_err(7)

c       *** X4
	real xbin4(11),xbin4_new(11)
	real bin4_err(10)

c       *** X5
	real xbin7(11),xbin7_new(11)
	real bin7_err(10)

c       *** psiAB
        real xbin5(nbins5+1),xbin5_new(nbins5+1)
        real bin5_err(nbins5)

        real xbin15(nbins15+1),xbin15_new(nbins15+1)
        real bin15_err(nbins15)

        real xbin115(nbins115+1),xbin115_new(nbins115+1)
        real bin115_err(nbins115)

c       *** XA
        real xbin6(11),xbin6_new(11)
        real bin6_err(10)

        real xbin16(7),xbin16_new(7)
        real bin16_err(6)

        real xbin116(10),xbin116_new(10)
        real bin116_err(9)

c       *** xgam3 and xgam4, y
        real xbin8(11),xbin8_new(11)
        real bin8_err(10)

        real xbin18(9),xbin18_new(9)
        real bin18_err(8)

c       *** xprot3 and xprot4
        real xbin9(21),xbin9_new(21)
        real bin9_err(20)

c       *** y
        real xbin10(10),xbin10_new(10)
        real bin10_err(9)

	real xfactor(2)
        real inv_xtot

        data xbin00/0.,25.,30.,40.,50.,65.,90.,120.,160.,200./
        data xbin0/20, 25, 30, 35, 
     1             40, 45, 50, 55, 60, 65, 72, 80, 90, 100, 120, 160/
        data xbin1/-0.8, -0.7, -0.6, -0.5, -0.4, -0.3, -0.2, -0.1, 0.0,
     1            0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8/	
        data xbin11/-0.8,-0.66667,-0.53333,-0.4,-0.266667,-0.13333,0,
     1              0.13333,0.266667,0.4,0.53333,0.66667,0.8/
        data xbin111/-0.8, -0.64, -0.48, -0.32, -0.16, 0.0, 0.16, 0.32, 
     1            0.48,0.64, 0.8/	
        data xbin2/0.0,0.19625,0.3925,0.58875,0.785,0.98125,1.1775,
     1            1.37375,1.57,1.76625,1.9625,2.15875,2.355,
     1            2.55125,2.7475,2.94375,3.14/
        data xbin12/0.0,0.26166667,0.523333,0.785,1.046667,1.3083333,
     1       1.57,1.83166667,2.093333,2.355,2.6166667,2.8783333,3.14/
        data xbin112/0.0,0.314,0.628,0.942,1.256,1.57,1.884,
     1            2.198,2.512,2.826,3.14/
        data xbin3/0.67, 0.71, 0.75, 0.79, 0.83, 0.87, 0.91, 0.95/
        data xbin4/0.5, 0.55, 0.6, 0.65, 0.7, 0.75, 0.8, 0.85,
     1            0.9, 0.95, 1.0/
        data xbin5/0.0, 0.19625, 0.3925, 0.58875, 0.785, 0.98125,
     1            1.1775, 1.37375, 1.57, 1.76625, 1.9625, 2.15875, 
     1            2.355, 2.55125, 2.7475, 2.94375, 3.14/
        data xbin15/0.0,0.26166667,0.523333,0.785,1.046667,1.3083333,
     1       1.57,1.83166667,2.093333,2.355,2.6166667,2.8783333,3.14/
        data xbin115/0.0,0.314,0.628,0.942,1.256,1.57,1.884,
     1            2.198,2.512,2.826,3.14/
        data xbin6/0.5, 0.55, 0.6, 0.65, 0.7, 0.75, 0.8, 0.85,
     1            0.9, 0.95, 1.0/
        data xbin16/0.5,0.58,0.66, 0.74, 0.82, 0.9, 1.0/
        data xbin116/0.5,0.55,0.6, 0.65, 0.7, 0.75, 0.8,0.85,0.9,1.0/
        data xbin7/0., 0.07, 0.14, 0.21, 0.28, 0.35, 0.42, 0.49,
     1            0.56, 0.63, 0.70/
        data xbin8/0., 0.10, 0.20, 0.30, 0.40, 0.50, 0.60, 0.70,
     1            0.80, 0.90, 1.0/
        data xbin18/0., 0.125, 0.25, 0.375, 0.50, 0.625, 0.75, 0.875,
     1            1.0/
        data xbin9/0., 0.025,0.05, 0.075, 0.1, 0.125, 0.15, 0.175,
     1             0.2, 0.225, 0.25, 0.275, 0.3, 0.325, 0.35, 0.375,
     1            0.40, 0.425, 0.45, 0.475, 0.5/
	data xbin10/0.0,0.20, 0.292857, 0.385714, 0.478571,
     1            0.571429,0.664286,0.757143,0.85,1.0/

c       Measured cross sections as of ICHEP02849
	integeri_bin


	real cterr1(nbins1),ctstat1(nbins1),
     &       ctsigma1(nbins1),ctsys1(nbins1)
	real phierr1(nbins2),phistat1(nbins2),
     &       phisigma1(nbins2),phisys1(nbins2)
	real cterr2(nbins1),ctstat2(nbins1),
     &       ctsigma2(nbins1),ctsys2(nbins1)
	real phierr2(nbins2),phistat2(nbins2),
     &       phisigma2(nbins2),phisys2(nbins2)
	real x3err1(7),x3stat1(7),x3sigma1(7),x3sys1(7)
	real x4err1(10),x4stat1(10),x4sigma1(10),x4sys1(10)
	real x3err2(7),x3stat2(7),x3sigma2(7),x3sys2(7)
	real x4err2(10),x4stat2(10),x4sigma2(10),x4sys2(10)
	real xgerr1(10),xgstat1(10),xgsigma1(10),xgsys1(10)
	real xgerr2(10),xgstat2(10),xgsigma2(10),xgsys2(10)
	real yerr1(10),ystat1(10),ysigma1(10),ysys1(10)
	real yerr2(10),ystat2(10),ysigma2(10),ysys2(10)
c
c       *Inclusive Sample*
c        
c       CosTheta3    ||  411
	DATA cterr1 / 16*0 /
	DATA ctsigma1 / 0.586665,0.528975,0.579715,0.584106,0.593543,
     &   0.455071,0.575617,0.59067,0.543411,0.637503,0.674174,0.58414
     &    ,0.692469,0.740835,0.784154,0.84895 /
	DATA ctstat1 / 0.0362824,0.0340723,0.0381383,0.0379676,0.0394546
     &    ,0.0315845,0.0368761,0.0395239,0.0340377,0.0405114,0
     &    .0397215,0.0341109,0.0391429,0.0392368,0.0402239,0.0439611 /
	DATA ctsys1 / 0.1128511, 0.0962033,
     &    .1113660,   .0962033,
     &    0.104304,   0.0852424, 
     &    0.0903647  ,    0.1274323,  
     &    0.0848345 ,    0.0974298,  
     &    0.1093035,    0.0835001,  
     &    0.0915022 ,   0.1320349,  
     &    0.1101014  ,   0.1087376 /

c       Psi3    ||  412
	DATA phierr1 / 16*0 /
      DATA phisigma1 / 0.262654,0.261804,0.369467,0.387046,0.324996
     &     ,0.282047,0.227062,0.243799,0.265285,0.289183,
     &     0.331516, 
     &     0.367674,0.447282,0.380203,0.352893,0.302629 /
	DATA phistat1 /0.0179879,0.017215 ,0.0213156,0.0226822,0.0194586
     &    ,0.0183214,0.0165255,0.0184314,0.0181073,0.018556 ,
     &    .0191039,0.0200141,0.0223154,0.0208591,0.0199607,0.0182975/

	DATA phisys1 /  0.0471082,  0.0557254,  0.0572234,  0.0526569,  
     &     0.0450273,  0.0503327,  0.0424823,  0.0600737
     &    ,  0.0512141,  0.051299,
     &    0.0410101,  0.0532076,  
     &    0.0556419,  0.0536229,  
     &    0.0593071,  0.0429049 /

c       x3    ||  413
	DATA x3err1 / 7*0 /
	DATA x3sigma1 / 0.875328, 1.96877,  3.12547,  4.0811 ,  5.00081,
     &    4.85255,  5.09597 /
	DATA x3stat1 /0.0698802,0.106287 ,0.137164 ,0.149992 ,0.175874
     &    ,0.162159 ,0.167969 /
	DATA x3sys1 /0.1929678,  0.27864, 
     &     0.407706, 0.371357, 0.428091, 0.463243, 0.446458/


c       x4    ||  414
	DATA x4err1 / 10*0 /
	DATA x4sigma1 / 0.368595,2.73807  ,4.82171  ,5.73396  ,3.66136
     &    ,1.92511  ,0.686141 ,0.0650449,0        ,0        /
	DATA x4stat1 / 0.0381202, 0.110168 , 0.146658 , 0.16423  , 
     &    0.129127 , 0.0955689, 0.0581865, 0.01925  , 0        , 0
     &    /
	DATA x4sys1 /0.1108432,0.317064,0.358161, 0.402504,
     &      0.412711,0.226887, 0.1387834, 0.053009, 
     &      0.00078, 0  /  

c       XGamma     || 418
	DATA xgerr1 / 10*0 /
	DATA xgsigma1 / 0,124.23 , 295.396, 352.075, 393.18, 413.211,
     &    274.687, 335.305, 525.837 ,278.832 /
	DATA xgstat1 / 0,7.45114,11.8566,13.7166,15.5239,16.8618,12.0509
     &    ,15.5681,23.643 ,18.2257 /
	DATA xgsys1 / 0, 57.0221,
     &     91.2 , 94.6549, 67.4419, 61.5782, 
     &     42.4369, 44.3878, 64.0287, 46.694/ 

c       Y         || 419
	DATA yerr1 / 10*0 /
	DATA ysigma1 / 0,0,292.413,475.813,553.711,525.693,519.586,433
     &    .331,238.548, 0 /
	DATA ystat1 /  0      , 0      , 21.9243, 24.1201, 22.4096, 18
     &    .7755, 16.6866, 14.207 , 10.8844, 0      /
	DATA ysys1 / 0, 0, 81.0948, 72.8112, 89.5806,
     &    78.722, 97.7195, 86.809, 265.2431, 0/ 

c       CosTheta3    ||  431
	DATA cterr2 / 16*0 /
	DATA ctsigma2 /0.970891,0.641804,0.548746,0.361469,0.341578,0
     &     .363941,0.296497,0.375629,0.235027,0.533038,0.415246,0
     &    .465861,0.896445,1.17797 ,0.995339,1.38052 /
	DATA ctstat2 /0.119854 ,0.0911128,0.0892747,0.0700573,0.06399
     &    ,0.0670673,0.0529227,0.0740433,0.0461784,0.0960996,0
     &    .0796065,0.0731392,0.132954 ,0.151103 ,0.112374 ,0.138039 /
	DATA ctsys2 /0.301565,0.215053
     &    , 0.241084   ,0.2071645,
     &    0.1448539,0.1528207, 
     &    0.138756  ,0.1689602, 
     &    0.112545  ,0.2169685, 
     &    0.1991002  , 0.1783652,
     &    0.41527   ,0.436314,
     &    0.302481,0.323067/

c       psi3    ||  432
	DATA phierr2 / 16*0 /
	DATA phisigma2 / 0.35628 , 0.275354, 0.36766 , 0.338148, 0
     &    .289996, 0.229627, 0.180199, 0.255359, 0.165023, 0.314041,
     &    0.242478, 0.302481, 0.437892, 0.556263, 0.397368, 0.387371/
	DATA phistat2 / 0.0531945, 0.0443235, 0.050256 , 0.0512916, 0
     &    .044005 , 0.036071 , 0.0349242, 0.0470565, 0.0313427, 0
     &    .0508037, 0.0373989, 0.048491 , 0.0545393, 0.0707861, 0
     &    .0546805, 0.0510109/
	DATA phisys2 /0.1239161, 0.1353533
     &    ,0.1578685,0.1577947
     &    ,0.0961077,0.1010492
     &    ,0.094299,0.1201923
     &    ,0.1048678,0.176624
     &    ,0.0822075,0.1230812
     &    ,0.1393149,0.1832517
     &    ,0.1692049,0.1109554/


c       x3    ||  433
	DATA x3err2 / 7*0 /
	DATA x3sigma2 /1.06065,2.80715,3.90918,4.46059,
     &    4.98148,3.87225,3.9087 /
	DATA x3stat2 / 0.178458, 0.31265 , 0.351285, 0.390191, 0.44276 ,
     &    0.365078, 0.39669 /
	DATA x3sys2 / 0.589041, 0.81644, 
     &    0.959138, 1.004226, 1.248632, 0.872293, 0.958951/

c       x4    ||  434
	DATA x4err2 / 10*0 /
	DATA x4sigma2 / 0.187382, 1.64567 , 3.39706 , 6.0479  , 4.11565
     &    , 2.84222 , 1.50668 , 0.257442, 0       , 0       /
	DATA x4stat2 /0.0770148,0.237398 ,0.282918 ,0.415975 ,0.328033
     &    ,0.305281 ,0.222129 ,0.093176 ,0        ,0        /
	DATA x4sys2 / 0.2169248,  0.714267,
     &    0.742949,  1.023383, 0.915288,0.767894, 0.612779
     &    , 0.262835,  0.00472755, 0/

c       XGamma     || 438
	DATA xgerr2 / 10*0 /
	DATA xgsigma2 /  0      , 0      , 50.3944, 32.9479, 46.6318, 52
     &    .1766, 45.1084, 95.891 , 186.858, 154.069/
	DATA xgstat2 /  0 , 0 , 18.5967, 5.51419, 5.64699, 
     &    5.27514, 4.13463, 8.1385 , 14.4059, 14.1481/
	DATA xgsys2 /0,0,88.3468,35.5268,49.5213,46.0663,32.9355,  59.1275
     &    ,98.6843,60.3403/

c       Y         || 439
	DATA yerr2 / 10*0 /
	DATA ysigma2 / 0      ,0      ,20.9289,71.6928,98.9242,107.434
     &    ,128.499,119.658,85.3735,0      /
	DATA ystat2 /  0,0      ,5.92703,9.56681,9.83523,9.21625,9
     &   .80057,9.53774,9.92189,0      /
	DATA ysys2 /    0,    0,    20.9026,    45.11,    83.4832,
     &    66.8654,    73.8182,    63.2033,    113.1696,    0/ 

c       Scale factors to convert low & high mass data plots to nb.
	real norm_scale1, norm_scale2
	parameter (norm_scale1=0.295)
	parameter (norm_scale2=0.063)

	integer intiflag, chjet, iproc, iflag
	save chjet

C       --- same jet finder as in paper -- KTCLUS
	data chjet/3/

	real pttemp(12)
	integer ivec,jvec

	LOGICAL parton_showers

	parton_showers=.FALSE.
	intiflag=iflag
	IF (intiflag.gt.10000) THEN
	   parton_showers=.TRUE.
	   intiflag=MOD(iflag,10000)
	ENDIF
	
C       --- choose direct/resolved
C       --- recommend 1 - DIRECT, 2 - RESOLVED
	if (intiflag.gt.1000) then
	   iproc=INT(intiflag/1000)
	   intiflag=MOD(intiflag,1000)
	endif
	if ((iproc.gt.2).or.(iproc.lt.1)) then
	   PRINT*,'ichep02849   : Badly called routine'
	   PRINT*,'          please set intiflag +1000 or +2000'
	endif
	
C       --- choose jet finder
	if ((intiflag.gt.10).and.(MOD(intiflag,10).eq.1)) then
	   chjet=INT(intiflag/10)
	   intiflag=MOD(intiflag,10)
	endif
	
	IF (parton_showers) THEN
	   chjet=-1*chjet
	ENDIF

	if (intiflag.eq.1) then  

C       *** Initialisation run ***
C       *** Create directories ***

	   if (iproc.eq.1) then
	      CALL HCDIR('//HISTO',' ')
	      IF (parton_showers) THEN
		 CALL HMDIR('psi02849','S')
		 CALL HCDIR('//PAWC',' ')
		 CALL HMDIR('psi02849','S')
	      ELSE
		 CALL HMDIR('i02849','S')
		 CALL HCDIR('//PAWC',' ')
		 CALL HMDIR('i02849','S')
	      ENDIF
	   endif
	   
	   if (iproc.eq.2) then
	      Ntot = 0
	   endif
	   IF (parton_showers) THEN
	      CALL HCDIR('//HISTO/psi02849',' ')
	      CALL HCDIR('//PAWC/psi02849',' ')
	   ELSE
	      CALL HCDIR('//HISTO/i02849',' ')
	      CALL HCDIR('//PAWC/i02849',' ')
	   ENDIF
	   
C       --- 3-jet histograms

	   if (iproc.eq.1) then
             do i=1,massnum
	       call hbookb((3000+i*10),
     &	     'M345, Direct',15,xbin0,0.)
	       call hbookb((3001+i*10),
     &       'Cos_theta3, Direct',nbins1,xbin1,0.)
	       call hbookb((3002+i*10),
     &       'psi3, Direct',nbins2,xbin2,0.)
	       call hbookb((3003+i*10),'X3, Direct',7,xbin3,0.)
	       call hbookb((3004+i*10),'X4, Direct',10,xbin4,0.)
               call hbookb((3005+i*10),'X5, Direct',10,xbin7,0.)
               call hbookb((3006+i*10),'xgam, Direct',10,xbin8,0.)
               call hbookb((3007+i*10),'yjb, Direct',10,xbin8,0.)
c              Histos for different bins of xgamma 
               call hbookb((30011+i*100),'Cos3, Dir',nbins1,xbin1,0.)
               call hbookb((30012+i*100),'Cos3, Dir',nbins1,xbin1,0.)
               call hbookb((30013+i*100),'Cos3, Dir',nbins1,xbin1,0.)
               call hbookb((30014+i*100),'Cos3, Dir',nbins1,xbin1,0.)
               call hbookb((30021+i*100),'psi3, Dir',nbins2,xbin2,0.)
               call hbookb((30022+i*100),'psi3, Dir',nbins2,xbin2,0.)
               call hbookb((30023+i*100),'psi3, Dir',nbins2,xbin2,0.)
               call hbookb((30024+i*100),'psi3, Dir',nbins2,xbin2,0.)
               call hbookb((30025+i*100),'psi3, Dir',nbins2,xbin2,0.)
               call hbookb((30026+i*100),'psi3, Dir',nbins2,xbin2,0.)

C       ---- 3-jet histograms      
	       call hbookb((300+i*10),'Invariant Mass M3J',15,xbin0,0.)
	       call hbookb((301+i*10),'Cos_theta3',nbins1,xbin1,0.)
	       call hbookb((302+i*10),'psi3',nbins2,xbin2,0.)
	       call hbookb((303+i*10),'X3',7,xbin3,0.)
	       call hbookb((304+i*10),'X4',10,xbin4,0.)  
               call hbookb((305+i*10),'X5',10,xbin7,0.) 
               call hbookb((306+i*10),'xgam',10,xbin8,0.) 
               call hbookb((307+i*10),'yjb',10,xbin8,0.)
c              Histos for different bins of xgamma 
               call hbookb((3011+i*100),'Cos3',nbins1,xbin1,0.)
               call hbookb((3012+i*100),'Cos3',nbins1,xbin1,0.)
               call hbookb((3013+i*100),'Cos3',nbins1,xbin1,0.)
               call hbookb((3014+i*100),'Cos3',nbins1,xbin1,0.)
               call hbookb((3021+i*100),'psi3',nbins2,xbin2,0.)
               call hbookb((3022+i*100),'psi3',nbins2,xbin2,0.)
               call hbookb((3023+i*100),'psi3',nbins2,xbin2,0.)
               call hbookb((3024+i*100),'psi3',nbins2,xbin2,0.) 
               call hbookb((3025+i*100),'psi3',nbins2,xbin2,0.)
               call hbookb((3026+i*100),'psi3',nbins2,xbin2,0.) 
	     enddo
	   endif

	   if (iproc.eq.2) then
             do i=1,massnum
	       call hbookb((3040+i*10),'M345, Resolved',15,xbin0,0.)
	       call hbookb((3041+i*10),
     &  	    'Cos_theta3, Resolved',nbins1,xbin1,0.)
	       call hbookb((3042+i*10),'psi3, Resolved',nbins2,xbin2,0.)
	       call hbookb((3043+i*10),'X3, Resolved',7,xbin3,0.)
	       call hbookb((3044+i*10),'X4, Resolved',10,xbin4,0.)
               call hbookb((3045+i*10),'X5, Resolved',10,xbin7,0.)
               call hbookb((3046+i*10),'xgam, Resolved',10,xbin8,0.)
               call hbookb((3047+i*10),'yjb, Resolved',10,xbin8,0.)
c              Histos for different bins of xgamma 
               call hbookb((30411+i*100),'Cos3, Res',nbins1,xbin1,0.)
               call hbookb((30412+i*100),'Cos3, Res',nbins1,xbin1,0.)
               call hbookb((30413+i*100),'Cos3, Res',nbins1,xbin1,0.)
               call hbookb((30414+i*100),'Cos3, Res',nbins1,xbin1,0.)
               call hbookb((30421+i*100),'psi3, Res',nbins2,xbin2,0.)
               call hbookb((30422+i*100),'psi3, Res',nbins2,xbin2,0.)
               call hbookb((30423+i*100),'psi3, Res',nbins2,xbin2,0.)
               call hbookb((30424+i*100),'psi3, Res',nbins2,xbin2,0.)
               call hbookb((30425+i*100),'psi3, Res',nbins2,xbin2,0.)
               call hbookb((30426+i*100),'psi3, Res',nbins2,xbin2,0.)
             enddo
	   endif
	   
C       --- 4-jet histograms
	   
           if (iproc.eq.1) then
             do i=1,massnum
		call hbookb((4000+i*10),'M3456, Direct',15,xbin0,0.)
	        call hbookb((14000+i*10),'M3456, Direct',9,xbin00,0.)
		call hbookb((4001+i*10),'Cos_theta3_p, Direct',nbins1
     &  	     ,xbin1,0.)
		call hbookb((14001+i*10),'Cos_theta3_p, Direct',nbins11
     &  	     ,xbin11,0.)
		call hbookb((114001+i*10),
     &               'Cos_theta3_p, Direct',nbins111,xbin111,0.)
		call hbookb((4002+i*10),'psi3_p, Direct'
     &	     ,nbins2,xbin2,0.)
		call hbookb((14002+i*10),'psi3_p, Direct'
     &	     ,nbins12,xbin12,0.)
		call hbookb((114002+i*10),'psi3_p, Direct'
     &	     ,nbins112,xbin112,0.)
	       call hbookb((4003+i*10),'X3_p, Direct',7,xbin3,0.)
	       call hbookb((4004+i*10),'X4_p, Direct',10,xbin4,0.)
               call hbookb((4005+i*10),'X5_p, Direct',10,xbin7,0.)
               call hbookb((4006+i*10),'psiAB, Direct',nbins5,xbin5,0.)
               call hbookb((14006+i*10),'psiAB, Direct',nbins15,
     1                     xbin15,0.)
               call hbookb((114006+i*10),'psiAB, Direct',nbins115,
     1                     xbin115,0.)
               call hbookb((4007+i*10),'XA, Direct',10,xbin6,0.)
               call hbookb((14007+i*10),'XA, Direct',6,xbin16,0.)
               call hbookb((114007+i*10),'XA, Direct',9,xbin116,0.)
               call hbookb((4008+i*10),'xgam, Direct',10,xbin8,0.) 
               call hbookb((14008+i*10),'xgam, Direct',8,xbin18,0.) 
               call hbookb((4009+i*10),'y, Direct',10,xbin8,0.)
               call hbookb((14009+i*10),'y, Direct',9,xbin10,0.)
c              Histos for different bins of xgamma 
               call hbookb((40011+i*100),'Cos3p, Dir',nbins1,xbin1,0.)
               call hbookb((40012+i*100),'Cos3p, Dir',nbins1,xbin1,0.)
               call hbookb((40013+i*100),'Cos3p, Dir',nbins1,xbin1,0.)
               call hbookb((40014+i*100),'Cos3p, Dir',nbins1,xbin1,0.)
               call hbookb((40015+i*100),'Cos3p, Dir',nbins1,xbin1,0.)
               call hbookb((40016+i*100),'Cos3p, Dir',nbins1,xbin1,0.)
               call hbookb((40021+i*100),'psi3p, Dir',nbins2,xbin2,0.)
               call hbookb((40022+i*100),'psi3p, Dir',nbins2,xbin2,0.)
               call hbookb((40023+i*100),'psi3p, Dir',nbins2,xbin2,0.)
               call hbookb((40024+i*100),'psi3p, Dir',nbins2,xbin2,0.)
               call hbookb((40025+i*100),'psi3p, Dir',nbins2,xbin2,0.)
               call hbookb((40026+i*100),'psi3p, Dir',nbins2,xbin2,0.)

               call hbookb((140011+i*100),'Cos3p, Dir',
     1                     nbins11,xbin11,0.)
               call hbookb((140012+i*100),'Cos3p, Dir',
     1                     nbins11,xbin11,0.)
               call hbookb((140013+i*100),'Cos3p, Dir',
     1                     nbins11,xbin11,0.)
               call hbookb((140014+i*100),'Cos3p, Dir',
     1                      nbins11,xbin11,0.)
               call hbookb((140015+i*100),'Cos3p, Dir',
     1                      nbins11,xbin11,0.)
               call hbookb((140016+i*100),'Cos3p, Dir',
     1                      nbins11,xbin11,0.)
               call hbookb((140021+i*100),'psi3p, Dir',
     1                      nbins12,xbin12,0.)
               call hbookb((140022+i*100),'psi3p, Dir',
     1                      nbins12,xbin12,0.)
               call hbookb((140023+i*100),'psi3p, Dir',
     1                      nbins12,xbin12,0.)
               call hbookb((140024+i*100),'psi3p, Dir',
     1                      nbins12,xbin12,0.)
               call hbookb((140025+i*100),'psi3p, Dir',
     1                      nbins12,xbin12,0.)
               call hbookb((140026+i*100),'psi3p, Dir',
     1                      nbins12,xbin12,0.)

               call hbookb((1140011+i*100),'Cos3p, Dir',
     1                     nbins111,xbin111,0.)
               call hbookb((1140012+i*100),'Cos3p, Dir',
     1                     nbins111,xbin111,0.)
               call hbookb((1140013+i*100),'Cos3p, Dir',
     1                     nbins111,xbin111,0.)
               call hbookb((1140014+i*100),'Cos3p, Dir',
     1                     nbins111,xbin111,0.)
               call hbookb((1140015+i*100),'Cos3p, Dir',
     1                     nbins111,xbin111,0.)
               call hbookb((1140016+i*100),'Cos3p, Dir',
     1                     nbins111,xbin111,0.)
               call hbookb((1140021+i*100),'psi3p, Dir',
     1                     nbins112,xbin112,0.)
               call hbookb((1140022+i*100),'psi3p, Dir',
     1                     nbins112,xbin112,0.)
               call hbookb((1140023+i*100),'psi3p, Dir',
     1                     nbins112,xbin112,0.)
               call hbookb((1140024+i*100),'psi3p, Dir',
     1                     nbins112,xbin112,0.)
               call hbookb((1140025+i*100),'psi3p, Dir',
     1                     nbins112,xbin112,0.)
               call hbookb((1140026+i*100),'psi3p, Dir',
     1                     nbins112,xbin112,0.)


               call hbookb((400+i*10),'Invariant Mass M4J',15,xbin0,0.)
               call hbookb((1400+i*10),'Invariant Mass M4J',9,xbin00,0.)
	       call hbookb((401+i*10),'Cos_theta3_prime',nbins1,xbin1,0.)
               call hbookb((1401+i*10),'Cos_theta3_prime',
     1                     nbins11,xbin11,0.)
               call hbookb((11401+i*10),'Cos_theta3_prime',
     1                     nbins111,xbin111,0.)
	       call hbookb((402+i*10),'psi3_prime',nbins2,xbin2,0.)
	       call hbookb((1402+i*10),'psi3_prime',nbins12,xbin12,0.)
	       call hbookb((11402+i*10),'psi3_prime',nbins112,xbin112,0.)
	       call hbookb((403+i*10),'X3_prime',7,xbin3,0.)
	       call hbookb((404+i*10),'X4_prime',10,xbin4,0.)
               call hbookb((405+i*10),'X5_prime',10,xbin7,0.)
	       call hbookb((406+i*10),'psiAB',nbins5,xbin5,0.)
	       call hbookb((1406+i*10),'psiAB',nbins15,xbin15,0.)
	       call hbookb((11406+i*10),'psiAB',nbins115,xbin115,0.)
	       call hbookb((407+i*10),'XA',10,xbin6,0.)
	       call hbookb((1407+i*10),'XA',6,xbin16,0.)
	       call hbookb((11407+i*10),'XA',9,xbin116,0.)
               call hbookb((408+i*10),'xgam',10,xbin8,0.)
               call hbookb((1408+i*10),'xgam',8,xbin18,0.)
               call hbookb((409+i*10),'y',10,xbin8,0.)
               call hbookb((1409+i*10),'y',9,xbin10,0.)
c              Histos for different bins of xgamma 
               call hbookb((4011+i*100),'Cos3p',nbins1,xbin1,0.)
               call hbookb((4012+i*100),'Cos3p',nbins1,xbin1,0.)
               call hbookb((4013+i*100),'Cos3p',nbins1,xbin1,0.)
               call hbookb((4014+i*100),'Cos3p',nbins1,xbin1,0.)
               call hbookb((4015+i*100),'Cos3p',nbins1,xbin1,0.)
               call hbookb((4016+i*100),'Cos3p',nbins1,xbin1,0.)
               call hbookb((4021+i*100),'psi3p',nbins2,xbin2,0.)
               call hbookb((4022+i*100),'psi3p',nbins2,xbin2,0.)
               call hbookb((4023+i*100),'psi3p',nbins2,xbin2,0.)
               call hbookb((4024+i*100),'psi3p',nbins2,xbin2,0.) 
               call hbookb((4025+i*100),'psi3p',nbins2,xbin2,0.)
               call hbookb((4026+i*100),'psi3p',nbins2,xbin2,0.) 

               call hbookb((14011+i*100),'Cos3p',
     1                      nbins11,xbin11,0.)
               call hbookb((14012+i*100),'Cos3p',
     1                      nbins11,xbin11,0.)
               call hbookb((14013+i*100),'Cos3p',
     1                      nbins11,xbin11,0.)
               call hbookb((14014+i*100),'Cos3p',
     1                      nbins11,xbin11,0.)
               call hbookb((14015+i*100),'Cos3p',
     1                      nbins11,xbin11,0.)
               call hbookb((14016+i*100),'Cos3p',
     1                      nbins11,xbin11,0.)
               call hbookb((14021+i*100),'psi3p',
     1                      nbins12,xbin12,0.)
               call hbookb((14022+i*100),'psi3p',
     1                      nbins12,xbin12,0.)
               call hbookb((14023+i*100),'psi3p',
     1                      nbins12,xbin12,0.)
               call hbookb((14024+i*100),'psi3p',
     1                      nbins12,xbin12,0.) 
               call hbookb((14025+i*100),'psi3p',
     1                      nbins12,xbin12,0.)
               call hbookb((14026+i*100),'psi3p',
     1                      nbins12,xbin12,0.) 

               call hbookb((114011+i*100),'Cos3p',
     1                      nbins111,xbin111,0.)
               call hbookb((114012+i*100),'Cos3p',
     1                      nbins111,xbin111,0.)
               call hbookb((114013+i*100),'Cos3p',
     1                      nbins111,xbin111,0.)
               call hbookb((114014+i*100),'Cos3p',
     1                      nbins111,xbin111,0.)
               call hbookb((114015+i*100),'Cos3p',
     1                      nbins111,xbin111,0.)
               call hbookb((114016+i*100),'Cos3p',
     1                      nbins111,xbin111,0.)
               call hbookb((114021+i*100),'psi3p',
     1                      nbins112,xbin112,0.)
               call hbookb((114022+i*100),'psi3p',
     1                      nbins112,xbin112,0.)
               call hbookb((114023+i*100),'psi3p',
     1                      nbins112,xbin112,0.)
               call hbookb((114024+i*100),'psi3p',
     1                      nbins112,xbin112,0.) 
               call hbookb((114025+i*100),'psi3p',
     1                      nbins112,xbin112,0.)
               call hbookb((114026+i*100),'psi3p',
     1                      nbins112,xbin112,0.) 
	     enddo
	   endif
	   if (iproc.eq.2) then
             do i=1,massnum
		call hbookb((4040+i*10),'M3456, Resolved',15,xbin0,0.)
	        call hbookb((14040+i*10),'M3456, Resolved',9,xbin00,0.)
		call hbookb((4041+i*10),
     &               'Cos_theta3_p, Resolved',nbins1,xbin1,0.)
		call hbookb((14041+i*10),
     &               'Cos_theta3_p, Resolved',nbins11,xbin11,0.)
		call hbookb((114041+i*10),
     &               'Cos_theta3_p, Resolved',nbins111,xbin111,0.)
		call hbookb((4042+i*10),
     &               'psi3_p, Resolved',nbins2,xbin2,0.)
		call hbookb((14042+i*10),
     &               'psi3_p, Resolved',nbins12,xbin12,0.)
		call hbookb((114042+i*10),
     &               'psi3_p, Resolved',nbins112,xbin112,0.)
	       call hbookb((4043+i*10),'X3_p, Resolved',7,xbin3,0.)
	       call hbookb((4044+i*10),'X4_p, Resolved',10,xbin4,0.)
               call hbookb((4045+i*10),'X5_p, Resolved',10,xbin7,0.)
               call hbookb((4046+i*10),
     &               'psiAB, Resolved',nbins5,xbin5,0.)
               call hbookb((14046+i*10),'psiAB, Resolved',nbins15,
     1                      xbin15,0.)
               call hbookb((114046+i*10),'psiAB, Resolved',nbins115,
     1                      xbin115,0.)
               call hbookb((4047+i*10),'XA, Resolved',10,xbin6,0.)
               call hbookb((14047+i*10),'XA, Resolved',6,xbin16,0.)
               call hbookb((114047+i*10),'XA, Resolved',9,xbin116,0.)
               call hbookb((4048+i*10),'xgam, Resolved',10,xbin8,0.)
               call hbookb((14048+i*10),'xgam, Resolved',8,xbin18,0.)
               call hbookb((4049+i*10),'y, Resolved',10,xbin8,0.) 
               call hbookb((14049+i*10),'y, Resolved',9,xbin10,0.) 
c              Histos for different bins of xgamma 
               call hbookb((40411+i*100),'Cos3p, Res',nbins1,xbin1,0.)
               call hbookb((40412+i*100),'Cos3p, Res',nbins1,xbin1,0.)
               call hbookb((40413+i*100),'Cos3p, Res',nbins1,xbin1,0.)
               call hbookb((40414+i*100),'Cos3p, Res',nbins1,xbin1,0.)
               call hbookb((40415+i*100),'Cos3p, Res',nbins1,xbin1,0.)
               call hbookb((40416+i*100),'Cos3p, Res',nbins1,xbin1,0.)
               call hbookb((40421+i*100),'psi3p, Res',nbins2,xbin2,0.)
               call hbookb((40422+i*100),'psi3p, Res',nbins2,xbin2,0.)
               call hbookb((40423+i*100),'psi3p, Res',nbins2,xbin2,0.)
               call hbookb((40424+i*100),'psi3p, Res',nbins2,xbin2,0.)
               call hbookb((40425+i*100),'psi3p, Res',nbins2,xbin2,0.)
               call hbookb((40426+i*100),'psi3p, Res',nbins2,xbin2,0.)

               call hbookb((140411+i*100),'Cos3p, Res',
     1                     nbins11,xbin11,0.)
               call hbookb((140412+i*100),'Cos3p, Res',
     1                     nbins11,xbin11,0.)
               call hbookb((140413+i*100),'Cos3p, Res',
     1                     nbins11,xbin11,0.)
               call hbookb((140414+i*100),'Cos3p, Res',
     1                     nbins11,xbin11,0.)
               call hbookb((140415+i*100),'Cos3p, Res',
     1                     nbins11,xbin11,0.)
               call hbookb((140416+i*100),'Cos3p, Res',
     1                     nbins11,xbin11,0.)
               call hbookb((140421+i*100),'psi3p, Res',
     1                     nbins12,xbin12,0.)
               call hbookb((140422+i*100),'psi3p, Res',
     1                     nbins12,xbin12,0.)
               call hbookb((140423+i*100),'psi3p, Res',
     1                     nbins12,xbin12,0.)
               call hbookb((140424+i*100),'psi3p, Res',
     1                      nbins12,xbin12,0.)
               call hbookb((140425+i*100),'psi3p, Res',
     1                      nbins12,xbin12,0.)
               call hbookb((140426+i*100),'psi3p, Res',
     1                      nbins12,xbin12,0.)

               call hbookb((1140411+i*100),'Cos3p, Res',
     1                     nbins111,xbin111,0.)
               call hbookb((1140412+i*100),'Cos3p, Res',
     1                     nbins111,xbin111,0.)
               call hbookb((1140413+i*100),'Cos3p, Res',
     1                     nbins111,xbin111,0.)
               call hbookb((1140414+i*100),'Cos3p, Res',
     1                     nbins111,xbin111,0.)
               call hbookb((1140415+i*100),'Cos3p, Res',
     1                     nbins111,xbin111,0.)
               call hbookb((1140416+i*100),'Cos3p, Res',
     1                     nbins111,xbin111,0.)
               call hbookb((1140421+i*100),'psi3p, Res',
     1                     nbins112,xbin112,0.)
               call hbookb((1140422+i*100),'psi3p, Res',
     1                     nbins112,xbin112,0.)
               call hbookb((1140423+i*100),'psi3p, Res',
     1                     nbins112,xbin112,0.)
               call hbookb((1140424+i*100),'psi3p, Res',
     1                      nbins112,xbin112,0.)
               call hbookb((1140425+i*100),'psi3p, Res',
     1                      nbins112,xbin112,0.)
               call hbookb((1140426+i*100),'psi3p, Res',
     1                      nbins112,xbin112,0.)
             enddo
	   endif
	   
	   CALL HIDOPT(0,'stat')
	   CALL HBARX(0)
	   
c       -CUTS--------
	   ybjcut(1)=0.2
	   ybjcut(2)=0.85
	   
	   q2cut(1)=0.
	   q2cut(2)=1.
	   
	   rapcut(1)= -2.4
	   rapcut(2)=2.4
	   
	   Etcut(1)=5.0
	   Etcut(2)=6.0
	   
	   Masscut(1)=0.0
	   Masscut(2)=20.0
           Masscut(3)=50.0
           Masscut(4)=70.0
	   
           X3_cut=0.95
	   
	   cos_t3_cut(1)=-0.8
	   cos_t3_cut(2)= 0.8
	   
	   do i=1,massnum
	      counter3j(iproc,1,i) = 0
	      counter3j(iproc,2,i) = 0
	      counter4j(iproc,1,i) = 0
	      counter4j(iproc,2,i) = 0
	   enddo
	   
C       --- standard cone radius is 1.0
	   CALL hzjetrad(2,CONER)
	   if (CONER.lt.0.0) then
	      CONER=1.0
	   endif
	   CALL HZJTNAME(chjet,jetf)
	   WRITE(6,*)'***********************************************'
	   WRITE(6,*)'*ichep02849 called, histograms will be output.*'
	   WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
	   WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
	   WRITE(6,*)'**********************************************'
	   
 1001	   FORMAT(A29,F5.2,A13)
	   
	else if (intiflag.eq.2) then  
C       *** Filling runs ***
	   
C       *** Change directory to our graphs ***
	   IF (parton_showers) THEN              
              CALL HCDIR('//PAWC/psi02849',' ')
           ELSE
              CALL HCDIR('//PAWC/i02849',' ')
           ENDIF
	   
	   IF (gen(1:3).EQ.'HO1') THEN
	      
	      DO i=1,massnum
		 DO j=0,7
		    CALL HZSNLO(300+10*i+j,3000+10*(i+4)+j,3000+10*i+j)
		 ENDDO
                 DO k=1,4
                    CALL HZSNLO(3010+100*i+k,30010+100*(i+4)+k
     &                        ,30010+100*i+k)
                 ENDDO
                 DO k=1,6
                    CALL HZSNLO(3020+100*i+k,30020+100*(i+4)+k
     &                        ,30020+100*i+k)
                 ENDDO
	      ENDDO
	      DO i=1,massnum
		 DO j=0,9
		    CALL HZSNLO(400+10*i+j,4000+10*(i+4)+j,4000+10*i+j)
		 ENDDO
                 DO k=1,4
                    CALL HZSNLO(4010+100*i+k,40010+100*(i+4)+k
     &                        ,40010+100*i+k)
                 ENDDO
                 DO k=1,6
                    CALL HZSNLO(4020+100*i+k,40020+100*(i+4)+k
     &                        ,40020+100*i+k)
                 ENDDO
	      ENDDO
	      iproc = 1
	      
	   ENDIF
	   
c       *** get jets  
	   q2=hzphokin(1)
	   if (q2.eq.-1) then
	      PRINT*,'ichep02849: HZPHOKIN returns error for Q2'
	   elseif (q2.lt.0) then
	      q2=abs(q2)
	   endif
	   ybj=hzphokin(3)
	   if (ybj.eq.-1) then
	      PRINT*,'ichep02849: HZPHOKIN returns error for ybj'
	   endif
	   
	   CALL hzjtfind(chjet,CONER,NJET,Jets)

C *** Start 3-jet stuff
	   
	   DO i=1,4
	      et(i)=0.0
	   ENDDO

c       Take the three highest ET jets in the visible rapidity range.

	   njeti=0
	   do loop=1,njet
	      IF ( jets(loop,1).gt.rapcut(1).and.jets(loop,1).lt
     &  	   .rapcut(2)) THEN
		 njeti=njeti+1
		 IF (njeti.EQ.1) THEN
		    P3lab(4) = real(Jets(loop,4))
		    ET(njeti)=jets(loop,3)	
                    rap(njeti)=jets(loop,1)		    
		    DO j=1,3
		       P3lab(j) = real(Jets(loop,j+4))
		    ENDDO
		 ELSE IF (njeti.EQ.2) THEN
		    P4lab(4) = real(Jets(loop,4))
		    ET(njeti)=jets(loop,3)
                    rap(njeti)=jets(loop,1)	
		    DO j=1,3
		       P4lab(j) = real(Jets(loop,j+4))
		    ENDDO
		 ELSE IF (njeti.EQ.3) THEN
		    P5lab(4) = real(Jets(loop,4))
		    ET(njeti)=jets(loop,3)
                    rap(njeti)=jets(loop,1)	
		    DO j=1,3
		       P5lab(j) = real(Jets(loop,j+4))
		    ENDDO
		 ELSE IF (njeti.EQ.4) THEN
		    P6lab(4) = real(Jets(loop,4))
		    ET(njeti)=jets(loop,3)
                    rap(njeti)=jets(loop,1)	
		    DO j=1,3
		       P6lab(j) = real(Jets(loop,j+4))
		    ENDDO
		 ENDIF
	      ENDIF
	   ENDDO


	   if (njeti.lt.3) then
              do i=1,massnum
	         counter3j(iproc,2,i) = counter3j(iproc,2,i) + 1
                 counter4j(iproc,2,i) = counter4j(iproc,2,i) + 1
              enddo
	      return
	   endif
	   
c       Check the ET (note - the jets are already ordered in ET.)

	   if (et(2).lt.etcut(2).or.et(3).lt.etcut(1)) then
              do i=1,massnum
                 counter3j(iproc,2,i) = counter3j(iproc,2,i) + 1
                 counter4j(iproc,2,i) = counter4j(iproc,2,i) + 1
              enddo
	     return
	   endif
	   
c**********Calculate xgamma
           elecE=27.52
           protE=820.0
           xgam3=(et(1)*exp(-1*rap(1))+et(2)*exp(-1*rap(2))
     &           +et(3)*exp(-1*rap(3)))/(2*ybj*elecE) 
           xprot3=(et(1)*exp(1*rap(1))+et(2)*exp(1*rap(2))
     &           +et(3)*exp( 1*rap(3)))/(2*protE) 
c**********Calculate invariant mass

	   do i=1,4
	      Ptot(i) = P3lab(i) + P4lab(i) + P5lab(i)
	   enddo
	   
	   Mom2_tot = 0
	   
	   do i = 1,3
	      Mom2_tot = Mom2_tot + Ptot(i)*Ptot(i)
	   enddo
	   
	   E2 = Ptot(4)*Ptot(4)
	   E  = sqrt(E2)
	   M345 = sqrt(E2 - Mom2_tot)

C**********DO BOOST**into cms frame for 3-jets****
	   
	   xboost = -1.0*dble(Ptot(1)) / dble(E)
	   yboost = -1.0*dble(Ptot(2)) / dble(E)
	   zboost = -1.0*dble(Ptot(3)) / dble(E)
	   
	   do i=1,4
	      p3_act(i) = dble(P3lab(i))
	      p4_act(i) = dble(P4lab(i))
	      p5_act(i) = dble(P5lab(i))
	      ptot_act(i) = dble(Ptot(i))
	   enddo
	   
	   CALL HZBOOST(xboost,yboost, zboost,p3_act,P3new_dbl)
	   CALL HZBOOST(xboost,yboost, zboost,p4_act,P4new_dbl)
	   CALL HZBOOST(xboost,yboost, zboost,p5_act,P5new_dbl)
	   CALL HZBOOST(xboost,yboost, zboost,ptot_act,Pnew_dbl)
	   
	   do i=1,4
	      P3new(i) = real(P3new_dbl(i))
	      P4new(i) = real(P4new_dbl(i))
	      P5new(i) = real(P5new_dbl(i))
	      Pnew(i)  = real(Pnew_dbl(i))
	   enddo
	   
c****   swap so E3 > E4 > E5 ***
	   
	   If (P3new(4).lt.P4new(4)) then
	      do i=1,4
		 testy(i) = P3new(i)
		 P3new(i) = P4new(i) 
		 P4new(i) = testy(i)
	      enddo
	   endif
	   
	   If (P4new(4).lt.P5new(4)) then
	      do i=1,4
		 testy(i) = P4new(i)
		 P4new(i) = P5new(i) 
		 P5new(i) = testy(i)
	      enddo
	   endif
	   
	   If (P3new(4).lt.P4new(4)) then
	      do i=1,4
		 testy(i) = P3new(i)
		 P3new(i) = P4new(i) 
		 P4new(i) = testy(i)
	      enddo
	   endif
	   
	   do i=1,4
	      P3(i) = P3new(i)
	      P4(i) = P4new(i)
	      P5(i) = P5new(i)
	   enddo
	   
C       ***   DALITZ variables X3, X4, X5
	   X3 = (2 * P3(4)) /M345
	   X4 = (2 * P4(4)) /M345
	   X5 = (2 * P5(4)) /M345
	   
******* Pav ********= P1 - P2*************
	   P1_lab(1) = 0 
	   P1_lab(2) = 0
	   P1_lab(3) = 820.0
	   P1_lab(4) = 820.0
	   
	   P2_lab(1) = 0
	   P2_lab(2) = 0
	   P2_lab(3) = -27.52
	   P2_lab(4) = 27.52
	   
	   CALL HZBOOST(xboost,yboost,zboost, P1_lab,P1_dbl)
	   CALL HZBOOST(xboost,yboost,zboost, P2_lab,P2_dbl)
	   
	   do i=1,4
	      P1(i) = real(P1_dbl(i))
	      P2(i) = real(P2_dbl(i))
	   enddo
	   
	   do i=1,3
	      Pav(i) = P1(i) - P2(i)
	   enddo
	   
C       ***   Calculate cos theta 3  ***

	   dot4 = 0
	   dot5 = 0
	   dot6 = 0
	   do i=1,3
	      dot4 = dot4 + Pav(i)*P3(i)
	      dot5 = dot5 + Pav(i)*Pav(i)
	      dot6 = dot6 + P3(i)*P3(i)
	   enddo
	   
	   cos_theta3 = dot4 / ( sqrt(dot5)*sqrt(dot6) )
	   
C       ***    cos psi 3 = cos_psi3
	   
	   call cross(P3,Pav,cross1)
	   call cross(P4,P5,cross2)
	   
	   dot1 = 0
	   dot2 = 0
	   dot3 = 0
	   
	   do i=1,3
	      dot1 = dot1 + cross1(i)*cross1(i)
	      dot2 = dot2 + cross2(i)*cross2(i)
	      dot3 = dot3 + cross1(i)*cross2(i)
	   enddo
	   
	   
	   cos_psi3 = dot3 / (sqrt(dot1) * sqrt(dot2) )
	   psi3     = acos( cos_psi3)

C       ***    Implement cuts 
	   
	   cut_test(1) = 0
	   cut_test(2) = 0

	   if (ybj.ge.ybjcut(1).and.ybj.le.ybjcut(2))then
              cut_test(2) = cut_test(2)+1
           endif

           if (q2.lt.q2cut(2)) cut_test(2)=cut_test(2)+1
	   
	   IF (X3.lt.X3_cut) then
	      cut_test(2) = cut_test(2) + 1
	   endif
	   
	   if (cos_theta3.gt.cos_t3_cut(1).and.cos_theta3.lt
     &  	.cos_t3_cut(2) ) then
	      cut_test(2) = cut_test(2) + 1
	   endif
	   
C     if successful event cut_test(2) = 4
	   
           if (cut_test(2).eq.4) then 
              do i=1,massnum
                 if (M345.ge.Masscut(i)) then
	            counter3j(iproc,1,i) = counter3j(iproc,1,i) + 1
	            call hfill(int(3000+10*(iproc**2+iproc-2+i)), 
     1		    real(M345),0.,wtx)
	            call hfill(int(3001+10*(iproc**2+iproc-2+i)), 
     1		    real(Cos_theta3),0.,wtx) 
	            call hfill(int(3002+10*(iproc**2+iproc-2+i)), 
     1		    real(psi3),0.,wtx)
                    call hfill(int(3003+10*(iproc**2+iproc-2+i)), 
     1		    real(X3),0.,wtx)
                    call hfill(int(3004+10*(iproc**2+iproc-2+i)), 
     1		    real(X4),0.,wtx)
                    call hfill(int(3005+10*(iproc**2+iproc-2+i)), 
     1		    real(X5),0.,wtx)
                    call hfill(int(3006+10*(iproc**2+iproc-2+i)), 
     1		    real(xgam3),0.,wtx)
                    call hfill(int(3007+10*(iproc**2+iproc-2+i)), 
     1		    real(ybj),0.,wtx)
                    if(xgam3.ge.0.0.and.xgam3.lt.0.5) then
                      CALL HFILL(int(30011+100*(iproc**2+iproc-2+i))
     &                           ,REAL(cos_theta3),0.,wtx)
                      CALL HFILL(int(30021+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3),0.,wtx)
                    else if(xgam3.ge.0.5.and.xgam3.lt.0.75) then
                      CALL HFILL(int(30012+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3),0.,wtx)
                      CALL HFILL(int(30022+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3),0.,wtx)
                    else if(xgam3.ge.0.75.and.xgam3.lt.0.9) then
                      CALL HFILL(int(30013+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3),0.,wtx)
                      CALL HFILL(int(30023+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3),0.,wtx)
                    else if(xgam3.ge.0.9.and.xgam3.le.1.0) then
                      CALL HFILL(int(30014+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3),0.,wtx)
                      CALL HFILL(int(30024+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3),0.,wtx)
                    else
                       print *, 'Three Jet strange xgamma :', xgam3
                    endif
                    if(xgam3.ge.0.0.and.xgam3.lt.0.75) then
                      CALL HFILL(int(30025+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3),0.,wtx)
                    else if(xgam3.ge.0.75.and.xgam3.le.1.0) then
                      CALL HFILL(int(30026+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3),0.,wtx)
                    else
                      print *, 'Three Jet strange xgamma :', xgam3
                    endif
                 else 
		    counter3j(iproc,2,i)=counter3j(iproc,2,i)+1 
                 endif
	      enddo
	   else
	      do i=1,massnum			
	         counter3j(iproc,2,i) = counter3j(iproc,2,i) + 1
	      enddo
	   endif		
	   
C       *** Start 4-jet stuff


          if (njeti.lt.4) then
              do i=1,massnum
	         counter4j(iproc,2,i) = counter4j(iproc,2,i) + 1
              enddo
	      return
	   endif

c       Check the ET (note - the jets are already ordered in ET.)
	   if (et(2).lt.etcut(2).or.et(4).lt.etcut(1)) then
               do i=1,massnum
	         counter4j(iproc,2,i) = counter4j(iproc,2,i) + 1
              enddo
	      return
	   endif

c**********Calculate xgamma
           elecE=27.52
           xgam4=(et(1)*exp(-1*rap(1))+et(2)*exp(-1*rap(2))
     &           +et(3)*exp(-1*rap(3))+et(4)*exp(-1*rap(4)))
     &           /(2*ybj*elecE) 
           xprot4=(et(1)*exp(1*rap(1))+et(2)*exp(1*rap(2))
     &           +et(3)*exp( 1*rap(3))+et(4)*exp( 1*rap(4)))
     &           /(2*protE) 
C       *********Calculate 4-jet invariant mass

	   do i=1,4
              Ptot(i) = P3lab(i) + P4lab(i) + P5lab(i) + P6lab(i)
	   enddo
	   
	   Mom2_tot = 0
	   
	   do i = 1,3
	      Mom2_tot = Mom2_tot + Ptot(i)*Ptot(i)
	   enddo
	   
	   E2 = Ptot(4)*Ptot(4)
	   E  = sqrt(E2)
	   M3456 = sqrt(E2 - Mom2_tot)
	   
C**********DO BOOST**into cms frame for 4-jets****
	   
	   
	   xboost = -1.0*dble(Ptot(1)) / dble(E)
	   yboost = -1.0*dble(Ptot(2)) / dble(E)
	   zboost = -1.0*dble(Ptot(3)) / dble(E)
	   
	   do i=1,4
              p6_act(i) = dble(p6lab(i))
	      ptot_act(i) = dble(Ptot(i))
	   enddo
	   
	   CALL HZBOOST(xboost,yboost, zboost,p3_act,P3new_dbl)
	   CALL HZBOOST(xboost,yboost, zboost,p4_act,P4new_dbl)
	   CALL HZBOOST(xboost,yboost, zboost,p5_act,P5new_dbl)
           CALL HZBOOST(xboost,yboost, zboost,p6_act,P6new_dbl)
	   CALL HZBOOST(xboost,yboost, zboost,ptot_act,Pnew_dbl)
	   
	   do i=1,4
	      P3new(i) = real(P3new_dbl(i))
	      P4new(i) = real(P4new_dbl(i))
	      P5new(i) = real(P5new_dbl(i))
              P6new(i) = real(P6new_dbl(i))
	      Pnew(i)  = real(Pnew_dbl(i))
	   enddo

C       *******Test to find the 2 jets with lowest 2-jet mass in 4-jet
c       cms

           do i=1,4
              Ptot_test(1,i) = P3new(i)+P4new(i)
              Ptot_test(2,i) = P3new(i)+P5new(i)
              Ptot_test(3,i) = P3new(i)+P6new(i)
              Ptot_test(4,i) = P4new(i)+P5new(i)
              Ptot_test(5,i) = P4new(i)+P6new(i)
              Ptot_test(6,i) = P5new(i)+P6new(i)
           enddo

           do j=1,6
              Mom2_tot_test(j) = 0.
	      
	      do k=1,3
                 Mom2_tot_test(j) = Mom2_tot_test(j)
     &  	      +Ptot_test(j,k)*Ptot_test(j,k)
              enddo

              E2_test(j) = Ptot_test(j,4)*Ptot_test(j,4)
              Mass_test(j)=sqrt(E2_test(j)-Mom2_tot_test(j))
	   enddo

C       *********Find smallest 2-jet mass and rename jets A and B

	    m1 = Mass_test(1)
            m2 = Mass_test(2)
            m3 = Mass_test(3)
            m4 = Mass_test(4)
            m5 = Mass_test(5)
            m6 = Mass_test(6)

            if ((m1.lt.m2).and.(m1.lt.m3).and.(m1.lt.m4)
     &         .and.(m1.lt.m5).and.(m1.lt.m6)) then

               if (P3new(4).ge.P4new(4)) then
                  do j=1,4
                     PAnew(j)=P3new(j)
                     PBnew(j)=P4new(j)
                  enddo
               elseif (p3new(4).lt.p4new(4)) then
                  do j=1,4
                     PAnew(j)=P4new(j)
                     PBnew(j)=P3new(j)
                  enddo
               endif

               do k=1,4
                  P3new_prime(k)=PAnew(k)+PBnew(k)
                  P4new_prime(k)=P5new(k)
		  P5new_prime(k)=P6new(k)
               enddo
        
            elseif ((m2.lt.m1).and.(m2.lt.m3).and.(m2.lt.m4)
     &              .and.(m2.lt.m5).and.(m2.lt.m6)) then

               if (P3new(4).ge.P5new(4)) then
                  do j=1,4
                     PAnew(j)=P3new(j)
                     PBnew(j)=P5new(j)
                  enddo
               elseif (P3new(4).lt.P5new(4)) then
                  do j=1,4
                     PAnew(j)=P5new(j)
                     PBnew(j)=P3new(j)
                  enddo
               endif

               do k=1,4
                  P3new_prime(k)=PAnew(k)+PBnew(k)
                  P4new_prime(k)=P4new(k)
		  P5new_prime(k)=P6new(k)
               enddo


            elseif ((m3.lt.m1).and.(m3.lt.m2).and.(m3.lt.m4)
     &              .and.(m3.lt.m5).and.(m3.lt.m6)) then

               if (P3new(4).ge.P6new(4)) then
                  do j=1,4
                     PAnew(j)=P3new(j)
                     PBnew(j)=P6new(j)
                  enddo
               elseif (P3new(4).lt.P6new(4)) then
                  do j=1,4
                     PAnew(j)=P6new(j)
                     PBnew(j)=P3new(j)
                  enddo
               endif

               do k=1,4
                  P3new_prime(k)=PAnew(k)+PBnew(k)
                  P4new_prime(k)=P4new(k)
		  P5new_prime(k)=P5new(k)
               enddo

            elseif ((m4.lt.m1).and.(m4.lt.m2).and.(m4.lt.m3)
     &              .and.(m4.lt.m5).and.(m4.lt.m6)) then

               if (P4new(4).ge.P5new(4)) then
                  do j=1,4
                     PAnew(j)=P4new(j)
                     PBnew(j)=P5new(j)
                  enddo
               elseif (P4new(4).lt.P5new(4)) then
                  do j=1,4
                     PAnew(j)=P5new(j)
                     PBnew(j)=P4new(j)
                  enddo
               endif

               do k=1,4
                  P3new_prime(k)=PAnew(k)+PBnew(k)
                  P4new_prime(k)=P3new(k)
		  P5new_prime(k)=P6new(k)
               enddo

             elseif ((m5.lt.m1).and.(m5.lt.m2).and.(m5.lt.m3)
     &              .and.(m5.lt.m4).and.(m5.lt.m6)) then

               if (P4new(4).ge.P6new(4)) then
                  do j=1,4
                     PAnew(j)=P4new(j)
                     PBnew(j)=P6new(j)
                  enddo
               elseif (P4new(4).lt.P6new(4)) then
                  do j=1,4
                     PAnew(j)=P6new(j)
                     PBnew(j)=P4new(j)
                  enddo
               endif

               do k=1,4
                  P3new_prime(k)=PAnew(k)+PBnew(k)
                  P4new_prime(k)=P3new(k)
		  P5new_prime(k)=P5new(k)
               enddo

             elseif ((m6.lt.m1).and.(m6.lt.m2).and.(m6.lt.m3)
     &              .and.(m6.lt.m4).and.(m6.lt.m5)) then

               if (P5new(4).ge.P6new(4)) then
                  do j=1,4
                     PAnew(j)=P5new(j)
                     PBnew(j)=P6new(j)
                  enddo
               elseif (P5new(4).lt.P6new(4)) then
                  do j=1,4
                     PAnew(j)=P6new(j)
                     PBnew(j)=P5new(j)
                  enddo
               endif

                do k=1,4
                  P3new_prime(k)=PAnew(k)+PBnew(k)
                  P4new_prime(k)=P3new(k)
		  P5new_prime(k)=P4new(k)
               enddo
	 endif

*********  swap so E3' > E4' > E5' 

	   if (P3new_prime(4).lt.P4new_prime(4)) then
	      do i=1,4
		 testy(i) = P3new_prime(i)
		 P3new_prime(i) = P4new_prime(i) 
		 P4new_prime(i) = testy(i)
	      enddo
	   endif
	   
	   if (P4new_prime(4).lt.P5new_prime(4)) then
	      do i=1,4
		 testy(i) = P4new_prime(i)
		 P4new_prime(i) = P5new_prime(i) 
		 P5new_prime(i) = testy(i)
	      enddo
	   endif
	   
	   if (P3new_prime(4).lt.P4new_prime(4)) then
	      do i=1,4
		 testy(i) = P3new_prime(i)
		 P3new_prime(i) = P4new_prime(i) 
		 P4new_prime(i) = testy(i)
	      enddo
	   endif
	   
	   do i=1,4
              PA(i)       = PAnew(i)
              PB(i)       = PBnew(i)
	      P3_prime(i) = P3new_prime(i)
	      P4_prime(i) = P4new_prime(i)
	      P5_prime(i) = P5new_prime(i)
	   enddo
	   
C       ******* Pav ********= P1 - P2*************
	   
	   CALL HZBOOST(xboost,yboost,zboost, P1_lab,P1_dbl)
	   CALL HZBOOST(xboost,yboost,zboost, P2_lab,P2_dbl)
	   
	   do i=1,4
	      P1(i) = real(P1_dbl(i))
	      P2(i) = real(P2_dbl(i))
	   enddo
	   
	   do i=1,3
	      Pav(i) = P1(i) - P2(i)
	   enddo

C       ***    Calculate 4-jet quantities

C       ***    DALITZ variables X3', X4', X5'

	   X3_prime = (2 * P3_prime(4)) /M3456
	   X4_prime = (2 * P4_prime(4)) /M3456
	   X5_prime = (2 * P5_prime(4)) /M3456
	   
C       ***    cos theta 3' = cos_theta3_prime  ***
	   
           dot4 = 0
	   dot5 = 0
	   dot6 = 0
	   do i=1,3
	      dot4 = dot4 + Pav(i)*P3_prime(i)
	      dot5 = dot5 + Pav(i)*Pav(i)
	      dot6 = dot6 + P3_prime(i)*P3_prime(i)
	   enddo
	   
	   cos_theta3_prime = dot4 / ( sqrt(dot5)*sqrt(dot6) )
	   
C       ***     cos psi 3' = cos_psi3_prime  
	   
	   call cross(P3_prime,Pav,cross1)
	   call cross(P4_prime,P5_prime,cross2)
	   
	   dot1 = 0.
	   dot2 = 0.
	   dot3 = 0.
	   
	   do i=1,3
	      dot1 = dot1 + cross1(i)*cross1(i)
	      dot2 = dot2 + cross2(i)*cross2(i)
	      dot3 = dot3 + cross1(i)*cross2(i)
	   enddo
	   
	   cos_psi3_prime = dot3 / (sqrt(dot1) * sqrt(dot2) )
	   psi3_prime     = acos( cos_psi3_prime )

C       ***     XA
           XA = PA(4)/(PA(4)+PB(4))
	   
C       ***     cos psiAB = cos_psiAB

	   do i=1,4
              PAB(i) = PA(i) + PB(i) 
           enddo

           call cross(PA,PB,cross3)
           call cross(PAB,Pav,cross4)

	   dot7 = 0.
           dot8 = 0.
           dot9 = 0.

	   do i=1,3
	      dot7 = dot7 + cross3(i)*cross3(i)
              dot8 = dot8 + cross4(i)*cross4(i)
              dot9 = dot9 + cross3(i)*cross4(i)
           enddo

	   cos_psiAB = dot9/ (sqrt(dot7)*sqrt(dot8))
           psiAB     = acos( cos_psiAB )


C ***    Implement cuts 
	   
	   cut_test(1) = 0
	   cut_test(2) = 0

           if (ybj.ge.ybjcut(1).and.ybj.le.ybjcut(2))then
              cut_test(2) = cut_test(2)+1
           endif

           if (q2.lt.q2cut(2)) cut_test(2)=cut_test(2)+1	   
	   
           if ( X3_prime.lt.X3_cut) then
	      cut_test(2) = cut_test(2) + 1
	   endif
	   
	   if ( cos_theta3_prime.gt.cos_t3_cut(1).and.cos_theta3_prime
     &  	.lt.cos_t3_cut(2) ) then
	      cut_test(2) = cut_test(2) + 1
	   endif
	   
C      if successful event  cut_test(2) = 4

	   if (cut_test(2).eq.4) then 
              do i=1,massnum
                 if (M3456.ge.Masscut(i)) then
	            counter4j(iproc,1,i) = counter4j(iproc,1,i) + 1
	            call hfill(int(4000+10*(iproc**2+iproc-2+i)), 
     1 		    real(M3456),0.,wtx)
	            call hfill(int(14000+10*(iproc**2+iproc-2+i)), 
     1 		    real(M3456),0.,wtx)
	            call hfill(int(4001+10*(iproc**2+iproc-2+i)), 
     1		    real(Cos_theta3_prime),0.,wtx) 
	            call hfill(int(14001+10*(iproc**2+iproc-2+i)), 
     1		    real(Cos_theta3_prime),0.,wtx) 
	            call hfill(int(114001+10*(iproc**2+iproc-2+i)), 
     1		    real(Cos_theta3_prime),0.,wtx) 
	            call hfill(int(4002+10*(iproc**2+iproc-2+i)), 
     1		    real(psi3_prime),0.,wtx)
	            call hfill(int(14002+10*(iproc**2+iproc-2+i)), 
     1		    real(psi3_prime),0.,wtx)
	            call hfill(int(114002+10*(iproc**2+iproc-2+i)), 
     1		    real(psi3_prime),0.,wtx)
                    call hfill(int(4003+10*(iproc**2+iproc-2+i)), 
     1		    real(X3_prime),0.,wtx)
                    call hfill(int(4004+10*(iproc**2+iproc-2+i)), 
     1		    real(X4_prime),0.,wtx)
                    call hfill(int(4005+10*(iproc**2+iproc-2+i)), 
     1		    real(X5_prime),0.,wtx)
                    call hfill(int(4006+10*(iproc**2+iproc-2+i)), 
     1		    real(psiAB),0.,wtx)
                    call hfill(int(14006+10*(iproc**2+iproc-2+i)), 
     1		    real(psiAB),0.,wtx)
                    call hfill(int(114006+10*(iproc**2+iproc-2+i)), 
     1		    real(psiAB),0.,wtx)
                    call hfill(int(4007+10*(iproc**2+iproc-2+i)), 
     1		    real(XA),0.,wtx)
                    call hfill(int(14007+10*(iproc**2+iproc-2+i)), 
     1		    real(XA),0.,wtx)
                    call hfill(int(114007+10*(iproc**2+iproc-2+i)), 
     1		    real(XA),0.,wtx)
                    call hfill(int(4008+10*(iproc**2+iproc-2+i)), 
     1		    real(xgam4),0.,wtx)
                    call hfill(int(14008+10*(iproc**2+iproc-2+i)), 
     1		    real(xgam4),0.,wtx)
                    call hfill(int(4009+10*(iproc**2+iproc-2+i)), 
     1		    real(ybj),0.,wtx)
                    call hfill(int(14009+10*(iproc**2+iproc-2+i)), 
     1		    real(ybj),0.,wtx)
                    if(xgam4.ge.0.0.and.xgam4.lt.0.5) then
                      CALL HFILL(int(40011+100*(iproc**2+iproc-2+i))
     &                           ,REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(40021+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                      CALL HFILL(int(140011+100*(iproc**2+iproc-2+i))
     &                           ,REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(140021+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                      CALL HFILL(int(1140011+100*(iproc**2+iproc-2+i))
     &                           ,REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(1140021+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                    else if(xgam4.ge.0.5.and.xgam4.lt.0.75) then
                      CALL HFILL(int(40012+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(40022+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                      CALL HFILL(int(140012+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(140022+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                      CALL HFILL(int(1140012+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(1140022+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                    else if(xgam4.ge.0.75.and.xgam4.lt.0.9) then
                      CALL HFILL(int(40013+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(40023+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                      CALL HFILL(int(140013+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(140023+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                      CALL HFILL(int(1140013+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(1140023+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                    else if(xgam4.ge.0.9.and.xgam4.le.1.0) then
                      CALL HFILL(int(40014+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(40024+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                      CALL HFILL(int(140014+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(140024+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                      CALL HFILL(int(1140014+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                      CALL HFILL(int(1140024+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                    else
                      print *, 'Four Jet strange xgamma :', xgam4
                    endif
                    if(xgam4.ge.0.0.and.xgam4.lt.0.75) then
                       CALL HFILL(int(40015+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                       CALL HFILL(int(40025+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                       CALL HFILL(int(140015+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                       CALL HFILL(int(140025+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                       CALL HFILL(int(1140015+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                       CALL HFILL(int(1140025+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                    else if(xgam4.ge.0.75.and.xgam4.lt.1.0) then
                       CALL HFILL(int(40016+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                       CALL HFILL(int(40026+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                       CALL HFILL(int(140016+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                       CALL HFILL(int(140026+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                       CALL HFILL(int(1140016+100*(iproc**2+iproc-2+i))
     &                           , REAL(cos_theta3_prime),0.,wtx)
                       CALL HFILL(int(1140026+100*(iproc**2+iproc-2+i))
     &                           , REAL(psi3_prime),0.,wtx)
                    else
                      print *, 'Four Jet strange xgamma :', xgam4
                    endif
		 else
		    counter4j(iproc,2,i)=counter4j(iproc,2,i)+1 
		 endif               
	      enddo
	   else
	      do i=1,massnum			
	         counter4j(iproc,2,i) = counter4j(iproc,2,i) + 1
	      enddo
	   endif		
	   
	elseif (intiflag.eq.3) then
C       *** Termination run ***
C       *** Change directory ***
	   IF (parton_showers) THEN
              CALL HCDIR('//PAWC/psi02849',' ')
           ELSE
              CALL HCDIR('//PAWC/i02849',' ')
           ENDIF
C       *** Finish off histograms ***

	   IF (Xsec.eq.0) THEN
	      WRITE(6,*)
     &'ichep02849: termination called with zero cross section'
	      PRINT*,'          cross section graph meaningless'
	      PRINT*,'	      in process :',iproc*1000
	      Xsec=1
	   ENDIF
	   IF (Ntot.eq.0) THEN
	      WRITE(6,*)
     &'ichep02849: termination called with no total events'
	      PRINT*,'          cross section graph meaningless'
	      PRINT*,'	      in process :',iproc*1000
	      Ntot=1
	   ENDIF
	   
C       --- store Xsec and Ntot for both direct and resolved events.
	   mhXsec(iproc)=Xsec
	   mhntot(iproc)=Ntot
	   
*	print*,"Xsec",Xsec
*	print*,"Ntot",Ntot
	   
	   if (iproc.eq.2) then

c       Book & fill the real data histograms.
c       JMB
	      call hbookb(-411,'costheta* low mass',nbins1,xbin1,0.)
	      call hbookb(-412,'psi3 low mass',nbins2,xbin2,0.)
	      call hbookb(-413,'x3 low mass',7,xbin3,0.)
	      call hbookb(-414,'x4 low mass',10,xbin4,0.)
	      call hbookb(-418,'xgamma low mass',10,xbin8,0.)
	      call hbookb(-419,'y low mass',10,xbin8,0.)

	      call hbookb(-431,'costheta* high mass',nbins1,xbin1,0.)
	      call hbookb(-432,'psi3 high mass',nbins2,xbin2,0.)
	      call hbookb(-433,'x3 high mass',7,xbin3,0.)
	      call hbookb(-434,'x4 high mass',10,xbin4,0.)
	      call hbookb(-438,'xgamma low mass',10,xbin8,0.)
	      call hbookb(-439,'y low mass',10,xbin8,0.)

c       Loop to fill the angle plots.
	      do i_bin=1,nbins1
		 ctsigma1(i_bin)=ctsigma1(i_bin)*norm_scale1
		 ctsigma2(i_bin)=ctsigma2(i_bin)*norm_scale2
		 cterr1(i_bin)=sqrt(ctstat1(i_bin)**2+
     &             0.25*ctsys1(i_bin)**2)*norm_scale1
		 cterr2(i_bin)=sqrt(ctstat2(i_bin)**2+
     &             0.25*ctsys2(i_bin)**2)*norm_scale2
	      enddo
	      do i_bin=1,nbins2
		 phisigma1(i_bin)=phisigma1(i_bin)*norm_scale1
		 phisigma2(i_bin)=phisigma2(i_bin)*norm_scale2
		 phierr1(i_bin)=
     &  	      sqrt(phistat1(i_bin)**2+
     &                0.25*phisys1(i_bin)**2)*norm_scale1
		 phierr2(i_bin)=sqrt(phistat2(i_bin)**2+
     &                0.25*phisys2(i_bin)**2)*norm_scale2
	      enddo
	      call HPAK(-411,ctsigma1)
	      call HPAKE(-411,cterr1)
	      call HPAK(-431,ctsigma2)
	      call HPAKE(-431,cterr2)
	      call HPAK(-412,phisigma1)
	      call HPAKE(-412,phierr1)
	      call HPAK(-432,phisigma2)
	      call HPAKE(-432,phierr2)
c       Loop to fill the energy fraction plots.
	      do i_bin=1,7
		 x3sigma1(i_bin)=x3sigma1(i_bin)*norm_scale1
		 x3sigma2(i_bin)=x3sigma2(i_bin)*norm_scale2
		 x3err1(i_bin)=sqrt(x3stat1(i_bin)**2+
     &                0.25*x3sys1(i_bin)**2)*norm_scale1
		 x3err2(i_bin)=sqrt(x3stat2(i_bin)**2+
     &                0.25*x3sys2(i_bin)**2)*norm_scale2
	      enddo
	      call HPAK(-413,x3sigma1)
	      call HPAKE(-413,x3err1)
	      call HPAK(-433,x3sigma2)
	      call HPAKE(-433,x3err2)
c       Loop to fill the x4, xgamma and y plots
	      do i_bin=1,10
		 x4sigma1(i_bin)=x4sigma1(i_bin)*norm_scale1
		 x4sigma2(i_bin)=x4sigma2(i_bin)*norm_scale2
		 x4err1(i_bin)=sqrt(x4stat1(i_bin)**2+
     &             0.25*x4sys1(i_bin)**2)*norm_scale1
		 x4err2(i_bin)=sqrt(x4stat2(i_bin)**2+
     &             0.25*x4sys2(i_bin)**2)*norm_scale2
		 xgsigma1(i_bin)=xgsigma1(i_bin)/1000.
		 xgsigma2(i_bin)=xgsigma2(i_bin)/1000.
		 ysigma1(i_bin)=ysigma1(i_bin)/1000.
		 ysigma2(i_bin)=ysigma2(i_bin)/1000.
		 xgerr1(i_bin)=sqrt(xgstat1(i_bin)**2+
     &             0.25*xgsys1(i_bin)**2)/1000.
		 xgerr2(i_bin)=sqrt(xgstat2(i_bin)**2+
     &             0.25*xgsys2(i_bin)**2)/1000.
		 yerr1(i_bin)=sqrt(ystat1(i_bin)**2+
     &             0.25*ysys1(i_bin)**2)/1000.
		 yerr2(i_bin)=sqrt(ystat2(i_bin)**2+
     &             0.25*ysys2(i_bin)**2)/1000.
	      enddo
	      call HPAK(-414,x4sigma1)
	      call HPAKE(-414,x4err1)
	      call HPAK(-434,x4sigma2)
	      call HPAKE(-434,x4err2)
	      call HPAK(-418,xgsigma1)
	      call HPAKE(-418,xgerr1)
	      call HPAK(-438,xgsigma2)
	      call HPAKE(-438,xgerr2)
	      call HPAK(-419,ysigma1)
	      call HPAKE(-419,yerr1)
	      call HPAK(-439,ysigma2)
	      call HPAKE(-439,yerr2)




C       Book histograms for binwidths
	      
	      call hbookb(1000,'renorm mass',15 ,xbin0 ,0.)
	      call hbookb(11000,'renorm mass',9 ,xbin00 ,0.)
	      call hbookb(1001,'renorm cos(theta_3)',
     &          nbins1,xbin1 ,0.)
	      call hbookb(11001,'renorm cos(theta_3)',
     &          nbins11,xbin11 ,0.)
	      call hbookb(111001,'renorm cos(theta_3)',
     &          nbins111,xbin111 ,0.)
	      call hbookb(1002,'renorm psi0',
     &          nbins2,xbin2 ,0.)
	      call hbookb(11002,'renorm psi0',nbins12,xbin12 ,0.)
	      call hbookb(111002,'renorm psi0',nbins112,xbin112 ,0.)
	      call hbookb(1003,'renorm X3',7 ,xbin3 ,0.)
	      call hbookb(1004,'renorm X4',10 ,xbin4 ,0.)
	      call hbookb(1007,'renorm X5',10 ,xbin7 ,0.)
	      call hbookb(1005,'renorm psiAB',nbins5 ,xbin5, 0.)
	      call hbookb(11005,'renorm psiAB',nbins15 ,xbin15, 0.)
	      call hbookb(111005,'renorm psiAB',nbins115 ,xbin115, 0.)
	      call hbookb(1006,'renorm XA',10,xbin6,0.)
	      call hbookb(11006,'renorm XA',6,xbin16,0.)
	      call hbookb(111006,'renorm XA',9,xbin116,0.)
	      call hbookb(1008,'renorm xgam and yjb',10,xbin8,0.)
	      call hbookb(11008,'renorm xgam',8,xbin18,0.)
	      call hbookb(1009,'renorm y',9,xbin10,0.)
	        
	      print*,"ichep02849:"

	      do i=1,massnum
		 print*,"Direct  events, passed cuts ",counter3j(1,1,i)
		 print*,"Direct  events, failed cuts ",counter3j(1,2,i)
		 print*,"Total  ",( counter3j(1,1,i)+counter3j(1,2,i) )
		 print*," "    
		 print*,"Resolved evnts, passed cuts ",counter3j(2,1,i)
		 print*,"Resolved evnts, failed cuts ",counter3j(2,2,i)
		 print*,"Total  ",( counter3j(2,1,i)+counter3j(2,2,i) )
		 print*," "   
		 print*,"Direct  events, passed cuts ",counter4j(1,1,i)
		 print*,"Direct  events, failed cuts ",counter4j(1,2,i)
		 print*,"Total  ",( counter4j(1,1,i)+counter4j(1,2,i) )
		 print*," "    
		 print*,"Resolved evnts, passed cuts ",counter4j(2,1,i)
		 print*,"Resolved evnts, failed cuts ",counter4j(2,2,i)
		 print*,"Total  ",( counter4j(2,1,i)+counter4j(2,2,i) )
		 print*," "   
	      enddo
	      
c       -------- work out 1/bin_width for each histo ----------
c             costheta3	      
              do ivec=1,nbins1
                 xbin1_new(ivec)=1/( xbin1(ivec+1)-xbin1(ivec) )
                 bin1_err(ivec) = 0
              enddo
              do ivec=1,nbins11
                 xbin11_new(ivec)=1/( xbin11(ivec+1)-xbin11(ivec) )
                 bin11_err(ivec) = 0
              enddo
              do ivec=1,nbins111
                 xbin111_new(ivec)=1/( xbin111(ivec+1)-xbin111(ivec) )
                 bin111_err(ivec) = 0
              enddo

c             psi3
              do ivec=1,nbins2
                 xbin2_new(ivec)=1/( xbin2(ivec+1)-xbin2(ivec) )
		 bin2_err(ivec) = 0
              enddo
              do ivec=1,nbins12
                 xbin12_new(ivec)=1/( xbin12(ivec+1)-xbin12(ivec) )
		 bin12_err(ivec) = 0
              enddo
              do ivec=1,nbins112
                 xbin112_new(ivec)=1/( xbin112(ivec+1)-xbin112(ivec) )
		 bin112_err(ivec) = 0
              enddo

c             mass
	      do ivec=1,15
		 xbin0_new(ivec)=1.0/( xbin0(ivec+1)-xbin0(ivec) )
		 bin0_err(ivec)= 0
	      enddo
	      do ivec=1,9
		 xbin00_new(ivec)=1.0/( xbin00(ivec+1)-xbin00(ivec) )
		 bin00_err(ivec)= 0
	      enddo

c             psiAB
	      do ivec=1,nbins5
	         xbin5_new(ivec)=1/( xbin5(ivec+1)-xbin5(ivec) )       
	         bin5_err(ivec) = 0
	      enddo
	      do ivec=1,nbins15
	         xbin15_new(ivec)=1/( xbin15(ivec+1)-xbin15(ivec) )       
	         bin15_err(ivec) = 0
	      enddo
	      do ivec=1,nbins115
	         xbin115_new(ivec)=1/( xbin115(ivec+1)-xbin115(ivec) )       
	         bin115_err(ivec) = 0
	      enddo

c             X3	     
	      do ivec=1,7
		 xbin3_new(ivec)=1/( xbin3(ivec+1)-xbin3(ivec) )     
		 bin3_err(ivec) = 0
	      enddo
	      
c             X4,X5
	      do ivec=1,10
		 xbin4_new(ivec)=1/( xbin4(ivec+1)-xbin4(ivec) )
                 xbin7_new(ivec)=1/( xbin7(ivec+1)-xbin7(ivec) )
  		 bin4_err(ivec) = 0
                 bin7_err(ivec) = 0
	      enddo

c             XA
	      do ivec=1,10
                 xbin6_new(ivec)=1/( xbin6(ivec+1)-xbin6(ivec) )
                 bin6_err(ivec) = 0
	      enddo
	      do ivec=1,6
		 xbin16_new(ivec)=1/( xbin16(ivec+1)-xbin16(ivec) )     
		 bin16_err(ivec) = 0
	      enddo
              do ivec=1,9
		 xbin116_new(ivec)=1/( xbin116(ivec+1)-xbin116(ivec) )     
		 bin116_err(ivec) = 0
	      enddo

c             xgam
	      do ivec=1,10
                 xbin8_new(ivec)=1/( xbin8(ivec+1)-xbin8(ivec) ) 
                 bin8_err(ivec) = 0
	      enddo
              do ivec=1,8
		 xbin18_new(ivec)=1/( xbin18(ivec+1)-xbin18(ivec) )     
		 bin18_err(ivec) = 0
	      enddo

c             y
	      do ivec=1,9
                xbin10_new(ivec)=1/( xbin10(ivec+1)-xbin10(ivec) ) 
                bin10_err(ivec) = 0
              enddo

C       Pack the binwidth histograms

              call hpak(1000,xbin0_new)
              call hpake(1000,bin0_err)

              call hpak(11000,xbin00_new)
              call hpake(11000,bin00_err)

              call hpak(1001,xbin1_new)
              call hpake(1001,bin1_err)

              call hpak(11001,xbin11_new)
              call hpake(11001,bin11_err)

              call hpak(111001,xbin111_new)
              call hpake(111001,bin111_err)

              call hpak(1002,xbin2_new)
              call hpake(1002,bin2_err)

              call hpak(11002,xbin12_new)
              call hpake(11002,bin12_err)

              call hpak(111002,xbin112_new)
              call hpake(111002,bin112_err)

              call hpak(1003,xbin3_new)
              call hpake(1003,bin3_err)

              call hpak(1004,xbin4_new)
              call hpake(1004,bin4_err)
	      
              call hpak(1005,xbin5_new)
              call hpake(1005,bin5_err)

              call hpak(11005,xbin15_new)
              call hpake(11005,bin15_err)

              call hpak(111005,xbin115_new)
              call hpake(111005,bin115_err)

              call hpak(1006,xbin6_new)
              call hpake(1006,bin6_err)

              call hpak(11006,xbin16_new)
              call hpake(11006,bin16_err)

              call hpak(111006,xbin116_new)
              call hpake(111006,bin116_err)

              call hpak(1007,xbin7_new)
              call hpake(1007,bin7_err)

              call hpak(1008,xbin8_new)
              call hpake(1008,bin8_err)

              call hpak(11008,xbin18_new)
              call hpake(11008,bin18_err)

              call hpak(1009,xbin10_new)
              call hpake(1009,bin10_err)

	      xfactor(1) = REAL( mhXsec(1)/mhNtot(1) )
	      xfactor(2) = REAL( mhXsec(2)/mhNtot(2) )	      
	      
*	write(6,*)'Xsec dir',mhXsec(1),'      mNtot',mhntot(1)
*	write(6,*)'Xsec res',mhXsec(2),'      mNtot',mhntot(2)
*       write(6,*)'xfact(1)',xfactor(1),'      xfact(2)',xfactor(2)
	      

	      do i=1,massnum
		 dir_xsec3j(i) = 
     &             real((counter3j(1,1,i)*mhXsec(1))/mhntot(1))
		 res_xsec3j(i) = 
     &             real((counter3j(2,1,i)*mhXsec(2))/mhntot(2))
		 tot_xsec3j(i) = dir_xsec3j(i) + res_xsec3j(i)
		 dir_xsec4j(i) = 
     &             real((counter4j(1,1,i)*mhXsec(1))/mhntot(1))
		 res_xsec4j(i) = 
     &             real((counter4j(2,1,i)*mhXsec(2))/mhntot(2))
		 tot_xsec4j(i) = dir_xsec4j(i) + res_xsec4j(i)
	      enddo      

*       do i=1,massnum
* 	 write(6,*)'Cross section, direct 3j   = ', dir_xsec3j(i)
*	 write(6,*)'Cross section, resolved 3j = ', res_xsec3j(i)
*	 write(6,*)'Cross section, total 3j    = ', tot_xsec3j(i)
*	 write(6,*)'Cross section, direct 4j   = ', dir_xsec4j(i)
*	 write(6,*)'Cross section, resolved 4j = ', res_xsec4j(i)
*	 write(6,*)'Cross section, total 4j    = ', tot_xsec4j(i)
*       enddo
	      
*       1002	FORMAT(A8,F5.2,A5,F5.2)
	      
C       -- divide by tot cr-sec ----
	      inv_xtot = real( 1.0 / (mhXsec(1) + mhXsec(2)) )

        IF(gen(1:3).NE.'HO1') THEN

          do i=1,massnum
    
C       --- 3-jet histograms

	    call hopera((3000+i*10),'*e',1000,(5000+i*10),xfactor(1),1.)
	    call hopera((3001+i*10),'*e',1001,(5001+i*10),xfactor(1),1.)
	    call hopera((3002+i*10),'*e',1002,(5002+i*10),xfactor(1),1.)	   
            call hopera((3003+i*10),'*e',1003,(5003+i*10),xfactor(1),1.)
            call hopera((3004+i*10),'*e',1004,(5004+i*10),xfactor(1),1.)
            call hopera((3005+i*10),'*e',1007,(5005+i*10),xfactor(1),1.)
            call hopera((3006+i*10),'*e',1008,(5006+i*10),xfactor(1),1.)
            call hopera((3007+i*10),'*e',1008,(5007+i*10),xfactor(1),1.)
            do j=1,4
              call hopera((30010+100*i+j),'*e',1001,(50010+100*i+j),
     &                   xfactor(1),1.)
            enddo
            do j=1,6
              call hopera((30020+100*i+j),'*e',1002,(50020+100*i+j),
     &                   xfactor(1),1.)
            enddo
            call hopera((3040+i*10),'*e',1000,(5040+i*10),xfactor(2),1.)
            call hopera((3041+i*10),'*e',1001,(5041+i*10),xfactor(2),1.)
            call hopera((3042+i*10),'*e',1002,(5042+i*10),xfactor(2),1.)
            call hopera((3043+i*10),'*e',1003,(5043+i*10),xfactor(2),1.)
            call hopera((3044+i*10),'*e',1004,(5044+i*10),xfactor(2),1.)  
            call hopera((3045+i*10),'*e',1007,(5045+i*10),xfactor(2),1.)  
            call hopera((3046+i*10),'*e',1008,(5046+i*10),xfactor(2),1.) 
            call hopera((3047+i*10),'*e',1008,(5047+i*10),xfactor(2),1.) 
            do j=1,4
               call hopera((30410+100*i+j),'*e',1001,(50410+100*i+j),
     &                     xfactor(2),1.)
            enddo
            do j=1,6
               call hopera((30420+100*i+j),'*e',1002,(50420+100*i+j),
     &                     xfactor(2),1.)
            enddo
               
C       --- 4-jet histograms
            call hopera((4000+i*10),'*e',1000,(6000+i*10),xfactor(1),1.)
            call hopera((14000+i*10),'*e',11000,(16000+i*10),
     1                                          xfactor(1),1.)
            call hopera((4001+i*10),'*e',1001,(6001+i*10),xfactor(1),1.)
            call hopera((14001+i*10),'*e',11001,(16001+i*10),
     1                  xfactor(1),1.)
            call hopera((114001+i*10),'*e',111001,(116001+i*10),
     1                  xfactor(1),1.)
            call hopera((4002+i*10),'*e',1002,(6002+i*10),xfactor(1),1.)
            call hopera((14002+i*10),'*e',11002,(16002+i*10),
     1                  xfactor(1),1.)
            call hopera((114002+i*10),'*e',111002,(116002+i*10),
     1                  xfactor(1),1.)
            call hopera((4003+i*10),'*e',1003,(6003+i*10),xfactor(1),1.)
            call hopera((4004+i*10),'*e',1004,(6004+i*10),xfactor(1),1.)
            call hopera((4005+i*10),'*e',1007,(6005+i*10),xfactor(1),1.)
            call hopera((4006+i*10),'*e',1005,(6006+i*10),xfactor(1),1.)
            call hopera((14006+i*10),'*e',11005,(16006+i*10),
     1                  xfactor(1),1.)
            call hopera((114006+i*10),'*e',111005,(116006+i*10),
     1                  xfactor(1),1.)
	    call hopera((4007+i*10),'*e',1006,(6007+i*10),xfactor(1),1.)
	    call hopera((14007+i*10),'*e',11006,(16007+i*10),
     1                  xfactor(1),1.)
	    call hopera((114007+i*10),'*e',111006,(116007+i*10),
     1                  xfactor(1),1.)
            call hopera((4008+i*10),'*e',1008,(6008+i*10),xfactor(1),1.)
            call hopera((14008+i*10),'*e',11008,(16008+i*10),
     1                  xfactor(1),1.)
            call hopera((4009+i*10),'*e',1008,(6009+i*10),xfactor(1),1.)
            call hopera((14009+i*10),'*e',1009,(16009+i*10),
     1                                         xfactor(1),1.)
            do j=1,6
               call hopera((40010+100*i+j),'*e',1001,(60010+100*i+j),
     &                    xfactor(1),1.)
               call hopera((140010+100*i+j),'*e',11001,(160010+100*i+j),
     &                    xfactor(1),1.)
               call hopera((1140010+100*i+j),'*e',111001,
     &                     (1160010+100*i+j),xfactor(1),1.)
            enddo
            do j=1,6
               call hopera((40020+100*i+j),'*e',1002,(60020+100*i+j),
     &                    xfactor(1),1.)
               call hopera((140020+100*i+j),'*e',11002,(160020+100*i+j),
     &                    xfactor(1),1.)
               call hopera((1140020+100*i+j),'*e',111002,
     &                     (1160020+100*i+j),xfactor(1),1.)
            enddo
            call hopera((4040+i*10),'*e',1000,(6040+i*10),xfactor(2),1.)
            call hopera((14040+i*10),'*e',11000,(16040+i*10),
     1                                          xfactor(2),1.)
            call hopera((4041+i*10),'*e',1001,(6041+i*10),xfactor(2),1.)
            call hopera((14041+i*10),'*e',11001,(16041+i*10),
     1                  xfactor(2),1.)
            call hopera((114041+i*10),'*e',111001,(116041+i*10),
     1                  xfactor(2),1.)
            call hopera((4042+i*10),'*e',1002,(6042+i*10),xfactor(2),1.)
            call hopera((14042+i*10),'*e',11002,(16042+i*10),
     1                  xfactor(2),1.)
            call hopera((114042+i*10),'*e',111002,(116042+i*10),
     1                  xfactor(2),1.)
            call hopera((4043+i*10),'*e',1003,(6043+i*10),xfactor(2),1.)
	    call hopera((4044+i*10),'*e',1004,(6044+i*10),xfactor(2),1.)
            call hopera((4045+i*10),'*e',1007,(6045+i*10),xfactor(2),1.)
	    call hopera((4046+i*10),'*e',1005,(6046+i*10),xfactor(2),1.)
	    call hopera((14046+i*10),'*e',11005,(16046+i*10),
     1                  xfactor(2),1.)
	    call hopera((114046+i*10),'*e',111005,(116046+i*10),
     1                  xfactor(2),1.)
	    call hopera((4047+i*10),'*e',1006,(6047+i*10),xfactor(2),1.)
	    call hopera((14047+i*10),'*e',11006,(16047+i*10),
     1                  xfactor(2),1.)
	    call hopera((114047+i*10),'*e',111006,(116047+i*10),
     1                  xfactor(2),1.)
            call hopera((4048+i*10),'*e',1008,(6048+i*10),xfactor(2),1.)
            call hopera((14048+i*10),'*e',11008,(16048+i*10),
     1                  xfactor(2),1.)
            call hopera((4049+i*10),'*e',1008,(6049+i*10),xfactor(2),1.)
            call hopera((14049+i*10),'*e',1009,(16049+i*10),
     1                                         xfactor(2),1.)
            do j=1,6
               call hopera((40410+100*i+j),'*e',1001,(60410+100*i+j),
     &                    xfactor(2),1.)
               call hopera((140410+100*i+j),'*e',11001,(160410+100*i+j),
     &                    xfactor(2),1.)
               call hopera((1140410+100*i+j),'*e',111001,
     &                     (1160410+100*i+j),xfactor(2),1.)
            enddo
            do j=1,6
               call hopera((40420+100*i+j),'*e',1002,(60420+100*i+j),
     &                    xfactor(2),1.)
               call hopera((140420+100*i+j),'*e',11002,(160420+100*i+j),
     &                    xfactor(2),1.)
               call hopera((1140420+100*i+j),'*e',111002,
     &                     (1160420+100*i+j),xfactor(2),1.)
            enddo
         enddo

C       --- Final cross-sections
	
         do i=1,massnum

C       --- 3-jet
	    call hopera((5000+i*10),'+e',(5040+i*10),(300+i*10),1.,1.)
	    call hopera((5001+i*10),'+e',(5041+i*10),(301+i*10),1.,1.)
            call hopera((5002+i*10),'+e',(5042+i*10),(302+i*10),1.,1.)
	    call hopera((5003+i*10),'+e',(5043+i*10),(303+i*10),1.,1.)
	    call hopera((5004+i*10),'+e',(5044+i*10),(304+i*10),1.,1.)
            call hopera((5005+i*10),'+e',(5045+i*10),(305+i*10),1.,1.)
            call hopera((5006+i*10),'+e',(5046+i*10),(306+i*10),1.,1.)
            call hopera((5007+i*10),'+e',(5047+i*10),(307+i*10),1.,1.)
            do j=1,4
               call hopera((50010+100*i+j),'+e',(50410+100*i+j)
     &                    ,(3010+100*i+j),1.,1.)
            enddo
            do j=1,6
               call hopera((50020+100*i+j),'+e',(50420+100*i+j)
     &                    ,(3020+100*i+j),1.,1.)
            enddo

C       --- 4-jet
            call hopera((6000+i*10),'+e',(6040+i*10),(400+i*10),1.,1.)
            call hopera((16000+i*10),'+e',(16040+i*10),(1400+i*10),
     1                                                 1.,1.)
	    call hopera((6001+i*10),'+e',(6041+i*10),(401+i*10),1.,1.)
	    call hopera((16001+i*10),'+e',(16041+i*10),(1401+i*10),1.,1.)
	    call hopera((116001+i*10),'+e',
     &        (116041+i*10),(11401+i*10),1.,1.)
	    call hopera((6002+i*10),'+e',
     &        (6042+i*10),(402+i*10),1.,1.)
            call hopera((16002+i*10),'+e',(16042+i*10),
     1                   (1402+i*10),1.,1.)
            call hopera((116002+i*10),'+e',(116042+i*10),
     1                  (11402+i*10),1.,1.)
            call hopera((6003+i*10),'+e',(6043+i*10),(403+i*10),1.,1.)
	    call hopera((6004+i*10),'+e',(6044+i*10),(404+i*10),1.,1.)
            call hopera((6005+i*10),'+e',(6045+i*10),(405+i*10),1.,1.)
            call hopera((6006+i*10),'+e',(6046+i*10),(406+i*10),1.,1.)
            call hopera((16006+i*10),'+e',(16046+i*10),(1406+i*10),
     1                   1.,1.)
            call hopera((116006+i*10),'+e',(116046+i*10),(11406+i*10),
     1                   1.,1.)
	    call hopera((6007+i*10),'+e',(6047+i*10),(407+i*10),1.,1.)
	    call hopera((16007+i*10),'+e',(16047+i*10),(1407+i*10),1.,1.)
	    call hopera((116007+i*10),'+e',
     &        (116047+i*10),(11407+i*10),1.,1.)
            call hopera((6008+i*10),'+e',(6048+i*10),(408+i*10),1.,1.)
            call hopera((16008+i*10),'+e',(16048+i*10),
     1                  (1408+i*10),1.,1.)
            call hopera((6009+i*10),'+e',(6049+i*10),(409+i*10),1.,1.) 
            call hopera((16009+i*10),'+e',(16049+i*10),(1409+i*10),
     1                                                 1.,1.) 
            do j=1,6
                call hopera((60010+100*i+j),'+e',(60410+100*i+j),
     &                     (4010+100*i+j),1.,1.)
                call hopera((160010+100*i+j),'+e',(160410+100*i+j),
     &                     (14010+100*i+j),1.,1.)
                call hopera((1160010+100*i+j),'+e',(1160410+100*i+j),
     &                     (114010+100*i+j),1.,1.)
            enddo
            do j=1,6
                call hopera((60020+100*i+j),'+e',(60420+100*i+j),
     &                     (4020+100*i+j),1.,1.)
                call hopera((160020+100*i+j),'+e',(160420+100*i+j),
     &                     (14020+100*i+j),1.,1.)
                call hopera((1160020+100*i+j),'+e',(1160420+100*i+j),
     &                     (114020+100*i+j),1.,1.)
             enddo
           enddo
		 
		 
	 ELSE
		 
c       For NLO stuff, only need to take the bin width into account.
c       However, we must also deal with the final
c       round of integration first.
		 
           DO i=1,4
              DO j=0,7
		 CALL HZSNLO(300+10*i+j,3000+10*(i+4)+j,3000+10*i+j)
              ENDDO
              DO k=1,4
                 CALL HZSNLO(3010+100*i+k,30010+100*(i+4)+k
     &                     ,30010+100*i+k)
              ENDDO
              DO k=1,6
                 CALL HZSNLO(3020+100*i+k,30020+100*(i+4)+k
     &                     ,30020+100*i+k)
              ENDDO
           ENDDO
           DO i=1,4
              DO j=0,9
		 CALL HZSNLO(400+10*i+j,4000+10*(i+4)+j,4000+10*i+j)
	      ENDDO
              DO k=1,4
                 CALL HZSNLO(4010+100*i+k,40010+100*(i+4)+k
     &                     ,40010+100*i+k)
                 CALL HZSNLO(4020+100*i+k,40020+100*(i+4)+k
     &                     ,40020+100*i+k)
              ENDDO
           ENDDO

		 do i=1,massnum      
C       --- 3-jet histograms
		    call hopera((300+i*10),'*e',1000,300+i*10
     1		 ,.5,1.)
		    call hopera((301+i*10),'*e',1001,301+i*10
     2		 ,.5,1.)
c       There was a bug here!!
		    call hopera((302+i*10),'*e',1002,(302+i*10)
     3		 ,.5,1.)
		    call hopera((303+i*10),'*e',1003,(303+i*10)
     4		 ,.5,1.)
		    call hopera((304+i*10),'*e',1004,(304+i*10)
     5		 ,.5,1.)
                    call hopera((305+i*10),'*e',1007,(305+i*10)
     6		 ,.5,1.)
                    call hopera((306+i*10),'*e',1008,(306+i*10)
     7		 ,.5,1.) 
                    call hopera((307+i*10),'*e',1008,(307+i*10)
     8		 ,.5,1.) 
                    do j=1,4
                       call hopera((3010+100*i+j),'*e',1001,
     &                     (3010+100*i+j),.5,1.)
                    enddo
                    do j=1,6
                       call hopera((3020+100*i+j),'*e',1002,
     &                     (3020+100*i+j),.5,1.)
                    enddo
		    
		    
C       --- 4-jet histograms
		    call hopera((400+i*10),'*e',1000,(400+i*10)
     1		 ,.5,1.)
		    call hopera((1400+i*10),'*e',11000,(1400+i*10)
     1		 ,.5,1.)
		    call hopera((401+i*10),'*e',1001,(401+i*10)
     2		 ,.5,1.)
		    call hopera((1401+i*10),'*e',11001,(1401+i*10)
     2		 ,.5,1.)
		    call hopera((11401+i*10),'*e',111001,(11401+i*10)
     2		 ,.5,1.)
		    call hopera((402+i*10),'*e',1002,(402+i*10)
     3		 ,.5,1.)
		    call hopera((1402+i*10),'*e',11002,(1402+i*10)
     3		 ,.5,1.)
		    call hopera((11402+i*10),'*e',111002,(11402+i*10)
     3		 ,.5,1.)
		    call hopera((403+i*10),'*e',1003,(403+i*10)
     4		 ,.5,1.)
		    call hopera((404+i*10),'*e',1004,(404+i*10)
     5		 ,.5,1.)
                    call hopera((405+i*10),'*e',1007,(405+i*10)
     6		 ,.5,1.)
		    call hopera((406+i*10),'*e',1005,(406+i*10)
     7		 ,.5,1.)
		    call hopera((1406+i*10),'*e',11005,(1406+i*10)
     7		 ,.5,1.)
		    call hopera((11406+i*10),'*e',111005,(11406+i*10)
     7		 ,.5,1.)
		    call hopera((407+i*10),'*e',1006,(407+i*10)
     8		 ,.5,1.)
		    call hopera((1407+i*10),'*e',11006,(1407+i*10)
     8		 ,.5,1.)
		    call hopera((11407+i*10),'*e',111006,(11407+i*10)
     8		 ,.5,1.)
                    call hopera((408+i*10),'*e',1008,(408+i*10)
     9		 ,.5,1.)
                    call hopera((1408+i*10),'*e',11008,(1408+i*10)
     9		 ,.5,1.)
                    call hopera((409+i*10),'*e',1008,(409+i*10)
     &		 ,.5,1.)
                    call hopera((1409+i*10),'*e',1009,(1409+i*10)
     &		 ,.5,1.)
                    do j=1,6
                       call hopera((4010+100*i+j),'*e',1001,
     &                     (4010+100*i+j),.5,1.)
                       call hopera((14010+100*i+j),'*e',11001,
     &                     (14010+100*i+j),.5,1.)
                       call hopera((114010+100*i+j),'*e',111001,
     &                     (114010+100*i+j),.5,1.)
                    enddo
                    do j=1,6 
                       call hopera((4020+100*i+j),'*e',1002,
     &                     (4020+100*i+j),.5,1.)
                       call hopera((14020+100*i+j),'*e',11002,
     &                     (14020+100*i+j),.5,1.)
                       call hopera((114020+100*i+j),'*e',111002,
     &                     (114020+100*i+j),.5,1.)
                    enddo
		    
		 enddo

	      ENDIF
	   	      
	   endif
	else			!
C       *** End ***
	WRITE(6,*)
     &  'ichep02849: Please run routine with iflag set to 1,2 or 3'
	endif			!
	
        
	return
	end
 
 
**********************************
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
