*CMZ :  2.00/06 02/05/2002  14.48.09  by  Tancredi Carli
*CMZ :  1.01/11 09/12/97  16.44.11  by  Tancredi Carli
*CMZ :  1.01/07 10/10/97  19.46.50  by  Tancredi Carli
*CMZ :  1.01/01 28/03/96  15.40.42  by  Tancredi Carli
*-- Author :
      subroutine HZTH001(IFLAG)
****************************************************************************
* Purpose: produce histograms for:
*           o) transverse energy distribution in cms for the central
*              pseudorapidity bin as a fct. of x,Q**2
*
* Event selection:
* Eel>12 GeV, 173.0>Thel>157.0, w2>4400 GeV**2 and cut on forward energy
*
* Running: Ee=26.7, Ep=820, no polarization, HERA running 1993
* Reference: Event selection as in Phys. Lett. B356 (1995) 118, DESY 95-108
*            Pseudo data produced by Ariadne 4.03 in fair agreement with
*            uncorrected published data.
* Arguments: iflag=1 initialisation
*            iflag=1 filling
*            iflag=3 termination
*
* written by: Leif Lonnblad <leif@nordita.dk> adapted from the hz95108
*             routine by Michael Kuhlen (kuhlen@desy.de)
****************************************************************************
      IMPLICIT NONE
#include "hzfunc.inc"
#include "hepevtp.inc"
#include "heracmn.inc"
      Integer iflag,ihep,i
*     Not DESY Preprint number
      Character*5 xxxx
      character*30 string
*
      Real pi,rd,ee,eb
      Logical evcut,lp
      Integer iel,idum,id
      Parameter (pi=3.1415927,rd=180./pi,ee=26.7)
      Integer nev(9),nentry
      real zeroes(9), etcent
      Real x,y,q2,w2,enel,thel,efwd,th,rap,pt,et,p,thl
      Real etf11,etf12,etf13,etf21,etf22,etf23
      Integer ibin
*
      Integer ierr,ibeam
      Double precision pbeam(4),pgam(4),pcm(4),ph(5)
*
*
*********************************************************************
*
*                      data points from Ariadne 4.03
*
*********************************************************************
      Real xmin,xmax,binw,scale
      Integer nb,nbin
      Parameter (nb=20,xmin=0.0,xmax=10.0,nbin=9)
      Real et3(nb),eet3(nb)
      save xxxx,nev,lp,nentry,zeroes,binw
*
*     energy flow data in the 9 bins
*     note: systematic errors are not included
*
      Data  et3/0.3387,    0.3156,    0.3445,    0.2673,    0.2059,
     $          0.1233,    0.9901E-01,0.7632E-01,0.4579E-01,0.3878E-01,
     $          0.2970E-01,0.2310E-01,0.2104E-01,0.1403E-01,0.1196E-01,
     $          0.8663E-02,0.7838E-02,0.7426E-02,0.3713E-02,0.4125E-02/
      Data eet3/0.01183216,0.01141052,0.01192057,0.01050238,0.00921520,
     $          0.00713372,0.00639140,0.00561160,0.00434626,0.00400000,
     $          0.00350000,0.00308723,0.00294618,0.00240541,0.00222171,
     $          0.00189050,0.00179833,0.00174241,0.00123774,0.00130461/
*
      data zeroes /9*0/
      Data nev/9*0/,lp/.false./,nentry/0/
      Data xxxx/'TH001'/
*********************************************************************
*
*                      Initialization
*
*********************************************************************
      NENTRY=NENTRY+1
*
      IF (iflag.eq.1) then
*
*
*      Initialisation: The following MUST always be done
*      (i) make subdirectory in PAWC
*       - use the name as the xxxxxx in HZxxxxxx subroutine
*      (i) make subdirectory in o/p file
*
       Call hcdir('//PAWC',' ')
       call hmdir(xxxx,'S')
       Call hcdir('//HISTO',' ')
       call hmdir(xxxx,'S')
*                                             book data et-flow histos
       call hbook1(-13,'DATA |eta|<0.5  dN/dEt CMS bin= 3',
     $                  nb,xmin,xmax, 0.)
       Do i=1,nbin
         write(string,'(A,I2)') ' dN/dEt CMS bin= ',i
         call hbook1(10+i,'(etamax<4) |eta|<0.5 '//string,
     $        nb,xmin,xmax, 0.)
         call hbook1(30+i,'(etamax<4) -1.5<eta<-0.5'//string,
     $        nb,xmin,xmax, 0.)
         call hbook1(40+i,'(etamax<4) -2.5<eta<-1.5'//string,
     $        nb,xmin,xmax, 0.)
         call hbook1(50+i,'(etamax<5) -1.5<eta<-0.5'//string,
     $        nb,xmin,xmax, 0.)
         call hbook1(60+i,'(etamax<5) -2.5<eta<-1.5'//string,
     $        nb,xmin,xmax, 0.)
         call hbook1(70+i,'(etamax<7) -1.5<eta<-0.5'//string,
     $        nb,xmin,xmax, 0.)
         call hbook1(80+i,'(etamax<7) -2.5<eta<-1.5'//string,
     $        nb,xmin,xmax, 0.)
       enddo
       binw=real(nb)/(xmax-xmin)
*
*
*      pack data into histograms
*      pack data et/deta in the 9 bins into histograms
*
       Call hpak (-13,et3)
       Call hpake(-13,eet3)
*
*                                       book your histograms  (data: ID<0)

*          Count events in each x,Q2 bin

       call hbook1( 24,'# events vs kin. bin',9,1.,10.,0.)

       call hbarx(24)
*
*********************************************************************
*
*                      Event Processing
*
*********************************************************************

      Else if(iflag.eq.2) then

*      Filling: The following MUST always be done
*      (i) move to the correct sub-directory in PAWC
*
       call hcdir('//PAWC/'//xxxx,' ')
*
*                                             Event selection
*
       q2=real(HZDISKIN(1))
       x =real(HZDISKIN(2))
       y =real(HZDISKIN(3))
       w2=real(HZDISKIN(4))
*
       if (lp) write(6,*) 'Hz'//xxxx,
     & ' x= ',x,' q2 ',q2,' y= ',y,' w2= ',w2
*
*                                             get electron variables
*
       iel=HZIDELEC(idum)
       if (iel.eq.-1) then
        write(6,*) 'Hz'//xxxx,' electron not found '
        goto 9990
       endif
       enel=real(PHEP(4,iel))
       thel=real(HZPHMANG(PHEP(3,iel),
     &               sqrt(PHEP(1,iel)**2+PHEP(2,iel)**2)))*rd
       if (lp) write(6,*) 'Hz'//xxxx,
     & ' enel= ',enel,' thel= ',thel*rd
*
*                                             kinematic selection
*
       evcut= enel.gt.12.
       evcut= thel.gt.157. .and. thel.lt.173.0 .and. evcut
       evcut= w2.ge.4400..and. evcut
       if(.not.evcut) goto 9990
*
*                                             cut on forward energy
*
       efwd=0.
       Do ihep=1,nhep
        if (isthep(ihep).eq.1.and.ihep.ne.iel) then
         th=real(HZPHMANG(PHEP(3,ihep),
     &              sqrt(PHEP(1,ihep)**2+PHEP(2,ihep)**2)))*rd
         if(th.gt.4.4.and.th.lt.15.) then
          efwd=real(PHEP(4,ihep))+efwd
         endif
        endif
       enddo
       if (lp) write(6,*) 'Hz'//xxxx,
     & ' efwd= ',efwd,' evcut= ',evcut
       if(efwd.lt.0.5) goto 9990
*
*                                             find the kinematic bin number
       ibin = 0
       if(    q2.gt. 5..and.q2.le.10.) then
         if(x.gt.0.0001.and.x.le.0.0002) ibin=1
         if(x.gt.0.0002.and.x.le.0.0005.and.q2.gt.6.) ibin=2
       elseif(q2.gt.10..and.q2.le.20.) then
         if(x.gt.0.0002.and.x.le.0.0005) ibin=3
         if(x.gt.0.0005.and.x.le.0.0008) ibin=4
         if(x.gt.0.0008.and.x.le.0.0015) ibin=5
         if(x.gt.0.0015.and.x.le.0.0040) ibin=6
       elseif(q2.gt.20..and.q2.le.50.) then
         if(x.gt.0.0005.and.x.le.0.0014) ibin=7
         if(x.gt.0.0014.and.x.le.0.0030) ibin=8
         if(x.gt.0.0030.and.x.le.0.0100) ibin=9
       endif
*
*                                             events in 9 bins only
       if( ibin.eq.0) goto 9990
*                                             set up CMS boost needed later
*
       ierr=HZIBEAM(ibeam,idum)
       if (.not.(ierr.eq.1)) then
        write(6,*) 'HZ'//xxxx,' beams not found ! '
        goto 9990
       else
        Do i=1,4
         pbeam(i)=PHEP(i,IBEAM)
        enddo
       endif
*
       eb=real(PHEP(4,idum))
       if (abs(abs(eb)-ee).gt.0.2) then
        if (nentry.lt.10) then
         write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,'  ! '
        elseif (nentry.eq.10) then
         write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,
     &   ' ...last message ! '
        endif
C        goto 9990
       endif
*
       ierr=HZIPGAM(ph)
       if (ierr.eq.-1) then
        write(6,*) 'HZ'//xxxx,' boson vector not found ! '
C        call VZERO(ph,5)
        do i=1,5
         ph(i)=0.
        enddo
       else
        Do i=1,4
         pgam(i)=ph(i)
        enddo
       endif
*
       call HZHCMINI(pbeam,pgam,ierr)
       if (ierr.eq.1) then
        write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
        goto 9990
       endif
*
* loop over objects
*
*
* et in central rapidity bin
       etcent = 0.
       etf11=0.0
       etf12=0.0
       etf13=0.0
       etf21=0.0
       etf22=0.0
       etf23=0.0
       do 2000 ihep=1,nhep
*                                             skip unstable particles and el.
         if (isthep(ihep).ne.1.or.ihep.eq.iel) goto 2000
*
*                                             transform in CMS
*
         thl=real(HZPHMANG(PHEP(3,ihep),
     &        sqrt(PHEP(1,ihep)**2+PHEP(2,ihep)**2)))*rd

         Do i=1,4
           Ph(i)=PHEP(i,ihep)
         enddo
         call HZHCM(ph,pcm,ierr)
         if (ierr.eq.1) then
           write(6,*) 'HZ'//xxxx,' problem with boost to cms ! '
           goto 9990
         endif
* get particle momenta
         p=real(sqrt(pcm(1)**2+pcm(2)**2+pcm(3)**2 ))
         pt=real(sqrt( pcm(1)**2+pcm(2)**2 ))
         if (p.eq.0.) then
           if (lp) write(6,*) xxxx,' p = ',p
           goto 2000
         else
           et     = real(pcm(4)) * pt / p
           th     = (acos(real(pcm(3))/p))
           if (abs(     th*rd).lt.1.e-4 .or.
     &          abs(180.-th*rd).lt.1.e-4 ) then
             if (lp) write(6,*) xxxx,' th = ',th
             goto 2000
           else
             rap    = -log(tan( th/2. ))
* check on rap=nan (e.g. pt=0)
             if (rap.ne.rap) then
               if (lp) write(6,*) xxxx,'Warning:  rap = ',rap
     &              ,' th= ',th*rd,' particle skipped ! '
               goto 2000
             endif
           endif
         endif
*
         if (lp) write(6,*) 'Hz'//xxxx,
     &        'lab: et,pt,rap,th ',et,pt,rap,th*rd

* accumulate Et in central rap.
         if(rap.gt.-0.5.and.rap.lt.0.5) etcent = etcent + et
         if(rap.gt.-1.5.and.rap.lt.-0.5) then
           if (thl.gt.0.105) etf11=etf11+et
           if (thl.gt.0.772) etf12=etf12+et
           if (thl.gt.2.009) etf13=etf13+et
         endif
         if(rap.gt.-2.5.and.rap.lt.-1.5) then
           if (thl.gt.0.105) etf21=etf21+et
           if (thl.gt.0.772) etf22=etf22+et
           if (thl.gt.2.009) etf23=etf23+et
         endif

* end loop over objects
 2000  continue
*
* Fill Et histo
       nev(ibin)   = nev(ibin)+1
*
       call hf1(10+ibin,etcent,1.0)
       call hf1(30+ibin,etf13,1.0)
       call hf1(40+ibin,etf23,1.0)
       call hf1(50+ibin,etf12,1.0)
       call hf1(60+ibin,etf22,1.0)
       call hf1(70+ibin,etf11,1.0)
       call hf1(80+ibin,etf21,1.0)
* norm. histo for event counting
       call hf1(24,real(ibin),1.)
* end of event processing
 9990  continue


*********************************************************************
*
*                      Termination
*
*********************************************************************

      Else if(iflag.eq.3) then

* Termination: The following MUST always be done
* (i) Move to the correct PAW subdirectory
*
       call hcdir('//PAWC/'//xxxx,' ')
* normalize Et histos
       do id=1,9
         scale=binw/real(nev(id))
         Call hopera(id+10,'+E',id+10,id+10,scale,0.)
         Call hopera(id+30,'+E',id+30,id+30,scale,0.)
         Call hopera(id+40,'+E',id+40,id+40,scale,0.)
         Call hopera(id+50,'+E',id+50,id+50,scale,0.)
         Call hopera(id+60,'+E',id+60,id+60,scale,0.)
         Call hopera(id+70,'+E',id+70,id+70,scale,0.)
         Call hopera(id+80,'+E',id+80,id+80,scale,0.)
       enddo
        call hzchisq(-13,13)
* set errors of norm. histo 0
       call hpake(24,zeroes)
*
      endif
*
      RETURN
      END
