*CMZ :  1.01/01 28/03/96  16.21.35  by  Tancredi Carli
*CMZ :  1.00/02 04/03/96  13.37.02  by  Unknown
*-- Author :    Armen Buniatian   01/03/96


      SUBROUTINE H1GPCONE(ETALIM,NETA,NPHI,ETINI,ETMIN,ETBIN,ETMAX,
     +                 RJET, NJETS,JETS)
*********************************************************************
*
*      H1-Cone jet finder algorithm
*
*********************************************************************
*
*  INPUT:
*
*  ETALIM = psewdorapidity limit  (+ETALIM   -ETALIM)
*  NETA   = number of psewdorapidity bins in ETA-PHI space (MAX=50)
*  NPHI   = number of PHI  bins in ETA-PHI space (MAX=50)
*  ETINI  = min transvers energy of initiator
*  ETMIN  = Etjet min
*  ETBIN  = bin size for Etjet distribution
*  ETMAX  = Etjet max
*  RJET   = cone size
*
*  OUTPUT :
*
*  NJETS  = number of jets
*  JETS   = REAL ARRAY JETS(10,3)
*  JETS(I,1) = Etjet
*  JETS(I,2) = ETA jet (psewdorapidity)
*  JETS(I,3) = asimuthal angle of jet axes (0 - 2*PI)
*********************************************************************
*
      PARAMETER (MAXCE=50,PI=3.141593)
      REAL JETS(10,3),CELL(MAXCE,MAXCE)
      INTEGER LOCK(MAXCE,MAXCE)
      REAL DPHI
*
#include "hepevtp.inc"
*
      IF(NETA.GT.MAXCE) THEN
              PRINT *,'TO BIG NETA'
              GOTO 99
      ENDIF
      IF(NPHI.GT.MAXCE) THEN
              PRINT *,'TO BIG NPHI'
              GOTO 99
      ENDIF
*
*
*     Internal 'jet' steering:
*
*         Ecut for the cell Et energy |Et cell| > ETCUT
*
      ETCUT = 0.050
*
*
*
      NJETS = 0
      MAXCEQ = MAXCE*MAXCE
      CALL VZERO(JETS,30)
      CALL VZERO(CELL,MAXCEQ)
C*TC>
C      CALL VZERO(LOCK,MAXCEQ)
      do i=1,MAXCE
       do j=1,maxce
        lock(i,j)=0
       enddo
      enddo
C*TC>
*

      BINETA = 2.*ETALIM/FLOAT(NETA)
      BINPHI = 2.*PI/FLOAT(NPHI)
      NBETA = RJET/BINETA + 1
      NBPHI = RJET/BINPHI + 1
*
*     Fill CELL() here
*==================================================================
*
        NTRK  = NHEP
        ITRK0 = 1
c       write(6,*) NHEP,PHEP(1,1),PHEP(2,1),PHEP(3,1),
c    * PHEP(4,1),PHEP(5,1)
*===================================================================
        COEF=1.
* Is Z axis in the proton direction?
        DO ITRK=1,NHEP
          PZ   = PHEP(3,itrk)
          ICOD = IDHEP(itrk)
          if(ICOD.eq.2212) then
             IF(PZ.lt.0.) COEF=-1.
          endif
        ENDDO

        DO 100 ITRK = ITRK0, NTRK
          PX   = PHEP(1,itrk)
          PY   = PHEP(2,itrk)
          PZ   = PHEP(3,itrk)*COEF
          E    = PHEP(4,itrk)
          AM   = PHEP(5,itrk)
          ICOD = IDHEP(itrk)
          ISTA = ISTHEP(itrk)
c         write(6,*)'PX,PY,PZ,E,AM,ICOD,ISTA=',
c    *    itrk,PX,PY,PZ,E,AM,ICOD,ISTA
*     neutrino
          IF(ICOD.EQ.12 .OR. ICOD.EQ.14 .OR. ICOD.EQ.16) GO TO 100
          IF(ISTA.NE.1) GO TO 100
          PXY= SQRT(PX**2 + PY**2)
          IF(PXY.LT.1.E-8) GO TO 100
          P  = SQRT(PXY**2 + PZ**2)
          IF(ABS(P).LE.1.E-8 ) GO TO 100
          THETA = ACOS (PZ/P )
          IF(THETA.LT.1.E-8) GO TO 100
          IF(ABS(THETA-PI).LT.1.E-6) GO TO 100
          ETA = -ALOG(TAN(THETA/2.))
          IF(ABS(ETA).GT.ETALIM) GOTO 100
          PHI   = ATAN2(PY,PX)
          IF(PHI.LT.0.) PHI = 2 * PI + PHI
          ET = E*SIN(THETA)
          IETA = 1 + INT((ETALIM+ETA)/BINETA)
          IF(IETA.GT.NETA) IETA=NETA
          IPHI = 1 + INT(PHI/BINPHI)
          IF(IPHI.GT.NPHI) IPHI=NPHI
          CELL(IETA,IPHI) = ET + CELL(IETA,IPHI)
100     CONTINUE
*
*====================================================================
*

      IETMAX = NINT(ETMAX)
      IETMIN = NINT(ETMIN)
      IETBIN = NINT(ETBIN)
      NSTEPS=INT((etmax-etmin)/etbin)
      ETMAX0=ETMIN+ETBIN*NSTEPS
*
* Lock cells with Et<Etcut
      do 21 i=1,NETA
       do 22 j=1,nphi
            IF(ABS(CELL(I,J)).LT.ETCUT) LOCK(I,J) = 2
22     continue
21    continue
*
*     loop over Etjet distribution
      DO 80 IETM = 1,nsteps
      ETMIN1=ETMAX0-IETM*ETBIN

*     unlock not used initiator cells
        DO 47 I=1,NETA
          DO 44 J=1,NPHI
            IF(LOCK(I,J).EQ.1)  LOCK(I,J) = 0
44        CONTINUE
47      CONTINUE
*
*          Find a jet initiator
10    CONTINUE


* what is a total Et in unlocked cells?
      etrem=0.
      DO 57 I=1,NETA
         DO 54 J=1,NPHI
            if(LOCK(I,J).NE.2) etrem=etrem+cell(i,j)
54       CONTINUE
57    CONTINUE
*
* if total Et in unlocked cells is smaller than THRESHOULD...
*
        if(etrem.lt.etmin1) go to 80


      ET1 = 0.0
      IET1 = 0
      JET1 = 0
      DO 17 I=1,NETA
        DO 14 J=1,NPHI
          IF(LOCK(I,J).NE.0)     GOTO 14
          IF(CELL(I,J).LT.ETINI) GOTO 14
          IF(ET1.GE.CELL(I,J))   GOTO 14
          ET1 = CELL(I,J)
          IET1 = I
          JET1 = J
 14     CONTINUE
 17   CONTINUE

*
      IF(ET1.LT.ETINI) GOTO 80
      ETAET1 = -ETALIM + BINETA*FLOAT(IET1) - BINETA/2.0
      PHIET1 =           BINPHI*FLOAT(JET1) - BINPHI/2.0
      LOCK(IET1,JET1) = 1
*
*       Collect cells
*
      IMIN = MAX0(1,IET1-NBETA)
      IMAX = MIN0(NETA,IET1+NBETA)
*
      ETJET = 0.
      DO 27 I=IMIN,IMAX
        DO 24 J=1,NPHI
          IF(LOCK(I,J).GT.1) GOTO 24
          ETA = -ETALIM + BINETA*FLOAT(I) - BINETA/2.0
          PHI =           BINPHI*FLOAT(J) - BINPHI/2.0
          DPHI = ABS(PHI-PHIET1)
          IF(DPHI.GT.PI) DPHI = 2.*PI - DPHI
          DR=SQRT((ETA-ETAET1)**2+DPHI**2)
          IF(DR.GT.RJET) GOTO 24
c         write(6,*)' etaet1,phiet1,dr',etaet1,phiet1,dr
          ETJET = ETJET + CELL(I,J)
c         print '(''cell:'',4F6.2)',cell(i,j),eta,phi,etjet
 24     CONTINUE
 27   CONTINUE
*
      IF(ETJET.LT.ETMIN1) GOTO 10
*
*        Correct Eta and Phi
*
      ETAJET = 0.
      PHIJET = 0.
      ETPOS =  0.
      XJET = 0.
      YJET = 0.
*
      DO 37 I=IMIN,IMAX
        ETA = -ETALIM + BINETA*FLOAT(I) - BINETA/2.0
        DO 34 J=1,NPHI
          IF(LOCK(I,J).GT.1)   GOTO 34
          PHI =       BINPHI*FLOAT(J) - BINPHI/2.0
          DPHI = ABS(PHI-PHIET1)
          IF(DPHI.GT.PI) DPHI = 2.*PI - DPHI
          DR=SQRT((ETA-ETAET1)**2+(DPHI)**2)
          IF(DR.GT.RJET) GOTO 34
          LOCK(I,J) = 2
          IF(CELL(I,J).LE.0.0) GOTO 34
c         print '(''cell:'',3F6.2)',cell(i,j),eta,phi
          ETPOS = ETPOS + CELL(I,J)
          ETAJET = ETAJET + ETA*CELL(I,J)
          XJET = XJET + COS(PHI)*CELL(I,J)
          YJET = YJET + SIN(PHI)*CELL(I,J)
c         print '(''jet :'',3F6.2)',etpos,etajet/etpos,atan2(yjet,xjet)
 34     CONTINUE
 37   CONTINUE
      IF(ETPOS.GT.0.0) THEN
        ETAJET = ETAJET/ETPOS
        XJET = XJET / ETPOS
        YJET = YJET / ETPOS
        PHIJET = ATAN2(YJET,XJET)
        IF(PHIJET.LT.0.0) PHIJET = 2.*PI + PHIJET
      ELSE
        PRINT *,' to many negative energy cells ??????'
        GOTO 10
      ENDIF
*

      DPHI = PHIJET-PHIET1
      IF(DPHI.LT.-PI) DPHI = - DPHI- PI*2.
      IF(DPHI.GT.PI)  DPHI = PI*2.- DPHI
      IF(NJETS.LT.10) THEN
        NJETS = NJETS + 1
        JETS(NJETS,1) = ETJET
        JETS(NJETS,2) = ETAJET
        JETS(NJETS,3) = PHIJET
      ELSE
        PRINT *,'to many jets !!!!!??????'
        GOTO 99
      ENDIF
*
      IF(ETJET.GE.ETMIN1) GOTO 10
*
 80   CONTINUE
 99   CONTINUE
      RETURN
      END
