*CMZ :  1.02/02 23/10/98  16.02.12  by  Tancredi Carli
*-- Author :
      SUBROUTINE HzJetshp
     .(iNormMod, dconeRad, djshpRho, djshpPsi, ierr)
* ---------------------------------------------------------------------------
*     purpose:   this subroutine calculates jetshape variables:
*                differential jetshape rho, integrated jetshape psi
*                in standard bins of r/R: 0.0,0.1,0.2,...,1.0
*
*     usage:     1. call HzJtfind for jet finding
*                2. call this routine
*
*                input arguments for this routine:
*                   iNormMod     : normalization mode
*                             = 0: jetshapes will be normalized with
*                                  jet Et
*                             = 1: jetshapes will be normlized with
*                                  summed Et of all particles belonging
*                                  to jet AND lying inside a cone of
*                                  radius dconeR (with cone axis=jet axis)
*
*                   dconeRad     : cone radius R
*                                  (if dconeR <= 0.D0, cone radius returned
*                                   by HzJtfind is used)
*
*                output arguments for this routine:
*                   djshpRho(i,j): differential jetshape rho of jet no.j
*                                  in r/R bin no.i
*
*                   djshpPsi(i,j): integrated jetshape rho of jet no.j
*                                  in r/R bin no.i
*
*                   ierr         : error flag
*                             = 0: everything o.k.
*                             =-1: an error occured
*
*     structure: this subroutine will only work correctly, if the
*                common block HZJETCMN has been filled before
*                (by HzJtfind) !!!
*                this subroutine calls the following subroutine/functions:
*                from HzTool lib: HzEta, HzPhi, HzEt
*                from PXCONE lib: PXMDPI
*
*     written by: Andreas von Manteuffel
*
* ---------------------------------------------------------------------------

      IMPLICIT NONE


#include "hepevtp.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"

c --- pxcone function for shifting phi into the interval (-pi,+pi)
      Double Precision pxmdpi

c --- arguments
c --- input:
      Integer iNormMod
      Double Precision dconeRad
c --- output:
      Double Precision djshpRho(10,MAXHZJETS)
      Double Precision djshpPsi(10,MAXHZJETS)
      Integer ierr

      Character*8 cName
      Data cName /'HzJetshp'/
      save cName
      Integer iNmess, INMXMESS
      Parameter (INMXMESS=10) ! limit for printed messages
      Data iNmess /0/
      Save iNmess
      Integer ijt, ihad, ir, idum
      Double Precision dcRadius
      Double Precision dEtJet, detaJet, dphiJet
      Double Precision dEtpart, detapart, dphipart
      Double Precision dphiDlta, detaDlta, drtoR
      Double Precision drtoRbin(11)
      Data drtoRbin / 0.0D0, 0.1D0, 0.2D0, 0.3D0, 0.4D0, 0.5D0,
     .                0.6D0, 0.7D0, 0.8D0, 0.9D0, 1.0D0 /
      save drtoRbin


c --- initializations
c -------------------

* initialize errorflag
      ierr = 0

* check if number of jets valid
      if (NUMJETS.gt.MAXHZJETS) then
         ierr = -1
         if (iNmess.lt.INMXMESS) then
            write(6,*) cName, ' ERROR: number of jets too big !!!'
            iNmess = iNmess + 1
         endif
         goto 9999
      endif

* initialize arrays
      do ijt = 1, MAXHZJETS
         do ir = 1, 10
            djshpRho(ir,ijt) = 0.D0
            djshpPsi(ir,ijt) = 0.D0
         enddo
      enddo

* check cone radius
      if (dconeRad.lt.1.D-18) then
*     get cone radius from HzJetrad
         call HzJetrad(2,dcRadius)
         if (dcRadius.lt.1.D-18) then
            ierr = -1
            if (iNmess.lt.INMXMESS) then
               write(6,*) cName,' ERROR: invalid cone radius !'
               iNmess = iNmess + 1
            endif
            goto 9999
         endif
      else
         dcRadius = dconeRad
      endif

* anything to do ?
      if (NUMJETS.lt.1) goto 9999


c --- calculation
c ---------------


* loop over all jets found	
      do 1000 ijt = 1, NUMJETS

*     get kinematic variables of current jet
         detaJet = JETS(ijt,1)
         dphiJet = JETS(ijt,2)
         dEtJet  = JETS(ijt,3)

         if (dEtJet.lt.0.) then
            ierr = -1
            if (iNmess.lt.INMXMESS) then
               write(6,*) cName,' ERROR: jet Et less than 0 !'
               iNmess = iNmess + 1
            endif
            goto 9999
         elseif (dEtJet.eq.0.) then
            goto 1000
         endif

*     loop over all particles
         do 2000 ihad = 1, NHEP

*     does particle belong to jet ?
            if (iJetNo(ihad).ne.ijt) goto 2000
*     note: unstable particles and lepton should never be associated to
*           any jet (this is implemented in HzJtfind)

*     get eta, phi and Et of particle
            detapart = HzEta(ihad)
            dphipart = HzPhi(ihad)
            dEtpart = HzEt(ihad)

*     get (r/R)**2 for current particle
            dphiDlta = pxmdpi(dphiJet-dphipart)
            detaDlta = detaJet-detapart
            drtoR = SQRT((detaDlta**2+dphiDlta**2)/(dcRadius**2))
            do ir = 1, 10
               if (drtoR.lt.drtoRbin(ir+1)) then
                  djshpPsi(ir,ijt) = djshpPsi(ir,ijt) + dEtpart
		  if (drtoR.ge.drtoRbin(ir)) then
                     djshpRho(ir,ijt) = djshpRho(ir,ijt) + dEtpart
		  endif         ! contribution to rho ?
               endif            ! contribution to psi ?
            enddo               ! next r/R step

 2000    continue               ! next hadron

*     normalize jetshapes to jet Et
*     note: factor 10. is for binwith normalization of rho
         do ir = 1, 10
           if (iNormMod.eq.0) then
             djshpRho(ir,ijt) = 10.D0*djshpRho(ir,ijt)/dEtJet
             djshpPsi(ir,ijt) = djshpPsi(ir,ijt)/dEtJet
           elseif (iNormMod.eq.1) then
             djshpRho(ir,ijt) = 10.D0*djshpRho(ir,ijt)/djshpPsi(10,ijt)
             djshpPsi(ir,ijt) = djshpPsi(ir,ijt)/djshpPsi(10,ijt)
           endif
         enddo
 1000 continue                  ! next jet

 9999 continue

      RETURN
      END
