*CMZ :  1.02/01 23/10/98  17.59.07  by  Tancredi Carli
*CMZ :  1.02/00 02/10/98  17.58.49  by  Tancredi Carli
*-- Author :    Klaus Rabbertz   18/08/97
      SUBROUTINE DGFJAD(NJETS,SC)
************************************************************************
*                                                                       *
*   DGFJAD                                                             *
*   purpose: calculate y_ij values for the factorizable JADE jet       *
*            algorithm (Breit frame, Q^2 or fixed scale)               *
*                                                                      *
*   called by: user                                                    *
*                                                                      *
*   input : - Breit frame input 4 vectors and number: via common QCTBUF*
*             (first vector has to be the "remnant"=(0,0,Q/2,Q/2))     *
*           - # of jets to reconstruct: NJETS + 1 (remnant)            *
*           - scale (Q, fixed value)  : SC                             *
*                                                                      *
*   output: - NJETS+1 output jets in QPO, common QCTBUF                *
*           - y_ij value for transition N+1 -> N jets QTBOR(1,N),      *
*             common QCTBUF (filled for all N with N <= NJETS+1)       *
*                                                                      *
*   written by: Klaus Rabbertz, 18/08/97 15.40.00                      *
************************************************************************

ckr
      IMPLICIT NONE
ckr

#include "qjmacs.inc"
C----------------------------------------------------------------------
ckr add. declaration
      INTEGER NJETS
      DOUBLE PRECISION SC
      INTEGER I,J,JTBW,ICL,JCL
      DOUBLE PRECISION DMIN,DIST,SKAL
      DOUBLE PRECISION QPW(5,JTBIMX)
ckr ende

*---initialization
      JTBW = JTBI
      DO J=1,JTBI
         DO I=1,4
            QPW(I,J) = QPI(I,J)
         ENDDO
         QPW(5,J) = SQRT(QPI(1,J)**2+QPI(2,J)**2+QPI(3,J)**2)
         QTBIR(1,J) = 0.D0
      ENDDO
      JTBO = NJETS+1
      DO J=JTBI+1,JTBO
         DO I=1,5
            QPW(I,J) = 0.D0
         ENDDO
         QTBIR(1,J) = 0.D0
      ENDDO
      DO J=1,JTBO
         DO I=1,4
            QPO(I,J) = 0.D0
         ENDDO
         QTBOR(1,J) = 0.D0
      ENDDO

*---only 1 initial 4 vector
      IF (JTBI.LE.1) THEN
         DO I=1,4
            QPO(I,1) = QPW(I,1)
         ENDDO
         QTBOR(1,1) = 0.D0
         RETURN

*---# output 4 vectors >= # input 4 vectors
      ELSEIF (JTBO.GE.JTBI) THEN
         DO J=1,JTBI
            DO I=1,4
               QPO(I,J) = QPW(I,J)
            ENDDO
         ENDDO
      ENDIF

*---find minimal distance
 10   DMIN = 1.D6
      DO I=1,JTBW-1
         DO J=I+1,JTBW
            IF (QPW(5,I).GT.1.D-6.AND.QPW(5,J).GT.1.D-6) THEN
               SKAL = QPW(1,I)*QPW(1,J)+QPW(2,I)*QPW(2,J)+
     &                QPW(3,I)*QPW(3,J)
               DIST = 2.D0*QPW(4,I)*QPW(4,J)*
     &                (1.D0-SKAL/QPW(5,I)/QPW(5,J))
            ELSE
               DIST = 0.D0
            ENDIF
            IF (DIST.LT.DMIN) THEN
               DMIN = DIST
               ICL  = I
               JCL  = J
            ENDIF
         ENDDO
      ENDDO

*---recombine
      IF (ICL.GT.1) THEN
         DO J=1,4
            QPW(J,ICL) = QPW(J,ICL) + QPW(J,JCL)
         ENDDO
         QPW(5,ICL) = SQRT(QPW(1,ICL)**2+QPW(2,ICL)**2+QPW(3,ICL)**2)
      ENDIF
      DO I=JCL,JTBW-1
         DO J=1,5
            QPW(J,I) = QPW(J,I+1)
         ENDDO
      ENDDO
      JTBW = JTBW-1
      QTBIR(1,JTBW) = DMIN/SC**2

*---store final recombined jets according to NJETS
*---(can only be true for JTBI < JTBO)
      IF (JTBW.EQ.JTBO) THEN
         DO J=1,JTBO
            DO I=1,4
               QPO(I,J) = QPW(I,J)
            ENDDO
         ENDDO
      ENDIF

*---cluster until only remnant left
      IF (JTBW.GT.1) GOTO 10

*---store y values
      DO I=1,JTBO
         QTBOR(1,I) = QTBIR(1,I)
      ENDDO

      RETURN
      END
