*CMZ :  0.00/03 29/10/98  11.16.17  by  Tancredi Carli
*-- Author :
      SUBROUTINE HzSubjm(iflag, ijet, inycut, ycut, isubjm, ierr)
***********************************************************************
*
* purpose:
*     This subroutine is an HzTool interface for the calculation
*     of SUBJET MULTICPLITIES using the KTCLUS package.
*
* usage:
*     1. jet finding: call HzJtfind with jetfinder selection KTCLUS
*     2. preparation of subjet multiplicity calculation:
*        call this subroutine HzSubjm with iflag = 1
*     3. calculation of subjet multiplicities:
*        call this subroutine HzSubjm with iflag = 2 (for each jet)
*     note: the commons HEPEVTP and HZJETCMN should not be changed
*           (directly or for example through HzJtfind) between the
*           steps mentioned above !!!
*
*     input arguments:
*        iflag  = mode flag (=1: initialization step)
*                           (=2: subjet multiplicity calculation step)
*        ijet   = HzTool number of jet to be analysed
*        inycut = number of ycuts
*        ycut   = ycut values
*     output arguments:
*        isubjm = subjet multiplicities for different ycut values
*        ierr   = errorflag (=0: no error, <0: something went wrong)
*
* structure:
*     the following functions/subroutines are called by this routine:
*     from KTCLUS lib: KTCLUS, KTINCL, KTISUB
*     from HzTool lib: HzIdelec
*
* written by:
*     Andreas von Manteuffel (Oct. 1998)
*
***********************************************************************

      Implicit None
*
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"
*


*
c --- arguments
c --- input:
      integer iflag, ijet, inycut
      real ycut(*)
c --- output:
      integer isubjm(*)
      integer ierr
*
c --- general stuff ---
      character*9 cxxxx
      data cxxxx /'HZSUBJM'/
      save cxxxx
*
C --- KTCLUS stuff ---
      integer IKTPMX
      parameter (IKTPMX=512)
      double precision ppp(4,IKTPMX),ktpjet(4,IKTPMX)
      integer          ktmode,ktreco
      integer          iKTjetno(IKTPMX) ! internal KT jet number
      double precision etjeti,etjetj,help
      real             yy(IKTPMX),ecut
      integer          parttojet(IKTPMX)
      integer          njet,iKTjt
      save njet, iKTjetno,ktpjet
*
c --- some variables
      logical lp
      data lp /.true./
      save lp
      INTEGER loop,count,iel,idum,i,j,k
      integer npart

c ---------------------------------------------------------------------

C --- initializations
      ierr = -1
      do loop=1,iNycut
         isubjm(loop) = 0.
      enddo


      if (iflag.eq.1) then
c ---------------------------------------------------------------------
c     PREPARE CALCULATION
c ---------------------------------------------------------------------

C --- get lepton id
         iel=HZIDELEC(idum)
         IF (iel.le.0) Then
            write(6,*) cxxxx,' lepton not found !'
            goto 9999
         ENDIF

C --- avoid invalid array-access
         if ((NHEP.lt.1).or.(NHEP.gt.NMXHEP)) then
            write(6,*) cxxxx,'invalid number of particles !'
         endif

c --- prepare KTCLUS call
         ktreco=2
         ktmode=3212
         ecut   = 1.
         count=0
         DO loop=1,NHEP
*     skip unstable particles and scattered lepton
           IF ((ISTHEP(loop).eq.1).and.(loop.ne.iel)) THEN
              IF (count.eq.IKTPMX) THEN
                 write(6,*) cxxxx,
     &                'Too many particles for KTCLUS',count
                 GOTO 85
              ELSE
                 count=count+1
                 ppp(1,count) = PHEP(1,loop)
                 ppp(2,count) = PHEP(2,loop)
                 ppp(3,count) = PHEP(3,loop)
                 ppp(4,count) = PHEP(4,loop)
              ENDIF
           ENDIF
         ENDDO
 85    CONTINUE
*
       NPART=count

c --- do KTCLUS call
       call KTCLUS(ktmode,ppp,npart,ecut,yy,*8000)
       call KTINCL(ktreco,ppp,npart,ktpjet,parttojet,NJET,*8000)

       if (NJET.gt.IKTPMX) goto 8000

c --- sort jets in Et
       Do loop=1,NJET
          iKTjetno(loop)=loop
       ENDDO
       do i = 1, NJET
        etjeti = sqrt( ktpjet(1,i)**2 + ktpjet(2,i)**2 )
        do j = (i+1), NJET
         etjetj = sqrt( ktpjet(1,j)**2 + ktpjet(2,j)**2 )
         if( etjetj.gt.etjeti )then
          do k = 1, 4
           help      = ktpjet(k,i)
           ktpjet(k,i) = ktpjet(k,j)
           ktpjet(k,j) = help
          enddo
          k=iKTjetno(i)
          iKTjetno(i)=iKTjetno(j)
          iKTjetno(j)=k
         endif
        enddo ! j
       enddo ! i

      elseif (iflag.eq.2) then
c ---------------------------------------------------------------------
c     SUBJET MULTIPLICITY CALCULATION
c ---------------------------------------------------------------------

c --- check selected jet number
         if ((ijet.lt.1).or.(ijet.gt.NUMJETS)) then
            write(6,*) cxxxx,' invalid jet selection !'
            goto 9999
         endif
c --- get KTCLUS internal number of selected jet (see Et-ordering above)
         iKTjt = iKTjetno(ijet)
         if ((iKTjt.lt.1).or.(iKTjt.gt.IKTPMX)) then
            write(6,*) cxxxx,' internal information corrupted !!!'
            goto 9999
         endif

         call KTISUB(iKTjt, iNycut, ycut, isubjm, *8000)

      else
         write(6,*) cxxxx,' bad subroutine call !'
         goto 9999
      endif

      ierr = 0
      goto 9999

 8000 continue
      write(6,*) cxxxx,' ERROR in KTCLUS !'

 9999 continue
      Return
      End
