      SUBROUTINE HZC96132(iflag)
*****************************************************************************
* This photoproduction routine has to be run thrice with the following
* code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the SINGLY-RESOLVED component run.
*       +3000 for the DOUBLY-RESOLVED component run
* for all three phases (iflag=1,2,3)
*
* iflag = 1,2 or 3 depending on wether it is the initialization,
*		processing or termination phase (respectively)
*
* The default jetfinder is PXCONE
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
* This routine prduces the following graphs:
*     10  Corresponds to figure 7 in paper
*     11  Figure 8
*     20  Figure 9
*     21  Figure 10
*
* Cuts : 	Anti-tag on scattered beam particles (< 25 mrad)
*               Et > 3 GeV
*               |jet rapidity| < 1.0
*
* Recommended value for Ptmin is 2.2 GeV (to be set in MC set up)
*
* Author : Russell Taylor (rjt@hep.ucl.ac.uk)
*
* Reference: CERN-PPE/96-132
*            Zeit. fur Physik C73 (1997) 433
*
* Modified :
* J. Monk (allow to be called with iproc=0) 6/2/2006
*
*****************************************************************************
*
* HERA Tuning defined commons
 
	IMPLICIT NONE
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzjetcmn.inc" 
	CHARACTER*6 jetf
 
	INTEGER njet,njet2
c	DOUBLE PRECISION jets(50,8)
	DOUBLE PRECISION coner
        SAVE coner
        DOUBLE PRECISION hzeekin
	
	INTEGER loop,loop2,nloop
	DOUBLE PRECISION rap(10),Et(10)
	DOUBLE PRECISION ybj,q2,angle
	DOUBLE PRECISION ybjcut(2),q2cut(2)
	DOUBLE PRECISION Etcut(6)
	DOUBLE PRECISION mhxsec(3)
	real mhntot(3)
        REAL norm
 
        INTEGER ivec
        REAL temp(7)
 
C --- Data for one-jet Et graph (|eta|<1)
        REAL xbins(7)
        REAL det(6),det_e(6),det_esys(6)
        DATA xbins/3.0,4.0,5.0,6.5,8.5,11.0,16.0/
        DATA det/163.4,36.0,18.0,8.4,1.8,0.78/
        DATA det_e/5.8,2.3,1.5,1.0,0.3,0.17/
        DATA det_esys/26.7,7.7,3.6,1.8,0.5,0.24/
 
C --- Data for two-jet Et graph (|eta|<1)
        REAL det2(6),det2_e(6),det2_esys(6)
        DATA det2/36.0,18.4,8.7,3.7,1.13,0.32/
        DATA det2_e/2.9,2.0,1.1,0.6,0.28,0.09/
        DATA det2_esys/4.2,3.5,0.8,1.4,0.34,0.17/
 
C --- Data for rapidity graphs
C --- one-jet (Et>3 GeV)
        REAL drap(4),drap_e(4),drap_esys(4)
        DATA drap/277.,262.,228.,240./
        DATA drap_e/16.,15.,12.,13./
        DATA drap_esys/27.,29.,32.,56./
C --- two-jet (Et>3 GeV)
        REAL drap2(4),drap2_e(4),drap2_esys(4)
        DATA drap2/67.,92.,74.,72./
        DATA drap2_e/7.,9.,8.,8./
        DATA drap2_esys/11.,12.,9.,12./
 
	INTEGER intiflag, chjet, iproc, iflag
	SAVE chjet


C --- Default jet finder (as used in paper) -- PXCONE
	DATA chjet/2/
 
C --- choose direct/resolved
C --- recommend 1 - DIRECT, 2 - 1-RESOLVED, 3 - 2-RESOLVED
C --- 0 - all


        IF (iflag.gt.1000) THEN
           iproc=int(iflag*0.001)
           intiflag=MOD(iflag,1000)
        else
           intiflag=iflag
           iproc=0
        ENDIF

        if (iproc.gt.3.or.iproc.lt.0)then
           print*,'C96132 : Badly called routine'
           PRINT*,'           set intiflag = iproc*1000 + iflag'
           print*,'           with iproc = 0    (all)'
           print*,'                        1000 (direct)'
           print*,'                        2000 (single resolved)'
           print*,'                        3000 (double resolved)'
        endif



c	IF ((iproc.gt.3).or.(iproc.lt.1)) THEN
c	  PRINT*,'C96132 : Badly called routine'
c	  PRINT*,'           set intiflag +1000, +2000 or +3000'
c	ENDIF
 
C --- choose jet finder
	IF ((intiflag.gt.10).and.(MOD(intiflag,10).eq.1)) THEN
	  chjet=INT(intiflag/10)
	  intiflag=MOD(intiflag,10)
	ENDIF
 
	IF (intiflag.eq.1) THEN
C *** Initialisation run ***
C *** Create directories ***

           if(iproc.le.1)then
	    CALL HCDIR('//HISTO',' ')
	    CALL HMDIR('c96132','S')
	    CALL HCDIR('//PAWC',' ')
	    CALL HMDIR('c96132','S')
 
            CALL HBOOKB(-10,'Data 1-Jet X-Sec(pb), |eta|<1',6,xbins,0.)
            CALL HBOOKB(-110,'Data 1-Jet X-Sec(pb), |eta|<1',6,xbins,0.)
            CALL HPAK(-10,det)
            CALL HPAKE(-10,det_e)
 
            DO ivec=1,6
               temp(ivec)=sqrt(det_e(ivec)**2+det_esys(ivec)**2)
            ENDDO
            CALL HPAK(-110,det)
            CALL HPAKE(-110,temp)
 
            CALL HBOOKB(-11,'Data 2-Jet X-Sec(pb), |eta|<1',6,xbins,0.)
            CALL HBOOKB(-111,'Data 2-Jet X-Sec(pb), |eta|<1',6,xbins,0.)
            CALL HPAK(-11,det2)
            CALL HPAKE(-11,det2_e)
 
            DO ivec=1,6
               temp(ivec)=sqrt(det2_e(ivec)**2+det2_esys(ivec)**2)
            ENDDO
            CALL HPAK(-111,det2)
            CALL HPAKE(-111,temp)
 
            CALL HBOOK1(-20,'Data 1-jet X-Sec(pb), Et(jet)>3',
     &                        4,0.0,1.0,0.)
            CALL HBOOK1(-120,'Data 1-jet X-Sec(pb), Et(jet)>3',
     &                        4,0.0,1.0,0.)
            CALL HPAK(-20,drap)
            CALL HPAKE(-20,drap_e)
 
            DO ivec=1,4
               temp(ivec)=sqrt(drap_e(ivec)**2+drap_esys(ivec)**2)
            ENDDO
            CALL HPAK(-120,drap)
            CALL HPAKE(-120,temp)
 
            CALL HBOOK1(-21,'Data 2-jet X-Sec(pb), Et(jet1,2)>3',
     &                        4,0.0,1.0,0.)
            CALL HBOOK1(-121,'Data 2-jet X-Sec(pb), Et(jet1,2)>3',
     &                        4,0.0,1.0,0.)
            CALL HPAK(-21,drap2)
            CALL HPAKE(-21,drap2_e)
 
            DO ivec=1,4
               temp(ivec)=sqrt(drap2_e(ivec)**2+drap2_esys(ivec)**2)
            ENDDO
            CALL HPAK(-121,drap2)
            CALL HPAKE(-121,temp)
 
 
            CALL HBOOKB(10,'MC 1-jet X-Sec(pb), |eta|<1',6,xbins,0.)
            CALL HBOOKB(11,'MC 2-jet X-Sec(pb), |eta|<1',6,xbins,0.)
            CALL HBOOK1(20,'MC 1-jet X-Sec(pb), Et(jet)>3',
     &                        4,0.0,1.0,0.)
            CALL HBOOK1(21,'MC 2-jet X-Sec(pb), Et(jet1,2)>3',
     &                        4,0.0,1.0,0.)
 
          ENDIF
 
          CALL HCDIR('//HISTO/c96132',' ')
          CALL HCDIR('//PAWC/c96132',' ')
 
          if(iproc.ge.1) then
             CALL HBOOKB(10+iproc*100,'MC component',6,xbins,0.)
             CALL HBOOKB(11+iproc*100,'MC component',6,xbins,0.)
             CALL HBOOK1(20+iproc*100,'MC component',4,0.0,1.0,0.)
             CALL HBOOK1(21+iproc*100,'MC component',4,0.0,1.0,0.)
          endif

          CALL HBARX(0)
 
C	  ybjcut(1)=0.0
C	  ybjcut(2)=1.0
C	  q2cut(1)=0.
C	  q2cut(2)=1.8      !rough estimate - will do for now
 
C --- standard cone radius is 1.0
	  CALL HZJETRAD(2,CONER)
	  IF (CONER.lt.0.0) THEN
	    CONER=1.0
	  ENDIF
	  CALL HZJTNAME(chjet,jetf)
	  WRITE(6,*)'***********************************************'
	  WRITE(6,*)'* C96132 called, histograms will be output. *'
	  WRITE(6,*)'*     Jet finder selected : ',jetf,'            *'
	  WRITE(6,1001)'*        with cone radius : ',CONER,'          *'
	  WRITE(6,*)'***********************************************'
 
 1001	  FORMAT(A29,F5.2,A11)
 
       ELSE IF (intiflag.eq.2) THEN
C *** Filling runs ***
          

C *** Change directory to our graphs ***
           CALL HCDIR('//PAWC/c96132',' ')
 
C	  q2=hzeekin(1)
C	  IF (q2.eq.-1) THEN
C	    PRINT*,'C96132: HZEEKIN returns error for Q2'
C	  ELSEIF (q2.lt.0) THEN
C	    q2=abs(q2)
C	  ENDIF
 
           angle=hzeekin(2)
 
c           write(*,*)'Angle = ',angle

C *** Fill our histogram ***
 
           IF (angle.lt.0.025) Then
 
              CALL hzjtfind(chjet,CONER,NJET,Jets)
 
c              write(*,*)'njets = ',NJET


              IF (NJET.ge.1) THEN
                 njet2=1
                 DO loop=1,NJET
                    IF ((Jets(loop,3).ge.3.0).and.(ABS(Jets(loop,1))
     &                   .le.1.0)) THEN
                       Et(njet2)=Jets(loop,3)
                       rap(njet2)=Jets(loop,1)
                       CALL HFILL
     &                      (iproc*100+10,REAL(Et(njet2)),0.,wtx)
                       CALL HFILL
     &                      (iproc*100+20,REAL(ABS(rap(njet2))),0.,wtx)
                       njet2=njet2+1
                    ENDIF
                 ENDDO
                 IF (njet2.ge.3) THEN
                    CALL HFILL(iproc*100+11,REAL(Et(1)),0.,wtx)
                    CALL HFILL(iproc*100+11,REAL(Et(2)),0.,wtx)
                    CALL HFILL(iproc*100+21,REAL(ABS(rap(1))),0.,wtx)
                    CALL HFILL(iproc*100+21,REAL(ABS(rap(2))),0.,wtx)
                 ENDIF
              ENDIF
              
           ENDIF                !anti-tag cut
           
      ELSE IF (intiflag.eq.3) THEN
C *** Termination run ***
C *** Change directory ***

	  CALL HCDIR('//PAWC/c96132',' ')

C *** Finish off histograms ***
	  IF (Xsec.eq.0) THEN
	    PRINT*,'C96132: termination called with zero cross section'
	    PRINT*,'          cross section graph meaningless'
	    PRINT*,'	      in process :',iproc*1000
	    Xsec = 1
	  ENDIF
	  IF (Ntot.lt.1) THEN
	    PRINT*,'C96132: termination called with no total events'
	    PRINT*,'          cross section graph meaningless'
	    PRINT*,'          in process :',iproc*1000
	    Ntot=1
	  ENDIF
 
C --- store Xsec and Ntot for both direct and resolved events.
C --- and normalise direct/resolved histos
	  if(iproc.gt.0) then

             mhXsec(iproc)=1000*Xsec ! convert to fb^-1
             mhntot(iproc)=Ntot
             norm=REAL(mhntot(iproc)/mhXsec(iproc))

          else

             norm=REAL(Ntot *0.001 / Xsec)

          endif

          CALL HZHINRM(100*iproc+10,0,norm,1)
          CALL HZHINRM(100*iproc+20,0,norm,1)
          CALL HZHINRM(100*iproc+11,0,norm,1)
          CALL HZHINRM(100*iproc+21,0,norm,1)
 
          if(iproc.eq.0) then

             CALL HZCHISQ(-110,10)
             CALL HZCHISQ(-111,11)
             CALL HZCHISQ(-120,20)
             CALL HZCHISQ(-121,21)
          endif

          IF (iproc.eq.2) THEN
             CALL HOPERA(110,'+e',210,10,1.,1.)
             CALL HOPERA(120,'+e',220,20,1.,1.)
             CALL HOPERA(111,'+e',211,11,1.,1.)
             CALL HOPERA(121,'+e',221,21,1.,1.)
          ENDIF
 
          IF (iproc.eq.3) THEN

             CALL HOPERA(310,'+e',10,10,1.,1.)
             CALL HOPERA(320,'+e',20,20,1.,1.)
             CALL HOPERA(311,'+e',11,11,1.,1.)
             CALL HOPERA(321,'+e',21,21,1.,1.)
 
             CALL HZCHISQ(-110,10)
             CALL HZCHISQ(-111,11)
             CALL HZCHISQ(-120,20)
             CALL HZCHISQ(-121,21)
          ENDIF
 
        ELSE

C *** End ***
          PRINT*,'C96132:Please run with iflag set to 0, 1, 2 or 3'
          ENDIF
 
        RETURN
        END
