      SUBROUTINE HZC98113(iflag)
*****************************************************************************
* This photoproduction routine has to be run thrice with the following
* code additions:
* 	+1000 for the DIRECT component run.
*	+2000 for the SINGLY-RESOLVED component run.
*       +3000 for the DOUBLY-RESOLVED component run
* for all three phases (iflag=1,2,3)
*
* iflag = 1,2 or 3 depending on whether it is the initialization,
*		processing or termination phase (respectively)
*
* The default jetfinder is PXCONE
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
* This routine produces the following graphs:
*     1-2     Correspond to figure 3 in paper
*     10      Figure 6
*     20-22   Figure 8
*     23-25   Figure 9
*     26-28   Figure 10
*
*     30-33   Figure 4
*     34-37   Figure 5
*     39      Figure 7 (39: MC not correct yet !!!!!!!!)
*     NB The jet shape plots are profile histograms. They deal
c     correctly with weighted events, but the errors are only approximate
c     , and will be wrong unless the luminosity for each process is
c     kept roughly the same.
c     
*
* Cuts : Anti-tag on scattered beam particles (< 33 mrad)
*        Et > 3 GeV
*        |jet rapidity| < 2.0
*        In events with more than two jets, only the two jets with the
*        highest Et values are taken
*
* Recommended value for Ptmin is 2.2 GeV (to be set in MC set up)
*
* Author : Johannes Elmsheuser
*          Russell Taylor
c     Jon Butterworth 20/4/2000. Turned jet shapes into profile plots
c     to deal with weighted events.
c     
*
* Reference: CERN-EP/98-113
*	     Eur. Phys. J. C10 (1999) 547-561
*
* last change: 03 Aug 99
*
* Modified : J. Monk
*            8/2/2006 - allow iproc = 0
*****************************************************************************
*
* HERA Tuning defined commons
*
	IMPLICIT NONE
*
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzjetcmn.inc"
*
      CHARACTER*6 jetf
*
      Integer iNormMod
      Double Precision djshpRho(10,MAXHZJETS)
      Double Precision djshpPsi(10,MAXHZJETS)
      Integer ierr

      real et_in_this_jet(30:38,10),et_jet_tot
*
      DOUBLE PRECISION coneradius
      SAVE CONERADIUS
      DOUBLE PRECISION hzeekin
*	
      INTEGER loop,sum,veto1,veto2,i
      LOGICAL hadron
      DOUBLE PRECISION rap(2),Et(2),Etmean
      DOUBLE PRECISION xplus,xminus,xpdenom,xmdenom
      DOUBLE PRECISION costh,mjj,etabar
      DOUBLE PRECISION ybj,q2,angle
      DOUBLE PRECISION ybjcut(2),q2cut(2)
      DOUBLE PRECISION Etcut(6)
      DOUBLE PRECISION mhxsec(3)
      INTEGER mhntot(3)
      REAL norm, normal(3)
      Integer noent, jetflag(2), numflag
      Integer hzeebeam, kbeam, i1, i2
 
      INTEGER ivec
      REAL temp(9)
 
C     --- Data for cos theta* graphs
C     --- Resolved double-events
      REAL dctr(8),dctr_e(8),dctr_esys(8)
      DATA dctr/6.2,2.7,9.7,10.3,10.5,22.7,30.1,51.9/
      DATA dctr_e/1.7,1.1,2.1,2.2,2.2,3.3,3.8,5.0/
      DATA dctr_esys/0.3,0.2,1.1,2.3,1.7,2.4,2.5,3.6/
C     --- Direct results
      REAL dctd(8),dctd_e(8),dctd_esys(8)
      DATA dctd/4.2,5.0,5.6,7.8,6.8,7.2,6.5,15.8/
      DATA dctd_e/1.4,1.5,1.6,1.9,1.8,1.8,1.7,2.7/
      DATA dctd_esys/0.2,0.4,0.4,0.1,0.5,0.4,0.1,0.7/
 
C     --- Data for Et graph (-2<eta<2)
      REAL xbins(8)
      REAL det(7),det_e(7),det_esys(7)
      DATA xbins/3.0,4.0,5.0,6.5,8.5,11.0,15.0,20.0/
      DATA det/163.0,73.6,27.9,11.5,3.83,1.30,0.12/
      DATA det_e/3.0,2.0,0.9,0.5,0.26,0.14,0.04/
      DATA det_esys/16.0,8.7,3.9,1.4,0.6,0.35,0.13/
 
C     --- Data for rapidity graphs
C     --- Et(jet1)>4, Et(jet2)>3
      REAL drap(5),drap_e(5),drap_esys(5)
      DATA drap/118.7,114.4,93.4,95.1,81.2/
      DATA drap_e/3.7,3.6,2.9,3.5,3.8/
      DATA drap_esys/12.8,10.7,11.7,11.7,9.7/
C     --- 2-resolved
      REAL drap1(5),drap1_e(5),drap1_esys(5)
      DATA drap1/77.2,73.3,57.2,56.0,60.5/
      DATA drap1_e/3.3,3.1,2.5,2.8,3.4/
      DATA drap1_esys/11.8,10.4,10.9,11.5,10.9/
C     --- Direct
      REAL drap2(5),drap2_e(5),drap2_esys(5)
      DATA drap2/15.3,15.1,13.6,10.9,8.2/
      DATA drap2_e/1.2,1.2,1.1,1.3,1.2/
      DATA drap2_esys/2.5,2.4,2.4,2.2,1.9/
C     --- Et(jet1)>5, Et(jet2)>3
      REAL drap3(5),drap3_e(5),drap3_esys(5)
      DATA drap3/58.3,57.8,49.8,52.8,45.2/
      DATA drap3_e/2.6,2.4,2.0,2.4,2.9/
      DATA drap3_esys/7.7,7.6,6.5,6.6,7.2/
C     --- 2-resolved
      REAL drap4(5),drap4_e(5),drap4_esys(5)
      DATA drap4/35.0,33.6,31.3,28.8,30.2/
      DATA drap4_e/2.3,2.0,2.0,2.0,2.3/
      DATA drap4_esys/6.5,6.0,6.7,5.8,6.2/
C     --- Direct
      REAL drap5(5),drap5_e(5),drap5_esys(5)
      DATA drap5/8.8,9.5,9.2,7.9,5.9/
      DATA drap5_e/0.8,0.9,0.9,1.0,1.0/
      DATA drap5_esys/1.9,1.8,1.9,1.7,1.5/
C     --- Et(jet1)>5, Et(jet2)>5
      REAL drap6(5),drap6_e(5),drap6_esys(5)
      DATA drap6/31.0,32.0,27.6,27.5,18.3/
      DATA drap6_e/1.4,1.6,1.4,1.6,1.5/
      DATA drap6_esys/3.5,3.4,3.2,5.8,3.1/
C     --- 2-resolved
      REAL drap7(5),drap7_e(5),drap7_esys(5)
      DATA drap7/17.2,15.6,11.3,11.0,7.5/
      DATA drap7_e/1.1,1.1,0.9,0.9,0.9/
      DATA drap7_esys/4.4,2.9,3.3,3.1,3.3/
C     --- Direct
      REAL drap8(5),drap8_e(5),drap8_esys(5)
      DATA drap8/8.3,9.3,9.2,7.8,5.8/
      DATA drap8_e/0.9,1.0,1.0,1.0,1.2/
      DATA drap8_esys/1.7,1.8,1.7,1.6,1.5/
C     ---- Transverse energy flow
      Real trans(4),trans_e(4),tbin(5)
      Data trans/0.3905,0.3268,0.2437,0.1385/
      Data trans_e/0.0762,0.0312,0.0173,0.0104/
      Data tbin/0.,0.4,0.6,0.8,1./
C     ---- Jetshapes
C
C binning for histogram 34 (figure 5a)
      Integer Netb
      REAL XBMIN, XBMAX
      PARAMETER (NETB =4, XBMIN=2.5, XBMAX=16.4)
      REAL ETC(NETB)
 
C     ET-BINS: 3 < ET < 6 GeV
      Real jet1(9),jet1_e(9),jet1_s(9)
      Data jet1/.2209792,.3636379,.5036087,.6257820,.7289678,
     +     .8166003,.8935716,.9538166,1.000000/
      Data jet1_e/.4251001E-02,.4537700E-02,.4329407E-02,
     +     .3867119E-02,.3280415E-02,.2612657E-02,.1900075E-02,
     +     .1155724E-02,.0000000E+00/
      Data jet1_s/.2572477E-01,.1595122E-01,.1230733E-01,
     +     .7950806E-02,.5677925E-02,.3959488E-02,.2665337E-02,
     +     .1580911E-02,.0000000E+00/
C     ---- 6 < ET < 9 GeV
      Real jet2(9),jet2_e(9),jet2_s(9)
      Data jet2/.3612319,.5164152,.6422570,.7395944,.8189481,
     +     .8758704,.9265970,.9692727,1.000000/
      Data jet2_e/.1108723E-01,.1063599E-01,.9360796E-02,
     +     .7935040E-02,.6486599E-02,.5125404E-02,.3604599E-02,
     +     .2081644E-02,.0000000E+00/
      Data jet2_s/.3096824E-01,.2211963E-01,.1510940E-01,
     +     .1281377E-01,.8641682E-02,.6121607E-02,.3990377E-02,
     +     .2297599E-02,.0000000E+00/
 
C     ---- 9 < ET < 12 GeV
      Real jet3(9),jet3_e(9),jet3_s(9)
      Data jet3/.4528716,.5943354,.7030856,.7907856,.8553009,
     +     .9079834,.9437242,.9722496,1.000000/
      Data jet3_e/.2175449E-01,.2000897E-01,.1729812E-01,
     +     .1406680E-01,.1052341E-01,.8348793E-02,.5439536E-02,
     +     .3591785E-02,.0000000E+00/
      Data jet3_s/.2490672E-01,.2411000E-01,.2292768E-01,
     +     .2052256E-01,.2202065E-01,.1089703E-01,.9541475E-02,
     +     .4307039E-02,.0000000E+00/
 
C     ---- 12 < ET < 20 GeV
      Real jet4(9),jet4_e(9),jet4_s(9)
      Data jet4/.4908210,.6685935,.7625344,.8306012,.9050242,
     +     .9249423,.9613816,.9755640,1.000000/
      Data jet4_e/.2947415E-01,.2703022E-01,.2326954E-01,
     +     .1910693E-01,.1622896E-01,.1340461E-01,.8709117E-02,
     +     .4654132E-02,.0000000E+00/
      Data jet4_s/.8260044E-01,.4443698E-01,.3638402E-01,
     +     .2428289E-01,.2852339E-01,.1572620E-01,.9763868E-02,
     +     .5021494E-02,.0000000E+00/
 
C     ---- ET-BINS, r = 0.5
c     ETbin:  4.047              7.149          10.32          14.33
      Real etbin(5),jet5(4),jet5_e(4),jet5_s(4)
      Data etbin/2.5,5.6,8.7,12.3,16.4/
      Data jet5/.6257820,.7395944,.7907856,.8306012/
      Data jet5_e/.3280415E-02,.6486599E-02,.1052341E-01,
     +     .1622896E-01/
      Data jet5_s/.5677925E-02,.8641681E-02,.2202065E-01,
     +     .2852339E-01/
 
c     ETA-BINS, r = 0.5
c     eta:        0.25         0.75          1.25            1.75
      Real jet6(4),jet6_e(4),jet6_s(4)
      Data jet6/.6474866,.6473133,.6503943,.6857385/
      Data jet6_e/.9075225E-02,.8754758E-02,.1030088E-01,
     +     .1146265E-01/
      Data jet6_s/.2181574E-01,.1929426E-01,.1636410E-01,
     +     .1433685E-01/
 
c     ----- xg < 0.8
      Real jet7(9),jet7_e(9),jet7_s(9)
      Data jet7/.2136927,.3520198,.4895960,.6108904,.7168605,
     +     .8066437,.8875051,.9510415,1.000000/
      Data jet7_e/.1123197E-01,.1092855E-01,.9323972E-02,
     +     .7639683E-02,.6010768E-02,.4554836E-02,.3251283E-02,
     +     .1840144E-02,.0000000E+00/
      Data jet7_s/.5535422E-01,.1700652E-01,.1648848E-01,
     +     .1059836E-01,.8496173E-02,.7207649E-02,.4426425E-02,
     +     .2081397E-02,.0000000E+00/
 
c     ----- xg > 0.8
      Real jet8(9),jet8_e(9),jet8_s(9)
      Data jet8/.4236857,.5726522,.7018544,.8035479,.8672054,
     +     .9139441,.9483125,.9782956,1.000000/
      Data jet8_e/.4899294E-02,.5233969E-02,.5062019E-02,
     +     .4590211E-02,.3917607E-02,.3166375E-02,.2321225E-02,
     +     .1400615E-02,.0000000E+00/
      Data jet8_s/.3644661E-01,.3017373E-01,.1955125E-01,
     +     .1054519E-01,.6850822E-02,.5435510E-02,.3964528E-02,
     +     .2780109E-02,.0000000E+00/
*
*
      INTEGER intiflag, chjet, iproc, iflag
      SAVE chjet
C     --- Default jet finder (as used in paper) -- PXCONE
      DATA chjet/2/
 
C     --- choose direct/resolved
C     --- recommend 1 - DIRECT, 2 - 1-RESOLVED, 3 - 2-RESOLVED
      IF (iflag.gt.1000) THEN
        iproc=INT(iflag*0.001)
        intiflag=MOD(iflag,1000)
      else
         intiflag = iflag
         iproc=0
      ENDIF
      IF ((iproc.gt.3).or.(iproc.lt.0)) THEN
        PRINT*,'C98113 : Badly called routine'
        PRINT*,'           set intiflag = iproc*1000 + iflag'
        print*,'           with iproc = 0    (all)'
        print*,'                        1000 (direct)'
        print*,'                        2000 (single resolved)'
        print*,'                        3000 (double resolved)'
      ENDIF
 
C     --- choose jet finder
      IF ((intiflag.gt.10).and.(MOD(intiflag,10).eq.1)) THEN
        chjet=INT(intiflag/10)
        intiflag=MOD(intiflag,10)
      ENDIF
 
      IF (intiflag.eq.1) THEN
C     *** Initialisation run ***
C     *** Create directories ***
        IF (iproc.le.1) THEN
          CALL HCDIR('//HISTO',' ')
          CALL HMDIR('c98113','S')
          CALL HCDIR('//PAWC',' ')
          CALL HMDIR('c98113','S')
 
          CALL HBOOK1(-1,'Data X-Sec(pb), 2-Resolved',8,0.,0.85,0.)
          CALL HBOOK1(-101,'Data X-Sec(pb), 2-Resolved',8,0.,0.85,0.)
          CALL HPAK(-1,dctr)
          CALL HPAKE(-1,dctr_e)
 
          DO ivec=1,8
            temp(ivec)=sqrt(dctr_e(ivec)**2+dctr_esys(ivec)**2)
          ENDDO
          CALL HPAK(-101,dctr)
          CALL HPAKE(-101,temp)
 
          CALL HBOOK1(-2,'Data X-Sec(pb), Direct',8,0.,0.85,0.)
          CALL HBOOK1(-102,'Data X-Sec(pb), Direct',8,0.,0.85,0.)
          CALL HPAK(-2,dctd)
          CALL HPAKE(-2,dctd_e)
 
          DO ivec=1,8
            temp(ivec)=sqrt(dctd_e(ivec)**2+dctd_esys(ivec)**2)
          ENDDO
          CALL HPAK(-102,dctd)
          CALL HPAKE(-102,temp)
 
          CALL HBOOKB(-10,'Data X-Sec(pb), |eta|<2',7,xbins,0.)
          CALL HBOOKB(-110,'Data X-Sec(pb), |eta|<2',7,xbins,0.)
          CALL HPAK(-10,det)
          CALL HPAKE(-10,det_e)
 
          DO ivec=1,7
            temp(ivec)=sqrt(det_e(ivec)**2+det_esys(ivec)**2)
          ENDDO
          CALL HPAK(-110,det)
          CALL HPAKE(-110,temp)
 
          CALL HBOOK1(-20,'Data X-Sec(pb), Et(jet1)>4, Et(jet2)>3',
     &         5,0.0,2.0,0.)
          CALL HBOOK1(-120,'Data X-Sec(pb), Et(jet1)>4, Et(jet2)>3',
     &         5,0.0,2.0,0.)
          CALL HPAK(-20,drap)
          CALL HPAKE(-20,drap_e)
 
          DO ivec=1,5
            temp(ivec)=sqrt(drap_e(ivec)**2+drap_esys(ivec)**2)
          ENDDO
          CALL HPAK(-120,drap)
          CALL HPAKE(-120,temp)
 
          CALL HBOOK1(-21,'Data X-Sec(pb), 2-resolved',5,0.0,2.0,0.)
          CALL HBOOK1(-121,'Data X-Sec(pb), 2-resolved',5,0.0,2.0,0.)
          CALL HPAK(-21,drap1)
          CALL HPAKE(-21,drap1_e)
 
          DO ivec=1,5
            temp(ivec)=sqrt(drap1_e(ivec)**2+drap1_esys(ivec)**2)
          ENDDO
          CALL HPAK(-121,drap1)
          CALL HPAKE(-121,temp)
 
          CALL HBOOK1(-22,'Data X-Sec(pb), Direct',5,0.0,2.0,0.)
          CALL HBOOK1(-122,'Data X-Sec(pb), Direct',5,0.0,2.0,0.)
          CALL HPAK(-22,drap2)
          CALL HPAKE(-22,drap2_e)
 
          DO ivec=1,5
            temp(ivec)=sqrt(drap2_e(ivec)**2+drap2_esys(ivec)**2)
          ENDDO
          CALL HPAK(-122,drap2)
          CALL HPAKE(-122,temp)
 
          CALL HBOOK1(-23,'Data X-Sec(pb), Et(jet1)>5, Et(jet2)>3',
     &         5,0.0,2.0,0.)
          CALL HBOOK1(-123,'Data X-Sec(pb), Et(jet1)>5, Et(jet2)>3',
     &         5,0.0,2.0,0.)
          CALL HPAK(-23,drap3)
          CALL HPAKE(-23,drap3_e)
 
          DO ivec=1,5
            temp(ivec)=sqrt(drap3_e(ivec)**2+drap3_esys(ivec)**2)
          ENDDO
          CALL HPAK(-123,drap3)
          CALL HPAKE(-123,temp)
 
          CALL HBOOK1(-24,'Data X-Sec(pb), 2-resolved',5,0.0,2.0,0.)
          CALL HBOOK1(-124,'Data X-Sec(pb), 2-resolved',5,0.0,2.0,0.)
          CALL HPAK(-24,drap4)
          CALL HPAKE(-24,drap4_e)
 
          DO ivec=1,5
            temp(ivec)=sqrt(drap4_e(ivec)**2+drap4_esys(ivec)**2)
          ENDDO
          CALL HPAK(-124,drap4)
          CALL HPAKE(-124,temp)
 
          CALL HBOOK1(-25,'Data X-Sec(pb), Direct',5,0.0,2.0,0.)
          CALL HBOOK1(-125,'Data X-Sec(pb), Direct',5,0.0,2.0,0.)
          CALL HPAK(-25,drap5)
          CALL HPAKE(-25,drap5_e)
 
          DO ivec=1,5
            temp(ivec)=sqrt(drap5_e(ivec)**2+drap5_esys(ivec)**2)
          ENDDO
          CALL HPAK(-125,drap5)
          CALL HPAKE(-125,temp)
 
          CALL HBOOK1(-26,'Data X-Sec(pb), Et(jets)>5',5,0.0,2.0,0.)
          CALL HBOOK1(-126,'Data X-Sec(pb), Et(jets)>5',5,0.0,2.0,0.)
          CALL HPAK(-26,drap6)
          CALL HPAKE(-26,drap6_e)
 
          DO ivec=1,5
            temp(ivec)=sqrt(drap6_e(ivec)**2+drap6_esys(ivec)**2)
          ENDDO
          CALL HPAK(-126,drap6)
          CALL HPAKE(-126,temp)
 
          CALL HBOOK1(-27,'Data X-Sec(pb), 2-resolved',5,0.0,2.0,0.)
          CALL HBOOK1(-127,'Data X-Sec(pb), 2-resolved',5,0.0,2.0,0.)
          CALL HPAK(-27,drap7)
          CALL HPAKE(-27,drap7_e)
 
          DO ivec=1,5
            temp(ivec)=sqrt(drap7_e(ivec)**2+drap7_esys(ivec)**2)
          ENDDO
          CALL HPAK(-127,drap7)
          CALL HPAKE(-127,temp)
 
          CALL HBOOK1(-28,'Data X-Sec(pb), Direct',5,0.0,2.0,0.)
          CALL HBOOK1(-128,'Data X-Sec(pb), Direct',5,0.0,2.0,0.)
          CALL HPAK(-28,drap8)
          CALL HPAKE(-28,drap8_e)
 
          DO ivec=1,5
            temp(ivec)=sqrt(drap8_e(ivec)**2+drap8_esys(ivec)**2)
          ENDDO
          CALL HPAK(-128,drap8)
          CALL HPAKE(-128,temp)
 
          CALL HBOOK1(-30,'Data Jetshape, 3 < Et < 6 GeV',
     +         9,0.15,1.05,0.)
          CALL HBOOK1(-130,'Data Jetshape, 3 < Et < 6 GeV',
     +         9,0.15,1.05,0.)
          CALL HPAK(-30,jet1)
          CALL HPAKE(-30,jet1_e)
          DO ivec=1,9
            temp(ivec)=sqrt(jet1_e(ivec)**2+jet1_s(ivec)**2)
          ENDDO
          CALL HPAK(-130,jet1)
          CALL HPAKE(-130,temp)
 
          CALL HBOOK1(-31,'Data Jetshape, 6 < Et < 9 GeV',
     +         9,0.15,1.05,0.)
          CALL HBOOK1(-131,'Data Jetshape, 6 < Et < 9 GeV',
     +         9,0.15,1.05,0.)
          CALL HPAK(-31,jet2)
          CALL HPAKE(-31,jet2_e)
          DO ivec=1,9
            temp(ivec)=sqrt(jet2_e(ivec)**2+jet2_s(ivec)**2)
          ENDDO
          CALL HPAK(-131,jet2)
          CALL HPAKE(-131,temp)
 
          CALL HBOOK1(-32,'Data Jetshape, 9 < Et < 12 GeV',
     +         9,0.15,1.05,0.)
          CALL HBOOK1(-132,'Data Jetshape, 9 < Et < 12 GeV',
     +         9,0.15,1.05,0.)
          CALL HPAK(-32,jet3)
          CALL HPAKE(-32,jet3_e)
          DO ivec=1,9
            temp(ivec)=sqrt(jet3_e(ivec)**2+jet3_s(ivec)**2)
          ENDDO
          CALL HPAK(-132,jet3)
          CALL HPAKE(-132,temp)
 
          CALL HBOOK1(-33,'Data Jetshape, 12 < Et <20 GeV',
     +         9,0.15,1.05,0.)
          CALL HBOOK1(-133,'Data Jetshape, 12 < Et < 20 GeV',
     +         9,0.15,1.05,0.)
          CALL HPAK(-33,jet4)
          CALL HPAKE(-33,jet4_e)
          DO ivec=1,9
            temp(ivec)=sqrt(jet4_e(ivec)**2+jet4_s(ivec)**2)
          ENDDO
          CALL HPAK(-133,jet4)
          CALL HPAKE(-133,temp)
 
          CALL HBOOKB(-34,'Data Jetshape, r = 0.5, Et',4,etbin,0.)
          CALL HBOOKB(-134,'Data Jetshape, r = 0.5, Et',4,etbin,0.)
          CALL HPAK(-34,jet5)
          CALL HPAKE(-34,jet5_e)
          DO ivec=1,4
            temp(ivec)=sqrt(jet5_e(ivec)**2+jet5_s(ivec)**2)
          ENDDO
          CALL HPAK(-134,jet5)
          CALL HPAKE(-134,temp)
 
          CALL HBOOK1(-35,'Data Jetshape, r = 0.5, eta',4,0.,2.,0.)
          CALL HBOOK1(-135,'Data Jetshape, r = 0.5, eta',4,0.,2.,0.)
          CALL HPAK(-35,jet6)
          CALL HPAKE(-35,jet6_e)
          DO ivec=1,4
            temp(ivec)=sqrt(jet6_e(ivec)**2+jet6_s(ivec)**2)
          ENDDO
          CALL HPAK(-135,jet6)
          CALL HPAKE(-135,temp)
 
          CALL HBOOK1(-36,'Data Jetshape, xg < 0.8',9,0.15,1.05,0.)
          CALL HBOOK1(-136,'Data Jetshape, xg < 0.8',9,0.15,1.05,0.)
          CALL HPAK(-36,jet7)
          CALL HPAKE(-36,jet7_e)
          DO ivec=1,9
            temp(ivec)=sqrt(jet7_e(ivec)**2+jet7_s(ivec)**2)
          ENDDO
          CALL HPAK(-136,jet7)
          CALL HPAKE(-136,temp)
 
          CALL HBOOK1(-37,'Data Jetshape, xg > 0.8',9,0.15,1.05,0.)
          CALL HBOOK1(-137,'Data Jetshape, xg > 0.8',9,0.15,1.05,0.)
          CALL HPAK(-37,jet8)
          CALL HPAKE(-37,jet8_e)
          DO ivec=1,9
            temp(ivec)=sqrt(jet8_e(ivec)**2+jet8_s(ivec)**2)
          ENDDO
          CALL HPAK(-137,jet8)
          CALL HPAKE(-137,temp)
 
          CALL HBOOKB(-39,'Data Transverse energy flow',4,tbin,0.)
          CALL HBOOKB(-139,'Data Transverse energy flow',4,tbin,0.)
          CALL HPAK(-39,trans)
          CALL HPAKE(-39,trans_e)
          CALL HPAK(-139,trans)
          CALL HPAKE(-139,trans_e)
 
 
          CALL HBOOK1(1,'MC X-Sec(pb), 2-Resolved',8,0.,0.85,0.)
          CALL HBOOK1(2,'MC X-Sec(pb), Direct',8,0.,0.85,0.)
          CALL HBOOKB(10,'MC X-Sec(pb), |eta|<2',7,xbins,0.)
          CALL HBOOK1(20,'MC X-Sec(pb), Et(jet1)>4, Et(jet2)>3',
     &         5,0.0,2.0,0.)
          CALL HBOOK1(21,'MC X-Sec(pb), 2-resolved',5,0.0,2.0,0.)
          CALL HBOOK1(22,'MC X-Sec(pb), Direct',5,0.0,2.0,0.)
          CALL HBOOK1(23,'MC X-Sec(pb), Et(jet1)>5, Et(jet2)>3',
     &         5,0.0,2.0,0.)
          CALL HBOOK1(24,'MC X-Sec(pb), 2-resolved',5,0.0,2.0,0.)
          CALL HBOOK1(25,'MC X-Sec(pb), Direct',5,0.0,2.0,0.)
          CALL HBOOK1(26,'MC X-Sec(pb), Et(jet)>5',5,0.0,2.0,0.)
          CALL HBOOK1(27,'MC X-Sec(pb), 2-resolved',5,0.0,2.0,0.)
          CALL HBOOK1(28,'MC X-Sec(pb), Direct',5,0.0,2.0,0.)
 
          CALL HBPROF(1030,'MC Jetshape, 3 < Et < 6 GeV',9,0.15,1.05,0.
     &         ,20.,' ')
          CALL HBPROF(1031,'MC Jetshape, 6 < Et < 9 GeV',9,0.15,1.05,0.
     &         ,20.,' ')
          CALL HBPROF(1032,'MC Jetshape, 9 < Et <12 GeV',9,0.15,1.05,0.
     &         ,20.,' ')
          CALL HBPROF(1033,'MC Jetshape, 12< Et <20 GeV',9,0.15,1.05,0.
     &         ,20.,' ')
          
          CALL HBProf(1034,'MC Jetshape, psi(0.5), Et',
     +         NETB,XBMIN,XBMAX,0.,1.,' ')
          CALL HBProf(1035,'MC Jetshape, psi(0.5), eta',
     +         4,0.,2.,0.,1.,' ')
 
          CALL HBPROF(1036,'MC Jetshape, xg < 0.8',9,0.15,1.05,0.,20
     &         .,' ')
          CALL HBPROF(1037,'MC Jetshape, xg > 0.8',9,0.15,1.05,0.,20
     &         .,' ')
 
          CALL HBOOKB(39,'MC Transverse energy flow',4,tbin,0.)
 
        ENDIF
 
        CALL HCDIR('//HISTO/c98113',' ')
        CALL HCDIR('//PAWC/c98113',' ')
 
        if(iproc.ge.1)then

           CALL HBOOK1(1+iproc*100,'MC component',8,0.,0.85,0.)
           CALL HBOOK1(2+iproc*100,'MC component',8,0.,0.85,0.)
           CALL HBOOKB(10+iproc*100,'MC component',7,xbins,0.)
           CALL HBOOK1(20+iproc*100,'MC component',5,0.0,2.0,0.)
           CALL HBOOK1(21+iproc*100,'MC component',5,0.0,2.0,0.)
           CALL HBOOK1(22+iproc*100,'MC component',5,0.0,2.0,0.)
           CALL HBOOK1(23+iproc*100,'MC component',5,0.0,2.0,0.)
           CALL HBOOK1(24+iproc*100,'MC component',5,0.0,2.0,0.)
           CALL HBOOK1(25+iproc*100,'MC component',5,0.0,2.0,0.)
           CALL HBOOK1(26+iproc*100,'MC component',5,0.0,2.0,0.)
           CALL HBOOK1(27+iproc*100,'MC component',5,0.0,2.0,0.)
           CALL HBOOK1(28+iproc*100,'MC component',5,0.0,2.0,0.)
           
           CALL HBPROF(
     &          1030+iproc*100,'MC Jetshape',9,0.15,1.05,0.,20.,' ')
           CALL HBPROF(
     &          1031+iproc*100,'MC Jetshape',9,0.15,1.05,0.,20.,' ')
           CALL HBPROF(
     &          1032+iproc*100,'MC Jetshape',9,0.15,1.05,0.,20.,' ')
           CALL HBPROF(
     &          1033+iproc*100,'MC Jetshape',9,0.15,1.05,0.,20.,' ')
           
           CALL HBProf(1034+iproc*100,'MC Jetshape',
     +          NETB,XBMIN,XBMAX,0.,1.,' ')
           CALL HBProf(1035+iproc*100,'MC Jetshape',4,0.,2.,0.,1.,' ')
           
           CALL HBPROF(
     &          1036+iproc*100,'MC Jetshape',9,0.15,1.05,0.,20.,' ')
           CALL HBPROF(
     &          1037+iproc*100,'MC Jetshape',9,0.15,1.05,0.,20.,' ')
           
           CALL HBOOKB(39+iproc*100,'MC component',4,tbin,0.)
        endif

        CALL HBARX(0)
        CALL HBAR2(0)
        CALL HIDOPT(0,'stat')
 
C     --- standard cone radius is 1.0
        CALL HZJETRAD(2,CONERADIUS)
        IF (CONERADIUS.lt.0.0) THEN
          CONERADIUS=1.0
        ENDIF
        CALL HZJTNAME(chjet,jetf)
        WRITE(6,*)'***********************************************'
        WRITE(6,*)'* c98113 called, histograms will be output. *'
        WRITE(6,*)'*     Jet finder selected : ',jetf,'            *'
        WRITE(6,1001)'*        with cone radius : ',CONERADIUS
     +       ,'          *'
        WRITE(6,*)'***********************************************'
 
 1001   FORMAT(A29,F5.2,A14)
 
      ELSE IF (intiflag.eq.2) THEN
        angle=1.D0
        iNormMod=0              ! jetshapes normalized with jet pt
        xplus=0.
        xminus=0.
        numflag=0               ! number of jets for filling
        jetflag(1)=0            ! pos. in hzjetcmn of jet 1 (with highest Et)
        jetflag(2)=0            ! pos. in hzjetcmn of jet 2
        Et(1)=0D0               ! Et of jet 1
        Et(2)=0D0               ! Et of jet 2
 
C     *** Filling runs ***
C     *** Change directory to our graphs ***
 
        CALL HCDIR('//PAWC/c98113',' ')
 
        angle=hzeekin(2)
        IF ((angle.ne.-1D0).and. ! anti-tag cut
     +       ((angle.lt.0.033D0).or.(angle.gt.3.1085927D0))) Then
          CALL HzJtFind(chJet,coneRadius,numJets,jets)
          Call HzJetShp(iNormMod,coneRadius,djshpRho,djshpPsi,iErr)
 
          If (numJets.ge.2) Then
            If (numJets.eq.2) Then ! easy with only two jets
              jetflag(1)=1
              jetflag(2)=2
            Else                ! find the two jets with highest Et
              Do loop=1,numJets
                If (Jets(loop,3).gt.Et(1)) Then
                  Et(2)=Et(1)
                  Et(1)=Jets(loop,3)
                  jetflag(2)=jetflag(1)
                  jetflag(1)=loop
                EndIf
                If ((Jets(loop,3).gt.Et(2)).and.
     +               (Jets(loop,3).ne.Et(1))) Then
                  Et(2)=Jets(loop,3)
                  jetflag(2)=loop
                EndIf
              EndDo
            EndIf
            Do loop=1,2         ! fill jet1/2 data
              IF ((Jets(jetflag(loop),3).ge.3.0).and.
     +             (ABS(Jets(jetflag(loop),1)).le.2.0)) THEN
                numflag=numflag+1
                Et(numflag)=Jets(jetflag(loop),3)
                rap(numflag)=Jets(jetflag(loop),1)
                xplus=xplus+(Jets(jetflag(loop),4)+
     +               Jets(jetflag(loop),7))
                xminus=xminus+(Jets(jetflag(loop),4)-
     +               Jets(jetflag(loop),7))
              EndIf
            EndDo
          EndIf                 ! numJets.ge.2
 
C     *** Fill our histograms for the two jets ***
 
          If (numflag.eq.2) Then
 
C     --- Stuff to separate out resolved & direct events
C     --- Events where one or both scattered electrons are detected are
C     --- vetoed (anti-tagged)
            IF (GEN(1:3).eq.'HRW') THEN
              veto1=1
              veto2=2
            ELSEIF (GEN(1:3).eq.'PYT') THEN
              kbeam= hzeebeam(i1,i2) ! find e- and e+ position
              If ((i1.eq.1).and.(i2.eq.2)) Then
                veto1=3
                veto2=4
              ElseIf ((i1.eq.2).and.(i2.eq.1)) Then
                veto1=4
                veto2=3
              EndIf
            ELSEIF (GEN(1:3).eq.'PHO') THEN
              veto1=3
              veto2=5
            ELSE
              write(6,*)'Unknown generator'
            ENDIF
 
            xpdenom=0.
            xmdenom=0.
            DO sum=1,NHEP
              IF ((ISTHEP(sum).eq.1).and.
     +             (JMOHEP(1,sum).ne.veto1).and.
     +             (JMOHEP(1,sum).ne.veto2)) THEN
                xpdenom=xpdenom+(PHEP(4,sum)+PHEP(3,sum))
                xmdenom=xmdenom+(PHEP(4,sum)-PHEP(3,sum))
              ENDIF
            ENDDO
            xplus=xplus/xpdenom
            xminus=xminus/xmdenom
 
            costh=ABS(tanh((rap(1)-rap(2))/2))
            mjj=(Et(1)+Et(2))/sqrt(1-(costh**2))
            etabar=ABS((rap(1)+rap(2))/2)
 
            IF ((mjj.gt.12.).and.(etabar.lt.1.)) THEN
              IF ((xplus.lt.0.8).and.(xminus.lt.0.8)) THEN
                CALL HFILL(iproc*100+1,REAL(costh),0.,wtx)
              ENDIF
              IF ((xplus.gt.0.8).and.(xminus.gt.0.8)) THEN
                CALL HFILL(iproc*100+2,REAL(costh),0.,wtx)
              ENDIF
            ENDIF
 
C     --- End of that lot
 
            CALL HFILL(iproc*100+10,REAL(Et(1)),0.,wtx)
            CALL HFILL(iproc*100+10,REAL(Et(2)),0.,wtx)
 
C     --- Jetshapes -----
 
            Etmean=0.5D0*(Et(1)+Et(2))
            Call Hfill(1000+iproc*100+34,Real(EtMean),
     +           Real(djshpPsi(5,jetflag(1))),1.)
 
c            DO I=1,NETB
c             IF ((Etmean.ge.xbmin+float(i-1)*
c     +          (xbmin-xbmax)/float(netb)).and.
c     +          (Etmean.lt.xbmin+float(i)*
c     +          (xbmin-xbmax)/float(netb)))
c     +          Etc(I)=Etc(I)+Etmean
c            ENDDO
 
            Call Hfill(1000+iproc*100+35,Real(rap(1)),
     +           Real(djshpPsi(5,jetflag(1))),wtx)
            Call Hfill(1000+iproc*100+35,Real(rap(2)),
     +           Real(djshpPsi(5,jetflag(2))),wtx)
 
            If (((Et(1).ge.3).and.(Et(1).le.6)).or.
     +           ((Et(2).ge.3).and.(Et(2).le.6))) Then
              Do i=1,10

                 Call HFill(1000+iproc*100+30,Real(i)*0.1,
     +                Real(djshpPsi(i,jetflag(1))),wtx)
                 Call HFill(1000+iproc*100+30,Real(i)*0.1,
     +                Real(djshpPsi(i,jetflag(2))),wtx)
              EndDo
            EndIf
 
            If (((Et(1).ge.6).and.(Et(1).le.9)).or.
     +           ((Et(2).ge.6).and.(Et(2).le.9))) Then
              Do i=1,10
                Call HFill(1000+iproc*100+31,Real(i)*0.1,
     +               Real(djshpPsi(i,jetflag(1))),wtx)
                Call HFill(1000+iproc*100+31,Real(i)*0.1,
     +               Real(djshpPsi(i,jetflag(2))),wtx)
              EndDo
            EndIf
 
            If (((Et(1).ge.9).and.(Et(1).le.12)).or.
     +           ((Et(2).ge.9).and.(Et(2).le.12))) Then
              Do i=1,10
                Call HFill(iproc*100+32,Real(i)*0.1,
     +               Real(djshpPsi(i,jetflag(1))),wtx)
                Call HFill(iproc*100+32,Real(i)*0.1,
     +               Real(djshpPsi(i,jetflag(2))),wtx)
              EndDo
            EndIf
 
            If (((Et(1).ge.12).and.(Et(1).le.20)).or.
     +           ((Et(2).ge.12).and.(Et(2).le.20))) Then
              Do i=1,10
                Call HFill(iproc*100+33,Real(i)*0.1,
     +               Real(djshpPsi(i,jetflag(1))),wtx)
                Call HFill(iproc*100+33,Real(i)*0.1,
     +               Real(djshpPsi(i,jetflag(2))),wtx)
              EndDo
            EndIf
 
            If ((xminus.le.0.8).and.(xplus.le.0.8)) Then
              Do i=1,10
                Call HFill(iproc*100+36,Real(i)*0.1,
     +               Real(djshpPsi(i,jetflag(1))),wtx)
                Call HFill(iproc*100+36,Real(i)*0.1,
     +               Real(djshpPsi(i,jetflag(2))),wtx)
              EndDo
            EndIf
 
            If ((xminus.ge.0.8).and.(xplus.ge.0.8)) Then
              Do i=1,10
                Call HFill(iproc*100+37,Real(i)*0.1,
     +               Real(djshpPsi(i,jetflag(1))),wtx)
                Call HFill(iproc*100+37,Real(i)*0.1,
     +               Real(djshpPsi(i,jetflag(2))),wtx)
              EndDo
            EndIf
 
C --- End of Jetshapes -----
C --- Transverse energy flow ---
            Call HFill(iproc*100+39,Real(xplus+xminus),0.,
     +           Real(Et(1)))
            Call HFill(iproc*100+39,Real(xplus+xminus),0.,
     +           Real(Et(2)))
 
C --- End of Transverse energy flow ---
 
            IF (Et(1).gt.4.0) THEN
              CALL HFILL(iproc*100+20,REAL(ABS(rap(1))),0.,wtx)
              CALL HFILL(iproc*100+20,REAL(ABS(rap(2))),0.,wtx)
              IF ((xplus.lt.0.8).and.(xminus.lt.0.8)) THEN
                CALL HFILL(iproc*100+21,REAL(ABS(rap(1))),0.,wtx)
                CALL HFILL(iproc*100+21,REAL(ABS(rap(2))),0.,wtx)
              ENDIF
              IF ((xplus.gt.0.8).and.(xminus.gt.0.8)) THEN
                CALL HFILL(iproc*100+22,REAL(ABS(rap(1))),0.,wtx)
                CALL HFILL(iproc*100+22,REAL(ABS(rap(2))),0.,wtx)
              ENDIF
              IF (Et(1).gt.5.) THEN
                CALL HFILL(iproc*100+23,REAL(ABS(rap(1))),0.,wtx)
                CALL HFILL(iproc*100+23,REAL(ABS(rap(2))),0.,wtx)
                IF ((xplus.lt.0.8).and.(xminus.lt.0.8)) THEN
                  CALL HFILL(iproc*100+24,REAL(ABS(rap(1))),0.,wtx)
                  CALL HFILL(iproc*100+24,REAL(ABS(rap(2))),0.,wtx)
                ENDIF
                IF ((xplus.gt.0.8).and.(xminus.gt.0.8)) THEN
                  CALL HFILL(iproc*100+25,REAL(ABS(rap(1))),0.,wtx)
                  CALL HFILL(iproc*100+25,REAL(ABS(rap(2))),0.,wtx)
                ENDIF
                IF (Et(2).gt.5) THEN
                  CALL HFILL(iproc*100+26,REAL(ABS(rap(1))),0.,wtx)
                  CALL HFILL(iproc*100+26,REAL(ABS(rap(2))),0.,wtx)
                  IF ((xplus.lt.0.8).and.(xminus.lt.0.8)) THEN
                    CALL HFILL(iproc*100+27,REAL(ABS(rap(1))),0.,wtx)
                    CALL HFILL(iproc*100+27,REAL(ABS(rap(2))),0.,wtx)
                  ENDIF
                  IF ((xplus.gt.0.8).and.(xminus.gt.0.8)) THEN
                    CALL HFILL(iproc*100+28,REAL(ABS(rap(1))),0.,wtx)
                    CALL HFILL(iproc*100+28,REAL(ABS(rap(2))),0.,wtx)
                  ENDIF
                ENDIF
              ENDIF
            ENDIF
          ENDIF                 ! numflag.eq.2
        ENDIF                   ! angle.lt.0.033
 
      ELSE IF (intiflag.eq.3) THEN
C     *** Termination run ***
C     *** Change directory ***
        CALL HCDIR('//PAWC/c98113',' ')
C     *** Finish off histograms ***
        IF (Xsec.eq.0) THEN
          PRINT*,'C98113: termination called with zero cross section'
          PRINT*,'          cross section graph meaningless'
          PRINT*,'	      in process :',iproc*1000
          Xsec=1
        ENDIF
        IF (Ntot.eq.0) THEN
          PRINT*,'C98113: termination called with no total events'
          PRINT*,'          cross section graph meaningless'
          PRINT*,'          in process :',iproc*1000
          Ntot=1
        ENDIF
 
C     --- store Xsec and Ntot for both direct and resolved events.
 
        if(iproc.gt.0) then
           mhXsec(iproc)=1000*Xsec
           mhntot(iproc)=Ntot
           norm=REAL(mhntot(iproc)/mhXsec(iproc))
        else
           norm = REAL(Ntot * 0.001 / Xsec)
        endif
C     --- normalise direct/resolved histograms
c        norm=0.
c        norm=REAL(mhntot(iproc)/mhXsec(iproc))
        normal(iproc)=norm

        CALL HZHINRM(100*iproc+1,0,norm,1)
        CALL HZHINRM(100*iproc+2,0,norm,1)
        CALL HZHINRM(100*iproc+10,0,norm,1)
        CALL HZHINRM(100*iproc+20,0,norm,1)
        CALL HZHINRM(100*iproc+21,0,norm,1)
        CALL HZHINRM(100*iproc+22,0,norm,1)
        CALL HZHINRM(100*iproc+23,0,norm,1)
        CALL HZHINRM(100*iproc+24,0,norm,1)
        CALL HZHINRM(100*iproc+25,0,norm,1)
        CALL HZHINRM(100*iproc+26,0,norm,1)
        CALL HZHINRM(100*iproc+27,0,norm,1)
        CALL HZHINRM(100*iproc+28,0,norm,1)
 
        if(iproc.eq.0)then
           do 1030 loop=1030, 1037 
              Call Hnoent(1030,noent)
              if(noent.ne.0) then
c                 write(*,*) loop
                 Call HzHinrm(loop,0,Real(noent),1)
              endif
 1030      continue

c          Call Hnoent(1031,noent)
c          Call HzHinrm(1031,0,Real(noent),1)
c          Call Hnoent(1032,noent)
c          Call HzHinrm(1032,0,Real(noent),1)
c          Call Hnoent(1033,noent)
c          Call HzHinrm(1033,0,Real(noent),1)
c          Call Hnoent(1036,noent)
c          Call HzHinrm(1036,0,Real(noent),1)
c          Call Hnoent(1037,noent)
c          Call HzHinrm(1037,0,Real(noent),1)
 
          CALL HZCHISQ(-101,1)
          CALL HZCHISQ(-102,2)
          CALL HZCHISQ(-110,10)
          CALL HZCHISQ(-120,20)
          CALL HZCHISQ(-121,21)
          CALL HZCHISQ(-122,22)
          CALL HZCHISQ(-123,23)
          CALL HZCHISQ(-124,24)
          CALL HZCHISQ(-125,25)
          CALL HZCHISQ(-126,26)
          CALL HZCHISQ(-127,27)
          CALL HZCHISQ(-128,28) 
        endif
           
*        Call Hnoent(100*iproc+30,noent)
*        Call HzHinrm(100*iproc+30,0,Real(noent),1)
*        Call Hnoent(100*iproc+31,noent)
*        Call HzHinrm(100*iproc+31,0,Real(noent),1)
*        Call Hnoent(100*iproc+32,noent)
*        Call HzHinrm(100*iproc+32,0,Real(noent),1)
*        Call Hnoent(100*iproc+33,noent)
*        Call HzHinrm(100*iproc+33,0,Real(noent),1)
*        Call Hnoent(100*iproc+36,noent)
*        Call HzHinrm(100*iproc+36,0,Real(noent),1)
*        Call Hnoent(100*iproc+37,noent)
*        Call HzHinrm(100*iproc+37,0,Real(noent),1)
 
        IF (iproc.eq.2) THEN
          CALL HOPERA(101,'+e',201,1,1.,1.)
          CALL HOPERA(102,'+e',202,2,1.,1.)
          CALL HOPERA(110,'+e',210,10,1.,1.)
          CALL HOPERA(120,'+e',220,20,1.,1.)
          CALL HOPERA(121,'+e',221,21,1.,1.)
          CALL HOPERA(122,'+e',222,22,1.,1.)
          CALL HOPERA(123,'+e',223,23,1.,1.)
          CALL HOPERA(124,'+e',224,24,1.,1.)
          CALL HOPERA(125,'+e',225,25,1.,1.)
          CALL HOPERA(126,'+e',226,26,1.,1.)
          CALL HOPERA(127,'+e',227,27,1.,1.)
          CALL HOPERA(128,'+e',228,28,1.,1.)
 
          CALL HOPERA(1130,'+e',1230,1030,1.,normal(1)/normal(2))
          CALL HOPERA(1131,'+e',1231,1031,1.,normal(1)/normal(2))
          CALL HOPERA(1132,'+e',1232,1032,1.,normal(1)/normal(2))
          CALL HOPERA(1133,'+e',1233,1033,1.,normal(1)/normal(2))
          CALL HOPERA(1134,'+e',1234,1034,1.,normal(1)/normal(2))
          CALL HOPERA(1135,'+e',1235,1035,1.,normal(1)/normal(2))
          CALL HOPERA(1136,'+e',1236,1036,1.,normal(1)/normal(2))
          CALL HOPERA(1137,'+e',1237,1037,1.,normal(1)/normal(2))
          CALL HOPERA(139,'+e',239,39,1.,1.)
 
        ENDIF
 
        IF (iproc.eq.3) THEN
          CALL HOPERA(301,'+e',1,1,1.,1.)
          CALL HOPERA(302,'+e',2,2,1.,1.)
          CALL HOPERA(310,'+e',10,10,1.,1.)
          CALL HOPERA(320,'+e',20,20,1.,1.)
          CALL HOPERA(321,'+e',21,21,1.,1.)
          CALL HOPERA(322,'+e',22,22,1.,1.)
          CALL HOPERA(323,'+e',23,23,1.,1.)
          CALL HOPERA(324,'+e',24,24,1.,1.)
          CALL HOPERA(325,'+e',25,25,1.,1.)
          CALL HOPERA(326,'+e',26,26,1.,1.)
          CALL HOPERA(327,'+e',27,27,1.,1.)
          CALL HOPERA(328,'+e',28,28,1.,1.)
 
          CALL HOPERA(1330,'+e',1030,1030,1.,normal(1)/normal(3))
          CALL HOPERA(1331,'+e',1031,1031,1.,normal(1)/normal(3))
          CALL HOPERA(1332,'+e',1032,1032,1.,normal(1)/normal(3))
          CALL HOPERA(1333,'+e',1033,1033,1.,normal(1)/normal(3))
          CALL HOPERA(1334,'+e',1034,1034,1.,normal(1)/normal(3))
          CALL HOPERA(1335,'+e',1035,1035,1.,normal(1)/normal(3))
          CALL HOPERA(1336,'+e',1036,1036,1.,normal(1)/normal(3))
          CALL HOPERA(1337,'+e',1037,1037,1.,normal(1)/normal(3))
          CALL HOPERA(339,'+e',39,39,1.,1.)
 
          Call Hnoent(1030,noent)
          Call HzHinrm(1030,0,Real(noent),1)
          Call Hnoent(1031,noent)
          Call HzHinrm(1031,0,Real(noent),1)
          Call Hnoent(1032,noent)
          Call HzHinrm(1032,0,Real(noent),1)
          Call Hnoent(1033,noent)
          Call HzHinrm(1033,0,Real(noent),1)
          Call Hnoent(1036,noent)
          Call HzHinrm(1036,0,Real(noent),1)
          Call Hnoent(1037,noent)
          Call HzHinrm(1037,0,Real(noent),1)
 
          CALL HZCHISQ(-101,1)
          CALL HZCHISQ(-102,2)
          CALL HZCHISQ(-110,10)
          CALL HZCHISQ(-120,20)
          CALL HZCHISQ(-121,21)
          CALL HZCHISQ(-122,22)
          CALL HZCHISQ(-123,23)
          CALL HZCHISQ(-124,24)
          CALL HZCHISQ(-125,25)
          CALL HZCHISQ(-126,26)
          CALL HZCHISQ(-127,27)
          CALL HZCHISQ(-128,28)
 
        ENDIF
 
      ELSE
C     *** End ***
        PRINT*,'C98113:Please run with iflag set to 1,2 or 3'
      ENDIF
 
      RETURN
      END
