      SUBROUTINE HZH0301013(iflag)
********************************************************************
*     Dijet production in untagged photon-photon collisions 
*     as measured by OPAL at average sqrt(s)=194 GeV
*
*     OPAL PR372, CERN-EP-2002-093, 
*     Eur.Phys.J.C31:307-325,2003
*     hep-ex/0301013
*
*    Author: Thorsten Wengler (Thorsten.Wengler@cern.ch)
*
* iflag = 1,2 or 3 depending on whether it is the initialization,
*		processing or termination phase (respectively)
*
* For this routine use
* 1000+iflag   for all combined (as the mix provided by pythia) OR
*              if running separately:
* 2000+iflag   for direct events
* 3000+iflag   for single resolved
* 4000+iflag   for double resolved
*
* Initialization: only has to be called once 
*                 
* Processing: 2002 for direct, 3002 for sngl res.,4002 for dble res.
*             OR if mix provided by generator, call 1002
*
* Termination: call once with 2003, 3003, and 4003 respectively and
*              be careful to set Xsec to the appropriate value in 
*              driver routine OR
*              if mix provided by generator, call 1003
*
* For analysis cuts and observable definition see ref. above 
*              
* Histograms -------------------------------------------------------
*                      |  OPAL - PR443 |
*                      +---------------+
*                    
* MC   histograms:   hist-Nr.
* DATA histograms:   (-1) * hist-Nr.            (total uncert.)
* DATA histograms:   (-1) * hist-Nr. + 100000   (stat. uncert.)
*
* ------------------------------------------------------------------
*     hist-Nr.     corresponds to Figure:
* ------------------------------------------------------------------
*     16410         9 left
*     10410         9 right
*     
*     10211        10 full xg+ - xg- space
*     15211        10 xg+ or xg- < 0.75
*     16211        10 xg+- < 0.75
*     
*     10313        11 upper left
*     10314        11 upper right
*     10323        11 lower left
*     10333        11 lower right
*
*     15313        13 upper left
*     15314        13 upper right
*     15323        13 lower left
*
*     16313        14 upper left
*     16314        14 upper right
*     16323        14 lower left
*
*     15413        15 upper
*     15411        15 lower left
*     15412        15 lower right
*
*     16413        16 upper
*     16411        16 lower left
*     16412        16 lower right
*
* 110333 di-jet x-sect vs. x_g for 11 < Et-mean < 25 GeV
*
* last change: 24 Jan 2003
*
* Modified :
* J. Monk (allow to be called with iproc=0) 14/3/2006
*
********************************************************************
	IMPLICIT NONE
*
      logical direnh,snglenh,dbleenh
      integer firstpa

#include "hepevtp.inc"
#include "heracmn.inc"

*
* --------------------------------------------------------------------
*
c ******************************************************************
c definitions for jet analysis
c ******************************************************************
      real dirres
      parameter (dirres=0.75)
      real ptmin,etamax
      parameter (ptmin=2.0, etamax=2.0)

c ##################################################################
c     DIJETS in Untagged events 
c ##################################################################
c general jet cuts
      real metmin,metmi1,metmi2,metmi3
      real metmax,metma1,metma2,metma3
      real jetasy
      real djmami,etabma
      parameter(
     +     metmin=4.0,          ! absolut mean Et min 
     +     metmax=1000.0,       ! absolut mean Et min 
     +     metmi1=5.0,          ! mean Et min 1
     +     metma1=7.0,          ! mean Et min 1
     +     metmi2=7.0,          ! mean Et min 2
     +     metma2=11.0,         ! mean Et min 2
     +     metmi3=11.0,         ! mean Et min 3
     +     metma3=25.0,         ! mean Et min 3
     +     jetasy=0.25,         ! max.asym: abs(Et1-Et2)/(Et1+Et2)
     +     djmami=15.0,         ! min djet mass for cos Th* plot
     +     etabma=1.0           ! eta-mean max for cos Th* plot
     +     )
c ##################################################################
      INTEGER etut1n                          ! mean Et - distr.
      PARAMETER (etut1n=8)
      REAL etut1b(etut1n+1)
      DATA etut1b/ 3.82, 5.0, 6.54, 8.55, 11.18, 14.62, 19.12, 
     +     25.00, 32.69 / 
c
      INTEGER xgut1n                          ! x-gamma 1
      PARAMETER (xgut1n=8)
      REAL xgut1b(xgut1n+1)
      DATA xgut1b/ 0.0, 0.125, 0.25, 0.375, 0.5, 0.625, 0.75, 
     +      0.875, 1.0 / 
c
      INTEGER xgut1sn                         ! x-gamma small x/log
      PARAMETER (xgut1sn=6)
      REAL xgut1sb(xgut1sn+1)
      DATA xgut1sb/ -1.98, -1.65, -1.32, -0.99, -0.66, -0.33,  
     +     0.0 / 
c
      INTEGER costhn                          ! cos theta*
      PARAMETER (costhn=8)
      REAL costhb(costhn+1)
      DATA costhb/ 0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8 /
c
      INTEGER eaut2n                          ! mean eta
      PARAMETER (eaut2n=8)
      REAL eaut2b(eaut2n+1)
      DATA eaut2b/ 0.0, 0.25, 0.5, 0.75, 1.0, 1.25, 1.5, 1.75, 2.0 /
c
      INTEGER eaut4n                          ! mean eta
      PARAMETER (eaut4n=8)
      REAL eaut4b(eaut4n+1)
      DATA eaut4b/ 0.0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5, 4.0 /
c --------------------------------------------------------------------

c --- DATA points ----------------------------------------------------

c --- costh

      real pctd(8),pctdet(8),pctdes(8)
      data pctd  /5.04, 4.96, 5.22, 5.35, 4.90, 6.73, 7.09, 8.42/ 
      data pctdet/0.58, 0.60, 0.59, 0.61, 0.55, 0.73, 0.69, 0.80/ 
      data pctdes/0.49, 0.49, 0.49, 0.48, 0.45, 0.56, 0.54, 0.61/ 

      real pctr(8),pctret(8),pctres(8)
      data pctr  /1.50, 2.63, 2.91, 3.73, 4.36, 6.52, 9.90, 13.05/ 
      data pctret/0.39, 0.58, 0.65, 0.75, 0.87, 1.09, 1.41,  1.40/ 
      data pctres/0.31, 0.46, 0.50, 0.62, 0.61, 0.80, 0.92,  0.96/ 

c --- et-mean

      real pet(8),petet(8),petes(8)
      data  pet  /0., 11.54, 4.16, 1.45, 0.543, 0.176, 0.0607, 0./ 
      data  petet/0.,  0.70, 0.30, 0.12, 0.058, 0.028, 0.0146, 0./ 
      data  petes/0.,  0.36, 0.16, 0.07, 0.036, 0.020, 0.0117, 0./ 
      
      real petsr(8),petsret(8),petsres(8)
      data  petsr  /0., 3.56, 1.32, 0.475, 0.190, 0.0518, 0., 0./ 
      data  petsret/0., 0.30, 0.14, 0.058, 0.035, 0.0147, 0., 0./ 
      data  petsres/0., 0.21, 0.09, 0.045, 0.025, 0.0127, 0., 0./ 

      real petdr(8),petdret(8),petdres(8)
      data  petdr  /0., 5.94, 1.84, 0.461, 0.0781, 0., 0., 0./ 
      data  petdret/0., 0.78, 0.28, 0.093, 0.0232, 0., 0., 0./ 
      data  petdres/0., 0.41, 0.16, 0.060, 0.0192, 0., 0., 0./ 

c --- xg - full phase space

      real pxg1(8),pxg1et(8),pxg1es(8)
      data pxg1/47.29, 46.29, 33.66, 25.47, 25.70, 38.91, 79.59, 51.97/ 
      data pxg1et/5.07, 5.43, 4.01, 2.96, 2.85, 3.66, 9.31, 7.50/ 
      data pxg1es/2.51, 2.73, 2.16, 1.93, 1.86, 2.47, 4.07, 5.12/ 

      real pxg2(6),pxg2et(6),pxg2es(6)
      data pxg2  / 0.,  4.00,   9.54,  16.55,  27.51,  76.87/ 
      data pxg2et/ 0.,  0.60,   1.32,   2.15,   2.73,   5.33/ 
      data pxg2es/ 0.,  0.42,   0.76,   1.12,   1.39,   1.69/ 

      real pxg3(8),pxg3et(8),pxg3es(8)
      data pxg3/12.41, 16.78, 12.64, 10.36, 11.67, 14.80, 38.84, 43.07/ 
      data pxg3et/2.19, 2.57, 1.99, 1.44, 1.56, 1.85, 4.83, 5.62/ 
      data pxg3es/1.37, 1.55, 1.34, 1.16, 1.24, 1.53, 2.72, 3.00/ 

      real pxg4(8),pxg4et(8),pxg4es(8)
      data pxg4  /1.51, 2.83, 2.78, 2.30, 2.97, 4.43, 11.04, 26.81/ 
      data pxg4et/0.66, 0.83, 0.69, 0.60, 0.70, 0.86,  1.55,  2.79/ 
      data pxg4es/0.53, 0.52, 0.54, 0.48, 0.57, 0.66,  1.23,  1.85/ 


c --- xg - single resolved

      real pxs1(8),pxs1et(8),pxs1es(8)
      data pxs1/12.47, 8.87, 7.00, 5.12, 5.99, 12.36, 38.50, 11.80/ 
      data pxs1et/ 1.52, 1.38, 1.08, 0.87, 0.89, 1.61, 4.61, 2.15/ 
      data pxs1es/ 0.96, 0.74, 0.69, 0.58, 0.69, 1.27, 1.92, 1.72/ 

      real pxs2(6),pxs2et(6),pxs2es(6)
      data pxs2  / 0.,  1.29,   2.38,   3.64,   4.87,  25.87/ 
      data pxs2et/ 0.,  0.25,   0.40,   0.54,   0.60,   2.38/ 
      data pxs2es/ 0.,  0.18,   0.24,   0.30,   0.38,   0.99/ 

      real pxs3(8),pxs3et(8),pxs3es(8)
      data pxs3  /5.44, 4.84, 3.45, 2.91, 3.28, 5.38, 15.03, 8.63/ 
      data pxs3et/0.96, 0.71, 0.54, 0.46, 0.54, 0.86,  2.27, 1.77/ 
      data pxs3es/0.73, 0.52, 0.44, 0.41, 0.48, 0.76,  1.17, 1.19/ 

c --- xg - double resolved

      real pxd1(8),pxd1et(8),pxd1es(8)
      data pxd1  /42.37, 37.66, 26.87, 19.63, 17.84, 26.71, 0., 0./ 
      data pxd1et/ 5.55,  4.96,  3.32,  2.40,  1.90,  4.23, 0., 0./ 
      data pxd1es/ 2.94,  2.44,  1.99,  1.88,  1.55,  3.86, 0., 0./ 

      real pxd2(6),pxd2et(6),pxd2es(6)
      data pxd2  /0., 3.08,   7.81,  13.95,  19.39,  18.01/ 
      data pxd2et/0., 0.61,   1.20,   1.80,   2.11,   2.05/ 
      data pxd2es/0., 0.44,   0.63,   0.92,   1.13,   1.53/ 

      real pxd3(8),pxd3et(8),pxd3es(8)
      data pxd3  /8.81, 11.00, 10.30, 8.10, 7.49, 8.51, 0., 0./ 
      data pxd3et/1.88,  1.98,  1.57, 1.11, 1.14, 1.74, 0., 0./ 
      data pxd3es/1.08,  1.00,  0.96, 0.81, 0.94, 1.58, 0., 0./ 

c --- eta single resolved

      real pdes(8),pdeset(8),pdeses(8)
      data pdes  /6.68, 5.51, 3.98, 2.34, 1.22, 0.631, 0.251, 0./ 
      data pdeset/0.46, 0.39, 0.31, 0.22, 0.15, 0.110, 0.066, 0./ 
      data pdeses/0.27, 0.24, 0.20, 0.15, 0.11, 0.081, 0.056, 0./ 

      real pecs(8),pecset(8),pecses(8)
      data pecs  / 11.48, 9.25, 7.19, 5.30, 3.49, 2.60, 1.49, 0./ 
      data pecset/  0.82, 0.71, 0.60, 0.44, 0.34, 0.35, 0.27, 0./ 
      data pecses/  0.50, 0.44, 0.38, 0.32, 0.26, 0.24, 0.21, 0./ 

      real pefs(8),pefset(8),pefses(8)
      data pefs  /0.906, 2.68, 4.30, 5.65, 6.18, 6.86, 7.27, 7.03/ 
      data pefset/0.152, 0.29, 0.39, 0.47, 0.47, 0.50, 0.62, 0.70/ 
      data pefses/0.137, 0.23, 0.29, 0.33, 0.34, 0.36, 0.40, 0.44/ 

c --- eta double resolved
      real pded(8),pdedet(8),pdedes(8)
      data pded  /7.47, 6.89, 5.42, 3.93, 2.44, 1.43, 0.618, 0./ 
      data pdedet/0.75, 0.69, 0.54, 0.38, 0.35, 0.29, 0.182, 0./ 
      data pdedes/0.36, 0.35, 0.29, 0.24, 0.19, 0.16, 0.124, 0./ 
     
      real pecd(8),pecdet(8),pecdes(8)
      data pecd  / 14.93, 13.38, 10.83, 7.17, 5.59, 3.20, 2.01, 0./ 
      data pecdet/  1.41,  1.34,  1.13, 0.76, 0.71, 0.58, 0.52, 0./ 
      data pecdes/  0.67,  0.65,  0.60, 0.47, 0.46, 0.41, 0.41, 0./ 

      real pefd(8),pefdet(8),pefdes(8)
      data pefd  /1.03, 4.11, 5.64, 6.71, 8.40, 10.72, 11.30, 9.86/ 
      data pefdet/0.23, 0.66, 0.72, 0.65, 0.88,  1.12,  1.36, 1.20/ 
      data pefdes/0.17, 0.38, 0.40, 0.41, 0.47,  0.62,  0.68, 0.72/ 

*
* --------------------------------------------------------------------
*
      double precision scelze,scelpi
      parameter(
     +     scelze=0.033D0,      ! max angle scat. el at ZERO
     +     scelpi=3.1085927D0   ! max angle scat. el at PI
     +     )

      DOUBLE PRECISION mhxsec(4)
      INTEGER mhntot(4)
      REAL norm
      INTEGER initflag, iproc, iflag
      DOUBLE PRECISION angle

c local work variables
      integer i,j,maxtrk
      PARAMETER (maxtrk=2000)
      integer veto1,veto2,kbeam,i1,i2
      integer maxjet
      parameter (maxjet=100)
      double precision zero
      PARAMETER (ZERO   =  0.D0)
      integer stflag
      data stflag/0/
      logical istpfs

c     local variables for ktclus
      double precision vin(4,maxtrk),vpre(4,maxjet)
      integer vpart(maxjet),nftrk,njet,nktin
      real yc(maxjet)
      real jtheta,jeta
      real ktinpx(maxjet),ktinpy(maxjet),ktinpz(maxjet)
      real ktinen(maxjet)
      real pihalf,pival,wght
      parameter (pihalf = 1.5707963, pival = 3.1415927)
      real etj1,etj2,etjm,thj1,thj2,etaj1,etaj2
      real jesum,jpzsum,xgpl,xgmi
      real etatmp,etajm,detaj,etcom
      real etab,madj,cost
      real mcpxhv,mcpyhv,mcpzhv,mcenhv
      real eamcmi,eamcma

c
c --- functions ----------------------------------------------------
      DOUBLE PRECISION hzeekin
      INTEGER hzeebeam


C     --- choose direct/resolved -----------------------------------
C     ---0 - All  1 - DIRECT, 2 - single-RESOLVED, 3 - double-RES

      iflag = iflag + 1000

      IF (iflag.gt.1000) THEN
        iproc=INT(iflag/1000)
        initflag=MOD(iflag,1000)
      ELSE
        PRINT*,'H0301013 : initflag < 1000 ???'
      ENDIF
      IF ((iproc.gt.4).or.(iproc.lt.1)) THEN
        PRINT*,'H0301013 : Badly called routine'
        PRINT*,'           set initflag +1000, +2000 or +3000'
      ENDIF
      
c --- Initialization -----------------------------------------------
      IF (initflag.eq.1) THEN
	IF (stflag.eq.0) THEN
         stflag=1
C --- Create directories 
         CALL HCDIR('//HISTO',' ')
         CALL HMDIR('h0301013','S')
         CALL HCDIR('//PAWC',' ')
         CALL HMDIR('h0301013','S')
         CALL HCDIR('//HISTO/h0301013',' ')
         CALL HCDIR('//PAWC/h0301013',' ')

c --- book the MC histograms
         CALL HBOOKB(10211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 />G<E>V',
     +        etut1n,etut1b,0.)
         CALL HBOOKB(10313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(10314,'log10(<x?[g]! )',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(10323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(10333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(10410,'cosThst',costhn,costhb,0.)
         
         CALL HBOOKB(110211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 />G<E>V',
     +        etut1n,etut1b,0.)
         CALL HBOOKB(110313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(110314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(110323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(110333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(110410,'cosThst',costhn,costhb,0.)
         
         CALL HBOOKB(210211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 />G<E>V',
     +        etut1n,etut1b,0.)
         CALL HBOOKB(210313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(210314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(210323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(210333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(210410,'cosThst',costhn,costhb,0.)
         
         CALL HBOOKB(310211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 />G<E>V',
     +        etut1n,etut1b,0.)
         CALL HBOOKB(310313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(310314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(310323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(310333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(310410,'cosThst',costhn,costhb,0.)
         
c --------------------------------------------------------------------

         CALL HBOOKB(15211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 
     +        />G<E>V',etut1n,etut1b,0.)
         CALL HBOOKB(15313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(15314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(15323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(15333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(15411,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(15412,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(15413,'[c]?jet! />G<E>V',eaut4n,eaut4b,0.)
         
         CALL HBOOKB(115211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 
     +        />G<E>V',etut1n,etut1b,0.)
         CALL HBOOKB(115313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(115314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(115323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(115333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(115411,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(115412,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(115413,'[c]?jet! />G<E>V',eaut4n,eaut4b,0.)
         
         CALL HBOOKB(215211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 
     +        />G<E>V',etut1n,etut1b,0.)
         CALL HBOOKB(215313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(215314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(215323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(215333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(215411,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(215412,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(215413,'[c]?jet! />G<E>V',eaut4n,eaut4b,0.)
         
         CALL HBOOKB(315211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 
     +        />G<E>V',etut1n,etut1b,0.)
         CALL HBOOKB(315313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(315314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(315323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(315333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(315411,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(315412,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(315413,'[c]?jet! />G<E>V',eaut4n,eaut4b,0.)
         
c --------------------------------------------------------------------

         CALL HBOOKB(16211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 
     +        />G<E>V',etut1n,etut1b,0.)
         CALL HBOOKB(16313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(16314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(16323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(16333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(16410,'cosThst',costhn,costhb,0.)
         CALL HBOOKB(16411,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(16412,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(16413,'[c]?jet! />G<E>V',eaut4n,eaut4b,0.)
         
         CALL HBOOKB(116211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 
     +        />G<E>V',etut1n,etut1b,0.)
         CALL HBOOKB(116313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(116314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(116323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(116333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(116410,'cosThst',costhn,costhb,0.)
         CALL HBOOKB(116411,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(116412,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(116413,'[c]?jet! />G<E>V',eaut4n,eaut4b,0.)
         
         CALL HBOOKB(216211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 
     +        />G<E>V',etut1n,etut1b,0.)
         CALL HBOOKB(216313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(216314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(216323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(216333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(216410,'cosThst',costhn,costhb,0.)
         CALL HBOOKB(216411,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(216412,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(216413,'[c]?jet! />G<E>V',eaut4n,eaut4b,0.)
         
         CALL HBOOKB(316211,'(E?T!&&^jet1!+ E?T!&&^jet2!)/2 
     +        />G<E>V',etut1n,etut1b,0.)
         CALL HBOOKB(316313,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(316314,'log10(<x?[g]!)',xgut1sn,xgut1sb,0.)
         CALL HBOOKB(316323,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(316333,'<x?[g]!',xgut1n,xgut1b,0.)
         CALL HBOOKB(316410,'cosThst',costhn,costhb,0.)
         CALL HBOOKB(316411,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(316412,'[c]?jet! />G<E>V',eaut2n,eaut2b,0.)
         CALL HBOOKB(316413,'[c]?jet! />G<E>V',eaut4n,eaut4b,0.)

c --------------------------------------------------------------------
c copy the histograms for the data points and fill them
c --------------------------------------------------------------------

         CALL HCOPY(10211,-10211,' ')
         CALL HCOPY(10313,-10313,' ')
         CALL HCOPY(10314,-10314,' ')
         CALL HCOPY(10323,-10323,' ')
         CALL HCOPY(10333,-10333,' ')
         CALL HCOPY(10410,-10410,' ')

         CALL HCOPY(10211,-110211,' ')
         CALL HCOPY(10313,-110313,' ')
         CALL HCOPY(10314,-110314,' ')
         CALL HCOPY(10323,-110323,' ')
         CALL HCOPY(10333,-110333,' ')
         CALL HCOPY(10410,-110410,' ')

         CALL HCOPY(15211,-15211,' ')
         CALL HCOPY(15313,-15313,' ')
         CALL HCOPY(15314,-15314,' ')
         CALL HCOPY(15323,-15323,' ')
         CALL HCOPY(15411,-15411,' ')
         CALL HCOPY(15412,-15412,' ')
         CALL HCOPY(15413,-15413,' ')

         CALL HCOPY(15211,-115211,' ')
         CALL HCOPY(15313,-115313,' ')
         CALL HCOPY(15314,-115314,' ')
         CALL HCOPY(15323,-115323,' ')
         CALL HCOPY(15411,-115411,' ')
         CALL HCOPY(15412,-115412,' ')
         CALL HCOPY(15413,-115413,' ')

         CALL HCOPY(16211,-16211,' ')
         CALL HCOPY(16313,-16313,' ')
         CALL HCOPY(16314,-16314,' ')
         CALL HCOPY(16323,-16323,' ')
         CALL HCOPY(16410,-16410,' ')
         CALL HCOPY(16411,-16411,' ')
         CALL HCOPY(16412,-16412,' ')
         CALL HCOPY(16413,-16413,' ')

         CALL HCOPY(16211,-116211,' ')
         CALL HCOPY(16313,-116313,' ')
         CALL HCOPY(16314,-116314,' ')
         CALL HCOPY(16323,-116323,' ')
         CALL HCOPY(16410,-116410,' ')
         CALL HCOPY(16411,-116411,' ')
         CALL HCOPY(16412,-116412,' ')
         CALL HCOPY(16413,-116413,' ')
        
         CALL HRESET(0,' ')
         CALL HBARX(0)

c --- fill the histos with data point and total uncertainty ----------

c --- content
         CALL HPAK(-10211,pet)
         CALL HPAK(-10313,pxg1)
         CALL HPAK(-10314,pxg2)
         CALL HPAK(-10323,pxg3)
         CALL HPAK(-10333,pxg4)
         CALL HPAK(-10410,pctd)
                           
         CALL HPAK(-15211,petsr)
         CALL HPAK(-15313,pxs1)
         CALL HPAK(-15314,pxs2)
         CALL HPAK(-15323,pxs3)
         CALL HPAK(-15411,pecs)
         CALL HPAK(-15412,pefs)
         CALL HPAK(-15413,pdes)
                           
         CALL HPAK(-16211,petdr)
         CALL HPAK(-16313,pxd1)
         CALL HPAK(-16314,pxd2)
         CALL HPAK(-16323,pxd3)
         CALL HPAK(-16410,pctr)
         CALL HPAK(-16411,pecd)
         CALL HPAK(-16412,pefd)
         CALL HPAK(-16413,pded)

c --- total uncertainty                           
         CALL HPAKE(-10211,petet)
         CALL HPAKE(-10313,pxg1et)
         CALL HPAKE(-10314,pxg2et)
         CALL HPAKE(-10323,pxg3et)
         CALL HPAKE(-10333,pxg4et)
         CALL HPAKE(-10410,pctdet)
                           
         CALL HPAKE(-15211,petsret)
         CALL HPAKE(-15313,pxs1et)
         CALL HPAKE(-15314,pxs2et)
         CALL HPAKE(-15323,pxs3et)
         CALL HPAKE(-15411,pecset)
         CALL HPAKE(-15412,pefset)
         CALL HPAKE(-15413,pdeset)
                           
         CALL HPAKE(-16211,petdret)
         CALL HPAKE(-16313,pxd1et)
         CALL HPAKE(-16314,pxd2et)
         CALL HPAKE(-16323,pxd3et)
         CALL HPAKE(-16410,pctret)
         CALL HPAKE(-16411,pecdet)
         CALL HPAKE(-16412,pefdet)
         CALL HPAKE(-16413,pdedet)
                           
c --- fill the histos with data point and stat. uncertainty ----------

c --- content
         CALL HPAK(-110211,pet)
         CALL HPAK(-110313,pxg1)
         CALL HPAK(-110314,pxg2)
         CALL HPAK(-110323,pxg3)
         CALL HPAK(-110333,pxg4)
         CALL HPAK(-110410,pctd)
                           
         CALL HPAK(-115211,petsr)
         CALL HPAK(-115313,pxs1)
         CALL HPAK(-115314,pxs2)
         CALL HPAK(-115323,pxs3)
         CALL HPAK(-115411,pecs)
         CALL HPAK(-115412,pefs)
         CALL HPAK(-115413,pdes)
                           
         CALL HPAK(-116211,petdr)
         CALL HPAK(-116313,pxd1)
         CALL HPAK(-116314,pxd2)
         CALL HPAK(-116323,pxd3)
         CALL HPAK(-116410,pctr)
         CALL HPAK(-116411,pecd)
         CALL HPAK(-116412,pefd)
         CALL HPAK(-116413,pded)

c --- stat. uncertainty                           
         CALL HPAKE(-110211,petes)
         CALL HPAKE(-110313,pxg1es)
         CALL HPAKE(-110314,pxg2es)
         CALL HPAKE(-110323,pxg3es)
         CALL HPAKE(-110333,pxg4es)
         CALL HPAKE(-110410,pctdes)
                           
         CALL HPAKE(-115211,petsres)
         CALL HPAKE(-115313,pxs1es)
         CALL HPAKE(-115314,pxs2es)
         CALL HPAKE(-115323,pxs3es)
         CALL HPAKE(-115411,pecses)
         CALL HPAKE(-115412,pefses)
         CALL HPAKE(-115413,pdeses)
                           
         CALL HPAKE(-116211,petdres)
         CALL HPAKE(-116313,pxd1es)
         CALL HPAKE(-116314,pxd2es)
         CALL HPAKE(-116323,pxd3es)
         CALL HPAKE(-116410,pctres)
         CALL HPAKE(-116411,pecdes)
         CALL HPAKE(-116412,pefdes)
         CALL HPAKE(-116413,pdedes)

c --------------------------------------------------------------------

      ENDIF                     ! stflag=0        
        
      WRITE(6,*)'************************************************'
      WRITE(6,*)'* H0301013 called, histograms will be generated *'
      WRITE(6,*)'* for dijet prod. in untagged photon-photon    *'
      WRITE(6,*)'* collisions. See OPAL PR 372 for reference.   *'
      WRITE(6,*)'************************************************'
         
c --- run the event loop -------------------------------------------
      ELSE IF (initflag.eq.2) THEN

c --- reset variables
	 veto1=0
	 veto2=0
	 i1=0
	 i2=0
         etj1  = 0.0
         etj2  = 0.0
         etjm  = 0.0
         etcom = 0.0
         thj1  = 0.0
         thj2  = 0.0
         etaj1 = 0.0
         etaj2 = 0.0
         etatmp= 0.0
         etajm = 0.0
         detaj = 0.0
         jesum = 0.0
         jpzsum= 0.0
         mcpxhv = 0.0
         mcpyhv = 0.0
         mcpzhv = 0.0
         mcenhv = 0.0
         eamcmi  = 0.0
         eamcma  = 0.0
         etab    = 0.0
         madj    = 0.0
         cost    = 0.0

         xgpl= 0.0
         xgmi= 0.0
         njet=0

         
         do 10 i=1,maxtrk
            vin(1,i)=0.0D0
            vin(2,i)=0.0D0
            vin(3,i)=0.0D0
            vin(4,i)=0.0D0
 10      continue
         do 11 i=1,maxjet
            do 12 j=1,4
               vpre(j,i)=0.0D0
 12         continue
            ktinpx(i)=0.0
            ktinpy(i)=0.0
            ktinpz(i)=0.0
            ktinen(i)=0.0
            vpart(i)=0.0
            yc(i)=0
 11      continue

         nftrk = 0

         wght = 1.0
         angle=1.D0

         CALL HCDIR('//PAWC/h0301013',' ')

         angle=hzeekin(2)

C --- Make sure scattered electrons are not included in hadronic
c     final state. (NB. Not robust against changes in positions of
c     electrons in HEP common for a given generator).
            IF (GEN(1:3).eq.'HRW') THEN
              veto1=1
              veto2=2
            ELSEIF (GEN(1:3).eq.'PYT') THEN
              kbeam= hzeebeam(i1,i2) ! find e- and e+ position
              If ((i1.eq.1).and.(i2.eq.2)) Then
                veto1=3
                veto2=4
              ElseIf ((i1.eq.2).and.(i2.eq.1)) Then
                veto1=4
                veto2=3
              EndIf
            ELSEIF (GEN(1:3).eq.'PHO') THEN
              veto1=3
              veto2=5
            ELSE
              write(6,*)'H0301013: Unknown generator'
            ENDIF
            IF ((abs(IDHEP(veto1)).ne.11).or.
     +		      (abs(IDHEP(veto2)).ne.11)) then
	      print*,'H0301013: electrons not correctly found (a)!'
              write(*,*) veto1,":",IDHEP(veto1),",",
     &             veto2,":",IDHEP(veto2)
	    ENDIF
            
            firstpa=-1
            istpfs=.true.
            DO 100 I=1,NHEP

C --- Another check that electrons found correctly
	       IF((ISTHEP(I).eq.1).and.((JMOHEP(1,I).eq.veto1)
     +		 .or.(JMOHEP(1,I).eq.veto2))
     +		 .and.(abs(IDHEP(I)).ne.11)) THEN
		  print*,'H0301013: electr. not correctly found (b)!'
	       ENDIF
C --- select final particles excluding electrons
               IF((ISTHEP(I).EQ.1).and.(JMOHEP(1,I).ne.veto1)
     +		 .and.(JMOHEP(1,I).ne.veto2)) THEN
c     hadronic final state - prepare input array for jet finders
c                     print *,'HAD: ',IDHEP(I),PHEP(4,I)
                  if (nftrk.LT.maxtrk) then
                     nftrk=nftrk+1
                     vin(1,nftrk) = PHEP(1,I)
                     vin(2,nftrk) = PHEP(2,I)
                     vin(3,nftrk) = PHEP(3,I)
                     vin(4,nftrk) = PHEP(4,I)
                     mcpxhv=mcpxhv+sngl(PHEP(1,I))
                     mcpyhv=mcpyhv+sngl(PHEP(2,I))
                     mcpzhv=mcpzhv+sngl(PHEP(3,I))
                     mcenhv=mcenhv+sngl(PHEP(4,I))
                  ELSE
                     WRITE(*,*) 'H0301013: Too many tracks in event'
                  ENDIF
               ENDIF
 100        CONTINUE
            
c --- call jet finders
c     kt_incl hadronic final state
            call ktclur(4212,vin,nftrk,1.0,1.0,yc,*999)
            call ktincl(2,vin,nftrk,vpre,vpart,njet,*998)



c --- apply pt & eta jet cut and order with largest pt first
            nktin=0
            do 30 i=njet,1,-1
               if(sngl(vpre(1,i)**2 + vpre(2,i)**2) 
     +              .gt. ptmin**2) then
                  jtheta=acos(sngl(vpre(3,i))/sngl(vpre(4,i)))
                  jeta=-log(tan(jtheta/2))
                  if(abs(jeta).lt.etamax) then
                     if (nktin.LT.maxjet) then
                        nktin=nktin+1
                        ktinpx(nktin) = sngl(vpre(1,i))
                        ktinpy(nktin) = sngl(vpre(2,i))
                        ktinpz(nktin) = sngl(vpre(3,i))
                        ktinen(nktin) = sngl(vpre(4,i))
                     else
                        WRITE(*,*) 'HZPRN: Too many jets'
                     endif
                  endif
               endif
 30         continue
c     
c -------------------------------------------------------------------
c fill histograms for HADRONIC FINAL STATE
c -------------------------------------------------------------------
c     jump out if less than 2 jets
            if(nktin.lt.2) goto 880
*     calculate jet quantities
            etj1  = sqrt(ktinpx(1)**2+ktinpy(1)**2)
            etj2  = sqrt(ktinpx(2)**2+ktinpy(2)**2)
            etjm  = (etj1+etj2)/2
            etcom = abs(etj1-etj2)/(etj1+etj2)
            thj1  = atan2(sqrt(ktinpx(1)**2+ktinpy(1)**2),ktinpz(1))
            thj2  = atan2(sqrt(ktinpx(2)**2+ktinpy(2)**2),ktinpz(2))
            etaj1 = -log(tan(thj1/2))
            etaj2 = -log(tan(thj2/2))
            etajm=(abs(etaj1)+abs(etaj2))/2
            detaj=abs(etaj1-etaj2)
            jesum = ktinen(1)+ktinen(2)
            jpzsum= ktinpz(1)+ktinpz(2)
            
            xgpl=( jesum+jpzsum ) / ( mcenhv+mcpzhv )
            xgmi=( jesum-jpzsum ) / ( mcenhv-mcpzhv )
            
            eamcmi=min(abs(etaj1),abs(etaj2))
            eamcma=max(abs(etaj1),abs(etaj2))

            cost = abs(tanh((etaj1-etaj2)/2))
            madj = 2*etjm/(sqrt(1-(cost**2)))
            etab = (etaj1+etaj2)/2


c     apply the jet-cuts and fill the histos
c

            if(etjm.gt.metmi1 .and. etcom.lt.jetasy) then

c --- direct enhanced sample test condition
               direnh=.false.
               if( (xgpl.ge.0.75) .and. (xgmi .ge.0.75) )
     +              direnh=.true.
c --- single resovled enhanced sample test condition
               snglenh=.false.
               if( ((xgpl.ge.0.75) .and. (xgmi .lt.0.75)) .or.
     +              ((xgpl.lt.0.75) .and. (xgmi .ge.0.75)) ) 
     +              snglenh=.true.
c --- double resovled enhanced sample test condition
               dbleenh=.false.
               if( (xgpl.lt.0.75) .and. (xgmi .lt.0.75) )
     +              dbleenh=.true.

c --- fill the Et-mean distribution
               if(iproc.eq.1) then                              ! All
                  call hf1(10211,etjm,wght)          
                  if(snglenh) call hf1(15211,etjm,wght)
                  if(dbleenh) call hf1(16211,etjm,wght)
                  
                  if(direnh) then
                     if(etab.lt.etabma .and. madj.gt.djmami)
     +                    call hf1(10410,cost,wght)
                  endif
                  if(snglenh) then
                     call hf1(15411,abs(eamcmi),wght)
                     call hf1(15412,abs(eamcma),wght)
                     call hf1(15413,abs(etaj1-etaj2),wght)
                  endif
                  if(dbleenh) then
                     if(etab.lt.etabma .and. madj.gt.djmami)
     +                     call hf1(16410,cost,wght)
                     call hf1(16411,abs(eamcmi),wght)
                     call hf1(16412,abs(eamcma),wght)
                     call hf1(16413,abs(etaj1-etaj2),wght)
                  endif
               elseif(iproc.eq.2) then                       ! direct
                  call hf1(110211,etjm,wght)
                  if(snglenh) call hf1(115211,etjm,wght)
                  if(dbleenh) call hf1(116211,etjm,wght)

                  if(direnh) then
                     if(etab.lt.etabma .and. madj.gt.djmami)
     +                    call hf1(110410,cost,wght)
                  endif
                  if(snglenh) then
                     call hf1(115411,abs(eamcmi),wght)
                     call hf1(115412,abs(eamcma),wght)
                     call hf1(115413,abs(etaj1-etaj2),wght)
                  endif
                  if(dbleenh) then
                     if(etab.lt.etabma .and. madj.gt.djmami)
     +                     call hf1(116410,cost,wght)
                     call hf1(116411,abs(eamcmi),wght)
                     call hf1(116412,abs(eamcma),wght)
                     call hf1(116413,abs(etaj1-etaj2),wght)
                  endif
               elseif(iproc.eq.3) then              ! single resolved
                  call hf1(210211,etjm,wght)
                  if(snglenh) call hf1(215211,etjm,wght)
                  if(dbleenh) call hf1(216211,etjm,wght)

                  if(direnh) then
                     if(etab.lt.etabma .and. madj.gt.djmami)
     +                    call hf1(210410,cost,wght)
                  endif
                  if(snglenh) then
                     call hf1(215411,abs(eamcmi),wght)
                     call hf1(215412,abs(eamcma),wght)
                     call hf1(215413,abs(etaj1-etaj2),wght)
                  endif
                  if(dbleenh) then
                     if(etab.lt.etabma .and. madj.gt.djmami)
     +                     call hf1(216410,cost,wght)
                     call hf1(216411,abs(eamcmi),wght)
                     call hf1(216412,abs(eamcma),wght)
                     call hf1(216413,abs(etaj1-etaj2),wght)
                  endif
               elseif(iproc.eq.4) then              ! double resolved
                  call hf1(310211,etjm,wght)
                  if(snglenh) call hf1(315211,etjm,wght)
                  if(dbleenh) call hf1(316211,etjm,wght)

                  if(direnh) then
                     if(etab.lt.etabma .and. madj.gt.djmami)
     +                    call hf1(310410,cost,wght)
                  endif
                  if(snglenh) then
                     call hf1(315411,abs(eamcmi),wght)
                     call hf1(315412,abs(eamcma),wght)
                     call hf1(315413,abs(etaj1-etaj2),wght)
                  endif
                  if(dbleenh) then
                     if(etab.lt.etabma .and. madj.gt.djmami)
     +                     call hf1(316410,cost,wght)
                     call hf1(316411,abs(eamcmi),wght)
                     call hf1(316412,abs(eamcma),wght)
                     call hf1(316413,abs(etaj1-etaj2),wght)
                  endif
               endif

c --- fill the x-gamma distributions for region 1
               if(etjm.gt.metmi1 .and. etjm.le.metma1) then 
                  if(iproc.eq.1) then                           ! ALL
                     call hf1(10313,xgpl,wght)
                     call hf1(10313,xgmi,wght)
                     if(snglenh) call hf1(15313,xgpl,wght)
                     if(snglenh) call hf1(15313,xgmi,wght)
                     if(dbleenh) call hf1(16313,xgpl,wght)
                     if(dbleenh) call hf1(16313,xgmi,wght)
                     call hf1(10314,log10(xgpl),wght)
                     call hf1(10314,log10(xgmi),wght)
                     if(snglenh) call hf1(15314,log10(xgpl),wght)
                     if(snglenh) call hf1(15314,log10(xgmi),wght)
                     if(dbleenh) call hf1(16314,log10(xgpl),wght)
                     if(dbleenh) call hf1(16314,log10(xgmi),wght)
                  elseif(iproc.eq.2) then                    ! direct
                     call hf1(110313,xgpl,wght)
                     call hf1(110313,xgmi,wght)
                     if(snglenh) call hf1(115313,xgpl,wght)
                     if(snglenh) call hf1(115313,xgmi,wght)
                     if(dbleenh) call hf1(116313,xgpl,wght)
                     if(dbleenh) call hf1(116313,xgmi,wght)
                     call hf1(110314,log10(xgpl),wght)
                     call hf1(110314,log10(xgmi),wght)
                     if(snglenh) call hf1(115314,log10(xgpl),wght)
                     if(snglenh) call hf1(115314,log10(xgmi),wght)
                     if(dbleenh) call hf1(116314,log10(xgpl),wght)
                     if(dbleenh) call hf1(116314,log10(xgmi),wght)
                  elseif(iproc.eq.3) then           ! single resolved
                     call hf1(210313,xgpl,wght)
                     call hf1(210313,xgmi,wght)
                     if(snglenh) call hf1(215313,xgpl,wght)
                     if(snglenh) call hf1(215313,xgmi,wght)
                     if(dbleenh) call hf1(216313,xgpl,wght)
                     if(dbleenh) call hf1(216313,xgmi,wght)
                     call hf1(210314,log10(xgpl),wght)
                     call hf1(210314,log10(xgmi),wght)
                     if(snglenh) call hf1(215314,log10(xgpl),wght)
                     if(snglenh) call hf1(215314,log10(xgmi),wght)
                     if(dbleenh) call hf1(216314,log10(xgpl),wght)
                     if(dbleenh) call hf1(216314,log10(xgmi),wght)
                  elseif(iproc.eq.4) then           ! double resolved
                     call hf1(310313,xgpl,wght) 
                     call hf1(310313,xgmi,wght)
                     if(snglenh) call hf1(315313,xgpl,wght)
                     if(snglenh) call hf1(315313,xgmi,wght)
                     if(dbleenh) call hf1(316313,xgpl,wght)
                     if(dbleenh) call hf1(316313,xgmi,wght)
                     call hf1(310314,log10(xgpl),wght)
                     call hf1(310314,log10(xgmi),wght)
                     if(snglenh) call hf1(315314,log10(xgpl),wght)
                     if(snglenh) call hf1(315314,log10(xgmi),wght)
                     if(dbleenh) call hf1(316314,log10(xgpl),wght)
                     if(dbleenh) call hf1(316314,log10(xgmi),wght)
                  endif
               endif
            
c --- fill the x-gamma distributions for region 2
               if(etjm.gt.metmi2 .and. etjm.le.metma2) then 
                  if(iproc.eq.1) then                           ! ALL
                     call hf1(10323,xgpl,wght)     
                     call hf1(10323,xgmi,wght)
                     if(snglenh) call hf1(15323,xgpl,wght)     
                     if(snglenh) call hf1(15323,xgmi,wght)
                     if(dbleenh) call hf1(16323,xgpl,wght)     
                     if(dbleenh) call hf1(16323,xgmi,wght)
                  elseif(iproc.eq.2) then                    ! direct
                     call hf1(110323,xgpl,wght) 
                     call hf1(110323,xgmi,wght)
                     if(snglenh) call hf1(115323,xgpl,wght)     
                     if(snglenh) call hf1(115323,xgmi,wght)
                     if(dbleenh) call hf1(116323,xgpl,wght)     
                     if(dbleenh) call hf1(116323,xgmi,wght)
                  elseif(iproc.eq.3) then           ! single resolved
                     call hf1(210323,xgpl,wght) 
                     call hf1(210323,xgmi,wght)
                     if(snglenh) call hf1(215323,xgpl,wght)     
                     if(snglenh) call hf1(215323,xgmi,wght)
                     if(dbleenh) call hf1(216323,xgpl,wght)     
                     if(dbleenh) call hf1(216323,xgmi,wght)
                  elseif(iproc.eq.4) then           ! double resolved
                     call hf1(310323,xgpl,wght) 
                     call hf1(310323,xgmi,wght)
                     if(snglenh) call hf1(315323,xgpl,wght)     
                     if(snglenh) call hf1(315323,xgmi,wght)
                     if(dbleenh) call hf1(316323,xgpl,wght)     
                     if(dbleenh) call hf1(316323,xgmi,wght) 
                  endif
               endif
               
c --- fill the x-gamma distributions for region 3
               if(etjm.gt.metmi3 .and. etjm.le.metma3) then 
                  if(iproc.eq.1) then                           ! ALL
                     call hf1(10333,xgpl,wght)   
                     call hf1(10333,xgmi,wght)
                  elseif(iproc.eq.2) then                    ! direct
                     call hf1(110333,xgpl,wght) 
                     call hf1(110333,xgmi,wght)
                  elseif(iproc.eq.3) then           ! single resolved
                     call hf1(210333,xgpl,wght) 
                     call hf1(210333,xgmi,wght)
                  elseif(iproc.eq.4) then           ! double resolved
                     call hf1(310333,xgpl,wght) 
                     call hf1(310333,xgmi,wght)
                  endif
               endif
            endif

c     jump point for not having 2 jets in HFS
 880        continue

         
c --- run termination ----------------------------------------------
      ELSE IF (initflag.eq.3) THEN

C --- Change directory 
         CALL HCDIR('//PAWC/h0301013',' ')

C --- check x-sections
         IF (Xsec.eq.0) THEN
            PRINT*,'H0301013: term. called with zero cross section'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'	      in process :',iproc*1000
            Xsec=1
         ENDIF
         IF (Ntot.eq.0) THEN
            PRINT*,'H0301013: term. called with no total events'
            PRINT*,'          cross section graph meaningless'
            PRINT*,'          in process :',iproc*1000
            Ntot=1
         ENDIF
         
C --- store Xsec and Ntot for both direct and resolved events.
        
         mhXsec(iproc)=1000*Xsec
         mhntot(iproc)=Ntot

         norm=0.
         norm=REAL(mhntot(iproc)/mhXsec(iproc))
 

c --- All
         if(iproc.eq.1) then
            call HzHinrm(10211,0,norm,1) ! Et: metmi1
            call HzHinrm(10313,0,norm,1) ! xg : metmi1
            call HzHinrm(10314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(10323,0,norm,1) ! xg : metmi2
            call HzHinrm(10333,0,norm,1) ! xg : metmi3     
            call HzHinrm(10410,0,norm,1) ! eta    
c --- sngl enhanced sample
            call HzHinrm(15211,0,norm,1) ! Et: metmi1
            call HzHinrm(15313,0,norm,1) ! xg : metmi1
            call HzHinrm(15314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(15323,0,norm,1) ! xg : metmi2
            call HzHinrm(15333,0,norm,1) ! xg : metmi3     
            call HzHinrm(15411,0,norm,1) ! eta    
            call HzHinrm(15412,0,norm,1) ! eta    
            call HzHinrm(15413,0,norm,1) ! eta    
c --- dble enhanced sample
            call HzHinrm(16211,0,norm,1) ! Et: metmi1
            call HzHinrm(16313,0,norm,1) ! xg : metmi1
            call HzHinrm(16314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(16323,0,norm,1) ! xg : metmi2
            call HzHinrm(16333,0,norm,1) ! xg : metmi3     
            call HzHinrm(16410,0,norm,1) ! eta    
            call HzHinrm(16411,0,norm,1) ! eta    
            call HzHinrm(16412,0,norm,1) ! eta    
            call HzHinrm(16413,0,norm,1) ! eta    
         endif

c --- direct 
         if(iproc.eq.2) then
c --- all xg-values
            call HzHinrm(110211,0,norm,1) ! Et: metmi1
            call HzHinrm(110313,0,norm,1) ! xg : metmi1
            call HzHinrm(110314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(110323,0,norm,1) ! xg : metmi2
            call HzHinrm(110333,0,norm,1) ! xg : metmi3     
            call HzHinrm(110410,0,norm,1) ! eta     
c --- sngl enhanced sample
            call HzHinrm(115211,0,norm,1) ! Et: metmi1
            call HzHinrm(115313,0,norm,1) ! xg : metmi1
            call HzHinrm(115314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(115323,0,norm,1) ! xg : metmi2
            call HzHinrm(115333,0,norm,1) ! xg : metmi3     
            call HzHinrm(115411,0,norm,1) ! eta     
            call HzHinrm(115412,0,norm,1) ! eta     
            call HzHinrm(115413,0,norm,1) ! eta     
c --- dble enhanced sample
            call HzHinrm(116211,0,norm,1) ! Et: metmi1
            call HzHinrm(116313,0,norm,1) ! xg : metmi1
            call HzHinrm(116314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(116323,0,norm,1) ! xg : metmi2
            call HzHinrm(116333,0,norm,1) ! xg : metmi3     
            call HzHinrm(116410,0,norm,1) ! eta     
            call HzHinrm(116411,0,norm,1) ! eta     
            call HzHinrm(116412,0,norm,1) ! eta     
            call HzHinrm(116413,0,norm,1) ! eta     
         endif                 
                               
c --- single resolved          
         if(iproc.eq.3) then   
c --- all xg-values
            call HzHinrm(210211,0,norm,1) ! Et: metmi1
            call HzHinrm(210313,0,norm,1) ! xg : metmi1
            call HzHinrm(210314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(210323,0,norm,1) ! xg : metmi2
            call HzHinrm(210333,0,norm,1) ! xg : metmi3     
            call HzHinrm(210410,0,norm,1) ! eta    
c --- sngl enhanced sample
            call HzHinrm(215211,0,norm,1) ! Et: metmi1
            call HzHinrm(215313,0,norm,1) ! xg : metmi1
            call HzHinrm(215314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(215323,0,norm,1) ! xg : metmi2
            call HzHinrm(215333,0,norm,1) ! xg : metmi3     
            call HzHinrm(215411,0,norm,1) ! eta    
            call HzHinrm(215412,0,norm,1) ! eta    
            call HzHinrm(215413,0,norm,1) ! eta    
c --- dble enhanced sample
            call HzHinrm(216211,0,norm,1) ! Et: metmi1
            call HzHinrm(216313,0,norm,1) ! xg : metmi1
            call HzHinrm(216314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(216323,0,norm,1) ! xg : metmi2
            call HzHinrm(216333,0,norm,1) ! xg : metmi3     
            call HzHinrm(216410,0,norm,1) ! eta    
            call HzHinrm(216411,0,norm,1) ! eta    
            call HzHinrm(216412,0,norm,1) ! eta    
            call HzHinrm(216413,0,norm,1) ! eta    
            
c --- all xg-values
            CALL HOPERA(210211,'+e',110211,1110211,1.,1.)
            CALL HOPERA(210313,'+e',110313,1110313,1.,1.)
            CALL HOPERA(210314,'+e',110314,1110314,1.,1.)
            CALL HOPERA(210323,'+e',110323,1110323,1.,1.)
            CALL HOPERA(210333,'+e',110333,1110333,1.,1.)
            CALL HOPERA(210410,'+e',110410,1110410,1.,1.)
c --- sngl enhanced sample
            CALL HOPERA(215211,'+e',115211,1115211,1.,1.)
            CALL HOPERA(215313,'+e',115313,1115313,1.,1.)
            CALL HOPERA(215314,'+e',115314,1115314,1.,1.)
            CALL HOPERA(215323,'+e',115323,1115323,1.,1.)
            CALL HOPERA(215333,'+e',115333,1115333,1.,1.)
            CALL HOPERA(215411,'+e',115411,1115411,1.,1.)
            CALL HOPERA(215412,'+e',115412,1115412,1.,1.)
            CALL HOPERA(215413,'+e',115413,1115413,1.,1.)
c --- sngl enhanced sample
            CALL HOPERA(216211,'+e',116211,1116211,1.,1.)
            CALL HOPERA(216313,'+e',116313,1116313,1.,1.)
            CALL HOPERA(216314,'+e',116314,1116314,1.,1.)
            CALL HOPERA(216323,'+e',116323,1116323,1.,1.)
            CALL HOPERA(216333,'+e',116333,1116333,1.,1.)
            CALL HOPERA(216410,'+e',116410,1116410,1.,1.)
            CALL HOPERA(216411,'+e',116411,1116411,1.,1.)
            CALL HOPERA(216412,'+e',116412,1116412,1.,1.)
            CALL HOPERA(216413,'+e',116413,1116413,1.,1.)

         endif

c --- double resolved
         if(iproc.eq.4) then
c --- all xg-values
            call HzHinrm(310211,0,norm,1) ! Et: metmi1
            call HzHinrm(310313,0,norm,1) ! xg : metmi1
            call HzHinrm(310314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(310323,0,norm,1) ! xg : metmi2
            call HzHinrm(310333,0,norm,1) ! xg : metmi3     
            call HzHinrm(310410,0,norm,1) ! eta    
c --- sngl enhanced sample
            call HzHinrm(315211,0,norm,1) ! Et: metmi1
            call HzHinrm(315313,0,norm,1) ! xg : metmi1
            call HzHinrm(315314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(315323,0,norm,1) ! xg : metmi2
            call HzHinrm(315333,0,norm,1) ! xg : metmi3     
            call HzHinrm(315411,0,norm,1) ! eta    
            call HzHinrm(315412,0,norm,1) ! eta    
            call HzHinrm(315413,0,norm,1) ! eta    
c --- dble enhanced sample
            call HzHinrm(316211,0,norm,1) ! Et: metmi1
            call HzHinrm(316313,0,norm,1) ! xg : metmi1
            call HzHinrm(316314,0,norm,1) ! xg : metmi1 log
            call HzHinrm(316323,0,norm,1) ! xg : metmi2
            call HzHinrm(316333,0,norm,1) ! xg : metmi3     
            call HzHinrm(316410,0,norm,1) ! eta    
            call HzHinrm(316411,0,norm,1) ! eta    
            call HzHinrm(316412,0,norm,1) ! eta    
            call HzHinrm(316413,0,norm,1) ! eta    
            
c --- all xg-values
            CALL HOPERA(1110211,'+e',310211,10211,1.,1.)
            CALL HOPERA(1110313,'+e',310313,10313,1.,1.)
            CALL HOPERA(1110314,'+e',310314,10314,1.,1.)
            CALL HOPERA(1110323,'+e',310323,10323,1.,1.)
            CALL HOPERA(1110333,'+e',310333,10333,1.,1.)
            CALL HOPERA(1110410,'+e',310410,10410,1.,1.)
c --- sngl enhanced sample
            CALL HOPERA(1115211,'+e',315211,15211,1.,1.)
            CALL HOPERA(1115313,'+e',315313,15313,1.,1.)
            CALL HOPERA(1115314,'+e',315314,15314,1.,1.)
            CALL HOPERA(1115323,'+e',315323,15323,1.,1.)
            CALL HOPERA(1115333,'+e',315333,15333,1.,1.)
            CALL HOPERA(1115411,'+e',315411,15411,1.,1.)
            CALL HOPERA(1115412,'+e',315412,15412,1.,1.)
            CALL HOPERA(1115413,'+e',315413,15413,1.,1.)
c --- dble enhanced sample
            CALL HOPERA(1116211,'+e',316211,16211,1.,1.)
            CALL HOPERA(1116313,'+e',316313,16313,1.,1.)
            CALL HOPERA(1116314,'+e',316314,16314,1.,1.)
            CALL HOPERA(1116323,'+e',316323,16323,1.,1.)
            CALL HOPERA(1116333,'+e',316333,16333,1.,1.)
            CALL HOPERA(1116410,'+e',316410,16410,1.,1.)
            CALL HOPERA(1116411,'+e',316411,16411,1.,1.)
            CALL HOPERA(1116412,'+e',316412,16412,1.,1.)
            CALL HOPERA(1116413,'+e',316413,16413,1.,1.)

c --- all xg-values
            CALL HDELET(1110211)
            CALL HDELET(1110313)
            CALL HDELET(1110314)
            CALL HDELET(1110323)
            CALL HDELET(1110333)
            CALL HDELET(1110410)
c --- sngl enhanced sample
            CALL HDELET(1115211)
            CALL HDELET(1115313)
            CALL HDELET(1115314)
            CALL HDELET(1115323)
            CALL HDELET(1115333)
            CALL HDELET(1115411)
            CALL HDELET(1115412)
            CALL HDELET(1115413)
c --- sngl enhanced sample
            CALL HDELET(1116211)
            CALL HDELET(1116313)
            CALL HDELET(1116314)
            CALL HDELET(1116323)
            CALL HDELET(1116333)
            CALL HDELET(1116410)
            CALL HDELET(1116411)
            CALL HDELET(1116412)
            CALL HDELET(1116413)

         endif
      else
C     *** End ***
         PRINT*,'H0301013:Please run with iflag set to 1,2 or 3'
      ENDIF
      
      RETURN
      
c --- jet finder jump out
 999  continue
      print *,'H0301013: KTCLUR ERROR'
      return
 998  continue
      print *,'H0301013: KTINCL ERROR'
      return
c --- end of routine H0301013 ---------------------------------------
      END

