      SUBROUTINE HZHO1COP
#ifdef NEVER
c     -------------------
c     Fill hepevt common block with final state partons from the NLO
c     code of Frixione and Ridolfi.
c     Author: Jon Butterworth 23/4/00
c     -------------------

      IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "rfparam_hera.inc"

      INTEGER i, loop, ivec
      DOUBLE PRECISION pi, e, ycm_lab
      PARAMETER (pi=3.14159265358979312D0)
      LOGICAL lp
      DATA lp/.false./

c x1: momentum fraction of the photon relative to the maximum energy of the
c     photon beam (therefore, x1=1 if the incoming particle is a monochromatic
c     photon, and x1=E_gamma/E_e if the incoming particle is an electron). 
c     WARNING!!!! This can only be used when running the pontlike code.
c     With the hadronic code in the e-p mode, there is no information
c     available on E_gamma/E_e, since the convolution of the 
c     Weizsaecker-Williams function with the photon parton densities
c     is carried out before running this code. In the case of the
c     hadronic component, x1 is set to -1

c     The kinematical variables of the final state partons are given in
c     common blocks. The user can compute here the needed physical
c     observables and add the corresponding weight to the appropriate
c     histogram.
c     --------------------------------------------------

c     !!!! Do not alter the following variables !!!!
c     xsc_min2 = minimum transverse energy squared
c     xlam = lambda_QCD (5 flavours)
c     xmuf2 = factorization scale squared
c     xmur2 = renormalization scale squared
c     xmues2 = Ellis-Sexton scale squared
c     xmuww2 = Weizsaecker-Williams scale squared
c     zg = strong coupling = sqrt(4*pi*alfas)
c     ze2 = electron electric charge squared
      DOUBLE PRECISION xsc_min2,xlam,xmuf2,xmur2,xmues2,xmuww2,zg,ze2
      COMMON/FIXVAR/xsc_min2,xlam,xmuf2,xmur2,xmues2,xmuww2,zg,ze2

c     xkt_cm(i)=modulus of the transverse momentum of parton # i (i=1,2
c     ,3) in GeV
c     xeta_cm(i)=pseudorapidity of parton # i (i=1,2,3)
c     xphi_cm(i)=azimuthal angle of parton # i (i=1,2,3)
c     These quantities are defined in the center-of-mass frame of the
c     colliding particles (photon-proton or electron-proton system at
c     HERA). 
c     If needed, the user can perform a boost to an arbitrary frame 
c     in this routine
      DOUBLE PRECISION xkt_cm,xeta_cm,xphi_cm
      COMMON/CMKIN/xkt_cm(1:3),xeta_cm(1:3),xphi_cm(1:3)

c     sh is the square of the total CM energy of the colliding system
c     (in GeV^2)
      DOUBLE PRECISION sh
      COMMON/SHADR/sh

c     imtt and imtt2 are matrices used in the jet-finding algorithm
c     implemented in this version of the user file. They can be removed
c     when the user implements his own version of the jet definition
      DOUBLE PRECISION ykt(1:3),yeta(1:3),yphi(1:3)
      DOUBLE PRECISION zkt(1:3),zeta(1:3),zphi(1:3)
      INTEGER imtt(3:4,3:5)
      INTEGER imtt2(3:5,3:5)
      COMMON/IMTT/imtt
      COMMON/IMTT2/imtt2

      DOUBLE PRECISION phi_r, random, theta

c     Boost to the lab frame (HERA physics)
c     NB: The photon was in the positive direction - switch to HERA frame,
c     i.e. photon in -ve direction. Also rotate phi by pi.
      e=sqrt(sh)
      ycm_lab = 0.5*log(e_p/e_gamma)

      IF (lp) THEN
         write(*,*) 'ETA of CM in Lab is:',ycm_lab
      ENDIF

c     Event counter.
      nevhep = nevhep+1.

c     Roll a random phi.
      phi_r=2.0*pi*random(nevhep)

      DO i=1,3
         IF (lp) THEN
            write(*,*) 'CM:',xkt_cm(i),xeta_cm(i),xphi_cm(i)
         ENDIF
         ykt(i)=xkt_cm(i)
         IF (ykt(i).GT.0.00000001) THEN
c            write(*,*) 'LAB:yeta',yeta(i),xeta_cm(i),ycm_lab
            yeta(i)=-xeta_cm(i)+ycm_lab
c            write(*,*) 'LAB:yeta',yeta(i),xeta_cm(i),ycm_lab
c            write(*,*) 'LAB:yphi',yphi(i),xphi_cm(i),phi_r+pi
            yphi(i)=xphi_cm(i)+pi+phi_r
c            write(*,*) 'LAB:yphi',yphi(i),xphi_cm(i),phi_r+pi
         ELSE
            yeta(i)=0.
            yphi(i)=0.
         ENDIF
         IF (lp) THEN
            write(*,*) 'LAB:',ykt(i),yeta(i),yphi(i)
         ENDIF
      ENDDO
c     At this stage, ykt(*), yeta(*), yphi(*) are the moduli of
c     transverse momenta, the pseudorapidities and the azimuthal angles
c     of the outgoing partons in the HERA lab frame. 

c     Start filling the hepevt prime common.
c     --------------------------------------


      nhep=0
      DO loop = 1, 7
                  
         IF (loop.EQ.1) THEN
            nhep=nhep+1
            idhep(nhep) = 11
            isthep(nhep) = 0
            jmohep(1,nhep) = 0 
            jmohep(2,nhep) = 0
            jdahep(1,nhep) = 3
            jdahep(2,nhep) = 4
            phep(1,nhep) = 0.D0
            phep(2,nhep) = 0.D0
            phep(3,nhep) =-e_e
            phep(4,nhep) = e_e
            phep(5,nhep) = 0.D0
         ELSE IF (loop.EQ.2) THEN
            nhep=nhep+1
            idhep(nhep) = 2212
            isthep(nhep) = 0
            jmohep(1,nhep) = 0 
            jmohep(2,nhep) = 0
            jdahep(1,nhep) = 5
            jdahep(2,nhep) = 6
            phep(1,nhep) = 0.D0
            phep(2,nhep) = 0.D0
            phep(3,nhep) = e_p
            phep(4,nhep) = e_p
            phep(5,nhep) = 0.D0
         ELSE IF (loop.EQ.3) THEN
            nhep=nhep+1
            idhep(nhep) = 22
            isthep(nhep) = 0
            jmohep(1,nhep) = 1 
            jmohep(2,nhep) = 0
            jdahep(1,nhep) = 5
            jdahep(2,nhep) = 6
            phep(1,nhep) = 0.D0
            phep(2,nhep) = 0.D0
            phep(3,nhep) =-e_gamma 
            phep(4,nhep) = e_gamma
            phep(5,nhep) = 0.D0
         ELSE IF (loop.EQ.4) THEN
            nhep=nhep+1
            idhep(nhep) = 11
            isthep(nhep) = 1
            jmohep(1,nhep) = 1 
            jmohep(2,nhep) = 0
            jdahep(1,nhep) = 0
            jdahep(2,nhep) = 0
            phep(1,nhep) = 0.D0
            phep(2,nhep) = 0.D0
            phep(3,nhep) =-e_e+e_gamma
            phep(4,nhep) = e_e-e_gamma
            phep(5,nhep) = 0.D0
         ELSE  
            i=loop-4
            IF (ykt(i).GT.0.000001) THEN
               IF (lp) write(*,*) 'ykt(',i,')=',ykt(i)
               nhep=nhep+1
               idhep(nhep) = 0
               isthep(nhep) = 1
               jmohep(1,nhep) = 2 
               jmohep(2,nhep) = 3
               jdahep(1,nhep) = 0
               jdahep(2,nhep) = 0
               phep(1,nhep) = ykt(i)*cos(yphi(i)) 
               phep(2,nhep) = ykt(i)*sin(yphi(i)) 
*               theta=2.*atan(exp(-yeta(i)))
*               phep(3,nhep) = ykt(i)/tan(theta)
*               phep(4,nhep) = ykt(i)/sin(theta)
               phep(3,nhep) = 0.5*ykt(i)*(exp(yeta(i))-exp(-yeta(i)))
               IF (yeta(i).LT.0) THEN
                  phep(3,nhep) = -1.*ABS(phep(3,nhep))
               ENDIF
               phep(4,nhep) = SQRT(phep(1,nhep)**2+phep(2,nhep)**2
     &              +phep(3,nhep)**2)
               phep(5,nhep) = 0.D0
            ENDIF
         ENDIF
      
         DO ivec = 1, 4
            vhep(ivec,nhep) = 0.D0
         ENDDO

         ihchrg(nhep) = HZLCHGE(idhep(nhep))

      ENDDO
            
      DO loop=1,nhep
         IF (lp) THEN
            WRITE(6,*) loop,' phep= ',(phep(i,loop),i=1,4)
         ENDIF
      ENDDO

      IF (lp) THEN
         write(6,*) ' Number of particles nhep = ',nhep
      ENDIF

#endif
      RETURN
      END










