	subroutine hzc88172b(iflag,avrg)
*****************************************************************************
*       
*       Make plots for UA5 paper on minimum bias events.
*       This particular code produces KNO distributions.
*       ( Z. Phys. C 43, 357 (1989) ) 
*       
*       This sub-routine generates 13 histograms: 6 containing experimental
*       data and 7 with MC generated distributions.
*       
*       Histogram list:
*       
*       1 - KNO distribution - NSD events at 900 GeV (UA5 data)  
*           * Note: it uses non-KNO variable for the x axis!
*       2 - KNO distribution - NSD events at 900 GeV (UA5 data)  
*       3 - KNO distribution - NSD events at 900 GeV (UA5 data)  
*           * Note: it uses non-KNO variable for the x axis!
*           In this case, we use x=n/<n_1> (multi parton interaction
*  	    variable.
*       4 - dN_chg/deta at eta=0 - NSD events at 900 GeV (UA5 data)  
*       5 - <n_chg> - NSD events at 900 GeV (UA5 data)  
*           * Note: 4 and 5 contain just a single point.
*       6 - dN_chg/deta (UA5 eta range) - NSD events at 900 GeV (UA5 data)  
*       
*       10 - dN_chg/deta (full eta range) - NSD events at 900 GeV (MC)  
*       11 - KNO distribution - NSD events at 900 GeV (MC)  
*           * Note: it uses non-KNO variable for the x axis!
*       12 - KNO distribution - NSD events at 900 GeV (MC)  
*       13 - KNO distribution - NSD events at 900 GeV (MC)  
*           * Note: it uses non-KNO variable for the x axis!
*           In this case, we use x=n/<n_1> (multi parton interaction
*  	    variable.
*       14 - dN_chg/deta at eta=0 - NSD events at 900 GeV (MC)  
*       15 - <n_chg> - NSD events at 900 GeV (MC)  
*           * Note: 4 and 5 contain just a single point.
*       16 - dN_chg/deta (UA5 eta range) - NSD events at 900 GeV (MC)  
*       
*       
*       
*       iflag = 1,2 or 3 depending on wether it is the initialization,
*       processing or termination phase (respectively)
*       
*       Arthur Moraes - University of Sheffield - 09/12/2002.
*       Adapted for generator independence - Jon Butterworth
*       
*****************************************************************************
	
        IMPLICIT NONE

#include "hepevtp.inc"
#include "hzfunc.inc"
        
*       *********************************************************     
*       Variable Declaration     
*       *********************************************************      
	
	INTEGER iflag,intflag      
        INTEGER I, N, ntot, nchev, nrap0, events
        REAL avsel, avsele, rap, avn1, ecm, rap0, rap0e, ndata
        REAL factr10, factr11, factr12, factr13, factr16
	DOUBLE PRECISION avrg
        
        INTEGER NPAD, K, l1, l2
        DOUBLE PRECISION P, V, HI
	REAL P900(55), Z900(55), X900(55), KNO(54),EKNO(54) 
	REAL S900(2), ETA0(1), EETA0(1), avn(1), eavn(1)
	REAL etabin(20), dndeta(19), edndeta(19)
        
	SAVE ntot, nrap0, events
        
C...The event record. Particle code (pdg)
c      COMMON/PYJETS/N,NPAD,K(4000,5),P(4000,5),V(4000,5)
      
	intflag = iflag
	ndata = 35.6		! av. chgd. multiplicity (data)
	ecm = 900.0		! collision energy (GeV)
	
	
        DATA P900/1.0,7.0,9.0,11.0,13.0,15.0,
     &    17.0,19.0,21.0,23.0,25.0,27.0,29.0,31.0,
     &    33.0,35.0,37.0,39.0,41.0,43.0,45.0,47.0,
     &    49.0,51.0,53.0,55.0,57.0,59.0,61.0,63.0,
     &    65.0,67.0,69.0,71.0,73.0,75.0,77.0,79.0,
     &    81.0,83.0,85.0,87.0,89.0,91.0,93.0,95.0,
     &    97.0,99.0,101.0,103.0,105.0,109.0,113.0,
     &    121.0,137.0 /
	
	
        DATA KNO/0.007,0.014,0.018,0.025,0.033,0.041,0.050,
     &    0.053,0.057,0.056,0.054,0.050,0.047,0.042,0.039,
     &    0.036,0.033,0.030,0.028,0.025,0.023,0.021,0.020,
     &    0.019,0.017,0.016,0.015,0.014,0.013,0.012,0.011,
     &    0.010,0.009,0.008,0.007,0.007,0.006,0.005,0.004,
     &    0.004,0.003,0.0029,0.0025,0.0022,0.0018,0.0015,0.0013,
     &    0.0011,0.0009,0.0008,0.0012,0.0008,0.0009,0.0008 / 
	
	DATA EKNO/0.009,0.002,0.002,0.003,0.003,0.003,0.004,
     &    0.004,0.004,0.004,0.004,0.004,0.004,0.003,0.003,
     &    0.003,0.003,0.003,0.003,0.003,0.003,0.002,0.002,
     &    0.002,0.002,0.002,0.002,0.002,0.002,0.002,0.002, 
     &    0.002,0.002,0.002,0.001,0.001,0.001,0.001,0.001,
     &    0.001,0.001,0.0009,0.0008,0.0008,0.0007,0.0007,0.0006,
     &    0.0006,0.0006,0.0005,0.0006,0.0005,0.0005,0.0005 / 
	
        
	DATA etabin/0.000,0.250,0.500,0.750,1.000,1.250,
     &    1.500,1.750,2.000,2.250,2.500,2.750,3.000,3.250,
     &    3.500,3.750,4.000,4.250,4.500,4.750 / 
	
	
	DATA dndeta/3.48,3.38,3.52,3.68,3.71,3.86,
     &    3.76,3.66,3.72,3.69,3.56,3.41,3.15,3.09,
     &    2.74,2.73,2.32,1.99,1.69 / 
	
	DATA edndeta/0.07,0.07,0.07,0.07,0.07,0.07,
     &    0.07,0.07,0.07,0.07,0.07,0.07,0.07,0.08,
     &    0.08,0.09,0.09,0.10,0.13 / 
	
	
	DATA S900/899,901 /
	
	DATA ETA0/3.490 /
	DATA EETA0/0.13 /
	
	DATA avn/35.6 /
	DATA eavn/0.2 /
	
	avn1 = -7.3 + 2.56*log(ecm**2.d0) ! average multiplicity due to a 
	               			  ! single parton-parton scattering.
	
*       *** Initialisation run ***
*       *** Create directories ***
	  
	IF (intflag.eq.1) THEN
          
*       The average charged particle multiplicity in NSD events at 900
*       GeV is 35.6 . We now calculate the KNO variable Z900.
	  	 
	  do l1=1,55
	    Z900(l1)=P900(l1)/35.6
	    X900(l1)=P900(l1)/avn1
	  enddo
	  
	  do l2=1,54
	    KNO(l2)=KNO(l2)*ndata
	    EKNO(l2)=EKNO(l2)*ndata
	  enddo
	  
	  CALL HCDIR('//HISTO',' ')
	  CALL HMDIR('c88172b','S')
	  CALL HCDIR('//PAWC',' ')
	  CALL HMDIR('c88172b','S')
	  
	  CALL HBOOKB(1
     &      ,'KNO distribution (NSD at 900 GeV) - 
     &      UA5 data: non-KNO variables',54
     &      ,P900,0.0)
	  CALL HBOOKB(2
     &      ,'KNO distribution (NSD at 900 GeV) 
     &      - UA5 data: KNO variables',54
     &      ,Z900,0.0)
	  CALL HBOOKB(3
     &      ,'KNO (NSD at 900 GeV) - UA5 data: x=n/<n1> 
     &      (m.p.i. variable)',54,X900,0.0) 
	  CALL HBOOKB(4
     &      ,'Charged particle density at eta=0 (NSD at 900 GeV)
     &      - UA5 data',1,S900,0.0)
	  CALL HBOOKB(5
     &      ,'Average chg particle multiplicity (NSD at 900 GeV) 
     &      - UA5 data',1,S900,0.0) 
	  CALL HBOOKB(6
     &      ,'Charged particle density - dN_ch/deta - UA5 data',19
     &      ,etabin, 0.0)
	  
	  CALL HBOOK1(10
     &      ,'Charged particle density - dN_ch/deta - MC full eta range'
     &      ,60, -15.0,15.0,0.0)
	  
          CALL HBOOKB(11,'KNO - NSD events (non-KNO variables)',
     &      54,P900, 0.0) 
	  
          CALL HBOOKB(12,'KNO - NSD events (KNO variables)',54,Z900,0.0)  
	  CALL HBOOKB(13
     &      ,'KNO (NSD at 900 GeV): x=n/<n1> (m.p.i. variable)',54
     &      ,X900,0.0) 
	  CALL HBOOKB(14
     &      ,'Charged particle density at eta=0 (NSD at 900 GeV)',
     &      1,S900,0.0)
	  CALL HBOOKB(15
     &      ,'Average chg particle multiplicity (NSD at 900 GeV)',
     &      1,S900,0.0) 
	  CALL HBOOKB(16
     &      ,'Charged particle density - dN_ch/deta - (NSD at 900 GeV)'
     &      ,19,etabin,0.0)
	  
                    
           ntot = 0             ! Counter - all particles
           rap0 = 0.0
	   events = 0		! Counter - number of events
	   nrap0 = 0		! Counter for particle density at eta=0
           
*	   write(*,*)'this is the avergage:',avrg,'   n1',avn1
	   
	  CALL HCDIR('//HISTO/c88172b',' ')
	  CALL HCDIR('//PAWC/c88172b',' ')
	  	  
	  
*       *** Loop over particles in the event ***
	  
	  
        ELSE IF (intflag.eq.2) THEN
          	  
	  CALL HCDIR('//PAWC/c88172b',' ')
          
	  events = events + 1	! event counter
          nchev = 0             ! selected charged particles counter (per event)
	  
      DO 150 I=1,NHEP
        
	   if(HZLCHGE(IDHEP(I)).ne.0)then
	      if((IDHEP(I).eq.11).or.(IDHEP(I).eq.-11).or. 
     &  	   (IDHEP(I).eq.13).or.(IDHEP(I).eq.-13).or.
     &  	   (IDHEP(I).eq.15).or.(IDHEP(I).eq.-15).or.
     &  	   (IDHEP(I).eq.2212).or.(IDHEP(I).eq.-2212).or.
     &  	   (IDHEP(I).eq.211).or.(IDHEP(I).eq.-211).or.
     &  	   (IDHEP(I).eq.321).or.(IDHEP(I).eq.-321).or.
     &  	   (IDHEP(I).gt.3000).or.(IDHEP(I).lt.-3000))then

          nchev = nchev + 1
          rap = hzeta(I)	! Pseudorapidity.
	  call HF1(10,real(rap),1.0)
	  call HFILL(16,real(rap),0.0,1.0)	
	  
	  if((rap.gt.-0.05).and.(rap.lt.0.05))then
	    nrap0 = nrap0 + 1
	  endif
	  
	    
	   endif
	 endif          
 
 150     CONTINUE
         
         ntot = ntot + nchev
         

        
	call HFILL(11,real(nchev),0.0,1.0)		
	call HFILL(12,real(nchev/avrg),0.0,1.0)	
	call HFILL(13,real(nchev/avn1),0.0,1.0)	
	
*       *** Third section: produce output and end. ***
      
	ELSE IF (intflag.eq.3) THEN
	  
	  CALL HCDIR('//PAWC/c88172b',' ')
	  
          avsel = ntot*1.d0/events
	  rap0 = nrap0*1.d0/(events*0.1)
          avsele = sqrt(ntot*1.d0)/events
	  rap0e = sqrt(nrap0*1.d0)/(events*0.1)
        
	  write(*,*)rap0
	  
        CALL HPAK(1,KNO)
        CALL HPAKE(1,EKNO)
        CALL HPAK(2,KNO)
        CALL HPAKE(2,EKNO)
        CALL HPAK(3,KNO)
        CALL HPAKE(3,EKNO)
        CALL HPAK(4,ETA0)
        CALL HPAKE(4,EETA0)
        CALL HPAK(5,avn)
        CALL HPAKE(5,eavn)
        CALL HPAK(6,dndeta)
        CALL HPAKE(6,edndeta)
	  
	CALL HPAK(14,rap0)
	CALL HPAKE(14,rap0e)
	CALL HPAK(15,avsel)
	CALL HPAKE(15,avsele)
	  
	WRITE(*,*)'                                     '
	WRITE(*,*)'Number of selected charged particles: ',avsel
	WRITE(*,*)'                                     '
      
      
C...Histograms.
      
*          write(*,*)'Number of generated events:  ',events

	factr10 = 1.d0/(events * 0.5)
	factr11 = 1.d0*avsel/(events * 1.d0)
	factr12 = 1.d0*avsel/(events * 1.d0)
	factr13 = 1.d0*avsel/(events * 1.d0)
	  
	factr16 = 1.d0/(events * 0.250)
	  
	CALL HOPERA(10,'+E',10,10,real(factr10),0.)      
	CALL HOPERA(11,'+E',11,11,real(factr11),0.)
	CALL HOPERA(12,'+E',12,12,real(factr12),0.)
	CALL HOPERA(13,'+E',13,13,real(factr13),0.)
	CALL HOPERA(16,'+E',16,16,real(factr16),0.)
*          CALL HRPUT(0,'histos.jets','N')
          
	ENDIF
        
	RETURN

	END 
        
