	subroutine hzprt154247(iflag,avrg)
*****************************************************************************
*       
*       Make plots for UA5 paper on minimum bias events.
*       This particular code produces KNO distributions.
*       (Phys Rep 154(5,6) 247-383, 1987) 
*       
*       This sub-routine generates 11 histograms: 5 containing experimental
*       data and 6 with MC generated distributions.
*       
*       Histogram list:
*       
*       1 - KNO distribution - NSD events at 546 GeV (UA5 data)  
*           * Note: it uses non-KNO variable for the x axis!
*       2 - KNO distribution - NSD events at 546 GeV (UA5 data)  
*       3 - KNO distribution - NSD events at 546 GeV (UA5 data)  
*           * Note: it uses non-KNO variable for the x axis!
*           In this case, we use x=n/<n_1> (multi parton interaction
*  	    variable.
*       4 - dN_chg/deta at eta=0 - NSD events at 546 GeV (UA5 data)  
*       5 - <n_chg> - NSD events at 546 GeV (UA5 data)  
*           * Note: 4 and 5 contain just a single point.
*       
*       10 - dN_chg/deta (full eta range) - NSD events at 546 GeV (MC)  
*       11 - KNO distribution - NSD events at 546 GeV (MC)  
*           * Note: it uses non-KNO variable for the x axis!
*       12 - KNO distribution - NSD events at 546 GeV (MC)  
*       13 - KNO distribution - NSD events at 546 GeV (MC)  
*           * Note: it uses non-KNO variable for the x axis!
*           In this case, we use x=n/<n_1> (multi parton interaction
*  	    variable.
*       14 - dN_chg/deta at eta=0 - NSD events at 546 GeV (MC)  
*       15 - <n_chg> - NSD events at 546 GeV (MC)  
*           * Note: 4 and 5 contain just a single point.
*       
*       
*       
*       iflag = 1,2 or 3 depending on wether it is the initialization,
*       processing or termination phase (respectively)
*       
*       Arthur Moraes - University of Sheffield - 15/11/2002.
*       Adapted for generator independence - Jon Butterworth
*       
*****************************************************************************
	
        IMPLICIT NONE

#include "hepevtp.inc"
#include "hzfunc.inc"
        
*       *********************************************************     
*       Variable Declaration     
*       *********************************************************      
	
	INTEGER iflag,intflag      
        INTEGER I, J, K, ntot, nchev, nrap0, events
	INTEGER nrap0ev
        REAL avsel, avsele, rap, avn1, ecm, rap0, rap0e
        REAL factr10, factr11, factr12, factr13
	DOUBLE PRECISION avrg
        
        INTEGER NPAD, l1, l2
        DOUBLE PRECISION HI
	REAL P546(48), Z546(48), X546(48), KNO(47),EKNO(47) 
	REAL S546(2), ETA0(1), EETA0(1), avn(1), eavn(1)

	REAL P546W(120),xup,xdown,weight
        
	SAVE ntot, nrap0, events
        
C...The event record. Particle code (pdg)
c      COMMON/PYJETS/N,NPAD,K(4000,5),P(4000,5),V(4000,5)
      
	intflag = iflag
	ecm = 546.0		! collision energy (GeV)
	
        DATA P546/1.0,3.0,5.0,7.0,9.0,11.0,13.0,15.0,
     &    17.0,19.0,21.0,23.0,25.0,27.0,29.0,31.0,
     &    33.0,35.0,37.0,39.0,41.0,43.0,45.0,47.0,
     &    49.0,51.0,53.0,55.0,57.0,59.0,61.0,63.0,
     &    65.0,67.0,69.0,71.0,73.0,75.0,77.0,79.0,
     &    81.0,85.0,89.0,93.0,97.0,101.0,111.0,121.0 / 
	
	
        DATA KNO/0.0794,0.2264,0.3587,0.5733,0.9526,1.3318,1.5611,
     &    1.6376,1.6640,1.7317,1.7875,1.7287,1.6199,1.4876,1.3259,
     &    1.1995,1.0966,1.0114,0.9819,0.9437,0.8526,0.7468,0.6115,
     &    0.5027,0.4586,0.4116,0.3646,0.3205,0.2793,0.2440,0.2117,
     &    0.1852,0.1617,0.1411,0.1176,0.0941,0.0706,0.0559,0.0441,
     &    0.0382,0.0326,0.0247,0.0173,0.0099,0.0044,0.0015,0.0015 /  
	
	DATA EKNO/0.0235,0.0382,0.0412,0.0499,0.0617,0.0735,0.0764,
     &    0.0794,0.0794,0.0794,0.0823,0.0794,0.0764,0.0735,0.0706,
     &    0.0677,0.0647,0.0617,0.0588,0.0588,0.0559,0.0529,0.0470,
     &    0.0441,0.0412,0.0382,0.0353,0.0353,0.0323,0.0294,0.0265,
     &    0.0265,0.0235,0.0235,0.0206,0.0176,0.0176,0.0147,0.0118,
     &    0.0118,0.0076,0.0068,0.0056,0.0044,0.0029,0.0012,0.0012 /
	

	DATA S546/545,547 /
	
	DATA ETA0/3.05 /
	DATA EETA0/0.03 /
	
	DATA avn/29.4 /
	DATA eavn/0.3 /
	
	avn1 = -7.3 + 2.56*log(ecm**2.d0) ! average multiplicity due to a 
	               			  ! single parton-parton scattering.
	
*       *** Initialisation run ***
*       *** Create directories ***
	  
	IF (intflag.eq.1) THEN

*       Fill array of weights to compensate for bin widths
*       Index is multiplicity (nchev). Value is 1/(bin width) for corresponding bin.
	   xdown = P546(1)
	   DO I = 2, 48
	      xup = P546(I)
	      DO J = int(xdown+0.5), int(xup-0.5)
		 P546W(J) = 2.0 / (xup-xdown)
	      ENDDO
	      xdown = xup
	   ENDDO

*       The average charged particle multiplicity in NSD events at 546
*       GeV is 29.4 . We now calculate the KNO variable Z546.
	  	 
	  do l1=1,48
	    Z546(l1)=P546(l1)/29.4
	    X546(l1)=P546(l1)/avn1
	  enddo
	  
	  
	  CALL HCDIR('//HISTO',' ')
	  CALL HMDIR('ua5546','S')
	  CALL HCDIR('//PAWC',' ')
	  CALL HMDIR('ua5546','S')
	  
	  CALL HBOOKB(1
     &      ,'KNO distribution (NSD at 546 GeV) - 
     &      UA5 data: non-KNO variables',47
     &      ,P546,0.0)
	  CALL HBOOKB(2
     &      ,'KNO distribution (NSD at 546 GeV) - 
     &      UA5 data: KNO variables',47
     &      ,Z546,0.0)
	  CALL HBOOKB(3
     &      ,'KNO (NSD at 546 GeV) - UA5 data: x=n/<n1> 
     &      (m.p.i. variable)',47,X546,0.0) 
	  CALL HBOOKB(4
     &      ,'Charged particle density at eta=0 (NSD at 546 GeV) 
     &      - UA5 data',1,S546,0.0)
	  CALL HBOOKB(5
     &      ,'Average chg particle multiplicity (NSD at 546 GeV) 
     &      - UA5 data',1,S546,0.0) 
	  CALL HBOOK1(10,'Charged particle density - dN_ch/deta',60,-15
     &      .0,15.0,0.0)
	  
          CALL HBOOKB(11,'KNO - NSD events (non-KNO variables)',47,P546
     &      ,0.0) 
	  
          CALL HBOOKB(12,'KNO - NSD events (KNO variables)',47,Z546,0.0)  
	  CALL HBOOKB(13
     &      ,'KNO (NSD at 546 GeV): x=n/<n1> (m.p.i. variable)',47
     &      ,X546,0.0) 
          CALL HBOOKB(14
     &      ,'Charged particle density at eta=0 (NSD at 546 GeV)',
     &      1,S546,0.0)
          CALL HBOOKB(15
     &      ,'Average chg particle multiplicity (NSD at 546 GeV)',
     &      1,S546,0.0) 

*       Need to calculate errors correctly for histograms filled with weights
	  CALL HBARX(11)
	  CALL HBARX(12)
	  CALL HBARX(13)
                    
           ntot = 0             ! Counter - all particles
           rap0 = 0.0
	   events = 0		! Counter - number of events
	   nrap0 = 0		! Counter for particle density at eta=0
	   
*	   write(*,*)'this is the avergage:',avrg,'   n1',avn1
	   
	  CALL HCDIR('//HISTO/ua5546',' ')
	  CALL HCDIR('//PAWC/ua5546',' ')
	  	  
	  
*       *** Loop over particles in the event ***
	  
	  
        ELSE IF (intflag.eq.2) THEN
          	  
	   CALL HCDIR('//PAWC/ua5546',' ')
	   
	   events = events + 1	! event counter
	   nchev = 0		! selected charged particles counter (per event)
	   nrap0ev = 0
	  
	   DO 150 I=1,NHEP
	      
   	    if(HZLCHGE(IDHEP(I)).ne.0)then
	      if((IDHEP(I).eq.11).or.(IDHEP(I).eq.-11).or. 
     &   	   (IDHEP(I).eq.13).or.(IDHEP(I).eq.-13).or.
     &  	   (IDHEP(I).eq.15).or.(IDHEP(I).eq.-15).or.
     &  	   (IDHEP(I).eq.2212).or.(IDHEP(I).eq.-2212).or.
     &  	   (IDHEP(I).eq.211).or.(IDHEP(I).eq.-211).or.
     &  	   (IDHEP(I).eq.321).or.(IDHEP(I).eq.-321).or.
     &  	   (IDHEP(I).gt.3000).or.(IDHEP(I).lt.-3000))then
		 
		 nchev = nchev + 1
c       rap = pyp(I,19)	! Pseudorapidity.
		 rap = hzeta(I)	! Pseudorapidity.
		 call HF1(10,real(rap),1.0)
		 
		 if((rap.gt.-0.05).and.(rap.lt.0.05))then
		    nrap0 = nrap0 + 1
		    nrap0ev = nrap0ev + 1
		 endif
	  
		 
	      endif
	    endif
           
 150	   CONTINUE
         
         ntot = ntot + nchev
         

	 IF (nchev.ge.0.and.nchev.le.120) THEN
	    weight = P546W(nchev)
	 ELSE
	    weight = 1.0
	 ENDIF
	call HFILL(11,real(nchev),0.0,1.0)		
	call HFILL(12,real(nchev/avrg),0.0,1.0)	
	call HFILL(13,real(nchev/avn1),0.0,1.0)	
        call HFILL(14,ecm,real(nrap0ev),1.)
        call HFILL(15,ecm,real(nchev),1.)

*       *** Third section: produce output and end. ***
      
	ELSE IF (intflag.eq.3) THEN
	  
	  CALL HCDIR('//PAWC/ua5546',' ')
	  
          avsel = ntot*1.d0/events
	  rap0 = nrap0*1.d0/(events*0.1)
          avsele = sqrt(ntot*1.d0)/events
	  rap0e = sqrt(nrap0*1.d0)/(events*0.1)
        
	  write(*,*)rap0
	  
	  CALL HPAK(1,KNO)
	  CALL HPAKE(1,EKNO)
	  CALL HPAK(2,KNO)
	  CALL HPAKE(2,EKNO)
	  CALL HPAK(3,KNO)
	  CALL HPAKE(3,EKNO)
	  CALL HPAK(4,ETA0)
	  CALL HPAKE(4,EETA0)
	  CALL HPAK(5,avn)
	  CALL HPAKE(5,eavn)
          
          CALL HPAK(14,rap0)
          CALL HPAKE(14,rap0e)
          CALL HPAK(15,avsel)
          CALL HPAKE(15,avsele)
	  
	  WRITE(*,*)'                                     '
	  WRITE(*,*)'Number of selected charged particles: ',avsel
	  WRITE(*,*)'                                     '
      
      
C...Histograms.
      
*          write(*,*)'Number of generated events:  ',events

	  factr10 = 1.d0/(events * 0.5)
          factr11 = 1.d0*avsel/(events * 1.d0)
          factr12 = 1.d0*avsel/(events * 1.d0)
          factr13 = 1.d0*avsel/(events * 1.d0)
      
          CALL HOPERA(10,'+E',10,10,real(factr10),0.)      
	  CALL HOPERA(11,'+E',11,11,real(factr11),0.)
	  CALL HOPERA(12,'+E',12,12,real(factr12),0.)
	  CALL HOPERA(13,'+E',13,13,real(factr13),0.)
*          CALL HRPUT(0,'histos.jets','N')
          
	ENDIF
        
	RETURN

	END 
        
