	subroutine hzf89201e(iflag,avrg)
*****************************************************************************
*       
*       Generates plots for dN/deta (minimum bias events)
*       comparing MC to CDF data.
*       ( Phys. Rev. D 41, 2330 (1990), FNAL-PUB 89-201-E ) 
*
*	This sub-routine generates 7 histograms: 3 containing
*       experimental data and 4 with MC generated distributions.
*       
*       Histogram list:
*       
*       -4 - dN_chg/deta at eta=0 - NSD events at 1.8 TeV (CDF data)  
*       -5 - <n_chg> - NSD events at 1.8 TeV (Tevatron data)  
*            * Note: 4 and 5 contain just a single point.
*       -6 - dN_chg/deta (CDF eta range) - NSD events at 1.8 TeV (CDF data)  
*       
*       10 - dN_chg/deta (full eta range) - NSD events at 1.8 TeV (MC)  
*
*        4 - dN_chg/deta at eta=0 - NSD events at 1.8 TeV (MC)  
*        5 - <n_chg> - NSD events at 1.8 TeV (MC)  
*           * Note: 4 and 5 contain just a single point.
*        6 - dN_chg/deta (CDF eta range) - NSD events at 1.8 TeV (MC)  
*       
*       
*       
*       iflag = 1,2 or 3 depending on wether it is the initialization,
*       processing or termination phase (respectively)
*       
*       Arthur Moraes - University of Sheffield - 24/06/2003.
*       
*****************************************************************************
	
        IMPLICIT NONE

#include "hepevtp.inc"
#include "hzfunc.inc"
        
*       *********************************************************     
*       Variable Declaration     
*       *********************************************************      
	
	INTEGER iflag,intflag      
        INTEGER I, N, ntot, nchev, nrap0, events
        REAL avsel, avsele, rap, ecm, rap0, rap0e
        REAL factr10, factr6
	DOUBLE PRECISION avrg
	
        INTEGER NPAD, K, l1, l2, evloop, navrg, mev
	INTEGER j
        DOUBLE PRECISION P, V, HI, HIE
	REAL S1800(2), ETA0(1), EETA0(1), avn(1), eavn(1)
	REAL etabin(10), dndeta(9), edndeta(9)
	REAL etahis(9), hrap(9), hrape(9)
	
	SAVE ntot, nrap0, events

C...The event record. Particle code (pdg)
      COMMON/PYJETS/N,NPAD,K(4000,5),P(4000,5),V(4000,5)
      
	intflag = iflag
	ecm = 1800.0		! collision energy (GeV)
	
	
        
	DATA etabin/0.000,0.25,0.50,0.75,1.00,1.50,
     &    2.00,2.50,3.00,3.50 /
	
	DATA dndeta/3.95,4.00,4.05,4.21,4.38,4.74,
     &    4.70,4.47,4.19 /    

	DATA edndeta/0.11,0.12,0.43,0.50,0.53,0.53,
     &    0.53,0.50,0.64 /
	
	DATA S1800/1799,1801 /
	
	DATA ETA0/3.95 /
	DATA EETA0/0.13 /
	
	DATA avn/45.81 /
	DATA eavn/0.77 /
	
*       *** Initialisation run ***
*       *** Create directories ***
	  
	IF (intflag.eq.1) THEN
          
	  CALL HCDIR('//HISTO',' ')
	  CALL HMDIR('f89201e','S')
	  CALL HCDIR('//PAWC',' ')
	  CALL HMDIR('f89201e','S')
	  
	  CALL HBOOKB(-4
     &      ,'Charged particle density at eta=0 (NSD at 1.8 TeV) 
     &      - CDF data',1,S1800,0.0)
	  CALL HBOOKB(-5
     &     ,'Average chg particle multiplicity (NSD at 1.8 GeV) - 
     &     CDF data',1,S1800,0.0) 
	  CALL HBOOKB(-6
     &      ,'Charged particle density - dN_ch/deta - CDF data',9
     &      ,etabin, 0.0)
	  
	  CALL HBOOK1(10
     &      ,'Charged particle density - dN_ch/deta - MC full eta range'
     &      ,60,-15.0,15.0,0.0)
	  
	  CALL HBOOKB(4
     &      ,'Charged particle density at eta=0 (NSD at 1.8 TeV)',
     &      1,S1800,0.0)
	  CALL HBOOKB(5
     &      ,'Average chg particle multiplicity (NSD at 1.8 TeV)',
     &      1,S1800,0.0) 
	  CALL HBOOKB(6
     &      ,'Charged particle density - dN_ch/deta - (NSD at 1.8 TeV)'
     &      ,9,etabin,0.0)
	  
                    
           ntot = 0             ! Counter - all particles
           rap0 = 0.0
	   events = 0		! Counter - number of events
	   nrap0 = 0		! Counter for particle density at eta=0
           
	   
*	   write(*,*)'this is the avergage:',avrg
	   
	  CALL HCDIR('//HISTO/f89201e',' ')
	  CALL HCDIR('//PAWC/f89201e',' ')
	  	  
	  
*       *** Loop over particles in the event ***
	  
	  
        ELSE IF (intflag.eq.2) THEN
          	  
	  CALL HCDIR('//PAWC/f89201e',' ')
          
	  events = events + 1	! event counter
c          write(*,*) 'events = ',events
          nchev = 0             ! selected charged particles counter (per event)
	  
      DO 150 I=1,NHEP

	if(HZLCHGE(IDHEP(I)).ne.0)then
	  if((IDHEP(I).eq.11).or.(IDHEP(I).eq.-11).or. 
     &    (IDHEP(I).eq.13).or.(IDHEP(I).eq.-13).or.
     &    (IDHEP(I).eq.15).or.(IDHEP(I).eq.-15).or.
     &    (IDHEP(I).eq.2212).or.(IDHEP(I).eq.-2212).or.
     &    (IDHEP(I).eq.211).or.(IDHEP(I).eq.-211).or.
     &    (IDHEP(I).eq.321).or.(IDHEP(I).eq.-321).or.
     &    (IDHEP(I).gt.3000).or.(IDHEP(I).lt.-3000))then

	    nchev = nchev + 1
	    rap = hzeta(I)	! Pseudorapidity.
	    call HF1(10,real(rap),1.0)
	    call HFILL(6,real(rap),0.0,1.0)	
	  
	    if((rap.gt.-0.05).and.(rap.lt.0.05))then
	      nrap0 = nrap0 + 1
	    endif

	  endif
	    
	endif
	
 150	CONTINUE
         
	ntot = ntot + nchev

*       *** Third section: produce output and end. ***
      
	ELSE IF (intflag.eq.3) THEN
	  
          write (*,*) 'Finishing: events = ',events

	  CALL HCDIR('//PAWC/f89201e',' ')
	  
          avsel = ntot*1.d0/events
	  rap0 = nrap0*1.d0/(events*0.1)
	  avsele = sqrt(ntot*1.d0)/events
	  rap0e = sqrt(nrap0*1.d0)/(events*0.1)
        
*	  write(*,*)rap0
	  
        CALL HPAK(-4,ETA0)
        CALL HPAKE(-4,EETA0)
        CALL HPAK(-5,avn)
        CALL HPAKE(-5,eavn)
        CALL HPAK(-6,dndeta)
        CALL HPAKE(-6,edndeta)
	  
	CALL HPAK(4,rap0)
	CALL HPAKE(4,rap0e)
	CALL HPAK(5,avsel)
	CALL HPAKE(5,avsele)

*...dN/deta plot has non-uniform bin sizes. The lines below
*...correct this.	

	do j=1,9
	  hrap(j) = HI(6,j)
	  hrape(j) = HIE(6,j)
	  if(j.lt.5) then
	     hrap(j) = hrap(j)*2.0 
	     hrape(j) = hrape(j)*2.0 
	  endif
	enddo
	CALL HPAK(6,hrap)
	CALL HPAKE(6,hrape)
	
	  
C...Histograms.
      
*          write(*,*)'Number of generated events:  ',events

	factr10 = 1.d0/(events * 0.5)
	factr6 = 1.d0/(events * 0.50)
	  
	CALL HOPERA(10,'+E',10,10,real(factr10),0.)      
	CALL HOPERA(6,'+E',6,6,real(factr6),0.)
	
*          CALL HRPUT(0,'histos.jets','N')
          
	ENDIF
        
	RETURN

	END 
        
