      subroutine hzh0010026(IFLAG)
****************************************************************************
*   Measurement of the inclusive differential cross section for Z bosons as  a
*   function of transverse momentum in anti-p p collisions at  s**(1/2) =
*   1.8-TeV (Dzero).
*
*   Reference:
*             Published in Phys.Lett.,B513,292-300,2001
*             e-Print Archive: hep-ex/0010026
*     This routine produces the W pT distribution at the Tevatron.  
*     Histogram -1 ---> the data from Table I and Fig.2
*                1 ---> the simulated results to compare with it.
*                2 ---> the normalisation values for histo 1
*     The normalisation of the graph will be meaningless unless Xsec and
*     Ntot are set before calling the termination routine.  (Xsec -
*     total cross section returned by MC) (Ntot - number of events
*     passed to this routine)
*
* Arguments: iflag=1 initialisation
*            iflag=1 filling
*            iflag=3 termination
*
* written by: Emily Nurse (UCL) on 10/03/2005
****************************************************************************
      IMPLICIT NONE

#include "hzfunc.inc"
#include "hzhbook.inc"
#include "hepevtp.inc"
#include "heracmn.inc"

******* declare variables
      Integer ihep,iflag,count
      Integer nbins/22/
      Real ptbin(23),pt(22),dsigdpt(22)
      Real stat(22),syst(22),error(22),norm(22),norme(22)
      Real wpt

******* private copies of Xsec and Ntot from HERACMN
      DOUBLE PRECISION mhxsec(2),mhntot(2)
      
****** bins used for Monte Carlo and data histograms
      DATA ptbin/0.,2.,4.,6.,8.,10.,12.,14.,16.,18.,20.,25.,30.,35.,
     + 40.,50.,60.,70.,80.,100.,120.,160.,200./

****** data points from Table I
******   dsigdpt
      DATA dsigdpt/109.37,205.91,171.28,133.62,103.30,77.58,63.66,
     + 47.88,37.72,30.65,22.02,13.94,9.47,6.84,3.95,1.81,1.15,0.75,
     + 0.313,0.084,0.044,0.0077/

******   stat. error on dsigdpt
      DATA stat/4.60,6.84,5.64,4.65,4.03,3.47,3.21,2.77,2.43,2.21, 
     + 1.23,0.93,0.73,0.63,0.36,0.24,0.21,0.18,0.059,0.029,0.012,  
     + 0.0054/ 

******   syst. error on dsigdpt
      DATA syst/10.64,22.80,9.16,9.81,7.17,7.15,4.18,4.03,2.50,1.60,
     + 1.11,0.98,0.79,0.52,0.31,0.23,0.25,0.21,0.091,0.018,0.014, 
     + 0.0045/

****** INITIALISATION ******
      IF (iflag.eq.1) THEN
****** make subdirectory in PAWC
       call hcdir('//PAWC',' ')
       call hmdir('hzh0010026','S')
****** make subdirectory in o/p file
       call hcdir('//HISTO',' ')
       call hmdir('hzh0010026','S')

****** book histograms:
******  Monte Carlo
      call hbookb(1,'1/sigma dsigma/d-pt(MC)',nbins,ptbin,0.)
******  normalisation values
      call hbookb(2,'normalisation values',nbins,ptbin,0.)
******  data
      call hbookb(-1,'1/sigma dsigma/d-pt(data)',nbins,ptbin,0.0)

      call hcdir('//HISTO/hzh0010026',' ')

      write(6,*)'**********************************************'
      write(6,*)'*  hzh0010026 called, histos will be output. *'
      write(6,*)'**********************************************'

******  LOOP OVER EVENTS ******
      ELSE IF(iflag.eq.2) THEN

******  move to the correct sub-directory in PAWC
       call hcdir('//PAWC/hzh0010026',' ')
******  loop through particles in event  and fill histo
        do ihep=1,nhep
******  pick out W's AFTER the parton shower.
*****   the W will decay to an electron and neutrino or a photon and neutrino
             if(abs(IDHEP(ihep)).eq.24.and.JDAHEP(1,ihep).ne.0) then
               if( (abs(IDHEP(JDAHEP(1,ihep))).eq.12.or.
     +              abs(IDHEP(JDAHEP(1,ihep))).eq.11.or.
     +              IDHEP(JDAHEP(1,ihep)).eq.22).and.
     +             (abs(IDHEP(JDAHEP(2,ihep))).eq.12.or.
     +              abs(IDHEP(JDAHEP(2,ihep))).eq.11.or.
     +              IDHEP(JDAHEP(2,ihep)).eq.22) ) then
                 wpt = sqrt(phep(1,ihep)**2+phep(2,ihep)**2)
                 call hfill(1,real(wpt),0.,wtx)     
               endif
             endif
         end do

******  TERMINATION ******
      ELSE IF(iflag.eq.3) THEN
******  Move to the correct PAW subdirectory
       call hcdir('//PAWC/hzh0010026',' ')

****** loop through histo bins 
      do  count=1,nbins
******  add stat. and syst. errors in quadrature
        error(count) = sqrt(stat(count)**2 + 
     +  syst(count)**2)
****** get normalisation values per bin, convert to pb from nb
        norm(count)=1000.*xsec/((ptbin(count+1)-ptbin(count))*Ntot)
        norme(count)=0.
      end do     

****** fill normalisation histogram
      call hpak(2,norm)
      call hpake(2,norme)
****** multiply Monte Carlo histo by normalisation histo
      call hopera(1,'*E',2,1,1.0,1.0) 
***** fill data histogram       
      call hpak(-1,dsigdpt)
      call hpake(-1,error)
      write(6,*) 'hzh0010026 X-section is ',xsec

      ENDIF

      RETURN
      END

