	Subroutine hzh9912022(iflag)
*****************************************************************************
*
* TEVATRON DIJET CDF NEED TO BE RUN WITH MERGING/SPLITTING 
* THRESHOLD SET TO 75% IN THE JET ALGORITHM
*
* iflag = 1,2 or 3 depending on wether it is the initialization,
*		processing or termination phase (respectively)
* 
* To change jetfinder use these values to call the initialisation phase.
* 1 + 10 * (jetfinder number)
*
* This routine produces two Monte Carlo graphs.
* id 10 is the normalised cross section in nb 
*		These graph will be
*		meaningless unless Xsec and Ntot are set before
*		calling the termination routine.
*		(Xsec - total cross section returned by MC)
*		(Ntot - number of events passed to this routine)
*
*       CROSS SECTIONS ARE IN PB. THE FINAL CROSS SECTION TO BE COMPARED
*       WITH THE DATA IS HISTOGRAM 76.
*
*       Phys.Rev.D61:091101,2000
*       hep-ex/9912022
*                    
* Author : Ivan Borozan 
*
*
*****************************************************************************
*
* HERA Tuning defined commons

	IMPLICIT NONE

#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
#include "hzjetcmn.inc"    
	DOUBLE PRECISION const1

	CHARACTER*6 jetf

        INTEGER NJET,ivx
c        DOUBLE PRECISION Jets(MAXHZJETS,8)
        DOUBLE PRECISION CONER,mtotal,mtotalUE
	SAVE CONER

	INTEGER myil,myip
	INTEGER loop,loop2,loop1,ISTAT,loop4,E_max(100)
	DOUBLE PRECISION phi(50),px(2),py(2),pz(2),ptsq(2),NRG,petite(50)
	DOUBLE PRECISION ppsq(2),energy(2),ivy(100),event,cs,mass(2)
	DOUBLE PRECISION rap(50),Et(50),Ei(50),Ri(50),
     &    Emax,secmax,k,rap2(2)
        
C       --- cuts arrays
	DOUBLE PRECISION rapcut2(2),Et2(2),rapcut3(2)
	SAVE RAPCUT2, RAPCUT3

C       --- private copies of Xsec and Ntot from HERACMN
	DOUBLE PRECISION mhxsec(2),mhntot(2)

	INTEGER intiflag, chjet, iproc, iflag,MAXEV
	SAVE chjet
C       --- define default jet algorithm
	DATA chjet/7/
	INTEGER hzflag
        
        Real pi
	REAL pttemp(12)
	INTEGER ivec,jvec,ICYCLE
	INTEGER p,j,i,l,m,ierr

        REAL norm(17),bins2(18),mass2(17),masserr(17),stater(17)
	REAL norme(17)
        REAL totalerr(17)
            
	Data bins2/180.0,198.0,217.0,241.0,265.0,292.0,321.0,353.0
     &      ,388.0,427.0,470.0,517.0,568.0,625.0,688.0,756.0,832.0
     &      ,915.0/
                     

c        write(*,*) Ntot
c        write(*,*) Xsec

c         CALL HBOOKB(66,'Normalisation values',17,bins2,0.0)

c       storing real data ******
          
	Data mass2/6.07d2,3.42d2,1.81d2,9.81d1,4.98d1,2.78d1,1.43d1,7
     &      .41d0,3.83d0,1.89d0,9.07d-1,4.50d-1,1.90d-1,7.42d-2,2.92d-2
     &      ,1.81d-2,3.57d-3/


        Data masserr/1.9424d1,1.4022d1,1.81,1.3734,8.964d-1,3.058d-1,2
     &      .002d-1,1.4079d-1,3.447d-2,2.268d-2,1.5419d-2,1.035d-2,6
     &      .27d-3,3.7842d-3,2.2484d-3,1.298d-3,7.14d-4/
        

c       storing systematic error ******

        stater(1)=1.1533d2  
        stater(2)=6.156d1
        stater(3)=3.1675d1
        stater(4)=1.71675d1
        stater(5)=8.964
        stater(6)=5.004
        stater(7)=2.6455
        stater(8)=1.3338
        stater(9)=7.4685d-1
        stater(10)=3.78d-1
        stater(11)=1.85935d-1
        stater(12)=9.675d-2
        stater(13)=4.37d-2
        stater(14)=1.7808d-2
        stater(15)=7.446d-3
        stater(16)=3.245d-3
        stater(17)=1.0353d-3


        intiflag=iflag
	iproc=2

C       --- choose jet finder
	
	IF ((intiflag.gt.10).and.(MOD(intiflag,10).eq.1)) THEN
	   chjet=INT(intiflag/10)
	   intiflag=MOD(intiflag,10)
	ENDIF
	
	IF (intiflag.eq.1) THEN
C       *** Initialisation run ***
C       *** Create directories ***

           CALL HCDIR('//HISTO',' ')
	   CALL HMDIR('h9912022','S')
	   CALL HCDIR('//PAWC',' ')
	   CALL HMDIR('h9912022','S')
	   

c$$$    c       open all the hbook's for generating the final histo for the
c$$$    c       paper number hep-ex/9912022

	   CALL HBOOKB(65,'Differential Dijet Mass Cross Section',17
     &  	,bins2,0.) 
	   
	   CALL HBOOKB(66,'Normalisation values',17,bins2,0.0) 
	   
	   CALL HBOOKB(75,'Differential Dijet Mass Xsec - UE(CDF)',17
     &  	,bins2,0.)

	   CALL HBOOKB(51,'Measured data dijet mass (CDF)',
     &       17,bins2,0.0)

	   CALL HIDOPT(0,'stat')
	   CALL HBARX(0)

	   CALL HCDIR('//HISTO/h9912022',' ')
	   CALL HCDIR('//PAWC/h9912022',' ')

c$$$    
c$$$    C       --- basic cuts for rapidity plots
           rapcut2(1)=-2.0
           rapcut2(2)=2.0
   	   const1=(2.0/3.0)
           rapcut3(1)=(-const1) 
           rapcut3(2)=(2.0/3.0)


C       --- standard cone radius is 1.0** change cone radius to 0.7

           CONER=0.7
	   
           CALL HZJETRAD(1,CONER) 
	   IF (CONER.lt.0.0) THEN
	      CONER=0.7
	   ENDIF
	   CALL HZJTNAME(chjet,jetf)
	   
	   WRITE(6,*)'**********************************************'
	   WRITE(6,*)'*   H9912022 called, histograms will be output. *'
	   WRITE(6,*)'*     Jet finder selected : ',jetf,'           *'
	   WRITE(6,1001)'*        with cone radius : ',CONER,'   *'
	   WRITE(6,*)'**********************************************'
	   
 1001	   FORMAT(A29,F5.2,A13)
	   
	ELSE IF (intiflag.eq.2) THEN

C       *** Filling runs ***
	   
C       *** Change directory to our graphs ***
	   
           CALL HCDIR('//PAWC/h9912022',' ')
	   
	   CALL hzjtfind(chjet,CONER,NJET,Jets)
	   

C       ******Initilasing  all  array to 0.

	   DO  m=1,2   
	      Et2(m)=0.0
	      rap2(m)=0.0
	      Px(m)=0.0
	      Py(m)=0.0
	      Pz(m)=0.0
	   END DO

	   IF (NJET.ge.2) then
	      
	      DO loop=1,2
		 mass(loop)=Jets(loop,8)
		 rap2(loop)=Jets(loop,1)  
		 Et2(loop)=Jets(loop,4)
		 Px(loop)=Jets(loop,5)
		 Py(loop)=Jets(loop,6)
		 Pz(loop)=Jets(loop,7)  
	      END DO 

C this will bin the events for the Differential Dijet Mass Xsec*****        

	      IF (((rap2(1).gt.rapcut2(1)).and.(rap2(1).lt
     &  	   .rapcut2(2)).and.(rap2(2).gt.rapcut2(1)).and
     &  	   .(rap2(2).lt.rapcut2(2))).and.((TANH((rap2(1)
     &  	   -rap2(2))/2.0).gt.rapcut3(1)).and.(TANH((rap2(1
     &  	   )-rap2(2))/2.0).lt.rapcut3(2)))) then
		 
		 mtotal=SQRT((Et2(1)+Et2(2))**2-(Px(1)+Px(2))**2
     &  	      -(Py(1)+Py(2))**2-(Pz(1)+Pz(2))**2) 

C       this is the Dijet with UE events subtracted
c       **************           

		 mtotalUE=SQRT((Et2(1)+Et2(2)+(2.0*0.5))**2-(Px(1)+Px(2))**2
     &  	      -(Py(1)+Py(2))**2-(Pz(1)+Pz(2))**2)
		 
		 CALL hfill(65,real(mtotal),0.,wtx)
		 CALL hfill(75,real(mtotalUE),0.,wtx)

	      END IF
	      
	   END IF
                   
c       this is the end of the Dijet plot ***********************************

        ELSE IF (intiflag.eq.3) THEN

	   write(*,*) 'finishing up cdf dij.'
C       *** Termination run ***
C       *** Change directory ***
	   CALL HCDIR('//PAWC/h9912022',' ')
	   

c       write(*,*) Ntot
c       write(*,*) Xsec

	   norm(1)=(1.0/((Ntot/(Xsec*1.d3))*18.0))
	   norm(2)=(1.0/((Ntot/(Xsec*1.d3))*19.0))
	   norm(3)=(1.0/((Ntot/(Xsec*1.d3))*24.0))          
	   norm(5)=(1.0/((Ntot/(Xsec*1.d3))*27.0))
	   norm(4)=(1.0/((Ntot/(Xsec*1.d3))*24.0))
	   norm(6)=(1.0/((Ntot/(Xsec*1.d3))*29.0))
	   norm(7)=(1.0/((Ntot/(Xsec*1.d3))*32.0))
	   norm(8)=(1.0/((Ntot/(Xsec*1.d3))*35.0))
	   norm(9)=(1.0/((Ntot/(Xsec*1.d3))*39.0))
	   norm(10)=(1.0/((Ntot/(Xsec*1.d3))*43.0))
	   norm(11)=(1.0/((Ntot/(Xsec*1.d3))*47.0))
	   norm(12)=(1.0/((Ntot/(Xsec*1.d3))*51.0))
	   norm(13)=(1.0/((Ntot/(Xsec*1.d3))*57.0))
	   norm(14)=(1.0/((Ntot/(Xsec*1.d3))*63.0))
	   norm(15)=(1.0/((Ntot/(Xsec*1.d3))*68.0))
	   norm(16)=(1.0/((Ntot/(Xsec*1.d3))*76.0))
	   norm(17)=(1.0/((Ntot/(Xsec*1.d3))*83.0))
	   
	   do loop4 = 1, 17
	      norme(loop4) = 0.
	   end do

c       this will store the above numbers in the histogram 

	   CALL HPAK(66,norm)
	   CALL HPAKE(66,norme)

c       this will plot the right cross section ***********        

	   CALL HOPERA(65,'*E',66,67,1.0,1.0)    
	   CALL HOPERA(75,'*E',66,76,1.0,1.0)           


C fills the histo with real data *****

           DO loop4=1,17
	      totalerr(loop4)=sqrt(masserr(loop4)**2+stater(loop4)**2)
	   END DO
        
C       CALL HBOOKB(51,'Real data',17,bins2,0.0)
	   
	   CALL HPAK(51,mass2)
	   CALL HPAKE(51,totalerr)        
           
C       *** Finish off histograms ***
	   
	   IF (Xsec.eq.0) THEN
	      PRINT*,
     &          'CDF Dijet: termination called with zero cross section'
	      PRINT*,
     &          '          cross section graph meaningless'
	      Xsec=1
	   ENDIF
	   IF (Ntot.eq.0) THEN
	      PRINT*,'CDF Dijet: termination called with no total events'
	      PRINT*,'          cross section graph meaningless'
	      Ntot=1
	   ENDIF
	   
C       --- store Xsec and Ntot for both direct and resolved events.
	   
	   mhXsec(iproc)=Xsec
	   mhntot(iproc)=Ntot

	ELSE

c       *** End ****

	   print*
     &  	,'CDF Inc: Please run routine with iflag set to 1,2 or 3'
	   
	ENDIF

	RETURN

	END 
