*****************************************************************************
	subroutine hzplb435453(iflag,avrg)
*****************************************************************************
*       
*       E735 paper on minimum bias events.
*       This particular code produces KNO distributions comparing
*	MC to the E735 data.
*       ( Phys. Lett. B 435, 453 (1998) ) 
*       
*	This sub-routine generates 7 histograms: 3 containing
*       experimental data and 4 with MC generated distributions.
*       
*       Histogram list:
*       
*       -1 - KNO distribution - NSD events at 1.8 TeV (E735 data)  
*            * Note: it uses non-KNO variable for the x axis!
*       -2 - KNO distribution - NSD events at 1.8 TeV (E735 data)  
*       -3 - KNO distribution - NSD events at 1.8 TeV (E735 data)  
*            * Note: it uses non-KNO variable for the x axis!
*            In this case, we use x=n/<n_1> (multi parton interaction
*  	     variable.
*       
*       10 - dN_chg/deta (full eta range) - NSD events at 1.8 TeV (MC)  
*
*        1 - KNO distribution - NSD events at 1.8 TeV (MC)  
*           * Note: it uses non-KNO variable for the x axis!
*        2 - KNO distribution - NSD events at 1.8 TeV (MC)  
*        3 - KNO distribution - NSD events at 1.8 TeV (MC)  
*           * Note: it uses non-KNO variable for the x axis!
*           In this case, we use x=n/<n_1> (multi parton interaction
*  	    variable.
*       
*       
*       iflag = 1,2 or 3 depending on wether it is the initialization,
*       processing or termination phase (respectively)
*       
*       Arthur Moraes - University of Sheffield - 24/06/2003.
*       
*****************************************************************************
	
        IMPLICIT NONE

#include "hepevtp.inc"
#include "hzfunc.inc"
        
*       *********************************************************     
*       Variable Declaration     
*       *********************************************************      
	
	INTEGER iflag,intflag      
        INTEGER I, N, ntot, nchev, nrap0, events
        REAL avsel, rap, avn1, ecm, rap0, ndata
        REAL factr10, factr1, factr2, factr3
	DOUBLE PRECISION avrg
	
        INTEGER NPAD, K, l1, l2, evloop, navrg, mev
	INTEGER j
        DOUBLE PRECISION P, V, pyp, HI
	REAL P18(129), Z18(129), X18(129), KNO(128),EKNO(128) 
	REAL avn(1), eavn(1)
	REAL etahis(9), hrap(9)

	SAVE ntot, nrap0, events
	
C...The event record. Particle code (pdg)
      COMMON/PYJETS/N,NPAD,K(4000,5),P(4000,5),V(4000,5)
      
	intflag = iflag
	ndata = 45.81		! av. chgd. multiplicity (data)
	ecm = 1800.0		! collision energy (GeV)
	
	
        DATA P18/1.0,3.0,5.0,7.0,9.0,11.0,13.0,15.0,17.0,19.0,21.0,
     &    23.0,25.0,27.0,29.0,31.0,33.0,35.0,37.0,39.0,41.0,
     &    43.0,45.0,47.0,49.0,51.0,53.0,55.0,57.0,59.0,61.0,
     &    63.0,65.0,67.0,69.0,71.0,73.0,75.0,77.0,79.0,81.0,
     &    83.0,85.0,87.0,89.0,91.0,93.0,95.0,97.0,99.0,101.0,
     &    103.0,105.0,107.0,109.0,111.0,113.0,115.0,117.0,119.0,
     &    121.0,123.0,125.0,127.0,129.0,131.0,133.0,135.0,137.0,
     &    139.0,141.0,143.0,145.0,147.0,149.0,151.0,153.0,155.0,
     &    157.0,159.0,161.0,163.0,165.0,167.0,169.0,171.0,173.0,
     &    175.0,177.0,179.0,181.0,183.0,185.0,187.0,189.0,191.0,
     &    193.0,195.0,197.0,199.0,201.0,203.0,205.0,207.0,209.0,
     &    211.0,213.0,215.0,217.0,219.0,221.0,223.0,225.0,227.0,
     &    229.0,231.0,233.0,235.0,237.0,239.0,241.0,243.0,245.0,
     &    247.0,249.0,251.0,253.0,255.0,257.0 /    
	
	
        DATA KNO/0.1747E0,0.2236E2,0.2239E2,0.1668E2,0.1609E2,
     &    0.2314E2,0.2369E2,0.2467E2,0.2813E2,0.3418E2,0.3543E2,
     &    0.3688E2,0.3506E2,0.3604E2,0.3302E2,0.3409E2,0.3173E2,
     &    0.2951E2,0.2809E2,0.2554E2,0.2498E2,0.2323E2,0.2229E2,
     &    0.2063E2,0.2000E2,0.1956E2,0.1938E2,0.1792E2,0.1777E2,
     &    0.1571E2,0.1568E2,0.1483E2,0.1422E2,0.1418E2,0.1304E2,
     &    0.1232E2,0.1168E2,0.1073E2,0.1018E2,0.1002E2,0.8899E1,
     &    0.8739E1,0.8209E1,0.7822E1,0.7651E1,0.7107E1,0.6585E1,
     &    0.5518E1,0.5829E1,0.5471E1,0.5016E1,0.4706E1,0.4365E1,
     &    0.3991E1,0.3776E1,0.3634E1,0.3158E1,0.3047E1,0.2836E1,
     &    0.2643E1,0.2353E1,0.2292E1,0.1945E1,0.1674E1,0.1713E1,
     &    0.1686E1,0.1574E1,0.1288E1,0.1151E1,0.1049E1,0.9726E0,
     &    0.8791E0,0.7724E0,0.7103E0,0.5733E0,0.5936E0,0.5342E0,
     &    0.4989E0,0.4023E0,0.3486E0,0.3436E0,0.2958E0,0.3059E0,
     &    0.2465E0,0.2213E0,0.2068E0,0.1719E0,0.1526E0,0.1212E0,
     &    0.1231E0,0.1082E0,0.8530E-1,0.7703E-1,0.7515E-1,0.5812E-1,
     &    0.5269E-1,0.4825E-1,0.4339E-1,0.3629E-1,0.3278E-1,0.2741E-1,
     &    0.1910E-1,0.1955E-1,0.1507E-1,0.1553E-1,0.1026E-1,0.9704E-2,
     &    0.1086E-1,0.8862E-2,0.5725E-2,0.6354E-2,0.4519E-2,0.4178E-2,    
     &    0.3216E-2,0.3401E-2,0.3640E-2,0.2714E-2,0.2221E-2,0.2295E-2,    
     &    0.1722E-2,0.1572E-2,0.1294E-2,0.1393E-2,0.1128E-2,0.8180E-3,    
     &    0.8552E-3,0.8784E-3,0.6769E-3 /    
	

	
	DATA EKNO/0.1038E3,0.2906E2,0.3269E2,0.1496E2,0.2016E2,0.1805E2,
     &   0.1109E2,0.1236E2,0.1048E2,0.8635E1,0.7995E1,0.7519E1,0.5159E1,
     &   0.3395E1,0.2884E1,0.2722E1,0.2491E1,0.2459E1,0.2139E1,0.1813E1,    
     &   0.1425E1,0.1477E1,0.1434E1,0.1166E1,0.1349E1,0.1269E1,0.1531E1,    
     &   0.1258E1,0.1362E1,0.1073E1,0.1221E1,0.9594E0,0.1083E1,0.1096E1,    
     &   0.8296E0,0.8285E0,0.9070E0,0.8474E0,0.6639E0,0.6611E0,0.6707E0,    
     &   0.5554E0,0.6139E0,0.4936E0,0.5201E0,0.6392E0,0.4166E0,0.5338E0,    
     &   0.4249E0,0.3694E0,0.4056E0,0.3223E0,0.3155E0,0.2899E0,0.2639E0,    
     &   0.2655E0,0.2299E0,0.2274E0,0.2323E0,0.2173E0,0.2502E0,0.2457E0,    
     &   0.2334E0,0.2137E0,0.1697E0,0.1993E0,0.1632E0,0.1413E0,0.1451E0,    
     &    0.1281E0,0.1412E0,0.1095E0,0.1058E0,
     &    0.9231E-1,0.7963E-1,0.7009E-1,    
     &    0.6606E-1,0.6235E-1,0.6147E-1,0.4960E-1,0.4473E-1,0.3821E-1,    
     &    0.4261E-1,0.3509E-1,0.2991E-1,0.2672E-1,0.2414E-1,0.1952E-1,    
     &    0.1771E-1,0.2172E-1,0.1591E-1,0.1434E-1,0.1414E-1,0.1289E-1,    
     &    0.9475E-2,0.8585E-2,0.1209E-2,0.7200E-2,0.7010E-2,0.9489E-2,    
     &    0.7936E-2,0.5152E-2,0.5100E-2,0.4421E-2,0.3789E-2,0.2482E-2,    
     &    0.2838E-2,0.2573E-2,0.2347E-2,0.2740E-2,0.1969E-2,0.1500E-2,    
     &    0.1456E-2,0.9734E-3,0.1603E-2,0.1043E-2,0.7821E-3,0.7396E-3,    
     &    0.8262E-3,0.7563E-3,0.8712E-3,0.4711E-3,0.6001E-3,0.5315E-3,    
     &    0.2372E-3,0.2909E-3,0.2612E-3,0.3666E-3 /    
	
	
	DATA avn/45.81 /
	DATA eavn/0.77 /
	
	avn1 = -7.3 + 2.56*log(ecm**2.d0) ! average multiplicity due to a 
	               			  ! single parton-parton scattering.
	
*       *** Initialisation run ***
*       *** Create directories ***
	  
	IF (intflag.eq.1) THEN
          
*       The average charged particle multiplicity in NSD events at 1.8
*       TeV is 45.81 . We now calculate the KNO variable Z18.
	  	 
	   do l1=1,129
	      Z18(l1)=P18(l1)/45.81
	      X18(l1)=P18(l1)/avn1
	   enddo
	  
	   do l2=1,128
	      KNO(l2)=KNO(l2)*ndata*0.001 ! Data was scaled by 1000. 
	      EKNO(l2)=EKNO(l2)*ndata*0.001 ! Data was scaled by 1000.
	   enddo
	  
	   CALL HCDIR('//HISTO',' ')
	   CALL HMDIR('plb435453','S')
	   CALL HCDIR('//PAWC',' ')
	   CALL HMDIR('plb435453','S')
	  
	   CALL HBOOKB(-1
     &	,'KNO distribution (NSD at 1.8 TeV) - 
     &  E735 data: non-KNO variables',128
     &  ,P18,0.0)
	   CALL HBOOKB(-2
     &	,'KNO distribution (NSD at 1.8 TeV) - E735 data: KNO variables'
     &  ,128,Z18,0.0)
	   CALL HBOOKB(-3
     &       ,'KNO (NSD at 1.8 TeV) - E735 data: x=n/<n1> 
     &       (m.p.i. variable)',128,X18,0.0) 
	   
	   CALL HBOOK1(10
     &	     ,'Charged particle density - dN_ch/deta - MC full eta range'
     &       ,60,-15.0,15.0,0.0)
	   
	   CALL HBOOKB(1,'KNO - NSD events (non-KNO variables)',
     &       128,P18,0.0) 
	   
	   CALL HBOOKB(2,'KNO - NSD events (KNO variables)',128,Z18,0.0)  
	   CALL HBOOKB(3
     &  	,'KNO (NSD at 1.8 TeV): x=n/<n1> (m.p.i. variable)',128
     &  	,X18,0.0) 
	   
	   
           ntot = 0             ! Counter - all particles
           rap0 = 0.0
	   events = 0		! Counter - number of events
	   nrap0 = 0		! Counter for particle density at eta=0
           
	   
*       write(*,*)'this is the average:',avrg,'   n1',avn1
	   
	   CALL HCDIR('//HISTO/plb435453',' ')
	   CALL HCDIR('//PAWC/plb435453',' ')
	  	  
	  
*       *** Loop over particles in the event ***
	   
	  
        ELSE IF (intflag.eq.2) THEN
          	  
	   CALL HCDIR('//PAWC/plb435453',' ')
          
	   events = events + 1	! event counter
	   nchev = 0		! selected charged particles counter (per event)
	  
	   DO 150 I=1,NHEP

	      if(HZLCHGE(IDHEP(I)).ne.0)then
		 if((IDHEP(I).eq.11).or.(IDHEP(I).eq.-11).or. 
     &	              (IDHEP(I).eq.13).or.(IDHEP(I).eq.-13).or.
     &	              (IDHEP(I).eq.15).or.(IDHEP(I).eq.-15).or.
     &	              (IDHEP(I).eq.2212).or.(IDHEP(I).eq.-2212).or.
     &  	      (IDHEP(I).eq.211).or.(IDHEP(I).eq.-211).or.
     &  	      (IDHEP(I).eq.321).or.(IDHEP(I).eq.-321).or.
     &  	      (IDHEP(I).gt.3000).or.(IDHEP(I).lt.-3000))then
		    
		    nchev = nchev + 1
		    rap = hzeta(I) ! Pseudorapidity.
		    call HF1(10,real(rap),1.0)
		    
		    if((rap.gt.-0.05).and.(rap.lt.0.05))then
		       nrap0 = nrap0 + 1
		    endif
		    
		 endif
		 
	      endif
	      
 150	   CONTINUE
	   
	   ntot = ntot + nchev
	   
	   call HFILL(1,real(nchev),0.0,1.0)		
	   call HFILL(2,real(nchev/avrg),0.0,1.0)	
	   call HFILL(3,real(nchev/avn1),0.0,1.0)	
	   
*       *** Third section: produce output and end. ***
	   
	ELSE IF (intflag.eq.3) THEN
	   
	   CALL HCDIR('//PAWC/plb435453',' ')
	   
	   avsel = ntot*1.d0/events
	   rap0 = nrap0*1.d0/(events*0.1)
	   
*       write(*,*)rap0
	   
	   CALL HPAK(-1,KNO)
	   CALL HPAKE(-1,EKNO)
	   CALL HPAK(-2,KNO)
	   CALL HPAKE(-2,EKNO)
	   CALL HPAK(-3,KNO)
	   CALL HPAKE(-3,EKNO)
	   
	   
C...    Histograms.
	   
*       write(*,*)'Number of generated events:  ',events
	   
	   factr10 = 1.d0/(events * 0.5)
	   factr1 = 1.d0*avsel/(events * 1.d0 )
	   factr2 = 1.d0*avsel/(events * 1.d0 )
	   factr3 = 1.d0*avsel/(events * 1.d0 )
	   
	   CALL HOPERA(10,'+E',10,10,real(factr10),0.)      
	   CALL HOPERA(1,'+E',1,1,real(factr1),0.)
	   CALL HOPERA(2,'+E',2,2,real(factr2),0.)
	   CALL HOPERA(3,'+E',3,3,real(factr3),0.)
	
*       CALL HRPUT(0,'histos.jets','N')
          
	ENDIF
        
	RETURN

	END
