*CMZ :  2.00/03 17/01/2000  12.06.37  by  Tancredi Carli
*CMZ :  2.00/02 17/01/2000  10.26.04  by  Tancredi Carli
*CMZ :  1.02/02 06/11/98  11.46.52  by  Tancredi Carli
*-- Author :
      SUBROUTINE HzMeanHi(iid,dX,dY,wtx)
* ---------------------------------------------------------------------------
*     purpose: this subroutine calculates weighted mean values of
*              a variable in different bins, their errors and stores
*              them in a hbook histogram
*
*              the produced histograms are similiar to hbprof-histos,
*              but this routine allows you to use non-aequidistant bins
*
*
*     usage:   1. initialization step:
*                 book histogram with hbook1 or hbookb
*
*              2. filling step:
*                 call this routine
*
*              3. termination step:
*                 nothing to do
*
*              input arguments for this routine:
*                 iid = histo id in hbook context
*                      (that one used for booking the histo in step 1)
*                 dX  = current value of binned variable (Abszissa)
*                 dY  = current value of variable, whose mean value
*                       is of interest (Ordinate)
*                 wtx = event weight
*                 (note: dX, dY declared double precision !)
*
*     structure:  this subroutine calls the following subroutine/functions
*                 (only the ones not included in this file are listed):
*                 hexist,hgive,hi,hie,hix,hpak,hpake (all from hbook-lib)
*
*
*     written by: Andreas von Manteuffel
*
*     reference:  the formula for the error is taken from:
*                 Michael Kuhlen (H1-01/95-418)
* ---------------------------------------------------------------------------
 
      IMPLICIT NONE
 
 
c --- input arguments
      integer iid,j
      double precision dX, dY
      real wtx
 
c --- internal limits
      integer MAXHISTS, MAXBINS, ARRSIZE
      parameter (MAXHISTS=150, MAXBINS=40)
      parameter (ARRSIZE=MAXHISTS*MAXBINS)
 
c --- hbook functions
      logical hexist
      real hi
      real hie
 
c --- argument variables for hbook subrouines
      character*80 ctit
      integer incX, incY, inwt, idum
      real Xmin, Xmax, Ymin, Ymax, Xleft
      real Ycont(MAXBINS), Yerr(MAXBINS)
      real testcont, testerr
 
c --- other internal stuff
      logical lfound ! indicates if histo is already in internal list
      integer ibin, ihist  ! counting variables for loops
      integer icbin ! current X-bin number in filling step
      integer icHi  ! current internal histo index
      integer iNhistos ! last internal histo index (see comment in code)
      integer iIDlist(MAXHISTS) ! list of hbook histo IDs
      integer iNbins(MAXHISTS) ! number of X-bins for histo
      double precision dmeanY, dNeff, dtemp
      double precision dwtx(MAXHISTS,MAXBINS)
      double precision dwtx2(MAXHISTS,MAXBINS)
      double precision dwtxY(MAXHISTS,MAXBINS)
      double precision dwtxY2(MAXHISTS,MAXBINS)
      double precision dbins(MAXHISTS,MAXBINS+1)
      logical lp
      Data lp/.false./
      data iNhistos /0/
      data iIDlist /MAXHISTS*0/
      save iNhistos, iIDlist, iNbins
      save dwtx,dwtx2,dwtxY,dwtxY2,dbins
      character*80 errMsgTxt
      character rout*10
      Data rout/'**hzmeanhi'/
      save rout
 
c --------------------------------------------------------------------------
c     look up histo
c --------------------------------------------------------------------------
c --- look up if histo already initialized in this routine
c     and get internal histo index
      lfound = .false.
      do ihist = 1, iNhistos
         if (iIDlist(ihist).eq.iid) then
            lfound = .true.
            icHi = ihist
            goto 100
         endif
      enddo
 100  continue
 
 
         if (.not.lfound) then
c --------------------------------------------------------------------------
c --- prepare new histo
c --------------------------------------------------------------------------
 
c --- CHECKS
c ----------
c --- check if histo ID already in use in hbook
            if (.not.hexist(iid)) then
               write(6,*) rout,' histo must be booked in hbook first !'
               goto 9999
c --- check last histo index currently used
c     this is identical with total number of histos used, before the
c     first histo is removed (see termination step)
            elseif (iNhistos.eq.MAXHISTS) then
               write(6,*) rout,'too many histos requested !'
               goto 9999
            endif
c --- get histo features
            call hgive(iid,ctit,incX,Xmin,Xmax,incY,Ymin,Ymax,inwt,idum)
c --- check if 1dimensional histo
            if (incY.ne.0) then
               write(6,*) rout,'can only handle 1d-histos !'
               goto 9999
c --- check number of channels
            elseif (incX.gt.MAXBINS) then
               write(6,*) rout,'too many channels !'
               goto 9999
            elseif (incX.lt.1) then
               write(6,*) rout,'invalid number of channels !'
               goto 9999
            endif
c --- check if histo is empty
            do ibin = 1, incX
               testcont = hi(iid,ibin)
               testerr = hie(iid,ibin)
               if ((testcont.ne.0.).or.(testerr.ne.0.)) then
                  write(6,*) rout,
     .             'histogram is not empty (contents will be ignored) !'
                  goto 200
               endif
            enddo
 200        continue
 
c --- INITIALIZATIONS
c -------------------
            iNhistos = iNhistos + 1
            icHi = iNhistos
            do ibin = 1, incX
               dwtx(icHi,ibin) = 0.D0
               dwtx2(icHi,ibin) = 0.D0
               dwtxY(icHi,ibin) = 0.D0
               dwtxY2(icHi,ibin) = 0.D0
*     get binning info
               call hix(iid,ibin,Xleft)
               dbins(icHi,ibin) = dble(Xleft)
            enddo
            dbins(icHi,incX+1) = dble(Xmax)
            iNbins(icHi) = incX
            iIDlist(icHi) = iid
         endif
 
c --------------------------------------------------------------------------
c     internal filling
c --------------------------------------------------------------------------
*     get bin number
         do ibin = 0, iNbins(icHi)
            if (dX.le.dbins(icHi,ibin+1)) goto 500
         enddo
 500     continue
         icbin = ibin
         if ((icbin.ge.1).and.(icbin.le.iNbins(icHi))) then
            dwtx(icHi,icbin) = dwtx(icHi,icbin)+dble(wtx)
            dwtx2(icHi,icbin) = dwtx2(icHi,icbin)+dble(wtx)**2
            dwtxY(icHi,icbin) = dwtxY(icHi,icbin)+dble(wtx)*dY
            dwtxY2(icHi,icbin) = dwtxY2(icHi,icbin)+dble(wtx)*(dY)**2
         endif
 
c --------------------------------------------------------------------------
c     calculation of current means and their errors
c --------------------------------------------------------------------------
         do ibin = 1, iNbins(icHi)
           if (dwtx(icHi,ibin).eq.0.D0) then
             Ycont(ibin) = 0.
             Yerr(ibin) = 0.
           else
             dmeanY = dwtxY(icHi,ibin)/dwtx(icHi,ibin)
             Ycont(ibin) = real(dmeanY)
             Yerr(ibin) = 0.
             if (dwtx2(icHi,ibin).le.0.D0) then
               write(6,*) rout,'numerical problems with calculation !'
             else
               dNeff = dwtx(icHi,ibin)**2/dwtx2(icHi,ibin)
               if (dNeff.gt.1.D0) then
                 dtemp=dwtxY2(icHi,ibin)/dwtx(icHi,ibin) - dmeanY**2
                 Yerr(ibin)=sqrt(real(dtemp*(1.D0/(dNeff-1.D0))))
               endif
             endif
           endif
         enddo
c --------------------------------------------------------------------------
c     histo filling
c --------------------------------------------------------------------------
         if (lp) write(6,*) rout,iid,' ibin= ',iNbins(icHi),
     &   ' cont= ',(ycont(j),j=1,iNbins(icHi))
*
         call hpak(iid,Ycont)
         call hpake(iid,Yerr)
 
 
 9999 continue
      RETURN
      END
