*CMZ :  1.01/15 15/05/98  09.57.45  by  Tancredi Carli
*CMZ :  1.01/11 20/11/97  13.57.45  by  Tancredi Carli
*CMZ :  1.01/10 03/11/97  16.49.01  by  Tancredi Carli
*CMZ :  1.01/07 10/10/97  19.51.17  by  Tancredi Carli
*-- Author :
	Subroutine HZpscon(iflag)
*****************************************************************************
*
* This routine checks transversal momentum and s(cmsen) conservation
* in generated events on the parton level.
*
* produced histogramms: 10 relative error of calc. cmsen to actual s
*                       11 relative error of x-momentum
*                       12 relative error of y-momentum
*
* Arguments: iflag=1 initialisation
*            iflag=2 filling
*            iflag=3 termination
*
*
*  written by: Reimer Selle (rselle@rec06.desy.de)
*****************************************************************************
*
*
	IMPLICIT NONE
*
#include "hepevtp.inc"
#include "heracmn.inc"
#include "hzfunc.inc"
*

*                                          subroutine-related var.
*                                          -----------------------
      Integer plist(nmxhep)
      Integer iflag,ihep,i,j,k,l,npart,entry
      Data entry/0/
*     DESY Preprint number
      Character*4 xxxx
      Data xxxx/'psco'/
      character*40 string
      Integer iel,idum,ierr,ibeam,ee,eb
      Parameter (ee=27.6)
*                                          kinetic variables and
*                                          errors
      Double precision px,py,pz,en,cmsen,mass2
      real ser,perx,pery
*                                          normalization-var.
      Integer nev
      Real rnev,norm
*                                          test-variable
      Logical lp,test
      Data lp/.false./
      Data test/.true./

*********************************************************************
*
*                      Initialization
*
*********************************************************************
*
      IF (iflag.eq.1) then
*
*
*      Initialisation: The following MUST always be done
*      (i) make subdirectory in PAWC
*       - use the name as the xxxxxx in HZxxxxxx subroutine
*      (i) make subdirectory in o/p file
*
       Call hcdir('//PAWC',' ')
       call hmdir(xxxx,'S')
       Call hcdir('//HISTO',' ')
       call hmdir(xxxx,'S')

*                                         book generator histos

       call hbook1(10,'rel. error in s(cmsen) in %',40,-1.,1.,0.)
       call hbook1(11,'rel. error in px in %',40,-1.,1.,0.)
       call hbook1(12,'rel. error in py in %',40,-1.,1.,0.)
*
*********************************************************************
*
*                      Event Processing
*
*********************************************************************

       ELSE IF(iflag.eq.2) then

*      Filling: The following MUST always be done
*      (i) move to the correct sub-directory in PAWC
*
       call hcdir('//PAWC/'//xxxx,' ')

*                                      center-of-mass energy for 1993/94
*                                      HERA-runnings
       cmsen=4.*ee*820.
*
       entry=entry+1
*

       iel=HZIDELEC(idum)
       if (iel.eq.-1) then
        write(6,*) 'Hz'//xxxx,' electron not found '
        goto 9990
       endif

*                                      'pre-check' for generated events
*
       ierr=HZIBEAM(ibeam,idum)
       if (.not.(ierr.eq.1)) then
        write(6,*) 'HZ'//xxxx,' beams not found ! '
        goto 9990
       endif
*
       eb=real(PHEP(4,idum))
       if (abs(abs(eb)-ee).gt.0.2) then
        write(6,*) 'HZ'//xxxx,' Electron beam not at ',ee,'  ! '
        goto 9990
       endif


       call hzparton(npart,plist)
        px=0.
        py=0.
        pz=0.
        en=0.
        mass2=0.

*                                     get total parton 4-vector
       do ihep=1,npart
         px= PHEP(1,plist(ihep))+ px
         py= PHEP(2,plist(ihep))+ py
         pz= PHEP(3,plist(ihep))+ pz
         en= PHEP(4,plist(ihep))+ en
        enddo

*                                   check transversal momentum conservation
        perx=((-abs(PHEP(1,iel))+abs(px))/abs(PHEP(1,iel)))*100
        pery=((-abs(PHEP(2,iel))+abs(py))/abs(PHEP(2,iel)))*100

       call hf1(11,perx,1.)
       call hf1(12,pery,1.)
*                                    add lepton contribution 4-vector
         px= PHEP(1,iel)+ px
         py= PHEP(2,iel)+ py
         pz= PHEP(3,iel)+ pz
         en= PHEP(4,iel)+ en
*                                     compare with cmsen
        mass2=en**2-px**2-py**2-pz**2

       if (lp) write(6,*) 'Hz'//xxxx,': s=',cmsen
*                                     calculate relative error (ser)
       ser=((mass2-cmsen)/cmsen)*100
       call hf1(10,ser,1.)
       nev= nev+1
      if (test) then
       if (entry.le.10.or.abs(perx).gt.0.2.or.abs(pery).gt.0.2) then
        write(6,*)'*********ENTRY*********',entry
        write(6,*)'px/part =',px -PHEP(1,iel)
        write(6,*)'pxel=',PHEP(1,iel)
        write(6,*)'py/part =',py -PHEP(2,iel)
        write(6,*)'pyel=',PHEP(2,iel)
        write(6,*)'perx =',perx
        write(6,*)'pery =',pery
        write(6,*)'masse =',mass2
        write(6,*)'s-error =',ser
        do i=1,npart
         print*,i,'**plist:',plist(i)
        enddo
       endif
      endif
9990   continue	
*
*********************************************************************
*
*                      Termination
*
*********************************************************************

      Else if(iflag.eq.3) then

*      Termination: The following MUST always be done
*      (i) Move to the correct PAW subdirectory
*
       call hcdir('//PAWC/'//xxxx,' ')
*                                              Normalization
       if (lp) write(6,*)'events',nev
       rnev=real(nev)
       norm= 1/rnev

       call hopera (10,'+E',10,10,norm,0.)
       call hopera (11,'+E',11,11,norm,0.)
       call hopera (12,'+E',12,12,norm,0.)

       ENDIF

       RETURN

       END

